/*******************************************************************************
*                                 AMetal
*                       ----------------------------
*                       innovating embedded platform
*
* Copyright (c) 2001-2018 Guangzhou ZHIYUAN Electronics Co., Ltd.
* All rights reserved.
*
* Contact information:
* web site:    http://www.zlg.cn/
*******************************************************************************/

/**
 * \file
 * \brief  NVICжϱ׼ӿ
 *
 * \internal
 * \par Modification History
 * - 1.01 17-04-10  sdy, modified.
 * - 1.00 15-01-29  hbt, first implementation.
 * \endinternal
 */

#ifndef __AM_ARM_NVIC_H
#define __AM_ARM_NVIC_H

#include "hw/amhw_arm_nvic.h"

#ifdef __cplusplus
extern "C" {
#endif

/**
 * \addtogroup am_arm_if_nvic
 * \copydoc am_arm_nvic.h
 * @{
 */

/**
 * \brief ϵͳ쳣Ŷ
 */
#define    INUM_NMI             -14    /**< \brief ж */
#define    INUM_HARDFAULT       -13    /**< \brief Ӳ */
#define    INUM_MEMMANAGE       -12    /**< \brief 洢쳣M0M0+޴쳣 */
#define    INUM_BUSFAULT        -11    /**< \brief 쳣M0M0+޴쳣 */
#define    INUM_USAGEFAULT      -10    /**< \brief Ӧ쳣M0M0+޴쳣 */
#define    INUM_SVCALL          -5     /**< \brief ϵͳ쳣 */
#define    INUM_PENDSV          -2     /**< \brief ϵͳ쳣 */
#define    INUM_SYSTICK         -1     /**< \brief ϵͳδʱ쳣 */

enum {
    AM_ARM_NVIC_CORE_M0,      /**< \brief M0 ں */
    AM_ARM_NVIC_CORE_M0PLUS,  /**< \brief M0+ں */
    AM_ARM_NVIC_CORE_M3,      /**< \brief M3 ں */
    AM_ARM_NVIC_CORE_M4,      /**< \brief M4 ں */
};

/**
 * \brief жϷϢ
 */
struct am_arm_nvic_servinfo {

    /** \brief ʼжϺ */
    int inum_start;

    /** \brief ĩβжϺ */
    int inum_end;
};

/** \brief жϷϢ */
struct am_arm_nvic_isr_info {

    /** \brief ص */
    am_pfnvoid_t  pfn_isr;

    /** \brief صĲ */
    void         *p_arg;
};


/** \brief ж豸Ϣ */
typedef struct am_arm_nvic_devinfo {
    
    /** \brief жϷϢʵ */
    const struct am_arm_nvic_servinfo int_servinfo;
    
    /** \brief ں */
    uint8_t core;

    /** \brief ȼλ */
    uint8_t prio_bits;

    /** \brief ȼ */
    int group;

    /** \brief ж */
    int input_cnt;

    /** \brief ISR Ϣӳ(С input_cnt һ) */
    uint8_t *p_isrmap;

    /** \brief ISR Ϣ */
    int isrinfo_cnt;

    /** \brief ISR Ϣӳڴ(С isrinfo_cnt һ) */
    struct am_arm_nvic_isr_info *p_isrinfo;
    
    /** \brief ƽ̨ʼ */
    void     (*pfn_plfm_init)(void);

    /** \brief ƽ̨ʼ */
    void     (*pfn_plfm_deinit)(void);
    
} am_arm_nvic_devinfo_t;

/**
 * \brief ж豸ʵ 
 */
typedef struct am_arm_nvic_dev {
    
    /** \brief ָж豸Ϣָ */
    const am_arm_nvic_devinfo_t *p_devinfo;
    
    /** \brief Ч־ */
    am_bool_t                    valid_flg;
    
} am_arm_nvic_dev_t;


/**
 * \brief	жȼ
 *
 * \param[in]	  inum             : жϺ
 * \param[in]	  preempt_priority : ȼ
 * \param[in]     sub_priority     : ȼm0ں˴ֵЧ
 *
 * \retval  AM_OK       : ɹ
 * \retval -AM_EINVAL   : Ч
 */
int am_arm_nvic_priority_set(int inum,
                             uint32_t preempt_priority,
                             uint32_t sub_priority);

/**
 * \brief жϳʼ 
 *
 * \param[in] p_dev     : ָж豸ָ 
 * \param[in] p_devinfo : ָж豸Ϣָ 
 *
 * \retval  AM_OK       : ɹ 
 * \retval -AM_EINVAL   : Ч 
 */
int am_arm_nvic_init (am_arm_nvic_dev_t           *p_dev,
                      const am_arm_nvic_devinfo_t *p_devinfo);

/**
 * \brief жȥʼ
 * \return   
 */
void am_arm_nvic_deinit (void);

/**
 * @}
 */

#ifdef __cplusplus
}
#endif


#endif /* __AM_ARM_NVIC_H */

/* end of file */
