/*******************************************************************************
*                                 AMetal
*                       ----------------------------
*                       innovating embedded platform
*
* Copyright (c) 2001-2018 Guangzhou ZHIYUAN Electronics Co., Ltd.
* All rights reserved.
*
* Contact information:
* web site:    http://www.zlg.cn/
*******************************************************************************/

/**
 * \file
 * \brief ep24cxx ͨEEPROM
 *
 * EP24CXX ָ볣CAT24C02CAT24C08һϵEEPROMоƬоƬ
 * 1kbit  1Mbit
 *
 * \internal
 * \par modification history:
 * - 1.00 16-08-03  tee, first implementation
 * \endinternal
 */

#ifndef __AM_EP24CXX_H
#define __AM_EP24CXX_H

#ifdef __cplusplus
extern "C" {
#endif

#include "ametal.h"
#include "am_types.h"
#include "am_i2c.h"
#include "am_nvram.h"

/**
 * @addtogroup am_if_ep24cxx
 * @copydoc am_ep24cxx.h
 * @{
 */

/**
 * \brief  EEPROM оƬԶһоƬ
 *
 *     ͬоƬ֮һЩԣҳСȣܴڲ죬ͨԣ
 * ȷһоƬһϵͬоƬ6ԣ
 *
 * 1. ҳС
 *     AT24C02зҳҳСΪ8ֽڣAT24C1024ҳСΪ256ֽڡ޷ҳʱ
 * Ч
 *
 * 2. ޷ҳ
 *     AT24C02зҳҳСΪ8ֽڣ繫˾FRAM޷ҳ
 *
 * 3. ݵַַеλ 
 *     һأAT24C02ַвݵַAT24C1024
 * ϴ󣬹1024Kbit128K = 2 ^ 17ݵַҪ17 bitsоƬ
 * ʱΪ˽ʡֽͨĴַʹ2ֽڣ16 bitsԱʾݵַ
 * 16λݵַλŵַСʱԵֵΪ1
 *
 * 4. Ĵַֽ
 *     AT24C02ĴַΪ1ֽڣԱʾݵַʱԵֵΪ1
 * AT24C1024ĴַΪ2ֽڣԱʾݵַĵ16λʱԵֵ
 * Ϊ2
 *
 * 5. оƬbyte
 *     AT24C02Ϊ2K bitΪ 256 ֽڡAT24C10241024K
 * bit128K ֽڡһĹɣΪ128ֽڵ2ݴ
 *      = (2^n) * 128
 * ˣʵʱʾʱʹֵnʾAT24C02nֵΪ1
 * AT24C1024 n ֵΪ 10
 *
 * 6. дʱ
 *     EEPROMоƬдʱв죬AT24C02дʱΪ 5ms繫˾FRAM
 * ޷ҳдʱΪ0,ȴ
 *
 *     6ԼȷĶһоƬһϵͬԵоƬʹһ32λ
 * ͳһıʾЩԡ32λݵλ£
 *
 * - bit[15:0]  ҳС
 * - bit[18:16] ݵַַеλ 
 * - bit[19]    Ĵַֽ0: ֽ  1: ˫ֽ
 * - bit[23:20] оƬ, (2^n)*128 bytes, n = 0 ~ 15
 * - bit[31:24] дʱ䣬λmsЧֵ0 ~ 255
 *
 * \param[in] page_size                  : ҳС
 * \param[in] data_addr_bits_in_slv_addr : ݵַַеλ
 * \param[in] reg_addr_double_bytes      : Ĵַ˫ֽڡ0ֽڣ1:˫ֽ
 * \param[in] chip_size                  : оƬsize = (2^n)*128 bytes
 * \param[in] write_time                 : дʱ䣬λms 
 *
 * \return 32λݣʾһضоƬһϵͬоƬ
 * 
 * \note һأûֱʹøú꣬ӦʹѾõоƬͺš
 */
#define AM_EP24CXX_TYPE_DEF(page_size,                  \
                            data_addr_bits_in_slv_addr, \
                            reg_addr_double_bytes,      \
                            chip_size,                  \
                            write_time)                 \
       (((page_size) << 0)                   | \
        ((data_addr_bits_in_slv_addr) << 16) | \
        ((reg_addr_double_bytes) << 19)      | \
        ((chip_size)      << 20)             | \
        ((write_time)     << 24))
 
/**
 * \name ֪һЩоƬͺŶ
 *
 *     ʹõоƬͺŴ˴δ壬ʹ \sa AM_EP24CXX_CHIP_DEF() 궨һ
 * µоƬ
 * 
 * @{
 */

/** \brief AT24C01 */
#define AM_EP24CXX_AT24C01      AM_EP24CXX_TYPE_DEF(8,   0,  0,  0,  5)

/** \brief AT24C02 */
#define AM_EP24CXX_AT24C02      AM_EP24CXX_TYPE_DEF(8,   0,  0,  1,  5)

/** \brief AT24C04 */
#define AM_EP24CXX_AT24C04      AM_EP24CXX_TYPE_DEF(16,  1,  0,  2,  5)

/** \brief AT24C08 */
#define AM_EP24CXX_AT24C08      AM_EP24CXX_TYPE_DEF(16,  2,  0,  3,  5)

/** \brief AT24C16 */
#define AM_EP24CXX_AT24C16      AM_EP24CXX_TYPE_DEF(16,  3,  0,  4,  5)

/** \brief AT24C32 */
#define AM_EP24CXX_AT24C32      AM_EP24CXX_TYPE_DEF(32,  0,  1,  5,  5)

/** \brief AT24C64 */
#define AM_EP24CXX_AT24C64      AM_EP24CXX_TYPE_DEF(32,  0,  1,  6,  5)

/** \brief AT24C128 */
#define AM_EP24CXX_AT24C128     AM_EP24CXX_TYPE_DEF(64,  0,  1,  7,  5)

/** \brief AT24C256 */
#define AM_EP24CXX_AT24C256     AM_EP24CXX_TYPE_DEF(64,  0,  1,  8,  5)

/** \brief AT24C512 */
#define AM_EP24CXX_AT24C512     AM_EP24CXX_TYPE_DEF(128, 0,  1,  9,  5)

/** \brief AT24CM01 */
#define AM_EP24CXX_AT24CM01     AM_EP24CXX_TYPE_DEF(256, 1,  1,  10, 5)

/** \brief CAT24C01 */
#define AM_EP24CXX_CAT24C01     AM_EP24CXX_TYPE_DEF(16,  0,  0,  0,  5)

/** \brief CAT24C02 */
#define AM_EP24CXX_CAT24C02     AM_EP24CXX_TYPE_DEF(16,  0,  0,  1,  5)

/** \brief CAT24C04 */
#define AM_EP24CXX_CAT24C04     AM_EP24CXX_TYPE_DEF(16,  1,  0,  2,  5)

/** \brief CAT24C08 */
#define AM_EP24CXX_CAT24C08     AM_EP24CXX_TYPE_DEF(16,  2,  0,  3,  5)

/** \brief CAT24C16 */
#define AM_EP24CXX_CAT24C16     AM_EP24CXX_TYPE_DEF(16,  3,  0,  4,  5)

/** \brief FM24C02 */
#define AM_EP24CXX_FM24C02      AM_EP24CXX_TYPE_DEF(8,   0,  0,  1,  5)

/** \brief FM24C04 */
#define AM_EP24CXX_FM24C04      AM_EP24CXX_TYPE_DEF(16,  1,  0,  2,  5)

/** \brief FM24C08 */
#define AM_EP24CXX_FM24C08      AM_EP24CXX_TYPE_DEF(16,  2,  0,  3,  5)

/** \brief FM24C16 */
#define AM_EP24CXX_FM24C16      AM_EP24CXX_TYPE_DEF(16,  3,  0,  4,  5)

/** \brief FM24C32 */
#define AM_EP24CXX_FM24C32      AM_EP24CXX_TYPE_DEF(32,  0,  1,  5,  5)

/** \brief FM24C64 */
#define AM_EP24CXX_FM24C64      AM_EP24CXX_TYPE_DEF(32,  0,  1,  6,  5)

/** \brief FM24C128 */
#define AM_EP24CXX_FM24C128     AM_EP24CXX_TYPE_DEF(64,  0,  1,  7,  5)

/** \brief FM24C256 */
#define AM_EP24CXX_FM24C256     AM_EP24CXX_TYPE_DEF(64,  0,  1,  8,  5)

/** \brief FM24C512 */
#define AM_EP24CXX_FM24C512     AM_EP24CXX_TYPE_DEF(128, 0,  1,  9,  5)

/** \brief FM24C1024 */
#define AM_EP24CXX_FM24C1024    AM_EP24CXX_TYPE_DEF(256, 1,  1,  10, 5)

/** @} */

/**
 * \brief ep24cxx豸Ϣṹ嶨
 */
typedef struct am_ep24cxx_devinfo {
    
    /** \brief 豸 7-bit ӻַ */
    uint8_t                        slv_addr;

    /** \brief оƬͺţ磺 \sa #AM_EP24CXX_AT24C02 */
    uint32_t                       type;
    
} am_ep24cxx_devinfo_t;
 
/**
 * \brief ep24cxx豸ṹ嶨
 */
typedef struct am_ep24cxx_dev {

    /** \brief I2Cӻ豸             */
    am_i2c_device_t                i2c_dev;
 
    /** \brief ep24cxx豸ṩ NVRAM ׼ */
    am_nvram_dev_t                *p_serv;
    
    /** \brief ָ豸Ϣָ      */
    const am_ep24cxx_devinfo_t    *p_devinfo;
    
} am_ep24cxx_dev_t;

/** \brief ep24cxx  */
typedef am_ep24cxx_dev_t *am_ep24cxx_handle_t;
 
/**
 * \brief ep24cxxʼ
 *
 * ep24xxṩ׼ NVRAM 񣬸óʼر׼ NVRAM 
 *
 * \param[in] p_dev      : ָep24cxx豸ָ
 * \param[in] p_devinfo  : ָep24cxx豸Ϣָ
 * \param[in] i2c_handle : I2C׼ʹøI2Cep24cxxͨţ
 *
 * \return ep24cxxhandleʼʧܣ򷵻ֵΪNULL
 *
 * \note 豸ָp_devָ豸ֻҪ壬Ҫڵñǰʼ
 */
am_ep24cxx_handle_t am_ep24cxx_init (am_ep24cxx_dev_t           *p_dev,
                                     const am_ep24cxx_devinfo_t *p_devinfo,
                                     am_i2c_handle_t             i2c_handle);
                                   
/**
 * \brief ʼEP24CXXNVRAMܣϵͳṩ  NVRAM ׼
 * 
 *   úEP24CXX豸Ϊ׼NVRAM豸עᵽϵͳ
 * 
 * \param[in] handle      : ep24cxx
 * \param[in] p_dev       : NVRAM ׼豸
 * \param[in] p_dev_name  : NVRAM ׼豸豸
 *
 * \return AM_OK, ṩNVRAMɹֵṩNVRAMʧܡ
 *
 * \note ʹøú󣬼ʹNVRAM׼ӿ am_nvram_set()  am_nvram_get() 
 * EP24CXX洢ʹõĶα am_nvram_cfg.cļж壬洢豸
 * дΪ˴ p_name ָ֣ p_name Ϊ  "fm24c02", g_nvram_segs[]
 * б5洢ΣΪʾ
 *    {"ip",         0,  0,  4,   "fm24c02"},
 *    {"ip",         1,  4,  4,   "fm24c02"},
 *    {"temp_limit", 0,  8,  4,   "fm24c02"},
 *    {"system",     0,  12, 50,  "fm24c02"},
 *    {"test",       0,  62, 178, "fm24c02"},
 */
int am_ep24cxx_nvram_init (am_ep24cxx_handle_t   handle,
                           am_nvram_dev_t       *p_dev,
                           const char           *p_dev_name);

/**
 * \brief д
 * 
 * \param[in] handle     : ep24cxx
 * \param[in] start_addr : дʼַ
 * \param[in] p_buf      : д
 * \param[in] len        дݳ
 *
 * \return AM_OK, дɹֵдʧܡ
 */                      
int am_ep24cxx_write (am_ep24cxx_handle_t  handle, 
                      int                  start_addr, 
                      uint8_t             *p_buf, 
                      int                  len);

  
/**
 * \brief ݶȡ
 * 
 * \param[in]  handle    : ep24cxx
 * \param[in]  start_addr: ݶȡʼַ
 * \param[out] p_buf     : ŶȡݵĻ
 * \param[in]  len       ȡݵĳ
 *
 * \return AM_OK, ݶȡɹֵݶȡʧܡ
 */
int am_ep24cxx_read (am_ep24cxx_handle_t handle,
                     int                 start_addr, 
                     uint8_t            *p_buf, 
                     int                 len);
                              
/**
 * \brief ep24cxxʼ
 *
 * ʹָep24cxx豸ʱʹøúʼ豸ͷԴ
 *
 * \param[in] handle : ep24cxx
 *
 * \return AM_OK, ʼɹֵʼʧܡ
 */                              
int am_ep24cxx_deinit (am_ep24cxx_handle_t handle);
                              
/** 
 * @}
 */

#ifdef __cplusplus
}
#endif

#endif /* __AM_EP24CXX_H */

/* end of file */
