/*******************************************************************************
*                                 AMetal
*                       ----------------------------
*                       innovating embedded platform
*
* Copyright (c) 2001-2018 Guangzhou ZHIYUAN Electronics Co., Ltd.
* All rights reserved.
*
* Contact information:
* web site:    http://www.zlg.cn/
*******************************************************************************/

/**
 * \file
 * \brief ȫоƬFMSE-A03 ͷļ
 *
 * \internal
 * \par modification history
 * - 1.00 20-07-01  yrz, first implementation.
 * \endinternal
 */

#ifndef __AM_FMSE_H
#define __AM_FMSE_H

#ifdef __cplusplus
extern "C" {
#endif

#include "am_i2c.h"
#include "am_spi.h"

/* gen rsa key cmd para define */
#define RSA1024				0
#define RSA1280				1
#define RSA2048				2

#define EXP65537			0
#define EXP3				1

typedef struct{
    uint8_t cla;
    uint8_t ins;
    uint8_t p1;
    uint8_t p2;
    union{
        uint8_t lc;
        uint8_t le;
    }p3;
    uint8_t capdu[256];
} apdu_pack_t;
    
/**
 * \brief fmse ṹ
 */
struct am_fmse_drv_funcs{

    uint8_t (*fmse_apdu_transceive)(void     *p_drv,
                                    uint8_t  *sbuf,
                                    uint16_t  slen,
                                    uint8_t  *rbuf,
                                    uint16_t *rlen, 
                                    uint16_t  poll_inv,
                                    uint32_t  poll_timeout);
};

/**
 * \brief fmse 
 */
typedef struct am_fmse_serv {

    /** \brief fmse ṹָ */
    const struct am_fmse_drv_funcs *p_funcs;

    /** \brief ĵһ */
    void                           *p_drv;
    
} am_fmse_serv_t;

/** \brief fmse ׼Ͷ  */
typedef am_fmse_serv_t *am_fmse_handle_t;

/**
 * \brief ڴʼ
 *
 * \param[in] p_dst  : ָʼڴָ
 * \param[in] val    : ʼֵ
 * \param[in] count  : ʼڴ泤
 *
 * \return ʼڴ׵ַ
 */
void* am_fmse_memset(void* p_dst, int val, size_t count);

/**
 * \brief ڴݸ
 *
 * \param[in] p_dst  : ָĿĵַָ
 * \param[in] p_src  : ָԴַָ
 * \param[in] count  : Ƶݳ
 *
 * \return Ŀĵַָ
 */
void* am_fmse_memmove(void* p_dst, const void* p_src, size_t count);

/**
 * \brief ļ
 *
 * \param[in] handle : FMSE׼
 *
 * \return ״̬
 */
uint16_t am_fmse_file_create (am_fmse_handle_t handle,
                              uint16_t         fid,
                              uint8_t          type,
                              uint16_t         space,
                              uint16_t         rank);

/**
 * \brief ȡ
 *
 * \param[in] handle : FMSE׼
 *
 * \return ״̬
 */
uint16_t am_fmse_challenge_get (am_fmse_handle_t handle,
                                uint16_t         inlen,
                                uint8_t         *rbuf,
                                uint16_t        *rlen);

/**
 * \brief ļѡ
 *
 * \param[in] handle : FMSE׼
 *
 * \return ״̬
 */
uint16_t am_fmse_file_select (am_fmse_handle_t handle,
                              uint16_t         fid,
                              uint8_t         *rbuf,
                              uint16_t        *rlen);

/**
 * \brief У
 *
 * \param[in] handle : FMSE׼
 *
 * \return ״̬
 */
uint16_t am_fmse_pin_verify (am_fmse_handle_t  handle,
                             uint16_t          inlen,
                             uint8_t          *inbuf);

/**
 * \brief ļȡ
 *
 * \param[in] handle : FMSE׼
 * \param[in] sfi    : ļʼλ
 * \param[in] inlen  : ȡĳ
 * \param[in] rbuf   : ջ
 * \param[in] rlen   : յĳ
 *
 * \return ״̬
 */
uint16_t am_fmse_binary_read (am_fmse_handle_t handle,
                              uint16_t         sfi,
                              uint16_t         inlen,
                              uint8_t         *rbuf,
                              uint16_t         *rlen);

/**
 * \brief RSA Կ
 *
 * \param[in] handle  : FMSE׼
 * \param[in] key_len : ԿԳ
 * \param[in] exp     : Կָ
 * \param[in] fid     : the Fid that save the dest RSA Key Pair
 *                      input 4Bytes Fid format: public key fid + private key fid
 *                      single 2Bytes Fid format:high byte + low byte
 * \param[in] rbuf    : ջ
 * \param[in] rlen    : յĳ
 *
 * \return ״̬
 */
uint16_t am_fmse_rsa_key_pair_gen(am_fmse_handle_t handle,
                                  uint8_t          key_len,
                                  uint8_t          exp,
                                  uint32_t         fid,
                                  uint8_t         *rbuf,
                                  uint16_t        *rlen);

/**
 * \brief SM2 Կ
 *
 * \param[in] handle  : FMSE׼
 * \param[in] fid     : the Fid that save the dest RSA Key Pair
 *                      input 4Bytes Fid format: public key fid + private key fid
 *                      single 2Bytes Fid format:high byte + low byte
 * \param[in] rbuf    : ջ
 * \param[in] rlen    : յĳ
 *
 * \return ״̬
 */
uint16_t am_fmse_sm2_key_pair_gen (am_fmse_handle_t handle,
                                   uint32_t         fid,
                                   uint8_t         *rbuf,
                                   uint16_t        *rlen);

/**
 * \brief ỰԿ
 *
 * \param[in] handle  : FMSE׼
 * \param[in] p1p2    : key type and data block flag and RSA type
 * \param[in] inlen   : file length to read
 * \param[in] inbuf   : input session key buffer
 * \param[in] rbuf    : ջ
 * \param[in] rlen    : յĳ
 *
 * \return ״̬
 */
uint16_t am_fmse_session_key_import (am_fmse_handle_t handle,
                                     uint16_t         p1p2,
                                     uint16_t         inlen,
                                     uint8_t         *inbuf,
                                     uint8_t         *rbuf,
                                     uint16_t        *rlen);

/**
 * \brief data encrypt and decrypt
 *
 * \param[in] handle  : FMSE׼
 * \param[in] p1p2    : key type and data block flag and RSA type
 * \param[in] inlen   : file length to read
 * \param[in] inbuf   : input data buffer
 * \param[in] rbuf    : ջ
 * \param[in] rlen    : յĳ
 *
 * \return ״̬
 */
uint16_t am_fmse_data_encrypt_and_decrypt(am_fmse_handle_t handle,
                                          uint16_t         p1p2,
                                          uint16_t         inlen,
                                          uint8_t         *inbuf,
                                          uint8_t         *rbuf,
                                          uint16_t        *rlen);

/**
 * \brief RSA public key calculate
 *
 * \param[in] handle  : FMSE׼
 * \param[in] p1p2    : key type and data block flag and RSA type
 * \param[in] inlen   : file length to read
 * \param[in] inbuf   : input data buffer
 * \param[in] rbuf    : ջ
 * \param[in] rlen    : յĳ
 *
 * \return ״̬
 */
uint16_t am_fmse_rsa_pub_key_cal(am_fmse_handle_t handle,
                                 uint16_t         p1p2,
                                 uint16_t         inlen,
                                 uint8_t         *inbuf,
                                 uint8_t         *rbuf,
                                 uint16_t        *rlen);

/**
 * \brief RSA private key calculate
 *
 * \param[in] handle  : FMSE׼
 * \param[in] p1p2    : key type and data block flag and RSA type
 * \param[in] inlen   : file length to read
 * \param[in] inbuf   : input data buffer
 * \param[in] rbuf    : ջ
 * \param[in] rlen    : յĳ
 *
 * \return ״̬
 */
uint16_t am_fmse_rsa_pri_key_cal(am_fmse_handle_t handle,
                                 uint16_t         p1p2,
                                 uint16_t         inlen,
                                 uint8_t         *inbuf,
                                 uint8_t         *rbuf,
                                 uint16_t        *rlen);

/**
 * \brief SM2 public key calculate
 *
 * \param[in] handle  : FMSE׼
 * \param[in] p1p2    : key type and data block flag and RSA type
 * \param[in] inlen   : file length to read
 * \param[in] inbuf   : input data buffer
 * \param[in] rbuf    : ջ
 * \param[in] rlen    : յĳ
 *
 * \return ״̬
 */
uint16_t am_fmse_sm2_pub_key_cal(am_fmse_handle_t handle,
                                 uint8_t          mode,
                                 uint16_t         inlen,
                                 uint8_t         *inbuf,
                                 uint8_t         *rbuf,
                                 uint16_t        *rlen);

/**
 * \brief SM2 private key calculate
 *
 * \param[in] handle  : FMSE׼
 * \param[in] p1p2    : key type and data block flag and RSA type
 * \param[in] inlen   : file length to read
 * \param[in] inbuf   : input data buffer
 * \param[in] rbuf    : ջ
 * \param[in] rlen    : յĳ
 *
 * \return ״̬
 */
uint16_t am_fmse_sm2_pri_key_cal(am_fmse_handle_t handle,
                                 uint8_t          mode,
                                 uint16_t         inlen,
                                 uint8_t         *inbuf,
                                 uint8_t         *rbuf,
                                 uint16_t        *rlen);

/**
 * \brief Install RSA key
 *
 * \param[in] handle  : FMSE׼
 * \param[in] p1p2    : key type and data block flag and RSA type
 * \param[in] inlen   : file length to read
 * \param[in] inbuf   : input data buffer
 * \param[in] rbuf    : ջ
 * \param[in] rlen    : յĳ
 *
 * \return ״̬
 */
uint16_t am_fmse_rsa_key_install (am_fmse_handle_t handle,
                                  uint16_t         p1p2,
                                  uint16_t         inlen,
                                  uint8_t         *inbuf,
                                  uint8_t         *rbuf,
                                  uint16_t        *rlen);

/**
 * \brief WriteBinary
 *
 * \param[in] handle  : FMSE׼
 * \param[in] sfi     : file start position to write
 * \param[in] inlen   : file length to read
 * \param[in] inbuf   : input data buffer
 * \param[in] rbuf    : ջ
 * \param[in] rlen    : յĳ
 *
 * \return ״̬
 */
uint16_t am_fmse_binary_write (am_fmse_handle_t handle,
                               uint16_t         sfi,
                               uint16_t         inlen,
                               uint8_t         *inbuf,
                               uint8_t         *rbuf,
                               uint16_t        *rlen);

/**
 * \brief Data_Compress
 *
 * \param[in] handle  : FMSE׼
 * \param[in] p1p2    : 
 * \param[in] inlen   : file length to read
 * \param[in] inbuf   : input data buffer
 * \param[in] rbuf    : ջ
 * \param[in] rlen    : յĳ
 *
 * \return ״̬
 */
uint16_t am_fmse_data_compress(am_fmse_handle_t handle,
                               uint16_t         p1p2,
                               uint16_t         inlen,
                               uint8_t         *inbuf,
                               uint8_t         *rbuf,
                               uint16_t        *rlen);

/**
 * \brief Export Public Key
 *
 * \param[in] handle  : FMSE׼
 * \param[in] p1p2    : 
 * \param[in] inlen   : file length to write
 * \param[in] inbuf   : input data buffer
 * \param[in] rbuf    : ջ
 * \param[in] rlen    : յĳ
 *
 * \return ״̬
 */
uint16_t am_fmse_pub_key_export(am_fmse_handle_t handle,
                                uint16_t         p1p2,
                                uint16_t         inlen,
                                uint8_t         *inbuf,
                                uint8_t         *rbuf,
                                uint16_t         *rlen);

/**
 * \brief Gen ECC Key Pair
 *
 * \param[in] handle  : FMSE׼
 * \param[in] p1p2    : 
 * \param[in] Fid     : the Fid that save the dest Key Pair
 *                      input 4Bytes Fid format: public key fid + private key fid
 *                      single 2Bytes Fid format:high byte + low byte
 * \param[in] rbuf    : ջ
 * \param[in] rlen    : յĳ
 *
 * \return ״̬
 */
uint16_t am_fmse_ecc_key_pair_gen(am_fmse_handle_t handle,
                                  uint16_t         p1p2,
                                  uint32_t         fid,
                                  uint8_t         *rbuf,
                                  uint16_t        *rlen);

/**
 * \brief ECC public key calculate
 *
 * \param[in] handle  : FMSE׼
 * \param[in] p1p2    : 
 * \param[in] inlen   : file length to read
 * \param[in] inbuf   : input data buffer
 * \param[in] rbuf    : the return data buffer
 * \param[in] rlen    : the return data length
 *
 * \return ret
 */
uint16_t am_fmse_ecc_pub_key_cal(am_fmse_handle_t handle,
                                 uint16_t         p1p2,
                                 uint16_t         inlen,
                                 uint8_t         *inbuf,
                                 uint8_t         *rbuf,
                                 uint16_t        *rlen);

/**
 * \brief ECC private key calculate
 *
 * \param[in] handle  : FMSE׼
 * \param[in] p1p2    : 
 * \param[in] inlen   : file length to read
 * \param[in] inbuf   : input data buffer
 * \param[in] rbuf    : the return data buffer
 * \param[in] rlen    : the return data length
 *
 * \return ret
 */
uint16_t am_fmse_ecc_pri_key_cal(am_fmse_handle_t handle,
                                 uint16_t         p1p2,
                                 uint16_t         inlen,
                                 uint8_t         *inbuf,
                                 uint8_t         *rbuf,
                                 uint16_t        *rlen);

/**
 * \brief External Authentication
 *
 * \param[in]  handle      : FMSE׼
 * \param[in]  random_send : input lacal random data 
 * \param[in]  random_get  : input random data from SE
 * \param[in]  ea_key      : input External_Authentication Key
 * \param[out] None
 *
 * \return ret
 */
uint16_t am_fmse_external_authentication (am_fmse_handle_t  handle,
                                          uint8_t          *random_send,
                                          uint8_t          *random_get,
                                          uint8_t          *ea_key);

/**
 * \brief Change PIN
 *
 * \param[in]  handle      : FMSE׼
 * \param[in]  new_pin     : input pin 
 * \param[in]  new_pin_len : input new_pin_len
 * \param[in]  old_pin     : input old_pin
 * \param[in]  old_pin_len : input old_pin_len
 *
 * \return ret
 */
uint16_t am_fmse_pin_change (am_fmse_handle_t handle,
                             uint8_t         *new_pin,
                             uint8_t          new_pin_len,
                             uint8_t         *old_pin,
                             uint8_t          old_pin_len);

/**
 * \brief Write Key
 *
 * \param[in]  handle : FMSE׼
 * \param[in]  p1p2   : 
 * \param[in]  inlen  : data length to write
 * \param[in]  inbuf  : input data buffer
 * \param[in]  rbuf   : the return data buffer
 * \param[in]  rlen   : the return data length
 *            
 * \return ret
 */
uint16_t am_fmse_key_write (am_fmse_handle_t handle,
                            uint16_t         p1p2,
                            uint16_t         inlen,
                            uint8_t         *inbuf,
                            uint8_t         *rbuf,
                            uint16_t        *rlen);

#ifdef __cplusplus
}
#endif

#endif /* __AM_FMSE_H */

/* end of file */
