/*******************************************************************************
*                                 AMetal
*                       ----------------------------
*                       innovating embedded platform
*
* Copyright (c) 2001-2018 Guangzhou ZHIYUAN Electronics Co., Ltd.
* All rights reserved.
*
* Contact information:
* web site:    http://www.zlg.cn/
*******************************************************************************/

/**
 * \file
 * \brief GS200ӿں
 *
 * \internal
 * \par Modification History
 * - 1.00 19-12-16  fzb, first implementation.
 * \endinternal
 */


#ifndef __AM_GS200_H
#define __AM_GS200_H

#ifdef __cplusplus
extern "C" {
#endif

#include "ametal.h"
#include "am_types.h"
#include "am_softimer.h"
#include "am_uart_rngbuf.h"

/**
 * @addtogroup am_if_gs200
 * @copydoc am_gs200.h
 * @{
 */

#define  AM_GS200_COM_FRAME_LEN             (0x05)  /*    */

/**
 * \name GS200ͨÿָ(֡ʶ)
 * @{
 */
#define  AM_GS200_SET_BAUDRATE              (0x01)  /* ģͨѶӿڲָ       */
#define  AM_GS200_SET_GESTURE_STATE         (0x02)  /* ģƹʹָ   */
#define  AM_GS200_SET_WORKING_CONDITION     (0x03)  /* ù״ָ̬                 */
#define  AM_GS200_SET_HOVERING_GESTURE_MODE (0x04)  /* ͣ(ʱ/)ָ  */
#define  AM_GS200_SET_HOVERING_TIMING_TIME  (0x05)  /* ͣƶʱʱ     */
#define  AM_GS200_SET_HOVERING_SINGLE_TIME  (0x06)  /* ͣƵδʱ         */
/* GS200 ڿָ */
#define  AM_GS200_NREAD                     (0x07)  /* ȡջѽյݸ */
#define  AM_GS200_NWRITE                    (0x08)  /* ȡͻдݸ */
#define  AM_GS200_FLUSH                     (0x09)  /* շͻͽջ       */
#define  AM_GS200_WFLUSH                    (0x0A)  /* շͻ                   */
#define  AM_GS200_RFLUSH                    (0x0B)  /* սջ                   */
#define  AM_GS200_TIMEOUT                   (0x0C)  /* ýճʱʱ(λms)       */

/* \brief  GS200 ѡ */
#define  AM_GS200_BAUDRATE_9600             (0x00)  // (Ĭ)
#define  AM_GS200_BAUDRATE_4800             (0x01)
#define  AM_GS200_BAUDRATE_115200           (0x02)

//ģƹʹѡ
#define  AM_GS200_SET_MOVEUP_DISABLE        (0x01)  /* ýֹGS200ƶ */
#define  AM_GS200_SET_MOVEDOWN_DISABLE      (0x02)  /* ýֹGS200ƶ */
#define  AM_GS200_SET_MOVELEFT_DISABLE      (0x03)  /* ýֹGS200ƶ */
#define  AM_GS200_SET_MOVERIGHT_DISABLE     (0x04)  /* ýֹGS200ƶ */
#define  AM_GS200_SET_HOVER_DISABLE         (0x05)  /* ýֹGS200ͣƶ */

#define  AM_GS200_SET_MOVEUP_ENABLE         (0x11)  /* ʹGS200ƶ */
#define  AM_GS200_SET_MOVEDOWN_ENABLE       (0x12)  /* ʹGS200ƶ */
#define  AM_GS200_SET_MOVELEFT_ENABLE       (0x13)  /* ʹGS200ƶ */
#define  AM_GS200_SET_MOVERIGHT_ENABLE      (0x14)  /* ʹGS200ƶ */
#define  AM_GS200_SET_HOVER_ENABLE          (0x15)  /* ʹGS200ͣƶ */

//ùģʽָ
#define  AM_GS200_SET_SLEEP_MODE            (0x01)  /* GS200˯ģʽ */
#define  AM_GS200_SET_WAKEUP                (0x02)  /* GS200ģ */

//ͣʹ
#define  AM_GS200_SET_OUTPUT_TIMING_ENABLE  (0x02)  /* GS200ָͣʱʹ */
#define  AM_GS200_SET_OUTPUT_SINGLE_ENABLE  (0x03)  /* GS200ָͣʹ */


/**
 * \brief GS200ʵϢṹ嶨
 */
typedef struct am_gs200_devinfo {

    /** \brief ģʼ        */
    uint32_t    baudrate;

    /** \brief ڴڽյĻС32     */
    uint8_t     *p_uart_rxbuf;

    /** \brief ڴڷ͵ĻС32     */
    uint8_t     *p_uart_txbuf;

    /** \brief ͻ  */
    uint8_t     *p_cmd_txbuf;

    /** \brief ڴڽյĻС */
    uint16_t    rxbuf_size;

    /** \brief ڴڷ͵ĻС */
    uint16_t    txbuf_size;

    /** \brief ͻĴС  */
    uint16_t    cmd_txbuf_len;

} am_gs200_devinfo_t;

/**
 * \brief GS200豸ṹ嶨
 */
typedef struct am_gs200_dev {

    /** \brief ״̬  */
    volatile int                cmd_proc_state;

    /** \brief UARTʵ                    */
    am_uart_handle_t            uart_handle;

    /** \brief ݻλ      */
    struct am_rngbuf            rx_rngbuf;

    /** \brief ݻλ      */
    struct am_rngbuf            tx_rngbuf;

    /** \brief ڱ豸Ϣָ */
    const am_gs200_devinfo_t *p_devinfo;

    /** \brief ڽӦĵȴṹ       */
    am_wait_t                   ack_wait;

    /** \brief ڽݵĵȴṹ   */
    am_wait_t                   rx_wait;

    /** \brief ճʱʱ                */
    volatile uint32_t           timeout_ms;

    /** \brief Ӧջ  */
    uint8_t                    *p_cmd_rxbuf;

    /** \brief Ӧջĳ  */
    volatile int                cmd_rxbuf_len;

    /** \brief ӦѾյĳ  */
    volatile int                cmd_rx_len;

    /** \brief ʱжһ֡Ƿճʱ */
    am_softimer_t               timer;
} am_gs200_dev_t;


/** \brief GS200  */
typedef struct am_gs200_dev *am_gs200_handle_t;

/**
 * \brief GS200 ʼ
 *
 * ʹGS200ǰӦȵøóʼԻȡGS200handle
 *
 * \param[in] p_dev      : ָGS200豸ָ
 * \param[in] p_devinfo  : ָGS200豸Ϣָ
 * \param[in] uart_handle : UART׼ʹøUARTGS200ͨţ
 *
 * \return GS200handleʼʧܣ򷵻ֵΪNULL
 *
 * \note 豸ָp_devָ豸ֻҪ壬Ҫڵñǰʼ
 */
am_gs200_handle_t am_gs200_init (am_gs200_dev_t             *p_dev,
                                 const am_gs200_devinfo_t   *p_devinfo,
                                 am_uart_handle_t           uart_handle);

/**
 * \brief GS200 ƺ
 *
 * \param[in] handle    : GS200ʵ
 * \param[in] cmd       : ָ磺#AM_GS200_SET_BAUDRATE
 * \param[in,out] p_arg : ӦĲ磺 115200
 *
 * \retval AM_OK       : ִָгɹ
 * \retval -AM_EIO     : ִд
 * \retval -AM_ENOTSUP : ָ֧
 *
 * \note 
 */
int am_gs200_ioctl(am_gs200_handle_t    handle,
                   int                  cmd,
                   void                 *p_arg);

/**
 * \brief 
 *
 * \param[in] handle : GS200ʵ
 * \param[in] p_buf  : ͵
 * \param[in] len    : ݳ
 *
 * \return Ǹʵʷ͵ֽʧ
 */
int am_gs200_send (am_gs200_handle_t    handle,
                   const uint8_t        *p_buf,
                   int                  len);


/**
 * \brief 
 *
 * \param[in] handle : GS200ʵ
 * \param[in] p_buf  : ݵĻ
 * \param[out] len   : ݳ
 *
 * \return Ǹʵʽյֽʧ
 */
int am_gs200_recv (am_gs200_handle_t    handle,
                   uint8_t              *p_buf,
                   int                  len);

#ifdef __cplusplus
}
#endif

#endif


