/*******************************************************************************
*                                 AMetal
*                       ----------------------------
*                       innovating embedded platform
*
* Copyright (c) 2001-2018 Guangzhou ZHIYUAN Electronics Co., Ltd.
* All rights reserved.
*
* Contact information:
* web site:    http://www.zlg.cn/
*******************************************************************************/
/**
 * \file
 * \brief ISSI˾IS25ϵIS25XXоƬ 
 * 
 * ֪оƬ 
 *   - IS25WP032
 *   - IS25WP064 
 *   - IS25WP0128
 * 
 * \internal
 * \par Modification history
 * - 1.00 18-09-03  yrz, first implementation.
 * \endinternal
 */
 
#ifndef __AM_IS25XX_H
#define __AM_IS25XX_H

#ifdef __cplusplus
extern "C" {
#endif
    
#include "ametal.h"
#include "am_types.h"
#include "am_spi.h"
#include "am_gpio.h"
#include "am_mtd.h"
    
/**
 * \addtogroup am_if_is25xx
 * \copydoc am_is25xx.h
 * @{
 */

/**
 * \brief һоƬͺ
 *
 * \param[in] page_size        : ҳСʹ2ݱʾҳСΪ256ֵΪ8
 * \param[in] pages_in_sector  : аҳĿʹ2ݱʾ
 * \param[in] sectors_in_block : аĿʹ2ݱʾ
 * \param[in] nblocks          : оƬܿ
 * \param[in] id               : оƬID
 *
 * \return оƬͺţֱΪʵϢ type Աֵ
 */
#define AM_IS25XX_TYPE_DEF(page_size,         \
                           pages_in_sector,   \
                           sectors_in_block,  \
                           nblocks,           \
                           id)                \
        {page_size, pages_in_sector, sectors_in_block, nblocks, id},

/**
 * \name ֪һЩоƬͺŶ
 *
 *     ʹõоƬͺŴ˴δ壬ʹ \sa AM_is25XX_TYPE_DEF() 궨һ
 * µоƬ ЩֱΪʵϢ type Աֵ
 * 
 * @{
 */
#define AM_IS25XX_IS25WP032     AM_IS25XX_TYPE_DEF(8, 4, 4, 6, 0x16609D)
#define AM_IS25XX_IS25WP064     AM_IS25XX_TYPE_DEF(8, 4, 4, 7, 0x17609D)
#define AM_IS25XX_IS25WP128     AM_IS25XX_TYPE_DEF(8, 4, 4, 8, 0x18609D)

#define AM_IS25XX_IS25LQ025B    AM_IS25XX_TYPE_DEF(8, 4, 3, 0, 0x09409D)
#define AM_IS25XX_IS25LQ512B    AM_IS25XX_TYPE_DEF(8, 4, 3, 1, 0x10409D)
#define AM_IS25XX_IS25LQ010B    AM_IS25XX_TYPE_DEF(8, 4, 4, 1, 0x11409D)
#define AM_IS25XX_IS25LQ020B    AM_IS25XX_TYPE_DEF(8, 4, 4, 2, 0x12409D)
#define AM_IS25XX_IS25LQ040B    AM_IS25XX_TYPE_DEF(8, 4, 4, 3, 0x13409D)

#define AM_IS25XX_IS25LP064A    AM_IS25XX_TYPE_DEF(8, 4, 4, 7, 0x17609D)  
/** @} */

/**
 * \brief ͺţҳСϢ
 */
typedef struct am_is25xx_type {

    uint8_t   page_size;        /**< \brief ҳСʹ2ݱʾ  */
    uint8_t   pages_in_sector;  /**< \brief ҳʹ2ݱʾ */
    uint8_t   sectors_in_block; /**< \brief ҳʹ2ݱʾ */
    uint8_t   nblocks;          /**< \brief оƬܿʹ2ݱʾ  */

    /**
     * IDɣmanufacture_id + memory type + memory density
     * 磺IS25LP064 manufacture_id memory type memory densityֱΪ
     * 0x9D0x600x17ֵΪ0x17609D
     */
    uint32_t  id;
 
} am_is25xx_type_t;
 
/**
 * \brief ISSI25XX ʵϢ
 */
typedef struct am_is25xx_devinfo {
 
    uint16_t          spi_mode;      /**< \brief ʹõSPIģʽ */
    int               spi_cs_pin;    /**< \brief SPIƬѡ */
    uint32_t          spi_speed;     /**< \brief ʹõSPI */
    am_is25xx_type_t  type;          /**< \brief ͺ */
 
} am_is25xx_devinfo_t;
     
/**
 * \brief ISSI25XX ʵ
 */
typedef struct am_is25xx_dev {
    am_spi_device_t            spi_dev;        /**< \brief SPI豸 */
    uint32_t                   addr_offset;    /**< \brief ַռ */
    const am_is25xx_devinfo_t *p_devinfo;      /**< \brief ʵϢ */
} am_is25xx_dev_t;

/** \brief  ISSI25XX ʵ */
typedef struct am_is25xx_dev *am_is25xx_handle_t;


/** 
 * \brief ʼ IS25XXȡ IS25XX  handle
 * 
 * \param[in] p_dev      : ָ IS25XX ʵָ
 * \param[in] p_devinfo  : ָ IS25XX ʵϢָ
 * \param[in] handle     : SPI׼handle,ʹø handle  IS25XX ͨ
 *
 * \return IS25XX ʼʧܣ򷵻ֵΪNULL
 */
am_is25xx_handle_t am_is25xx_init(am_is25xx_dev_t            *p_dev,
                                  const am_is25xx_devinfo_t  *p_devinfo,
                                  am_spi_handle_t             handle);

/**
 * \brief IS25xx MTD ܳʼ
 *
 * ʼ󣬼ʹMTD׼ӿڶ IS25xxв
 *
 * \param[in] handle         : IS25XX 
 * \param[in] p_mtd          : ָ MTD ʵָ
 * \param[in] reserved_nblks : Ŀʼ reserved_nblks 鲻ᱻMTDʹá
 *
 * \return MTDʵ
 *
 */
am_mtd_handle_t am_is25xx_mtd_init(am_is25xx_handle_t  handle,
                                   am_mtd_serv_t      *p_mtd,
                                   uint32_t            reserved_nblks);


/**
 * \brief 
 *
 *    ҪǽȫΪ0xFFд洢ԪλΪ1
 * ֱӲĳһַԪСԪΪÿֻܲ
 * ڲСԪΪˣʼַΪĳʼַ
 * ȱΪС
 *
 * \param[in] handle : IS25XX 
 * \param[in] addr   : ׵ַΪĳʼַ
 * \param[in] len    : ĳȣΪС
 *
 * \retval  AM_OK     : ɹ
 * \retval -AM_EINVAL : ʧ, 
 * \retval -AM_EIO    : ʧ, SPIͨų
 */
int am_is25xx_erase(am_is25xx_handle_t  handle,
                    uint32_t            addr,
                    uint32_t            len);


/**
 * \brief ȡ
 *
 * \param[in] handle : IS25XX 
 * \param[in] addr   : ȡݵ׵ַ
 * \param[in] p_buf  : ȡݴŵĻ
 * \param[in] len    : ݶȡĳ
 *
 * \retval  AM_OK     : ȡݳɹ
 * \retval -AM_EINVAL : ȡʧ, 
 * \retval -AM_EIO    : ȡʧ, SPIͨų
 */
int am_is25xx_read(am_is25xx_handle_t  handle,
                   uint32_t            addr,
                   uint8_t            *p_buf,
                   uint32_t            len);


/** 
 * \brief д
 *
 * \param[in] handle : IS25XX 
 * \param[in] addr   : дݵ׵ַ
 * \param[in] p_buf  : дݴŵĻ
 * \param[in] len    : ݶȡĳ
 *
 * \retval  AM_OK     : ȡݳɹ
 * \retval -AM_EINVAL : ȡʧ, 
 * \retval -AM_EIO    : ȡʧ, SPIͨų
 */
int am_is25xx_write(am_is25xx_handle_t  handle,
                    uint32_t            addr,
                    uint8_t            *p_buf,
                    uint32_t            len);

/**
 * \brief ȡ JEDEC ID
 *
 * IDɣmanufacture_id + memory type + memory density
 * 
 * \param[in]  handle   : IS25XX 
 * \param[out] p_id     : JEDEC ID
 *
 * \retval  AM_OK     : ȡɹ
 * \retval -AM_EINVAL : ȡʧ, 
 * \retval -AM_EIO    : ȡʧ, SPIͨų
 */ 
int am_is25xx_id_read(am_is25xx_handle_t handle, uint32_t *p_id);
                                        
/** 
 * \brief ʹREMSȡ ID1ֽڣID1ֽڣ
 * 
 * \param[in]  handle           : IS25XX 
 * \param[out] p_manufacture_id : ID1ֽڣ
 * \param[out] p_device_id      : 豸ID1ֽڣ
 *
 * \retval  AM_OK     : ȡɹ
 * \retval -AM_EINVAL : ȡʧ, 
 * \retval -AM_EIO    : ȡʧ, SPIͨų
 */                                        
int am_is25xx_id_rems_read(am_is25xx_handle_t  handle, 
                           uint8_t            *p_manufacture_id,
                           uint8_t            *p_device_id);
                            
/** 
 * \brief ȡIS25XX״̬Ĵֵ
 * 
 * \param[in]  handle : IS25XX 
 * \param[out] p_stat : ȡ״̬Ĵֵ
 *
 * \retval  AM_OK     : ȡ״̬Ĵɹ
 * \retval -AM_EINVAL : ȡʧ, 
 * \retval -AM_EIO    : ȡʧ, SPIͨų
 */
int am_is25xx_status_read(am_is25xx_handle_t  handle,
                          uint8_t            *p_stat);

/** 
 * \brief IS25XX״̬Ĵֵ
 * 
 * \param[in] handle : IS25XX 
 * \param[in] val    : д״̬Ĵֵ
 *
 * \retval  AM_OK     : д״̬Ĵɹ
 * \retval -AM_EINVAL : дʧ, 
 * \retval -AM_EIO    : дʧ, SPIͨų
 *
 * \note Ӧã򲻽ʹøúֱд״̬Ĵֵ
 */
int am_is25xx_status_write(am_is25xx_handle_t  handle,
                           uint8_t             val);

/** 
 * \brief ģʽ͹ģ
 * 
 * \param[in] handle : IS25XX 
 *
 * \retval  AM_OK     : ģʽɹ
 * \retval -AM_EINVAL : ģʽʧ, 
 * \retval -AM_EIO    : ģʽʧ, SPIͨų
 */                              
int am_is25xx_deep_power_down_enter(am_is25xx_handle_t  handle);

/** 
 * \brief ˳ģʽ
 * 
 * \param[in] handle : IS25XX 
 *
 * \retval  AM_OK     : ģʽɹ
 * \retval -AM_EINVAL : ģʽʧ, 
 * \retval -AM_EIO    : ģʽʧ, SPIͨų
 */
int am_is25xx_deep_power_down_exit(am_is25xx_handle_t  handle);

/** 
 * \brief IS25XXʼͷԴ
 * 
 *     ʼIS25XXغʹãʹIS25XXȡ
 * NVRAM׼handle
 * 
 * \param[in] p_dev : ָ IS25XX 豸ָ
 *
 * \return 
 */
void am_is25xx_deinit(am_is25xx_dev_t *p_dev);

/**
 * @}
 */

#ifdef __cplusplus
}
#endif
 
#endif /* __AM_IS25XX_H */

/* end of file */
