/*******************************************************************************
*                                 AMetal
*                       ----------------------------
*                       innovating embedded platform
*
* Copyright (c) 2001-2018 Guangzhou ZHIYUAN Electronics Co., Ltd.
* All rights reserved.
*
* Contact information:
* web site:    http://www.zlg.cn/
*******************************************************************************/

/**
 * \file
 * \brief MK100ײ
 * 
 * \internal
 * \par Modification history
 * - 1.00 19-12-20  mxy, first implementation.
 * \endinternal
 */
#ifndef __AM_MK100_H
#define __AM_MK100_H

#ifdef __cplusplus
extern "C" {
#endif

#include "am_uart_rngbuf.h"
#include "am_uart.h"

/**
 * \addtogroup am_mk100
 * \copydoc am_mk100.h
 * @{
 */
    
#define AM_MK100_DEFAULT_BAUDRATE                   9600          /**< \brief  Ĭϲ9600 */
#define AM_MK100_FRAME_LEN                          5             /**< \brief  ֡5ֽ */
#define AM_MK100_DIR_CW                             0x00          /**< \brief  ˳ʱ뷽 */
#define AM_MK100_DIR_CCW                            0x01          /**< \brief  ʱ뷽 */

#define AM_MK100_DIRECTION_UP                       0x00          /**< \brief   */
#define AM_MK100_DIRECTION_DOWN                     0x01          /**< \brief   */ 
#define AM_MK100_DIRECTION_LEFT                     0x02          /**< \brief   */
#define AM_MK100_DIRECTION_RIGHT                    0x03          /**< \brief   */

#define AM_MK100_BAUDRATE_TEST                      0x00          /**< \brief  òԲ */
#define AM_MK100_BAUDRATE_2400                      0x01          /**< \brief  2400 */
#define AM_MK100_BAUDRATE_4800                      0x02          /**< \brief  4800 */
#define AM_MK100_BAUDRATE_9600                      0x03          /**< \brief  9600 */
#define AM_MK100_BAUDRATE_56000                     0x04          /**< \brief  56000 */
#define AM_MK100_BAUDRATE_115200                    0x05          /**< \brief  115200 */

#define AM_MK100_SET_ROTATION_FUNC                  0x00          /**< \brief  ת */
#define AM_MK100_SET_INFRARED_DETECT                0x10          /**< \brief  ת+¼⣨⣩*/
#define AM_MK100_SET_STATUS_DETECT                  0x20          /**< \brief  ת+¼⣨״̬⣩ */
#define AM_MK100_SET_LEFT_RIGNH_DETECT              0x33          /**< \brief  ʹҷƶ */
#define AM_MK100_SET_UP_DOWN_DETECT                 0x3C          /**< \brief  ʹ·ƶ */
#define AM_MK100_SET_UP_DOWN_LEFT_RIGNH_DETECT      0x3F          /**< \brief  ʹƶ */

#define AM_MK100_GET_ANGLE                          0x2F          /**< \brief  жԶϱģʽݣʾǽǶֵ */
#define AM_MK100_GET_KNOB_POS                       0x2E          /**< \brief  жԶϱģʽݣʾǴťλ */
/**
 * \brief mk100 豸Ϣ
 */
typedef struct am_mk100_devinfo {
    /** \brief ڴڽյĻС32     */
    uint8_t     *p_uart_rxbuf;

    /** \brief ڴڷ͵ĻС32     */
    uint8_t     *p_uart_txbuf;

    /** \brief ڴڽյĻС */
    uint16_t     rxbuf_size;

    /** \brief ڴڷ͵ĻС */
    uint16_t     txbuf_size;

} am_mk100_devinfo_t;

/**
 * \brief mk100豸ʵ
 */
typedef struct am_mk100_dev {
    /** \brief UART λʵ                    */
    am_uart_rngbuf_handle_t uart_rngbuf_handle;
    /** \brief UARTʵ                    */
    am_uart_handle_t uart_handle;
    /** \brief                     */
    uint8_t error_frame[AM_MK100_FRAME_LEN];
    /** \brief ڱ豸Ϣָ */
    const am_mk100_devinfo_t *p_devinfo;
} am_mk100_dev_t;

/**
 * \brief mk100豸
 */
typedef struct am_mk100_dev *am_mk100_handle_t;
/**
 * \brief mk100ʼ
 *
 * \param[in] p_dev              : mk100豸ָ
 * \param[in] p_devinfo          : mk100豸Ϣָ
 * \param[in] uart_handle        : uart
 *
 * \return    am_mk100_handle_t  mk100
 */
am_mk100_handle_t am_mk100_init (am_mk100_dev_t           *p_dev,
                                 const am_mk100_devinfo_t *p_devinfo,
                                 am_uart_handle_t          uart_handle);
/**
 * \brief ȡǶֵ
 *
 * \param[in] handle                  : mk100
 * \param[in] gear                    : ȼ
 * \return    -AM_EINVAL              : 
 *            >=0                     : Ƕֵ
 */
int am_mk100_get_angle(am_mk100_handle_t handle, uint8_t gear);
/**
 * \brief ȡťλ
 *
 * \param[in] handle                  : mk100
 * \return    -AM_EINVAL              : 
 *            >=0                     : ťλ
 */             
int am_mk100_get_knob(am_mk100_handle_t handle);
/**
 * \brief ȡǰųǿֵupper_limitlower_limitָڴ
 *
 * \param[in] handle                  : mk100
 * \param[in] upper_limit             : 
 * \param[in] lower_limit             : 
 * \return    -AM_EINVAL              : 
 *            AM_OK                   : ɹ
 */                                 
int am_mk100_get_strength_threshold(am_mk100_handle_t handle, uint8_t* upper_limit, uint8_t* lower_limit);
/**
 * \brief ȡǰת
 *
 * \param[in] handle                  : mk100
 * \return    -AM_EINVAL              : 
 *            >=0                     : ǰת
 */
int am_mk100_get_rotation_direction(am_mk100_handle_t handle);
/**
 * \brief ȡǰųǿֵ״̬upper_limit_statuslower_limit_statusָڴ
 *
 * \param[in] handle                  : mk100
 * \param[in] upper_limit_status      : 0x01 ʾǰǿѴﵽޣ 0x00 ʾǰǿδﵽ
 * \param[in] lower_limit_status      : 0x01 ʾǰǿѴﵽޣ 0x00 ʾǰǿδﵽ
 * \return    -AM_EINVAL              : 
 *            AM_OK                   : ɹ
 */
int am_mk100_get_strength_threshold_status(am_mk100_handle_t handle, am_bool_t* upper_limit_status, am_bool_t* lower_limit_status);
/**
 * \brief ȡ򴫸
 *
 * \param[in] handle                  : mk100
 * \param[in] direction               : ҷֱΪAM_MK100_DIRECTION_UPAM_MK100_DIRECTION_DOWNAM_MK100_DIRECTION_LEFTAM_MK100_DIRECTION_RIGHT    
 * \return    -AM_EINVAL              : 
 *            >= 0                    : ÷
 */
int am_mk100_get_direction_data(am_mk100_handle_t handle, uint8_t direction);
/**
 * \brief ȡ⴫
 *
 * \param[in] handle                  : mk100
 * \return    -AM_EINVAL              : 
 *            >= 0                    : ⴫
 */
int am_mk100_get_infrared_data(am_mk100_handle_t handle);
/**
 * \brief õǰǶֵ
 *
 * \param[in] handle                  : mk100
 * \param[in] angle                   : Ƕֵ0~360
 * \return    -AM_EINVAL              : 
 *            >= 0                    : ǰõĽǶֵ
 */
int am_mk100_set_angle(am_mk100_handle_t handle, uint16_t angle);
/**
 * \brief õǰųǿֵ
 *
 * \param[in] handle                  : mk100
 * \param[in] limit                   : ֵޣֵΪ0x07
 * \return    -AM_EINVAL              : 
 *            >= 0                    : ǰõֵ
 */
int am_mk100_set_strength_upper_limit(am_mk100_handle_t handle, uint8_t limit);
/**
 * \brief õǰųǿֵ
 *
 * \param[in] handle                  : mk100
 * \param[in] limit                   : ֵޣֵΪ0x07
 * \return    -AM_EINVAL              : 
 *            >= 0                    : ǰõֵ
 */
int am_mk100_set_strength_lower_limit(am_mk100_handle_t handle, uint8_t limit);
/**
 * \brief õǰת
 *
 * \param[in] handle                  : mk100
 * \param[in] direction               : AM_MK100_DIR_CW: ˳ʱ롢AM_MK100_DIR_CCW :ʱ
 * \return    -AM_EINVAL              : 
 *            >= 0                    : ǰõķ
 */
int am_mk100_set_rotation_direction(am_mk100_handle_t handle, uint8_t direction);
/**
 * \brief 򴫸ʼֵУ׼
 *
 * \param[in] handle                  : mk100
 * \return    -AM_EINVAL              : 
 *            AM_OK                   : ɹ
 */
int am_mk100_direction_cal(am_mk100_handle_t handle);
/**
 * \brief ÷򴫸
 *
 * \param[in] handle                  : mk100
 * \param[in] threshold               : 򴫸
 * \return    -AM_EINVAL              : 
 *            >= 0                    : ǰõķ򴫸
 */
int am_mk100_set_direction_parameter(am_mk100_handle_t handle, uint8_t direction, uint16_t threshold);
/**
 * \brief ú⴫
 *
 * \param[in] handle                  : mk100
 * \param[in] data                    : ⴫
 * \return    -AM_EINVAL              : 
 *            >= 0                    : ǰõĺ⴫
 */
int am_mk100_set_infrared_parameter(am_mk100_handle_t handle, uint16_t data);
/**
 * \brief ϵͳ
 *
 * \param[in] handle                  : mk100
 * \param[in] baudrate_setting        : ѡֻAM_MK100_BAUDRATE_***Щ궨
 * \return    -AM_EINVAL              : 
 *            >= 0                    : ǰõĲ
 */
int am_mk100_set_baudrate(am_mk100_handle_t handle, uint8_t baudrate_setting);
/**
 * \brief ϵͳ
 *
 * \param[in] handle                  : mk100
 * \param[in] mode                    : ѡֻЧ룺
 *                                      AM_MK100_SET_ROTATION_FUNC
 *                                      AM_MK100_SET_INFRARED_DETECT
 *                                      AM_MK100_SET_STATUS_DETECT
 *                                      AM_MK100_SET_LEFT_RIGNH_DETECT
 *                                      AM_MK100_SET_UP_DOWN_DETECT
 *                                      AM_MK100_SET_UP_DOWN_LEFT_RIGNH_DETECT
 * \return    -AM_EINVAL              : 
 *            >= 0                    : ǰ
 */
int am_mk100_set_system_func(am_mk100_handle_t handle, uint8_t mode);
/**
 * \brief ϵͳΪѯģʽ
 *
 * \param[in] handle                  : mk100
 * \return    -AM_EINVAL              : 
 *            >= 0                    : ǰģʽ
 */
int am_mk100_set_query_mode(am_mk100_handle_t handle);
/**
 * \brief ϵͳΪϱģʽ, иıʱɺҪam_mk100_get_reportȡݣʱݶȡ
 *
 * \param[in] handle                  : mk100
 * \return    -AM_EINVAL              : 
 *            >= 0                    : ǰģʽ
 */
int am_mk100_set_report_change_mode(am_mk100_handle_t handle);
/**
 * \brief ϵͳΪϱģʽ, ̶ƵɺҪam_mk100_get_reportȡݣʱݶȡ
 *
 * \param[in] handle                  : mk100
 * \param[in] time                    : ̶ʱ,λΪ msΧΪ 50 - 2000
 * \return    -AM_EINVAL              : 
 *            >= 0                    : ǰģʽ
 */
int am_mk100_set_report_periodically_mode(am_mk100_handle_t handle, uint16_t time);
/**
 * \brief ť
 *
 * \param[in] handle                  : mk100
 * \param[in] sensitivity             : ȵȼѡֵķΧΪ 2-50
 * \return    -AM_EINVAL              : 
 *            >= 0                    : ǰȵȼ
 */
int am_mk100_set_knob_sensitivity(am_mk100_handle_t handle, uint8_t sensitivity);
/**
 * \brief ʱڿ
 *
 * \param[in] handle                  : mk100
 * \param[in] delay_window            : ʱڿȣλΪʱ 10msֵķΧΪ 10-100õֵΪ 50˴ʱΪ 500ms
 * \return    -AM_EINVAL              : 
 *            >= 0                    : ǰʱڿ
 */
int am_mk100_set_delay_window(am_mk100_handle_t handle, uint8_t delay_window);
/**
 * \brief ý͹ʱ
 *
 * \param[in] handle                  : mk100
 * \param[in] enter_low_power_time    : ͹ʱ䣬λΪ, ֵķΧΪ 0-255
 * \return    -AM_EINVAL              : 
 *            >= 0                    : ǰ趨ʱ
 */
int am_mk100_set_enter_low_power_time(am_mk100_handle_t handle, uint8_t enter_low_power_time);
/**
 * \brief ȡԶϱģʽʱMK100ص֡
 *
 * \param[in] handle                  : mk100
 * \param[in] p_type                  : ָŷݵֱͣΪǶݻťλ
 * \param[in] p_data                  : ָ
 * \return    -AM_EINVAL              : 
 *            AM_OK                   : ɹ
 */
int am_mk100_get_report(am_mk100_handle_t handle, uint8_t* p_type, uint8_t *p_data);
/**
 * \brief ӡ
 *
 * \param[in] handle              : mk100
 * \return     
 */
void am_mk100_display_error_recevice(am_mk100_handle_t handle);

#ifdef __cplusplus
}
#endif

#endif /* __AM_MK100_H */

/* end of file */
