/*******************************************************************************
*                                 AMetal
*                       ----------------------------
*                       innovating embedded platform
*
* Copyright (c) 2001-2018 Guangzhou ZHIYUAN Electronics Co., Ltd.
* All rights reserved.
*
* Contact information:
* web site:    http://www.zlg.cn/
*******************************************************************************/
/**
 * \file
 * \brief 깫˾MX25ϵMX25XXоƬ 
 * 
 * ֪оƬ 
 *   - MX25L1606E
 *   - MX25L8006E
 * 
 * \internal
 * \par Modification history
 * - 1.00 15-09-14  tee, first implementation.
 * \endinternal
 */
 
#ifndef __AM_MX25XX_H
#define __AM_MX25XX_H

#ifdef __cplusplus
extern "C" {
#endif
    
#include "ametal.h"
#include "am_types.h"
#include "am_spi.h"
#include "am_gpio.h"
#include "am_mtd.h"
    
/**
 * \addtogroup am_if_mx25xx
 * \copydoc am_mx25xx.h
 * @{
 */

/**
 * \brief һоƬͺ
 *
 * \param[in] page_size        : ҳСʹ2ݱʾҳСΪ256ֵΪ8
 * \param[in] pages_in_sector  : аҳĿʹ2ݱʾ
 * \param[in] sectors_in_block : аĿʹ2ݱʾ
 * \param[in] nblocks          : оƬܿ
 * \param[in] id               : оƬID
 *
 * \return оƬͺţֱΪʵϢ type Աֵ
 */
#define AM_MX25XX_TYPE_DEF(page_size,         \
                           pages_in_sector,   \
                           sectors_in_block,  \
                           nblocks,           \
                           id)                \
        {page_size, pages_in_sector, sectors_in_block, nblocks, id},

/**
 * \name ֪һЩоƬͺŶ
 *
 *     ʹõоƬͺŴ˴δ壬ʹ \sa AM_MX25XX_TYPE_DEF() 궨һ
 * µоƬ ЩֱΪʵϢ type Աֵ
 * 
 * @{
 */
#define AM_MX25XX_MX25L8006     AM_MX25XX_TYPE_DEF(8, 4, 4, 4, 0x1420C2)
#define AM_MX25XX_MX25L1606     AM_MX25XX_TYPE_DEF(8, 4, 4, 5, 0x1520C2)

/** @} */

/**
 * \brief ͺţҳСϢ
 */
typedef struct am_mx25xx_type {

    uint8_t   page_size;         /**< \brief ҳСʹ2ݱʾ  */
    uint8_t   pages_in_sector;   /**< \brief ҳʹ2ݱʾ  */
    uint8_t   sectors_in_block;  /**< \brief ҳʹ2ݱʾ  */
    uint8_t   nblocks;           /**< \brief оƬܿʹ2ݱʾ  */

    /**
     * IDɣmanufacture_id + memory type + memory density
     * 磺MX25L1606 manufacture_id memory type memory densityֱΪ
     * 0xC20x200x15ֵΪ0x1520C2
     */
    uint32_t  id;
 
} am_mx25xx_type_t;
 
/**
 * \brief MX25XX ʵϢ
 */
typedef struct am_mx25xx_devinfo {
 
    uint16_t          spi_mode;      /**< \brief ʹõSPIģʽ */
    int               spi_cs_pin;    /**< \brief SPIƬѡ                */
    uint32_t          spi_speed;     /**< \brief ʹõSPI           */
    am_mx25xx_type_t  type;          /**< \brief ͺ                        */
 
} am_mx25xx_devinfo_t;
     
/**
 * \brief MX25XX ʵ
 */
typedef struct am_mx25xx_dev {
    am_spi_device_t            spi_dev;        /**< \brief SPI豸              */
    uint32_t                   addr_offset;    /**< \brief ַռ  */
    const am_mx25xx_devinfo_t *p_devinfo;      /**< \brief ʵϢ            */
} am_mx25xx_dev_t;

/** \brief  MX25XX ʵ */
typedef struct am_mx25xx_dev *am_mx25xx_handle_t;


/** 
 * \brief ʼ MX25XXȡ MX25XX  handle
 * 
 * \param[in] p_dev      : ָ MX25XX ʵָ
 * \param[in] p_devinfo  : ָ MX25XX ʵϢָ
 * \param[in] handle     : SPI׼handle,ʹø handle  MX25XX ͨ
 *
 * \return MX25XX ʼʧܣ򷵻ֵΪNULL
 */
am_mx25xx_handle_t am_mx25xx_init(am_mx25xx_dev_t            *p_dev,
                                  const am_mx25xx_devinfo_t  *p_devinfo,
                                  am_spi_handle_t             handle);

/**
 * \brief MX25xx MTD ܳʼ
 *
 * ʼ󣬼ʹMTD׼ӿڶ MX25xxв
 *
 * \param[in] handle         : MX25XX 
 * \param[in] p_mtd          : ָ MTD ʵָ
 * \param[in] reserved_nblks : Ŀʼ reserved_nblks 鲻ᱻMTDʹá
 *
 * \return MTDʵ
 *
 */
am_mtd_handle_t am_mx25xx_mtd_init(am_mx25xx_handle_t  handle,
                                   am_mtd_serv_t      *p_mtd,
                                   uint32_t            reserved_nblks);


/**
 * \brief 
 *
 *    ҪǽȫΪ0xFFд洢ԪλΪ1
 * ֱӲĳһַԪСԪΪÿֻܲ
 * ڲСԪΪˣʼַΪĳʼַ
 * ȱΪС
 *
 * \param[in] handle : MX25XX 
 * \param[in] addr   : ׵ַΪĳʼַ
 * \param[in] len    : ĳȣΪС
 *
 * \retval  AM_OK     : ɹ
 * \retval -AM_EINVAL : ʧ, 
 * \retval -AM_EIO    : ʧ, SPIͨų
 */
int am_mx25xx_erase(am_mx25xx_handle_t  handle,
                    uint32_t            addr,
                    uint32_t            len);


/**
 * \brief ȡ
 *
 * \param[in] handle : MX25XX 
 * \param[in] addr   : ȡݵ׵ַ
 * \param[in] p_buf  : ȡݴŵĻ
 * \param[in] len    : ݶȡĳ
 *
 * \retval  AM_OK     : ȡݳɹ
 * \retval -AM_EINVAL : ȡʧ, 
 * \retval -AM_EIO    : ȡʧ, SPIͨų
 */
int am_mx25xx_read(am_mx25xx_handle_t  handle,
                   uint32_t            addr,
                   uint8_t            *p_buf,
                   uint32_t            len);


/** 
 * \brief д
 *
 * \param[in] handle : MX25XX 
 * \param[in] addr   : дݵ׵ַ
 * \param[in] p_buf  : дݴŵĻ
 * \param[in] len    : ݶȡĳ
 *
 * \retval  AM_OK     : ȡݳɹ
 * \retval -AM_EINVAL : ȡʧ, 
 * \retval -AM_EIO    : ȡʧ, SPIͨų
 */
int am_mx25xx_write(am_mx25xx_handle_t  handle,
                    uint32_t            addr,
                    uint8_t            *p_buf,
                    uint32_t            len);

/**
 * \brief ȡ JEDEC ID
 *
 * IDɣmanufacture_id + memory type + memory density
 * 
 * \param[in]  handle   : MX25XX 
 * \param[out] p_id     : JEDEC ID
 *
 * \retval  AM_OK     : ȡɹ
 * \retval -AM_EINVAL : ȡʧ, 
 * \retval -AM_EIO    : ȡʧ, SPIͨų
 */ 
int am_mx25xx_id_read(am_mx25xx_handle_t handle, uint32_t *p_id);
                                        
/** 
 * \brief ʹREMSȡ ID1ֽڣID1ֽڣ
 * 
 * \param[in]  handle           : MX25XX 
 * \param[out] p_manufacture_id : ID1ֽڣ
 * \param[out] p_device_id      : 豸ID1ֽڣ
 *
 * \retval  AM_OK     : ȡɹ
 * \retval -AM_EINVAL : ȡʧ, 
 * \retval -AM_EIO    : ȡʧ, SPIͨų
 */                                        
int am_mx25xx_id_rems_read(am_mx25xx_handle_t  handle, 
                           uint8_t            *p_manufacture_id,
                           uint8_t            *p_device_id);
                            
/** 
 * \brief ȡMX25XX״̬Ĵֵ
 * 
 * \param[in]  handle : MX25XX 
 * \param[out] p_stat : ȡ״̬Ĵֵ
 *
 * \retval  AM_OK     : ȡ״̬Ĵɹ
 * \retval -AM_EINVAL : ȡʧ, 
 * \retval -AM_EIO    : ȡʧ, SPIͨų
 */
int am_mx25xx_status_read(am_mx25xx_handle_t  handle,
                          uint8_t            *p_stat);

/** 
 * \brief MX25XX״̬Ĵֵ
 * 
 * \param[in] handle : MX25XX 
 * \param[in] val    : д״̬Ĵֵ
 *
 * \retval  AM_OK     : д״̬Ĵɹ
 * \retval -AM_EINVAL : дʧ, 
 * \retval -AM_EIO    : дʧ, SPIͨų
 *
 * \note Ӧã򲻽ʹøúֱд״̬Ĵֵ
 */
int am_mx25xx_status_write(am_mx25xx_handle_t  handle,
                           uint8_t             val);

/** 
 * \brief ģʽ͹ģ
 * 
 * \param[in] handle : MX25XX 
 *
 * \retval  AM_OK     : ģʽɹ
 * \retval -AM_EINVAL : ģʽʧ, 
 * \retval -AM_EIO    : ģʽʧ, SPIͨų
 */                              
int am_mx25xx_deep_power_down_enter(am_mx25xx_handle_t  handle);

/** 
 * \brief ˳ģʽ
 * 
 * \param[in] handle : MX25XX 
 *
 * \retval  AM_OK     : ģʽɹ
 * \retval -AM_EINVAL : ģʽʧ, 
 * \retval -AM_EIO    : ģʽʧ, SPIͨų
 */
int am_mx25xx_deep_power_down_exit(am_mx25xx_handle_t  handle);

/** 
 * \brief 밲ȫOTP󣬿ʹöдOTP
 * 
 * \param[in] handle : MX25XX 
 *
 * \retval  AM_OK     : 밲ȫɹ
 * \retval -AM_EINVAL : 밲ȫʧ, 
 * \retval -AM_EIO    : 밲ȫʧ, SPIͨų
 *
 * \note ֻܶȡֵ޸ֵ
 */
int am_mx25xx_secured_otp_enter(am_mx25xx_handle_t handle);

/**
 * \brief ˳ȫOTP
 * 
 * \param[in] handle : MX25XX 
 *
 * \retval  AM_OK     : ˳ȫɹ
 * \retval -AM_EINVAL : ˳ȫʧ, 
 * \retval -AM_EIO    : ˳ȫʧ, SPIͨų
 */
int am_mx25xx_secured_otp_exit(am_mx25xx_handle_t handle);

/** 
 * \brief ⰲȫOTPǷѱ
 * 
 * \param[in]  handle     : MX25XX 
 * \param[out] p_lockdown : ȡǰ״̬AM_TRUE, AM_FALSE,δ
 *
 * \retval  AM_OK     : ɹ
 * \retval -AM_EINVAL : ʧ, 
 * \retval -AM_EIO    : ʧ, SPIͨų
 */   
int am_mx25xx_secured_lockdown_check(am_mx25xx_handle_t   handle,
                                     am_bool_t           *p_lockdown);

/** 
 * \brief ȫOTP󣬽޷Ҳ޷޸OTPе
 * 
 * \param[in] handle : MX25XX 
 *
 * \retval  AM_OK     : ɹ
 * \retval -AM_EINVAL : ʧ, 
 * \retval -AM_EIO    : ʧ, SPIͨų
 */   
int am_mx25xx_secured_lockdown(am_mx25xx_handle_t handle);


/** 
 * \brief MX25XXʼͷԴ
 * 
 *     ʼMX25XXغʹãʹMX25XXȡ
 * NVRAM׼handle
 * 
 * \param[in] p_dev : ָ MX25XX 豸ָ
 *
 * \return 
 */
void am_mx25xx_deinit(am_mx25xx_dev_t *p_dev);

/**
 * @}
 */

#ifdef __cplusplus
}
#endif
 
#endif /* __AM_MX25XX_H */

/* end of file */
