/*******************************************************************************
*                                 AMetal
*                       ----------------------------
*                       innovating embedded platform
*
* Copyright (c) 2001-2018 Guangzhou ZHIYUAN Electronics Co., Ltd.
* All rights reserved.
*
* Contact information:
* web site:    http://www.zlg.cn/
*******************************************************************************/

/**
 * \file
 * \brief  RX8025SA Ӧýӿļ
 *
 * \internal
 * \par Modification History
 * - 1.00 17-08-07  vir, first implementation.
 * \endinternal
 */

#ifndef __AM_RX8025SA_H
#define __AM_RX8025SA_H

#ifdef __cplusplus
extern "C" {
#endif

/**
 * @addtogroup am_if_rx8025sa
 * @copydoc am_rx8025sa.h
 * @{
 */
 
#include "ametal.h"
#include "am_time.h"
#include "am_rtc.h"
#include "am_alarm_clk.h"
#include "am_i2c.h"

/**
 * \brief RX8025SA 豸Ϣṹ
 *
 * \note Ҫź
 */
typedef struct am_rx8025sa_devinfo {

    /** \brief INTAӵGPIOű */
    uint8_t inta_pin;

    /** \brief INTAӵGPIOű */
    uint8_t intb_pin;
    /**
     * \brief RX8025SAFOE(CLK_EN)ӵGPIOű
     *  ڿFOUTǷƵ,ҪʱֵΪ -1
     */
    int clk_en_pin;

} am_rx8025sa_devinfo_t;

/**
 * \brief RX8025SA 豸ṹ
 */
typedef struct am_rx8025sa_dev {

    am_bool_t         conn_stat_inta;  /**< \brief ǰINTAж״̬ */
    uint8_t           time_format;            /** \brief Сʱ */

    /* ±ҪжждоƬļĴ */
    uint8_t           status;     /**< \brief ǰ״̬ */
    uint8_t           intr_flag;  /**< \brief жϱ־ */
    uint8_t           sub_addr;   /**< \brief I2Cӵַ */
    uint8_t           buf[1];     /**< \brief I2Cݻ */
    uint8_t           nbytes;     /**< \brief ݳ */
    am_i2c_transfer_t trans[2];   /**< \brief RX8025SA״̬transfer */
    am_i2c_message_t  msg;        /**< \brief RX8025SA״̬msg */

    am_i2c_device_t  i2c_dev;     /**< \brief RX8025SA I2C豸 */

    /* INTŵĴϢ */
    struct am_rx8025sa_trigger_info {

        /** \brief ص */
        am_pfnvoid_t  pfn_callback;

        /** \brief صĲ */
        void *p_arg;

    } triginfo[3];

    const am_rx8025sa_devinfo_t * p_devinfo;  /* ָ豸Ϣָ */

} am_rx8025sa_dev_t;

/** \brief RX8025SA */
typedef am_rx8025sa_dev_t *am_rx8025sa_handle_t;

/**
 * \brief RX8025SA 豸ʼ
 *
 * \param[in] p_dev      : ָRX8025SA豸ṹָ
 * \param[in] p_devinfo  : ָRX8025SA豸Ϣָ
 * \param[in] i2c_handle : I2C׼
 *
 * \return RX8025SA,Ϊ NULLʼʧ
 *
 * \note ҪƵܣ豸Ϣṹеclk_en_pin ֵΪ -1 
 */
am_rx8025sa_handle_t am_rx8025sa_init (am_rx8025sa_dev_t           *p_dev,
                                       const am_rx8025sa_devinfo_t *p_devinfo,
                                       am_i2c_handle_t              i2c_handle);

/**
 * \brief RX8025SA 豸ʼ
 * \param[in] handle : RX8025SA
 * \return 
 */
void am_rx8025sa_deinit (am_rx8025sa_handle_t handle);

/**
 * \brief ȡRTC׼
 *
 * \param[in] handle     : RX8025SA
 * \param[in] p_rtc      : ָRTC׼
 *
 * return RTC׼
 */
am_rtc_handle_t am_rx8025sa_rtc_init (am_rx8025sa_handle_t  handle,
                                      am_rtc_serv_t        *p_rtc);

/**
 * \brief ȡRX8025SA ALARM_CLK׼
 *
 * \param[in] handle       : RX8025SA
 * \param[in] p_alarm_clk  : ָALARM_CLK׼
 *
 * return ALARM_CLK׼
 */
am_alarm_clk_handle_t am_rx8025sa_alarm_clk_init (am_rx8025sa_handle_t   handle,
                                                  am_alarm_clk_serv_t   *p_alarm_clk);

/**
 * \name rx8025ӵֵ
 * \anchor grp_rx8025sa_alarm_week
 *  eg Ҫ    alarm_weekֵΪ 
 *        AM_RX8025SA_ALARM_SUNDAY | AM_RX8025SA_ALARM_WEDNESDAY
 */
#define AM_RX8025SA_ALARM_SUNDAY    0X01
#define AM_RX8025SA_ALARM_MONDAY    0X02
#define AM_RX8025SA_ALARM_TUESDAY   0X04
#define AM_RX8025SA_ALARM_WEDNESDAY 0X08
#define AM_RX8025SA_ALARM_THURSDAY  0X10
#define AM_RX8025SA_ALARM_FRIDAY    0X20
#define AM_RX8025SA_ALARM_SATURDAY  0X40

/**
 * \brief Ϣṹ
 */
typedef struct am_rx8025sa_alarm_info {
    int min;     /**< \brief minutes after the hour       - [0, 59]  */
    int hour;    /**< \brief hours after midnight         - [1, 12] / [0, 23]   */
    int alarm_week;    /**< \brief ֵοgrp_rx8025sa_alarm_week  alarm_dӿԺ*/
}am_rx8025sa_alarm_info_t;

/**
 * \brief ʹRX8025SA ALARM_D
 *
 * \param[in] handle       : RX8025SA
 *
 * \retval AM_OK : ɹ
 * \retval   : ,鿴ش
 */
int am_rx8025sa_alarm_d_enable (am_rx8025sa_handle_t handle);
/**
 * \brief RX8025SA  ALARM_D 
 *
 * \param[in] handle : RX8025SA
 *
 * \retval AM_OK : ɹ
 * \retval   : ,鿴ش
 */
int am_rx8025sa_alarm_d_disable (am_rx8025sa_handle_t handle);


/**
 * \brief ʹRX8025SA ALARM_W
 *
 * \param[in] handle       : RX8025SA
 *
 * \retval AM_OK : ɹ
 * \retval   : ,鿴ش
 */
int am_rx8025sa_alarm_w_enable (am_rx8025sa_handle_t handle);
/**
 * \brief RX8025SA  ALARM_W 
 *
 * \param[in] handle : RX8025SA
 *
 * \retval AM_OK : ɹ
 * \retval   : ,鿴ش
 */
int am_rx8025sa_alarm_w_disable (am_rx8025sa_handle_t handle);

/**
 * \name RX8025SA  жϵһЩģʽ
 * \anchor grp_int_mode
 * note  ÿ00
 *       ÿʱ0000
 *       ÿ100ʱ0000                        INTA 
 */

#define AM_RX8025SA_INTA_OFF                 0X00  /**< \brief INTAųָߵƽ */
#define AM_RX8025SA_INTA_LOW                 0X01  /**< \brief INTA͵ƽ */
#define AM_RX8025SA_PULSE_2HZ_MODE           0X02  /**< \brief 2HZģʽ */
#define AM_RX8025SA_PULSE_1HZ_MODE           0X03  /**< \brief 1HZģʽ */
#define AM_RX8025SA_LEVEL_PER_SECOND_MODE    0X04  /**< \brief ÿINTAŵĵƽģʽ */
#define AM_RX8025SA_LEVEL_PER_MINUTE_MODE    0X05  /**< \brief ÿINTAŵĵƽģʽ */
#define AM_RX8025SA_LEVEL_PER_HOUR_MODE      0X06  /**< \brief ÿʱINTAŵĵƽģʽ */
#define AM_RX8025SA_LEVEL_PER_MONTH_MODE     0X07  /**< \brief ÿINTAŵĵƽģʽ */


/**
 * \brief RX8025SA жϺ
 *
 * \param[in] handle   : rx8025sa
 * \param[in] int_mode :жģʽοgrp_int_mode
 *
 * \param[in] pfn_periodic_int_callback   :жϵĻص
 * \param[in] p_periodic_int_arg          :жϵĻصĲ
 *
 * \retval AM_OK : ɹ
 * \retval   : ,鿴ش
 */
int am_rx8025sa_periodic_int_set (am_rx8025sa_handle_t  handle,
                                   uint8_t              int_mode,
                                  am_pfnvoid_t          pfn_periodic_int_callback,
                                  void                 *p_periodic_int_arg);


/**
 * \brief RX8025SA Dصʱ
 *
 * \param[in] handle       : rx8025sa
 * \param[in] p_alarm_info : ʱϢ
 *
 * \param[in] pfn_alarm_callback   :жϵĻص
 * \param[in] p_alarm_arg          :жϵĻصĲ
 *
 * \retval AM_OK : ɹ
 * \retval   : ,鿴ش
 */
int am_rx8025sa_alarm_d_set(am_rx8025sa_handle_t      handle,
                            am_rx8025sa_alarm_info_t *p_alarm_info,
                            am_pfnvoid_t              pfn_alarm_callback,
                            void                     *p_alarm_arg);
/**
 * \brief RX8025SA Wصʱ
 *
 * \param[in] handle       : rx8025sa
 * \param[in] p_alarm_info : ʱϢ
 *
 * \param[in] pfn_alarm_callback   :жϵĻص
 * \param[in] p_alarm_arg          :жϵĻصĲ
 *
 * \retval AM_OK : ɹ
 * \retval   : ,鿴ش
 */
int am_rx8025sa_alarm_w_set(am_rx8025sa_handle_t      handle,
                            am_rx8025sa_alarm_info_t *p_alarm_info,
                            am_pfnvoid_t              pfn_alarm_callback,
                            void                     *p_alarm_arg);

/*
 * \brief Դѹ𵴼
 * \brief\param[in] handle       : rx8025sa
 *
 * \note     ֵ˵
 *   return 0  Դѹֹͣͣ
 *          1  Դѹֹͣͣ
 *          2  
 *          3  ȻԴͣԼ
 *          4,5 Դ½0v
 *          6,7 Դ˲ͣĿԴ
 *
 */
int am_rx8025sa_power_check(am_rx8025sa_handle_t  handle);

/**
 * \name ͵ѹֵ趨
 * \anchor grp_rx8025sa_check_voltage
 */
#define RX8025SA_CHECK_VOLTAGE_1_3V  0   /** \brief Դ ͼ⹦ܵı׼ѹֵ趨Ϊ1.3v */
#define RX8025SA_CHECK_VOLTAGE_2_1V  1   /** \brief Դ ͼ⹦ܵı׼ѹֵ趨Ϊ2.1v */

/**
 * \brief ͵ѹ⹦ܵı׼ѹֵ趨
 *
 * \param[in] handle        : RX8025SA
 * \param[in] check_v_value : ͵ѹֵ ο grp_rx8025sa_check_voltage
 *
 * \retval AM_OK : ɹ
 * \retval   : ,鿴ش
 */
int am_rx8025sa_check_voltage_set (am_rx8025sa_handle_t handle, uint8_t check_v_value);

/*
 * \brief ʱƵʽе
 * \param[in] handle   : RX8025SA
 * \param[in] clk      : ҪƵʴС  ķΧ -189.1*10^-6 <= (clk-32768)/32768 <= +189.1*10^-6
 *
 * \retval AM_OK : ɹ
 * \retval   : ,鿴ش
 *
 * \note clk ĵλΪHZFOUTƵûӰ졣
 */
int am_rx8025sa_clk_adjust ( am_rx8025sa_handle_t handle, float clk );

/*
 * \brief   clk FOUT32.768KHZƵ
 * \param[in] handle     : RX8025SA
 *
 * \retval AM_OK : ɹ
 * \retval   : ,鿴ش
 *
 * \note ҪʹоƬƵʾͱ豸Ϣclk_en_pin
 */
int am_rx8025sa_clkout_set(am_rx8025sa_handle_t handle);

/*
 * \brief  ر clk
 * \param[in] handle     : RX8025SA
 *
 * \retval AM_OK : ɹ
 * \retval   : ,鿴ش
 */
int am_rx8025sa_clkout_close(am_rx8025sa_handle_t handle);
                              
/** 
 * @}
 */

#ifdef __cplusplus
}
#endif

#endif /* _AM_RX8025SA_H */

/* end of file */

