/*******************************************************************************
*                                 AMetal
*                       ----------------------------
*                       innovating embedded platform
*
* Copyright (c) 2001-2018 Guangzhou ZHIYUAN Electronics Co., Ltd.
* All rights reserved.
*
* Contact information:
* web site:    http://www.zlg.cn/
*******************************************************************************/

/**
 * \file
 * \brief ts10xָģ飩ӿں
 *
 * \internal
 * \par Modification History
 * - 1.00 16-09-12  tee, first implementation.
 * \endinternal
 */

#ifndef __AM_TS10X_H
#define __AM_TS10X_H

#ifdef __cplusplus
extern "C" {
#endif

#include "am_types.h"
#include "am_i2c.h"
#include "am_softimer.h"
#include "am_uart_rngbuf.h"

/**
 * @addtogroup am_if_ts10x
 * @copydoc am_ts10x.h
 * @{
 */

/**
 * \name ִн״̬
 *
 *     ЩֵΪصstatusֵܵ Ϊ  AM_TS10X_SUCCESS ʱ
 * űִгɹ
 * @{
 */

#define AM_TS10X_SUCCESS       0x00   /**< \brief ִгɹ               */
#define AM_TS10X_FAIL          0x01   /**< \brief ִʧ               */
#define AM_TS10X_FULL          0x04   /**< \brief ݿ               */
#define AM_TS10X_NOUSER        0x05   /**< \brief ûû     */
#define AM_TS10X_USER_EXIST    0x07   /**< \brief ûѴ           */
#define AM_TS10X_TIMEOUT       0x08   /**< \brief ͼɼʱ      */
#define AM_TS10X_BREAK         0x18   /**< \brief ǰֹ */
#define AM_TS10X_IMAGEFAIL     0x84   /**< \brief ͼ񲻺ϸ           */

/** @} */

/**
 * \brief TS10Xص
 *
 * \param[in] p_arg    : ûԶ
 * \param[in] status   : ִе״̬
 *
 *
 * \note statusֵΪ0ʾɹִɡΪΪָԶϢ
 *   AM_TS10X_USER_EXISTΪΪAMetalƽ̨ԶĴϢ  -AM_ENOMEM
 * һûصʱʾɣԼִһβأ
 * ûעam_ts10x_user_register_*ҪΰѹÿΰѹҪ
 * ָ̧ٴΰѹ£ÿΰѹ󶼻ûصҪѹ
 * Աעᣬʱ, statusֵΪ -AM_EINPROGRESSʾڼִСֻҪ
 * ע̽ʱstatusֵŲΪ -AM_EINPROGRESS
 */
typedef void (*am_ts10x_process_cb_t) (void *p_arg, int status);

/**
 * \brief ts10x豸ṹ嶨
 */
typedef struct am_ts10x_dev {

    am_softimer_t    timer;       /**< \brief The Timer for timeout */
    am_uart_handle_t uart_handle; /**< \brief The UART handle       */

    am_bool_t        cmd_ack_complete;   /**< \brief cmd/ack is complete?     */

    uint8_t   data[4];            /**< \brief The data for first packet       */
    uint8_t   cmd_ack_type;       /**< \brief The type of this cmd            */
    uint8_t   is_send_sec;        /**< \brief there is second packet to send? */
    uint8_t   send_sec_num;       /**< \brief the buffer num for the second   */
    uint8_t   is_recv_sec;        /**< \brief there is second packet to recv? */
    uint8_t   recv_sec_num;       /**< \brief the buffer num for the second   */
    uint8_t   chk_value;          /**< \brief the check value for send/recv   */

    int       int_pin;            /**< \brief The int pin                     */
    int       complete_status;    /**< \brief The status for complete         */

    uint32_t  timeout_val;        /**< \brief timeout val for recv now        */
    uint32_t  timeout_expect_val; /**< \brief timeout expect value            */

    size_t    max_length;         /**< \brief The max length of sec data      */
    size_t    data_index;         /**< \brief The data index for now          */
    size_t    sec_buf_index;      /**< \brief The buffer index for sec pack   */


    /** \brief The data buffer info for send second packet */
    struct {
        const uint8_t  *p_buf;
        size_t          expect_len;
    } send_sec_buf_info[3];

    /** \brief The data buffer info for receive second packet */
    struct {
        uint8_t  *p_buf;
        size_t    expect_len;
    } recv_sec_buf_info[3];

    /** \brief private callback   */
     void                       (*pfn_private_cb) (struct am_ts10x_dev *, int);

    /** \brief user callback  */
     am_ts10x_process_cb_t        pfn_process_cb;

    /** \brief The argument for user callback */
    void                         *p_process_arg;

    /** \brief user callback  */
    am_pfnvoid_t                  pfn_int_pin_cb;

    /** \brief The argument for user callback */
    void                         *p_int_pin_arg;

    /** \brief ڲʹ                 */
    const void                   *p_tx_state;

    /** \brief ڲʹ                 */
    const void                   *p_rx_state;

    /** \brief ʼյACKһַĳʱʱ  */
    uint32_t                      timeout_cmd_start;

    /** \brief The data use internel only */
    union {
        struct {
            volatile  size_t   *p_info_len;
        } version_info_get;

        struct {
            volatile uint16_t *p_user_count;
        } user_count_get;

        struct {
            volatile uint8_t  *p_user_role;
        } user_role_get;

        struct {
            volatile uint16_t  *p_user_id;
              volatile uint8_t   *p_user_role;
        } compare_1_n;

        struct {
            volatile uint16_t  *p_user_id;
        } unused_user_id_get;

        struct {
            volatile uint32_t   baudrate;
        } baudrate_set;

        struct {
            volatile size_t   *p_actual_len;
        } image_data_get;

        struct {
            volatile uint8_t   *p_old_level;
        } compare_level_set;

        struct {
              uint8_t   user_number[2];
              uint16_t *p_user_count;
        } user_all_id_role_get;

        struct {
              am_bool_t *p_allow_same;
        } register_mode_get;

        struct {
              uint8_t  user_id_role[3];
        } add_user_with_feature;

        struct {
              uint8_t  user_id_role[3];
        } compare_feature_with_one_user;

        struct {
              uint16_t  *p_user_id;
        } compare_feature_with_all_user;

        struct {
              uint8_t  data[3];
        } zero_data;

        struct {
               uint16_t  *p_user_id;
               uint8_t    user_role;
               void     (*pfn_continue) (struct am_ts10x_dev *);
        } register_data;

    } private_data;

} am_ts10x_dev_t;


/** \brief ts10x  */
typedef struct am_ts10x_dev *am_ts10x_handle_t;

/**
 * \brief ts10xʼ
 *
 * ʹts10xǰӦȵøóʼԻȡts10xhandle
 *
 * \param[in] p_dev       : ָts10xʵָ
 * \param[in] int_pin     : TS10XжӵMCUź
 * \param[in] uart_handle : UART׼
 *
 * \return ts10xhandleʼʧܣ򷵻ֵΪNULL
 *
 * \note 豸ָp_devָ豸ֻҪ壬Ҫڵñǰʼ
 */
am_ts10x_handle_t am_ts10x_init (am_ts10x_dev_t           *p_dev,
                                 int                       int_pin,
                                 am_uart_handle_t          uart_handle);

/**
 * \brief ts10x ʼ
 * \param[in] handle  : TS10Xʵ
 * \return AM_OKʼɹֵʼʧܡ
 */
int am_ts10x_deinit (am_ts10x_handle_t handle);

/**
 * \brief  ts10x ص
 *
 * йصĽӿںʼͽʼ⣩첽ִеģ
 * أһ׶ʱô˴õص
 *
 * \param[in] handle         : TS10Xʵ
 * \param[in] pfn_process_cb : ص
 * \param[in] p_arg          : ص
 *
 * \return ts10xhandleʼʧܣ򷵻ֵΪNULL
 */
int am_ts10x_process_cb_set (am_ts10x_handle_t        handle,
                             am_ts10x_process_cb_t    pfn_process_cb,
                             void                    *p_arg);

/**
 * \brief  ts10x жŻص
 *
 * ָѹָʱжŽߵƽʱô˴õжŻص
 * 
 *
 * \param[in] handle         : ָts10xʵָ
 * \param[in] pfn_int_pin_cb : жŻص
 * \param[in] p_arg          : ص
 *
 * \return ts10xhandleʼʧܣ򷵻ֵΪNULL
 */
int am_ts10x_int_pin_cb_set (am_ts10x_handle_t  handle,
                             am_pfnvoid_t       pfn_int_pin_cb,
                             void              *p_arg);

/**
 * \name עûʹõע᷽ʽ
 *
 * ûעṲ4ע᷽ʽ1C3R 2C2R 3C3R MOFN(Ĭϵı׼汾1C3R
 * ģʽ)aCbR˼˷aCommandģ˷bReturnab
 * أ2C2RҲ1C3R MOFN˼M-NΰѹעᣬMΣNΣNλע
 * ᲻ɹش
 *
 * ضָһֻ֧ضһע᷽ʹ  am_ts10x_user_register() 
 * ûעʱӦȷʹȷע᷽ʽ
 *
 * @{
 */

#define AM_TS10X_USER_REG_METHOD_1C3R      1    /**< \brief 1C3R ע᷽ʽ   */
#define AM_TS10X_USER_REG_METHOD_2C2R      2    /**< \brief 2C2R ע᷽ʽ   */
#define AM_TS10X_USER_REG_METHOD_3C3R      3    /**< \brief 3C3R ע᷽ʽ   */
#define AM_TS10X_USER_REG_METHOD_MOFN      4    /**< \brief MOFN ע᷽ʽ   */

/** @} */

/**
 * \brief ûע
 *
 * \param[in]     handle     : ts10xʵ
 * \param[in,out] p_user_id  : 룺עûID Чֵ1 ~ 10000Ϊ0ʱ
 *                                   ԶһδʹõID
 *                             ʵעûIDֵΪ0ʱֵΪԶ
 *                                   ûID
 *
 * \param[in]     user_role  : ûȨޣЧֵ1 ~ 3ֵĺûԶ
 * \param[in]     reg_method : ע᷽ʽ磺#AM_TS10X_USER_REG_METHOD_1C3R
 *
 * \retval AM_OK            : ûעɹ
 * \retval -AM_EINPROGRESS  : עУעɺ󽫵ûص
 * \retval -AM_EBUSY        : 豸æ
 */
int am_ts10x_user_register (am_ts10x_handle_t  handle,
                            uint16_t          *p_user_id,
                            uint8_t            user_role,
                            uint8_t            reg_method);

/**
 * \brief ɾһضû
 *
 * \param[in]  handle       : ts10xʵ
 * \param[in]  user_id      : ɾû ID
 *
 * \retval AM_OK            : ɾûɹ
 * \retval -AM_EINPROGRESS  : ɾУɾɺ󽫵ûص
 * \retval -AM_EBUSY        : 豸æ
 */
int am_ts10x_user_one_del (am_ts10x_handle_t  handle, uint16_t user_id);

/**
 * \brief ɾû
 *
 * \param[in]  handle       : ts10xʵ
 *
 * \retval AM_OK            : ɾûɹ
 * \retval -AM_EINPROGRESS  : ɾУɾɺ󽫵ûص
 * \retval -AM_EBUSY        : 豸æ
 */
int am_ts10x_user_all_del (am_ts10x_handle_t  handle);


/**
 * \brief ȡǰû
 *
 * \param[in]  handle       : ts10xʵ
 * \param[out] p_user_count : ȡûָ
 *
 * \retval AM_OK            : ȡûɹ
 * \retval -AM_EINPROGRESS  : ڻȡУȡɺ󽫵ûص
 * \retval -AM_EBUSY        : 豸æ
 */
int am_ts10x_user_count_get (am_ts10x_handle_t handle, uint16_t *p_user_count);

/**
 * \brief ȡָûȨ
 *
 * \param[in]  handle      : ts10xʵ
 * \param[in]  user_id     : ûID
 * \param[out] p_user_role : ȡûȨ޵ָ
 *
 * \retval AM_OK            : ȡûȨ޳ɹ
 * \retval -AM_EINPROGRESS  : ڻȡУȡɺ󽫵ûص
 * \retval -AM_EBUSY        : 豸æ
 */
int am_ts10x_user_role_get (am_ts10x_handle_t handle,
                            uint16_t          user_id,
                            uint8_t          *p_user_role);

/**
 * \brief ȡһδʹõûID
 *
 * \param[in]  handle      : ts10xʵ
 * \param[out] p_user_id   : ڻȡδʹõûIDָ
 *
 * \retval AM_OK            : ȡɹ
 * \retval -AM_EINPROGRESS  : ڻȡУȡɺ󽫵ûص
 * \retval -AM_EBUSY        : 豸æ
 */
int am_ts10x_user_id_unused_get (am_ts10x_handle_t  handle,
                                 uint16_t          *p_user_id);

/**
 * \brief òʣÿθλָ115200
 *
 * \param[in]  handle        : ts10xʵ
 * \param[out] baudrate      : ʹõĲʡ
 *                             ָǽ֧֣9600192003840057600  115200
 *
 * \retval AM_OK            : ȡɹ
 * \retval -AM_EINPROGRESS  : ڻȡУȡɺ󽫵ûص
 * \retval -AM_EBUSY        : 豸æ
 *
 * \note ʹøúòʱҪȷUARTָ֧ò
 */
int am_ts10x_baudrate_set (am_ts10x_handle_t  handle,
                           uint32_t           baudrate);

/**
 * \brief ɼָͼ,ȡָͼֵ494ֽڣ
 *
 * \param[in]  handle     : ts10xʵ
 * \param[out] p_data_buf : ȡֵĻ
 * \param[in]  buf_len    : ȣֵΪ494ֽڣ˸ֵΪ 494
 *
 * \retval AM_OK            : ȡɹ
 * \retval -AM_EINPROGRESS  : ȡУȡɺ󽫵ûص
 * \retval -AM_EBUSY        : 豸æ
 *
 * \note ִиúָǽɼָ״̬Ҫ8sڰѹָƣԱɼ
 */
int am_ts10x_collect_image_feature_get (am_ts10x_handle_t  handle,
                                        uint8_t           *p_data_buf,
                                        size_t             buf_len);

/**
 * \brief ɼָͼ,ָͼλͼ
 *
 * \param[in]  handle       : ts10xʵ
 * \param[out] p_data_buf   : ͼֵĻ
 * \param[in]  buf_len      : ȣӦʵʵͼݳһ£ʵ
 *                            ͼݵĳ봫ֱأ
 *                            Ϊwidth * 64 + Height / 4
 *
 * \param[out] p_actual_len : ʵλͼݵĳ
 *
 * \retval AM_OK            : ȡɹ
 * \retval -AM_EINPROGRESS  : ȡУȡɺ󽫵ûص
 * \retval -AM_EBUSY        : 豸æ
 *
 * \note ִиúָǽɼָ״̬Ҫ8sڰѹָƣԱɼ
 * һأʹõָǴķֱΪ 176 * 176ʱͼݵĴСΪ
 * 176 * 64 + 176 / 4 = 11308 ֽڡԼ 11KʹøúʱҪһ11K
 * ϵͳRAMСĳϣʹøúֱӻȡͼݣֻҪȡֵ
 * Ϳˡ
 */
int am_ts10x_collect_image_data_get (am_ts10x_handle_t  handle,
                                     uint8_t           *p_data_buf,
                                     size_t             buf_len,
                                     size_t            *p_actual_len);

/**
 * \brief ȡָģİ汾Ϣַ
 *
 *  ȡģ汾Ϣа˳汾͡ע᷽ʽʱϢ
 *
 *  磬дʱȡϢΪ漸еַΪ140ַ
 *   Version:  7V33.35_stdrl_024
 *   Sensor:TS1060
 *   RegMode:1C3R
 *   MaxUser:394
 *   Time:Jul  4-15:50:02
 *   Tooan Co.ltd. All rights reserved.
 *
 * \param[in]  handle       : ts10xʵ
 * \param[in]  p_buf        : Ű汾ϢĻ
 * \param[in]  buf_size     : 
 * \param[out] p_info_size  : ʵϢĳ
 *
 * \retval AM_OK            : ȡ汾Ϣɹ
 * \retval -AM_EINPROGRESS  : ڴУɺ󽫵ûص
 * \retval -AM_EBUSY        : 豸æ
 */
int am_ts10x_version_info_get (am_ts10x_handle_t  handle,
                               uint8_t           *p_buf,
                               size_t             buf_size,
                               size_t            *p_info_size);
/**
 * \brief ñȶԵȼ
 *
 *  ȶԵȼΪ 0-3Ĭϵǹѧ/TS10X11뵼10603ȶԵȼԽߣ
 *  ԽϸģָĬֵ
 *
 * \param[in]  handle       : ts10xʵ
 * \param[in]  new_level    : õıȶԵȼ
 * \param[out] p_old_level  : ȡ֮ǰʹõıȶԵȼ
 *
 * \retval AM_OK            : ñȶԵȼɹ
 * \retval -AM_EINPROGRESS  : ڴУɺ󽫵ûص
 * \retval -AM_EBUSY        : 豸æ
 */
int am_ts10x_compare_level_set (am_ts10x_handle_t  handle,
                                uint8_t            new_level,
                                uint8_t           *p_old_level);
/**
 * \brief ȡעûIDȨϢ
 *
 * \param[in]  handle       : ts10xʵ
 * \param[out] p_user_count : ǰʵûĿ
 * \param[out] p_buf        : ûIDȨϢĻÿռֽڣ
 *                             user_id(8λ)  user_id (8λ)role
 *
 * \param[in]  buf_len      : ȣӦΪûĿ * 3 ֽ
 *
 * \retval AM_OK            : ȡɹ
 * \retval -AM_EINPROGRESS  : ڻȡУȡɺ󽫵ûص
 * \retval -AM_EBUSY        : 豸æ
 */
int am_ts10x_user_all_id_role_get (am_ts10x_handle_t   handle,
                                   uint16_t           *p_user_count,
                                   uint8_t            *p_buf,
                                   size_t              buf_len);

/**
 * \brief עģʽǷظעѾڵû
 *
 * \param[in] handle      : ts10xʵ
 * \param[in] allow_same  : ǷظעѾڵûAM_TRUEAM_FALSE
 *
 * \retval AM_OK            : óɹ
 * \retval -AM_EINPROGRESS  : Уɺ󽫵ûص
 * \retval -AM_EBUSY        : 豸æ
 *
 * \note ĬΪظעѾڵûÿϵָĬֵ
 */
int am_ts10x_register_mode_set (am_ts10x_handle_t   handle,
                                am_bool_t           allow_same);

/**
 * \brief ȡǰעģʽǷظעѾڵû
 *
 * \param[in] handle        : ts10xʵ
 * \param[out] p_allow_same : ȡǰעģʽAM_TRUEAM_FALSE
 *
 * \retval AM_OK            : ȡɹ
 * \retval -AM_EINPROGRESS  :  ȡУɺ󽫵ûص
 * \retval -AM_EBUSY        : 豸æ
 */
int am_ts10x_register_mode_get (am_ts10x_handle_t   handle,
                                am_bool_t          *p_allow_same);

/**
 * \brief ȡָûֵָ
 *
 * \param[in]  handle      : ts10xʵ
 * \param[in]  user_id     : ûID
 * \param[out] p_user_role : ȡûȨ޵ָ
 * \param[out] p_buf       : ȡֵĻ
 * \param[in]  buf_len     : ȣֵΪ494ֽڣ˸ֵΪ 494
 *
 * \retval AM_OK            : ȡɹ
 * \retval -AM_EINPROGRESS  :  ȡУɺ󽫵ûص
 * \retval -AM_EBUSY        : 豸æ
 */
int am_ts10x_user_feature_get (am_ts10x_handle_t   handle,
                               uint16_t            user_id,
                               uint8_t            *p_user_role,
                               uint8_t            *p_buf,
                               size_t              buf_len);

/**
 * \brief ´ûֵָ洢ָģУһû
 *
 * \param[in] handle      : ts10xʵ
 * \param[in] user_id     : ֵָû ID
 * \param[in] user_role   : ûȨϢЧֵ 1 ~ 3ֵûԶ
 * \param[in] p_feature   : 洢ֵָռ׵ַ
 * \param[in] feature_len : ֵָĳȣֽΪ 494
 *
 * \retval AM_OK            : ɾûɹ
 * \retval -AM_EINPROGRESS  : ڴУɺ󽫵ûص
 * \retval -AM_EBUSY        : 豸æ
 */
int am_ts10x_user_add_with_feature (am_ts10x_handle_t  handle,
                                    uint16_t           user_id,
                                    uint8_t            user_role,
                                    const uint8_t     *p_feature,
                                    size_t             feature_len);

/**
 * \brief ɼָͼ񣬽 1  1 ȶԣָƿָIDûбȽϣ
 *
 * \param[in] handle  : ts10xʵ
 * \param[in] user_id : ûIDָڱȽϵû
 *
 * \retval AM_OK            : ȶԳɹ
 * \retval -AM_EINPROGRESS  : ڱȶУȶɺ󽫵ûص
 * \retval -AM_EBUSY        : 豸æ
 *
 * \note ִиúָǽɼָ״̬Ҫ8sڰѹָƣԱɼ
 */
int am_ts10x_compare_collect_with_one_user (am_ts10x_handle_t handle,
                                            uint16_t          user_id);

/**
 * \brief ɼָͼ񣬽 1   N ȶԣָƿûбȽϣ
 *
 * \param[in]  handle      : ts10xʵ
 * \param[out] p_user_id   : ȶԳɹָɻȡȶԳɹûID
 * \param[out] p_user_role : ȶԳɹָɻȡȶԳɹûRole
 *
 *
 * \retval AM_OK            : ȶԳɹ
 * \retval -AM_EINPROGRESS  : ڱȶУȶɺ󽫵ûص
 * \retval -AM_EBUSY        : 豸æ
 *
 * \note ִиúָǽɼָ״̬Ҫ8sڰѹָƣԱɼ
 */
int am_ts10x_compare_collect_with_all_user (am_ts10x_handle_t  handle,
                                            uint16_t          *p_user_id,
                                            uint8_t           *p_user_role);


/**
 * \brief ɼָͼֵָָȶ
 *
 * \param[in] handle      : ts10xʵ
 * \param[in] p_feature   : 洢ֵָռ׵ַ
 * \param[in] feature_len : ֵָĳȣֽΪ 494
 *
 * \retval AM_OK            : ȶԳɹ
 * \retval -AM_EINPROGRESS  : ڱȶУȶɺ󽫵ûص
 * \retval -AM_EBUSY        : 豸æ
 *
 * \note ִиúָǽɼָ״̬Ҫ8sڰѹָƣԱɼ
 */
int am_ts10x_compare_collect_with_feature (am_ts10x_handle_t  handle,
                                           const uint8_t     *p_feature,
                                           size_t             feature_len);


/**
 * \brief ֵָָûֵбȶ
 *
 * \param[in] handle      : ts10xʵ
 * \param[in] p_feature   : 洢ֵָռ׵ַ
 * \param[in] feature_len : ֵָĳȣֽΪ 494
 * \param[in] user_id     : ûIDָڱȶԵû
 * \param[in] user_role   : ָڱȶԵûȨֵ
 *
 * \retval AM_OK            : ȶԳɹ
 * \retval -AM_EINPROGRESS  : ڱȶУȶɺ󽫵ûص
 * \retval -AM_EBUSY        : 豸æ
 *
 * \note ִиúָǽɼָ״̬Ҫ8sڰѹָƣԱɼ
 */
int am_ts10x_compare_feature_with_one_user (am_ts10x_handle_t  handle,
                                            const uint8_t     *p_feature,
                                            size_t             feature_len,
                                            uint16_t           user_id,
                                            uint8_t            user_role);

/**
 * \brief ֵָûֵбȽ
 *
 * \param[in] handle      : ts10xʵ
 * \param[in] p_feature   : 洢ֵָռ׵ַ
 * \param[in] feature_len : ֵָĳȣֽΪ 494
 * \param[in] p_user_id   : ûIDָ
 *
 * \retval AM_OK            : ȶԳɹ
 * \retval -AM_EINPROGRESS  : ڱȶУȶɺ󽫵ûص
 * \retval -AM_EBUSY        : 豸æ
 *
 * \note ִиúָǽɼָ״̬Ҫ8sڰѹָƣԱɼ
 */
int am_ts10x_compare_feature_with_all_user (am_ts10x_handle_t  handle,
                                            const uint8_t     *p_feature,
                                            size_t             feature_len,
                                            uint16_t          *p_user_id);

/** 
 * @}
 */

#ifdef __cplusplus
}
#endif

#endif /* __AM_TS10X_H */

/* end of file */
