/*******************************************************************************
*                                 AMetal
*                       ----------------------------
*                       innovating embedded platform
*
* Copyright (c) 2001-2018 Guangzhou ZHIYUAN Electronics Co., Ltd.
* All rights reserved.
*
* Contact information:
* web site:    http://www.zlg.cn/
*******************************************************************************/

/**
 * \file
 * \brief 
 *
 * \internal
 * \par Modification History
 * - 1.00 15-09-  win, first implementation.
 * \endinternal
 */
#ifndef __AM_IC_MIFARE_H
#define __AM_IC_MIFARE_H

#ifdef __cplusplus
extern "C" {
#endif

#include "am_common.h"
#include "am_uartcmd_ic_iso14443.h"

/**
 * \name MifareԿ
 * @{
 */

#define AM_IC_MIFARE_KEY_TYPE_MASK    0x7F    /**< \brief Կ */
#define AM_IC_MIFARE_KEY_TYPE_A       0x60    /**< \brief ԿA֤ */
#define AM_IC_MIFARE_KEY_TYPE_B       0x61    /**< \brief ԿB֤ */

#define AM_IC_MIFARE_KEY_SRC_MASK     0x80    /**< \brief ԿԴ */
#define AM_IC_MIFARE_KEY_SRC_EXT      0x00    /**< \brief ʹⲿԿ֤ */
#define AM_IC_MIFARE_KEY_SRC_E2       0x80    /**< \brief ʹڲE2Կ֤ */

/** @} */

/**
 * \brief Mifare 
 * @{
 */
#define AM_IC_MIFARE1_CMD_READ            0x30    /**< \brief (16ֽ) */
#define AM_IC_MIFARE1_CMD_WRITE           0xA0    /**< \brief д(16ֽ) */
#define AM_IC_MIFARE1_CMD_TRANSFER        0xB0    /**< \brief  */
#define AM_IC_MIFARE1_CMD_NO_COMMAND      0x00    /**< \brief  */
#define AM_IC_MIFARE1_CMD_INCREMENT       0xC1    /**< \brief ֵ */
#define AM_IC_MIFARE1_CMD_DECREMENT       0xC0    /**< \brief ֵ */
#define AM_IC_MIFARE1_CMD_RESTORE         0xC2    /**< \brief  */
#define AM_IC_MIFARE0_CMD_WRITE           0xA2    /**< \brief UltraLight д(4ֽ) */
#define AM_IC_MIFARE0_CMD_AUTH            0xA3    /**< \brief UltraLight ֤ */

/** @} */

/**
 * \brief Mifare׼
 */
struct am_ic_mifare_drv_funcs {

    /** \brief Կ֤ */
    int (*pfn_ic_key_auth) (void       *p_drv,
                            uint8_t     mode,
                            const void *p_key,
                            uint32_t    key_nbytes,
                            const void *p_uid,
                            uint32_t    uid_nbytes,
                            uint8_t     block);

    /** \brief Mifareչ */
    int (*pfn_ic_activate) (void                 *p_drv,
                            uint8_t               mode,
                            uint8_t               req_code,
                            am_ic_a_reset_info_t *p_reset_info);

    /** \brief Mifareȡ */
    int (*pfn_ic_blocks_read) (void    *p_drv,
                               uint8_t  start_block,
                               uint8_t  block_num,
                               void    *p_buf);

    /** \brief Mifareд */
    int (*pfn_ic_blocks_write) (void       *p_drv,
                                uint8_t     start_block,
                                uint8_t     block_num,
                                const void *p_buf);

    /** \brief Mifareȡֵ */
    int (*pfn_ic_value_block_get) (void    *p_drv,
                                   uint8_t  block,
                                   int32_t *p_value);

    /** \brief Mifareֵֵ */
    int (*pfn_ic_value_block_set) (void    *p_drv,
                                   uint8_t  block,
                                   int32_t  value);

    /** \brief Mifareֵ */
    int (*pfn_ic_value_operation) (void    *p_drv,
                                   uint8_t  mode,
                                   uint8_t  src_block,
                                   int32_t  value,
                                   uint8_t  dst_block);

    /** \brief Mifare UltraLightȡ */
    int (*pfn_ic_ultralight_blocks_read) (void    *p_drv,
                                          uint8_t  start_block,
                                          uint8_t  block_num,
                                          void    *p_buf);

    /** \brief Mifare UltraLightö */
    int (*pfn_ic_ultralight_blocks_write) (void       *p_drv,
                                           uint8_t     start_block,
                                           uint8_t     block_num,
                                           const void *p_buf);

    /** \brief Mifare UltraLight */
    int (*pfn_ic_ultralight_transfer) (void       *p_drv,
                                       const void *p_txbuf,
                                       void       *p_rxbuf,
                                       uint32_t   *p_rx_nbytes);
};

/**
 * \brief Mifare׼ṹ
 */
typedef struct am_ic_mifare_serv {

    /** Mifare׼ṹָ */
    struct am_ic_mifare_drv_funcs *p_funcs;

    /* \brief ĵһ */
    void                          *p_drv;
} am_ic_mifare_serv_t;

/** \brief Mifare׼ */
typedef am_ic_mifare_serv_t *am_ic_mifare_handle_t;

/**
 * \brief MifareԿ֤
 *
 * \param[in] handle     : Mifare׼
 * \param[in] mode       : ֤ģʽ
 *                         - mode.7         -- ԿԴ
 *                           - AM_IC_MIFARE_KEY_SRC_EXT ʹⲿԿ
 *                           - AM_IC_MIFARE_KEY_SRC_E2  ʹڲE2Կ
 *                         - mode.6-0       -- Կ
 *                           - AM_IC_MIFARE_KEY_TYPE_A  ԿA
 *                           - AM_IC_MIFARE_KEY_TYPE_B  ԿB
 * \param[in] p_key      : Կ
 * \param[in] key_nbytes : Կ
 * \param[in] p_uid      : UID
 * \param[in] uid_nbytes : UID
 * \param[in] block      : 
 *
 * \retval AM_OK      : ɹ
 * \retval -AM_EINVAL : 
 */
am_static_inline
int am_ic_mifare_key_auth (am_ic_mifare_handle_t  handle,
                           uint8_t                mode,
                           const void            *p_key,
                           uint32_t               key_nbytes,
                           const void            *p_uid,
                           uint32_t               uid_nbytes,
                           uint8_t                block)
{
    return handle->p_funcs->pfn_ic_key_auth(handle->p_drv,
                                            mode,
                                            p_key,
                                            key_nbytes,
                                            p_uid,
                                            uid_nbytes,
                                            block);
}

/**
 * \brief Mifare
 *
 * \param[in] handle       : Mifare׼
 * \param[in] mode         : ײģʽ
 *                           - 0  ִзײѭ
 *                           - 1  ִֻһηײ
 * \param[in] req_code     : 
 *                           - AM_IC_ISO14443_3A_REQIDL п
 *                           - AM_IC_ISO14443_3A_REQALL п
 * \param[in] p_reset_info : ISO14443-3AӦϢ
 *
 * \retval AM_OK      : ɹ
 * \retval -AM_EINVAL : 
 */
am_static_inline
int am_ic_mifare_activate (am_ic_mifare_handle_t  handle,
                           uint8_t                mode,
                           uint8_t                req_code,
                           am_ic_a_reset_info_t  *p_reset_info)
{
    return handle->p_funcs->pfn_ic_activate(handle->p_drv,
                                            mode,
                                            req_code,
                                            p_reset_info);
}

/**
 * \brief Mifareȡ
 *
 * \param[in] handle       : Mifare׼
 * \param[in] start_block  : ʼ
 * \param[in] block_num    : Ŀпͬһ
 * \param[in] p_buf        : 16 * block_num ֽ
 *
 * \retval AM_OK      : ɹ
 * \retval -AM_EINVAL : 
 */
am_static_inline
int am_ic_mifare_blocks_read (am_ic_mifare_handle_t  handle,
                              uint8_t                start_block,
                              uint8_t                block_num,
                              void                  *p_buf)
{
    return handle->p_funcs->pfn_ic_blocks_read(handle->p_drv,
                                               start_block,
                                               block_num,
                                               p_buf);

}

/**
 * \brief Mifareд
 *
 * \param[in] handle       : Mifare׼
 * \param[in] start_block  : дʼ
 * \param[in] block_num    : дĿпͬһ
 * \param[in] p_buf        : д16 * block_num ֽ
 *
 * \retval AM_OK      : ɹ
 * \retval -AM_EINVAL : 
 */
am_static_inline
int am_ic_mifare_blocks_write (am_ic_mifare_handle_t  handle,
                               uint8_t                start_block,
                               uint8_t                block_num,
                               const void            *p_buf)
{
    return handle->p_funcs->pfn_ic_blocks_write(handle->p_drv,
                                                start_block,
                                                block_num,
                                                p_buf);
}

/**
 * \brief Mifareȡֵ
 *
 * \param[in] handle  : Mifare׼
 * \param[in] block   : 
 * \param[in] p_value : ֵ(з)
 *
 * \retval AM_OK      : ɹ
 * \retval -AM_EINVAL : 
 */
am_static_inline
int am_ic_mifare_value_block_get (am_ic_mifare_handle_t  handle,
                                  uint8_t                block,
                                  int32_t               *p_value)
{
    return handle->p_funcs->pfn_ic_value_block_get(handle->p_drv,
                                                   block,
                                                   p_value);
}

/**
 * \brief Mifareֵֵ
 *
 * \param[in] handle : Mifare׼
 * \param[in] block  : 
 * \param[in] value  : ֵ(з)
 *
 * \retval AM_OK      : ɹ
 * \retval -AM_EINVAL : 
 */
am_static_inline
int am_ic_mifare_value_block_set (am_ic_mifare_handle_t handle,
                                  uint8_t               block,
                                  int32_t               value)
{
    return handle->p_funcs->pfn_ic_value_block_set(handle->p_drv,
                                                   block,
                                                   value);
}

/**
 * \brief Mifareֵ
 *
 * \param[in] handle    : Mifare׼
 * \param[in] mode      : ֵģʽ
 *                        - AM_IC_MIFARE1_CMD_INCREMENT ԴֵֵָŵĿĿ
 *                        - AM_IC_MIFARE1_CMD_DECREMENT ԴֵȥֵָŵĿĿ
 *                        - AM_IC_MIFARE1_CMD_RESTORE   ԴֵƵĿĿ
 * \param[in] src_block : Դ
 * \param[in] value     : ֵ(λЧ, ûзλ)
 * \param[in] dst_block : ĿĿ
 *
 * \retval AM_OK      : ɹ
 * \retval -AM_EINVAL : 
 */
am_static_inline
int am_ic_mifare_value_operation (am_ic_mifare_handle_t handle,
                                  uint8_t               mode,
                                  uint8_t               src_block,
                                  int32_t               value,
                                  uint8_t               dst_block)
{
    return handle->p_funcs->pfn_ic_value_operation(handle->p_drv,
                                                   mode,
                                                   src_block,
                                                   value,
                                                   dst_block);
}

/**
 * \brief Mifare UltraLightȡ
 *
 * \param[in] handle      : Mifare׼
 * \param[in] start_block : ʼ
 * \param[in] block_num   : Ŀ
 * \param[in] p_buf       : 4 * block_numֽ
 *
 * \retval AM_OK      : ɹ
 * \retval -AM_EINVAL : 
 */
am_static_inline
int am_ic_mifare_ultralight_blocks_read (am_ic_mifare_handle_t   handle,
                                         uint8_t                 start_block,
                                         uint8_t                 block_num,
                                         void                   *p_buf)
{
    return handle->p_funcs->pfn_ic_ultralight_blocks_read(handle->p_drv,
                                                          start_block,
                                                          block_num,
                                                          p_buf);
}

/**
 * \brief Mifare UltraLightö
 *
 * \param[in] handle      : Mifare׼
 * \param[in] start_block : дʼ
 * \param[in] block_num   : дĿ
 * \param[in] p_buf       : д4 * block_numֽ
 *
 * \retval AM_OK      : ɹ
 * \retval -AM_EINVAL : 
 */
am_static_inline
int am_ic_mifare_ultralight_blocks_write (am_ic_mifare_handle_t   handle,
                                          uint8_t                 start_block,
                                          uint8_t                 block_num,
                                          const void             *p_buf)
{
    return handle->p_funcs->pfn_ic_ultralight_blocks_write(handle->p_drv,
                                                           start_block,
                                                           block_num,
                                                           p_buf);
}

/**
 * \brief Mifare UltraLight
 *
 * \param[in] handle      : Mifare׼
 * \param[in] p_txbuf     : ͻҪ͸Ƭ
 * \param[in] p_rxbuf     : ջŴӿƬȡ
 * \param[in] p_rx_nbytes : ܵݳȣֽڣ
 *
 * \retval AM_OK      : ɹ
 * \retval -AM_EINVAL : 
 */
am_static_inline
int am_ic_mifare_ultralight_transfer (am_ic_mifare_handle_t   handle,
                                      const void             *p_txbuf,
                                      void                   *p_rxbuf,
                                      uint32_t               *p_rx_nbytes)
{
    return handle->p_funcs->pfn_ic_ultralight_transfer(handle->p_drv,
                                                       p_txbuf,
                                                       p_rxbuf,
                                                       p_rx_nbytes);
}

#ifdef __cplusplus
}
#endif

#endif /* __AM_IC_MIFARE_H */

/* end of file */
