/*******************************************************************************
*                                 AMetal
*                       ----------------------------
*                       innovating embedded platform
*
* Copyright (c) 2001-2018 Guangzhou ZHIYUAN Electronics Co., Ltd.
* All rights reserved.
*
* Contact information:
* web site:    http://www.zlg.cn/
*******************************************************************************/

/**
 * \file
 * \brief ZLG600ײ
 * 
 * \internal
 * \par Modification history
 * - 1.01 16-08-03  sky, modified.
 * - 1.00 15-09-03  tee, first implementation.
 * \endinternal
 */

#ifndef __AM_ZLG600_H
#define __AM_ZLG600_H
 
#include "am_types.h"
#include "am_uart_rngbuf.h"
#include "am_i2c.h"

/**
 * @addtogroup am_if_zlg600
 * @copydoc am_zlg600.h
 * @{
 */
 
/** 
 * \name ZLG600ͨģʽģʽ
 * @{
 */

#define AM_ZLG600_MODE_AUTO_CHECK         0         /**< \brief Զģʽ  */
#define AM_ZLG600_MODE_I2C                1         /**< \brief I2Cģʽ       */
#define AM_ZLG600_MODE_UART               2         /**< \brief ģʽ      */
#define AM_ZLG600_MODE_MASTER       (1 << 4)        /**< \brief ģʽܣ */
#define AM_ZLG600_MODE_SLAVE        (0 << 4)        /**< \brief ӻģʽĬϣ */

/** @}*/

/**
 * \name ZLG600ӷʽ
 * @{
 */

#define AM_ZLG600_CMNT_UART        0            /**< \brief UART */
#define AM_ZLG600_CMNT_I2C         1            /**< \brief I2C */

/** @} */

 /** 
 * \name ZLG600֡ʽ
 * @{
 */

#define AM_ZLG600_FRAME_FMT_OLD  0             /**< \brief  ֡ʽ      */
#define AM_ZLG600_FRAME_FMT_NEW  1             /**< \brief  ֡ʽ      */

 /** @} */
 
/**
 * \name ֵ֧Ĳ
 * @{
 */

#define  AM_ZLG600_BAUDRATE_9600      9600     /**< \brief  9600   */
#define  AM_ZLG600_BAUDRATE_19200     19200    /**< \brief  19200  */
#define  AM_ZLG600_BAUDRATE_28800     28800    /**< \brief  28800  */
#define  AM_ZLG600_BAUDRATE_38400     38400    /**< \brief  38400  */
#define  AM_ZLG600_BAUDRATE_57600     57600    /**< \brief  57600  */
#define  AM_ZLG600_BAUDRATE_115200    115200   /**< \brief  115200 */
#define  AM_ZLG600_BAUDRATE_172800    172800   /**< \brief  172800 */
#define  AM_ZLG600_BAUDRATE_230400    230400   /**< \brief  230400 */

/** @}*/

/**
 * \brief ZLG600ԶϢṹ
 */
typedef struct am_zlg600_auto_detect_info {

    /**
     * \brief ģʽü⶯Ƿжϡ⡢ɶ #AM_ZLG600_MIFARE_CARD_AD_HALT
     */
    uint8_t        ad_mode;

    /**
     * \brief ģʽһͬʱ #AM_ZLG600_MIFARE_CARD_TX_TX1
     */
    uint8_t        tx_mode;

    /**
     * \brief ģʽпп #AM_ZLG600_MIFARE_CARD_REQ_IDLE
     */
    uint8_t        req_mode;

    /**
     * \brief ֤ģʽE2Կֱ֤֤֤ #AM_ZLG600_MIFARE_CARD_AUTH_E2
     */
    uint8_t        auth_mode;           /**< \brief ֤ģʽ */

    /**
     * \brief Կͣ#AM_ZLG600_IC_KEY_TYPE_A  #AM_ZLG600_IC_KEY_TYPE_B
     *        auth_modeѡAM_ZLG600_MIFARE_CARD_AUTH_NO
     *        key_typeΪֵ
     */
    uint8_t        key_type;

    /** \brief Կ */
    uint8_t        key[16];

    /** \brief Կ */
    uint8_t        key_len;

    /**
     * \brief ֤
     *          S50 063
     *          S70 0255
     *          PLUS CPU 2K 0127
     *          PLUS CPU 4K 0255
     */
    uint8_t        nblock;

} am_zlg600_auto_detect_info_t;

/**
 * \brief ׼ZLG600豸ṹ嶨
 */
typedef struct am_zlg600_serv {

    void          *p_cookie;            /**< \brief ڲʹ */

    uint32_t       timeout_ms;          /**< \brief ȴʱ() */

    uint8_t        frame_seq;           /**< \brief ¼ */
    uint8_t        work_mode;           /**< \brief ǰʹõĹģʽ(ģʽ) */
    uint8_t        cur_addr;            /**< \brief ǰĵַ(7bit) */

    uint8_t        auto_detect;         /**< \brief Զģʽ */
    uint8_t        auto_detect_read;    /**< \brief Զ־ */
    am_pfnvoid_t   pfn_callback;        /**< \brief ص */
    void          *p_arg;               /**< \brief صĲ */

    am_pfnvoid_t                  pfn_cmd_timeout_set;          /**< \brief ָʱʱ */
    am_pfnvoid_t                  pfn_frame_proc;               /**< \brief ֡ */
    am_pfnvoid_t                  pfn_auto_detect_trigger_set;  /**< \brief Զⴥ */
    am_zlg600_auto_detect_info_t *p_auto_detect_info;           /**< \brief ԶϢ */

} am_zlg600_serv_t;

/**
 * \brief ZLG600׼handle
 */
typedef am_zlg600_serv_t *am_zlg600_handle_t;

/** 
 * \brief ZLG600ͨţ豸
 */
typedef struct am_zlg600_uart_dev {

    /** \brief ZLG600 ׼ */
    am_zlg600_serv_t         zlg600_serv;

    /** \brief ڣ豸           */
    am_uart_rngbuf_dev_t     uart_ringbuf_dev;
    
    /** \brief ڣ׼handle */
    am_uart_rngbuf_handle_t  uart_ringbuf_handle;

    /** \brief ֡ */
    am_pfnvoid_t   pfn_frame_proc;

} am_zlg600_uart_dev_t;

/** 
 * \brief ZLG600 (UARTͨģʽ) Ϣ
 */
typedef struct am_zlg600_uart_dev_info {

    /** \brief ڴڽյĻС64     */
    uint8_t     *p_uart_rxbuf;

    /** \brief ڴڷ͵ĻС64     */
    uint8_t     *p_uart_txbuf;

    /** \brief ڴڽյĻС */
    uint16_t     rxbuf_size;

    /** \brief ڴڷ͵ĻС */
    uint16_t     txbuf_size;

    /** \brief ZLG600ͨŵĲ(UARTģʽʹ) */
    uint32_t     baudrate;

    /** \brief ʼʱʹõģʽ #AM_ZLG600_MODE_AUTO_CHECK */
    uint8_t      now_mode;

    /** \brief ZLG600Ĵӻַ(7bit) */
    uint8_t      local_addr;

} am_zlg600_uart_dev_info_t;

/**
 * \brief ZLG600I2Cͨţ豸
 */
typedef struct am_zlg600_i2c_dev {
    
    /** \brief ZLG600 ׼ */
    am_zlg600_serv_t          zlg600_serv;
 
    /** \brief I2C׼ */
    am_i2c_handle_t           i2c_handle;
    
    /** \brief ж           */
    int                       pin;
    
    /** \brief I2C transfer ַ */
    am_i2c_transfer_t        *p_trans;

    /** \brief I2C transfer 泤 */
    uint8_t                   trans_len;

    /** \brief ڵȴI2C */
    am_wait_t                 wait;

    /** \brief ֡ */
    am_pfnvoid_t   pfn_frame_proc;
 
} am_zlg600_i2c_dev_t;

/**
 * \brief ZLG600 (I2Cͨģʽ) Ϣ
 */
typedef struct am_zlg600_i2c_dev_info {

    /** \brief жţʹI2CͨʱҪʹõһж */
    int                pin;

    /** \brief ZLG600Ĵӻַ(7bit) */
    uint8_t            local_addr;

    /** \brief ʼʱʹõģʽ #AM_ZLG600_MODE_AUTO_CHECK */
    uint8_t            now_mode;

    /** \brief I2C transfer ַ */
    am_i2c_transfer_t *p_trans;

    /** \brief I2C transfer 泤(ĿǰָҪ9trans) */
    uint8_t            trans_len;

} am_zlg600_i2c_dev_info_t;

/**
 * \brief ʼZLG600ʹUART֡ͨģʽȡZLG600ı׼handle
 *
 * ظʵӲʹúͨӷʽѡʹUARTʹøúʼ
 * Իhandleʹam_zlg600_i2c_init()handle
 *
 * \param[in] p_dev        : ָZLG600豸ָ
 * \param[in] uart_handle  : ZLG600UARTʹøhandleUARTͨ
 * \param[in] p_uart_info  : UARTͨģʽϢ
 *
 * \return  ZLG600 Ĳʼʧܣ򷵻NULL
 */ 
am_zlg600_handle_t am_zlg600_uart_old_init (am_zlg600_uart_dev_t            *p_dev,
                                            am_uart_handle_t                 uart_handle,
                                            const am_zlg600_uart_dev_info_t *p_uart_info);

/**
 * \brief ʼZLG600ʹUART֡ͨģʽȡZLG600ı׼handle
 *
 * ظʵӲʹúͨӷʽѡʹUARTʹøúʼ
 * Իhandleʹam_zlg600_i2c_init()handle
 *
 * \param[in] p_dev        : ָZLG600豸ָ
 * \param[in] uart_handle  : ZLG600UARTʹøhandleUARTͨ
 * \param[in] p_uart_info  : UARTͨģʽϢ
 *
 * \return  ZLG600 Ĳʼʧܣ򷵻NULL
 */
am_zlg600_handle_t am_zlg600_uart_new_init (am_zlg600_uart_dev_t            *p_dev,
                                            am_uart_handle_t                 uart_handle,
                                            const am_zlg600_uart_dev_info_t *p_uart_info);

/**
 * \brief ʼZLG600ʹI2C֡ͨģʽȡZLG600ı׼handle
 *
 * ظʵӲʹúͨӷʽѡʹI2Cʹøúʼ
 * Իhandleʹam_zlg600_uart_init()handle
 *
 * \param[in] p_dev        : ָZLG600豸ָ
 * \param[in] i2c_handle   : ZLG600I2CʹøhandleI2Cͨ
 * \param[in] p_i2c_info   : I2CͨģʽϢ
 *
 * \return  ZLG600 Ĳʼʧܣ򷵻NULL
 */ 
am_zlg600_handle_t am_zlg600_i2c_old_init (am_zlg600_i2c_dev_t            *p_dev,
                                           am_i2c_handle_t                 i2c_handle,
                                           const am_zlg600_i2c_dev_info_t *p_i2c_info);

/**
 * \brief ʼZLG600ʹI2C֡ͨģʽȡZLG600ı׼handle
 *
 * ظʵӲʹúͨӷʽѡʹI2Cʹøúʼ
 * Իhandleʹam_zlg600_uart_init()handle
 *
 * \param[in] p_dev        : ָZLG600豸ָ
 * \param[in] i2c_handle   : ZLG600I2CʹøhandleI2Cͨ
 * \param[in] p_i2c_info   : I2CͨģʽϢ
 *
 * \return  ZLG600 Ĳʼʧܣ򷵻NULL
 */
am_zlg600_handle_t am_zlg600_i2c_new_init (am_zlg600_i2c_dev_t            *p_dev,
                                           am_i2c_handle_t                 i2c_handle,
                                           const am_zlg600_i2c_dev_info_t *p_i2c_info);

/**
 * \brief ZLG600ĲƷ汾Ϣ磺ZLG600SP/T V1.00
 *
 * \param[in] handle     : ZLG600Ĳ
 * \param[in] p_info     : װػȡϢĻСӦΪ 20 ֽ
 *
 * \retval AM_OK      : ȡϢɹ
 * \retval -AM_EINVAL : ȡϢʧܣ
 * \retval -AM_EIO    : ȡϢʧܣͨų
 */ 
int am_zlg600_info_get (am_zlg600_handle_t handle,
                        uint8_t           *p_info);

/**
 * \brief ICӿڣִгɹICӿڣĬ֧TypeA
 *
 * \param[in] handle     : ZLG600Ĳ
 *
 * \retval AM_OK      : óɹ
 * \retval -AM_EINVAL : ʧܣ
 * \retval -AM_EIO    : ʧܣͨų
 */ 
int am_zlg600_ic_port_config (am_zlg600_handle_t handle);

/**
 * \brief رICӿ
 *
 * \param[in] handle     : ZLG600Ĳ
 *
 * \retval AM_OK      : رճɹ
 * \retval -AM_EINVAL : رʧܣ
 * \retval -AM_EIO    : رʧܣͨų
 */ 
int am_zlg600_ic_port_close (am_zlg600_handle_t handle);
                            
/**
 * \name ICӿڵЭ飨ģʽ
 * @{
 */
#define  AM_ZLG600_IC_ISOTYPE_A      0x00    /**< \brief ISO14443-3A */
#define  AM_ZLG600_IC_ISOTYPE_B      0x04    /**< \brief ISO14443-3B */
/** @}*/

/**
 * \brief ICӿЭ飨ģʽTypeA  TypeB
 *
 * \param[in] handle     : ZLG600Ĳ
 * \param[in] isotype    : ICӿڵЭͣʹк 
 *                          - #AM_ZLG600_IC_ISOTYPE_A
 *                          - #AM_ZLG600_IC_ISOTYPE_B
 *
 * \retval AM_OK      : óɹ
 * \retval -AM_EINVAL : ʧܣ
 * \retval -AM_EIO    : ʧܣͨų
 */ 
int am_zlg600_ic_isotype_set (am_zlg600_handle_t handle,
                              uint8_t            isotype);
                             
/**
 * \name ICܳ
 * @{
 */

#define  AM_ZLG600_IC_KEY_TYPE_A      0x60    /**< \brief  A  */
#define  AM_ZLG600_IC_KEY_TYPE_B      0x61    /**< \brief  B  */

/** @}*/

/**
 * \brief װICԿ
 *
 * ԿģڲģԿʧZLG600SP/Tģ鹲ܱ
 * AԿ16顢BԿ16顣
 *
 * \param[in] handle     : ZLG600Ĳ
 * \param[in] key_type   : Կͣʹֵ
 *                          - #AM_ZLG600_IC_KEY_TYPE_A
 *                          - #AM_ZLG600_IC_KEY_TYPE_B
 * \param[in] key_sec    : Կţɱ16Կţ0 ~ 15
 * \param[in] p_key      : Կ
 * \param[in] key_length : ԿĳȣֻΪ66ֽԿ1616ֽԿ
 *
 * \retval AM_OK      : װԿɹ
 * \retval -AM_EINVAL : װʧܣ
 * \retval -AM_EIO    : װʧܣͨų
 */ 
int am_zlg600_ic_key_load (am_zlg600_handle_t  handle,
                           uint8_t             key_type,
                           uint8_t             key_sec,
                           uint8_t            *p_key,
                           uint8_t             key_length);

/**
 * \brief ICӿڵļĴֵ
 *
 * ģ϶доƬڲļĴֵͨʵֺܶ
 * ɵĹ
 *
 * \param[in] handle     : ZLG600Ĳ
 * \param[in] reg_addr   : ĴַΧ0x00 ~ 0x3F
 * \param[in] reg_val    : õļĴֵ
 *
 * \retval AM_OK      : óɹ
 * \retval -AM_EINVAL : ʧܣ
 * \retval -AM_EIO    : ʧܣͨų
 */ 
int am_zlg600_ic_reg_set (am_zlg600_handle_t handle,
                          uint8_t            reg_addr,
                          uint8_t            reg_val);

/**
 * \brief ȡICӿڵļĴֵ
 *
 * ģ϶доƬڲļĴֵ
 * ͨʵֺܶɵĹ
 *
 * \param[in]  handle     : ZLG600Ĳ
 * \param[in]  reg_addr   : ĴַΧ0x00 ~ 0x3F
 * \param[out] p_val      : ȡĴֵָ
 *
 * \retval AM_OK      : ȡɹ
 * \retval -AM_EINVAL : ȡʧܣ
 * \retval -AM_EIO    : ȡʧܣͨų
 */ 
int am_zlg600_ic_reg_get (am_zlg600_handle_t  handle,
                          uint8_t             reg_addr,
                          uint8_t            *p_val);

/**
 * \brief ò
 *
 * UARTͨŹиıͨŵĲʣֵ
 *
 * \param[in] handle        : ZLG600Ĳ
 * \param[in] baudrate_flag : ʱ־֧еĲʣӦʹú
 *                            AM_ZLG600_BAUDRATE_* òΪ115200
 *                            ʹú #AM_ZLG600_BAUDRATE_115200
 *
 * \retval AM_OK      : óɹ
 * \retval -AM_EINVAL : ʧܣ
 * \retval -AM_EIO    : ʧܣͨų
 */ 
int am_zlg600_baudrate_set (am_zlg600_handle_t handle,
                            uint32_t           baudrate_flag);

/**
 * \name ʽ
 * @{
 */

#define  AM_ZLG600_ANT_TX1_ONLY      1    /**< \brief TX1         */
#define  AM_ZLG600_ANT_TX2_ONLY      2    /**< \brief TX2         */
#define  AM_ZLG600_ANT_TX1_AND_TX2   3    /**< \brief TX1TX2ͬʱ  */
#define  AM_ZLG600_ANT_NONE          0    /**< \brief ͬʱرTX1TX2      */

/** @}*/
      
/**
 * \brief ʽ򿪡رĳ
 *
 * \param[in] handle       : ZLG600Ĳ
 * \param[in] antmode_flag : ʹú AM_ZLG600_ANT_*#AM_ZLG600_ANT_TX1_ONLY
 *
 * \retval AM_OK      : óɹ
 * \retval -AM_EINVAL : ʧܣ
 * \retval -AM_EIO    : ʧܣͨų
 */ 
int am_zlg600_ant_mode_set (am_zlg600_handle_t handle,
                            uint8_t            antmode_flag);

/**
 * \brief ֡ʽл¡֡ʽóɹ粻ʧ
 *
 * \param[in] handle     : ZLG600Ĳ
 * \param[in] flag       : ֡ʽ־ʹк꣺
 *                           - #AM_ZLG600_FRAME_FMT_NEW
 *                           - #AM_ZLG600_FRAME_FMT_OLD
 *
 * \retval AM_OK      : óɹ
 * \retval -AM_EINVAL : ʧܣ
 * \retval -AM_EIO    : ʧܣͨų
 *
 * \note ֡лɹҪµĿ֡ʽĳʼ
 */ 
int am_zlg600_frame_fmt_set (am_zlg600_handle_t handle,
                             uint8_t            flag);

/**
 * \brief 豸ģʽʹӻַ
 *
 * \param[in] handle     : ZLG600Ĳ
 * \param[in] mode       : ģĹģʽʹкֵ
 *                         - #AM_ZLG600_MODE_MASTER      ģʽ
 *                         - #AM_ZLG600_MODE_SLAVE       ӻģʽ Ĭϣ
 *                         - #AM_ZLG600_MODE_AUTO_CHECK  Զģʽ
 *                         - #AM_ZLG600_MODE_I2C         I2Cģʽ
 *                         - #AM_ZLG600_MODE_UART        UARTģʽ
 *
 * \param[in] new_addr   : õZLG600ӻַһλΪ0Чֵ0x02 ~ 0xFE
 *
 * \retval AM_OK      : óɹ,óɹ󣬺Ӧʹµĵַͨ
 * \retval -AM_EINVAL : ʧܣ
 * \retval -AM_EIO    : ʧܣͨų
 */ 
int am_zlg600_mode_addr_set (am_zlg600_handle_t handle,
                             uint8_t            mode,
                             uint8_t            new_addr);
      
/**
 * \brief ȡ豸ģʽʹӻַ
 *
 * \param[in]  handle     : ZLG600Ĳ
 * \param[out] p_mode     : ڻȡģ鹤ģʽָ룬ȡֵλ£
 *                          - bit7 ~ 40,ӻģʽ; 1,Զ⿨Ƭģʽģʽ
 *                          - bit3 ~ 00,Զģʽ; 1,I2Cͨģʽ; 2,UARTģʽ 
 * \param[out] p_addr     : ڻȡǰZLG600Ĵӻַָ
 *
 * \retval AM_OK      : ȡɹ
 * \retval -AM_EINVAL : ȡʧܣ
 * \retval -AM_EIO    : ȡʧܣͨų
 */ 
int am_zlg600_mode_addr_get (am_zlg600_handle_t  handle,
                             uint8_t            *p_mode,
                             uint8_t            *p_addr);
                             
/**
 * \brief װûԿģṩ216ֽڵĴ洢ռڱûԿ
 *
 * \param[in] handle     : ZLG600Ĳ
 * \param[in] key_sec    : Կţɱ2ûԿţ0 ~ 1
 * \param[in] p_key      : ԿԿĳȹ̶Ϊ16ֽ
 *
 * \retval AM_OK      : װԿɹ
 * \retval -AM_EINVAL : װʧܣ
 * \retval -AM_EIO    : װʧܣͨų
 */ 
int am_zlg600_user_key_load (am_zlg600_handle_t handle,
                             uint8_t            key_sec,
                             uint8_t           *p_key);
                              
/**
 * \brief дEEPROMģڲӵһ256ByteEEPROM
 *
 * \param[in] handle      : ZLG600Ĳ
 * \param[in] eeprom_addr : EEPROMƬڵַ0 ~ 255
 * \param[in] p_buf       : дݵĻ
 * \param[in] nbytes      : дݵֽ247ֽ
 *
 * \retval AM_OK      : дݳɹ
 * \retval -AM_EINVAL : дʧܣ
 * \retval -AM_EIO    : дʧܣͨų
 */ 
int am_zlg600_eeprom_write (am_zlg600_handle_t handle,
                            uint8_t            eeprom_addr,
                            uint8_t           *p_buf,
                            uint8_t            nbytes);

/**
 * \brief EEPROMģڲӵһ256ByteEEPROM
 *
 * \param[in]  handle      : ZLG600Ĳ
 * \param[in]  eeprom_addr : EEPROMƬڵַ0 ~ 255
 * \param[out] p_buf       : ݵĻ
 * \param[in]  nbytes      : ݵֽ249ֽ
 *
 * \retval AM_OK      : ȡݳɹ
 * \retval -AM_EINVAL : ȡʧܣ
 * \retval -AM_EIO    : ȡʧܣͨų
 */ 
int am_zlg600_eeprom_read (am_zlg600_handle_t handle,
                           uint8_t            eeprom_addr,
                           uint8_t           *p_buf,
                           uint8_t            nbytes);

/**
 * \name Mifareģʽ
 * @{
 */

#define AM_ZLG600_MIFARE_CARD_REQ_IDLE 0x26 /**< \brief IDLEģʽ,еĿ */
#define AM_ZLG600_MIFARE_CARD_REQ_ALL  0x52 /**< \brief ALLģʽ,еĿ  */

/** @}*/ 

/**
 * \brief Mifare
 *
 *  úΪֻҪISO14443A׼ĿӦܷӦ
 *
 * \param[in] handle      : ZLG600Ĳ
 * \param[in] req_mode    : ģʽʹк꣺
 *                           - #AM_ZLG600_MIFARE_CARD_REQ_IDLE
 *                           - #AM_ZLG600_MIFARE_CARD_REQ_ALL
 * \param[in] p_atq       : ȡӦϢATQָ룬ֵΪ16λͬ
 *                          ͷصϢͬ,ͷصATQ£
 *
 *            Mifare1 S50    | Mifare1 S70 | Mifare1 Light | Mifare0 UltraLight 
 *       --------------------|-------------|---------------|-------------------
 *              0x0004       |    0x0002   |    0x0010     |      0x0044 
 *       ----------------------------------------------------------------------
 *          Mifare3 DESFire  |   SHC1101   |    SHC1102    |      11RF32
 *       --------------------|-------------|---------------|-------------------
 *               0x0344      |    0x0004   |    0x3300     |      0x0004
 *
 * \retval AM_OK      : ɹ
 * \retval -AM_EINVAL : ʧܣ
 * \retval -AM_EIO    : ʧܣͨų
 *
 * \note ߺ󣬴ƵлȡӶõ縴λλ󿨴IDLEģʽ
 * ģʽһʱʱĿӦĳһſɹ˹
 * HaltDeSelectHaltģʽʱĿֻӦALL0x52
 * ģʽ󣬳ǽ뿪߸Ӧٽ롣
 */ 
int am_zlg600_mifare_card_request (am_zlg600_handle_t handle,
                                   uint8_t            req_mode,
                                   uint16_t          *p_atq);

/**
 * \name Mifareײȼ
 * @{
 */
#define AM_ZLG600_MIFARE_CARD_ANTICOLL_1  0x93  /**< \brief һײ  */
#define AM_ZLG600_MIFARE_CARD_ANTICOLL_2  0x95  /**< \brief ڶײ  */
#define AM_ZLG600_MIFARE_CARD_ANTICOLL_3  0x97  /**< \brief ײ  */
/** @}*/ 

/**
 * \brief Mifareķײ
 *
 *  Ҫɹִһɹܽзײ򷵻ش
 *
 * \param[in] handle         : ZLG600Ĳ
 * \param[in] anticoll_level : ײȼʹк꣺
 *                             - #AM_ZLG600_MIFARE_CARD_ANTICOLL_1
 *                             - #AM_ZLG600_MIFARE_CARD_ANTICOLL_2
 *                             - #AM_ZLG600_MIFARE_CARD_ANTICOLL_3
 *
 * \param[in] know_uid : ֪Ŀк
 * \param[in] nbit_cnt : λ֪кuidλֵС32
 *                       - nbit_cntֵΪ0,û֪кţкŵλ
 *                         Ҫӱá
 *                       - nbit_cntΪ0к֪кŵֵʾ
 *                         кŵǰnbit_cntλΪ֪кʾǰnbit_cntλ
 *                         ֵλҪá
 * \param[in] *p_uid   : ײȡĿк
 *
 * \retval AM_OK      : ɹ
 * \retval -AM_EINVAL : ʧܣ
 * \retval -AM_EIO    : ʧܣͨų
 *
 * \note  ISO14443A׼кŶȫΨһģΨһԣʵַ
 * ײ㷨߼ſͬʱ߸ӦܹҵһкŽ
 * Ŀ
 */ 
int am_zlg600_mifare_card_anticoll (am_zlg600_handle_t handle,
                                    uint8_t            anticoll_level,
                                    uint32_t           know_uid,
                                    uint8_t            nbit_cnt,
                                    uint32_t          *p_uid);
                       
/**
 * \brief Mifareѡ
 *
 *  Ҫɹִһηײسɹܽпѡ򷵻ش
 *
 * \param[in] handle         : ZLG600Ĳ
 * \param[in] anticoll_level : ײȼʹк꣺
 *                             - #AM_ZLG600_MIFARE_CARD_ANTICOLL_1
 *                             - #AM_ZLG600_MIFARE_CARD_ANTICOLL_2
 *                             - #AM_ZLG600_MIFARE_CARD_ANTICOLL_3
 * \param[in] uid   : ǰһײȡUID
 * \param[in] p_sak : صϢbit2Ϊ1UID
 *
 * \retval AM_OK      : ɹ
 * \retval -AM_EINVAL : ʧܣ
 * \retval -AM_EIO    : ʧܣͨų
 *
 * \note  кų֣4ֽڡ7ֽں10ֽڡ 4ֽڵֻҪһѡ񼴿
 * õкţMifare1 S50/S70ȣ7ֽڵҪöѡܵõ
 * ţǰһõкŵֽΪ־0x88кֻк3ֽڿã
 * һѡܵõ4ֽкţ߰˳ӼΪ7ֽкţUltraLight
 * DesFireȣ10ֽڵԴƣδִ࿨
 */ 
int am_zlg600_mifare_card_select (am_zlg600_handle_t handle,
                                  uint8_t            anticoll_level,
                                  uint32_t           uid,
                                  uint8_t           *p_sak);

/**
 * \brief MifareĹʹѡĿHALT״̬
 *
 *  HALT״̬£ӦIDLEģʽ󣬳ǽλ뿪߸
 *  Ӧٽ롣ӦALL
 *
 * \param[in] handle         : ZLG600Ĳ
 *
 * \retval AM_OK      : ɹ
 * \retval -AM_EINVAL : ʧܣ
 * \retval -AM_EIO    : ʧܣͨų
 */ 
int am_zlg600_mifare_card_halt (am_zlg600_handle_t handle);

                               
/**
 * \brief MifareE2Կ֤
 *
 *     ģڲѴԿ뿨Կ֤ʹøǰӦáװICԿ
 * Կɹģڡ⣬Ҫ֤ĿŲģԿȡ
 *
 * \param[in] handle     : ZLG600Ĳ
 * \param[in] key_type   : Կͣʹֵ
 *                          - #AM_ZLG600_IC_KEY_TYPE_A
 *                          - #AM_ZLG600_IC_KEY_TYPE_B
 * \param[in] p_uid      : кţ4ֽ
 * \param[in] key_sec    : Կţ0 ~ 7
 * \param[in] nblock     : Ҫ֤ĿţȡֵΧ뿨йأ
 *                         - S500 ~ 63
 *                         - S700 ~ 255
 *                         - PLUS CPU 2K0 ~ 127
 *                         - PLUS CPU 4K0 ~ 255
 *
 * \retval AM_OK      : ֤ɹ
 * \retval -AM_EINVAL : ֤ʧܣ
 * \retval -AM_EIO    : ֤ʧܣͨų
 * 
 * \note PLUS CPUϵеĿĿ4ֽں7ֽ֣֮7ֽڿŵĿ
 *       ֻҪŵĸ4ֽڣȼ2ײõĿţΪ֤Ŀżɡ
 */
int am_zlg600_mifare_card_e2_authent (am_zlg600_handle_t  handle,
                                      uint8_t             key_type,
                                      uint8_t            *p_uid,
                                      uint8_t             key_sec,
                                      uint8_t             nblock);

/**
 * \brief MifareֱԿ֤
 *
 *    Ϊݣڴ֮ǰáװICԿǰ
 * ΪPLUS CPUĵȼ2ȼ3ֻ6ֽڣģԶ
 * 2Σȡǰ16ֽΪǰ֤Կ
 *
 * \param[in] handle     : ZLG600Ĳ
 * \param[in] key_type   : Կͣʹֵ
 *                          - #AM_ZLG600_IC_KEY_TYPE_A
 *                          - #AM_ZLG600_IC_KEY_TYPE_B
 * \param[in] p_uid      : кţ4ֽ
 * \param[in] p_key      : Կ
 * \param[in] key_len    : ԿĳȣֻΪ66ֽԿ1616ֽԿ
 * \param[in] nblock     : Ҫ֤ĿţȡֵΧ뿨йأ
 *                         - S500 ~ 63
 *                         - S700 ~ 255
 *                         - PLUS CPU 2K0 ~ 127
 *                         - PLUS CPU 4K0 ~ 255
 *
 * \retval AM_OK      : ֤ɹ
 * \retval -AM_EINVAL : ֤ʧܣ
 * \retval -AM_EIO    : ֤ʧܣͨų
 * 
 * \note PLUS CPUϵеĿĿ4ֽں7ֽ֣֮7ֽڿŵĿ
 *       ֻҪŵĸ4ֽڣȼ2ײõĿţΪ֤Ŀżɡ
 */
int am_zlg600_mifare_card_direct_authent (am_zlg600_handle_t  handle,
                                          uint8_t             key_type,
                                          uint8_t            *p_uid,
                                          uint8_t            *p_key,
                                          uint8_t             key_len,
                                          uint8_t             nblock);
          
/**
 * \brief Mifareݶȡ
 *
 *     ֤ɹ֮󣬲ܶӦĿݣ֤Ŀűͬһڣ
 * Mifare1ӿ0ʼ˳ÿ41ҪһſеĶв
 * ĳһϺ󣬱һܶһֱӽ֤
 * ʼPLUS CPUһԿ͵ǰԿͬ
 * Ҫٴ֤ԿֱӶɡ
 *
 * \param[in]  handle     : ZLG600Ĳ
 * \param[in]  nblock     : ȡݵĿ
 *                          - S500 ~ 63
 *                          - S700 ~ 255
 *                          - PLUS CPU 2K0 ~ 127
 *                          - PLUS CPU 4K0 ~ 255
 * \param[out] p_buf      : ŶȡݣСΪ 16
 *
 * \retval AM_OK      : ȡɹ
 * \retval -AM_EINVAL : ȡʧܣ
 * \retval -AM_EIO    : ȡʧܣͨų
 */
int am_zlg600_mifare_card_read (am_zlg600_handle_t  handle,
                                uint8_t             nblock,
                                uint8_t            *p_buf);

/**
 * \brief Mifareд,д֮ǰɹԿ֤
 *
 *      Կĳһ֤ɹ󣬼ɶͬһĸдֻҪ
 *  аλβ飬ǸΨһPLUS CPUȼ23AES
 *  Կλ޸Կ
 *
 * \param[in] handle     : ZLG600Ĳ
 * \param[in] nblock     : ȡݵĿ
 *                         - S500 ~ 63
 *                         - S700 ~ 255
 *                         - PLUS CPU 2K0 ~ 127
 *                         - PLUS CPU 4K0 ~ 255
 * \param[in] p_buf      : дݻСΪ 16
 *
 * \retval AM_OK      : дɹ
 * \retval -AM_EINVAL : дʧܣ
 * \retval -AM_EIO    : дʧܣͨų
 */
int am_zlg600_mifare_card_write (am_zlg600_handle_t  handle,
                                 uint8_t             nblock,
                                 uint8_t            *p_buf);
                                
/**
 * \brief Mifareд
 *
 * \param[in] handle     : ZLG600Ĳ
 * \param[in] nblock     : ȡݵĿ, 1 ~ 15
 * \param[in] p_buf      : дݻСΪ 4
 *
 * \retval AM_OK      : дɹ
 * \retval -AM_EINVAL : дʧܣ
 * \retval -AM_EIO    : дʧܣͨų
 *
 * ֻUltraLightЧUltraLightжMifare1һ
 */
int am_zlg600_ultralight_card_write (am_zlg600_handle_t  handle,
                                     uint8_t             nblock,
                                     uint8_t            *p_buf);

/**
 * \name Mifareֵģʽ
 * @{
 */

#define AM_ZLG600_MIFARE_CARD_VALUE_ADD  0xC1  /**< \brief   */
#define AM_ZLG600_MIFARE_CARD_VALUE_SUB  0xC0  /**< \brief   */

/** @}*/

/**
 * \brief MifareֵMifareֵмӼ
 *
 * \param[in] handle     : ZLG600Ĳ
 * \param[in] mode       : ֵģʽǼӻʹк꣺
 *                          - AM_ZLG600_MIFARE_CARD_VALUE_ADD
 *                          - AM_ZLG600_MIFARE_CARD_VALUE_SUB
 * \param[in] nblock     : ֵĿ
 *                           - S500 ~ 63
 *                            - S700 ~ 255
 *                            - PLUS CPU 2K0 ~ 127
 *                            - PLUS CPU 4K0 ~ 255
 * \param[in] ntransblk  : ţֵŵĿ
 * \param[in] value      : 4ֽз
 *
 * \retval AM_OK      : дɹ
 * \retval -AM_EINVAL : дʧܣ
 * \retval -AM_EIO    : дʧܣͨų
 *
 * \note ҪдݱҪֵĸʽɲοNXPĵ
 * 봫ͬ򽫲ĽдԭĿڣ봫Ųͬ
 * 򽫲Ľд봫ڣڵݱǣԭڵֵ䡣
 * ڵȼ2PLUS CPUֵ֧ȼ13֧֡
 */
int am_zlg600_mifare_card_val_operate (am_zlg600_handle_t  handle,
                                       uint8_t             mode,
                                       uint8_t             nblock,
                                       uint8_t             ntransblk,
                                       int32_t             value);

/**
 * \brief MifareλͨزźŹرָʱ䣬ٿʵֿƬλ
 * 
 * \param[in] handle     : ZLG600Ĳ
 * \param[in] time_ms    : رյʱ䣨λ:ms0Ϊһֱر
 *
 * \retval AM_OK      : ɹ
 * \retval -AM_EINVAL : ʧܣ
 * \retval -AM_EIO    : ʧܣͨų
 *
 * \note úźŹر룬һֱرգȵִһʱ򿪡
 */
int am_zlg600_mifare_card_reset (am_zlg600_handle_t  handle,
                                 uint8_t             time_ms);

/**
 * \brief MifareúڼƬ󡢷ײѡϡ
 *
 * \param[in] handle     : ZLG600Ĳ
 * \param[in] req_mode   : ģʽʹк꣺
 *                           - #AM_ZLG600_MIFARE_CARD_REQ_IDLE
 *                           - #AM_ZLG600_MIFARE_CARD_REQ_ALL
 * \param[in] p_atq      : Ӧ
 * \param[in] p_saq      : һѡӦSAK
 * \param[in] p_len      : кŵʵʳȣ4ֽڡ7ֽڻ10ֽ
 * \param[in] p_uid      : кŻӦкųȱһ£
 *                         粻ȷʹ󳤶10
 * \param[in]  uid_len    p_uid ָĳ
 *
 * \retval AM_OK      : дɹ
 * \retval -AM_EINVAL : дʧܣ
 * \retval -AM_EIO    : дʧܣͨų
 */
int am_zlg600_mifare_card_active (am_zlg600_handle_t  handle,
                                  uint8_t             req_mode,
                                  uint16_t           *p_atq,
                                  uint8_t            *p_saq,
                                  uint8_t            *p_len,
                                  uint8_t            *p_uid,
                                  uint8_t             uid_len);

/**
 * \brief Mifareֵֵֵ
 *
 * \param[in] handle     : ZLG600Ĳ
 * \param[in] nblock     : ȡݵֵַ
 * \param[in] value      : õֵ
 *
 * \retval AM_OK      : óɹ
 * \retval -AM_EINVAL : ʧܣ
 * \retval -AM_EIO    : ʧܣͨų
 */
int am_zlg600_mifare_card_val_set (am_zlg600_handle_t  handle,
                                   uint8_t             nblock,
                                   int32_t             value);
               
/**
 * \brief Mifareֵȡֵֵ
 *
 * \param[in] handle     : ZLG600Ĳ
 * \param[in] nblock     : ȡݵֵַ
 * \param[in] p_value    : ȡֵָ
 *
 * \retval AM_OK      : ȡɹ
 * \retval -AM_EINVAL : ȡʧܣ
 * \retval -AM_EIO    : ȡʧܣͨų
 */
int am_zlg600_mifare_card_val_get (am_zlg600_handle_t  handle,
                                   uint8_t             nblock,
                                   int32_t            *p_value);

/**
 * \brief 
 *
 *     úģչܣģƬⳤϵݴ
 * NXPƳNTAG213FUltralight CϵпƬǸÿƬ
 * дԿܡԿ֤ô亯ʵ֡
 *
 * \param[in] handle     : ZLG600Ĳ
 * \param[in] p_tx_buf   : ݵĻ
 * \param[in] tx_nbytes  : ֽ
 * \param[in] p_rx_buf   : ݵĻ
 * \param[in] rx_nbytes  : ֽ
 *
 * \retval AM_OK      : ɹ
 * \retval -AM_EINVAL : ʧܣ
 * \retval -AM_EIO    : ʧܣͨų
 */
int am_zlg600_mifare_card_cmd_trans (am_zlg600_handle_t  handle,
                                     uint8_t            *p_tx_buf,
                                     uint8_t             tx_nbytes,
                                     uint8_t            *p_rx_buf,
                                     uint8_t             rx_nbytes);

/**
 * \brief ݽ
 *
 * öд뿨Ƭݽͨúʵֶдйܡ
 *
 * \param[in]  handle     : ZLG600Ĳ
 * \param[in]  p_data_buf : Ҫ͵
 * \param[in]  len        : ݵĳ
 * \param[in]  wtxm_crc   : b7~b2,wtxmb1,RFU(0); b0,0,CRCֹܣ1,CRCʹܡ
 * \param[in]  fwi        : ʱȴʱ, ʱʱ=0x01 << fwi*302us
 * \param[out] p_rx_buf   : ݽӦ֡Ϣ
 * \param[in]  buf_size   : ջӦϢĻС
 * \param[in]  p_real_len : ڻȡʵʽյϢȣССʵ
 *                         Ϣȣಿֽ
 *
 * \retval AM_OK      : ɹ
 * \retval -AM_EINVAL : ʧܣ
 * \retval -AM_EIO    : ʧܣͨų
 */
int am_zlg600_mifare_card_data_exchange (am_zlg600_handle_t  handle,
                                         uint8_t            *p_data_buf,
                                         uint8_t             len,
                                         uint8_t             wtxm_crc,
                                         uint8_t             fwi,
                                         uint8_t            *p_rx_buf,
                                         uint8_t             buf_size,
                                         uint16_t           *p_real_len);

/**
 * \name Mifareģʽ
 * @{
 */

/** \brief ִһԶִHalt  */
#define AM_ZLG600_MIFARE_CARD_AD_HALT  0x08

/** @}*/

/**
 * \name Mifareģʽ
 * @{
 */

#define AM_ZLG600_MIFARE_CARD_TX_IN_TURN   0x00 /**< \brief TX1TX2 */
#define AM_ZLG600_MIFARE_CARD_TX_TX1       0x01 /**< \brief TX1 */
#define AM_ZLG600_MIFARE_CARD_TX_TX2       0x02 /**< \brief TX2 */
#define AM_ZLG600_MIFARE_CARD_TX_ALL       0x03 /**< \brief TX1TX2ͬʱ */

/** @}*/

/**
 * \name MifareԿ֤ģʽ
 * @{
 */

#define AM_ZLG600_MIFARE_CARD_AUTH_E2      0x45 /**< \brief E2Կ֤(E) */
#define AM_ZLG600_MIFARE_CARD_AUTH_DIRECT  0x46 /**< \brief ֱԿ֤(F) */
#define AM_ZLG600_MIFARE_CARD_AUTH_NO      0x00 /**< \brief ֤ */

/** @}*/

/**
 * \brief MifareԶ
 * \note Զڼ䣬κγԶģݳС3
 *       ˳Զģʽam_zlg600_mifare_card_active()
 *       ڴڼ䣬ģ齫ݳȴ2
 *
 * \param[in] handle    : ZLG600Ĳ
 * \param[in] p_adinfo  : ԶϢ
 *
 * \retval AM_OK      : дɹ
 * \retval -AM_EINVAL : дʧܣ
 * \retval -AM_EIO    : дʧܣͨų
 */
int am_zlg600_mifare_card_auto_detect_start (am_zlg600_handle_t                  handle,
                                             const am_zlg600_auto_detect_info_t *p_adinfo);

/**
 * \brief رMifareԶ
 *
 * \param[in] handle  : ZLG600Ĳ
 *
 * \retval AM_OK      : дɹ
 * \retval -AM_EINVAL : дʧܣ
 * \retval -AM_EIO    : дʧܣͨų
 */
int am_zlg600_mifare_card_auto_detect_stop (am_zlg600_handle_t handle);

/**
 * \brief MifareԶص
 *
 * \param[in] handle       : ZLG600Ĳ
 * \param[in] pfn_callback : صָ
 * \param[in] p_arg        : صڲ
 *
 * \retval AM_OK      : дɹ
 * \retval -AM_EINVAL : дʧܣ
 * \retval -AM_EIO    : дʧܣͨų
 */
int am_zlg600_mifare_card_auto_detect_cb_set (am_zlg600_handle_t  handle,
                                              am_pfnvoid_t        pfn_callback,
                                              void               *p_arg);

/**
 * \name MifareԶʶ
 * @{
 */

#define AM_ZLG600_MIFARE_CARD_AD_READ_CANCEL    0x00  /**< \brief ȡ */
#define AM_ZLG600_MIFARE_CARD_AD_READ_CONTINUE  0x01  /**< \brief  */

/** @} */

/**
 * \brief MifareƬϢ
 */
typedef struct am_zlg600_mifare_card_info {
    uint8_t     tx_mode;        /**< \brief ģʽ */
    uint8_t     uid_len;        /**< \brief UID(Ψһк)ȣ4710 */
    uint8_t     uid[10];        /**< \brief UID */
    uint8_t     card_data[16];  /**< \brief Ƭ */
} am_zlg600_mifare_card_info_t;

/**
 * \brief MifareȡԶ
 *
 * \param[in]  handle      : ZLG600Ĳ
 * \param[out] p_card_info : ָſƬݵָ
 *
 * \retval AM_OK      : дɹ
 * \retval -AM_EINVAL : дʧܣ
 * \retval -AM_EIO    : дʧܣͨų
 */
int am_zlg600_mifare_card_auto_detect_read (am_zlg600_handle_t            handle,
                                            am_zlg600_mifare_card_info_t *p_card_info);

/**
 * \name ICλĲ
 *
 *      ע⣬òʲZLG600ͨŲʣָZLG600ģӴʽIC
 *  ͨŲʡ
 *
 * @{
 */
#define  AM_ZLG600_CICC_RST_BAUDRATE_9600   0x11  /**< \brief  9600   */
#define  AM_ZLG600_CICC_RST_BAUDRATE_38400  0x13  /**< \brief  38400  */

/** @}*/

/**
 * \brief ӴʽICλԶPPS
 *
 *     ú临λִгɹԶICĸλϢִPPSȻ
 * ѡam_zlg600_cicc_tpdu()ʹõĴЭ飨T = 0 T = 1
 * 
 * \param[in]  handle        : ZLG600Ĳ
 * \param[in]  baudrate_flag : ICλĲʣʹк꣺
 *                             - AM_ZLG600_CICC_RST_BAUDRATE_9600
 *                             - AM_ZLG600_CICC_RST_BAUDRATE_38400
 * \param[out] p_rx_buf      : ŸλϢĻ
 * \param[in]  rx_bufsize    : ĳ
 * \param[out] p_rx_len      : ȡλϢȵָ룬ջĳС
 *                            ʵʳȣಿֶ
 *
 * \retval AM_OK      : ɹ
 * \retval -AM_EINVAL : ʧܣ
 * \retval -AM_EIO    : ʧܣͨų
 */
int am_zlg600_cicc_reset (am_zlg600_handle_t  handle,
                          uint8_t             baudrate_flag,
                          uint8_t            *p_rx_buf,
                          uint8_t             rx_bufsize,
                          uint16_t           *p_rx_len);

/**
 * \brief ӴʽICЭ
 *
 *     ݽӴʽICĸλϢԶѡT = 0T = 1Э飬
 * Ҫʹ߸ԤڴAPDU
 *
 * \param[in] handle        : ZLG600Ĳ
 * \param[in] p_tx_buf      : ͻ
 * \param[in] tx_bufsize    : ͻĳȣ 1 ~ 272
 * \param[in] p_rx_buf      : ջ
 * \param[in] rx_bufsize    : ջĳ
 * \param[in] p_rx_len      : ڻȡʵʽϢȵָ룬ջ
 *                            ĳСʵʳȣಿֶ
 *
 * \retval AM_OK      : ɹ
 * \retval -AM_EINVAL : ʧܣ
 * \retval -AM_EIO    : ʧܣͨų
 */
int am_zlg600_cicc_tpdu (am_zlg600_handle_t  handle,
                         uint8_t            *p_tx_buf,
                         uint16_t            tx_bufsize,
                         uint8_t            *p_rx_buf,
                         uint8_t             rx_bufsize,
                         uint16_t           *p_rx_len);
                        
/**
 * \brief ӴʽIC临λ
 *
 *     ú临λִ˽ӴʽICϵʱִгɹԶICĸλ
 * Ϣѡam_zlg600_cicc_tpdu()ʹõĴЭ飨T = 0 T = 1úû
 * ԶִPPSҪûݸλϢжǷʹam_zlg600_cicc_pps()
 * ޸ЭͲ
 *
 * \param[in] handle        : ZLG600Ĳ
 * \param[in] baudrate_flag : ICλĲʣʹк꣺
 *                             - AM_ZLG600_CICC_RST_BAUDRATE_9600
 *                             - AM_ZLG600_CICC_RST_BAUDRATE_38400
 * \param[in] p_rx_buf      : ŸλϢĻ
 * \param[in] rx_bufsize    : ĳ
 * \param[in] p_rx_len      : ȡλϢȵָ룬ջĳС
 *                            ʵʳȣಿֶ
 *
 * \retval AM_OK      : ɹ
 * \retval -AM_EINVAL : ʧܣ
 * \retval -AM_EIO    : ʧܣͨų
 */
int am_zlg600_cicc_coldreset (am_zlg600_handle_t  handle,
                              uint8_t             baudrate_flag,
                              uint8_t            *p_rx_buf,
                              uint8_t             rx_bufsize,
                              uint16_t           *p_rx_len);
                        
/**
 * \brief ӴʽICȸλ
 *
 *    úȸλûִ˽ӴʽICϵʱִгɹԶIC
 * λϢѡam_zlg600_cicc_tpdu()ʹõĴЭ飨T = 0 T = 1
 * ȸλ临λΨһûִICϵҪûݸλϢж
 * Ƿʹam_zlg600_cicc_pps()޸ЭͲúICʱӺ͵Դ
 * Ч²ִС
 *
 * \param[in] handle        : ZLG600Ĳ
 * \param[in] baudrate_flag : ICλĲʣʹк꣺
 *                             - AM_ZLG600_CICC_RST_BAUDRATE_9600
 *                             - AM_ZLG600_CICC_RST_BAUDRATE_38400
 * \param[in] p_rx_buf      : ŸλϢĻ
 * \param[in] rx_bufsize    : ĳ
 * \param[in] p_rx_len      : ȡλϢȵָ룬ջĳС
 *                            ʵʳȣಿֶ
 *
 * \retval AM_OK      : ɹ
 * \retval -AM_EINVAL : ʧܣ
 * \retval -AM_EIO    : ʧܣͨų
 */
int am_zlg600_cicc_warmreset (am_zlg600_handle_t  handle,
                              uint8_t             baudrate_flag,
                              uint8_t            *p_rx_buf,
                              uint8_t             rx_bufsize,
                              uint16_t           *p_rx_len);

/**
 * \brief ӴʽICͣرսӴʽICĵԴʱ
 *
 * \param[in] handle        : ZLG600Ĳ
 *
 * \retval AM_OK      : ɹ
 * \retval -AM_EINVAL : ʧܣ
 * \retval -AM_EIO    : ʧܣͨų
 */
int am_zlg600_cicc_deactive (am_zlg600_handle_t  handle);

/**
 * \brief ӴʽICЭͲѡ
 *
 *     ú临λȸλ֮ұִУЭģʽҪִУרģʽ
 * ҪִУԽӴʽIC˽⣬ʹam_zlg600_cicc_reset()иλ
 *
 * \param[in] handle     : ZLG600Ĳ
 * \param[in] p_pps      : PPSСһΪ4ֽ
 *                         - PPS[0] bit 3 ~ bit0  bit7
 *                         - PPS[0] bit 41  PPS1ڣ0  PPS1
 *                         - PPS[0] bit 51  PPS2ڣ0  PPS2
 *                         - PPS[0] bit 61  PPS3ڣ0  PPS3
                           - PPS[1] F/D
                           - PPS[2] N
                           - PPS[3] 
 *
 * \retval AM_OK      : ɹ
 * \retval -AM_EINVAL : ʧܣ
 * \retval -AM_EIO    : ʧܣͨų
 */
int am_zlg600_cicc_pps (am_zlg600_handle_t  handle,
                        uint8_t            *p_pps);

/**
 * \brief ӴʽICЭ飨T = 0
 *
 *     T = 0Э顣ӴʽICĴЭΪT = 0úͬ
 * am_zlg600_cicc_tpdu()
 *
 * \param[in] handle        : ZLG600Ĳ
 * \param[in] p_tx_buf      : ͻ
 * \param[in] tx_bufsize    : ͻĳȣ 1 ~ 272
 * \param[in] p_rx_buf      : ջ
 * \param[in] rx_bufsize    : ջĳ
 * \param[in] p_rx_len      : ڻȡʵʽϢȵָ룬ջ
 *                            ĳСʵʳȣಿֶ
 *
 * \retval AM_OK      : ɹ
 * \retval -AM_EINVAL : ʧܣ
 * \retval -AM_EIO    : ʧܣͨų
 */
int am_zlg600_cicc_tpdu_tp0 (am_zlg600_handle_t  handle,
                             uint8_t            *p_tx_buf,
                             uint16_t            tx_bufsize,
                             uint8_t            *p_rx_buf,
                             uint8_t             rx_bufsize,
                             uint16_t           *p_rx_len);

/**
 * \brief ӴʽICЭ飨T = 1
 *
 *     T = 1Э顣ӴʽICĴЭΪT = 1úͬ
 * am_zlg600_cicc_tpdu()
 *
 * \param[in] handle        : ZLG600Ĳ
 * \param[in] p_tx_buf      : ͻ
 * \param[in] tx_bufsize    : ͻĳȣ 1 ~ 272
 * \param[in] p_rx_buf      : ջ
 * \param[in] rx_bufsize    : ջĳ
 * \param[in] p_rx_len      : ڻȡʵʽϢȵָ룬ջ
 *                            ĳСʵʳȣಿֶ
 *
 * \retval AM_OK      : ɹ
 * \retval -AM_EINVAL : ʧܣ
 * \retval -AM_EIO    : ʧܣͨų
 */
int am_zlg600_cicc_tpdu_tp1 (am_zlg600_handle_t  handle,
                             uint8_t            *p_tx_buf,
                             uint16_t            tx_bufsize,
                             uint8_t            *p_rx_buf,
                             uint8_t             rx_bufsize,
                             uint16_t           *p_rx_len);

/**
 * \name PICCAģʽ
 * @{
 */
#define AM_ZLG600_PICCA_REQ_IDLE 0x26 /**< \brief IDLEģʽ,еĿ */
#define AM_ZLG600_PICCA_REQ_ALL  0x52 /**< \brief ALLģʽ,еĿ  */
/** @}*/ 

/**
 * \brief PICCA
 *
 *  úΪֻҪISO14443A׼ĿӦܷӦ
 *
 * \param[in] handle      : ZLG600Ĳ
 * \param[in] req_mode    : ģʽʹк꣺
 *                           - #AM_ZLG600_PICCA_REQ_IDLE
 *                           - #AM_ZLG600_PICCA_REQ_ALL
 * \param[in] p_atq       : ȡӦϢATQָ룬ֵΪ16λͬ
 *                          ͷصϢͬ,ͷصATQ£
 *
 *            Mifare1 S50    | Mifare1 S70 | Mifare1 Light | Mifare0 UltraLight 
 *       --------------------|-------------|---------------|-------------------
 *              0x0004       |    0x0002   |    0x0010     |      0x0044 
 *       ----------------------------------------------------------------------
 *          Mifare3 DESFire  |   SHC1101   |    SHC1102    |      11RF32
 *       --------------------|-------------|---------------|-------------------
 *               0x0344      |    0x0004   |    0x3300     |      0x0004
 *
 * \retval AM_OK      : ɹ
 * \retval -AM_EINVAL : ʧܣ
 * \retval -AM_EIO    : ʧܣͨų
 *
 * \note ߺ󣬴ƵлȡӶõ縴λλ󿨴IDLEģʽ
 * ģʽһʱʱĿӦĳһſɹ˹
 * HaltDeSelectHaltģʽʱĿֻӦALL0x52
 * ģʽ󣬳ǽ뿪߸Ӧٽ롣
 */ 
int am_zlg600_picca_request (am_zlg600_handle_t handle,
                             uint8_t            req_mode,
                             uint16_t          *p_atq);

/**
 * \name PICCAײȼ
 * @{
 */
#define AM_ZLG600_PICCA_ANTICOLL_1  0x93  /**< \brief һײ  */
#define AM_ZLG600_PICCA_ANTICOLL_2  0x95  /**< \brief ڶײ  */
#define AM_ZLG600_PICCA_ANTICOLL_3  0x97  /**< \brief ײ  */
/** @}*/ 

/**
 * \brief PICCAķײ
 *
 *  Ҫɹִһɹܽзײ򷵻ش
 *
 * \param[in] handle         : ZLG600Ĳ
 * \param[in] anticoll_level : ײȼʹк꣺
 *                             - #AM_ZLG600_MIFARE_CARD_ANTICOLL_1
 *                             - #AM_ZLG600_MIFARE_CARD_ANTICOLL_2
 *                             - #AM_ZLG600_MIFARE_CARD_ANTICOLL_3
 *
 * \param[in] know_uid : ֪Ŀк
 * \param[in] nbit_cnt : λ֪кuidλֵС32
 *                       - nbit_cntֵΪ0,û֪кţкŵλ
 *                         Ҫӱá
 *                       - nbit_cntΪ0к֪кŵֵʾ
 *                         кŵǰnbit_cntλΪ֪кʾǰnbit_cntλ
 *                         ֵλҪá
 * \param[in] *p_uid   : ײȡĿк
 *
 * \retval AM_OK      : ɹ
 * \retval -AM_EINVAL : ʧܣ
 * \retval -AM_EIO    : ʧܣͨų
 *
 * \note  ISO14443A׼кŶȫΨһģΨһԣʵַ
 * ײ㷨߼ſͬʱ߸ӦܹҵһкŽ
 * Ŀ
 */ 
int am_zlg600_picca_anticoll (am_zlg600_handle_t handle,
                              uint8_t            anticoll_level,
                              uint32_t           know_uid,
                              uint8_t            nbit_cnt,
                              uint32_t          *p_uid);
                       
/**
 * \brief PICC Aѡ
 *
 *  Ҫɹִһηײسɹܽпѡ򷵻ش
 *
 * \param[in] handle         : ZLG600Ĳ
 * \param[in] anticoll_level : ײȼʹк꣺
 *                             - #AM_ZLG600_MIFARE_CARD_ANTICOLL_1
 *                             - #AM_ZLG600_MIFARE_CARD_ANTICOLL_2
 *                             - #AM_ZLG600_MIFARE_CARD_ANTICOLL_3
 * \param[in] uid   : ǰһײȡUID
 * \param[in] p_sak : صϢbit2Ϊ1UID
 *
 * \retval AM_OK      : ɹ
 * \retval -AM_EINVAL : ʧܣ
 * \retval -AM_EIO    : ʧܣͨų
 *
 * \note  кų֣4ֽڡ7ֽں10ֽڡ 4ֽڵֻҪһѡ񼴿
 * õкţMifare1 S50/S70ȣ7ֽڵҪöѡܵõ
 * ţǰһõкŵֽΪ־0x88кֻк3ֽڿã
 * һѡܵõ4ֽкţ߰˳ӼΪ7ֽкţUltraLight
 * DesFireȣ10ֽڵԴƣδִ࿨
 */ 
int am_zlg600_picca_select (am_zlg600_handle_t handle,
                            uint8_t            anticoll_level,
                            uint32_t           uid,
                            uint8_t           *p_sak);

/**
 * \brief PICCAĹʹѡĿHALT״̬
 *
 *  HALT״̬£ӦIDLEģʽ󣬳ǽλ뿪߸
 *  Ӧٽ롣ӦALL
 *
 * \param[in] handle         : ZLG600Ĳ
 *
 * \retval AM_OK      : ɹ
 * \retval -AM_EINVAL : ʧܣ
 * \retval -AM_EIO    : ʧܣͨų
 */ 
int am_zlg600_picca_halt (am_zlg600_handle_t handle);

/**
 * \brief PICC TypeARATSrequest for answer to select
 *
 *     RATSrequest for answer to selectISO14443-4Эģ鷢RATS
 * ƬATSanswer to selectΪRATSӦִиǰȽһ
 * ѡִйһRATSٴִRATSȽ
 *
 * \param[in] handle     : ZLG600Ĳ
 * \param[in] cid        : ʶcard IDentifierȡֵΧ0x000x0E
 * \param[in] p_ats_buf  : ڽջӦϢֽͬвͬ
 * \param[in] bufsize    : С
 * \param[in] p_rx_len   : ڻȡʵʽյϢȣССʵ
 *                         Ϣȣಿֽ
 *
 * \retval AM_OK      : дɹ
 * \retval -AM_EINVAL : дʧܣ
 * \retval -AM_EIO    : дʧܣͨų
 */
int am_zlg600_picca_rats (am_zlg600_handle_t  handle,
                          uint8_t             cid,
                          uint8_t            *p_ats_buf,
                          uint8_t             bufsize,
                          uint16_t           *p_rx_len);

/**
 * \brief PICC TypeAPPSrequest for answer to select
 *
 *     PPSprotocal and parameter selectionISO14443-4Эڸı
 * صרЭǱģֻ֧ĬϲĲΪ
 * 0ɡִиǰȳɹִһRATS
 *
 * \param[in] handle     : ZLG600Ĳ
 * \param[in] dsi_dri    : ģ뿨ͨŲ,Ϊ0Ĭϣ
 *
 * \retval AM_OK      : дɹ
 * \retval -AM_EINVAL : дʧܣ
 * \retval -AM_EIO    : дʧܣͨų
 */
int am_zlg600_picca_pps (am_zlg600_handle_t  handle,
                         uint8_t             dsi_dri);
                         
/**
 * \brief PICC TypeA
 *
 *   ISO14443-4ЭڽƬΪHALT״̬ڹ
 *HALT״̬ĿáΪALL¼ִֻСRATS
 * Ŀø
 *
 * \param[in] handle     : ZLG600Ĳ
 *
 * \retval AM_OK      : дɹ
 * \retval -AM_EINVAL : дʧܣ
 * \retval -AM_EIO    : дʧܣͨų
 */
int am_zlg600_picca_deselect (am_zlg600_handle_t  handle);
                             
/**
 * \brief PICCT=CL
 *
 *     T=CLǰ˫鴫Э飬ISO14443-4Эڶд뿨Ƭ֮
 * һISO14443ЭCPUøЭдͨšøʱֻҪ
 * CPUCOSΪ뼴ɣ͡ʶCID֡ȴʱ
 * FWTȴʱչWTXMwaiting time extensionmultiplierȵɸ
 * Զɡ
 *
 * \param[in] handle      : ZLG600Ĳ
 * \param[in] p_cos_buf   : COS
 * \param[in] cos_bufsize COSĳ
 * \param[in] p_res_buf   : ڽջӦϢͬCOSֽвͬ
 * \param[in] res_bufsize : ջӦϢĻС
 * \param[in] p_rx_len    : ڻȡʵʽյϢȣССʵ
 *                          Ϣȣಿֽ
 *
 * \retval AM_OK      : дɹ
 * \retval -AM_EINVAL : дʧܣ
 * \retval -AM_EIO    : дʧܣͨų
 */
int am_zlg600_picc_tpcl (am_zlg600_handle_t  handle,
                         uint8_t            *p_cos_buf,
                         uint8_t             cos_bufsize,
                         uint8_t            *p_res_buf,
                         uint8_t             res_bufsize,
                         uint16_t           *p_rx_len);

/**
 * \brief ݽ
 *
 * öд뿨Ƭݽͨúʵֶдйܡ
 *
 * \param[in] handle     : ZLG600Ĳ
 * \param[in] p_data_buf : Ҫ͵ݣʵʹõĿйأ
 * \param[in] len        : ݵĳ
 * \param[in] wtxm_crc   : b7~b2,wtxmb1,RFU(0); b0,0-CRCֹܣ1-CRCʹܡ
 * \param[in] fwi        : ʱȴʱ, ʱʱ=0x01 << fwi*302us
 * \param[in] p_rx_buf   : ݽӦ֡Ϣ
 * \param[in] bufsize    : ջӦϢĻС
 * \param[in] p_rx_len   : ڻȡʵʽյϢȣССʵ
 *                         Ϣȣಿֽ
 *
 * \retval AM_OK      : ɹ
 * \retval -AM_EINVAL : ʧܣ
 * \retval -AM_EIO    : ʧܣͨų
 */
int am_zlg600_picc_data_exchange (am_zlg600_handle_t  handle,
                                  uint8_t            *p_data_buf,
                                  uint8_t             len,
                                  uint8_t             wtxm_crc,
                                  uint8_t             fwi,
                                  uint8_t            *p_rx_buf,
                                  uint8_t             bufsize,
                                  uint16_t           *p_rx_len);

/**
 * \brief PICC AλͨزźŹرָʱ䣬ٿʵֿƬλ
 * 
 * \param[in] handle     : ZLG600Ĳ
 * \param[in] time_ms    : رյʱ䣨λ:ms0Ϊһֱر
 *
 * \retval AM_OK      : ɹ
 * \retval -AM_EINVAL : ʧܣ
 * \retval -AM_EIO    : ʧܣͨų
 *
 * \note úźŹر룬һֱرգȵִһʱ򿪡
 */
int am_zlg600_picca_reset(am_zlg600_handle_t  handle,
                          uint8_t             time_ms);
                          
/**
 * \brief PICC AúڼƬ󡢷ײѡϡ
 *
 * \param[in] handle     : ZLG600Ĳ
 * \param[in] req_mode   : ģʽʹк꣺
 *                           - #AM_ZLG600_PICCA_REQ_IDLE
 *                           - #AM_ZLG600_PICCA_REQ_ALL
 * \param[in] p_atq      : Ӧ
 * \param[in] p_saq      : һѡӦSAK
 * \param[in] p_len      : кųȣ4ֽڻ7ֽ
 * \param[in] p_uid      : кŻӦкųȱһ
 * \param[in] uid_len    : p_uid ָĳ
 *
 * \retval AM_OK      : дɹ
 * \retval -AM_EINVAL : дʧܣ
 * \retval -AM_EIO    : дʧܣͨų
 */
int am_zlg600_picca_active (am_zlg600_handle_t  handle,
                            uint8_t             req_mode,
                            uint16_t           *p_atq,
                            uint8_t            *p_saq,
                            uint8_t            *p_len,
                            uint8_t            *p_uid,
                            uint8_t             uid_len);

/**
 * \name PICCBģʽ
 * @{
 */

#define AM_ZLG600_PICCB_REQ_IDLE 0x00 /**< \brief IDLEģʽ,еĿ */
#define AM_ZLG600_PICCB_REQ_ALL  0x08 /**< \brief ALLģʽ,еĿ  */

/** @}*/ 

/**
 * \brief PICC B
 *
 *    ڵøúǰҪȷICģʽѾΪTypeBģʽ
 *
 * \param[in] handle     : ZLG600Ĳ
 * \param[in] req_mode   : ģʽʹк꣺
 *                           - #AM_ZLG600_PICCB_REQ_IDLE
 *                           - #AM_ZLG600_PICCB_REQ_ALL
 * \param[in] p_uid      : UIDϢΪ12
 *
 * \retval AM_OK      : дɹ
 * \retval -AM_EINVAL : дʧܣ
 * \retval -AM_EIO    : дʧܣͨų
 */
int am_zlg600_piccb_active (am_zlg600_handle_t  handle,
                            uint8_t             req_mode,
                            uint8_t            *p_uid);

/**
 * \brief PICC BλͨزźŹرָʱ䣬ٿʵֿƬλ
 * 
 * \param[in] handle     : ZLG600Ĳ
 * \param[in] time_ms    : رյʱ䣨λ:ms0Ϊһֱر
 *
 * \retval AM_OK      : ɹ
 * \retval -AM_EINVAL : ʧܣ
 * \retval -AM_EIO    : ʧܣͨų
 *
 * \note úźŹر룬һֱرգȵִһʱ򿪡
 */
int am_zlg600_piccb_reset (am_zlg600_handle_t  handle,
                           uint8_t             time_ms);

/**
 * \brief PICCA
 *
 *  úΪֻҪISO14443A׼ĿӦܷӦ
 *
 * \param[in] handle      : ZLG600Ĳ
 * \param[in] req_mode    : ģʽʹк꣺
 *                           - #AM_ZLG600_PICCB_REQ_IDLE
 *                           - #AM_ZLG600_PICCB_REQ_ALL
 * \param[in] slot_time   : ʱ϶0 ~ 4
 * \param[in] p_uid       : صUIDϢΪ12
 *
 * \retval AM_OK      : ɹ
 * \retval -AM_EINVAL : ʧܣ
 * \retval -AM_EIO    : ʧܣͨų
 */ 
int am_zlg600_piccb_request (am_zlg600_handle_t  handle,
                             uint8_t             req_mode,
                             uint8_t             slot_time,
                             uint8_t            *p_uid);

/**
 * \brief PICC B ķײ
 *
 * \param[in] handle      : ZLG600Ĳ
 * \param[in] slot_flag   : ʱ϶ǣ2 ~ 16òֵʱ϶й
 *                          ϵʱ϶Ϊnʱ϶N < 2^n
 * \param[in] p_uid       : صUIDϢΪ12
 *
 * \retval AM_OK      : ɹ
 * \retval -AM_EINVAL : ʧܣ
 * \retval -AM_EIO    : ʧܣͨų
 */
int am_zlg600_piccb_anticoll (am_zlg600_handle_t  handle,
                              uint8_t             slot_flag,
                              uint8_t            *p_uid);
                             
/**
 * \brief PICC BĹʹѡĿHALT״̬
 *
 *     úBͿִйǰִгɹһִ
 * ɹ󣬿Ƭڹ״̬ģͨALLʽƬIDLE
 * ʽ
 *
 * \param[in] handle     : ZLG600Ĳ
 * \param[in] p_pupi     : PUPI,СΪ4ֽڵıʶ
 *
 * \retval AM_OK      : ɹ
 * \retval -AM_EINVAL : ʧܣ
 * \retval -AM_EIO    : ʧܣͨų
 */ 
int am_zlg600_piccb_halt (am_zlg600_handle_t handle,
                          uint8_t            *p_pupi);
 
/**
 * \brief ޸BĴ
 *
 *     úBͿִйǰִгɹһִ
 * ɹ󣬿Ƭڹ״̬ģͨALLʽƬIDLE
 * ʽ
 *
 * \param[in] handle     : ZLG600Ĳ
 * \param[in] p_pupi     : PUPI,СΪ4ֽڵıʶ
 * \param[in] cid        : ȡֵΧΪ 0 - 14, ֧CIDΪ0
 * \param[in] protype    : ֵ֧Э
 *                          - bit3 : 1-PCDֹPICCͨ,0-PCDPICCͨ
 *                          - bit2 ~ bit1 : 
 *                              - 11 : 10 etu + 512 / fs
 *                              - 10 : 10 etu + 256 / fs
 *                              - 01 : 10 etu + 128 / fs
 *                              - 00 : 10 etu + 32 / fs
 *                          - bit0 : 1-ѭISO14443-4,0-ѭISO14443-4
 *                                   (֤λΪ1)
 *
 * \retval AM_OK      : ɹ
 * \retval -AM_EINVAL : ʧܣ
 * \retval -AM_EIO    : ʧܣͨų
 */ 
int am_zlg600_piccb_attrib_set (am_zlg600_handle_t handle,
                                uint8_t           *p_pupi,
                                uint8_t            cid,
                                uint8_t            protype);

/**
 * \brief PLUS CPU˻
 *
 *     SL0Security Level 0ȫȼ0PLUS CPU˻PLUS CPU
 * ʱİȫȼΪSL0õȼ£Ҫκ֤Ϳдݣд
 * Ϊȫȼĳʼֵ磺SL00x0003д룺0xA0 0xA1 0xA2 0xA3 
 * 0xA4 0xA5 0xFF 0x07 0x80 0x69 0xFF 0xFF 0xFF 0xFF 0xFF 0xFFƬSL1
 * 0AԿΪ0xA0 0xA1 0xA2 0xA3 0xA4 0xA5Ĭϵ0xFF 0xFF 0xFF 
 * 0xFF 0xFF 0xFFSL0޸ĿƬĬݺԿ
 *
 * \param[in] handle        : ZLG600Ĳ
 * \param[in] sector_addr   : PLUS CPU洢ַ
 * \param[in] p_buf         : ͻ,ݸΪȷ16
 *
 * \retval AM_OK      : ɹ
 * \retval -AM_EINVAL : ʧܣ
 * \retval -AM_EIO    : ʧܣͨų
 */
int am_zlg600_pluscpu_persotcl_write (am_zlg600_handle_t  handle,
                                      uint16_t            sector_addr,
                                      uint8_t            *p_buf);

/**
 * \brief PLUS CPUύ˻
 *
 *  am_zlg600_pluscpu_persotcl_write()ֻǸ¿еݣݻδЧ
 * ύݲЧִиPLUS CPUİȫȼߵSL1SL3
 * ֧SL1ĿִиƬȫȼߵSL1ֻ֧SL0SL3
 * ִиƬȫȼߵSL3
 * ע⣺SL0Уֻ޸µִַύ
 * ?- 0x9000Կ
 *   - 0x9001ÿԿ
 *   - 0x9002SL2Կֻ֧SL2ĿиԿ
 *   - 0x9003SL3Կֻ֧SL3ĿиԿ
 *
 * \param[in] handle        : ZLG600Ĳ
 *
 * \retval AM_OK      : ɹ
 * \retval -AM_EINVAL : ʧܣ
 * \retval -AM_EIO    : ʧܣͨų
 */
int am_zlg600_pluscpu_persotcl_commit (am_zlg600_handle_t  handle);


/**
 * \brief PLUS CPU״ֱ֤Կ֤
 *
 * \param[in] handle       : ZLG600Ĳ
 * \param[in] authent_addr : ֤16λԿַ
 * \param[in] p_key        : ԿСΪ16ֽ
 *
 * \retval AM_OK      : ֤ɹ
 * \retval -AM_EINVAL : ֤ʧܣ
 * \retval -AM_EIO    : ֤ʧܣͨų
 *
 * \note PLUS CPUԿA/Bַͨż֣AESԿַݿ
 * ϵӦ:
 * ԿAַ=0x4000 +   2
 * ԿBַ=0x4000 +   2 + 1
 * ˣ֤1ԿAԿַΪ0x4002
 */
int am_zlg600_pluscpu_sl3_first_direct_authent (am_zlg600_handle_t  handle,
                                                uint16_t            authent_addr,
                                                uint8_t            *p_key);

/**
 * \brief PLUS CPU״֤E2Կ֤֤Կģڲ粻ʧ
 *
 * \param[in] handle        : ZLG600Ĳ
 * \param[in] authent_addr  : ֤16λԿַ
 * \param[in] key_sector    : Կ
 *
 * \retval AM_OK      : ֤ɹ
 * \retval -AM_EINVAL : ֤ʧܣ
 * \retval -AM_EIO    : ֤ʧܣͨų
 */
int am_zlg600_pluscpu_sl3_first_e2_authent (am_zlg600_handle_t  handle,
                                            uint16_t            authent_addr,
                                            uint8_t             key_sector);

/**
 * \brief PLUS CPUֱ֤Կ֤
 *
 *     úSL3 PLUS CPUĸԿ֤֤Կִֻ
 * ״֤ɹʹø֤ʹõʱ䲻ͬ
 * ״֤Ҫʱȡ֤ʱҪЩ
 *
 * \param[in] handle       : ZLG600Ĳ
 * \param[in] authent_addr : ֤16λԿַ
 * \param[in] p_key        : ԿСΪ16ֽ
 *
 * \retval AM_OK      : ֤ɹ
 * \retval -AM_EINVAL : ֤ʧܣ
 * \retval -AM_EIO    : ֤ʧܣͨų
 *
 */
int am_zlg600_pluscpu_sl3_follow_direct_authent (am_zlg600_handle_t  handle,
                                                 uint16_t            authent_addr,
                                                 uint8_t            *p_key);

/**
 * \brief PLUS CPU֤E2Կ֤,֤Կģڲ粻ʧ
 *
 * \param[in] handle        : ZLG600Ĳ
 * \param[in] authent_addr  : ֤16λԿַ
 * \param[in] key_sector    : Կ
 *
 * \retval AM_OK      : ֤ɹ
 * \retval -AM_EINVAL : ֤ʧܣ
 * \retval -AM_EIO    : ֤ʧܣͨų
 */
int am_zlg600_pluscpu_sl3_follow_e2_authent (am_zlg600_handle_t  handle,
                                             uint16_t            authent_addr,
                                             uint8_t             key_sector);

/**
 * \brief PLUS CPUSL3λ֤E2Կ֤,֤Կģڲ粻ʧ
 *
 *  PLUS CPUͨ״֤ʹùУλд֤Ϣ
 *
 * \param[in] handle        : ZLG600Ĳ
 *
 * \retval AM_OK      : ֤ɹ
 * \retval -AM_EINVAL : ֤ʧܣ
 * \retval -AM_EIO    : ֤ʧܣͨų
 */
int am_zlg600_pluscpu_sl3_reset_authent (am_zlg600_handle_t  handle);

/**
 * \name PLUS CPUݵģʽ
 * @{
 */
 
/** \brief MAC(CIPHER)ӦMAC */
#define  AM_ZLG600_PLUSCPU_SL3_READMODE_M_C_NM    0x30

/** \brief MAC(CIPHER)ӦMAC */
#define  AM_ZLG600_PLUSCPU_SL3_READMODE_M_C_M     0x31

/** \brief MACģӦMAC */
#define  AM_ZLG600_PLUSCPU_SL3_READMODE_M_NC_NM   0x32

/** \brief MACģӦMAC */
#define  AM_ZLG600_PLUSCPU_SL3_READMODE_M_NC_M    0x33

/** \brief MACģӦMAC */
#define  AM_ZLG600_PLUSCPU_SL3_READMODE_NM_C_NM   0x34

/** \brief MACģӦMAC */
#define  AM_ZLG600_PLUSCPU_SL3_READMODE_NM_C_M    0x35

/** \brief MACģӦMAC */
#define  AM_ZLG600_PLUSCPU_SL3_READMODE_NM_NC_NM  0x36

/** \brief MACģӦMAC */
#define  AM_ZLG600_PLUSCPU_SL3_READMODE_NM_NC_M   0x37

/** @}*/

/**
 * \brief PLUS CPUSL3ݿ
 *
 *  úڶȡSL3ݿ飬ڶݿ֮ǰɹִһԿ֤
 *
 * \param[in]  handle        : ZLG600Ĳ
 * \param[in]  read_mode     : ģʽʹú AM_ZLG600_PLUSCPU_SL3_READMODE_*
 *                             (#AM_ZLG600_PLUSCPU_SL3_READMODE_M_C_M)
 * \param[in]  start_block   : ݵʼ
 * \param[in]  block_num     : Ŀ1 ~ 3
 * \param[out] p_buf         : ȡݴŻСӦΪblock_num * 16
 *
 * \retval AM_OK      : ֤ɹ
 * \retval -AM_EINVAL : ֤ʧܣ
 * \retval -AM_EIO    : ֤ʧܣͨų
 */
int am_zlg600_pluscpu_sl3_read (am_zlg600_handle_t  handle,
                                uint8_t             read_mode,
                                uint16_t            start_block,
                                uint8_t             block_num,
                                uint8_t            *p_buf);

/**
 * \name PLUS CPUдݵģʽ
 * @{
 */
 
/** \brief MAC(CIPHER)ӦMAC */
#define  AM_ZLG600_PLUSCPU_SL3_WRITEMODE_M_C_NM     0xA0

/** \brief MAC(CIPHER)ӦMAC */
#define  AM_ZLG600_PLUSCPU_SL3_WRITEMODE_M_C_M      0xA1

/** \brief MACģӦMAC */
#define  AM_ZLG600_PLUSCPU_SL3_WRITEMODE_M_NC_NM    0xA2

/** \brief MACģӦMAC */
#define  AM_ZLG600_PLUSCPU_SL3_WRITEMODE_M_NC_M     0xA3

/** @}*/

/**
 * \brief PLUS CPUSL3дݿ
 *
 *  úдSL3ݿ飬дݿ֮ǰɹִһԿ֤
 *
 * \param[in] handle        : ZLG600Ĳ
 * \param[in] write_mode    : дģʽʹúAM_ZLG600_PLUSCPU_SL3_WRITEMODE_*
 *                            (#AM_ZLG600_PLUSCPU_SL3_WRITEMODE_M_C_NM)
 * \param[in] start_block   : дݵʼ
 * \param[in] block_num     : дĿ1 ~ 3
 * \param[in] p_buf         : дݴŻСӦΪblock_num * 16
 *
 * \retval AM_OK      : ɹ
 * \retval -AM_EINVAL : ʧܣ
 * \retval -AM_EIO    : ʧܣͨų
 */
int am_zlg600_pluscpu_sl3_write (am_zlg600_handle_t  handle,
                                 uint8_t             write_mode,
                                 uint16_t            start_block,
                                 uint8_t             block_num,
                                 uint8_t            *p_buf);

/**
 * \name PLUS CPU ֵģʽ
 * @{
 */
#define AM_ZLG600_PLUSCPU_VALUE_ADD  0xB7  /**< \brief   */
#define AM_ZLG600_PLUSCPU_VALUE_SUB  0xB9  /**< \brief   */
/** @}*/     
                                
/**
 * \brief PLUS CPUֵ
 *
 * \param[in] handle     : ZLG600Ĳ
 * \param[in] mode       : ֵģʽǼӻʹк꣺
 *                          - AM_ZLG600_PLUSCPU_VALUE_ADD
 *                          - AM_ZLG600_PLUSCPU_VALUE_SUB
 * \param[in] src_block  : ֵԴ
 * \param[in] dst_block  : ֵݴŵĿĿ
 * \param[in] value      : 4ֽз
 *
 * \retval AM_OK      : дɹ
 * \retval -AM_EINVAL : дʧܣ
 * \retval -AM_EIO    : дʧܣͨų
 *
 * \note ҪдݱҪֵĸʽɲοNXPĵ
 * 봫ͬ򽫲ĽдԭĿڣ봫Ųͬ
 * 򽫲Ľд봫ڣڵݱǣԭڵֵ䡣
 * ڵȼ2PLUS CPUֵ֧ȼ13֧֡
 */
int am_zlg600_pluscpu_val_operate (am_zlg600_handle_t  handle,
                                   uint8_t             mode,
                                   uint16_t            src_block,
                                   uint16_t            dst_block,
                                   int32_t             value);

/**
 * \brief ȡ֤Ϣ
 *
 * \param[in]  handle   : ZLG600Ĳ
 * \param[in]  req_mode : ģʽʹк꣺
 *                           - #AM_ZLG600_PICCB_REQ_IDLE
 *                           - #AM_ZLG600_PICCB_REQ_ALL
 * \param[out] p_info : ֤ϢСӦΪ8ֽڣ
 *
 * \retval AM_OK      : дɹ
 * \retval -AM_EINVAL : дʧܣ
 * \retval -AM_EIO    : дʧܣͨų
 *
 * \note Ҫж֤Ϣȵ am_zlg600_ic_isotype_set ӿڽΪB,
 *       ȡ֮Ӧ am_zlg600_ic_isotype_set ͻԭ򲻻ԭ
 */
int am_zlg600_id_card_info_get (am_zlg600_handle_t handle, uint8_t req_mode, uint8_t *p_info);

/**
 * \brief ָʱʱ
 *
 * \param[in]  handle     : ZLG600Ĳ
 * \param[in]  timeout_ms : ָʱʱ䣨룩
 *
 * \retval AM_OK : óɹ
 * \retval      : ʧ
 */
int am_zlg600_cmd_timeout_set (am_zlg600_handle_t handle, uint32_t timeout_ms);

/** 
 * @}
 */
 
#endif /* __AM_ZLG600_H */

/* end of file */
