/*******************************************************************************
*                                 AMetal
*                       ----------------------------
*                       innovating embedded platform
*
* Copyright (c) 2001-2018 Guangzhou ZHIYUAN Electronics Stock Co., Ltd.
* All rights reserved.
*
* Contact information:
* web site:    http://www.zlg.cn/
* e-mail:      ametal.support@zlg.cn
*******************************************************************************/

/**
 * \file
 * \brief ZigBee ģ ZM516X ӿ
 *
 * ʹñҪͷļ am_zm516x.h
 *
 * \par ʹʾ
 * \code
 * #include "am_zm516x.h"
 * \endcode
 *
 * \internal
 * \par modification history
 * - 1.01 18-01-16  pea, update API to v1.03 manual
 * - 1.00 15-09-06  afm, first implementation
 * \endinternal
 */

#ifndef __AM_ZM516X_H
#define __AM_ZM516X_H

#ifdef __cplusplus
extern "C" {
#endif

/**
 * \addtogroup am_if_zm516x
 * \copydoc am_zm516x.h
 * @{
 */

#include "am_types.h"
#include "am_wait.h"
#include "am_uart.h"
#include "am_rngbuf.h"

#define ZM516X_UART_RNGBUF_NREAD     0x0100   /**< \brief ɶݸ        */
#define ZM516X_UART_RNGBUF_NWRITE    0x0200   /**< \brief Ѿдݸ  */
#define ZM516X_UART_RNGBUF_FLUSH     0x0300   /**< \brief ն/д     */
#define ZM516X_UART_RNGBUF_WFLUSH    0x0400   /**< \brief д        */
#define ZM516X_UART_RNGBUF_RFLUSH    0x0500   /**< \brief ն        */
#define ZM516X_UART_RNGBUF_TIMEOUT   0x0600   /**< \brief õȴʱ        */

typedef struct am_zm516x_rngbuf_dev {

    /** \brief UART׼    */
    am_uart_handle_t  handle;

    /** \brief ݻλ      */
    struct am_rngbuf  rx_rngbuf;

    /** \brief ݻλ      */
    struct am_rngbuf  tx_rngbuf;

    /** \brief ʱ                 */
    uint32_t          timeout_ms;

    /** \brief ڽյȴ           */
    am_wait_t         rx_wait;
} am_zm516x_rngbuf_dev_t;

/** \brief UARTring bufferжģʽ׼Ͷ */
typedef struct am_zm516x_rngbuf_dev * am_zm516x_rngbuf_handle_t;

#define __ZM516X_CMD_BUF_LEN       100    /**< \brief 泤 */
#define __ZM516X_RSP_BUF_LEN       100    /**< \brief Ӧ𻺴泤 */

/** \brief ZM516X ģʵϢ
 * IO ֽڵbit0~bit3 ΪIO1~IO4bit0 ʾIO1bit1 ʾIO2bit2 ʾIO3bit3 ʾ IO4
 * ֽڷģIO ǰĵƽֵ1 Ϊߵƽ0 Ϊ͵ƽ
 *
 * ģʽ£IO1 IO2 ĹܹܽţΪIO 
 * AD0~AD3 ʾģ4 ·AD תֵΪ10 λAD תֵҪת Ϊѹֵ
 * ģADC ĲοѹΪ2.47V
 */
typedef struct am_zm516x_io_adc_info {
    uint8_t  io;     /**< \brief IO״̬  */
    uint16_t adc0;   /**< \brief AD0 */
    uint16_t adc1;   /**< \brief AD1 */
    uint16_t adc2;   /**< \brief AD2 */
    uint16_t adc3;   /**< \brief AD3  */
} *am_zm516x_io_adc_info_t;

typedef void (*pfn_upload_callback_t) (void                   *p_arg,
                                       am_zm516x_io_adc_info_t p_info);

/** \brief ZM516X Ϣ */
typedef struct am_zm516x_buf_info {
    uint8_t                 cmd_buf[__ZM516X_CMD_BUF_LEN]; /**< \brief  */
    uint8_t                 rsp_buf[__ZM516X_RSP_BUF_LEN]; /**< \brief Ӧ𻺳 */
    uint32_t                cur_rsp_len;       /**< \brief ǰӦ𳤶 */
    uint32_t                rsp_len;           /**< \brief Ӧ𳤶 */
} am_zm516x_buf_info_t;

/** \brief ZM516X ģʵϢ */
typedef struct am_zm516x_dev_info {
    int      rst_pin;       /**< \brief λ */
    int      ack_pin;       /**< \brief ACK  */
    uint16_t ack_timeout;   /**< \brief ACK ʱʱ */
    uint8_t *p_txbuf;       /**< \brief ͻ */
    uint32_t txbuf_size;    /**< \brief ͻСֽ */
    uint8_t *p_rxbuf;       /**< \brief ջ */
    uint32_t rxbuf_size;    /**< \brief ջСֽ */
} am_zm516x_dev_info_t;

/** \brief ZM516X ģ */
typedef struct am_zm516x_dev {
    am_zm516x_rngbuf_handle_t uart_handle;     /**< \brief UART (λ) */
    am_zm516x_rngbuf_dev_t    uart_rngbuf_dev; /**< \brief UART 豸(λ) */
    am_zm516x_buf_info_t      buf_info;        /**< \brief ZM516X Ϣ */

    am_wait_t                 ack_wait;        /**< \brief ACK ȴź */
    am_zm516x_dev_info_t     *p_devinfo;       /**< \brief 豸ʵϢ */
    uint8_t                   prtl_type[2];    /**< \brief Э */

    uint8_t                   is_rsp;          /**< \brief ǰrsp״̬ */
} am_zm516x_dev_t;

/** \brief ZM516X ģ׼Ͷ */
typedef struct am_zm516x_dev *am_zm516x_handle_t;

/** \brief ZM516X ģϢ */
typedef struct am_zm516x_cfg_info {
    char    dev_name[16];     /**< \brief 豸 */
    char    dev_pwd[16];      /**< \brief 豸 */
    uint8_t dev_mode;         /**< \brief 豸ͣ0: ն豸 1: ·豸 */
    uint8_t chan;             /**< \brief ͨ */
    uint8_t panid[2];         /**< \brief  ID (PanID) */
    uint8_t my_addr[2];       /**< \brief ַ */
    uint8_t my_mac[8];        /**< \brief ַ(MAC) */
    uint8_t dst_addr[2];      /**< \brief Ŀַ */
    uint8_t dst_mac[8];       /**< \brief Ŀַ() */
    uint8_t reserve;          /**< \brief  */

    /**
     * \brief 书ʣֵΪ 0~3ֱӦ书ʣ
     *        -32dBm-20.5dBm-9dBm+2.5dBm
     */
    uint8_t power_level;
    uint8_t retry_num;        /**< \brief Դ */
    uint8_t tran_timeout;     /**< \brief ʱ(10ms) */

    /**
     * \brief ڲʣֵΪ 1~7ֱӦʣ
     *        240048009600192003840057600115200
     */
    uint8_t serial_rate;
    uint8_t serial_data;      /**< \brief λֵΪ 5~8 */
    uint8_t serial_stop;      /**< \brief ֹͣλֵΪ 1~2 */
    uint8_t serial_parity;    /**< \brief Уλ 0: У 1: У 2: żУ */
    uint8_t send_mode;        /**< \brief ģʽ 0:   1: 㲥 */
} am_zm516x_cfg_info_t;

/** \brief ZM516X ģĻϢ(ͨ D4 ) */
typedef struct am_zm516x_base_info {
    uint8_t dev_mode[2];      /**< \brief 豸 */
    uint8_t chan;             /**< \brief ͨ */
    uint8_t data_rate;        /**< \brief RFͨѶ  */
    uint8_t panid[2];         /**< \brief  ID (PanID) */
    uint8_t addr[2];          /**< \brief ַ */
    uint8_t state;            /**< \brief ״̬ */
} am_zm516x_base_info_t;

/** \brief ZM516X ԶģϢ(ͨȡԶϢ D5 ) */
typedef struct am_zm516x_remote_info {
    am_zm516x_cfg_info_t cfg_info;       /**< \brief Ϣ */
    uint8_t              state;          /**< \brief ״̬ */
    uint8_t              dev_mode[2];    /**< \brief 豸 */
    uint8_t              version[2];     /**< \brief ̼汾 */
} am_zm516x_remote_info_t;

/**
 * \name ZM516X ģ AD ͨ
 * @{
 */
#define AM_ZM516X_AD_CHANNEL0         0
#define AM_ZM516X_AD_CHANNEL1         1
#define AM_ZM516X_AD_CHANNEL2         2
#define AM_ZM516X_AD_CHANNEL3         3
/** @} */

/** \brief ZM516X ģͨѶģʽ */
typedef enum am_zm516x_comm_mode {
    AM_ZM516X_COMM_UNICAST = 0,    /**< \brief  */
    AM_ZM516X_COMM_BROADCAST,      /**< \brief 㲥 */
} am_zm516x_comm_mode_t;

/** \brief ZM516X ģ MAC ַ */
typedef struct am_zm516x_mac_addr {
    uint8_t mac[8];                /**< \brief MAC ַ */
}am_zm516x_mac_addr_t;

/** \brief ZM516X ӻģϢ(ͨѯģ洢ĴӻϢ E7 ) */
typedef struct am_zm516x_slave_info {
    uint8_t mac[8];                /**< \brief MAC ַ */
    uint8_t addr[2];               /**< \brief ַ */
}am_zm516x_slave_info_t;

/**
 * \brief ZM516X ģĵַ
 *
 * ַΪ 2 ֽڵַ 8 ֽڵ MAC ַʹ addr_size
 * ֵַַ
 */
typedef struct am_zm516x_addr {
    uint8_t *p_addr;               /**< \brief ַ */
    uint8_t  addr_size;            /**< \brief ַȣֽ */
}am_zm516x_addr_t;

/**
 * \brief ZM516X ģ PWM Ϣ
 */
typedef struct am_zm516x_pwm {

    /**
     * \brief PWM ƵʣĸֽΪһ 32 λСģʽλΪ Hzͨ
     *        124 ƵʷΧ趨 62Hz~800000Hzͨ 3 ƵʷΧ趨
     *         62Hz~60000Hz PWM źŵƵΪ 0 ʱʾ PWM ź
     */
    uint8_t freq[4];

    /**
     * \brief PWM ռձȣռձȵֵ趨 1~99ʾռձȵİٷֱ
     */
    uint8_t duty_cycle;
}am_zm516x_pwm_t;

/**
 * \brief  ZM516X ģ鷢 nbytes ֽ
 *
 * ӿ ZM516X ģ鷢 nbytes ֽݡ豸ڲ岻´
 * ʣݲᱻд룬ֵΪѾ͵ݸ
 *
 * \param[in] handle ZM516X ģ
 * \param[in] p_buf  Ҫдģݻָ
 * \param[in] nbytes Ҫдֽڸ
 *
 * \return ɹ͵ݸС 0 Ϊʧܣʧԭɲ鿴 errno
 *
 * \attention ÷ͺΪ첽ZM516X Ǹݴַ֡ʱ
 *            жĽڵ am_zm516x_send ͺҪ
 *            ŵ ZM516X Ҫʱȴͻȫ͵ʱ
 *             ZM516X ֡ʱ󣬲ܵ ZM516X  ZM516X
 *            ᵱݷ ZM516X ϣʱȴʱ佨ǣ
 *            T(s) = (1 / ) * 10 * ֽ + 0.05
 */
am_err_t am_zm516x_send (am_zm516x_handle_t handle,
                         const void        *p_buf,
                         size_t             nbytes);

/**
 * \brief  ZigBee ģ鷢 nbytes ֽݣȴ ACK
 *
 * ӿ ZM516X ģ鷢 nbytes ֽݡ豸ڲ岻´
 * ʣݲᱻд룬ֵΪѾ͵ݸ
 *
 * \param[in] handle ZM516X ģ
 * \param[in] p_buf  Ҫдģݻָ
 * \param[in] nbytes Ҫдֽڸ
 *
 * \retval  > 0          ɹ͵ݸ
 * \retval -AM_ENOTSUP   û ACK 
 * \retval -AM_ETIMEDOUT Ӧ
 *
 * \attention 1. ÷ͺΪͬݺ󽫵ȴ ACKյ ACK 
 *               ʱʱŻ᷵أ
 *            2. ģʽΪ㲥ģʽʱ ACK
 */
am_err_t am_zm516x_send_with_ack (am_zm516x_handle_t handle,
                                  const void        *p_buf,
                                  size_t             nbytes);

/**
 * \brief  ZM516X ģ maxbytes ֽ
 *
 * - ӿڴ ZM516X ģ maxbytes ֽ
 * - 豸ɽݸΪ 0
 *
 * \param[in]  handle   ZM516X ģ
 * \param[out] p_buf    ŶȡݵĻָ
 * \param[in]  maxbytes Ҫȡֽڸ
 *
 * \return ɹյݸС 0 Ϊʧܣʧԭɲ鿴 errno
 */
am_err_t am_zm516x_receive (am_zm516x_handle_t handle,
                            void              *p_buf,
                            size_t             maxbytes);

/**
 * \brief ȡ ZM516X ģϢD1
 *
 * \param[in]  handle ZM516X ģ
 * \param[out] p_info ȡ ZM516X ģϢ
 *
 * \retval  AM_OK    ȡɹ
 * \retval -AM_EPERM ȡʧ
 */
am_err_t am_zm516x_cfg_info_get (am_zm516x_handle_t    handle,
                                 am_zm516x_cfg_info_t *p_info);

/**
 * \brief  ZigBee ģͨţD2
 *
 * ģɹ 16 ͨϣģزƵʲͬͬͨģ˴˼
 * ͨͨϿʵϻεЧ
 *
 * \param[in] handle ZM516X ģ
 * \param[in] chan   ͨ 11~26
 *
 * \retval  AM_OK     óɹ
 * \retval -AM_EPERM  ʧ
 * \retval -AM_EINVAL Ч
 */
am_err_t am_zm516x_cfg_channel_set (am_zm516x_handle_t handle, uint8_t chan);

/**
 * \brief ģ飨D4
 *
 * ģյ󣬻ڵͬͨģ鷢㲥б
 * ˾̼ ZigBee ģ鶼Ӧ˹㲥ӦݻὫԼػϢص
 * Ŀڵ㡣ǰҪͨŲӦͨģ顣
 * ˺ΪͬӿڣǿƵȴtimeoutʱԽӦ
 *
 * \param[in]  handle      ZM516X ģ
 * \param[in]  buf_size    p_base_info ָĻСλΪ
 *                         sizeof(am_zm516x_base_info_t)
 * \param[out] p_base_info 洢յĴӻϢĻ
 * \param[out] p_get_size  ʵģ
 * \param[in]  timeout     ʱʱ䣬ʱӦ𽫱
 *
 * \retval  AM_OK     ȡɹ
 * \retval -AM_EPERM  ȡʧ
 * \retval -AM_EINVAL Ч
 */
am_err_t am_zm516x_discover (am_zm516x_handle_t     handle,
                             uint8_t                buf_size,
                             am_zm516x_base_info_t *p_base_info,
                             uint8_t               *p_get_size,
                             uint32_t               timeout);

/**
 * \brief ȡԶϢD5
 *
 * \param[in]  handle        ZM516X ģ
 * \param[in]  p_zb_addr     Ŀַ
 * \param[out] p_remote_info  ZM516X ģϢ
 *
 * \retval  AM_OK     ȡɹ
 * \retval -AM_EPERM  ȡʧ
 * \retval -AM_EINVAL Ч
 * \retval -AM_EFAULT ַ
 *
 * \attention λ58λΪ 567 λ򲻿ԻȡϢ
 */
am_err_t am_zm516x_remote_info_get (am_zm516x_handle_t       handle,
                                    am_zm516x_addr_t        *p_zb_addr,
                                    am_zm516x_remote_info_t *p_remote_info);

/**
 * \brief ޸ ZM516X ģϢD6óɹ踴λ
 *
 * \param[in] handle ZM516X ģ
 * \param[in] p_info ZM516X ģϢ
 *
 * \retval  AM_OK    ޸ĳɹ
 * \retval -AM_EPERM ޸ʧ
 *
 * \attention AMetal ƽ̨ ZM516X ģĴڲֲֻ֧ãλ̶Ϊ
 *            8 λУΪУ飬ֹͣλΪ 1 λڸģλУλֹͣ
 *            λʱҪע⣬СĸˣҪģָĬϲܲ
 */
am_err_t am_zm516x_cfg_info_set (am_zm516x_handle_t    handle,
                                 am_zm516x_cfg_info_t *p_info);

/**
 * \brief ʹ ZM516X ģ鸴λD9
 *
 * λ֡Ӧ
 *
 * \param[in] handle ZM516X ģ
 *
 * \return 
 */
void am_zm516x_reset (am_zm516x_handle_t handle);

/**
 * \brief ָ ZM516X ģãDAóɹ踴λ
 *
 * \param[in] handle ZM516X ģ
 *
 * \retval  AM_OK    óɹ
 * \retval -AM_EPERM ʧ
 */
am_err_t am_zm516x_default_set (am_zm516x_handle_t handle);

/**
 * \brief  ZigBee ģյݰͷǷʾԴַDC
 *        óɹ踴λ
 *
 * \param[in] handle    ZM516X ģ
 * \param[in] p_zb_addr Ŀַ
 * \param[in] flag      ͷǷʾԴַ AM_TRUE: ʾAM_FALSE: ʾ
 *
 * \retval  AM_OK     óɹ
 * \retval -AM_EPERM  ʧ
 * \retval -AM_EINVAL Ч
 * \retval -AM_EFAULT ַ
 */
am_err_t am_zm516x_cfg_display_head_set (am_zm516x_handle_t handle,
                                         am_zm516x_addr_t  *p_zb_addr,
                                         am_bool_t          flag);

/**
 * \brief ַָ ZigBee ģ GPIO E1
 *
 * \param[in] handle    ZM516X ģ
 * \param[in] p_zb_addr Ŀַ
 * \param[in] dir       ܽŷ򣬸ֽڸλ 1 Ϊ0 Ϊ룻
 *                      bit0 Ӧ GPIO1bit3 Ӧ GPIO4
 *
 * \retval  AM_OK     óɹ
 * \retval -AM_EPERM  ʧ
 * \retval -AM_EINVAL Ч
 * \retval -AM_EFAULT ַ
 */
am_err_t am_zm516x_cfg_gpio_dir_set (am_zm516x_handle_t handle,
                                     am_zm516x_addr_t  *p_zb_addr,
                                     uint8_t            dir);

/**
 * \brief ȡַָ ZigBee ģ GPIO E1
 *
 * \param[in]  handle    ZM516X ģ
 * \param[in]  p_zb_addr Ŀַ
 * \param[out] p_dir     ȡĹܽŷ򣬸ֽڸλ 1 Ϊ0 Ϊ룻
 *                       bit0 Ӧ GPIO1bit3 Ӧ GPIO4
 *
 * \retval  AM_OK     ȡɹ
 * \retval -AM_EPERM  ȡʧ
 * \retval -AM_EINVAL Ч
 * \retval -AM_EFAULT ַ
 */
am_err_t am_zm516x_cfg_gpio_dir_get (am_zm516x_handle_t handle,
                                     am_zm516x_addr_t  *p_zb_addr,
                                     uint8_t           *p_dir);

/**
 * \brief IO/AD ɼãE2óɹ踴λ
 *
 * AW516x ϵ ZigBee ģ 4 · IO  4 · ADIO/AD ɼ趨ģ
 * IO ϴ IO  AD ״̬õʱ䶨ʱϴ IO  AD ״̬
 *
 * \param[in] handle     ZM516X ģ
 * \param[in] p_zb_addr  Ŀַ
 * \param[in] dir        IO أbit0~bit3 Ӧ IO1~IO4ЩλΪ 1 ʱ
 *                       ʾشΪ 0 ʱʾ½ش
 * \param[in] period     ϴڣλΪ 10msõֵΪ 65535
 *                       õΪ 655350ms
 * \param[in] is_dormant ǷߣָʾģǷ״ֽ̬Ϊ 1 ʾģ鴦
 *                       ״̬IO õ״̬仯ʱģ飬Ŀ
 *                       ڵַ IO/AD ɼ֡ڴ 0ģ˿ IO
 *                       ⣬ڶʱʱѣȻĿڵַ
 *                       IO/AD ɼ֡ IO/AD ɼ֡ģ״̬
 *                       ֽΪ 0 ʾģ鲻
 *
 * \retval  AM_OK     óɹ
 * \retval -AM_EPERM  ʧ
 * \retval -AM_EINVAL Ч
 * \retval -AM_EFAULT ַ
 */
am_err_t am_zm516x_cfg_io_adc_upload_set (am_zm516x_handle_t handle,
                                          am_zm516x_addr_t  *p_zb_addr,
                                          uint8_t            dir,
                                          uint16_t           period,
                                          am_bool_t          is_dormant);

/**
 * \brief IO/AD ɼûȡE2
 * \param[in] handle        ZM516X ģ
 * \param[in] p_zb_addr     Ŀַ
 * \param[out] p_dir        ȡ IO ãҪȡ NULL 
 *                          ɡbit0~bit3 Ӧ IO1~IO4ЩλΪ 1 ʱʾ
 *                          Ϊ 0 ʱʾ½ش
 * \param[out] p_period     ȡϴãλΪ 10msҪȡ
 *                           NULL 
 * \param[out] p_is_dormant ȡǷãҪȡ NULL ɡ
 *                          ָʾģǷ״ֽ̬Ϊ 1 ʾģ鴦״
 *                          ̬IO õ״̬仯ʱģ飬Ŀڵ
 *                          ַ IO/AD ɼ֡ڴ 0ģ˿ IO
 *                          ⣬ڶʱʱѣȻĿڵַ
 *                          IO/AD ɼ֡ IO/AD ɼ֡ģ״
 *                          ֽ̬Ϊ 0 ʾģ鲻
 *
 * \retval  AM_OK     ȡɹ
 * \retval -AM_EPERM  ȡʧ
 * \retval -AM_EINVAL Ч
 * \retval -AM_EFAULT ַ
 */
am_err_t am_zm516x_cfg_io_adc_upload_get (am_zm516x_handle_t handle,
                                          am_zm516x_addr_t  *p_zb_addr,
                                          uint8_t           *p_dir,
                                          uint16_t          *p_period,
                                          uint8_t           *p_is_dormant);

/**
 * \brief ַָ ZigBee ģ GPIO ֵE3
 *
 * ڵøǰҪȵ I/O Ӧ IO Ϊ
 *
 * \param[in] handle    ZM516X ģ
 * \param[in] p_zb_addr Ҫõ ZM516X ģĿַ
 * \param[in] value     GPIO ֵIO ֽڵ bit0~bit3 Ӧ IO1~IO4ӦλΪ
 *                      1ʾ I/O ߵƽӦλΪ 0ʾ I/O ͵ƽ
 *
 * \retval  AM_OK     óɹ
 * \retval -AM_EPERM  ʧ
 * \retval -AM_EINVAL Ч
 * \retval -AM_EFAULT ַ
 */
am_err_t am_zm516x_cfg_gpio_set (am_zm516x_handle_t handle,
                                 am_zm516x_addr_t  *p_zb_addr,
                                 uint8_t            value);


/**
 * \brief ȡַָ ZigBee ģ GPIO ֵE3
 *
 * ڵøǰҪȵ I/O Ӧ IO Ϊ
 *
 * \param[in]  handle    ZM516X ģ
 * \param[in]  p_zb_addr Ҫõ ZM516X ģĿַ
 * \param[out] p_value   ȡ GPIO ֵIO ֽڵ bit0~bit3 Ӧ IO1~IO4
 *                       ӦλΪ 1ʾ I/O ߵƽӦλΪ 0ʾ I/O
 *                       ͵ƽ
 *
 * \retval  AM_OK     ȡɹ
 * \retval -AM_EPERM  ȡʧ
 * \retval -AM_EINVAL Ч
 * \retval -AM_EFAULT ַ
 */
am_err_t am_zm516x_cfg_gpio_get (am_zm516x_handle_t handle,
                                 am_zm516x_addr_t  *p_zb_addr,
                                 uint8_t           *p_value);

/**
 * \brief ַָ ZigBee ģ PWM ֵE4
 *
 * \param[in] handle    ZM516X ģ
 * \param[in] p_zb_addr Ҫõ ZM516X ģĿַ
 * \param[in] p_pwm     PWM ָ룬ָָڴΪ sizeof(am_zm516x_pwm_t)
 *                       4 Сܴ洢 4  PWM ͨĲ
 *
 * \retval  AM_OK     óɹ
 * \retval -AM_EPERM  ʧ
 * \retval -AM_EINVAL Ч
 * \retval -AM_EFAULT ַ
 */
am_err_t am_zm516x_cfg_pwm_set (am_zm516x_handle_t handle,
                                am_zm516x_addr_t  *p_zb_addr,
                                am_zm516x_pwm_t   *p_pwm);


/**
 * \brief ȡַָ ZigBee ģ PWM ֵãE4
 *
 * \param[in]  handle    ZM516X ģ
 * \param[in]  p_zb_addr Ҫõ ZM516X ģĿַ
 * \param[out] p_pwm     PWM ָ룬ָָڴΪ sizeof(am_zm516x_pwm_t)
 *                        4 Сܴ洢 4  PWM ͨĲ
 *
 * \retval  AM_OK     ȡɹ
 * \retval -AM_EPERM  ȡʧ
 * \retval -AM_EINVAL Ч
 * \retval -AM_EFAULT ַ
 */
am_err_t am_zm516x_cfg_pwm_get (am_zm516x_handle_t handle,
                                am_zm516x_addr_t  *p_zb_addr,
                                am_zm516x_pwm_t   *p_pwm);

/**
 * \brief ܣE5óɹ踴λ
 *
 * Ϊһӵ˽ṹӻźŲɴʱɼм̽ڵ
 * źŴݡģģʽ£ģԶѡΧûбʹõ PANID ͨ
 * γһ磬ԶһΨһıַӻģ飬ӻģʹ
 * ʱʹܺͲҪκεòӻģڼܸ
 * ͨѶһģ 200 ӻģ顣ϸοĵ
 * AW516x_ZigBee.pdf
 *
 * \param[in] handle ZM516X ģ
 * \param[in] state  ʹܣ0: ʧ 1: ʹ
 * \param[in] type   ڵͣ0: ڵ 1: ӻڵ
 *
 * \retval  AM_OK     óɹ
 * \retval -AM_EPERM  ʧ
 * \retval -AM_EINVAL Ч
 */
am_err_t am_zm516x_cfg_network_set (am_zm516x_handle_t handle,
                                    am_bool_t          state,
                                    uint8_t            type);

/**
 * \brief ӻ磨E6
 *
 * 翪ĴʱΪʼܴӻ
 * 󣬵ʱٽܴӻ󣬴ʱ
 * Ĺ״̬
 *
 * \param[in] handle ZM516X ģ
 * \param[in] time   翪Ĵʱ䣬λΪ
 *
 * \retval  AM_OK     óɹ
 * \retval -AM_EPERM  ʧ
 * \retval -AM_EINVAL Ч
 */
am_err_t am_zm516x_cfg_join_enable (am_zm516x_handle_t handle,
                                    uint16_t           time);

/**
 * \brief ѯģ洢ĴӻϢE7
 *
 * \param[in]  handle       ZM516X ģ
 * \param[in]  buf_size     p_slave_info ָĻСλΪ
 *                          sizeof(am_zm516x_slave_info_t)
 * \param[out] p_slave_info 洢յĴӻϢĻ
 * \param[out] p_get_size   ʵʻȡĴӻϢ
 *
 * \retval  AM_OK     ѯɹ
 * \retval -AM_EPERM  ѯʧ
 * \retval -AM_EINVAL Ч
 */
am_err_t am_zm516x_cfg_slave_get (am_zm516x_handle_t      handle,
                                  uint8_t                 buf_size,
                                  am_zm516x_slave_info_t *p_slave_info,
                                  uint8_t                *p_get_size);

/**
 * \brief ѯӻ״̬E8
 *
 * \param[in]  handle           ZM516X ģ
 * \param[out] p_state          ȡ״̬0: ڵ 1: ӻڵ㣬粻Ҫ
 *                              ȡò NULL 
 * \param[out] p_response_state ȡӦ״̬ģΪʱ
 *                                  0: 
 *                                  1: ڼ
 *                                  2: ӻ
 *                              ģΪӻʱ
 *                                  0: ӻڼ
 *                                  1: ӻѼ
 *                                  2: ӻ˳
 *                              粻Ҫȡò NULL 
 *
 * \retval  AM_OK     ѯɹ
 * \retval -AM_EPERM  ѯʧ
 * \retval -AM_EINVAL Ч
 */
am_err_t am_zm516x_cfg_state_get (am_zm516x_handle_t handle,
                                  uint8_t           *p_state,
                                  uint8_t           *p_response_state);

/**
 * \brief  ZM516X ģͨţʱD1
 *
 * \param[in] handle ZM516X ģ
 * \param[in] chan   ͨ
 *
 * \retval  AM_OK    óɹ
 * \retval -AM_EPERM ʧ
 */
am_err_t am_zm516x_channel_set (am_zm516x_handle_t handle, uint8_t chan);

/**
 * \brief  ZM516X ģĿַʱD2
 *
 * \param[in] handle    ZM516X ģ
 * \param[in] p_zb_addr Ŀַ
 *
 * \retval  AM_OK    óɹ
 * \retval -AM_EPERM ʧ
 */
am_err_t am_zm516x_dest_addr_set (am_zm516x_handle_t handle,
                                  am_zm516x_addr_t  *p_zb_addr);

/**
 * \brief  ZM516X ģյݰͷǷʾԴַʱD3
 *
 * \param[in] handle ZM516X ģ
 * \param[in] flag   ͷǷʾԴַ AM_TRUE: ʾAM_FALSE: ʾ
 *
 * \retval  AM_OK    óɹ
 * \retval -AM_EPERM ʧ
 */
am_err_t am_zm516x_display_head_set (am_zm516x_handle_t handle, am_bool_t flag);

/**
 * \brief ַָ ZM516X ģ GPIO ʱD4
 *
 * \param[in] handle    ZM516X ģ
 * \param[in] p_zb_addr Ҫõ ZM516X ģĿַ
 * \param[in] dir       ܽŷ򣬸ֽڸλ 1 Ϊ0 Ϊ룻
 *                      bit0 Ӧ GPIO0bit7 Ӧ GPIO7
 *
 * \retval  AM_OK    óɹ
 * \retval -AM_EPERM ʧ
 */
am_err_t am_zm516x_gpio_dir (am_zm516x_handle_t handle,
                             am_zm516x_addr_t  *p_zb_addr,
                             uint8_t            dir);

/**
 * \brief ȡַָ ZM516X ģ GPIO ֵʱD5
 *
 * \param[in]  handle    ZM516X ģ
 * \param[in]  p_zb_addr Ҫȡ ZM516X ģĿַ
 * \param[out] p_value   ȡĹֵֽܽڸλ 1 Ϊߵƽ0 Ϊ͵ƽ
 *                       bit0 Ӧ GPIO0bit7 Ӧ GPIO7
 *
 * \retval  AM_OK    ȡɹ
 * \retval -AM_EPERM ȡʧ
 */
am_err_t am_zm516x_gpio_get (am_zm516x_handle_t handle,
                             am_zm516x_addr_t  *p_zb_addr,
                             uint8_t           *p_value);

/**
 * \brief ַָ ZM516X ģ GPIO ֵʱD6
 *
 * \param[in] handle    ZM516X ģ
 * \param[in] p_zb_addr Ҫõ ZM516X ģĿַ
 * \param[in] value     ֵֽܽڸλ 1 Ϊߵƽ0 Ϊ͵ƽ
 *                      bit0 Ӧ GPIO0bit7 Ӧ GPIO7
 *
 * \retval  AM_OK    óɹ
 * \retval -AM_EPERM ʧ
 */
am_err_t am_zm516x_gpio_set (am_zm516x_handle_t handle,
                             am_zm516x_addr_t  *p_zb_addr,
                             uint8_t            value);

/**
 * \brief ȡַָ ZM516X ģ AD תֵʱD7
 *
 * ȡ AD ֵΪ AD תֵҪο ZM516X ģûֲ
 *  ZM516X ģ AD λοѹתΪѹֵ
 *
 * \param[in]  handle    ZM516X ģ
 * \param[in]  p_zb_addr Ҫȡ ZM516X ģĿַ
 * \param[in]  chan      AD ͨ
 * \param[out] p_value   AD תֵ
 *
 * \retval  AM_OK    ȡɹ
 * \retval -AM_EPERM ȡʧ
 */
am_err_t am_zm516x_ad_get (am_zm516x_handle_t handle,
                           am_zm516x_addr_t  *p_zb_addr,
                           uint8_t            chan,
                           uint16_t          *p_value);

/**
 * \brief  ZM516X ģ˯ģʽʱD8
 *
 * \param[in] handle ZM516X ģ
 *
 * \return 
 */
void am_zm516x_enter_sleep (am_zm516x_handle_t handle);

/**
 * \brief ZM516XģͨѶģʽʱD9
 *
 * \param[in] handle ZM516Xģ
 * \param[in] mode   ģͨѶģʽ
 *
 * \retval  AM_OK    óɹ
 * \retval -AM_EPERM ʧ
 */
am_err_t am_zm516x_mode_set (am_zm516x_handle_t    handle,
                             am_zm516x_comm_mode_t mode);

/**
 * \brief ȡַָ ZM516X ģźǿȣʱDA
 *
 * \param[in]  handle    ZM516X ģ
 * \param[in]  p_zb_addr Ҫȡı ZM516X ģĿַ֮źǿ
 * \param[out] p_signal  źǿ
 *
 * \retval  AM_OK    ȡɹ
 * \retval -AM_EPERM ȡʧ
 */
am_err_t am_zm516x_sigal_get (am_zm516x_handle_t handle,
                              am_zm516x_addr_t  *p_zb_addr,
                              uint8_t           *p_signal);

/**
 * \brief ʼ ZM516X ģ
 *
 * \param[in] p_dev       ZM516X ģ豸ṹ
 * \param[in] p_info      ZM516X ģʵϢ
 * \param[in] uart_handle  ZM516X ģĴھ
 *
 * \return ZM516X ׼Ϊ NULLʼʧ
 */
am_zm516x_handle_t am_zm516x_init (am_zm516x_dev_t            *p_dev,
                                   const am_zm516x_dev_info_t *p_info,
                                   am_uart_handle_t            uart_handle);

/**
 * \brief ʼ ZM516X ģ
 *
 * \param[in] handle ͨam_zm516x_initȡZM516X
 *
 * \retval  AM_OK     ʼɹ
 * \retval -AM_EINVAL Ч
 */
am_err_t am_zm516x_deinit (am_zm516x_handle_t handle);

/** @} am_if_zm516x */

#ifdef __cplusplus
}
#endif

#endif /* __AM_ZM516X_H */

/* end of file */
