/*******************************************************************************
*                                 AMetal
*                       ----------------------------
*                       innovating embedded platform
*
* Copyright (c) 2001-2018 Guangzhou ZHIYUAN Electronics Co., Ltd.
* All rights reserved.
*
* Contact information:
* web site:    http://www.zlg.cn/
*******************************************************************************/

/**
 * \file
 * \brief RX8025SAʵ
 *
 * \internal
 * \par Modification history
 * - 1.00 17-08-07  vir, first implementation.
 * \endinternal
 */
#include "ametal.h"
#include "am_rx8025sa.h"
#include "am_gpio.h"
/*****************************************************************************
 * 궨
 ****************************************************************************/
 
 /** \brief RX8025SA豸ַ  (λ)*/
#define __RX8025SA_ADDR               0x32

#define __RX8025SA_12H_FORMAT  0   /** \brief 12Сʱ */
#define __RX8025SA_24H_FORMAT  1   /** \brief 24Сʱ */
/**
 * \name RX8025SA Ĵַ + ģʽ  ĬΪ0
 *
 * \note λΪĴַ
 *       λΪģʽ        0h ׼readģʽ
 *                      4h ˳readģʽ Ƽ
 */
 
#define __RX8025SA_REG_SECONDS          0x00    /**< \brief Ĵַ + ׼ģʽ */
#define __RX8025SA_REG_MINUTES          0x10    /**< \brief ּĴַ + ׼ģʽ */
#define __RX8025SA_REG_HOURS            0x20    /**< \brief ʱĴַ + ׼ģʽ */
#define __RX8025SA_REG_WEEKDAYS         0x30    /**< \brief ڼĴַ + ׼ģʽ */
#define __RX8025SA_REG_DAYS             0x40    /**< \brief ռĴַ + ׼ģʽ */
#define __RX8025SA_REG_MONTHS           0x50    /**< \brief ¼Ĵַ + ׼ģʽ */
#define __RX8025SA_REG_YEAERS           0x60    /**< \brief Ĵַ + ׼ģʽ */

#define __RX8025SA_REG_DIGITAL_OFFSET   0x70    /**< \brief ʱƫƼĴַ + ׼ģʽ */

#define __RX8025SA_REG_MINUTES_ALARM_W  0x80    /**< \brief WӼĴַ + ׼ģʽ */
#define __RX8025SA_REG_HOURS_ALARM_W    0x90    /**< \brief WСʱĴַ  + ׼ģʽ*/
#define __RX8025SA_REG_WEEKDAYS_ALARM_W 0xa0    /**< \brief WڼĴַ + ׼ģʽ*/

#define __RX8025SA_REG_MINUTES_ALARM_D  0xb0    /**< \brief DӼĴַ + ׼ģʽ */
#define __RX8025SA_REG_HOURS_ALARM_D    0xc0    /**< \brief DСʱĴַ  + ׼ģʽ*/
 /* 0x0d is reserved */
#define __RX8025SA_REG_CONTROL1         0xe0    /**< \brief ״̬Ĵ1ַ + ׼ģʽ */
#define __RX8025SA_REG_CONTROL2         0xf0    /**< \brief ״̬Ĵ2ַ + ׼ģʽ */

/**
 * \name RX8025SA Ĵλ
 * \anchor grp_rx8025sa_bitfield_descriptions
 * @{
 */


#define __RX8025SA_C1_ALARM_W_EN   0x80   /**< \brief ʹALARM_W */
#define __RX8025SA_C1_ALARM_D_EN   0x40   /**< \brief ʹALARM_D */

#define __RX8025SA_C2_DAFG_ALARM_D        (uint8_t)(1<<0)    /**< \brief DAFGλ*/
#define __RX8025SA_C2_WAFG_ALARM_W        (uint8_t)(1<<1)    /**< \brief WAFGλ*/
#define __RX8025SA_C2_CTFG_INT            (uint8_t)(1<<2)    /**< \brief CTFGλ*/

#define __RX8025SA_C1_24H_FORMAT   0x20  /**< \brief 24Сʱ */

#define __RX8025SA_C2_VDSL_1_3_SET   0x80  /**< \brief ͵ѹ⹦ܵı׼ѹֵ趨Ϊ1.3v*/



/**
 * \name RX8025SA I2Cǰ״̬
 * \anchor grp_rx8025sa_i2c_operation
 * @{
 */

#define __RX8025SA_I2C_RD_STATUS        0x00    /**< \brief ȡ״̬ */
#define __RX8025SA_I2C_WR_STATUS        0x01    /**< \brief ״̬ */
#define __RX8025SA_I2C_DATA_PROCESSING  0x02    /**< \brief ݴ׶ */

/* RTC driver function implementation */
static int __rx8025sa_time_set (void *p_drv, am_tm_t *p_tm);
static int __rx8025sa_time_get (void *p_drv, am_tm_t *p_tm);

/**
 * \brief RTC driver functions definition
 */
static struct am_rtc_drv_funcs __g_rtc_drv_funcs = {
    __rx8025sa_time_set,
    __rx8025sa_time_get,
};

/*  */
static void __rx8025sa_intb_isr (void *p_arg);

/* ALARM_CLK driver function implementation */
static int __rx8025sa_alarm_clk_time_set (void *p_drv, am_alarm_clk_tm_t *p_tm);
static int __rx8025sa_alarm_clk_callback_set (void          *p_drv,
                                              am_pfnvoid_t   pfn_alarm_callback,
                                              void          *p_arg);
static int __rx8025sa_alarm_clk_on (void *p_drv);
static int __rx8025sa_alarm_clk_off (void *p_drv);

/**
 * \brief ALARM_CLK driver functions definition
 */
static struct am_alarm_clk_drv_funcs __g_alarm_clk_drv_funcs = {
    __rx8025sa_alarm_clk_time_set,
    __rx8025sa_alarm_clk_callback_set,
    __rx8025sa_alarm_clk_on,
    __rx8025sa_alarm_clk_off,
};
/**
 * \brief ƽÿ¶Ӧ
 */
static const uint8_t __mdays[2][12] = {
    /* ƽ */
    {31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31},
    /*  */
    {31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31}
};

/**
 * \brief жǷΪ
 *
 * \param[in] year : 
 *
 * \retval 1 : 
 * \retval 0 : ƽ
 */
static uint8_t __rx8025sa_leap_year_check (uint16_t year)
{
    return (((!(year % 4) && (year % 100)) || !(year % 400)) ? 1 : 0);
}

/**
 * \brief һеĵڼ
 *
 * \param[in] year  : 
 * \param[in] month : ·
 * \param[in] day   : 
 *
 * \retval 1 : 
 * \retval 0 : ƽ
 */
static uint8_t __rx8025sa_day_of_year (uint16_t year, uint8_t month, uint8_t day)
{
    uint8_t i;

    if (__rx8025sa_leap_year_check(year)) {
        for (i = 0; i < month; i++) {
            day += __mdays[1][i];
        }
    } else {
        for (i = 0; i < month; i++) {
            day += __mdays[0][i];
        }
    }

    return (day);
}
/**
 * \brief ʱϢЧ
 *
 * \param[in] p_tm : ָʱṹָ
 *
 * \retval AM_OK      : óɹ
 * \retval -AM_EINVAL : 
 */
int __rx8025sa_time_validator (am_tm_t *p_tm)
{
    if (p_tm->tm_year > 199 || p_tm->tm_year < 0 ||
        p_tm->tm_mon > 11   || p_tm->tm_mon < 0 ||
        p_tm->tm_mday >
        __mdays[__rx8025sa_leap_year_check(1900 + p_tm->tm_year)][p_tm->tm_mon] ||
        p_tm->tm_mday < 0 ||
        p_tm->tm_wday > 6 || p_tm->tm_wday < 0 ||
        p_tm->tm_hour > 23 || p_tm->tm_hour < 0 ||
        p_tm->tm_min > 59 || p_tm->tm_min < 0 ||
        p_tm->tm_sec > 59 || p_tm->tm_sec < 0 ) {
        return -AM_EINVAL;
    } else {
        return AM_OK;
    }
}
/**
 * \brief RTC׼-ʱ
 *
 * \param[in] p_drv : ָ豸ṹָ
 * \param[in] p_tm  : ָʱṹָ
 *
 * \retval AM_OK      : óɹ
 * \retval -AM_EINVAL : 
 */
static int __rx8025sa_time_set (void *p_drv, am_tm_t *p_tm)
{
    /* ÷ֵ */
    int ret = AM_OK;

    uint8_t temp;

    /* ݻ */
    uint8_t buf[7] = {0};

    /* I2C豸ָ */
    am_i2c_device_t *p_i2c_dev = NULL;

    /* Ч */
    if (NULL == p_drv || NULL == p_tm) {
        return -AM_EINVAL;
    }

    /* ֤ʱϢЧ */
    if ((AM_OK != __rx8025sa_time_validator(p_tm)) ||
        (p_tm->tm_hour > 23) || (p_tm->tm_hour < 0)) {
        return -AM_EINVAL;
    }

    /* 豸ṹлȡi2c豸ָ */
    p_i2c_dev = &(((am_rx8025sa_dev_t *)p_drv)->i2c_dev);

    /* ʱϢbuf */
    buf[__RX8025SA_REG_SECONDS >> 4 ] = AM_HEX_TO_BCD(p_tm->tm_sec);
    buf[__RX8025SA_REG_MINUTES >> 4 ] = AM_HEX_TO_BCD(p_tm->tm_min);

    if (((am_rx8025sa_dev_t *)p_drv)->time_format == __RX8025SA_24H_FORMAT){
        buf[__RX8025SA_REG_HOURS >> 4] = AM_HEX_TO_BCD(p_tm->tm_hour);         /* 24СʱƵСʱ */
    } else {
        temp = AM_HEX_TO_BCD((p_tm->tm_hour % 12));
        if (temp == 0) {
            temp = 12;
        }
        buf[__RX8025SA_REG_HOURS >> 4] = temp | ((p_tm->tm_hour / 12) << 5);   /* 12СʱƵСʱ */
    }

    buf[__RX8025SA_REG_WEEKDAYS >> 4] = AM_HEX_TO_BCD(p_tm-> tm_wday);
    buf[__RX8025SA_REG_DAYS >> 4]     = AM_HEX_TO_BCD(p_tm->tm_mday);
    buf[__RX8025SA_REG_MONTHS >> 4]   = AM_HEX_TO_BCD(p_tm->tm_mon + 1);
    buf[__RX8025SA_REG_YEAERS >> 4]   = AM_HEX_TO_BCD(p_tm->tm_year);

    /* I2C */
    ret = am_i2c_write(p_i2c_dev,
                       __RX8025SA_REG_SECONDS,
                       buf,
                       sizeof(buf));

    if (AM_OK != ret) {
        return -AM_EINVAL;
    }

    return AM_OK;

}

/**
 * \brief RTC׼-ʱȡ
 *
 * \param[in] p_drv : ָ豸ṹָ
 * \param[in] p_tm  : ָʱṹָ
 *
 * \retval AM_OK      : óɹ
 * \retval -AM_EINVAL : 
 */
static int __rx8025sa_time_get (void *p_drv, am_tm_t *p_tm)
{
    /* ÷ֵ */
    int ret = AM_OK;

    uint8_t  temp;

    /* ݻ */
    uint8_t buf[7] = {0};

    /* I2C豸ָ */
    am_i2c_device_t *p_i2c_dev = NULL;

    /* Ч */
    if (NULL == p_drv || NULL == p_tm) {
        return -AM_EINVAL;
    }

    /* 豸ṹлȡi2c豸ָ */
    p_i2c_dev = &(((am_rx8025sa_dev_t *)p_drv)->i2c_dev);

    /* ͨI2Cȡ */
    ret = am_i2c_read(p_i2c_dev,
                      __RX8025SA_REG_SECONDS,
                      buf,
                      sizeof(buf));

    if (AM_OK != ret) {
        return -AM_EINVAL;
    }

    /** bufлȡʱϢ */
    p_tm->tm_sec  = AM_BCD_TO_HEX(buf[__RX8025SA_REG_SECONDS >> 4] & 0x7F);
    p_tm->tm_min  = AM_BCD_TO_HEX(buf[__RX8025SA_REG_MINUTES >> 4] & 0x7F);

    if (((am_rx8025sa_dev_t *)p_drv)->time_format == __RX8025SA_24H_FORMAT){
        p_tm->tm_hour = AM_BCD_TO_HEX(buf[__RX8025SA_REG_HOURS >> 4] & 0x3F);        /* 24СʱƵСʱ */
    } else {
        temp = AM_BCD_TO_HEX(buf[__RX8025SA_REG_HOURS >> 4]& 0x1F);
        if (temp == 12) {
            temp =0;
        }
        p_tm->tm_hour = temp + ((buf[__RX8025SA_REG_HOURS >> 4] >> 5) & 0x01) * 12;  /* 12СʱƵСʱ */
    }

    p_tm->tm_wday = AM_BCD_TO_HEX(buf[__RX8025SA_REG_WEEKDAYS >> 4] & 0x07);
    p_tm->tm_mday = AM_BCD_TO_HEX(buf[__RX8025SA_REG_DAYS >> 4] & 0x3F );
    p_tm->tm_mon  = AM_BCD_TO_HEX(buf[__RX8025SA_REG_MONTHS >> 4] & 0x1F) - 1;
    p_tm->tm_year = AM_BCD_TO_HEX(buf[__RX8025SA_REG_YEAERS >> 4]);
    p_tm->tm_yday = __rx8025sa_day_of_year(p_tm->tm_year, p_tm->tm_mon, p_tm->tm_mday);
    return AM_OK;

}

/**
 * \brief ALARM_CLK ׼-ʱú
 *
 * \param[in] p_drv : ָ豸ṹָ
 * \param[in] p_tm  : ָʱṹָ
 *
 * \retval AM_OK      : óɹ
 * \retval -AM_EINVAL : 
 */
static int __rx8025sa_alarm_clk_time_set (void *p_drv, am_alarm_clk_tm_t *p_tm)
{
    /* ÷ֵ */
    int ret = AM_OK;

    /* ݻ */
    uint8_t buf[3] = {0};

    /* I2C豸ָ */
    am_i2c_device_t *p_i2c_dev = NULL;

    /* Ч */
    if (NULL == p_drv || NULL == p_tm) {
        return -AM_EINVAL;
    }

    /* ֤ʱϢЧ */
    if ((p_tm->min  > 59) || (p_tm->min  < 0) ||
        (p_tm->hour > 23) || (p_tm->hour < 0)) {
        return -AM_EINVAL;
    }

    /* 豸ṹлȡi2c豸ָ */
    p_i2c_dev = &(((am_rx8025sa_dev_t *)p_drv)->i2c_dev);

    /* ʱϢbuf */
    buf[0] = AM_HEX_TO_BCD(p_tm->min);
    buf[1] = AM_HEX_TO_BCD(p_tm->hour);
    /* תBCD */
    buf[2] = p_tm->wdays;

    /* I2C */
    ret = am_i2c_write(p_i2c_dev,
                       __RX8025SA_REG_MINUTES_ALARM_W,
                       buf,
                       sizeof(buf));

    if (AM_OK != ret) {
        return -AM_EINVAL;
    }

    return AM_OK;
}

/**
 * \brief ALARM_CLK ׼-ӻصú
 *
 * \param[in] p_drv         : ָ豸ṹָ
 * \param[in] pfn_callback  : ָصָ
 * \param[in] p_arg         : ָصָ
 *
 * \retval AM_OK  : óɹ
 * \retval              : 鿴
 */
static int __rx8025sa_alarm_clk_callback_set (void         *p_drv,
                                              am_pfnvoid_t  pfn_alarm_callback,
                                              void          *p_arg)
{
	/* ÷ֵ */
    int ret = AM_OK;

    /* RX8025SA 豸ָ */
    am_rx8025sa_handle_t  rx8025sa_handle = NULL;

    /* 豸Ϣָ */
   const am_rx8025sa_devinfo_t *p_devinfo = NULL;

    /* Ч */
    if (NULL == p_drv) {
        return -AM_EINVAL;
    }

    rx8025sa_handle =  (am_rx8025sa_handle_t)p_drv;
    p_devinfo      = rx8025sa_handle->p_devinfo;

    /* жϻص */
    ret = am_gpio_trigger_connect(p_devinfo->intb_pin,
                                  __rx8025sa_intb_isr,
                                  rx8025sa_handle);
    if (AM_OK != ret) {
        return ret;
    }

    /* Ϊ½ش */
    am_gpio_trigger_cfg(p_devinfo->intb_pin, AM_GPIO_TRIGGER_FALL);
    am_gpio_trigger_on(p_devinfo->intb_pin);

    /* жϻص */
    rx8025sa_handle->triginfo[2].pfn_callback = pfn_alarm_callback;
    rx8025sa_handle->triginfo[2].p_arg        = p_arg;

    /* еĻصΪ */
    if (rx8025sa_handle->triginfo[2].pfn_callback == NULL) {

        /* ɾжϻص */
        ret = am_gpio_trigger_disconnect(p_devinfo->intb_pin,
                                         __rx8025sa_intb_isr,
                                         rx8025sa_handle);

        if (AM_OK != ret) {
            return ret;
        }

        /* رŴ */
        am_gpio_trigger_off(p_devinfo->intb_pin);
    }

    return AM_OK;
}

/**
 * \brief ALARM_CLK ׼-Ӻ
 *
 * \param[in] p_drv : ָ豸ṹָ
 *
 * \retval AM_OK      : óɹ
 * \retval -AM_EINVAL : 
 */
static int __rx8025sa_alarm_clk_on (void *p_drv)
{
    /* ÷ֵ */
    int ret = AM_OK;

    /* ݻ */
    uint8_t buf[1]  = {0};

    /* I2C豸ָ */
    am_i2c_device_t *p_i2c_dev = NULL;

    /* ֤Ч */
    if (NULL == p_drv) {
        return -AM_EINVAL;
    }

    /* p_drvлȡi2c豸ָ */
    p_i2c_dev = &(((am_rx8025sa_dev_t *)p_drv)->i2c_dev);

    /* ȡƼĴ1 */
    ret = am_i2c_read(p_i2c_dev,
                      __RX8025SA_REG_CONTROL1,
                      buf,
                      1);

    if (AM_OK != ret) {
        return ret;
    }

    /* жʹ */
    buf[0] |= __RX8025SA_C1_ALARM_W_EN;

    /* жϢд״̬Ĵ1 */
    ret = am_i2c_write(p_i2c_dev,
                       __RX8025SA_REG_CONTROL1,
                       buf,
                       1);

    if (AM_OK != ret) {
        return ret;
    }
    return AM_OK;
}

/**
 * \brief ALARM_CLK ׼-رӺ
 *
 * \param[in] p_drv : ָ豸ṹָ
 *
 * \retval AM_OK      : óɹ
 * \retval -AM_EINVAL : 
 */
static int __rx8025sa_alarm_clk_off (void *p_drv)
{
    /* ÷ֵ */
    int ret = AM_OK;

    /* ݻ */
    uint8_t buf[1]  = {0};

    /* I2C豸ָ */
    am_i2c_device_t *p_i2c_dev = NULL;

    /* ֤Ч */
    if (NULL == p_drv) {
        return -AM_EINVAL;
    }

    /* p_drvлȡi2c豸ָ */
    p_i2c_dev = &(((am_rx8025sa_dev_t *)p_drv)->i2c_dev);

    /* ȡƼĴ */
    ret = am_i2c_read(p_i2c_dev,
                      __RX8025SA_REG_CONTROL1,
                      buf,
                      1);

    if (AM_OK != ret) {
        return ret;
    }

    /* رж */
    buf[0] &= ~__RX8025SA_C1_ALARM_W_EN;

    /* жϢд״̬Ĵ */
    ret = am_i2c_write(p_i2c_dev,
                       __RX8025SA_REG_CONTROL1,
                       buf,
                       1);

    if (AM_OK != ret) {
        return ret;
    }
    return AM_OK;
}

/**
 * \brief жϱ־ִûжϻص
 *
 * \param[in] p_dev     : ָRX8025SA豸ָ
 * \param[in] intr_flag : жϱ־
 *
 * \return 
 */
static void __rx8025sa_callback_exec (void    *p_drv,
                                      uint8_t  intr_flag)
{
    am_rx8025sa_dev_t *p_dev        = NULL;
    am_pfnvoid_t       pfn_callback = NULL;
    void              *p_arg        = NULL;

    /* Ч */
    if (0 == intr_flag || NULL == p_drv) {
        return;
    }

    p_dev = (am_rx8025sa_dev_t *)p_drv;

    /* жϣִжӦûжϻص */
    if (intr_flag & __RX8025SA_C2_CTFG_INT) {
        pfn_callback = p_dev->triginfo[0].pfn_callback;
        p_arg        = p_dev->triginfo[0].p_arg;
        if (NULL != pfn_callback) {
            pfn_callback(p_arg);
        }
    }

    /* ALARM_DжϣִжӦûжϻص */
    if (intr_flag & __RX8025SA_C2_DAFG_ALARM_D) {
        pfn_callback = p_dev->triginfo[1].pfn_callback;
        p_arg        = p_dev->triginfo[1].p_arg;
        if (NULL != pfn_callback) {
            pfn_callback(p_arg);
        }
    }

    /* ALARM_WжϣִжӦûжϻص */
    if (intr_flag & __RX8025SA_C2_WAFG_ALARM_W) {
        pfn_callback = p_dev->triginfo[2].pfn_callback;
        p_arg        = p_dev->triginfo[2].p_arg;
        if (NULL != pfn_callback) {
            pfn_callback(p_arg);
        }
    }
}
/**
 * \brief rx8025sa豸ʼ
 * \param[in] p_dev : ָrx8025sa豸ṹ
 * \return AM_OKʾʼɹ, ʧ
 */
static int __rx8025sa_init (am_rx8025sa_dev_t *p_dev)
{
    /* ÷ֵ */
    int ret = AM_OK;

    /* ݻ */
    uint8_t buf[1] = {0};

    /* I2C豸ָ */
    am_i2c_device_t *p_i2c_dev = NULL;

    /* Ч */
    if (NULL == p_dev) {
        return -AM_EINVAL;
    }

    /* 豸ṹлȡi2c豸ָ */
    p_i2c_dev = &(p_dev->i2c_dev);

    /* ״̬Ĵ2ʱ,жϱ־ */
    buf[0] = 0;

    /* I2C */
    ret = am_i2c_write(p_i2c_dev,
    		           __RX8025SA_REG_CONTROL1,
                       buf,
                       sizeof(buf));

    if (AM_OK != ret) {
        return ret;
    }

    return AM_OK;
}

/**
 * \brief жжȡݺĴ,жж
 *
 * \param[in,out] buf         : ݻ
 * \param[in]     p_intr_flag : жϱ־  μ \ref grp_rx8025sa_intr_flag
 *
 * \return 
 *
 * \note buf Ӧ CONTROL2Ĵ
 */
static void __rx8025sa_read_write_data_processing (void    *p_arg,
                                                   uint8_t *p_intr_flag)
{
    am_rx8025sa_dev_t *p_dev = NULL;

    if (NULL == p_arg) {
        return;
    }

    p_dev = (am_rx8025sa_dev_t *)p_arg;

    *p_intr_flag = 0;

    /* жǷΪDAFG־λ */
    if (p_dev->buf[0] & __RX8025SA_C2_DAFG_ALARM_D) {

        /* ¼ALARM_Dжϱ־ */
        *p_intr_flag |= __RX8025SA_C2_DAFG_ALARM_D;

        /* DAFG־ */
        p_dev->buf[0] &= ~__RX8025SA_C2_DAFG_ALARM_D;
    }

    /* жǷΪCTFG־λ */
    if (p_dev->buf[0] & __RX8025SA_C2_CTFG_INT) {
          /* ¼жϱ־,*/
        *p_intr_flag |= __RX8025SA_C2_CTFG_INT;

          /* CTFG־ */
        p_dev->buf[0] &= ~__RX8025SA_C2_CTFG_INT;
        }

    /* жǷΪWAFG־λ */
    if (p_dev->buf[0] & __RX8025SA_C2_WAFG_ALARM_W) {
        /* ¼жϱ־, */
        *p_intr_flag |= __RX8025SA_C2_WAFG_ALARM_W;

          /* WAFG־ */
        p_dev->buf[0] &= ~__RX8025SA_C2_WAFG_ALARM_W;
        }
}
/**
 * \brief I2Cде״̬
 * \param[in] p_arg : 
 * \return 
 * \note ״̬任˳   [1]ʼ --> [2]дsubaddr,ȡ״̬ --> [3]
 *                   --> [5]дsubaddr --> [6]д --> [7]
 */
static void __rx8025sa_i2c_read_write_fsm (void *p_arg)
{
    /* I2C豸 */
    am_i2c_device_t *p_dev_i2c = NULL;

    /* rx8025sa豸 */
    am_rx8025sa_dev_t *p_dev_rx8025sa = NULL;

    /* жϲЧ */
    if (NULL == p_arg) {
        return;
    }

    /* ȡi2c_status */
    p_dev_rx8025sa = (am_rx8025sa_dev_t *)p_arg;

    /*ȡi2c_device */
    p_dev_i2c = &(p_dev_rx8025sa->i2c_dev);

    switch (p_dev_rx8025sa->status) {

    /* ȡ״̬ */
    case __RX8025SA_I2C_RD_STATUS:

        /* һ״̬ */
        p_dev_rx8025sa->status = __RX8025SA_I2C_WR_STATUS;

        am_i2c_mktrans(&p_dev_rx8025sa->trans[0],
                       p_dev_i2c->dev_addr,
                       (p_dev_i2c->dev_flags | AM_I2C_M_WR),
                       (uint8_t *)&p_dev_rx8025sa->sub_addr,
                       1);

        am_i2c_mktrans(&p_dev_rx8025sa->trans[1],
                       p_dev_i2c->dev_addr,
                       (p_dev_i2c->dev_flags | AM_I2C_M_RD),
                       (uint8_t *)p_dev_rx8025sa->buf,
                       p_dev_rx8025sa->nbytes);

        am_i2c_mkmsg(&p_dev_rx8025sa->msg,
                     p_dev_rx8025sa->trans,
                     2,
                     (am_pfnvoid_t)__rx8025sa_i2c_read_write_fsm,
                     p_arg);

        am_i2c_msg_start(p_dev_i2c->handle, &p_dev_rx8025sa->msg);
        break;

    /* ״̬ */
    case __RX8025SA_I2C_WR_STATUS:

        /* һ״̬ */
        p_dev_rx8025sa->status = __RX8025SA_I2C_DATA_PROCESSING;

        /* ݣȡж״̬ */
        __rx8025sa_read_write_data_processing(p_arg, &p_dev_rx8025sa->intr_flag);

        am_i2c_mktrans(&p_dev_rx8025sa->trans[0],
                       p_dev_i2c->dev_addr,
                       (p_dev_i2c->dev_flags | AM_I2C_M_WR),
                       (uint8_t *)&p_dev_rx8025sa->sub_addr,
                       1);

        am_i2c_mktrans(&p_dev_rx8025sa->trans[1],
                       p_dev_i2c->dev_addr,
                       (p_dev_i2c->dev_flags | AM_I2C_M_NOSTART | AM_I2C_M_WR),
                       (uint8_t *)p_dev_rx8025sa->buf,
                       p_dev_rx8025sa->nbytes);

        am_i2c_mkmsg(&p_dev_rx8025sa->msg,
                     p_dev_rx8025sa->trans,
                     2,
                     (am_pfnvoid_t)__rx8025sa_i2c_read_write_fsm,
                     p_arg);

        am_i2c_msg_start(p_dev_i2c->handle, &p_dev_rx8025sa->msg);
        break;

    /* ݴ */
    case __RX8025SA_I2C_DATA_PROCESSING:

        /* һ״̬ */
        p_dev_rx8025sa->status = __RX8025SA_I2C_RD_STATUS;

        /* ִûжжϻص */
        __rx8025sa_callback_exec(p_arg, p_dev_rx8025sa->intr_flag);

        /* GPIOж */
        if ( (p_dev_rx8025sa->intr_flag & __RX8025SA_C2_DAFG_ALARM_D ) ||
             (p_dev_rx8025sa->intr_flag & __RX8025SA_C2_CTFG_INT )){
             am_gpio_trigger_on(p_dev_rx8025sa->p_devinfo->inta_pin);
        }
        if ( p_dev_rx8025sa->intr_flag & __RX8025SA_C2_WAFG_ALARM_W) {
             am_gpio_trigger_on(p_dev_rx8025sa->p_devinfo->intb_pin);
        }

        break;

    default:
        break;
    }
}

/**
 * \brief RX8025SA INTAжϻص
 * \param[in] p_arg : жϻص
 * \return 
 */
static void __rx8025sa_inta_isr (void *p_arg)
{
    am_rx8025sa_dev_t        *p_dev        = (am_rx8025sa_dev_t *)p_arg;

    if (NULL == p_arg) {
        return;
    }

    /* رGPIOж */
    am_gpio_trigger_off(p_dev->p_devinfo->inta_pin);

    if (p_dev->status == __RX8025SA_I2C_RD_STATUS) {

        /* д״̬ */
        __rx8025sa_i2c_read_write_fsm(p_arg);
    }
}

/**
 * \brief RX8025SA INTBжϻص
 * \param[in] p_arg : жϻص
 * \return 
 */
static void __rx8025sa_intb_isr (void *p_arg)
{
    am_rx8025sa_dev_t        *p_dev        = (am_rx8025sa_dev_t *)p_arg;

    if (NULL == p_arg) {
        return;
    }

    /* رGPIOж */
    am_gpio_trigger_off(p_dev->p_devinfo->intb_pin);

    if (p_dev->status == __RX8025SA_I2C_RD_STATUS) {

        /* д״̬ */
        __rx8025sa_i2c_read_write_fsm(p_arg);
    }
}

/**
 * \brief RX8025SA ʱʽ
 */
static int __rx8025sa_time_format_set (am_rx8025sa_handle_t handle, uint8_t format)
{
    /* ÷ֵ */
    int ret = AM_OK;

    /* ݻ */
    uint8_t buf[1]  = {0};

    /* I2C豸ָ */
    am_i2c_device_t *p_i2c_dev = NULL;

    /* ֤Ч */
    if (NULL == handle) {
        return -AM_EINVAL;
    }

    /* handleлȡi2c豸ָ */
    p_i2c_dev = &(handle->i2c_dev);

    handle->time_format = format;
    /* ȡƼĴ1 */
    ret = am_i2c_read(p_i2c_dev,
                      __RX8025SA_REG_CONTROL1,
                      buf,
                      1);

    if (AM_OK != ret) {
        return ret;
    }

    if ( format == __RX8025SA_24H_FORMAT ){
        buf[0] |= __RX8025SA_C1_24H_FORMAT;
    } else {
        buf[0] &= ~ __RX8025SA_C1_24H_FORMAT;
    }

    /* д״̬Ĵ1 */
    ret = am_i2c_write(p_i2c_dev,
                       __RX8025SA_REG_CONTROL1,
                       buf,
                       1);

    if (AM_OK != ret) {
        return ret;
    }

    return AM_OK;

}

/*****************************************************************************
 * ⲿʵ
 ****************************************************************************/

/**
 * \brief RX8025SA 豸ʼ
 */
am_rx8025sa_handle_t am_rx8025sa_init (am_rx8025sa_dev_t           *p_dev,
                                       const am_rx8025sa_devinfo_t *p_devinfo,
                                       am_i2c_handle_t              i2c_handle)
{
    /* ÷ֵ */
    int ret = AM_OK;
    int i;

    /* ֤Ч */
    if (NULL == p_dev || NULL == i2c_handle) {
        return NULL;
    }

    /* i2c豸 */
    am_i2c_mkdev(&(p_dev->i2c_dev),
                 i2c_handle,
                 __RX8025SA_ADDR,
                 AM_I2C_ADDR_7BIT | AM_I2C_SUBADDR_1BYTE);

    /* ʼ豸Ϣ */
    p_dev->conn_stat_inta = AM_FALSE;
    p_dev->p_devinfo      = p_devinfo;

    p_dev->time_format    = __RX8025SA_24H_FORMAT;        /* Ĭ24СʱƼʱ */

    for (i = 0; i < 3; i++) {
        p_dev->triginfo[i].pfn_callback = NULL;
        p_dev->triginfo[i].p_arg        = NULL;
    }
    p_dev->status   = __RX8025SA_I2C_RD_STATUS;      /* ʼ״̬ */
    p_dev->sub_addr =__RX8025SA_REG_CONTROL2;        /* ļĴַ */
    p_dev->nbytes   = 1;                             /* Ҫȡݸ */

    /* ʼRX8025SA豸 */
    ret = __rx8025sa_init(p_dev);
    if (AM_OK != ret) {
        return NULL;
    }

    /* ʼʱʽ */
    ret = __rx8025sa_time_format_set( p_dev, p_dev->time_format);

    if (AM_OK != ret) {
        return NULL;
    }

    /* ʼclk_en_pin GPIO */
    if (p_dev->p_devinfo->clk_en_pin != -1) {
         am_gpio_pin_cfg(p_dev->p_devinfo->clk_en_pin, AM_GPIO_OUTPUT_INIT_LOW | AM_GPIO_PUSH_PULL);  /* Ϊ */
    }

    return p_dev;
}


/**
 * \brief RX8025SA 豸ʼ
 */
void am_rx8025sa_deinit (am_rx8025sa_handle_t handle)
{

}

/**
 * \brief ȡRX8025SA RTC׼
 */
am_rtc_handle_t am_rx8025sa_rtc_init (am_rx8025sa_handle_t  handle,
                                      am_rtc_serv_t        *p_rtc)
{
    p_rtc->p_drv   = handle;
    p_rtc->p_funcs = &__g_rtc_drv_funcs;
    return p_rtc;
}

/**
 * \brief ȡRX8025SA ALARM_CLK׼
 */
am_alarm_clk_handle_t am_rx8025sa_alarm_clk_init (am_rx8025sa_handle_t   handle,
		                                         am_alarm_clk_serv_t    *p_alarm_clk)
{
	p_alarm_clk->p_drv   = handle;
	p_alarm_clk->p_funcs = &__g_alarm_clk_drv_funcs;
    return p_alarm_clk;
}

/**
 * \brief ʹrx8025 alarm_d 
 */
int am_rx8025sa_alarm_d_enable (am_rx8025sa_handle_t handle)
{
    /* ÷ֵ */
    int ret = AM_OK;

    /* ݻ */
    uint8_t buf[1]     = {0};

    /* I2C豸ָ */
    am_i2c_device_t *p_i2c_dev = NULL;

    /* ֤Ч */
    if (NULL == handle) {
        return -AM_EINVAL;
    }

    /* handleлȡi2c豸ָ */
    p_i2c_dev = &(handle->i2c_dev);

    /* ȡƼĴ1 */
    ret = am_i2c_read(p_i2c_dev,
                      __RX8025SA_REG_CONTROL1,
                      buf,
                      1);

    if (AM_OK != ret) {
        return ret;
    }

    /* жʹ */
    buf[0] |= __RX8025SA_C1_ALARM_D_EN;

    /* жϢд״̬Ĵ1 */
    ret = am_i2c_write(p_i2c_dev,
                       __RX8025SA_REG_CONTROL1,
                       buf,
                       1);

    if (AM_OK != ret) {
        return ret;
    }
    return AM_OK;
}
/**
 * \brief ʹrx8025 alarm_w 
 */
int am_rx8025sa_alarm_w_enable (am_rx8025sa_handle_t handle)
{
    /* ÷ֵ */
    int ret = AM_OK;

    /* ݻ */
    uint8_t buf[1]     = {0};

    /* I2C豸ָ */
    am_i2c_device_t *p_i2c_dev = NULL;

    /* ֤Ч */
    if (NULL == handle) {
        return -AM_EINVAL;
    }

    /* handleлȡi2c豸ָ */
    p_i2c_dev = &(handle->i2c_dev);

    /* ȡƼĴ1 */
    ret = am_i2c_read(p_i2c_dev,
                      __RX8025SA_REG_CONTROL1,
                      buf,
                      1);

    if (AM_OK != ret) {
        return ret;
    }

    /* жϽ */
    buf[0] |= __RX8025SA_C1_ALARM_W_EN;

    /* жϢд״̬Ĵ1 */
    ret = am_i2c_write(p_i2c_dev,
                       __RX8025SA_REG_CONTROL1,
                       buf,
                       1);

    if (AM_OK != ret) {
        return ret;
    }

    return AM_OK;
}



/**
 * \brief  RX8025SA  ALARM_D 
 */
int am_rx8025sa_alarm_d_disable (am_rx8025sa_handle_t handle)
{
    /* ÷ֵ */
    int ret = AM_OK;

    /* ݻ */
    uint8_t buf[2]     = {0};

    /* I2C豸ָ */
    am_i2c_device_t *p_i2c_dev = NULL;

    /* ֤Ч */
    if (NULL == handle) {
        return -AM_EINVAL;
    }

    /* handleлȡi2c豸ָ */
    p_i2c_dev = &(handle->i2c_dev);

    /* ȡƼĴ1,2(ж) */
    ret = am_i2c_read(p_i2c_dev,
                      __RX8025SA_REG_CONTROL1,
                      buf,
                      2);

    if (AM_OK != ret) {
        return ret;
    }

    /* жϽ */
    buf[0] &= ~__RX8025SA_C1_ALARM_D_EN;

    /* AF־ */
    buf[1] &= ~__RX8025SA_C2_DAFG_ALARM_D;

    /* жϢд״̬Ĵ1,2 */
    ret = am_i2c_write(p_i2c_dev,
                       __RX8025SA_REG_CONTROL1,
                       buf,
                       2);

    if (AM_OK != ret) {
        return ret;
    }

    return AM_OK;
}

/**
 * \brief  RX8025SA  ALARM_W 
 */
int am_rx8025sa_alarm_w_disable (am_rx8025sa_handle_t handle)
{
    /* ÷ֵ */
    int ret = AM_OK;

    /* ݻ */
    uint8_t buf[2]     = {0};

    /* I2C豸ָ */
    am_i2c_device_t *p_i2c_dev = NULL;

    /* ֤Ч */
    if (NULL == handle) {
        return -AM_EINVAL;
    }

    /* handleлȡi2c豸ָ */
    p_i2c_dev = &(handle->i2c_dev);

    /* ȡƼĴ1,2(ж) */
    ret = am_i2c_read(p_i2c_dev,
                      __RX8025SA_REG_CONTROL1,
                      buf,
                      2);

    if (AM_OK != ret) {
        return ret;
    }

    /* жϽ */
    buf[0] &= ~__RX8025SA_C1_ALARM_W_EN;

    /* AF־ */
    buf[1] &= ~__RX8025SA_C2_WAFG_ALARM_W;

    /* жϢд״̬Ĵ1,2 */
    ret = am_i2c_write(p_i2c_dev,
                       __RX8025SA_REG_CONTROL1,
                       buf,
                       2);

    if (AM_OK != ret) {
        return ret;
    }

    return AM_OK;
}



/**
 * \brief RX8025SA жϺ
 */
int am_rx8025sa_periodic_int_set (am_rx8025sa_handle_t  handle,
                                   uint8_t              int_mode,
                                  am_pfnvoid_t          pfn_periodic_int_callback,
                                  void                 *p_periodic_int_arg)
{
    /* ÷ֵ */
    int ret = AM_OK;

    /* ݻ */
    uint8_t buf[1]     = {0};

    /* I2C豸ָ */
    am_i2c_device_t *p_i2c_dev = NULL;

    /* 豸Ϣָ */
    const am_rx8025sa_devinfo_t *p_devinfo = NULL;

    /* Ч */
    if (NULL == handle) {
        return -AM_EINVAL;
    }

    p_devinfo = handle->p_devinfo;
    p_i2c_dev = &(handle->i2c_dev);

    /* conn_stat_intaڼ¼ǰж״̬,ظGPIOӺ */
    if (!(handle->conn_stat_inta)) {

        /* жϻص */
        ret = am_gpio_trigger_connect(p_devinfo->inta_pin,
                                      __rx8025sa_inta_isr,
                                      handle);
        if (AM_OK != ret) {
            return ret;
        }

        /* Ϊ½ش */
        am_gpio_trigger_cfg(p_devinfo->inta_pin, AM_GPIO_TRIGGER_FALL);
        am_gpio_trigger_on(p_devinfo->inta_pin);

        /* conn_stat״̬ΪTRUE */
        handle->conn_stat_inta = AM_TRUE;

    }

    /* жϻص */
    handle->triginfo[0].pfn_callback = pfn_periodic_int_callback;
    handle->triginfo[0].p_arg        = p_periodic_int_arg;

    /* еĻصΪ */
    if ((handle->triginfo[0].pfn_callback == NULL) &&
        (handle->triginfo[1].pfn_callback == NULL)) {

        /* conn_stat_intaΪTRUEGPIOӺ */
        if (handle->conn_stat_inta) {

            /* ɾжϻص */
            ret = am_gpio_trigger_disconnect(p_devinfo->inta_pin,
                                             __rx8025sa_inta_isr,
                                             handle);

            if (AM_OK != ret) {
                return ret;
            }

            /* رŴ */
            am_gpio_trigger_off(p_devinfo->inta_pin);

            /* conn_stat״̬ΪFALSE */
            handle->conn_stat_inta = AM_FALSE;
        }
    }

    /* ȡƼĴ1 */
    ret = am_i2c_read(p_i2c_dev,
                      __RX8025SA_REG_CONTROL1,
                      buf,
                      1);

    if (AM_OK != ret) {
        return ret;
    }
    buf[0] |= int_mode;

    /* жϢд״̬Ĵ1 */
    ret = am_i2c_write(p_i2c_dev,
                       __RX8025SA_REG_CONTROL1,
                       buf,
                       1);

    if (AM_OK != ret) {
        return ret;
    }
    return AM_OK;
}

/**
 * \brief RX8025SA Dصʱ
 */
int am_rx8025sa_alarm_d_set(am_rx8025sa_handle_t      handle,
                            am_rx8025sa_alarm_info_t *p_alarm_info,
                            am_pfnvoid_t              pfn_alarm_callback,
                            void                     *p_alarm_arg)
{
    /* ÷ֵ */
    int ret = AM_OK;

    /* ݻ */
    uint8_t buf[2] = {0};

    /* I2C豸ָ */
    am_i2c_device_t *p_i2c_dev = NULL;

    /* 豸Ϣָ */
    const am_rx8025sa_devinfo_t *p_devinfo = NULL;

    /* Ч */
    if (NULL == handle) {
        return -AM_EINVAL;
    }

    p_devinfo = handle->p_devinfo;
    p_i2c_dev = &(handle->i2c_dev);

    /* conn_stat_intaڼ¼ǰж״̬,ظGPIOӺ */
    if (!(handle->conn_stat_inta)) {

        /* жϻص */
        ret = am_gpio_trigger_connect(p_devinfo->inta_pin,
                                      __rx8025sa_inta_isr,
                                       handle);
        if (AM_OK != ret) {
            return ret;
        }

        /* Ϊ½ش */
        am_gpio_trigger_cfg(p_devinfo->inta_pin, AM_GPIO_TRIGGER_FALL);
        am_gpio_trigger_on(p_devinfo->inta_pin);

        /* conn_stat_inta״̬ΪTRUE */
        handle->conn_stat_inta = AM_TRUE;

    }
    /* жϻص */
    handle->triginfo[1].pfn_callback = pfn_alarm_callback;
    handle->triginfo[1].p_arg        = p_alarm_arg;

    /* еĻصΪ */
    if ((handle->triginfo[0].pfn_callback == NULL) &&
        (handle->triginfo[1].pfn_callback == NULL)) {

        /* conn_statΪTRUEGPIOӺ */
        if (handle->conn_stat_inta) {

            /* ɾжϻص */
            ret = am_gpio_trigger_disconnect(p_devinfo->inta_pin,
                                             __rx8025sa_inta_isr,
                                             handle);

            if (AM_OK != ret) {
                return ret;
            }

            /* رŴ */
            am_gpio_trigger_off(p_devinfo->inta_pin);

            /* conn_stat״̬ΪFALSE */
            handle->conn_stat_inta = AM_FALSE;
        }
    }

    /* ֤ϢֵЧ */
    if (p_alarm_info->hour > 23 || p_alarm_info->hour < 0 ||
        p_alarm_info->min  > 59 || p_alarm_info->min < 0) {
        return -AM_EINVAL;
    }

    buf[0] = AM_HEX_TO_BCD(p_alarm_info->min);
    buf[1] = AM_HEX_TO_BCD(p_alarm_info->hour);
    /* I2C */
    ret = am_i2c_write(p_i2c_dev,
                       __RX8025SA_REG_MINUTES_ALARM_D,
                       buf,
                       sizeof(buf));

    if (AM_OK != ret) {
        return -AM_EINVAL;
    }
    /* ʹALARM_D*/
    am_rx8025sa_alarm_d_enable(handle);

    return AM_OK;
}


/**
 * \brief RX8025SA Wصʱ
 */
int am_rx8025sa_alarm_w_set(am_rx8025sa_handle_t      handle,
                            am_rx8025sa_alarm_info_t *p_alarm_info,
                            am_pfnvoid_t              pfn_alarm_callback,
                            void                     *p_alarm_arg)
{
    /* ÷ֵ */
    int ret = AM_OK;

    /* ݻ */
    uint8_t buf[3] = {0};

    /* I2C豸ָ */
    am_i2c_device_t *p_i2c_dev = NULL;

    /* 豸Ϣָ */
    const am_rx8025sa_devinfo_t *p_devinfo = NULL;

    /* Ч */
    if (NULL == handle) {
        return -AM_EINVAL;
    }

    p_devinfo = handle->p_devinfo;
    p_i2c_dev = &(handle->i2c_dev);

     /* жϻص */
    ret = am_gpio_trigger_connect(p_devinfo->intb_pin,
                                  __rx8025sa_intb_isr,
                                  handle);
    if (AM_OK != ret) {
        return ret;
    }

    /* Ϊ½ش */
    am_gpio_trigger_cfg(p_devinfo->intb_pin, AM_GPIO_TRIGGER_LOW);
    am_gpio_trigger_on(p_devinfo->intb_pin);

    /* жϻص */
    handle->triginfo[2].pfn_callback = pfn_alarm_callback;
    handle->triginfo[2].p_arg        = p_alarm_arg;

    /* еĻصΪ */
    if ( handle->triginfo[2].pfn_callback == NULL) {

        /* ɾжϻص */
        ret = am_gpio_trigger_disconnect(p_devinfo->intb_pin,
                                         __rx8025sa_intb_isr,
                                         handle);

        if (AM_OK != ret) {
            return ret;
        }

        /* رŴ */
        am_gpio_trigger_off(p_devinfo->inta_pin);

         /* conn_stat״̬ΪFALSE */
        handle->conn_stat_inta = AM_FALSE;
    }

    /* ֤ϢֵЧ */
    if (p_alarm_info->alarm_week > 127  || p_alarm_info->alarm_week < 0 ||
        p_alarm_info->hour > 23 || p_alarm_info->hour < 0 ||
        p_alarm_info->min  > 59 || p_alarm_info->min < 0) {
        return -AM_EINVAL;
    }

    buf[0] = AM_HEX_TO_BCD(p_alarm_info->min);
    buf[1] = AM_HEX_TO_BCD(p_alarm_info->hour);
    buf[2] = p_alarm_info->alarm_week;
    /* I2C */
    ret = am_i2c_write(p_i2c_dev,
    		           __RX8025SA_REG_MINUTES_ALARM_W,
                       buf,
                       sizeof(buf));

    if (AM_OK != ret) {
        return -AM_EINVAL;
    }
    /* ʹALARM_W*/
    am_rx8025sa_alarm_w_enable(handle);

    return AM_OK;

}
/*
 * Դѹ𵴼
 */
int am_rx8025sa_power_check(am_rx8025sa_handle_t  handle)
{
    /* ÷ֵ */
	int ret;

    /* ݻ */
    uint8_t buf;

    /* I2C豸ָ */
    am_i2c_device_t *p_i2c_dev = NULL;

    /* Ч */
    if (NULL == handle) {
        return -AM_EINVAL;
    }

    /* 豸ṹлȡi2c豸ָ */
    p_i2c_dev = &(handle->i2c_dev);

    /* ͨI2Cȡ */
    ret = am_i2c_read(p_i2c_dev,
    		          __RX8025SA_REG_CONTROL2,
                      &buf,
                      1);

    if (AM_OK != ret) {
        return -AM_EINVAL;
    }
    buf = (buf>>4) & 0x07;

    return buf;
}

/**
 * ͵ѹ⹦ܵı׼ѹֵ趨
 */
int am_rx8025sa_check_voltage_set (am_rx8025sa_handle_t handle, uint8_t check_v_value)
{
    /* ÷ֵ */
    int ret = AM_OK;

    /* ݻ */
    uint8_t buf[1]     = {0};

    /* I2C豸ָ */
    am_i2c_device_t *p_i2c_dev = NULL;

    /* ֤Ч */
    if (NULL == handle) {
        return -AM_EINVAL;
    }

    /* handleлȡi2c豸ָ */
    p_i2c_dev = &(handle->i2c_dev);

    /* ȡƼĴ1 */
    ret = am_i2c_read(p_i2c_dev,
    		          __RX8025SA_REG_CONTROL2,
                      buf,
                      1);

    if (AM_OK != ret) {
        return ret;
    }

    if ( check_v_value == RX8025SA_CHECK_VOLTAGE_1_3V ){
        buf[0] |= __RX8025SA_C2_VDSL_1_3_SET;
    } else {
    	buf[0] &= ~ __RX8025SA_C2_VDSL_1_3_SET;
    }

    /* жϢд״̬Ĵ1 */
    ret = am_i2c_write(p_i2c_dev,
    		           __RX8025SA_REG_CONTROL2,
                       buf,
                       1);

    if (AM_OK != ret) {
        return ret;
    }

    return AM_OK;

}

/*
 * ʱƵʽе
 */
int am_rx8025sa_clk_adjust ( am_rx8025sa_handle_t handle, float clk )
{
    /* ÷ֵ */
    int   ret = AM_OK;

    double  clk_off;
    int     temp;

    /* ݻ */
    uint8_t buf[1]     = {0};

    /* I2C豸ָ */
    am_i2c_device_t *p_i2c_dev = NULL;

    /* ֤Ч */
    if (NULL == handle) {
        return -AM_EINVAL;
    }

    /* handleлȡi2c豸ָ */
    p_i2c_dev = &(handle->i2c_dev);

    clk_off = ( clk - 32768)/32768;
    if ( clk_off >= 0 ) {
        temp = (int)(clk_off / 0.00000305 +0.5);
        temp += 1;
    } else {
        temp = (int)(clk_off / 0.00000305 -0.5);
    	 temp +=128;
    }

    buf[0] = temp;

    ret = am_i2c_write(p_i2c_dev,
                       __RX8025SA_REG_DIGITAL_OFFSET,
                       buf,
                       1);

    if (AM_OK != ret) {
        return ret;
    }

    return AM_OK;
}
/**
 * Ƶ
 */
int am_rx8025sa_clkout_set(am_rx8025sa_handle_t handle)
{
    /* ÷ֵ */
    int ret = AM_OK;

    /* ݻ */
    uint8_t buf[1]  = {0};

    /* I2C豸ָ */
    am_i2c_device_t *p_i2c_dev = NULL;

    /* ֤Ч */
    if (NULL == handle) {
        return -AM_EINVAL;
    }

    /* handleлȡi2c豸ָ */
    p_i2c_dev = &(handle->i2c_dev);

    /* ȡƼĴ1 */
    ret = am_i2c_read(p_i2c_dev,
                      __RX8025SA_REG_CONTROL1,
                      buf,
                      1);

    if (AM_OK != ret) {
        return ret;
    }

    buf[0] &= 0xEF;  /* ƼĴ1CLEN2Ϊ */

    /* д״̬Ĵ1 */
    ret = am_i2c_write(p_i2c_dev,
                       __RX8025SA_REG_CONTROL1,
                       buf,
                       1);

    if (AM_OK != ret) {
        return ret;
    }
    /*  FOEøߣʹCLK */
    /*  FOEøߣʹCLK */
    if( handle->p_devinfo->clk_en_pin != -1) {
        ret = am_gpio_set(handle->p_devinfo->clk_en_pin, AM_GPIO_LEVEL_HIGH);
    }

    if (AM_OK != ret) {
        return ret;
    }

    return AM_OK;
}

/**
 * صƵ
 */
int am_rx8025sa_clkout_close(am_rx8025sa_handle_t handle)
{
    /* ÷ֵ */
    int ret = AM_OK;

    /*  FOEõͣرCLK */
    if( handle->p_devinfo->clk_en_pin != -1) {
        ret = am_gpio_set(handle->p_devinfo->clk_en_pin, AM_GPIO_LEVEL_LOW);
    }

    if (AM_OK != ret) {
        return ret;
    }

    return AM_OK;
}
/* end of file */
