/*******************************************************************************
*                                 AMORKS
*                       ----------------------------
*                       innovating embedded platform
*
* Copyright (c) 2001-2017 Guangzhou ZHIYUAN Electronics Stock Co., Ltd.
* All rights reserved.
*
* Contact information:
* web site:    http://www.zlg.cn/
* e-mail:      amorks.support@zlg.cn
*******************************************************************************/
/**
 * \file
 * \brief user interface of modbus master protocol.
 *
 * \internal
 * \par modification history
 * - 1.00 2017-03-06  mkr, transplant from amorks
 * \endinternal
 */
#ifndef __AM_MB_MASTER_H  /* { */
#define __AM_MB_MASTER_H

/**
 * \addtogroup grp_am_if_mb_master
 * \copydoc am_mb_master.h
 * @{
 */

#include "am_mb_err.h"
#include "am_mb_comm.h"
#include "am_mb_m_netif.h"

#ifdef __cplusplus
extern "C" {
#endif

/**
 * \brief PDUݴӿ
 * @note  ûעԶ幦ʱ趨ͽӿעᡣ
 * @note  ûʹøýӿڽp_paramsתΪp_pdudataݣ룩,PDU
 *        ݳ(p_pdudata_len)Դ˸֪ЭջPDUݼͳȡ
 *
 * @param[in]           p_params : ûԶ;
 *                                 òΪam_mb_master_requestӿڵp_param.
 * @param[in][out]     p_pdudata : PDUݻûתС
 * @param[in][out] p_pdudata_len : PDUݻȣòʱ֪û
 *                                 PDUݻ󳤶ȡ
 * @return PDUݴʧܣ򷵻Ӧ֮ͣAM_MB_ERR_NOERR
 */
typedef am_mb_err_t (*am_mb_snd_funcode_handler_t) (void    *p_params,
                                                    uint8_t *p_pdudata,
                                                    uint8_t *p_pdudata_len);

/**
 * \brief PDUݴӿ
 * @note  ûעԶ幦ʱ趨ͽӿעᡣ
 * @note  ûʹøýӿڽPDUתΪp_params.
 *
 * @param[in][out]  p_params : ûԶ,
 *                             òΪam_mb_master_requestӿڵp_param.
 * @param[in]      p_pdudata : PDUݻ().
 * @param[in]    pdudata_len : PDUݻ.
 * @return PDUݴʧܣ򷵻Ӧ֮ͣAM_MB_ERR_NOERR
 */
typedef am_mb_err_t (*am_mb_rcv_funcode_handler_t) (void    *p_params,
                                                    uint8_t *p_pdudata,
                                                    uint8_t  pdudata_len);

/** \brief  function code handler type */
typedef struct am_mb_m_funcode_handler {
    uint8_t                     funcode;     /** \brief function_code */
    am_mb_snd_funcode_handler_t snd_handler; /** \brief send handle interface */
    am_mb_rcv_funcode_handler_t rcv_handler; /** \brief recv handle interface */
} am_mb_m_funcode_handler_t;

/******************************************************************************/

/** \brief Modbus վ  */
typedef struct am_mb_master_serv {
    /** \brief point to asii/rtu mode private data */
    void                     *p_cookie;

    uint8_t                   slave_addr; /**< \brief slave address */
    enum am_mb_mode           mode;       /**< \brief master mode   */
    enum am_mb_master_status  status;     /**< \brief master status */

    /** \brief ûעṦ뻺 */
    am_mb_m_funcode_handler_t *p_funcode_hdl;

    /** \brief ֧עĹ */
    uint8_t                    funcode_num;

    am_mb_event_t            event;     /**< \brief Master event */

    const am_mb_m_netif_t     *p_netif;   /**< \brief net interface */

    uint32_t ack_timeout;  /**< \brief master wait ack timeout */
    uint32_t bd_delay;     /**< \brief rtu/ascii broadcast delay time */

} am_mb_master_serv_t;

/** \brief Modbus վ */
typedef am_mb_master_serv_t *am_mb_master_handle;

/** \brief Modbus վ豸 */
typedef struct am_mb_master_dev {
    /** \brief modbus master service */
    am_mb_master_serv_t mb_master_serv;

    union mode {
#if AM_MB_MASTER_RTU_ENABLE
        am_mb_m_rtu_t   mbm_rtu;     /**< \brief RTU mode private data */
#endif

#if AM_MB_MASTER_ASCII_ENABLE
        am_mb_m_ascii_t mbm_ascii;   /**< \brief ASCII mode private data */
#endif

#if AM_MB_MASTER_HANDLECP_ENABLE
        am_mb_m_tcp_t   mbm_tcp;     /**< \brief TCP mode private data */
#endif
    } common;

} am_mb_master_dev_t;

typedef struct am_mb_master_info {
    aw_mb_timer_info_t  timer;  /**< \brief ʱ豸Ϣ */
    am_mb_serial_info_t serial; /**< \brief ·豸Ϣ */

    enum am_mb_mode     mode;   /**< \brief վģʽ */
} am_mb_master_info_t;
/******************************************************************************/

/**
 * \brief Modbusվ
 *
 * @note ڵվ֮ǰȵøú.
 * @note TCPģʽ£ͬһվвͬһ߳.ûʹøýӿڽ.
 *
 * @brief p_m_dev  Modbusվ豸
 *
 * @return Modbusվ󷵻NULL
 */
am_mb_master_handle am_mb_master_create (am_mb_master_dev_t *p_dev);

/**
 * \brief Modbusվ
 *
 * @note ýӿڱվֹͣδµ.
 *
 * @param[in] master           : Modbusվ
 * @param[in] p_dev_handle      紮豸
 * @param[in] p_params         : ò
 *                               AM_MB_RTU/AM_MB_ASCIIģʽ
 *                               ʹam_mb_master_serial_params_tΪ;
 *                               AM_MB_TCPģʽ,
 *                               ʹam_mb_tcp_client_paramsΪ.
 *                               ûòòɴNULL.
 * @retval: AM_MB_ERR_NOERR    : վóɹ.
 * @retval: AM_MB_ERR_EINVAL   : .
 * @retval: AM_MB_ERR_EILLSTATE: ЭջǷ״̬.
 * @retval: AM_MB_ERR_EIO      : ó.
 */
am_mb_err_t am_mb_master_config (am_mb_master_handle        master,
                                 const am_mb_master_info_t *p_info);

/**
 * \brief رաɾվ
 * @note  ˳ϵͳ߲Ҫʹվʱ
 *
 * @param[in] master : Modbusվ
 *
 * @return ûд󷵻AM_MB_ERR_NOERR
 */
am_mb_err_t am_mb_master_delete (am_mb_master_handle master);

/**
 * \brief Modbusվ
 *
 * @param[in] master           : Modbusվ
 *
 * @retval: AM_MB_ERR_NOERR    : վɹ
 * @retval: AM_MB_ERR_EINVAL   : 
 * @retval: AM_MB_ERR_EILLSTATE: ЭջǷ״̬
 * @retval: AM_MB_ERR_EIO      : TCPģʽ£ӽʧ.
 * @retval: AM_MB_ERR_ETIMEDOUT: TCPģʽ£ӳʱ
 */
am_mb_err_t am_mb_master_start (am_mb_master_handle master);

/**
 * \brief ֹͣModbusվ
 *
 * @note øýӿڽֹͣ䣻
 *
 * @param[in] master           : Modbusվ
 *
 * @retval: AM_MB_ERR_NOERR    : վֹͣɹ
 * @retval: AM_MB_ERR_EINVAL   : 
 * @retval: AM_MB_ERR_EILLSTATE: ЭջǷ״̬
 * @retval: AM_MB_ERR_EIO      : IO
 */
am_mb_err_t am_mb_master_stop (am_mb_master_handle master);

/** \brief վд */
typedef struct am_mb_master_rw_param {
     uint16_t  rd_addr;     /**< \brief ʼַ  */
     uint16_t  rd_num;      /**< \brief Ŀ            */
     uint16_t *p_rd_buf;    /**< \brief             */
     uint16_t  wr_addr;     /**< \brief дʼַ  */
     uint16_t  wr_num;      /**< \brief дĿ            */
     uint16_t *p_wr_buf;    /**< \brief д            */
} am_mb_master_rw_param_t;

/**
 * \brief վ
 * @note  ýӿվɵ.
 * @note  ýӿΪͬӿ.
 * @note  վʱʱ䣬ͨӿ am_mb_master_ack_timeout_set 
 *
 * @note
 * @param[in] master       : Modbusվ
 * @param[in] slave_addr   : Ŀӻַ
 * @param[in] funcode      : 
 * @param[in][out] p_param : д;
 *                           ΪЭջֹ֧룬
 *                           òΪam_mb_master_rw_param_t;
 *                           ΪûԶ幦룬òûԶ塣
 *
 * @return ûд󷵻AM_MB_ERR_NOERRʧܣɲ鿴͡
 *
 */
am_mb_err_t am_mb_master_request (am_mb_master_handle master,
                                  uint8_t        slave_addr,
                                  uint8_t        funcode,
                                  void          *p_params);

/**
 * \brief עԶ幦봦ӿڵָվ.
 * @note   վ󣬿עûԶ幦봦ӿڡ
 * @note  ýӿڽ봦ӿעɹ󣬿ʹam_mb_master_requestӿ
 * @note  û¶ЭջĬϹ봦ӿڡûעͬһ봦ӿڣ
 *        ʹעĹ봦ӿڡ
 * @note  עĹ봦am_mb_config.h
 *        AM_MB_MASTER_FUNCODE_HANDLER_MAX壬ø޷עᡣ
 *
 * @param[in] master      : վ
 * @param[in] funcode     : 
 * @param[in] snd_handler : ʹ
 * @param[in] rcv_handler : մΪдû账PDUݣ
 *                          ɽ˽ӿNULL
 * @return עɹAM_MB_ERR_NOERR;ʧܷӦ.
 *
 * \code
 * #include "am_mb_master.h"
 *
 * #define MB_MASK_WRITE_REG_FUNCODE  22
 * struct wr_mask_param {
 *     uint16_t addr;
 *     uint16_t and_mask;
 *     uint16_t or_mask;
 * } my_mask_param;
 *
 * am_mb_err_t __mb_snd_mask_handler (void    *p_params,
 *                                    uint8_t *p_pdudata,
 *                                    uint8_t *pdudata_len)
 * {
 *     struct wr_mask_param *p_mask_param = (struct wr_mask_param *)p_params;
 *
 *     p_pdudata[0] = (uint8_t)(p_mask_param->addr >> 8);
 *     p_pdudata[1] = (uint8_t)(p_mask_param->addr);
 *     p_pdudata[2] = (uint8_t)(p_mask_param->and_mask >> 8);
 *     p_pdudata[3] = (uint8_t)(p_mask_param->and_mask);
 *     p_pdudata[4] = (uint8_t)(p_mask_param->or_mask >> 8);
 *     p_pdudata[5] = (uint8_t)(p_mask_param->or_mask);
 *
 *     *pdudata_len = 6;
 *
 *     return AM_MB_ERR_NOERR;
 * }
 *     //עдĴ빦봦
 *     am_mb_master_funcode_register(master,
 *                                   MB_MASK_WRITE_REG_FUNCODE,
 *                                   __mb_snd_mask_handler,
 *                                   NULL);
 *     //дĴ
 *     my_mask_param.addr     = 1000;
 *     my_mask_param.and_mask = 0x01;
 *     my_mask_param.or_mask  = 0;
 *     am_mb_master_request(master,
 *                          SLAVE_ADDR,
 *                          MB_MASK_WRITE_REG_FUNCODE,
 *                          &my_mask_param);
 * \endcode
 */
am_mb_err_t am_mb_master_funcode_register ( \
                                    am_mb_master_handle         master,
                                    uint8_t                     funcode,
                                    am_mb_snd_funcode_handler_t snd_handler,
                                    am_mb_rcv_funcode_handler_t rcv_handler);

/** \brief վͶ  */
typedef int am_mbm_cfg_type_t;
#define AM_MBM_TYPE_USE_HW_TIMER      0x01    /**< \brief ʹӲʱ */
#define AM_MBM_TYPE_BOARDCAST_DELAY   0x02    /**< \brief 㲥ʱʱ, p_argΪ㲥ʱʱ(ms)(uint32_t)   */
#define AM_MBM_TYPE_UID_AS_ADDR       0x03    /**< \brief TCPģʽUIDΪվַ, p_argΪǷΪվַ(bool_t)   */
#define AM_MBM_TYPE_TCP_CONTMOUT      0x04    /**< \brief TCPģʽӳʱʱ䣬 p_argΪʱʱ䣨ms(uint32_t)   */
#define AM_MBM_TYPE_ACK_TIMEOUT_SET   0x05    /**< \brief Ӧʱã p_argΪʱʱ䣨ms(uint32_t)   */
#define AM_MBM_TYPE_ACK_TIMEOUT_GET   0x06    /**< \brief Ӧʱȡ p_argΪʱʱ䣨ms(uint32_t *)   */

/**
 * \brief վ
 * @note  ޵øýӿڣòam_mb_config.hв.
 * @note  ýӿվǰ.
 *
 * @param[in] master : վ
 * @param[in] type   : :
 *                     - AM_MBM_TYPE_USER_FUNCODE   : Զ幦ĬϲΪ
 *                                                   Ϊ#AM_MB_MASTER_FUNCODE_HANDLER_MAX
 *                     - AM_MBM_TYPE_BOARDCAST_DELAY: 㲥ʱʱ䣬
 *                                                   ĬϲʱʱΪ#AM_MB_MASTER_DELAY_MS_CONVERT
 *                     - AM_MBM_TYPE_UID_AS_ADDR    : TCPģʽUIDΪվַ, ĬϲΪվַ
 *                     - AM_MBM_TYPE_TCP_CONTMOUT   : TCPģʽӳʱʱ䣬ĬϳʱʱΪ#AM_MB_TCP_CONNECT_TIMEOUT
 *                     - AM_MBM_TYPE_ACK_TIMEOUT_SET: Ӧʱã ĬʱΪ#am_mb_master_handleIMEOUT_MS_RESPOND
 *                                                   ӿam_mb_master_ack_timeout_setһ
 *                     - AM_MBM_TYPE_ACK_TIMEOUT_GET: Ӧʱȡӿam_mb_master_ack_timeout_getһ
 * @param[in] p_arg  : Ͳ
 * @return óɹAM_MB_ERR_NOERR;ʧܷӦ.
 */
am_mb_err_t am_mb_master_ioctl (am_mb_master_handle    master,
                                am_mbm_cfg_type_t      type,
                                void                  *p_arg);

/**
 * \brief Ӧʱʱ
 *
 * @param[in]      master      : վ
 * @param[in]      timeout     : Ӧʱʱ䣨룩, ȡֵΧ50 ~ 100000ms
 *
 * @retval AM_MB_ERR_EINVAL    : 
 */
am_mb_err_t am_mb_master_ack_timeout_set (am_mb_master_handle master,
                                          uint32_t       timeout);

/**
 * \brief ȡӦʱʱ
 *
 * @param[in]      master      : վ
 *
 * @return Ӧʱʱȡ 0Ϊ
 */
uint32_t am_mb_master_ack_timeout_get (am_mb_master_handle master);

/**
 * \brief ǰǷΪ㲥
 *
 * @param[in] p_master : վ豸
 * @retval TRUE  : Ϊ㲥
 * @retval FALSE : ǹ㲥
 */
am_static_inline am_bool_t am_mb_m_is_broadcast (am_mb_master_handle p_master)
{
    return p_master->slave_addr == AM_MB_ADDRESS_BROADCAST ? AM_TRUE : AM_FALSE;
}

/**
 * \brief ȡ봦ӿ
 *
 * @param[in] p_master : վ豸
 * @param[in] funcode  : 
 * @return Ӧ봦ӿڣ޸ù봦ӿڣNULL.
 */
const am_mb_m_funcode_handler_t *am_mb_m_funcode_handler_get (
                                         am_mb_master_handle p_master,
                                         uint8_t             funcode);
/**
 * \brief ȡӦʱʱȡ
 *
 * @param[in] p_master : վ豸
 * @return    ʱʱ
 */
am_static_inline uint32_t am_mb_m_ack_timeout_get (am_mb_master_handle p_master)
{
    return p_master->ack_timeout;
}


/**
 * \brief ȡ㲥ӳٻȡ
 *
 * @param[in] p_master : վ豸
 * @return    ʱʱ
 */
am_static_inline uint32_t am_mb_master_broadcast_delay_get (
                                        am_mb_master_handle p_master)
{
    return p_master->bd_delay;
}

#if am_mb_master_handleCP_ENABLE
/**
 * \brief TCPվӺԭ
 * @note  ùӺam_mb_master_startʱִ.
 *
 * @param[in] p_arg  : ûԶ
 * @param[in] socket : socket
 *
 * @return AM_MB_ERR_NOERR.
 */
typedef am_mb_err_t (*am_mb_master_handlecp_hook_t) (void *p_arg, int socket);

/**
 * \brief TCPģʽsocketӹӺע
 *
 * @param[in] master   : վ
 * @param[in] tcp_hook : Ӻ
 * @param[in] p_arg    : Ӻ
 *
 * @retval AM_MB_ERR_NOERR     : עɹ
 * @retval AM_MB_ERR_EINVAL    : 
 * @retval AM_MB_ERR_EILLSTATE : ЭջǷ״̬
 */
am_mb_err_t am_mb_master_handlecp_hook_register (am_mb_master_handle          master,
                                            am_mb_master_handlecp_hook_t tcp_hook,
                                            void                   *p_arg);

/**
 * \brief TCPģʽǷ񽫵ַΪUID
 *
 * @param[in] p_master : վ豸
 * @return    ʱʱ
 */
am_static_inline bool_t am_mbm_is_uid_as_addr (am_mb_m_dev_t *p_master)
{
    return p_master->uid_as_addr;
}

/**
 * \brief ȡTCPģʽʱʱȡ
 *
 * @param[in] p_master : վ豸
 * @return    ʱʱ
 */
am_static_inline uint32_t am_mb_tcp_contm_get (am_mb_m_dev_t *p_master)
{
    return p_master->tcp_contm;
}
#endif

/**
 * \brief û
 *
 * @param[in] p_master : վ豸
 * @param[in] num      : û
 * @return ޴󷵻AM_MB_ERR_NOERR
 */
am_mb_err_t am_mb_master_user_funcode_alloc (am_mb_master_handle        master,
                                             am_mb_m_funcode_handler_t *p_buf,
                                             uint8_t                    num);

/*****************************************************************************
 * Common Function's Code Interface(ӿ)
 *****************************************************************************/
/**
 * \brief Ȧ
 * @note [0x01]
 *
 * @param[in] master        : վ
 * @param[in] slave_addr    : Ŀӻַ
 * @param[in] start_addr    : Ȧʼַ
 * @param[in] quantity      : Ȧ, ȡֵΧ1~2000
 * @param[in][out] p_rd_val : ȡȦֵ
 *
 * @return ִгɹAM_MB_ERR_NOERR;ʧܷӦ
 */
am_mb_err_t am_mb_coils_read (am_mb_master_handle master,
                              uint8_t             slave_addr,
                              uint16_t            start_addr,
                              uint16_t            quantity,
                              uint8_t            *p_rd_val);

/**
 * \brief ɢ
 * @note [0x02]
 *
 * @param[in] master        : վ
 * @param[in] slave_addr    : Ŀӻַ
 * @param[in] start_addr    : ɢʼַ
 * @param[in] quantity      : ɢȡ, ȡֵΧ1~2000
 * @param[in][out] p_rd_val : ȡɢֵ
 *
 * @return ִгɹAM_MB_ERR_NOERR;ʧܷӦ
 */
am_mb_err_t am_mb_discrete_inputs_read (am_mb_master_handle master,
                                        uint8_t             slave_addr,
                                        uint16_t            start_addr,
                                        uint16_t            quantity,
                                        uint8_t            *p_rd_val);

/**
 * \brief ּĴ
 * @note [0x03]
 *
 * @param[in] master        : վ
 * @param[in] slave_addr    : Ŀӻַ
 * @param[in] start_addr    : ʼַ
 * @param[in] quantity      : ȡּĴȡֵΧ1~125
 * @param[in][out] p_rd_val : ȡּĴֵ
 *
 * @return ִгɹAM_MB_ERR_NOERR;ʧܷӦ
 */
am_mb_err_t am_mb_holding_reg_read (am_mb_master_handle master,
                                    uint8_t             slave_addr,
                                    uint16_t            start_addr,
                                    uint16_t            quantity,
                                    uint16_t           *p_rd_val);

/**
 * \brief Ĵ
 * @note [0x04]
 *
 * @param[in] master        : վ
 * @param[in] slave_addr    : Ŀӻַ
 * @param[in] start_addr    : ʼַ
 * @param[in] quantity      : ȡĴȡֵΧ1~125
 * @param[in][out] p_rd_val : ȡĴֵ
 *
 * @return ִгɹAM_MB_ERR_NOERR;ʧܷӦ
 */
am_mb_err_t am_mb_input_reg_read (am_mb_master_handle master,
                                  uint8_t             slave_addr,
                                  uint16_t            start_addr,
                                  uint16_t            quantity,
                                  uint16_t           *p_rd_val);

/**
 * \brief дȦ
 * @note [0x05]
 *
 * @param[in] master     : վ
 * @param[in] slave_addr : Ŀӻַ
 * @param[in] coil_addr  : дȦַ
 * @param[in] coil_val   : дȦֵֵΪ0x0000(COIL OFF) 0xFF00(COIL ON).
 *
 * @return ִгɹAM_MB_ERR_NOERR;ʧܷӦ
 */
am_mb_err_t am_mb_single_coil_write (am_mb_master_handle master,
                                     uint8_t             slave_addr,
                                     uint16_t            coil_addr,
                                     uint16_t            coil_val);

/**
 * \brief дĴ
 * @note [0x06]
 *
 * @param[in] master     : վ
 * @param[in] slave_addr : Ŀӻַ
 * @param[in] reg_addr   : дĴַ
 * @param[in] reg_val    : дĴֵ
 *
 * @return ִгɹAM_MB_ERR_NOERR;ʧܷӦ
 */
am_mb_err_t am_mb_single_reg_write (am_mb_master_handle master,
                                    uint8_t             slave_addr,
                                    uint16_t            reg_addr,
                                    uint16_t            reg_val);

/**
 * \brief дȦ
 * @note [0x0F]
 *
 * @param[in] master     : վ
 * @param[in] slave_addr : Ŀӻַ
 * @param[in] start_addr : дȦʼַ
 * @param[in] quantity   : дȦ0x01~0x7b0
 * @param[in] p_wr_buf   : дȦֵ()
 *
 * @return ִгɹAM_MB_ERR_NOERR;ʧܷӦ
 */
am_mb_err_t am_mb_multiple_coils_write (am_mb_master_handle master,
                                        uint8_t             slave_addr,
                                        uint16_t            start_addr,
                                        uint16_t            quantity,
                                        uint8_t            *p_wr_buf);

/**
 * \brief дĴ
 * @note [0x10]
 *
 * @param[in] master     : վ
 * @param[in] slave_addr : Ŀӻַ
 * @param[in] start_addr : дĴʼַ
 * @param[in] quantity   : дĴ
 * @param[in] p_wr_buf   : дĴֵ()
 *
 * @return ִгɹAM_MB_ERR_NOERR;ʧܷӦ
 */
am_mb_err_t am_mb_multiple_regs_write (am_mb_master_handle master,
                                       uint8_t             slave_addr,
                                       uint16_t            start_addr,
                                       uint16_t            quantity,
                                       uint16_t           *p_wr_buf);

/**
 * \brief дĴ
 * @note [0x17]
 *
 * @param[in]      master     : վ
 * @param[in]      slave_addr : Ŀӻַ
 * @param[in][out] p_rw_param : д
 *
 * @return ִгɹAM_MB_ERR_NOERR;ʧܷӦ
 */
am_mb_err_t am_mb_multiple_regs_read_write ( \
                                        am_mb_master_handle      master,
                                        uint8_t                  slave_addr,
                                        am_mb_master_rw_param_t *p_rw_param);
/*****************************************************************************/


#ifdef __cplusplus
}
#endif

/** @} grp_am_if_mb_master */

#endif /* } __AM_MB_MASTER_H */

/* end of file */

