/*******************************************************************************
*                                 AWORKS
*                       ----------------------------
*                       innovating embedded platform
*
* Copyright (c) 2001-2017 Guangzhou ZHIYUAN Electronics Stock Co., Ltd.
* All rights reserved.
*
* Contact information:
* web site:    http://www.zlg.cn/
* e-mail:      apollo.support@zlg.cn
*******************************************************************************/

/**
 * \file
 * \brief modbusӻЭջûӿͷļ
 *
 * \internal
 * \par modification history
 * - 1.00 2017-03-29  mkr, transplant from aworks
 * \endinternal
 */

#ifndef __AM_MB_SLAVE_H /* { */
#define __AM_MB_SLAVE_H

/**
 * \addtogroup grp_am_if_mb_slave
 * \copydoc am_mb_slave.h
 * @{
 */

#include "am_mb_config.h"
#include "am_mb_err.h"
#include "am_mb_comm.h"
#include "am_mb_netif.h"
#include "am_mb_compiler.h"

#ifdef __cplusplus
extern "C" {
#endif

/** \brief ӻIDϢ */
#define AM_MB_FUNC_OTHER_REP_SLAVEID_BUF  32

/** ص */
enum am_mb_func_cb_type {
	AM_MB_FUNC_COILS_CALLBACK,                /**< \brief дȦص */
	AM_MB_FUNC_DISCRETEINPUTS_CALLBACK,       /**< \brief ɢص */
	AM_MB_FUNC_HOLDREGISTERS_CALLBACK,        /**< \brief дּĴص */
	AM_MB_FUNC_INPUTREGISTERS_CALLBACK,       /**< \brief Ĵص */
};

/** ص */
enum am_mb_func_cb_op {
	AM_MB_FUNC_CALLBACK_READ,   /**< \brief  */
	AM_MB_FUNC_CALLBACK_WRITE,  /**< \brief д */
};

typedef struct am_mb_slave_serv am_mb_slave_serv_t;

/** Modbusվ */
typedef am_mb_slave_serv_t *am_mb_slave_handle;

/******************************************************************************/

/**
 * \file
 *
 * callback function
 *
 * Read the ON/OFF status of discrete outputs(DO)in the slave. The coils status
 * is packed as per bit of data field. Status is indicated as: 1=ON, 0=OFF.
 * The LSB of the first data byte contains the coil addressed in the query.
 * If the coil quantity is not a multiple of eight, the remaining bits in the
 * final data byte will be padded with zeros.
 *
 * For example. addr = 0x0013  num = 10,
 * status of coils 20 ~ 27 is ON-ON-OFF-OFF-ON-OFF-ON-OFF 28~29 ON-ON,
 * so the buf[0] = 01010011(0x53)  buf[1] = 00000011
 *
 * buf[0]:
 * 7  |6  |5  |4  |3  |2  |1  |0
 * ---|---|---|---|---|---|---|---
 * 27 |26 |25 |24 |23 |22 |21 |20
 * OFF|ON |OFF|ON |OFF|OFF|ON |ON
 * 0  |1  |0  |1  |0  |0  |1  |1
 *
 * buf[1]:
 * 7  |6  |5  |4  |3  |2  |1  |0
 * ---|---|---|---|---|---|---|----
 * -  |-  |-  |-  |-  |-  |29 |28
 * 0  |0  |0  |0  |0  |0  |ON |ON
 * 0  |0  |0  |0  |0  |0  |1  |1
 */

/**
 * \brief дȦɢдĴĴصԭ
 * @note  ʹñṩĶдȦɢдĴ
 * 		     Ĵȹ봦ʱҪע
 * @code
 * am_mb_exception_t my_coils_callback(am_mb_slave_t slave,
 *                                     uint8_t   *buf,
 *                                     uint16_t   addr,
 *                                     uint16_t   num)
 * {
 *     //Ȧ
 * }
 *
 * //slave֮ǰ²
 * am_mb_slave_register_callback(slave,
 *                            	 am_MB_FUNC_COILS_CALLBACK,
 *                            	 am_MB_FUNC_CALLBACK_READ,
 *                            	 my_coils_callback);
 * @endcode
 *
 * @param[in]     slave ӻ
 * @param[in,out] buf   д
 * @param[in]     addr  дʼַ
 * @param[in]     num   д
 * @return û쳣AM_MB_EXP_NONE򷵻ضӦ쳣
 * @see am_mb_slave_register_callback
 */
typedef am_mb_exception_t (*am_mb_slave_fn_code_callback_t)(am_mb_slave_handle  slave,
	                                                        uint8_t            *p_buf,
                                                            uint16_t            addr,
                                                            uint16_t            num);

/**
 * \brief û봦ԭ.
 * @note  ӻյһ֡Ժ󣬽ݹöӦĴ
 * 		     ûͨ\a am_mb_slave_register_handler עᡣ
 *		     ͬʱѾĶдȦɢдĴĴ
 * 		     ȴ
 *
 * @param[in] slave Modbusվ
 * @param[in,out] p_pdubuf PDU()
 *         - [in]  PDU
 *         - [out] ӦPDU
 * @param[in,out] p_pdulen PDUݳ
 *         - [in]  PDUݳ
 *         - [out] ӦPDUݳ
 * @return û쳣AM_MB_EXP_NONE򷵻ضӦ쳣
 */
typedef am_mb_exception_t (*am_mb_fn_code_handler_t) (am_mb_slave_handle  slave,
                                                  	  uint8_t            *p_pdubuf,
                                                  	  uint16_t           *p_pdulen);

/** \brief function code handler type */
typedef struct am_funcode_handle {
    uint8_t                 funcode;
    am_mb_fn_code_handler_t handler;
} am_funcode_handle_t;

/******************************************************************************/

/** \brief Modbus վ */
typedef struct am_mb_slave_serv {
    void                     *p_cookie;  /**< \brief  */
    enum am_mb_slave_status   status;    /**< \brief վ״̬  */
    enum am_mb_mode           mode;      /**< \brief ǰģʽ */
    const struct am_mb_netif *p_netif;   /**< \brief ģʽӿ */
    /** \brief ûעṦ */
    am_funcode_handle_t      *p_funcode_hdl;
    /** \brief ֧עĹ */
    uint8_t                   funcode_hdl_max;

    am_mb_event_t             event;   /**< \brief վ¼ */

    /** \brief ͨùص  */
    am_mb_slave_fn_code_callback_t fn_read_coils_cb;
    am_mb_slave_fn_code_callback_t fn_write_coils_cb;
    am_mb_slave_fn_code_callback_t fn_read_discre_cb;
    am_mb_slave_fn_code_callback_t fn_read_input_cb;
    am_mb_slave_fn_code_callback_t fn_read_hold_cb;
    am_mb_slave_fn_code_callback_t fn_write_hold_cb;
#if 1
    /** \brief ûIDϢɲ  */
    uint8_t  id[AM_MB_FUNC_OTHER_REP_SLAVEID_BUF];
    uint16_t id_len;
#endif
} am_mb_slave_serv_t;

/** \brief Modbusվ豸 */
typedef struct am_mb_slave_dev {
    /** \brief modbus slave service */
    am_mb_slave_serv_t am_mb_slave_serv;

    union mode {
        struct am_mb_rtu   mb_rtu;   /**< \brief RTU mode private data */
        struct am_mb_ascii mb_ascii; /**< \brief ASCII mode private data */
#if AM_MB_TCP_ENABLE
        struct am_mb_tcp   mb_tcp;   /**< \brief TCP mode private data */
#endif
    } common;

} am_mb_slave_dev_t;

/** \brief modbus slave info  */
typedef struct am_mb_slave_info {
    aw_mb_timer_info_t  timer_info;  /**< \brief ʱ豸Ϣ */
    am_mb_serial_info_t serial_info; /**< \brief ·豸Ϣ */
    uint8_t             slave_addr;  /**< \brief ӻַ */
    enum am_mb_mode     mode;        /**< \brief ģʽ */
} am_mb_slave_info_t;

/******************************************************************************/
/**
 * \brief ʼһModbusվ
 * @note  ڵдӻ֮ǰȵøú
 * @note  ͬ紴վ
 *
 * @param[in] p_s_dev : ָվ豸ָ
 * @param[in] mode    : ͨģʽ
 * @param[in] p_param : ʼMB_RTU/MB_ASCIIģʽʹ
 *                      struct am_mb_serial_paramΪ
 *                      AM_MB_TCPģʽparamΪ˿ںŲʹ,ֱӴβ
 * @param[out] p_err  : . ԴŲȤԴNULL
 * شֵ
 * -AM_MB_ERR_NOERR           : ʼɹ.
 * -AM_MB_ERR_EINVAL          : .
 * -AM_MB_ERR_ALLOC_FAIL      : ڴ治㣬վʧ.
 * -AM_MB_ERR_MODE_NO_SUPPORT : ģʽ֧
 * -AM_MB_ERR_EPORTERR         ģʽ䴮ڻӲʱʧ.
 * -AM_MB_ERR_EIO             : TCPģʽ˴ʧ.
 * @return Modbusվ󷵻NULL
 */
am_mb_slave_handle am_mb_slave_init (am_mb_slave_dev_t        *p_dev,
                                     const am_mb_slave_info_t *p_info,
								     am_mb_err_t              *p_err);

/**
 * \brief رModbusվ.
 * @note  ˳ϵͳ߲Ҫʹôӻʱ
 *
 * @param[in] slave : Modbusվ
 *
 * @retval AM_MB_ERR_NOERR     : ޴.
 * @retval AM_MB_ERR_EINVAL    : վ.
 * @retval AM_MB_ERR_EILLSTATE : ЭջǷ״̬.
 */
am_mb_err_t am_mb_slave_close (am_mb_slave_handle slave);

/**
 * \brief Modbusվ.
 * @note  úڵam_mb_slave_initߵam_mb_slave_stop󱻵
 *
 * @param[in] slave            : Modbusվ
 *
 * @retval AM_MB_ERR_NOERR     : ޴.
 * @retval AM_MB_ERR_EINVAL    : վ.
 * @retval AM_MB_ERR_EILLSTATE : ЭջǷ״̬.
 */
am_mb_err_t am_mb_slave_start (am_mb_slave_handle slave);

/**
 * \brief ֹͣModbusվ.
 *
 * @param[in] slave            : Modbusվ
 *
 * @retval AM_MB_ERR_NOERR     : ޴.
 * @retval AM_MB_ERR_EINVAL    : վ.
 * @retval AM_MB_ERR_EILLSTATE : ЭջǷ״̬.
 */
am_mb_err_t am_mb_slave_stop (am_mb_slave_handle slave);

/**
 * \brief עṦ봦.
 * @note  ʼModbusվԺ󣬿ԵøúעӦĹ봦չ
 *
 * @note עĹ봦am_mb_config.hAM_MB_FUNC_CODE_HANDLER_MAX塣
 *       ø޷עᡣͬһֻһעͬ
 *       һ봦ֻʹעĴ
 * @param[in] slave    : Modbusվ
 * @param[in] funcode  : 
 * @param[in] handler  : 봦
 *
 * @retval  AM_MB_ERR_NOERR  : ޴;
 * @retval  AM_MB_ERR_EINVAL : ;
 * @retval  AM_MB_ERR_ENORES : ޷עṦ.
 */
am_mb_err_t am_mb_slave_register_handler (am_mb_slave_handle       slave,
                                    	  uint8_t                  funcode,
                                    	  am_mb_fn_code_handler_t  handler);

/**
 * \brief עдȦɢдĴĴص.
 * @note  ʹñṩĶдȦɢдĴĴ
 *           ʱ, ʹøúлصעᣬ÷Ƿ쳣
 *
 * @param[in] slave ӻ
 * @param[in] type ص
 * @param[in] op ص
 * @param[in] callback ص
 *
 * @retval AM_MB_ERR_NOERR  : עɹ.
 * @retval AM_MB_ERR_EINVAL : .
 */
am_mb_err_t am_mb_slave_register_callback (am_mb_slave_handle             slave,
                                           enum am_mb_func_cb_type        type,
                                           enum am_mb_func_cb_op          op,
                                           am_mb_slave_fn_code_callback_t callback);

/**
 * \brief Modbusվѯ.
 * @note  ֡֡ΪModbusվЭĺġú뱻Եĵá
 *        ʱӦúվҪʱ
 * @note  ýӿRTUASCIITCPģʽ.
 * @note  RTU/ASCIIģʽ£ýӿʱȡam_mb_slave_cfgЭջýӿڡ
 *        TCPģʽ£ýӿΪ
 *
 * @param[in] slave : Modbusվ
 *
 * @retval AM_MB_ERR_NOERR     : ޴;
 * @retval AM_MB_ERR_EINVAL    : ;
 * @retval AM_MB_ERR_EILLSTATE : ЭջǷ״̬.
 * @retval AM_MB_ERR_ENORES    : ģʽӦʱյµ.
 * @retval AM_MB_ERR_EIO       : IO. ģʽ״̬. TCPģʽʧ.
 */
am_mb_err_t am_mb_slave_poll (am_mb_slave_handle slave);

/** \brief վ */
typedef int am_mbs_cfg_type_t;
#define AM_MBS_TYPE_USED_HW_TMR        0x01   /**< \brief ʹӲʱ */
#define AM_MBS_TYPE_TCP_NUM            0x03   /**< \brief TCPģʽ */
#define AM_MBS_TYPE_ACCEPT_TCP_NUM_GET 0x04   /**< \brief ȡǰѽյ */
#define AM_MBS_TYPE_ALLOW_NEW_TCP      0x05   /**< \brief Ӵﵽֵرվ */

/**
 * \brief վ
 *
 * @note ޵øýӿڣͰam_mb_config.hв.
 *
 * @param[in] type  : ͣ
 * - AM_MBS_TYPE_TCP_NUM            : p_argΪuint32_t;
 * - AM_MBS_TYPE_ACCEPT_TCP_NUM_GET : ȡǰѽp_argΪȡuint32_t *;
 * - AM_MBS_TYPE_ALLOW_NEW_TCP      : ﵽ޺ӶϿ,p_argΪ1ʱ;
 *
 * @param[in] p_arg :  òȡ
 *
 * @return óɹAM_MB_ERR_NOERR,֮Ӧֵ.
 */
am_mb_err_t am_mb_slave_ioctl (am_mb_slave_handle slave,
                               am_mbs_cfg_type_t  type,
                               void              *p_arg);

/**
 * \brief վģʽȡ
 *
 * @param[in]      slave  : Modbusվ
 * @param[in][out] p_mode : վģʽ
 *
 * @retval AM_MB_ERR_EINVAL : 
 * @retval AM_MB_ERR_NOERR  : ޴
 */
am_mb_err_t am_mb_slave_mode_get (am_mb_slave_handle  slave,
                                  enum am_mb_mode    *p_mode);

/**
 * \brief ModbusվϢ.
 * Modbusվ˵ǰ״̬һЩϢ
 * @note ʵֶModbusվϢ󳤶Ϊ32ֽڣУ
 *    - ջ˵ռһֽ
 *    - ǰ״̬ռһֽ
 *    - Ϣ<= 30Ϣ\n
 *  ʹҪرע.
 * @note ýӿ֧ãĬϲ֧
 *
 * @param[in] slave        : Modbusվ
 * @param[in] slave_id     : ˵
 * @param[in] is_running   : ǰ״̬> 0ʾУ== 0ʾֹͣ״̬
 * @param[in] p_additional : һЩϢ
 * @param[in] alen         : һЩϢ
 *
 * @retval AM_MB_ERR_NOERR  : ޴;
 * @retval AM_MB_ERR_EINVAL : ;
 * @retval AM_MB_ERR_ENORES : Դ.
 */
am_mb_err_t am_mb_slave_set_id (am_mb_slave_handle slave,
                                uint8_t            slave_id,
                                uint8_t            is_running,
                                const uint8_t     *p_additional,
                                uint16_t           alen);

/**
 * \brief Modbusվַ.
 * @note  Ĭڵam_mb_slave_initгʼʱʼѾôվַ
 * 		     Ҫʱ޸Ĵվַʹøú
 *
 * @param[in] slave         : Modbusվ
 * @param[in] addr          : µModbusվַ
 *
 * @retval AM_MB_ERR_NOERR  : ޴;
 * @retval AM_MB_ERR_EINVAL : ; վڣַַ õַΪ㲥ַ
 */
am_mb_err_t am_mb_slave_set_addr (am_mb_slave_handle slave, uint8_t addr);

/**
 * \brief ûԶ幦ڴ
 *
 * @param[in] p_slave : վ豸ʵ.
 * @param[in] p_buf   : ûԶ幦뻺ʼַ
 * @param[in] num     : 
 *
 * @return    p_err   ţûдֵΪam_MB_ERR_NOERR
 */
am_mb_err_t am_mbs_user_funcode_alloc (am_mb_slave_handle   slave,
                                       am_funcode_handle_t *p_buf,
                                       uint8_t              num);

#if AM_MB_TCP_ENABLE

/**
 * \brief TCPϢ
 */
struct sockaddr;
struct am_mb_slave_tcp_info {
    int              socket;      /**< \brief socketsocket  */
    struct sockaddr *p_sockaddr;  /**< \brief IPַϢ  */
};

/**
 * \brief TCPģʽsocketӹӺע
 *
 * @param[in] master      : վ
 * @param[in] accept_hook : ӹӺ
 * @param[in] close_hook  : ӱϿﵽ޶ϿӺ
 * @param[in] p_arg       : Ӻ
 *
 * @retval AM_MB_ERR_NOERR     : עɹ
 * @retval AM_MB_ERR_EINVAL    : 
 * @retval AM_MB_ERR_EILLSTATE : ЭջǷ״̬
 */
am_mb_err_t am_mb_slave_tcp_hook_register (am_mb_slave_t            slave,
                                           am_mb_tcp_accept_hook_t  accept_hook,
                                           am_mb_tcp_close_hook_t   close_hook,
                                           void                    *p_arg);
#endif

#ifdef __cplusplus
}
#endif

/** @} grp_am_if_mb_slave */

#endif /* } __AM_MB_SLAVE_H */

/* end of file */
