/*******************************************************************************
*                                 AMetal
*                       ----------------------------
*                       innovating embedded platform
*
* Copyright (c) 2001-2018 Guangzhou ZHIYUAN Electronics Co., Ltd.
* All rights reserved.
*
* Contact information:
* web site:    http://www.zlg.cn/
*******************************************************************************/

/**
 * \file
 * \brief  XmodemЭ Ӧýӿļ
 *
 * \internal
 * \par Modification History
 * - 1.00 18-8-31 , xgg, first implementation.
 * \endinternal
 */
#ifndef __AM_XMODEM_H
#define __AM_XMODEM_H

#ifdef __cplusplus
extern "C" {
#endif

#include "am_crc.h"
#include "am_uart.h"
#include "am_errno.h"
#include "am_softimer.h"

/**
 * @addtogroup am_if_am_xmodem
 * @copydoc am_xmodem.h
 * @{
 */

/**
 * \name Xmodem ģʽ
 *
 * @{
 */
#define AM_XMODEM_1K_MODE     0    /**< \brief ݶ1KֽڵĹģʽ*/
#define AM_XMODEM_128_MODE    1    /**< \brief ݶ128ֽڵĹģʽ*/

/** @} */

/**
 * \name XmodemУ鷽ʽ
 *
 * @{
 */
#define AM_XMODEM_CRC_MODE    0    /**< \brief CRCУ */
#define AM_XMODEM_SUM_MODE    1    /**< \brief ۻУ */

/** @} */

/**
 * \name Xmodemܹв¼
 * º궨Xmodemܻصд룬ûҪݴevent¼
 * һĲ
 * @{
 */
#define AM_DATA_CAN          0x19  /**< \brief ͷȡ*/
#define AM_DATA_SUC          0x20  /**< \brief ļճɹ */
#define AM_XMODEM_DOWN       0x25  /**< \brief Xmodemδ*/
#define AM_XMODEM_CHECK_ERR  0x26  /**< \brief У*/
/** @} */

/**
 * \name Xmodem͹в¼
 *
 * @{
 */
#define AM_XMODEM_ACK        0x06 /**< \brief շȷźţʾһ֡ݽ */
#define AM_XMODEM_NAK        0x15 /**< \brief Xmodem128ֽڹģʽ */
#define AM_XMODEM_1k         'C'  /**< \brief Xmodem1Kֽڹģʽ*/
#define AM_XMODEM_CAN        0x18 /**< \brief շȡ */
#define AM_XMODEM_NAK_TIME   0x40 /**< \brief طط */
#define AM_XMODEM_MOU_SUC    0x41 /**< \brief ļ */

/** @} */

/*******************************************************************************
  ״ָ̬Ͷ
*******************************************************************************/
/** \brief Xmodem״ָ̬ */
typedef am_bool_t (*pfn_xmodem_rx_t)(void *p_arg, char inchar);

/** \brief Xmodem״ָ̬ */
typedef am_bool_t (*pfn_xmodem_tx_t)(void *p_arg, char *p_outchar);

/******************************************************************************/
/**
 * \breif ûջص
 *
 * \param[in] p_arg    : ûصĲûڻص
 *                       ʹXmodemɽ˲
 * \param[in] p_frames : Xmodemյһ֡ݱĵַûֱ
 *                       ʹøõַȡյһ֡
 * \param[in] event    : жeventֵ0һ֡ݽ
 *                       ɹeventǽյֽevent
 *                       ֵΪֵΪ-AM_DATA_SUCļճ
 *                       ʧܿɸeventֵжִ
 *                       -AM_ETIME            : ǰ֡ݳʱδյ
 *                       -AM_EBADMSG          : δʱطȻ
 *                       -AM_DATA_CAN         : ͷȡ͵´
 *                       -AM_XMODEM_DOWN      : δλ
 *                       -AM_XMODEM_CHECK_ERR : У
 * \note ײ޷Դ¼ʱͨ˺֪ͨ
 *       ûûδӦ¼¼ûҪرע:
 * 1event > 0 ʱǰѾյһ֡ݣ֡ݵĴСevent,
 *   ûȡ¼ʱ,Ҫȡһ֡Ӧõam_xmodem_rec_ack_set
 *   
 * 2event == -AM_XMODEM_DOWNնδܽXmodemӣûҪ½
 *   ǰ¼ʱԵam_xmodem_rec_start½ӡ޲ֱ˳
 * 3event == -AM_ETIME նδڹ涨涨ʱܽյ֡ݣû
 *    ޲˳
 * 4event == -AM_DATA_CAN ͷȡ˴䣬ûֱ
 *    ˳
 * 5event == -AM_EBADMSG ǰ֡дѾطȻ
 *    ûԿλ͵Ƿ⡣
 * 6event == -AM_XMODEM_CHECK_ERR Уû½У顣
 *
 * \return 
 */
typedef void (*am_xmodem_user_rx_t) (void    *p_arg,
                                     void    *frames,
                                     int      event);

/** \brief xmodemЭ豸Ϣṹ */
typedef struct am_xmodem_rec_dev_info {
    char      *frames_info;     /**< \brief Xmodemһ֡Ϣָ */
    uint8_t    nak_max_times;   /**< \brief ط */
    uint32_t   frames_bytes;    /**< \brief һ֡ݵֽ */
    uint32_t   data_mode;       /**< \brief ģʽ  */
    uint32_t   parity_mode;     /**< \brief Уģʽ */
    uint32_t   rx_timeout;      /**< \brief ճʱʱ */
    /** \brief ʱ
     * ûڵam_xmodem_rec_startӿܺ󣬻ÿ
     * start_timeʱ䷢ֱַxmodemӣ
     * طnak_max_timesȻܽӣͨص
     * event = -AM_XMODEM_DOWN֪ͨûXmodemδ
     * ӡ
     */
    uint32_t   start_time;
} am_xmodem_rec_dev_info_t;

/**
 * \brief xmodem豸ṹ
 */
typedef struct am_xmodem_rec_dev {
    int                  rx_bytes;       /**< \brief ǰյֽ */
    char                 fra_sum_parity; /**< \brief ۻУλ */
    uint16_t             fra_crc_pry;    /**< \brief CRCУ */
    volatile uint8_t     char_get_state; /**< \brief ַȡ״̬־λ */
    volatile uint8_t     nak_state;      /**< \brief ǰط״̬ */
    volatile uint8_t     frames_num;     /**< \brief ǰһ֡ݵк */
    volatile uint8_t     con_state;      /**< \brief Xmodemͨ״̬*/
    am_softimer_t        rx_softimer;    /**< \brief ճʱʱ */
    am_uart_handle_t     uart_handle;    /**< \brief ھ */
    am_xmodem_user_rx_t  pfn_rx_callback;/**< \brief עûܻص */
    am_crc_handle_t      crc_handle;     /**< \brief ׼CRC*/
    am_crc_pattern_t     crc_pattern;    /**< \brief  CRC ģ */
    void                *p_arg;          /**< \brief ûص */
    volatile pfn_xmodem_rx_t  p_rec_func;/**< \brief ״̬*/

    /* 豸Ϣṹָ*/
    const am_xmodem_rec_dev_info_t *p_rec_devinfo;
} am_xmodem_rec_dev_t;

/** \brief xmodemܱ׼Ͷ */
typedef am_xmodem_rec_dev_t *am_xmodem_rec_handle_t;

/******************************************************************************/

/**
 * \brief ûͻص
 *
 * \param[in] p_arg : ûҪĲ,Լ
 * \param[in] event : صص¼ûɸ¼һ
 * 1event == AM_XMODEM_NAKʱ,Xmodem128ֽڹģʽ
 *   Ҫʼ͵һ֡ݣڻصֱӵam_xmodem_tx_packӿ
 *   ־ѭеô˽ӿڣκݣ޲
 * 2event == AM_XMODEM_1Kʱ,Xmodem1Kֽڹģʽ
 *   ͬûҪʼ͵һ֡ݣɵam_xmodem_tx_packӿڻ޲
 * 3event == AM_XMODEM_NAK_TIMEʱ,طﵽ涨ֵ(ڲȡ);
 *   ¼ѷ͵շշҪطʱڲѾҪ
 *   ȡ˷ͣɿǽշǷ쳣
 * 4event == AM_XMODEM_MOU_SUCʱ,ļ;
 *   ǰѾļϣXmodemѾϿûѡ޲
 * 5event == AM_XMODEM_CANʱ,շȡ˴;
 *   ¼շϿXmodemӣûԿ޲ֱ˳
 * 6event == AM_XMODEM_ACKʱһ֡ݷ;
 *   ¼ǰһ֡ѾϣûԵam_xmodem_tx_pack
 *   һ֡
 * 7event == -AM_ETIMEʱ,շյһ֡ݺδڹ涨ʱڻӦ;
 *
 * \return 
 */
typedef void (*am_xmodem_user_tx_t) (void *p_arg, int event);


/** \brief XmodemЭ鷢豸Ϣṹ*/
typedef struct am_xmodem_tx_dev_info {
    uint32_t   tx_timeout;       /**< \brief ͳʱʱ*/
    uint8_t    nak_times_max;    /**< \brief ط*/
} am_xmodem_tx_dev_info_t;

/**
 * \brief Xmodem豸
 */
typedef struct am_xmodem_tx_dev {
    char                           fra_sum_parity; /**< \brief ۻУλ */
    char                          *p_tx_buf;       /**< \brief ļָ */
    uint8_t                        nake_state;     /**< \brief ǰط״̬ */
    uint8_t                        tx_state;       /**< \brief ƻȡַ״̬*/
    uint16_t                       fra_crc_pry;    /**< \brief CRCУ */
    uint32_t                       frame_tx_bytes; /**< \brief ͵һ֡ݴС */
    uint32_t                       doc_bytes;      /**< \brief ļС */
    uint32_t                       ctrlz_num;      /**< \brief ֽ*/
    volatile uint8_t               state_flag;     /**< \brief ڷֱģʽַ*/
    volatile uint32_t              tx_bytes;       /**< \brief ǰһ֡ѷͶ */
    volatile uint32_t              frames_num;     /**< \brief ǰһ֡ݵк */
    pfn_xmodem_tx_t                p_tx_func;      /**< \brief ״̬*/
    am_softimer_t                  tx_softimer;    /**< \brief ճʱʱ */
    void                          *p_arg;          /**< \brief ûص */
    am_xmodem_user_tx_t            pfn_tx_callback;/**< \brief עûͻص */
    am_crc_handle_t                crc_handle;     /**< \brief CRC*/
    am_crc_pattern_t               crc_pattern;    /**< \brief CRCģ*/
    am_uart_handle_t               uart_handle;    /**< \brief ھ */
    const am_xmodem_tx_dev_info_t *p_tx_devinfo;   /**< \brief 豸Ϣṹָ*/
}am_xmodem_tx_dev_t;

/** \brief xmodemܱ׼Ͷ */
typedef am_xmodem_tx_dev_t *am_xmodem_tx_handle_t;

/******************************************************************************
 Xmodem ӿ
*******************************************************************************/
/**
 * \brief 豸ʼ
 *
 * \param[in] p_dev       : xmodem豸
 * \param[in] p_decinfo   : 豸Ϣṹ
 * \param[in] uart_handle : ھ
 * \param[in] crc_handle  : CRC
 *
 * \return ΪNULL򷵻ֵΪXmodemվ
 *         ΪNULLЧʱʼʧ
 */
am_xmodem_rec_handle_t  am_xmodem_rec_init (
                                 am_xmodem_rec_dev_t            *p_dev,
                                 const am_xmodem_rec_dev_info_t *p_devinfo,
                                 am_uart_handle_t                uart_handle,
                                 am_crc_handle_t                 crc_handle);

/**
 * \brief xmodemջصע
 *
 * \param[in] handle     : xmodem豸
 * \param[in] pfn_rec_cb : ûص
 * \param[in] p_arg      : ûĲ
 *
 * \retval AM_OK      : ִгɹ
 * \retval -AM_EINVAL : Ч
 */
am_err_t am_xmodem_rec_cb_reg(am_xmodem_rec_handle_t  handle,
                               am_xmodem_user_rx_t     pfn_rec_cb,
                               void                   *p_arg);

/**
 * \brief ʼܺ(Xmodemպ)
 *
 * \param[in] handle : xmodem豸
 *
 * \retval AM_OK      : ִгɹ
 * \retval -AM_EINVAL : Ч
 */
am_err_t am_xmodem_rec_start (am_xmodem_rec_handle_t handle);

/**
 * \brief պ
 *
 * \param[in] handle : xmodem豸
 *
 * \retval AM_OK      : ִгɹ
 * \retval -AM_EINVAL : Ч
 */
am_err_t am_xmodem_rec_ack_set (am_xmodem_rec_handle_t handle);

/**
 * \brief ȡ亯
 *
 * \param[in] p_dev xmodem豸
 *
 * \retval AM_OK      : ִгɹ
 * \retval -AM_EINVAL : Ч
 */
am_err_t am_xmodem_rec_can_set (am_xmodem_rec_handle_t handle);

/******************************************************************************
 Xmodem ӿ
*******************************************************************************/
/**
 * \brief 豸ʼ
 *
 * \param[in] p_dev         : Xmodem豸
 * \param[in] p_tra_devinfo : Xmodem豸Ϣṹ
 * \param[in] uart_handle   : ھ
 * \param[in] crc_handle    : CRC
 *
 * \return ΪNULLֵΪXmodem;ΪNULLЧʱʼʧ
 */
am_xmodem_tx_handle_t  am_xmodem_tx_init (
                              am_xmodem_tx_dev_t            *p_dev,
                              const am_xmodem_tx_dev_info_t *p_tra_devinfo,
                              am_uart_handle_t               uart_handle,
                              am_crc_handle_t                crc_handle);

/**
 * \brief ֡ݷͺ
 *
 * \param[in] handle    : Xmodem;
 * \param[in] p_doc     : ûҪ͵ָ
 * \param[in] pack_size : Ҫ͵鳤ȣص¼ΪAM_XMODEM_NAK,
 *                        pack_sizeֵΪ128,ص¼Ϊ
 *                        AM_XMODEM_1K,pack_sizeֵΪ1024ļһ
 *                        ֡ݲ1281024pack_sizeֵûԼ
 *                        Ϊļʣµֽ
 *
 * \retval AM_OK      : ִгɹ
 * \retval -AM_EINVAL : Ч
 */
am_err_t am_xmodem_tx_pack (am_xmodem_tx_handle_t  handle,
                            char                  *p_doc,
                            uint32_t               pack_size);

/**
 * \brief xmodemͻصע
 *
 * \param[in] handle    : Xmodem;
 * \param[in] pfn_tx_cb : ûصָ
 * \param[in] p_arg     : û豸
 *
 * \retval AM_OK      : ִгɹ
 * \retval -AM_EINVAL : Ч
 */
am_err_t am_xmodem_tx_cb_reg (am_xmodem_tx_handle_t handle,
                              am_xmodem_user_tx_t  pfn_tx_cb,
                              void                *p_arg);

/**
 * \brief ļͽ
 *
 * \param[in] handle : Xmodem;
 *
 * \retval AM_OK      : ִгɹ
 * \retval -AM_EINVAL : Ч
 */
am_err_t am_xmodem_tx_over (am_xmodem_tx_handle_t handle);

/**
 * \brief ûȡͺ
 *
 * \param[in] handle : Xmodem;
 *
 * \retval AM_OK      : ִгɹ
 * \retval -AM_EINVAL : Ч
 */
am_err_t am_xmodem_tx_can_set (am_xmodem_tx_handle_t handle);

/**
 * @}
 */

#ifdef __cplusplus
}
#endif

#endif /* __AM_XMODEM_H */

/* end of file */
