/*******************************************************************************
*                                 AMetal
*                       ----------------------------
*                       innovating embedded platform
*
* Copyright (c) 2001-2018 Guangzhou ZHIYUAN Electronics Co., Ltd.
* All rights reserved.
*
* Contact information:
* web site:    http://www.zlg.cn/
*******************************************************************************/

/**
 * \file
 * \brief  XmodemЭ 
 *
 * \internal
 * \par Modification History
 * - 1.00 18-8-31 , xgg, first implementation.
 * \endinternal
 */

#include "ametal.h"
#include "string.h"
#include "am_types.h"
#include "am_xmodem.h"

/*******************************************************************************
local defined
*******************************************************************************/
/* ļ*/
#define __AM_XMODEM_DOC_EOT     0x51

/* ļźѷͲӦ*/
#define __AM_XMODEM_EOT_ACK     0x52

/* λΪ128ֽڹģʽʼλ */
#define __AM_XMODEM_SOH         0x01

/* λΪ1Kֽڹģʽʼλ*/
#define __AM_XMODEM_STX         0x02

/* ļϵĽź */
#define __AM_XMODEM_EOT         0x04

/*  */
#define __AM_XMODEM_CTRLZ       0x1A

/* ڻȡַر״̬*/
#define __AM_XMODEM_CHAR_DOWN   0x00

/* ڻȡַ״̬*/
#define __AM_XMODEM_CHAR_UP     0x01

/* XmodemѾ*/
#define __AM_XMODEM_CON_UP      0x02

/* Xmodemδ*/
#define __AM_XMODEM_CON_DOWN    0x03

/* ڷ*/
#define __AM_XMODEM_DATA_TX     0x04

/* ڷʹ־*/
#define __AM_XMODEM_FLAG_TX     0x05

/* ֹȡַ*/
#define __AM_XMODEM_STOP_TX     0x06

/* Xmodem;*/
#define __AM_XMODEM_TX_READY    0x00

/* Xmodem*/
#define __AM_XMODEM_TX_START    0x01
/******************************************************************************/
/** \brief ֡ͷܺ*/
am_local am_bool_t am_xmodem_rec_frames (am_xmodem_rec_dev_t *p_dev,
                                         char                 inchar);

/** \brief ʼܺ*/
am_local void  __xmodem_rx_startup (am_xmodem_rec_dev_t  *p_dev);

/******************************************************************************/
/**
 * \brief ͷȡͺ
 */
am_local void __xmodem_rec_can (am_xmodem_rec_dev_t *p_dev, char inchar)
{
    p_dev->rx_bytes = -AM_DATA_CAN;
    /* ص֪ͨû ͷȡ*/
    p_dev->pfn_rx_callback(p_dev->p_arg,
                           p_dev->p_rec_devinfo->frames_info,
                           p_dev->rx_bytes);
}

/******************************************************************************/
/**
 * \brief շжϺ
 */
am_local int __xmodem_data_check(am_xmodem_rec_dev_t *p_dev)
{
    int       i;
    char      cks = 0;
    uint32_t  crc;
    uint16_t  tcrc;
    if (p_dev->p_rec_devinfo->parity_mode == AM_XMODEM_CRC_MODE)
    {
        am_crc_cal(p_dev->crc_handle,
                   (uint8_t *)p_dev->p_rec_devinfo->frames_info,
                   p_dev->p_rec_devinfo->frames_bytes);
        am_crc_final(p_dev->crc_handle, &crc);
        tcrc = p_dev->fra_crc_pry;
        /* ԼУʹõУбȽ*/
        if ((uint16_t)crc == tcrc) {
            return AM_TRUE;
        }
    }
    else
    {
        /* ۼ*/
        for (i = 0; i < p_dev->p_rec_devinfo->frames_bytes; i++)
        {
            cks += p_dev->p_rec_devinfo->frames_info[i];
        }
        /* мۼӺ봫ۼӺͽбȽ*/
        if (cks == p_dev->fra_sum_parity)
            return AM_TRUE;
    }
    return AM_FALSE;
}

/******************************************************************************/
/**
 * \brief յEOT
 */
am_local am_bool_t __xmodem_rec_eot (am_xmodem_rec_dev_t *p_dev, char inchar)
{
    /* ַȡɹ*/
    p_dev->char_get_state = __AM_XMODEM_CHAR_UP;

    /* ַܵӦͷ */
    p_dev->p_rec_devinfo->frames_info[0] = AM_XMODEM_ACK;

    /* ʹж*/
    am_uart_tx_startup(p_dev->uart_handle);

    /* ͳɹ*/
    p_dev->rx_bytes = -AM_DATA_SUC;

    /* ص֪ͨû */
    p_dev->pfn_rx_callback(p_dev->p_arg,
                           p_dev->p_rec_devinfo->frames_info,
                           p_dev->rx_bytes);

    return AM_TRUE;
}

/******************************************************************************/
/**
 * \brief SUMУ麯
 */
am_local am_bool_t __xmodem_rec_sum (am_xmodem_rec_dev_t *p_dev, char inchar)
{
    p_dev->fra_sum_parity = inchar;
    if (AM_TRUE == __xmodem_data_check(p_dev))
       {
          if (p_dev->pfn_rx_callback != NULL) {
              p_dev->frames_num++;
              p_dev->pfn_rx_callback(p_dev->p_arg,
                                     p_dev->p_rec_devinfo->frames_info,
                                     p_dev->rx_bytes);
              p_dev->rx_bytes = 0;

              return AM_TRUE;
          }
       }
    return AM_FALSE;
}

/******************************************************************************/
/**
 * \brief CRCУ麯
 */
am_local am_bool_t __xmodem__rec_crc (am_xmodem_rec_dev_t *p_dev, char inchar)
{
    uint8_t u_inchar = (uint8_t)inchar;

    if (p_dev->rx_bytes == p_dev->p_rec_devinfo->frames_bytes) {
        p_dev->fra_crc_pry |= (u_inchar << 8);
        p_dev->rx_bytes ++;
        return AM_TRUE;
    }

    if (p_dev->rx_bytes == p_dev->p_rec_devinfo->frames_bytes + 1) {
        p_dev->fra_crc_pry |= u_inchar;
        p_dev->rx_bytes--;
        if (AM_TRUE == __xmodem_data_check(p_dev)) {
            p_dev->pfn_rx_callback(p_dev->p_arg,
                                   p_dev->p_rec_devinfo->frames_info,
                                   p_dev->p_rec_devinfo->frames_bytes);
            p_dev->rx_bytes = 0;
            p_dev->frames_num++;
            p_dev->fra_crc_pry = 0;
            return AM_TRUE;
        }
    }

    return AM_FALSE;
}

/*******************************************************************************
      ״̬
*******************************************************************************/
/**
 * \brief ݶα溯
 */
am_local am_bool_t __xmodem_rec_data_rec (am_xmodem_rec_dev_t *p_dev, char inchar)
{
    /* ݽ*/
    if (p_dev->rx_bytes < p_dev->p_rec_devinfo->frames_bytes) {
        p_dev->p_rec_devinfo->frames_info[p_dev->rx_bytes] = inchar;
        p_dev->rx_bytes++;
        return AM_TRUE;
    }
    /* ݽϣsumУ*/
    if (AM_XMODEM_SUM_MODE == p_dev->p_rec_devinfo->parity_mode) {
        return __xmodem_rec_sum(p_dev, inchar);

    }
    /* ݽϣcrcУ*/
    if (AM_XMODEM_CRC_MODE == p_dev->p_rec_devinfo->parity_mode) {
        return __xmodem__rec_crc(p_dev, inchar);
    }
    return AM_FALSE;
}

/**
 * \brief кŷж
 */
am_local am_bool_t __xmodem_rec_rad (am_xmodem_rec_dev_t *p_dev, char inchar)
{
    if (inchar == ~ (char)p_dev->frames_num) {
        /* ĵһ״̬*/
        p_dev->p_rec_func = (pfn_xmodem_rx_t)__xmodem_rec_data_rec;
        return AM_TRUE;
    }

    return AM_FALSE;
}

/**
 * \brief кŽܺ
 */
am_local am_bool_t __xmodem_rec_pack (am_xmodem_rec_dev_t *p_dev, char inchar)
{
    if (p_dev->frames_num != (uint8_t)inchar) {
        return AM_FALSE;
    }
    p_dev->p_rec_func = (pfn_xmodem_rx_t)__xmodem_rec_rad;

    return AM_TRUE;
}

/**
 * \brief ״̬ʼ亯
 */
am_local am_bool_t am_xmodem_rec_frames (am_xmodem_rec_dev_t *p_dev, char inchar)
{
    if ((uint8_t)inchar == __AM_XMODEM_SOH ||
        (uint8_t)inchar == __AM_XMODEM_STX) {

        p_dev->con_state = __AM_XMODEM_CON_UP;

        p_dev->p_rec_func = (pfn_xmodem_rx_t)__xmodem_rec_pack;

        return AM_TRUE;
    }
    return AM_FALSE;
}

/*****************************************************************************/
/**
 * \brief ½һ֡
 */
am_local void __xmodem_rx_nak_set (am_xmodem_rec_dev_t  *p_dev)
{
    /* ط */
    ++p_dev->nak_state;

    /* жǷﵽط*/
    if (p_dev->nak_state == p_dev->p_rec_devinfo->nak_max_times) {
        /* Ҫط */
        p_dev->nak_state = AM_OK;
        p_dev->rx_bytes  = -AM_EBADMSG;
        /* δܽܵȷݣûص֪ͨûݴ */
        p_dev->pfn_rx_callback(p_dev->p_arg,
                               p_dev->p_rec_devinfo->frames_info,
                               p_dev->rx_bytes);
        return;
    }
    p_dev->p_rec_func     = (pfn_xmodem_rx_t)am_xmodem_rec_frames;
    p_dev->char_get_state = __AM_XMODEM_CHAR_UP;
    /* ͷNAKطź*/
    p_dev->p_rec_devinfo->frames_info[0] = AM_XMODEM_NAK;

    am_uart_tx_startup(p_dev->uart_handle);
}

/******************************************************************************/
/**
 * \brief ȡͺ
 */
am_err_t am_xmodem_rec_can_set (am_xmodem_rec_handle_t handle)
{
   uint8_t i = 0;

   if (handle == NULL) {
      return -AM_EINVAL;
   }

   /* Xmodem״̬*/
   handle->p_rec_func = (pfn_xmodem_rx_t)am_xmodem_rec_frames;

   /* ȡηֹͷδܽӵ*/
   for (i = 0; i < 3; i++) {
      /* 淢͵ȡ־*/
      handle->p_rec_devinfo->frames_info[0] = AM_XMODEM_CAN;
      handle->char_get_state = __AM_XMODEM_CHAR_UP;
      /* ж */
      am_uart_tx_startup(handle->uart_handle);
      while(handle->char_get_state == __AM_XMODEM_CHAR_UP);
   }

   return AM_OK;
}

/******************************************************************************/
/**
 * \brief պ
 */
am_err_t am_xmodem_rec_ack_set (am_xmodem_rec_handle_t handle)
{
    if (handle == NULL) {
       return -AM_EINVAL;
    }

    /* ط*/
    handle->nak_state      = 0;

    /* ǰXmodemѾ*/
    handle->con_state      = __AM_XMODEM_CON_UP;

    /* ý״̬*/
    handle->p_rec_func     = (pfn_xmodem_rx_t)am_xmodem_rec_frames;

    /* ǰΪ״̬̬*/
    handle->char_get_state = __AM_XMODEM_CHAR_UP;

    /* ACKȷźţ Ӧͷһ֡ */
    handle->p_rec_devinfo->frames_info[0] = AM_XMODEM_ACK;

    /* ж */
    am_uart_tx_startup(handle->uart_handle);

    /* ʱȴXmodemλĻӦ*/
    am_softimer_start(&handle->rx_softimer, \
                       handle->p_rec_devinfo->rx_timeout);

    return AM_OK;
}

/******************************************************************************/
/**
 * \brief xmodemջصע
 */
am_err_t am_xmodem_rec_cb_reg (am_xmodem_rec_handle_t handle,
                                am_xmodem_user_rx_t   pfn_rec_cb,
                                void                 *p_arg)
{
    if (pfn_rec_cb == NULL || pfn_rec_cb == NULL) {
       return -AM_EINVAL;
    }

    handle->p_arg = p_arg;

    handle->pfn_rx_callback = pfn_rec_cb;

    return AM_OK;
}

/******************************************************************************/
/**
 * \brief ʱص
 *
 * \param[in] p_arg ָXMODEM 豸ṹָ
 *
 * \return 
 */
am_local void __xmodem_rec_softimer_callback (void *p_arg)
{
    am_xmodem_rec_dev_t *p_dev = (am_xmodem_rec_dev_t *)p_arg;
    /* رʱ */
    am_softimer_stop(&p_dev->rx_softimer);
    //δܳʱûӣ
    if (p_dev->con_state == __AM_XMODEM_CON_DOWN) {
        ++p_dev->nak_state;
        if (p_dev->nak_state == p_dev->p_rec_devinfo->nak_max_times) {
            p_dev->nak_state = 0;
            p_dev->pfn_rx_callback(p_dev->p_arg,
                                   p_dev->p_rec_devinfo->frames_info,
                                   -AM_XMODEM_DOWN);
            return;
        }
        am_xmodem_rec_start(p_dev);
        return;
    }
    /*־λʱδյ*/
    p_dev->rx_bytes = -AM_ETIME;
    if (p_dev->pfn_rx_callback != NULL) {
        p_dev->pfn_rx_callback(p_dev->p_arg,
                               p_dev->p_rec_devinfo->frames_info,
                               p_dev->rx_bytes);
    }
}

/******************************************************************************/
/**
 * \brief ַȡ
 */
am_local int __xmodem_rec_getchar (am_xmodem_rec_dev_t  *p_dev, char *p_data)
{
    /* ״̬ʱֻĵһֽе */
    if (p_dev->char_get_state == __AM_XMODEM_CHAR_UP) {

        /* ȡҪ͵ĵһַ*/
       *p_data = p_dev->p_rec_devinfo->frames_info[0];
        /* һκ󣬽ȡַ*/
        p_dev->char_get_state = __AM_XMODEM_CHAR_DOWN;

        return AM_TRUE;
    }
    return AM_OK;
}

/******************************************************************************/
/**
 * \brief Xmodemȡһַͺ
 */
am_local int __xmodem_rec_txchar_get (void *p_arg, char *p_outchar)
{
    am_xmodem_rec_dev_t *p_dev = (am_xmodem_rec_dev_t *)p_arg;
    if (__xmodem_rec_getchar(p_dev, p_outchar) != AM_TRUE) {
        return -AM_EEMPTY;
    }
    return AM_OK;
}

/******************************************************************************/
/**
 * \brief ݴ״̬
 */
am_local am_bool_t __xmodem_rec_data_err (am_xmodem_rec_dev_t *p_dev,
                                          char                 inchar)
{
    int ret = 0;
    /* жϴϢͷȡͣص֪ͨû*/
    if (inchar == AM_XMODEM_CAN) {
        p_dev->rx_bytes++;
        if (p_dev->rx_bytes > 2) {
            __xmodem_rec_can(p_dev, inchar);
        }
        return AM_TRUE;
    }
    if (inchar == __AM_XMODEM_EOT &&
        p_dev->p_rec_func == (pfn_xmodem_rx_t)am_xmodem_rec_frames) {

        return __xmodem_rec_eot(p_dev, inchar);
    }
    if (inchar == AM_XMODEM_ACK) {
        return AM_TRUE;
    }
    if (p_dev->rx_bytes == p_dev->p_rec_devinfo->frames_bytes) {
        ret = -AM_XMODEM_CHECK_ERR;
        if (p_dev->pfn_rx_callback != NULL) {
            p_dev->pfn_rx_callback(p_dev->p_arg,
                                   p_dev->p_rec_devinfo->frames_info,
                                   ret);
        }
        return AM_TRUE;
    } else if (p_dev->p_rec_func == (pfn_xmodem_rx_t)__xmodem_rec_data_rec) {
        __xmodem_rx_nak_set(p_dev);
        return AM_TRUE;
    }
    //ϴκδֱӷ
    return AM_FALSE;
}

/******************************************************************************/
/**
 * \brief Xmodemݽպ
 */
am_local void __xmodem_rx_char (void *p_arg, char inchar)
{
    am_xmodem_rec_dev_t *p_dev = (am_xmodem_rec_dev_t *)p_arg;

    //ȡǰ״̬
    pfn_xmodem_rx_t pfn_rx_func = (pfn_xmodem_rx_t)p_dev->p_rec_func;

    //ʧݴطȡ
    if (AM_FALSE == pfn_rx_func(p_dev, inchar)) {
        if (AM_TRUE == __xmodem_rec_data_err(p_dev, inchar)) {
            am_softimer_stop (&p_dev->rx_softimer);
        }
    }else {
        am_softimer_stop (&p_dev->rx_softimer);
    }
}

/******************************************************************************/
/**
 * \brief Xmodemʼպ
 */
am_local void  __xmodem_rx_startup (am_xmodem_rec_dev_t  *p_dev)
{
    /* ݹģʽѡ͸ͷַ*/
    switch (p_dev->p_rec_devinfo->parity_mode) {
       /* ģʽΪλΪ128ֽʱַNAK*/
       case AM_XMODEM_SUM_MODE:
                 p_dev->p_rec_devinfo->frames_info[0] = AM_XMODEM_NAK;
                 break;
       /* ģʽΪλΪ1KֽʱַC*/
       case AM_XMODEM_CRC_MODE:
                 p_dev->p_rec_devinfo->frames_info[0] = 'C';
                 break;
    }
    /* ַ*/
    p_dev->char_get_state = __AM_XMODEM_CHAR_UP;
    /* 򿪷ж*/
    am_uart_tx_startup(p_dev->uart_handle);
    /* 򿪶ʱ*/
    am_softimer_start(&p_dev->rx_softimer, p_dev->p_rec_devinfo->start_time);
}

/******************************************************************************/
/**
 * \brief Xmodemļպ
 */
am_err_t am_xmodem_rec_start (am_xmodem_rec_handle_t handle)
{
    /* ʼ */
    if (handle != NULL) {
        __xmodem_rx_startup(handle);
        return AM_OK;
    }
    return -AM_EINVAL;
}

/******************************************************************************/
/**
 * \brief CRC-16
 */
static void __crc_16_pattern_set(am_crc_handle_t   crc_handle,
                                 am_crc_pattern_t *p_crc_pattern)
{
    /*  CRC-16 ģ */
    p_crc_pattern->width     = 16;       /* CRC (磺CRC5 ĿΪ 5) */
    p_crc_pattern->poly      = 0x1021;   /* CRC ɶʽ */
    p_crc_pattern->initvalue = 0x0000;   /* CRC ʼֵ */
    p_crc_pattern->refin     = AM_FALSE; /* ҪλתֵΪ TRUE */
    p_crc_pattern->refout    = AM_FALSE; /* ֵҪλתֵΪ TRUE */
    p_crc_pattern->xorout    = 0x0000;   /* ֵ */

    /* ʼCRC*/
    am_crc_init(crc_handle, p_crc_pattern);
}

/**
 * \brief 豸ʼ
 */
am_xmodem_rec_handle_t  am_xmodem_rec_init (
                            am_xmodem_rec_dev_t            *p_dev,
                            const am_xmodem_rec_dev_info_t *p_rec_devinfo,
                            am_uart_handle_t                uart_handle,
                            am_crc_handle_t                 crc_handle)
{
    if ((NULL == p_dev) || (NULL == p_rec_devinfo) || (NULL == uart_handle)) {
        return NULL;
    }
    p_dev->p_rec_devinfo = (am_xmodem_rec_dev_info_t *)p_rec_devinfo;
    /* ֡кųʼֵΪ1*/
    p_dev->frames_num      = 1;
    /* ʼۻУλΪ0 */
    p_dev->fra_sum_parity  = 0;
    /* ʼCRCУ */
    p_dev->fra_crc_pry     = 0;
    /* ʼصΪNULL*/
    p_dev->pfn_rx_callback = NULL;
    /* ʼǰֽ */
    p_dev->rx_bytes        = 0;
    /* ʼǰط״̬Ϊ0 */
    p_dev->nak_state       = 0;
    /* Xmodemǰ״̬*/
    p_dev->con_state       = __AM_XMODEM_CON_DOWN;
    /* XmodemҪĴھ*/
    p_dev->uart_handle     = uart_handle;
    /* ׼CRC*/
    p_dev->crc_handle      = crc_handle;
    /* ַȡĬΪ0 */
    p_dev->char_get_state  = __AM_XMODEM_CHAR_DOWN;
    /** ״̬*/
    p_dev->p_rec_func      = (pfn_xmodem_rx_t)am_xmodem_rec_frames;

    __crc_16_pattern_set(crc_handle, &p_dev->crc_pattern);

    /* ʹܴжģʽ */
    am_uart_ioctl(p_dev->uart_handle,
                  AM_UART_MODE_SET,
                  (void *)AM_UART_MODE_INT);
    /* עᷢͻص */
    am_uart_callback_set(p_dev->uart_handle,
                         AM_UART_CALLBACK_TXCHAR_GET,
                         __xmodem_rec_txchar_get,
                         (void *)(p_dev));
    /* עջص */
    am_uart_callback_set(p_dev->uart_handle,
                         AM_UART_CALLBACK_RXCHAR_PUT,
                         __xmodem_rx_char,
                         (void *)(p_dev));

    if (p_rec_devinfo->rx_timeout != 0) {
        if (am_softimer_init(&p_dev->rx_softimer,
                              __xmodem_rec_softimer_callback,
                              p_dev) != AM_OK) {
            return NULL;
        }
    }
    return (am_xmodem_rec_handle_t)(p_dev);
}

/********************************************************************************
Xmodem
********************************************************************************/

/**
 * \brief ȡļͽַ
 */
am_local void __xmodem_tx_eot (am_xmodem_tx_dev_t *p_dev,
                               uint8_t            *p_outchar)
{
     p_dev->tx_state = __AM_XMODEM_FLAG_TX;
    *p_outchar = __AM_XMODEM_EOT;
     p_dev->state_flag  = __AM_XMODEM_DOC_EOT;
}

/******************************************************************************/
/**
 * \brief ļͽ
 */
am_err_t am_xmodem_tx_over (am_xmodem_tx_handle_t handle)
{
    if (handle == NULL) {
       return -AM_EINVAL;
    }
    /* ȡַͱ־λ*/
    handle->tx_state = __AM_XMODEM_FLAG_TX;
    /* ʱ*/
    am_softimer_start(&handle->tx_softimer,
                       handle->p_tx_devinfo->tx_timeout);
    /* лļͽ״̬*/
    handle->p_tx_func = (pfn_xmodem_tx_t)__xmodem_tx_eot;
    /* ж */
    am_uart_tx_startup(handle->uart_handle);
    /* ʱ*/
    am_softimer_start(&handle->tx_softimer,
                       handle->p_tx_devinfo->tx_timeout);
    return AM_OK;
}

/******************************************************************************/
/**
 * \brief ȡȡͺַ
 */
am_local am_bool_t __xmodem_tx_can_get (am_xmodem_tx_handle_t handle,
                                        char                 *p_outchar)
{
    if (handle->tx_bytes < 3) {
        *p_outchar = AM_XMODEM_CAN;
         handle->tx_bytes++;
         return AM_TRUE;
    }
    handle->tx_bytes = 0;
    handle->tx_state = __AM_XMODEM_STOP_TX;
    return AM_FALSE;
}

/******************************************************************************/
/**
 * \brief ûȡͺ
 */
am_err_t am_xmodem_tx_can_set (am_xmodem_tx_handle_t handle)
{
    if (handle == NULL) {
        return -AM_EINVAL;
    }
    handle->tx_bytes = 0;
    handle->tx_state = __AM_XMODEM_DATA_TX;
    /* лȡļ״̬νȡź*/
    handle->p_tx_func = (pfn_xmodem_tx_t)__xmodem_tx_can_get;
    am_uart_tx_startup(handle->uart_handle);
    handle->state_flag = __AM_XMODEM_EOT_ACK;

    return AM_OK;
}

/******************************************************************************/
/**
 * \brief ַȡ
 */
am_local int __xmodem_tx_getchar (am_xmodem_tx_dev_t  *p_dev, char *p_data)
{
    pfn_xmodem_tx_t pfn_tx_func = (pfn_xmodem_tx_t)p_dev->p_tx_func;
    if (p_dev->tx_state != __AM_XMODEM_STOP_TX) {
        pfn_tx_func(p_dev, p_data);
        if (p_dev->tx_state == __AM_XMODEM_FLAG_TX) {
            p_dev->tx_state = __AM_XMODEM_STOP_TX;
        }
        return AM_TRUE;
    }
    return AM_OK;
}

/******************************************************************************/
/**
 * \brief Xmodemģʽȡһַͺ
 */
am_local int __xmodem_tx_char_get (void *p_arg, char *p_outchar)
{
    am_xmodem_tx_dev_t *p_dev = (am_xmodem_tx_dev_t *)p_arg;
    if (__xmodem_tx_getchar(p_dev, p_outchar) != AM_TRUE) {
        return -AM_EEMPTY;
    }
    return AM_OK;
}

/******************************************************************************/
/**
 * \brief xmodemͻصע
 */
am_err_t am_xmodem_tx_cb_reg (am_xmodem_tx_handle_t handle,
                              am_xmodem_user_tx_t   pfn_tx_cb,
                              void                 *p_arg)
{
    if (pfn_tx_cb == NULL || pfn_tx_cb == NULL) {
       return -AM_EINVAL;
    }

    handle->p_arg           = p_arg;
    handle->pfn_tx_callback = pfn_tx_cb;

    return AM_OK;
}

/**
 * \brief CRCУ뺯
 */
am_local uint16_t __xmodem_tx_crc_set (am_xmodem_tx_dev_t *p_dev,
                                       char               *ptr,
                                       uint32_t            count)
{
    uint8_t  i           = 0;
    uint8_t  data        = 0x1A;
    uint32_t crc         = 0;
    uint32_t ctrlz_count = 0;

    //׼CRC
    am_crc_cal(p_dev->crc_handle,
               (uint8_t *)ptr,
               p_dev->doc_bytes);

    if (p_dev->doc_bytes < p_dev->frame_tx_bytes) {
        ctrlz_count = p_dev->frame_tx_bytes - p_dev->doc_bytes;
        for (i = 0; i < ctrlz_count; i++) {
            am_crc_cal(p_dev->crc_handle,
                       &data,
                       1);
        }
    }

    am_crc_final(p_dev->crc_handle, &crc);

    return (uint16_t)(crc);
}

/*******************************************************************************
״̬
*******************************************************************************/
/**
 * \brief У뺯
 */
am_local void __xmodem_tx_frames_parity (am_xmodem_tx_dev_t *p_dev,
                                         char               *p_outchar)
{
    uint16_t crc_high = 0;
    /* ģʽΪ1KȡCRCУ*/
    if (p_dev->frame_tx_bytes == 1024) {
        crc_high = __xmodem_tx_crc_set(p_dev,
                                       (char *)p_dev->p_tx_buf,
                                        p_dev->doc_bytes);
        if (p_dev->fra_crc_pry == crc_high >> 8) {
            p_dev->fra_crc_pry = crc_high;
            p_dev->tx_state = __AM_XMODEM_STOP_TX;
        } else {
            p_dev->fra_crc_pry = crc_high >> 8;
        }
       *p_outchar = p_dev->fra_crc_pry;
    }
    /* ģʽΪ128ȡSUMУ*/
    if (p_dev->frame_tx_bytes == 128) {
        int i;
        char cks = 0;
        /* ۼ*/
        for (i = 0; i < p_dev->doc_bytes; i++)
        {
            cks += p_dev->p_tx_buf[i];
        }
        if (p_dev->ctrlz_num != 0) {
            for (i = 0; i < p_dev->ctrlz_num; i++) {
                cks += __AM_XMODEM_CTRLZ;
            }
            p_dev->ctrlz_num = 0;
        }
        p_dev->fra_sum_parity = cks;
       *p_outchar = p_dev->fra_sum_parity;
        p_dev->tx_state = __AM_XMODEM_STOP_TX;
    }
}

/******************************************************************************/
/**
 * \brief ݲһ֡亯
 */
am_local am_bool_t __xmodem_tx_ctrlz_set (am_xmodem_tx_dev_t *p_dev,
                                          char               *p_outchar)
{
   *p_outchar = __AM_XMODEM_CTRLZ;
    p_dev->tx_bytes++;
    if (p_dev->tx_bytes == p_dev->frame_tx_bytes) {
        p_dev->tx_bytes   = 0;
        p_dev->state_flag = AM_XMODEM_MOU_SUC;
        p_dev->p_tx_func  = (pfn_xmodem_tx_t)__xmodem_tx_frames_parity;
        return AM_TRUE;
     }
    return AM_FALSE;
}

/******************************************************************************/
/**
 * \brief ݶκ
 */
am_local am_bool_t __xmodem_tx_frames_data (am_xmodem_tx_dev_t *p_dev,
                                            char               *p_outchar)
{
    if (p_dev->doc_bytes == p_dev->frame_tx_bytes) {
       *p_outchar = p_dev->p_tx_buf[p_dev->tx_bytes];
        p_dev->tx_bytes++;
        if (p_dev->tx_bytes == p_dev->frame_tx_bytes) {
            p_dev->tx_bytes  = 0;
            p_dev->p_tx_func = (pfn_xmodem_tx_t)__xmodem_tx_frames_parity;
        }
        return AM_TRUE;
    }
    if (p_dev->doc_bytes < p_dev->frame_tx_bytes ) {
        if (p_dev->tx_bytes < p_dev->doc_bytes) {
           *p_outchar = p_dev->p_tx_buf[p_dev->tx_bytes];
            p_dev->tx_bytes++;
            if (p_dev->tx_bytes == p_dev->doc_bytes) {
                p_dev->p_tx_func  = (pfn_xmodem_tx_t)__xmodem_tx_ctrlz_set;
            }
            return AM_TRUE;
        }
    }
    return AM_FALSE;
}

/******************************************************************************/
/**
 * \brief кŷ뺯
 */
am_local am_bool_t __xmodem_tx_frames_pack_rmoc (am_xmodem_tx_dev_t *p_dev,
                                                 char               *p_outchar)
{
    /* жϷͺȡкŷ*/
   *p_outchar = ~p_dev->frames_num;

    /* лݷ״̬*/
    p_dev->p_tx_func = (pfn_xmodem_tx_t)__xmodem_tx_frames_data;

    return AM_TRUE;
}

/******************************************************************************/
/**
 * \brief кź
 */
am_local am_bool_t  __xomdem_tx_frames_packetno (am_xmodem_tx_dev_t *p_dev,
                                                 char               *p_outchar)
{
    /* кż1*/
    p_dev->frames_num++;

    /* жϷͺȡк*/
   *p_outchar = p_dev->frames_num;

    /* лȡкŷ״̬*/
    p_dev->p_tx_func = (pfn_xmodem_tx_t)__xmodem_tx_frames_pack_rmoc;

    return AM_TRUE;
}

/******************************************************************************/
/**
 * \brief ֡ͷ
 */
am_local am_bool_t __xmodem_tx_frames_head (am_xmodem_tx_dev_t *p_dev,
                                            char               *p_outchar)
{
    if (p_dev->frame_tx_bytes == 1024) {
       /* 1Kģʽ֡ͷΪSTX*/
       *p_outchar = __AM_XMODEM_STX;
    }
    if (p_dev->frame_tx_bytes == 128) {
        /* 128ģʽ֡ͷΪSOH*/
       *p_outchar = __AM_XMODEM_SOH;
    }
    /* лкŷ״̬*/
    p_dev->p_tx_func = (pfn_xmodem_tx_t)__xomdem_tx_frames_packetno;

    return AM_TRUE;
}

/******************************************************************************/
/**
 * \brief ʼͺ
 */
am_err_t am_xmodem_tx_pack (am_xmodem_tx_handle_t handle,
                            char                 *p_doc,
                            uint32_t              pack_size)
{
    if (handle == NULL || p_doc == NULL) {
        return -AM_EINVAL;
    }
    if (handle->state_flag != __AM_XMODEM_TX_START) {
        handle->state_flag = __AM_XMODEM_TX_START;
    }
    if (pack_size < handle->frame_tx_bytes) {
        handle->ctrlz_num = handle->frame_tx_bytes - pack_size;
    }
    /* ȡļָ*/
    handle->p_tx_buf  = p_doc;
    /* һη͵ģС*/
    handle->doc_bytes = pack_size;
    /* ȡַ*/
    handle->tx_state  = __AM_XMODEM_DATA_TX;
    /* л֡ͷ״̬*/
    handle->p_tx_func = (pfn_xmodem_tx_t)__xmodem_tx_frames_head;
    /* 򿪷ж*/
    am_uart_tx_startup(handle->uart_handle);
    /* ʱ*/
    am_softimer_start(&handle->tx_softimer,
                       handle->p_tx_devinfo->tx_timeout);
    return AM_OK;
}

/******************************************************************************/
am_local void __xmodem_tx_eot_ack_char (am_xmodem_tx_dev_t *p_dev,
                                        char               *p_outchar)
{
     *p_outchar = AM_XMODEM_ACK;
}

/******************************************************************************/
am_local int __xmodem_tx_eot_ack (am_xmodem_tx_dev_t *p_dev)
{
    if (p_dev == NULL) {
        return -AM_EINVAL;
    }
    p_dev->tx_state   = __AM_XMODEM_FLAG_TX;
    p_dev->state_flag = __AM_XMODEM_EOT_ACK;
    p_dev->p_tx_func  = (pfn_xmodem_tx_t)__xmodem_tx_eot_ack_char;
    am_uart_tx_startup(p_dev->uart_handle);
    return AM_TRUE;
}

/******************************************************************************/
/**
 * \brief շҪط
 */
am_local void __xmodem_tx_nak (am_xmodem_tx_dev_t *p_dev)
{
    if (p_dev->nake_state < p_dev->p_tx_devinfo->nak_times_max) {
        p_dev->tx_bytes   = 0;
        p_dev->tx_state   = __AM_XMODEM_FLAG_TX;
        p_dev->frames_num = p_dev->frames_num - 1;
        p_dev->p_tx_func  = (pfn_xmodem_tx_t)__xmodem_tx_frames_head;
        am_uart_tx_startup(p_dev->uart_handle);
    }
    if (p_dev->nake_state == p_dev->p_tx_devinfo->nak_times_max) {
        am_xmodem_tx_can_set(p_dev);
        p_dev->nake_state = 0;
        /* ֪ͨûǰݰطﵽ涨ط, Ѿȡ˷*/
        p_dev->pfn_tx_callback(p_dev->p_arg, AM_XMODEM_NAK_TIME);
    }
    p_dev->nake_state++;
}

/******************************************************************************/
/**
 * \brief ӦźжϺ
 */
am_local int __xmodem_tx_result_mode (am_xmodem_tx_dev_t *p_dev,
                                      char                inchar)
{
    if (p_dev->state_flag == __AM_XMODEM_TX_READY) {
        p_dev->state_flag = __AM_XMODEM_TX_START;
        if (inchar == AM_XMODEM_1k) {
            p_dev->frame_tx_bytes = 1024;
        }
        if (inchar == AM_XMODEM_NAK) {
            p_dev->frame_tx_bytes = 128;
        }
        if (p_dev->frame_tx_bytes == 128 || p_dev->frame_tx_bytes == 1024) {
           p_dev->tx_state = __AM_XMODEM_FLAG_TX;
           p_dev->pfn_tx_callback(p_dev->p_arg, inchar);
           return AM_TRUE;
        }
    }
    if (p_dev->state_flag == __AM_XMODEM_TX_START) {
        if (inchar == AM_XMODEM_ACK) {
            p_dev->pfn_tx_callback(p_dev->p_arg, inchar);
        }
        if (inchar == AM_XMODEM_NAK) {
            __xmodem_tx_nak(p_dev);
        }
        if (inchar == AM_XMODEM_CAN) {
           p_dev->pfn_tx_callback(p_dev->p_arg, inchar);
        }
        return AM_TRUE;
    }
    if (p_dev->state_flag == AM_XMODEM_MOU_SUC) {
        p_dev->pfn_tx_callback(p_dev->p_arg, AM_XMODEM_MOU_SUC);
    }
    return AM_FALSE;
}

/******************************************************************************/
/**
 * \brief ģ鷢߽շ׼÷֪ͨû
 */
am_local int __xmodem_tx_user_inform (am_xmodem_tx_dev_t *p_dev,
                                      char                inchar)
{
    /*ǰ״̬ΪļͽʱյӦźŽԶظ*/
    if (p_dev->state_flag == __AM_XMODEM_DOC_EOT) {
        return __xmodem_tx_eot_ack(p_dev);
    }
    if (p_dev->tx_bytes == 0 ) {
        p_dev->tx_state = __AM_XMODEM_FLAG_TX;
        return __xmodem_tx_result_mode(p_dev, inchar);
    }
    return AM_FALSE;
}

/******************************************************************************/
/**
 * \brief жϽַ
 */
am_local void __xmodem_tx_char (void *p_arg, uint8_t inchar)
{
    am_xmodem_tx_dev_t *p_dev = (am_xmodem_tx_dev_t *)p_arg;

    am_softimer_stop(&p_dev->tx_softimer);

    if (AM_FALSE == __xmodem_tx_user_inform(p_dev, inchar)) {
        return;
    }
}

/******************************************************************************/
/**
 * \brief Xmodemͳʱ
 */
am_local void __xmodem_tx_time_callback(void *p_arg)
{
    am_xmodem_tx_dev_t *p_dev = (am_xmodem_tx_dev_t *)p_arg;
    /* رʱ */
    am_softimer_stop(&p_dev->tx_softimer);
    if (p_dev->pfn_tx_callback != NULL) {
        am_xmodem_tx_can_set(p_dev);
        p_dev->pfn_tx_callback(p_dev->p_arg, -AM_ETIME);
     }
}

/******************************************************************************/
/**
 * \brief 豸ʼ
 */
am_xmodem_tx_handle_t  am_xmodem_tx_init (
                           am_xmodem_tx_dev_t            *p_dev,
                           const am_xmodem_tx_dev_info_t *p_tx_devinfo,
                           am_uart_handle_t               uart_handle,
                           am_crc_handle_t                crc_handle)
{
    if ((NULL == p_dev) || (NULL == p_tx_devinfo) || (NULL == uart_handle)) {
        return NULL;
    }

    p_dev->ctrlz_num    = 0;    /**< \brief ʼ*/
    p_dev->tx_bytes     = 0;    /**< \brief ʼļС*/
    p_dev->p_tx_buf     = NULL; /**< \brief ʼļָ */
    p_dev->state_flag   = 0;    /**< \brief ĬϵһյַΪģʽжַ */
    p_dev->nake_state   = 0;    /**< \brief ʼط״̬Ϊ0*/
    p_dev->p_tx_func    = NULL; /**< \brief ʼ״ָ̬ */
    p_dev->crc_handle   = crc_handle;   /**< \brief CRC*/
    p_dev->uart_handle  = uart_handle;  /**< \brief ھ*/
    p_dev->tx_state     = __AM_XMODEM_DATA_TX;
    p_dev->p_tx_devinfo = (am_xmodem_tx_dev_info_t *)p_tx_devinfo;

    /* ʼCRCģ*/
    __crc_16_pattern_set(crc_handle, &p_dev->crc_pattern);

    /* ʹܴжģʽ */
    am_uart_ioctl(p_dev->uart_handle,
                  AM_UART_MODE_SET,
                  (void *)AM_UART_MODE_INT);

    am_uart_callback_set(p_dev->uart_handle,
                         AM_UART_CALLBACK_RXCHAR_PUT,
                         __xmodem_tx_char,
                         (void *)(p_dev));

    /* עᷢͻص */
    am_uart_callback_set(p_dev->uart_handle,
                         AM_UART_CALLBACK_TXCHAR_GET,
                         __xmodem_tx_char_get,
                         (void *)(p_dev));

    if (p_tx_devinfo->tx_timeout != 0) {
        if (am_softimer_init(&p_dev->tx_softimer,
                             __xmodem_tx_time_callback,
                             (void *)p_dev) != AM_OK) {
            return NULL;
        }
    }
    return (am_xmodem_tx_handle_t)(p_dev);
}

