/*******************************************************************************
*                                 AMetal
*                       ----------------------------
*                       innovating embedded platform
*
* Copyright (c) 2001-2018 Guangzhou ZHIYUAN Electronics Co., Ltd.
* All rights reserved.
*
* Contact information:
* web site:    http://www.zlg.cn/
*******************************************************************************/

/**
 * \file
 * \brief WAIT implementation
 * 
 * \internal
 * \par Modification history
 * - 1.00 15-09-01  tee, first implementation.
 * \endinternal
 */
#include "am_wait.h"


/******************************************************************************/

/* ʹֵϢ */
#define __WAIT_VAL_INIT     0   /* ʼֵΪ0                    */
#define __WAIT_VAL_DONE     1   /* ʹam_wait_done()ֵΪ 1 */
#define __WAIT_VAL_TIMEOUT  2   /* ʱʱֵΪ2             */

/* ״̬Աʾǰȴṹ״̬ */
#define __WAIT_STAT_INIT             0   /* ʼ״̬            */
#define __WAIT_STAT_WAIT_ON          1   /* ȴ״̬            */
#define __WAIT_STAT_WAIT_ON_TIMEOUT  2   /* ȴ״̬ʱ   */

/******************************************************************************/

static void __timer_callback (void *p_arg)
{
    am_wait_t *p_wait = (am_wait_t *)p_arg;
    
    p_wait->val = __WAIT_VAL_TIMEOUT;
    
    am_softimer_stop(&p_wait->timer);
}

/******************************************************************************/

/* ȴźųʼ */
int am_wait_init (am_wait_t *p_wait)
{
    if (p_wait == NULL) {
        return -AM_EINVAL;
    }
    
    p_wait->val   = __WAIT_VAL_INIT;
    p_wait->stat  = __WAIT_STAT_INIT;
    
    am_softimer_init(&p_wait->timer,__timer_callback, p_wait);
    
    return AM_OK;
}

int am_wait_on (am_wait_t *p_wait)
{
    if (p_wait == NULL) {
        return -AM_EINVAL;
    }
    
    p_wait->stat = __WAIT_STAT_WAIT_ON;
    
    while(p_wait->val == __WAIT_VAL_INIT);
    
    p_wait->val  = __WAIT_VAL_INIT;
    p_wait->stat = __WAIT_STAT_INIT;
    
    return AM_OK;
}

int am_wait_done(am_wait_t *p_wait)
{
    if (p_wait == NULL) {
        return -AM_EINVAL;
    }
    
    p_wait->val  = __WAIT_VAL_DONE;

    return AM_OK;
}

int am_wait_on_timeout(am_wait_t *p_wait, uint32_t timeout_ms)
{
    int ret;
    
    /* ʼʱ */
    am_softimer_start(&p_wait->timer,timeout_ms);
    
    p_wait->stat = __WAIT_STAT_WAIT_ON_TIMEOUT;
    
    while(p_wait->val == __WAIT_VAL_INIT);
    
    if (p_wait->val == __WAIT_VAL_DONE) {  /* ɹȵź */
        ret = AM_OK;
    } else {                               /* ʱź     */
        ret = -AM_ETIME;
    }
    
    p_wait->val  = __WAIT_VAL_INIT;
    p_wait->stat = __WAIT_STAT_INIT;       /* صʼ̬   */
    
    am_softimer_stop(&p_wait->timer);
    
    return ret;
}

/* end of file */
