/*******************************************************************************
*                                 AMetal
*                       ----------------------------
*                       innovating embedded platform
*
* Copyright (c) 2001-2018 Guangzhou ZHIYUAN Electronics Co., Ltd.
* All rights reserved.
*
* Contact information:
* web site:    http://www.zlg.cn/
*******************************************************************************/

/**
 * \file
 * \brief HC32 ˫Ӧó ûļ
 *
 *
 *
 * \internal
 * \par Modification history
 * - 1.00 18-12-13  yrh, first implementation
 * \endinternal
 */

#include <am_bootconf_usrcfg/am_appconf_double_hc32l19x.h>

#include "am_hc32.h"
#include "am_hc32l19x_inst_init.h"
#include "am_hc32_boot_flash.h"
#include "am_arm_boot.h"
#include "am_boot_flash.h"
#include "am_boot_mem_flash.h"
#include "am_hc32_flash.h"
#include "hc32_regbase.h"
#include "am_boot_firmware_flash.h"
#include "am_int.h"
#include "am_hc32_clk.h"
#include "hw/amhw_hc32_rcc.h"
#include "am_arm_nvic.h"
#include "am_boot_msg_flag.h"

static am_boot_flash_handle_t flash_handle;
static am_boot_mem_handle_t   mem_handle;
/*******************************************************************************
 * flash
 ******************************************************************************/
/**
 * < \brief bootloader flash 豸Ϣ
 */
static am_hc32_boot_flash_devinfo_t __g_flash_devinfo = {

    {
        /** \brief flashʼַ */
        0x00000000,
        /** \brief flashܵĴС */
        256 * 1024,
        /** \brief flashС */
        512,
        /** \brief flashҳС */
        512,
    },

    /** \brief flashĴĻַ */
    HC32_FLASH_BASE,
    /** \brief ƽ̨ʼ */
    NULL,
    /** \brief ƽ̨ʼ */
    NULL,
};

/**
 * < \brief bootloader flash 豸ʵ
 */
static am_hc32_boot_flash_dev_t __g_flash_dev;

/**
 * \brief bootloader flashʵʼ׼
 *
 * \param 
 *
 * \return bootloader flash׼Ϊ NULLʼʧ
 */
am_boot_flash_handle_t am_hc32_boot_flash_inst_init(void)
{
    return am_hc32_boot_flash_init(&__g_flash_dev, &__g_flash_devinfo);
}

/*******************************************************************************
 * flagʽϢ
 ******************************************************************************/

static am_boot_msg_flag_devinfo_t __g_boot_msg_flag_info = {
    /** \brief ̼ʼַ */
    0x00023400,
    /** \brief ӦС */
    115 * 1024,
    /** \brief ־ŵַ */
    0x00006600,
    /** \brief ־ռõʵʴСһΪڴСд뵥Ԫʹflashڴ棬ΪСĲԪ*/
    4,
    /** \brief ̼Ҫӹ̼ıӦóִ(־Ӧóͬһڴ豸УԲʵָú)*/
    NULL,
};

am_boot_msg_handle_t am_hc32_boot_msg_flag_inst_init()
{
    mem_handle = am_boot_mem_flash_init(flash_handle);
    return am_boot_msg_flag_init(&__g_boot_msg_flag_info,
                                  mem_handle);
}

/*******************************************************************************
 * ̼flashķʽ洢
 ******************************************************************************/
static am_boot_firmware_flash_devinfo_t __g_firmware_flash_devinfo = {
    /** \brief ̼ڴдŵʼַ */
    0x00023400,
    /**< \brief ̼ܴС */
    115 * 1024,
    /** \brief Ź̼С ,ֵΪڴдСС */
    4,
    /** \brief ƽ̨ʼ */
    NULL,
    /** \brief ƽ̨ʼ */
    NULL,
};

static am_boot_firmware_flash_dev_t __g_firmware_flash_dev;

am_boot_firmware_handle_t am_hc32_boot_firmware_flash_inst_init()
{
    flash_handle = am_hc32_boot_flash_inst_init();
    return am_boot_firmware_flash_init (&__g_firmware_flash_dev,
                                        &__g_firmware_flash_devinfo,
                                         flash_handle);
}


/*******************************************************************************
 * ׼bootloaderû
 ******************************************************************************/

/* bootloaderõԴҪӳʼͷţ綨ʱڵ */
static void __hc32_boot_plfm_deinit()
{
    volatile uint32_t i = 1000000;
    /* ڽʼ ʼǰҪСεʱ֤ĴѾ*/
    while(i--){

    }

    am_hc32_clk_inst_deinit();

    am_int_disable(INUM_UART0_2);
    am_clk_disable(CLK_UART0);

    am_hc32_clk_reset(CLK_UART0);
    am_hc32_clk_reset(CLK_GPIO);

    am_int_disable(INUM_TIM2);
    am_clk_disable(CLK_TIM012);
    am_hc32_clk_reset(CLK_TIM012);

    HC32_RCCCTRL->peri_clken[0] = 0x80800000;
    HC32_RCCCTRL->peri_clken[1] = 0;

    am_arm_nvic_deinit();

    __disable_irq();
}

/**
 * < \brief bootloader׼豸Ϣ
 */
static am_arm_boot_devinfo_t __g_hc32_boot_devinfo = {
    /**< \brief Ӧôʼַ*/
    0x00006800,
    /** \brief ramʼַ */
    0x20000000,
    /** \brief ramС */
    32 * 1024,
    /** \brief ƽ̨ʼ */
    NULL,
    /** \brief ƽ̨ʼ */
    __hc32_boot_plfm_deinit,
};

int am_hc32_boot_inst_init()
{
    return am_arm_boot_init(&__g_hc32_boot_devinfo);
}

/** @} */

/* end of file */
