/*******************************************************************************
*                                 AMetal
*                       ----------------------------
*                       innovating embedded platform
*
* Copyright (c) 2001-2018 Guangzhou ZHIYUAN Electronics Co., Ltd.
* All rights reserved.
*
* Contact information:
* web site:    http://www.zlg.cn/
*******************************************************************************/

/**
 * \file
 * \brief TIMPWM ûļ
 * \sa am_hwconf_zlg116_tim_pwm.c
 * 
 * \internal
 * \par Modification history
 * - 1.00 17-04-24  nwt, first implementation.
 * \endinternal
 */

#include "ametal.h"
#include "am_zlg116.h"
#include "am_gpio.h"
#include "am_zlg_tim_pwm.h"
#include "am_clk.h"
#include "am_zlg116_clk.h"

/**
 * \addtogroup am_if_src_hwconf_zlg116_tim_pwm
 * \copydoc am_hwconf_zlg116_tim_pwm.c
 * @{
 */

/*******************************************************************************
 * TIM1 
 ******************************************************************************/

/** \brief TIM1PWM ƽ̨ʼ */
static void __zlg_plfm_tim1_pwm_init (void)
{
    /* ʹܸ߼ʱ1ʱ */
    am_clk_enable(CLK_TIM1);

    /* λ߼ʱ1 */
    am_zlg116_clk_reset(CLK_TIM1);
}

/** \brief TIM1ƽ̨ʼ */
static void __zlg_plfm_tim1_pwm_deinit (void)
{
    /* λ߼ʱ1 */
    am_zlg116_clk_reset(CLK_TIM1);

    /* ܸ߼ʱ1ʱ */
    am_clk_disable(CLK_TIM1);
}

/** \brief TIM1PWMܵͨϢб */
static am_zlg_tim_pwm_chaninfo_t __g_tim1_pwm_chaninfo_list[] = {
    /** \brief ͨ1 */
    {AM_ZLG_TIM_PWM_CH1, PIOA_8,  PIOA_8_TIM1_CH1  | PIOA_8_AF_PP,  PIOA_8_GPIO  | PIOA_8_INPUT_FLOAT},
    /** \brief ͨ2 */
    {AM_ZLG_TIM_PWM_CH2, PIOA_9,  PIOA_9_TIM1_CH2  | PIOA_9_AF_PP,  PIOA_9_GPIO  | PIOA_9_INPUT_FLOAT},
    /** \brief ͨ3 */
    {AM_ZLG_TIM_PWM_CH3, PIOA_10, PIOA_10_TIM1_CH3 | PIOA_10_AF_PP, PIOA_10_GPIO | PIOA_10_INPUT_FLOAT},
    /** \brief ͨ4 */
    {AM_ZLG_TIM_PWM_CH4, PIOA_11, PIOA_11_TIM1_CH4 | PIOA_11_AF_PP, PIOA_11_GPIO | PIOA_11_INPUT_FLOAT},
};

/** \brief TIM1PWM豸Ϣ */
static const am_zlg_tim_pwm_devinfo_t  __g_tim1_pwm_devinfo = {
    ZLG116_TIM1_BASE,                          /**< \brief TIM1ĴĻַ    */
    CLK_TIM1,                                  /**< \brief TIM1ʱID              */
    AM_NELEMENTS(__g_tim1_pwm_chaninfo_list),  /**< \brief ͨ        */
    AMHW_ZLG_TIM_PWM_MODE2,                    /**< \brief PWMģʽ2            */
    0,                                         /**< \brief PWMߵƽЧ       */
    &__g_tim1_pwm_chaninfo_list[0],            /**< \brief ͨϢб        */
    AMHW_ZLG_TIM_TYPE0,                        /**< \brief ʱ              */
    __zlg_plfm_tim1_pwm_init,                  /**< \brief ƽ̨ʼ          */
    __zlg_plfm_tim1_pwm_deinit                 /**< \brief ƽ̨ʼ        */
};

/** \brief TIM1PWM豸 */
static am_zlg_tim_pwm_dev_t  __g_tim1_pwm_dev;
static am_pwm_handle_t       __g_tim1_pwm_handle = NULL;

/** \brief tim1 pwm ʵʼpwm׼ */
am_pwm_handle_t am_zlg116_tim1_pwm_inst_init (void)
{
    if (__g_tim1_pwm_handle == NULL) {
        __g_tim1_pwm_handle = am_zlg_tim_pwm_init(&__g_tim1_pwm_dev,
                                                  &__g_tim1_pwm_devinfo);
    }
    
    return __g_tim1_pwm_handle;
}

/** \brief tim1 pwm ʵʼ */
void am_zlg116_tim1_pwm_inst_deinit (am_pwm_handle_t handle)
{
    if (__g_tim1_pwm_handle == handle) {
        am_zlg_tim_pwm_deinit(handle);
        __g_tim1_pwm_handle = NULL;
    }
}

/*******************************************************************************
 * TIM2 
 ******************************************************************************/

/** \brief TIM2PWM ƽ̨ʼ */
static void __zlg_plfm_tim2_pwm_init (void)
{
    /* ʹͨöʱ2ʱ */
    am_clk_enable(CLK_TIM2);

    /* λͨöʱ2 */
    am_zlg116_clk_reset(CLK_TIM2);
}

/** \brief TIM2ƽ̨ʼ */
static void __zlg_plfm_tim2_pwm_deinit (void)
{
    /* λͨöʱ2 */
    am_zlg116_clk_reset(CLK_TIM2);

    /* ͨöʱ2ʱ */
    am_clk_disable(CLK_TIM2);
}

/** \brief TIM2PWMܵͨϢб */
static am_zlg_tim_pwm_chaninfo_t __g_tim2_pwm_chaninfo_list[] = {
        /** \brief ͨ1 */
        {AM_ZLG_TIM_PWM_CH1, PIOA_0, PIOA_0_TIM2_CH1_ETR | PIOA_0_AF_PP, PIOA_0_GPIO | PIOA_0_INPUT_FLOAT},
        /** \brief ͨ2 */
        {AM_ZLG_TIM_PWM_CH2, PIOA_1, PIOA_1_TIM2_CH2 | PIOA_1_AF_PP, PIOA_1_GPIO | PIOA_1_INPUT_FLOAT},
};

/** \brief TIM2PWM豸Ϣ */
static const am_zlg_tim_pwm_devinfo_t  __g_tim2_pwm_devinfo = {
    ZLG116_TIM2_BASE,                          /**< \brief TIM2ĴĻַ    */
    CLK_TIM2,                                  /**< \brief TIM2ʱID              */
    AM_NELEMENTS(__g_tim2_pwm_chaninfo_list),  /**< \brief ͨ        */
    AMHW_ZLG_TIM_PWM_MODE2,                    /**< \brief PWMģʽ2            */
    0,                                         /**< \brief PWMߵƽЧ       */
    &__g_tim2_pwm_chaninfo_list[0],            /**< \brief ͨϢб        */
    AMHW_ZLG_TIM_TYPE1,                        /**< \brief ʱ              */
    __zlg_plfm_tim2_pwm_init,                  /**< \brief ƽ̨ʼ          */
    __zlg_plfm_tim2_pwm_deinit                 /**< \brief ƽ̨ʼ        */
};

/** \brief TIM2 PWM豸 */
static am_zlg_tim_pwm_dev_t  __g_tim2_pwm_dev;
static am_pwm_handle_t       __g_tim2_pwm_handle = NULL;

/** \brief tim2 pwmʵʼpwm׼ */
am_pwm_handle_t am_zlg116_tim2_pwm_inst_init (void)
{
    if (__g_tim2_pwm_handle == NULL) {
        __g_tim2_pwm_handle = am_zlg_tim_pwm_init(&__g_tim2_pwm_dev,
                                                  &__g_tim2_pwm_devinfo);
    }
    
    return __g_tim2_pwm_handle;
}

/** \brief tim2 pwmʵʼ */
void am_zlg116_tim2_pwm_inst_deinit (am_pwm_handle_t handle)
{
    if (__g_tim2_pwm_handle == handle) {
        am_zlg_tim_pwm_deinit(handle);
        __g_tim2_pwm_handle = NULL;
    }
}

/*******************************************************************************
 * TIM3 
 ******************************************************************************/

/** \brief TIM3PWM ƽ̨ʼ */
static void __zlg_plfm_tim3_pwm_init (void)
{
    /* ʹͨöʱ3ʱ */
    am_clk_enable(CLK_TIM3);

    /* λͨöʱ3 */
    am_zlg116_clk_reset(CLK_TIM3);
}

/** \brief TIM3 PWMƽ̨ʼ */
static void __zlg_plfm_tim3_pwm_deinit (void)
{
    /* λͨöʱ3 */
    am_zlg116_clk_reset(CLK_TIM3);

    /* ͨöʱ3ʱ */
    am_clk_disable(CLK_TIM3);
}

/** \brief TIM3PWMܵͨϢб */
static am_zlg_tim_pwm_chaninfo_t __g_tim3_pwm_chaninfo_list[] = {
    /** \brief ͨ1 */
    {AM_ZLG_TIM_PWM_CH1, PIOB_4,  PIOB_4_TIM3_CH1 | PIOB_4_AF_PP, PIOB_4_GPIO | PIOB_4_INPUT_FLOAT},
    /** \brief ͨ2 */
    {AM_ZLG_TIM_PWM_CH2, PIOB_5,  PIOB_5_TIM3_CH2 | PIOB_5_AF_PP, PIOB_5_GPIO | PIOB_5_INPUT_FLOAT},
};

/** \brief TIM3PWM豸Ϣ */
static const am_zlg_tim_pwm_devinfo_t  __g_tim3_pwm_devinfo = {
    ZLG116_TIM3_BASE,                          /**< \brief TIM3ĴĻַ    */
    CLK_TIM3,                                  /**< \brief TIM3ʱID              */
    AM_NELEMENTS(__g_tim3_pwm_chaninfo_list),  /**< \brief ͨ        */
    AMHW_ZLG_TIM_PWM_MODE2,                    /**< \brief PWMģʽ2            */
    0,                                         /**< \brief PWMߵƽЧ       */
    &__g_tim3_pwm_chaninfo_list[0],            /**< \brief ͨϢб        */
    AMHW_ZLG_TIM_TYPE1,                        /**< \brief ʱ              */
    __zlg_plfm_tim3_pwm_init,                  /**< \brief ƽ̨ʼ          */
    __zlg_plfm_tim3_pwm_deinit                 /**< \brief ƽ̨ʼ        */
};

/** \brief TIM3PWM豸 */
static am_zlg_tim_pwm_dev_t  __g_tim3_pwm_dev;
static am_pwm_handle_t       __g_tim3_pwm_handle = NULL;

/** \brief tim3 pwmʵʼPWM׼ */
am_pwm_handle_t am_zlg116_tim3_pwm_inst_init (void)
{
    if (__g_tim3_pwm_handle == NULL) {
        __g_tim3_pwm_handle = am_zlg_tim_pwm_init(&__g_tim3_pwm_dev,
                                                  &__g_tim3_pwm_devinfo);
    }
    return __g_tim3_pwm_handle;
}

/** \brief tim3 pwm ʵʼ */
void am_zlg116_tim3_pwm_inst_deinit (am_pwm_handle_t handle)
{
    if (__g_tim3_pwm_handle == handle) {
        am_zlg_tim_pwm_deinit(handle);
        __g_tim3_pwm_handle = NULL;
    }
}

/*******************************************************************************
 * TIM16
 ******************************************************************************/

/** \brief TIM16PWM ƽ̨ʼ */
static void __zlg_plfm_tim16_pwm_init (void)
{
    /* ʹͨöʱ16ʱ */
    am_clk_enable(CLK_TIM16);

    /* λͨöʱ16 */
    am_zlg116_clk_reset(CLK_TIM16);

}

/** \brief TIM16 PWMƽ̨ʼ */
static void __zlg_plfm_tim16_pwm_deinit (void)
{
    /* λͨöʱ16 */
    am_zlg116_clk_reset(CLK_TIM16);

    /* ͨöʱ16ʱ */
    am_clk_disable(CLK_TIM16);
}


/** \brief TIM16PWMܵͨϢб */
static am_zlg_tim_pwm_chaninfo_t __g_tim16_pwm_chaninfo_list[] = {
    /** \brief ͨ1 */
    {AM_ZLG_TIM_PWM_CH1N, PIOB_6, PIOB_6_TIM16_CH1N | PIOB_6_AF_PP, AM_GPIO_INPUT | AM_GPIO_FLOAT},
};

/** \brief TIM16PWM豸Ϣ */
static const am_zlg_tim_pwm_devinfo_t  __g_tim16_pwm_devinfo = {
    ZLG116_TIM16_BASE,                          /**< \brief TIM16ĴĻַ   */
    CLK_TIM16,                                  /**< \brief TIM16ʱID             */
    AM_NELEMENTS(__g_tim16_pwm_chaninfo_list),  /**< \brief ͨ        */
    AMHW_ZLG_TIM_PWM_MODE2,                     /**< \brief PWMģʽ2            */
    0,                                          /**< \brief PWMߵƽЧ       */
    &__g_tim16_pwm_chaninfo_list[0],            /**< \brief ͨϢб        */
    AMHW_ZLG_TIM_TYPE3,                         /**< \brief ʱ              */
    __zlg_plfm_tim16_pwm_init,                  /**< \brief ƽ̨ʼ          */
    __zlg_plfm_tim16_pwm_deinit                 /**< \brief ƽ̨ʼ        */
};

/** \brief TIM16PWM豸 */
static am_zlg_tim_pwm_dev_t __g_tim16_pwm_dev;
static am_pwm_handle_t      __g_tim16_pwm_handle = NULL;

/** \brief tim16 pwmʵʼPWM׼ */
am_pwm_handle_t am_zlg116_tim16_pwm_inst_init (void)
{
    if (__g_tim16_pwm_handle == NULL) {
        __g_tim16_pwm_handle = am_zlg_tim_pwm_init(&__g_tim16_pwm_dev,
                                                   &__g_tim16_pwm_devinfo);
    }

    return __g_tim16_pwm_handle;
}

/** \brief tim16 pwmʵʼ */
void am_zlg116_tim16_pwm_inst_deinit (am_pwm_handle_t handle)
{
    if (__g_tim16_pwm_handle == handle) {
        am_zlg_tim_pwm_deinit(handle);
        __g_tim16_pwm_handle = NULL;
    }
}

/*******************************************************************************
 * TIM17
 ******************************************************************************/

/** \brief TIM17PWM ƽ̨ʼ */
static void __zlg_plfm_tim17_pwm_init (void)
{
    /* ʹͨöʱ17ʱ */
    am_clk_enable(CLK_TIM17);

    /* λͨöʱ17 */
    am_zlg116_clk_reset(CLK_TIM17);
}

/** \brief TIM17 PWMƽ̨ʼ */
static void __zlg_plfm_tim17_pwm_deinit (void)
{
    /* λͨöʱ17 */
    am_zlg116_clk_reset(CLK_TIM17);

    /* ͨöʱ17ʱ */
    am_clk_disable(CLK_TIM17);
}

/** \brief TIM17PWMܵͨϢб */
static am_zlg_tim_pwm_chaninfo_t __g_tim17_pwm_chaninfo_list[] = {
    /** \brief ͨ1 */
    {AM_ZLG_TIM_PWM_CH1N, PIOB_7, PIOB_7_TIM17_CH1N | PIOB_7_AF_PP, AM_GPIO_INPUT | AM_GPIO_FLOAT},
};

/** \brief TIM17PWM豸Ϣ */
static const am_zlg_tim_pwm_devinfo_t  __g_tim17_pwm_devinfo = {
    ZLG116_TIM17_BASE,                          /**< \brief TIM17ĴĻַ   */
    CLK_TIM17,                                  /**< \brief TIM17ʱID             */
    AM_NELEMENTS(__g_tim17_pwm_chaninfo_list),  /**< \brief ͨ        */
    AMHW_ZLG_TIM_PWM_MODE2,                     /**< \brief PWMģʽ2            */
    0,                                          /**< \brief PWMߵƽЧ       */
    &__g_tim17_pwm_chaninfo_list[0],            /**< \brief ͨϢб        */
    AMHW_ZLG_TIM_TYPE3,                         /**< \brief ʱ              */
    __zlg_plfm_tim17_pwm_init,                  /**< \brief ƽ̨ʼ          */
    __zlg_plfm_tim17_pwm_deinit                 /**< \brief ƽ̨ʼ        */
};

/** \brief TIM17PWM豸 */
static am_zlg_tim_pwm_dev_t  __g_tim17_pwm_dev;
static am_pwm_handle_t       __g_tim17_pwm_handle = NULL;

/** \brief tim17 pwmʵʼPWM׼ */
am_pwm_handle_t am_zlg116_tim17_pwm_inst_init (void)
{
    if (__g_tim17_pwm_handle == NULL) {
        __g_tim17_pwm_handle = am_zlg_tim_pwm_init(&__g_tim17_pwm_dev,
                                                   &__g_tim17_pwm_devinfo);
    }
    return __g_tim17_pwm_handle;
}

/** \brief tim17 pwmʵʼ */
void am_zlg116_tim17_pwm_inst_deinit (am_pwm_handle_t handle)
{
    if (__g_tim17_pwm_handle == handle) {
        am_zlg_tim_pwm_deinit(handle);
        __g_tim17_pwm_handle = NULL;
    }
}

/**
 * @}
 */

/* end of file */
