/*******************************************************************************
*                                 AMetal
*                       ----------------------------
*                       innovating embedded platform
*
* Copyright (c) 2001-2018 Guangzhou ZHIYUAN Electronics Co., Ltd.
* All rights reserved.
*
* Contact information:
* web site:    http://www.zlg.cn/
*******************************************************************************/

/**
 * \file
 * \brief HC32 kboot KinetisFlashTool ûļ
 *
 *
 *
 * \internal
 * \par Modification history
 * - 1.00 18-12-13  yrh, first implementation
 * \endinternal
 */

/**
 * \addtogroup am_if_src_bootconf_hc32_kft
 * \copydoc am_bootconf_hc32_kft.c
 * @{
 */

#include <am_bootconf_usrcfg/am_bootconf_aml166.h>
#include "am_zlg116.h"
#include "am_aml166_inst_init.h"
#include "am_zlg_boot_flash.h"
#include "am_arm_boot.h"
#include "am_boot_flash.h"
#include "am_boot_mem_flash.h"
#include "am_zlg_flash.h"
#include "zlg116_regbase.h"
#include "am_boot_firmware_flash.h"
#include "am_int.h"
#include "am_zlg116_clk.h"
#include "hw/amhw_zlg116_rcc.h"
#include "am_arm_nvic.h"


/*******************************************************************************
 * flash
 ******************************************************************************/
/**
 * < \brief bootloader flash 豸Ϣ
 */
static am_zlg_boot_flash_devinfo_t __g_flash_devinfo = {

    {
        /** \brief flashʼַ */
        0x08000000,
        /** \brief flashܵĴС */
        64 * 1024,
        /** \brief flashС */
        1024,
        /** \brief flashҳС */
        64,
    },

    /** \brief flashĴĻַ */
    ZLG116_FLASH_BASE,
    /** \brief ƽ̨ʼ */
    NULL,
    /** \brief ƽ̨ʼ */
    NULL,
};

/**
 * < \brief bootloader flash 豸ʵ
 */
static am_zlg_boot_flash_dev_t __g_flash_dev;

/**
 * \brief bootloader flashʵʼ׼
 *
 * \param 
 *
 * \return bootloader flash׼Ϊ NULLʼʧ
 */
static am_boot_flash_handle_t am_aml166_boot_single_flash_inst_init(void)
{
    return am_zlg_boot_flash_init(&__g_flash_dev, &__g_flash_devinfo);
}

/*******************************************************************************
 * ̼flashķʽ洢
 ******************************************************************************/
static am_boot_firmware_flash_devinfo_t __g_firmware_flash_devinfo = {
    /** \brief ̼ڴдŵʼַ */
    0x08003C00,
    /**< \brief ӦôܴС */
    49 * 1024,
    /** \brief Ź̼С ,ֵΪڴдСС */
    4,
    /** \brief ƽ̨ʼ */
    NULL,
    /** \brief ƽ̨ʼ */
    NULL,
};

static am_boot_firmware_flash_dev_t __g_firmware_flash_dev;

am_boot_firmware_handle_t am_aml166_boot_single_firmware_flash_inst_init(void)
{
    am_boot_flash_handle_t   flash_handle;
    flash_handle = am_aml166_boot_single_flash_inst_init();
    return am_boot_firmware_flash_init (&__g_firmware_flash_dev,
                                        &__g_firmware_flash_devinfo,
                                         flash_handle);
}

/*******************************************************************************
 * ׼bootloaderӿ
 ******************************************************************************/

/* bootloaderõԴҪӳʼͷţ綨ʱڵ */
static void __aml166_boot_single_plfm_deinit()
{
    volatile uint32_t i = 1000000;
    /* ڽʼ ʼǰҪСεʱ֤ĴѾ*/
    while(i--){
    }

    am_aml166_clk_inst_deinit();

    am_int_disable(INUM_UART1);
    am_clk_disable(CLK_UART1);

    am_arm_nvic_deinit();

    __disable_irq();
}
/**
 * < \brief bootloader׼豸Ϣ
 */
static am_arm_boot_devinfo_t __g_aml166_boot_single_devinfo = {
    /**< \brief Ӧôʼַ*/
    0x08003C00,
    /** \brief ramʼַ */
    0x20000000,
    /** \brief ramС */
    8 * 1024,
    /** \brief ƽ̨ʼ */
    NULL,
    /** \brief ƽ̨ʼ */
    __aml166_boot_single_plfm_deinit,
};

int am_aml166_boot_single_inst_init()
{
    return am_arm_boot_init(&__g_aml166_boot_single_devinfo);
}


/** @} */

/* end of file */
