/*******************************************************************************
*                                 AMetal
*                       ----------------------------
*                       innovating embedded platform
*
* Copyright (c) 2001-2019 Guangzhou ZHIYUAN Electronics Co., Ltd.
* All rights reserved.
*
* Contact information:
* web site:    http://www.zlg.cn/
*******************************************************************************/

/**
 * \file
 * \brief I2C ӻ(̿ģ EEPROM)ͨ HW ӿʵ
 *
 * - 裺
 *   1. PIOA_11  I2C  SCL ţ
 *   2. PIOA_12  I2C  SDA š
 *
 * \note
 *    ǰ I2C  SCL ʹõ PIOA_11SDA ʹõ PIOA_12
 *    ɸʵš
 *
 * \par Դ
 * \snippet demo_zlg118_hw_i2c_slave_poll.c src_zlg118_hw_i2c_slave_poll
 *
 * \internal
 * \par Modification history
 * - 1.00 19-12-16  licl, first implementation
 * \endinternal
 */

/**
 * \addtogroup demo_if_zlg118_hw_i2c_slave_poll
 * \copydoc demo_zlg118_hw_i2c_slave_poll.c
 */

/** [src_zlg118_hw_i2c_slave_poll] */
#include "ametal.h"
#include "am_vdebug.h"
#include "hw/amhw_zlg118_i2c.h"
#include "am_int.h"

/*******************************************************************************
  궨
*******************************************************************************/

/** ӻַ */
#define OPERATE_ADDR      0x50

#define EEPROM_BUFFER_LEN (0x20)

/*******************************************************************************
  ȫֱ
*******************************************************************************/

static uint8_t   sub_addr;
static uint8_t   eeprom[EEPROM_BUFFER_LEN] = {0};   /**<  */

/**
 * \brief ȡһֽڻص
 */
static int __txbyte_get (void *p_arg, uint8_t *p_byte)
{
    *p_byte = eeprom[sub_addr];
    sub_addr++;
    return AM_OK;
}

/**
 * \brief һֽڻص
 */
static int __rxbyte_put(void *p_arg, uint8_t byte)
{
    eeprom[sub_addr] = byte;
    sub_addr++;
    return AM_OK;
}

/**
 * \brief ֹͣص
 */
static void  __tran_stop(void *p_arg)
{
    sub_addr = 0;
}

/**
 * \brief I2C ӻʼú
 *
 * \param[in] p_hw_i2c ָ I2C Ĵָ
 * \param[in] dev_addr ӻĵַ
 *
 * \retval AM_OK ʼ
 */
static int __i2c_slave_init (amhw_zlg118_i2c_t *p_hw_i2c, uint8_t dev_addr)
{
    if (p_hw_i2c == NULL) {
        return -AM_EINVAL;
    }

    /* I2CƼĴ */
    p_hw_i2c->i2c_cr = 0;

    /* ӻģʽӦ */
    amhw_zlg118_i2c_cr_set(p_hw_i2c, AMHW_ZLG118_I2C_REPLY_ENABLE);

    /* òʼ */
    amhw_zlg118_i2c_tm_disable(p_hw_i2c);

    /* ôӻַ */
    amhw_zlg118_i2c_slave_addr_set (p_hw_i2c, dev_addr);

    amhw_zlg118_i2c_enable(p_hw_i2c);

    return AM_OK;
}

/**
 * \brief I2C ״̬
 *
 * \param[in] p_hw_i2c ָ I2C Ĵָ
 *
 * \return 
 */
static void __i2c_slave_event (void *p_i2c_slv)
{
    static uint8_t           rx_len       = 0;
    amhw_zlg118_i2c_t       *p_hw_i2c_slv = p_i2c_slv;
    uint8_t                  state;
    uint8_t                  rx_data;
    uint8_t                  tx_data;

    state = amhw_zlg118_i2c_status_get(p_hw_i2c_slv);

    switch(state) {

    case 0xa0:  /* յֹͣظʼ */
        __tran_stop(NULL);
        rx_len = 0;
        amhw_zlg118_i2c_cr_set(p_hw_i2c_slv, AMHW_ZLG118_I2C_REPLY_ENABLE);
        break;

    case 0x60:  /* ѽյ(ƥ)SLA+WѽACK */
    case 0x70:  /* ѽͨõõַ0x00ѽACK */
        rx_len = 0;
        amhw_zlg118_i2c_cr_set(p_hw_i2c_slv, AMHW_ZLG118_I2C_REPLY_ENABLE);
        break;

    case 0x68:  /* ʱSLA+дʧٲãѽSLA+WѷACK */
    case 0x78:  /* ʱSLA+джʧٲãѽͨõõַѷACK */
    case 0x88:  /* ǰһѰַʹӵַѽֽڣѷطACK */
        amhw_zlg118_i2c_cr_set(p_hw_i2c_slv, AMHW_ZLG118_I2C_REPLY_ENABLE);
        break;

    case 0x80:  /* ǰһѰַʹӵַѽֽڣѷACK */
    case 0x98:  /* ǰһѰַʹͨõõַѽݣѷطACK */
    case 0x90:  /* ǰһѰַʹͨõõַѽݣѷACK */
        amhw_zlg118_i2c_cr_set(p_hw_i2c_slv, AMHW_ZLG118_I2C_REPLY_ENABLE);
        rx_data = amhw_zlg118_i2c_dat_read(p_hw_i2c_slv);/*  */
        if(rx_len) {    /* Խյĵһ(ӻ豸ӵַ) */
            __rxbyte_put(NULL, rx_data);    /* ܻص */
        }
        rx_len++;
        break;

    case 0xa8:  /* ѽSLA+RѷACK,ҪݲյACKλ */
    case 0xb0:  /* ʱSLA+джʧٲãѽSLA+RѷACK */
    case 0xc8:  /* װֽѱͣѽACK */
    case 0xb8:  /* ѷݣѽACK */
        __txbyte_get(NULL, &tx_data);
        amhw_zlg118_i2c_dat_write(p_hw_i2c_slv,tx_data);
        break;

    case 0xc0:  /* ݣշACKn */
        __tran_stop(NULL);
        break;

    default:
        break;
    }

    /* жϱ־ */
    amhw_zlg118_i2c_cr_clear(p_hw_i2c_slv, AMHW_ZLG118_I2C_INT_FLAG);
}

/**
 * \brief 
 */
void demo_zlg118_hw_i2c_slave_int_entry (void  *p_hw_i2c,
                                         int    inum_num)
{
    __i2c_slave_init(p_hw_i2c, OPERATE_ADDR); /* I2C ӻʼ */

    /* ж */
    am_int_connect(inum_num, __i2c_slave_event, (void *)p_hw_i2c);
    am_int_enable(inum_num);

    while(1) {
        ;
    }
}
/** [src_zlg118_hw_i2c_slave_poll] */

/* end of file */
