/*******************************************************************************
*                                 AMetal
*                       ----------------------------
*                       innovating embedded platform
*
* Copyright (c) 2001-2018 Guangzhou ZHIYUAN Electronics Co., Ltd.
* All rights reserved.
*
* Contact information:
* web site:    http://www.zlg.cn/
*******************************************************************************/

/**
 * \file
 * \brief UART DMA ̣ͨ HW ӿʵ
 *
 * - 裺
 *   1. ͨλһ MCU  5 ַ
 *
 * - ʵ
 *   1. ָڴӡյݣ
 *   2. Ȼָڴӡ "DMA transfer done!"
 *
 * \par Դ
 * \snippet demo_zlg118_hw_uart_rx_dma.c src_zlg118_hw_uart_rx_dma
 *
 *
 * \internal
 * \par Modification History
 * - 1.00 19-09-23  zp, first implementation
 * \endinternal
 */

/**
 * \addtogroup demo_if_zlg118_hw_uart_rx_dma
 * \copydoc demo_zlg118_hw_uart_rx_dma.c
 */

/** [src_zlg118_hw_uart_rx_dma] */
#include "ametal.h"
#include "am_int.h"
#include "am_zlg118_dma.h"
#include "hw/amhw_zlg118_uart.h"

/*******************************************************************************
  궨
*******************************************************************************/
#define UART_BAUDRATE      (115200)                    /**< \brief  */

/*******************************************************************************
  ȫֱ
*******************************************************************************/
static amhw_zlg118_dma_xfer_desc_t g_desc;             /**< \brief DMA  */
static volatile am_bool_t       g_trans_done = AM_FALSE; /**< \brief ɱ־ */
static uint8_t                  g_buf_dst[5] = {0};    /**< \brief Ŀݻ */
static int32_t                  g_dma_chan = 0;        /**< \brief DMA ͨ */
static amhw_zlg118_uart_t      *gp_hw_uart = NULL;     /**< \brief UART  */

/**
 * \brief DMA жϷ
 *
 * \param[in] p_arg : ûԶͨ am_zlg118_dma_isr_connect() 
 * \param[in] flag  : DMAжϱ־ɵײ룬òĿȡֵ
 *                    (#AM_ZLG116_DMA_INT_ERROR)  (#AM_ZLG116_DMA_INT_NORMAL)
 *
 * \return 
 */
static void uart_dma_isr (void *p_arg , uint32_t flag)
{
    int flag_chan  = (int)p_arg;

    if (flag == AM_ZLG118_DMA_INT_COMPLETE) {
        if (flag_chan == g_dma_chan) {

            /* UARTʱʹDMA */
            amhw_zlg118_uart_disable(gp_hw_uart, AMHW_ZLG118_UART_DMA_RX);

            g_trans_done = AM_TRUE;
        }
    } else {
        /* ûԶִд */
    }
}

/**
 * \brief UARTDMA
 */
static int uart_rx_dma_tran_cfg (amhw_zlg118_uart_t *p_hw_uart,
                                 int32_t          dma_chan,
                                 uint32_t         dma_tran_len)
{
    uint32_t i;
    uint32_t flags;

    for (i = 0; i < sizeof(g_buf_dst); i++) {
        g_buf_dst[i] = 0;
    }

    /* DMAжϷ */
    am_zlg118_dma_isr_connect(dma_chan, uart_dma_isr, (void *)dma_chan);

    /* DMA  */
    flags = AMHW_ZLG118_DMA_CHAN_TRANSFER_MODE_BLOCK    | /* SPIֻп鴫ģʽ */
            AMHW_ZLG118_DMA_CHAN_SIZE_8BIT              | /* λ 8bit */

            AMHW_ZLG118_DMA_CHAN_SRC_ADD_INC_DISABLE    | /* Դַ */
            AMHW_ZLG118_DMA_CHAN_RELOAD_SRC_ADD_ENABLE  | /* Դַ*/

            AMHW_ZLG118_DMA_CHAN_DST_ADD_INC_ENABLE     | /* Ŀַ */
            AMHW_ZLG118_DMA_CHAN_RELOAD_DST_ADD_DISABLE | /* Ŀַ */

            AMHW_ZLG118_DMA_CHAN_RELOAD_COUNTER_ENABLE  | /*  */
            AMHW_ZLG118_DMA_CHAN_INT_ERR_DISABLE        | /* жϽ */
            AMHW_ZLG118_DMA_CHAN_INT_TX_CMP_ENABLE      | /* жʹ */
            AMHW_ZLG118_DMA_CHAN_CIRCULAR_MODE_DISABLE ;  /* رѭģʽ */

    /* ͨ */
    am_zlg118_dma_xfer_desc_build(&g_desc,                      /* ͨ */
                                   (uint32_t)(&(p_hw_uart->sbuf)), /* Դݻ */
                                   (uint32_t)(g_buf_dst),       /* Ŀݻ */
                                   (uint32_t)dma_tran_len,      /* ֽ */
                                   flags);                      /*  */

    /* DMA䣬Ͽʼ */
    if (am_zlg118_dma_xfer_desc_chan_cfg(&g_desc,
                                          AMHW_ZLG118_DMA_PER_TO_MER, /* 赽 ڴ */
                                          (uint8_t)dma_chan) == AM_ERROR) {
        return AM_ERROR;
    } else {
        /*  */
    }

    return AM_OK;
}

/**
 * \brief UART ʼ
 */
static void uart_hw_init (amhw_zlg118_uart_t *p_hw_uart, uint32_t clk_rate)
{
    /* ģʽ  */
    amhw_zlg118_uart_mode_sel(p_hw_uart, AMHW_ZLG118_UART_WORK_MODE_1);

    /* Ƶϵ  */
    amhw_zlg118_uart_clk_div_sel(p_hw_uart, AMHW_ZLG118_UART_CLK_DIV_MODE13_8);

    /* ȴͿ */
    while (amhw_zlg118_uart_flag_check(p_hw_uart,
                                       AMHW_ZLG118_UART_FLAG_TX_EMPTY) ==
                                       AM_FALSE);

    /* ֹͣλ */
    amhw_zlg118_uart_stop_bit_sel(p_hw_uart  , AMHW_ZLG118_UART_STOP_1_0_BIT);

    /* У */
    amhw_zlg118_uart_parity_bit_sel(p_hw_uart, AMHW_ZLG118_UART_PARITY_NO);

    /*  */
    amhw_zlg118_uart_baudrate_set( p_hw_uart, clk_rate, UART_BAUDRATE);

    /* رдж */
    amhw_zlg118_uart_int_disable(p_hw_uart, AMHW_ZLG118_UART_INT_ALL);
}

/**
 * \brief UARTմʱDMAʼ
 */
static void uart_hw_dma_init (amhw_zlg118_uart_t *p_hw_uart)
{
    /* ڷDMAʹ */
    amhw_zlg118_uart_enable(p_hw_uart, AMHW_ZLG118_UART_DMA_RX);

    /* ʹ
     * Mode0:     0; 1
     * Mode1/2/3: 0; 1 /
     */
    amhw_zlg118_uart_enable(p_hw_uart,AMHW_ZLG118_UART_RX);
}

/**
 * \brief 
 */
void demo_zlg118_hw_uart_rx_dma_entry (void    *p_hw_uart,
                                       uint32_t clk_rate,
                                       int32_t  dma_chan,
                                       int32_t  dma_src)
{
    gp_hw_uart = (amhw_zlg118_uart_t *)p_hw_uart;
    g_dma_chan = dma_chan;

    /* UARTʼ */
    uart_hw_init(gp_hw_uart, clk_rate);

    /* UARTDMAĳʼ */
    uart_hw_dma_init(gp_hw_uart);

    amhw_zlg118_uart_poll_send(gp_hw_uart,
                               (uint8_t *)"UART DMA RX start:\r\n",
                               (uint32_t)sizeof("UART DMA RX start:\r\n") - 1);
    /* ʼDMA */
    uart_rx_dma_tran_cfg(gp_hw_uart, dma_chan, sizeof(g_buf_dst));

    /* Ĭͨ0ͨ1ȼһѯ*/
    am_zlg118_dma_priority_same();

    /* ôźԴ */
    am_zlg118_dma_chan_src_set (dma_chan, dma_src);

    /* ÿDMAСΪѿͨ1 */
    am_zlg118_dma_block_data_size(dma_chan, 1);

    /* ʼͨ */
    am_zlg118_dma_chan_start(dma_chan);

    /* ȴ */
    while (g_trans_done == AM_FALSE);

    g_trans_done = AM_FALSE;

    amhw_zlg118_uart_poll_send(gp_hw_uart, g_buf_dst, sizeof(g_buf_dst));
    amhw_zlg118_uart_poll_send(gp_hw_uart,
                               (uint8_t *)"\r\nDMA transfer done!\r\n",
                               sizeof("\r\nDMA transfer done!\r\n") - 1);

    AM_FOREVER {
        ; /* VOID */
    }
}
/** [src_zlg118_hw_uart_rx_dma] */

/* end of file */
