/*******************************************************************************
*                                 AMetal
*                       ----------------------------
*                       innovating embedded platform
*
* Copyright (c) 2001-2018 Guangzhou ZHIYUAN Electronics Stock Co., Ltd.
* All rights reserved.
*
* Contact information:
* web site:    http://www.zlg.cn/
* e-mail:      ametal.support@zlg.cn
*******************************************************************************/

/**
 * \file
 * \brief USB printer_counter 
 *
 * - ʵ
 * 1.USB˵PC
 * 2.¼̣ȴ3(3ʱΪģUSBγ)3pcʾװ
 *   Ĭϰװwindows ͨôӡϸɿװ˵ĵûҪԼи¡
 * 3.ڵ½һtxtļַ,֮ļӡڴпԿļݣʾôδ
 *   ʱ䡣
 *
 * \par Դ
 * \snippet demo_usbd_printer_counter.c src_usbd_printer_counter
 *
 * \internal
 * \par Modification History
 * - 1.00 19-1-09  adw, first implementation
 * \endinternal
 */

/**
 * \addtogroup demo_if_usbd_printer_counter
 * \copydoc demo_usbd_printer_counter.c
 */

/** [src_usbd_printer] */
#include "ametal.h"
#include "string.h"
#include "am_int.h"
#include "am_delay.h"
#include "am_vdebug.h"
#include "am_rngbuf.h"
#include "am_softimer.h"
#include "am_usbd_printer.h"
#include "demo_zlg_entries.h"

#define __RNG_BUFF_SIZE    1024                      /**< \brief С. */

static char __g_rng_buff[__RNG_BUFF_SIZE]   = {0};  /**< \brief λbuff*/

static char __g_read_buff[__RNG_BUFF_SIZE]  = {0};  /**< \brief ݶȡbuff*/

static struct am_rngbuf __g_rngbuff; /**< \brief һλʵ*/

static uint32_t __g_data_len  = 0;   /**< \brief һ͵ĵݳ. */
static uint32_t __g_timeout   = 0;   /**< \brief жϳʱ״̬*/
static uint32_t __g_clock     = 0;   /**< \brief ʱֵ,ms. */
static uint8_t  __g_int_state = 0;   /**< \brief ж״̬*/

/**
 * \brief ӡжص
 *
 * param[in] p_arg  : ص
 * param[in] p_data : buff
 * param[in] len    : ݵЧȣ
 */
static void __printer_recv_callback(void *p_arg, uint8_t *p_data, uint8_t len)
{
    __g_data_len += len;  /* . */
    __g_timeout   = 0;    /* 㳬ʱʱ*/
    __g_int_state = 1;    /* ж״̬־*/
    am_rngbuf_put(&__g_rngbuff, (char *)p_data, len);  /* 价λ*/
}

/**
 * \brief ӡжص
 *
 * param[in] p_arg  : ص
 */
static void __printer_send_callback(void *p_arg)
{
    //úΪӡ,USB device (Ķ˵)ʱͻú
    uint8_t data[] = "ZLG printer demo test string";
    am_usbd_printer_handle handle = (am_usbd_printer_handle)p_arg;

    am_usbd_printer_send(handle, AM_USBD_PRINTER_BULK_EP_IN, data, sizeof(data));
}

/**
 * \brief ʱжϻص
 */
static void  __softimer_callback_func(void * p_arg)
{
    static uint32_t time = 0;

    if (__g_int_state == 1) {       /* жǷжʱ*/
        __g_timeout++;              /* ʱʱۼӣж״̬ճʱʱ*/
        __g_clock++;                /* ηʱۼ*/

        if (__g_clock > 1000) {     /* ۼʱ1s,㣬timeֵۼ1 */
            __g_clock = 0;
            time++;
        }

        if (__g_timeout  > 1000) {  /* ʱʱ1s,ĬϱʾΪһݷ*/
            am_kprintf("\n total data: %d byte" \
                       "time-consuming: %d.%03d s\n", __g_data_len,
                                                      time - 1,
                                                      __g_clock);
            /* ״̬*/
            __g_data_len  = 0;
            __g_timeout   = 0;
            __g_int_state = 0;
            __g_clock     = 0;
            time          = 0;
        }
    }
}

/**
 * \brief usb device printer 
 */
void demo_usbd_printer_counter_entry (void* p_handle)
{
    uint32_t key = 0;
    am_usbd_printer_handle handle = p_handle;
    am_mdelay(3000);                                  /* ģUSB豸γĶ */
    am_softimer_t soft_time;                          /* ʱhandle*/

    /* ʼλ*/
    am_rngbuf_init(&__g_rngbuff, __g_rng_buff, __RNG_BUFF_SIZE);

    /* ʼʱ. */
    am_softimer_init(&soft_time, __softimer_callback_func, handle);
    am_softimer_start(&soft_time, 1);                 /* ʼʱ 1ms*/

    /* ʱպͷص*/
    am_usbd_printer_recv_request_callback(handle, __printer_recv_callback, handle);
    am_usbd_printer_send_request_callback(handle, __printer_send_callback, handle);

    while (1) {

        /* λΪգ*/
        if (!am_rngbuf_isempty(&__g_rngbuff)) {
            key = am_int_cpu_lock();
            am_rngbuf_get(&__g_rngbuff, __g_read_buff, __RNG_BUFF_SIZE);

            am_kprintf("%s", __g_read_buff);

            memset(__g_read_buff, 0, __RNG_BUFF_SIZE);
            am_int_cpu_unlock(key);
        }
    }
}
/** [src_usbd_printer_counter] */

/* end of file */
