/*******************************************************************************
*                                 AMetal
*                       ----------------------------
*                       innovating embedded platform
*
* Copyright (c) 2001-2018 Guangzhou ZHIYUAN Electronics Co., Ltd.
* All rights reserved.
*
* Contact information:
* web site:    http://www.zlg.cn/
*******************************************************************************/

/**
 * \file
 * \brief I2C׼ӿ
 *
 * \par ʾ
 * \code
 * #include "am_i2c.h"
 *
 * am_i2c_device_t dev;                     // 豸
 * uint8_t         read_buf[16];            // 
 * uint8_t         write_buf[16];           // д
 *
 * // ɴӻ豸
 * am_i2c_mkdev(&dev,
 *              i2c_handle,
 *              0x50,
 *              AM_I2C_ADDR_7BIT | AM_I2C_SUBADDR_1BYTE);
 *
 * // дݵӵַ0ʼ16ֽ
 * am_i2c_write(&dev, 0, &write_buf[0], 16);
 *
 * // ӵַ0ʼ16ֽ
 * am_i2c_read(&dev, 0, &read_buf[0], 16);
 * \endcode
 *
 * \internal
 * \par Modification History
 * - 1.03 15-12-08  cyl, delete the am_i2c_connect.
 * - 1.02 15-10-27  tee, add the concept of message.
 * - 1.01 15-08-19  tee, modified some interface.
 * - 1.00 14-11-01  jon, first implementation.
 * \endinternal
 */

#ifndef __AM_I2C_H
#define __AM_I2C_H

#ifdef __cplusplus
extern "C" {
#endif

#include "am_common.h"

/**
 * @addtogroup am_if_i2c
 * @copydoc am_i2c.h
 * @{
 */

/**
 * \name I2C Ʊʶam_i2c_transferflagsԱ
 * @{ 
 * I2Cĵַ͡д͡Ƶȱ־
 *
 * Щ־ࡰ֮ĽڸṹԱ
 * <code>am_i2c_transfer::flags</code>ֵδʽı־ʹĬֵ
 * 磬ЩֵõͬĿЧ
 * \code
 * am_i2c_transfer_t trans;
 * trans.flags = 0;             // 7-bitӻַ(Ĭ)д(Ĭ)
 * trans.flags = AM_I2C_M_WR;   // 7-bitӻַ(Ĭ)д
 * trans.flags = AM_I2C_M_RD;   // 7-bitӻַ(Ĭ)
 * trans.flags = AM_I2C_M_10BIT | AM_I2C_M_RD;   // 10-bitӻַ
 *
 * trans.flags = AM_I2C_M_7BIT |
 *               AM_I2C_M_WR |
 *               AM_I2C_M_IGNORE_NAK; // 7-bitӻַдӦ
 * \endcode
 *
 * \sa struct am_i2c_transfer
 */

#define AM_I2C_M_7BIT          0x0000u    /**< \brief 7-bits 豸ַ  */
#define AM_I2C_M_10BIT         0x0001u    /**< \brief 10-bits 豸ַ */
#define AM_I2C_M_WR            0x0000u    /**< \brief д           */
#define AM_I2C_M_RD            0x0002u    /**< \brief            */
#define AM_I2C_M_NOSTART       0x0004u    /**< \brief      */
#define AM_I2C_M_REV_DIR_ADDR  0x0008u    /**< \brief д־λת   */
#define AM_I2C_M_RECV_LEN      0x0010u    /**< \brief ݲ֧         */

/** \brief Ӧ (ӻַӦӦ) */
#define AM_I2C_M_IGNORE_NAK    0x0020u

/** \brief ڶнʱӦ    */
#define AM_I2C_M_NO_RD_ACK     0x0040u

/** \brief I2CƱ־ */
#define AM_I2C_M_MASK          0x00FFu

/** @} */


/**
 * \name I2C 豸Ա־ am_i2c_device  dev_flags Ա
 * @{ 
 * I2Cӻַ͡ӵַͺӵַȵȱ־
 *
 * 磬Щֵ岻ͬĴ豸ԣ
 * \code
 * am_i2c_device_t dev;
 * dev.flags = 0;                     // 7-bit ӻַӵַ
 * dev.flags = AM_I2C_SUBADDR_1BYTE;  // 7-bit ӻַ1ֽӵַ
 * dev.flags = AM_I2C_SUBADDR_2BYTE;  // 7-bit ӻַ2ֽӵַ
 *                                    // ӵַλַȴ
 *
 * dev.flags = AM_I2C_SUBADDR_2BYTE |
 *             AM_I2C_SUBADDR_LSB_FIRST; // 7-bit ӻַ2ֽӵַ
 *                                       // ӵַλֽȴ
 *
 * \endcode
 *
 * \sa struct am_i2c_device
 */

/** \brief 7λַģʽ(Ĭģʽ) */
#define AM_I2C_ADDR_7BIT         AM_I2C_M_7BIT

/** \brief 10λַģʽ */
#define AM_I2C_ADDR_10BIT        AM_I2C_M_10BIT

/** \brief 豸Ӧ */
#define AM_I2C_IGNORE_NAK        AM_I2C_M_IGNORE_NAK

/** \brief ӵַλֽȴ(Ĭ)*/
#define AM_I2C_SUBADDR_MSB_FIRST 0x0000u

/** \brief ӵַλֽȴ */
#define AM_I2C_SUBADDR_LSB_FIRST 0x0100u

#define AM_I2C_SUBADDR_NONE      0x0000u  /**< \brief ӵַ(Ĭ)  */
#define AM_I2C_SUBADDR_1BYTE     0x1000u  /**< \brief ӵַ1ֽ */
#define AM_I2C_SUBADDR_2BYTE     0x2000u  /**< \brief ӵַ2ֽ */

/** \brief ӱ־ȡӵַ */
#define AM_I2C_SUBADDR_LEN_GET(flags)  (((flags) & 0xF000) >> 12)

/** @} */

struct am_i2c_message;

/**
 * \brief I2Cṹ
 */
struct am_i2c_drv_funcs {
    
    /** \brief ʼϢ */
    int (*pfn_i2c_msg_start) (void *p_drv, struct am_i2c_message *p_trans);
    
};

/**
 * \brief I2C׼ṹ 
 */
typedef struct am_i2c_serv {
    struct am_i2c_drv_funcs   *p_funcs;   /**< \brief I2C         */
    void                      *p_drv;     /**< \brief I2Cڲ */
} am_i2c_serv_t;

/** \brief I2C׼ */
typedef am_i2c_serv_t *am_i2c_handle_t;

/**
 * \brief I2C豸ṹ (Ƽʹ am_i2c_mkdev() ñݽṹ) 
 */
typedef struct am_i2c_device {
    am_i2c_handle_t handle;    /**< \brief 豸I2C׼handle    */
    uint16_t        dev_addr;  /**< \brief ӻ豸ַ7λ10λ            */ 
    uint16_t        dev_flags; /**< \brief ӻ豸ԣI2C豸Ա־*/
} am_i2c_device_t;

/**
 * \brief I2C ṹ (Ƽʹ am_i2c_mktrans() ñݽṹ) 
 */ 
typedef struct am_i2c_transfer {
    uint16_t    addr;         /**< \brief I2C豸ַ                       */
    uint16_t    flags;        /**< \brief I2C ־ I2CƱ־  */
    uint8_t    *p_buf;        /**< \brief ݻ                        */
    uint32_t    nbytes;       /**< \brief ݸ                          */
} am_i2c_transfer_t;

/** 
 * \brief I2C Ϣ (Ƽʹ am_i2c_mkmsg() ñݽṹ) 
 */
typedef struct am_i2c_message {
    am_i2c_transfer_t  *p_transfers;  /**< \brief ϢĴ              */
    uint16_t            trans_num;    /**< \brief Ĵ          */
    uint16_t            done_num;     /**< \brief ɹĴ          */
    am_pfnvoid_t        pfn_complete; /**< \brief ɻص            */
    void               *p_arg;        /**< \brief ݸ pfn_complete Ĳ  */
    int                 status;       /**< \brief Ϣ״̬                  */
    void               *ctlrdata[2];  /**< \brief ʹ                  */
} am_i2c_message_t;


/**
 * \brief I2Cӻ豸ṹ
 *
 * \param[in] p_dev     : ָӻ豸ṹָ
 * \param[in] handle    : 豸I2C׼
 * \param[in] dev_addr  : ӻ豸ַ
 * \param[in] dev_flags : ӻ豸ԣI2C豸Ա־
 *
 * \return 
 *
 * \par 
 * \code
 * #include "am_i2c.h"
 *
 * am_i2c_device_t cat1025; // CAT1025豸ṹ
 *
 * // ʼCAT1025豸ṹλI2C0豸ַ0x50
 * // ĴַΪ1ֽڿ
 * am_i2c_mkdev(&cat1025,
 *              i2c_handle,
 *              0x50,
 *              AM_I2C_ADDR_7BIT | AM_I2C_SUBADDR_1BYTE);
 * \endcode
 */
am_static_inline
void am_i2c_mkdev (am_i2c_device_t *p_dev,
                   am_i2c_handle_t  handle,
                   uint16_t         dev_addr,
                   uint16_t         dev_flags)

{
    p_dev->handle     = handle;
    p_dev->dev_addr   = dev_addr;
    p_dev->dev_flags  = dev_flags;
}

/**
 * \brief I2CṹϢ
 *
 * \param[in] p_trans : ָṹָ
 * \param[in] addr    : Ĵӻ豸ַ
 * \param[in] flags   : еĿƱʶλAM_I2C_M_*ĺֵ߶
 *                      AM_I2C_M_*ĻORֵ (#AM_I2C_M_RD) 
 * \param[in] p_buf   : ͻ߽ݻ
 * \param[in] nbytes  : ݸ
 *
 * \retval  AM_OK     : ṹ
 * \retval -AM_EINVAL : 
 */
am_static_inline
void am_i2c_mktrans (am_i2c_transfer_t *p_trans, 
                     uint16_t           addr,  
                     uint16_t           flags, 
                     uint8_t           *p_buf, 
                     uint32_t           nbytes)
{
    p_trans->addr   = addr;
    p_trans->flags  = flags;
    p_trans->p_buf  = p_buf; 
    p_trans->nbytes = nbytes;
}

/**
 * \brief I2CϢṹ
 *
 * \param[in] p_msg          : I2CϢָ
 * \param[in] p_transfers    : ϢġI2C䡱
 * \param[in] trans_num      : I2C䡱ĸ
 * \param[in] pfn_complete   : Ϣɻص
 * \param[in] p_arg          : ݸصĲ
 *
 * \par 
 * \code
 * #include "am_i2c.h"
 *
 * am_i2c_message_t  msg;       // Ϣṹ
 * am_i2c_transfer_t trans[2];  // ṹ
 * uint8_t data0[16]            // ݻ 0
 * uint8_t data1[16];           // ݻ 1
 *
 * // ôṹ
 * am_i2c_mktrans(&trans[0],    // ṹ 0
 *                0x50,         // ҪĿ(ӻ)ַ
 *                AM_I2C_M_7BIT | AM_I2C_M_WR// 7-bitӻַд
 *                1,            // ݳȣ1ֽ
 *                &data0[0]);   // ݻ 0
 *
 * am_i2c_mktrans(&trans[1],    // ṹ 1
 *                0x50,         // ҪĿ(ӻ)ַ
 *                AM_I2C_M_7BIT | AM_I2C_M_RD// 7-bitӻַ
 *                16,           // ݳȣ16ֽ
 *                &data1[0]);   // ݻ1
 *
 * // Ϣṹ
 * am_i2c_mkmsg(&msg,           // Ϣṹ
 *              &trans[0],      // ϢĴ
 *              2,              // 
 *              my_callback,    // ɻص
 *              my_arg);        // ɻصĲ
 *
 * \endcode
 */
am_static_inline
void am_i2c_mkmsg (am_i2c_message_t  *p_msg,
                   am_i2c_transfer_t *p_transfers,
                   uint16_t           trans_num,
                   am_pfnvoid_t       pfn_complete,
                   void              *p_arg)
{
    p_msg->p_transfers  = p_transfers;
    p_msg->trans_num    = trans_num;
    p_msg->done_num     = 0;
    p_msg->pfn_complete = pfn_complete;
    p_msg->p_arg        = p_arg;
    p_msg->status       = -AM_ENOTCONN;
}

/**
 * \brief ʼһϢ
 *
 *     첽ķʽϢǰI2CУϢõִУ
 * УὫϢһŶӵȴI2C˳
 * еϢϢĴ״̬ͽӳ\a p_msg->status
 * Ϣ(ɹʱ)ʱ\a p_msg->pfn_complete 
 * \a p_msg->p_arg
 *
 * \param[in]     handle : I2C׼
 * \param[in,out] p_msg  : ҪϢ
 *
 * ķֵ£
 * \retval  AM_OK      : ϢŶӳɹȴ
 * \retval  -AM_EINVAL : 
 *
 * ɹĴӳ\a p_msg->done_num
 *
 * ϢĴ״̬ͽӳ\a p_msg->status 
 *
 *      \li  -AM_ENOTCONN    ϢδŶ
 *      \li  -AM_EISCONN     ϢŶӴ
 *      \li  -AM_EINPROGRESS Ϣڱ
 * ΪϢϵĽ (ͨ\a p_msg->done_num λĴ)
 *      \li  AM_OK           дɹ
 *      \li  -AM_ENOTSUP     ĳδĿƱ־֧
 *      \li  -AM_ENODEV      ĳδĿַӦ
 *      \li  -AM_ENOENT      ĳδ䷢͵Ӧ
 *      \li  -AM_EIO         Ϣʱߴٲʧܹȴ
 *      \li  -AM_ECANCELED   ϢȡԺ
 *
 * \par ʾ
 * \code
 *  #include "am_i2c.h"
 *
 * bool_t g_trans_complete = FALSE;
 *
 * void eeprom_callback (void *p_arg) 
 * {
 *      g_trans_complete = TRUE;
 *
 * }
 *
 * int main ()
 * {
 *     am_i2c_message_t  msg;
 *     am_i2c_transfer_t trans[2];
 *     uint8_t           subaddr;
 *     uint8_t           data[EEPROM_PAGE_SIZE];
 *     int               i;
 *     am_i2c_handle_t   i2c_handle = NULL;
 *
 *     // дEEPROMӵַ0x00ʼһҳ (ʱοֲ)
 *
 *     // 1䷢ӵַ
 *     subaddr = 0x00;
 *     am_i2c_mktrans(&trans[0],
 *                    EEPROM_ADDR,                 // Ŀַ
 *                    AM_I2C_M_7BIT | AM_I2C_M_WR, // 7-bitַ, д
 *                    &subaddr,                    // ӵַ
 *                    1);                          // ӵַȣ1byte
 *     
 *     // 2䷢
 *     for (i = 0; i < EEPROM_PAGE_SIZE; i++) {
 *         data[i] = i;
 *     }
 *     am_i2c_mktrans(&trans[1],
 *                    EEPROM_ADDR,                 // Ŀַ ()
 *     
 *                    // д, ʼźżӻַ
 *                    AM_I2C_M_WR | AM_I2C_M_NOSTART,
 *     
 *                    &data[0],                    // ݻ
 *                    EEPROM_PAGE_SIZE);           // ݸ
 *     
 *     // Ϣ
 *     am_i2c_mkmsg(&msg,
 *                  &trans[0],
 *                  2,
 *                  eeprom_callback,
 *                  &sem_sync);
 *     
 *     // ʼϢʹi2c_handle֮ǰӦʵƽ̨ȡȻȡһi2c_handle
 *     if (am_i2c_msg_start(i2c_handle, &msg) == AM_OK) {
 *          
 *          while (g_trans_complete == FALSE);
 *         
 *           if (msg.status != AM_OK) {
 *               // ִ
 *           }
 *     }
 *     
 *     while(1) {
 *
 *     }
 * }
 * 
 * \endcode
 */
am_static_inline
int am_i2c_msg_start (am_i2c_handle_t handle, am_i2c_message_t *p_msg)
{
    return handle->p_funcs->pfn_i2c_msg_start(handle->p_drv, p_msg);
}

/**
 * \brief I2Cд
 *
 * \param[in] p_dev    : ָӻ豸ϢĽṹָ
 * \param[in] sub_addr : ӻ豸ӵַ
 * \param[in] p_buf    : ָ/ݻ
 * \param[in] nbytes   : ݻ泤
 *
 * \return  AM_OK : д
 */
int am_i2c_write(am_i2c_device_t *p_dev,
                 uint32_t         sub_addr,
                 const uint8_t   *p_buf, 
                 uint32_t         nbytes);
/**
 * \brief I2C
 *
 * \param[in] p_dev    : ָӻ豸ϢĽṹָ
 * \param[in] sub_addr : ӻ豸ӵַ
 * \param[in] p_buf    : ָ/ݻ
 * \param[in] nbytes   : ݻ泤
 *
 * \return  AM_OK :  
 */
int am_i2c_read(am_i2c_device_t *p_dev,
                uint32_t         sub_addr,
                uint8_t         *p_buf, 
                uint32_t         nbytes);
/** 
 * @}
 */

#ifdef __cplusplus
}
#endif

#endif /* __AM_I2C_H */

/* end of file */
