/*******************************************************************************
*                                 AMetal
*                       ----------------------------
*                       innovating embedded platform
*
* Copyright (c) 2001-2018 Guangzhou ZHIYUAN Electronics Co., Ltd.
* All rights reserved.
*
* Contact information:
* web site:    http://www.zlg.cn/
*******************************************************************************/

/**
 * \file
 * \brief ӵַI2Cӻ׼ӿ
 *
 *
 * \internal
 * \par Modification History
 * - 1.00 17-09-19  vir, first implementation.
 * \endinternal
 */

#ifndef __AM_I2C_SLV_SUBADDR_H
#define __AM_I2C_SLV_SUBADDR_H

#ifdef __cplusplus
extern "C" {
#endif

#include "am_common.h"
#include "am_i2c_slv.h"

/**
 * @addtogroup am_if_i2c_slv_subaddr
 * @copydoc am_i2c_slv_subaddr.h
 * @{
 */

struct am_i2c_slv_subaddr_device;

/**
 * \brief ӵַ豸Ļص ڷ
 */
typedef struct am_i2c_slv_subaddr_cb_funcs {

    /** \brief ӻַƥʱص */
    void (*pfn_addr_match)( void       *p_arg,
                            am_bool_t   is_rx);

    /**
     * \brief ɻص
     *         is_rx:   ΪAM_TRUEʱʾ ӻ , Ϊ AM_FALSEʱʾӻ
     *         subaddr ӻ͵ӵַ   (Ĵλ޹ )
     *                  eg: ӻΪ int16_t buf[5]subaddr=2ʱ ӻbuf[2]ʼ
     *         size ӻ д˶ֽ
     *
     */
    void (*pfn_tran_finish)(void        *p_arg,
                            am_bool_t    is_rx,
                            uint32_t     subaddr,
                            uint32_t     size);

    /** \brief 㲥ص   */
    int (*pfn_gen_call)(void *p_arg, uint8_t byte);

}am_i2c_slv_subaddr_cb_funcs_t;

/**
 * \name 豸ĴռС
 * \anchor grp_i2c_slv_reg_mode
 * @{
 */

#define AM_I2C_SLV_REG_8_BIT           0X01u            /**< \brief ÿӵַӦļĴռСΪ8λ ( Ĭ) */
#define AM_I2C_SLV_REG_16_BIT          0X02u            /**< \brief ÿӵַӦļĴռСΪ16λ  */
#define AM_I2C_SLV_REG_32_BIT          0X04u            /**< \brief ÿӵַӦļĴռСΪ32λ  */
#define AM_I2C_SLV_REG_IRREGULAR       0X08u            /**< \brief ӵַӦļĴռС  */

/** @} */

/**
 * \brief ӵַļĴϢṹ
 *
 * \note ע͵ 1 ʾ ÿӵַӦļĴڴռ ͬ 
 *                     ĳӵַӦļĴռ СΪ124ֽ (8λ16λ32λ)
 *
 *              2 ʾ  ÿӵַӦļĴռСΪ1ֽ 2ֽ 4ֽ (8λ16λ32λ)
 *                     ĬмĴ
 *
 * \par 
 * \code
 * #include "am_i2c_slv_subaddr.h"
 *
 *   // 1  Ĵӻ豸Ĵ
 *    reg_mode = AM_I2C_SLV_REG_IRREGULAR
 *
 *   static uint8_t   reg1[5];    //ӵַΪ0
 *   static uint16_t  reg2[10];   //ӵַΪ1
 *   static uint32_t  reg3[1];    //ӵַ λ2
 *   .....
 *   static uint32_t  reg3_wr[1] = {0XFFFF0000}; //Ĵ3Ķд־
 *
 *   am_i2c_slv_subaddr_reg_info_t   i2c_slv_reg[] =
 *   {
 *      { sizeof(reg1), reg1, NULL },
 *      { sizeof(reg2), reg2, NULL },
 *      { sizeof(reg3), reg3, reg3_wr},
 *      ....
 *   }
 *
 *   //2 Ĵӻ豸Ĵ
 *   reg_mode = AM_I2C_SLV_REG_8_BIT   
 *   reg_mode = AM_I2C_SLV_REG_16_BIT  
 *   reg_mode = AM_I2C_SLV_REG_32_BIT
 *
 *   static uint16_t  reg[20];  //ӵַ
 *  am_i2c_slv_subaddr_reg_info_t i2c_slv_reg[1] =
 *  {
 *      { sizeof(reg), reg,  NULL },
 *  }
 *
 * \endcode
 *
 */
typedef struct am_i2c_slv_subaddr_reg_info {

    /**
     *  \brief 1 ĳӵַӦĴռĳ(ֽ)
     *         2 豸мĴܳ (ֽ)
     */
    uint32_t  reg_len;

    /**
     *  \brief 1 ĳӵַ ӦĴռ׵ַ
     *         2 豸Ĵ׵ַ
     */
    uint8_t  *p_reg_addr;

    /**
     * \brief  1 ĳӵַ ӦĴռĵĶд־(λֵΪ  0ɶд    1ֻɶд)
     *            ҪֵΪNULLĬϿɶд
     *         2 豸ĴĶд־
     */
    uint8_t  *p_reg_wr;
}am_i2c_slv_subaddr_reg_info_t;



/**
 * \brief ӵַI2Cӻ豸ṹ
 */
typedef struct am_i2c_slv_subaddr_device {
    uint32_t   subaddr ;               /**< \brief ӻӵַ */
    uint32_t   off_byte ;              /**< \brief Ҫݶ豸Ĵ׵ַƫ  */
    am_bool_t  is_subaddr;             /**< \brief жϵǰܵǷӵַ */
    am_bool_t  is_rx;                  /**< \brief жϵǰӻǷͻǽ  */
    uint8_t    reg_bit_width;          /**< \brief ӻĴλ ,8λ16λ32λ */
    uint8_t    subaddr_count_flag;     /**< \brief ӵַʶ  */
    uint32_t    data_count;            /**< \brief ֽܻͳ (ȥӵַֽ)*/
    uint16_t   subaddr_dev_flags;      /**< \brief ӵַ豸ʶ  */
    am_bool_t  reg_mode;               /**< \brief һӵַӦڴռСģʽ (8λ,16λ,32λ߲) */
    uint32_t   reg_num;                /**< \brief ӻĴ */

    am_i2c_slv_subaddr_reg_info_t     *reg_info;            /**< \brief 豸Ĵڴ׵ַ */
    am_i2c_slv_subaddr_reg_info_t     *cur_reg_info;        /**< \brief ǰӵַӦϢ */


    am_i2c_slv_subaddr_cb_funcs_t *p_cb_funs;    /**< \brief ӵַĻص  */
    void                          *p_arg;        /**< \brief ص  */

    am_i2c_slv_device_t            i2c_slv_dev;  /**< \brief I2Cӻ豸 */
}am_i2c_slv_subaddr_device_t;

/**
 * \brief ӵַ豸
 *
 * \param[in] p_dev                 : ָӵַӻ豸ṹָ
 * \param[in] handle                : 豸I2C׼
 * \param[in] p_cb_funs             : ûĻص
 * \param[in] p_arg                 : ص
 * \param[in] dev_addr              : ӻ豸ַ
 * \param[in] dev_flags             : ӻ豸ԣ磺豸ַӵַ ȣ I2C豸Ա־am_i2c_slv.h
 * \param[in] p_reg_info            : 豸Ĵ Ϣ 
 * \param[in] reg_num               : 豸Ĵ  ļĴռСɲֵΪ1
 * \param[in] reg_mode              : 豸Ĵģʽ  ֵο \ref grp_i2c_slv_reg_mode
 *
 * \return 
 *
 * \par 
 * \code
 * #include "am_i2c_slv_subaddr.h"
 *
 *   static uint8_t   reg1[5];    //ӵַΪ0
 *   static uint16_t  reg2[10];   //ӵַΪ1
 *   static uint32_t  reg3[1];    //ӵַ λ2
 *
 *   static uint32_t  reg3_wr[1] = {0XFFFF0000}; //Ĵ3Ķд־ 16λֻд
 *
 *   am_i2c_slv_subaddr_reg_info_t   i2c_slv_reg[3] =
 *   {
 *      { sizeof(reg1), reg1, NULL },
 *      { sizeof(reg2), reg2, NULL },
 *      { sizeof(reg3), reg3, reg3_wr},
 *   }
 *
 *   am_i2c_slv_subaddr_device_t  i2c_slv_dev;
 *   uint32_t reg_num = sizeof(i2c_slv_reg) / sizeof(am_i2c_slv_subaddr_reg_info_t );
 *
 *   am_i2c_slv_subaddr_mkdev( &i2c_slv_dev,
 *                              i2c_slv_handle,
 *                             &__g__cb_funs,
 *                             &i2c_slv_dev
 *                              0x55,
 *                              AM_I2C_SLV_ADDR_7BIT|AM_I2C_SLV_SUBADDR_1BYTE,
 *                              &i2c_slv_reg[0],
 *                              reg_num,
 *                              AM_I2C_SLV_REG_IRREGULAR);
 * \endcode
 *
 * \note  ӵַ豸ʱ  am_i2c_slv_setup  ӻ
 *       ͬʱɵ am_i2c_slv_shutdown رո豸
 */
void am_i2c_slv_subaddr_mkdev( am_i2c_slv_subaddr_device_t   *p_dev,
                               am_i2c_slv_handle_t            handle,
                               am_i2c_slv_subaddr_cb_funcs_t *p_cb_funs,
                               void                          *p_arg,
                               uint16_t                       dev_addr,
                               uint16_t                       dev_flags,
                               am_i2c_slv_subaddr_reg_info_t *p_reg_info,
                               uint32_t                       reg_num,
                               uint8_t                        reg_mode);



/**
 * @}
 */

#ifdef __cplusplus
}
#endif

#endif /* __AM_I2C_SLV_SUBADDR_H */

/* end of file */
