/*******************************************************************************
*                                 AMetal
*                       ----------------------------
*                       innovating embedded platform
*
* Copyright (c) 2001-2018 Guangzhou ZHIYUAN Electronics Co., Ltd.
* All rights reserved.
*
* Contact information:
* web site:    http://www.zlg.cn/
*******************************************************************************/

/**
 * \file
 * \brief ѹ
 *
 * \internal
 * \par modification history:
 * - 1.00 16-10-27  tee, copy from amorks
 * \endinternal
 */

#ifndef __AM_MEMHEAP_H
#define __AM_MEMHEAP_H

#include "ametal.h"

#ifdef __cplusplus
extern "C" {
#endif	/* __cplusplus	*/

/**
 * \addtogroup am_if_memheap
 * \copydoc am_memheap.h
 * @{
 */

/**
 * \brief memory item on the memory heap
 */
typedef struct am_memheap_item
{
    uint32_t                magic;           /**< magic number for memheap */
    struct am_memheap      *pool_ptr;        /**< point of pool            */

    struct am_memheap_item *next;            /**< next memheap item        */
    struct am_memheap_item *prev;            /**< prev memheap item        */

    struct am_memheap_item *next_free;       /**< next free memheap item   */
    struct am_memheap_item *prev_free;       /**< prev free memheap item   */
} am_memheap_item_t;

/**
 * \brief memory heap
 */
typedef struct am_memheap
{
    const char             *name;               /**< pool name */
    void                   *start_addr;         /**< pool start address and size */

    uint32_t                pool_size;          /**< pool size */
    uint32_t                available_size;     /**< available size */
    uint32_t                max_used_size;      /**< maximum allocated size */

    struct am_memheap_item *block_list;         /**< used block list */

    struct am_memheap_item *free_list;          /**< free block list */
    struct am_memheap_item  free_header;        /**< free block list header */
 
} am_memheap_t;

/**
 * \brief ʼһѹ
 *
 * \param[in] memheap    ָʼĶѹ
 * \param[in] name       : ѹ
 * \param[in] start_addr : öѹڴռʼַ
 * \param[in] size       : öѹڴռĴСֽ
 *
 * \retval AM_OK : ʼɹ
 * \retval  < 0  : ʼʧ
 */
am_err_t am_memheap_init(struct am_memheap *memheap,
                         const char        *name,
                         void              *start_addr,
                         uint32_t           size);
 
/**
 * \brief Ӷзռ
 *
 * \param[in] heap   ָѹ
 * \param[in] size   : ռĴС
 *
 * \return ռ׵ַNULLʧ
 */
void *am_memheap_alloc(struct am_memheap *heap, uint32_t size);

/**
 * \brief ȡռĴС
 *
 * \param[in] heap   ָѹ
 * \param[in] ptr    : ռ׵ַ
 *
 * \return ռĴСֽ
 */
size_t am_memheap_memsize(struct am_memheap *heap, void *ptr);

/**
 * \brief ̬ڴıѷռĴС
 *
 * \param[in] heap    ָѹ
 * \param[in] ptr     : ѷռ׵ַ
 * \param[in] newsize : ·ĿռĴС
 *
 * \return ·ռ׵ַNULLʧ
 * 
 * \note жϵǰָǷ㹻ռ䣬У ptr ַָ
 * ռ䣬ֱӽptrֵءռ䲻Ȱnew_sizeָĴСռ䣬
 * ԭݴͷβ·ڴ򣬶ͷԭptrָڴԭ
 * ָԶͷţҪ am_memheap_free()ͬʱ·ڴ׵ַ
 */
void *am_memheap_realloc(struct am_memheap *heap, void *ptr, size_t newsize);

/**
 * \brief ͷŶ̬Ķڴ
 * \param[in] ptr : Ŀռ׵ַ
 * \return 
 */
void am_memheap_free(void *ptr);

/** @}  am_if_memheap */

#ifdef __cplusplus
}
#endif	/* __cplusplus 	*/

#endif /* __AM_MEMHEAP_H    */

/* end of file */
