/*******************************************************************************
*                                 AMetal
*                       ----------------------------
*                       innovating embedded platform
*
* Copyright (c) 2001-2018 Guangzhou ZHIYUAN Electronics Co., Ltd.
* All rights reserved.
*
* Contact information:
* web site:    http://www.zlg.cn/
*******************************************************************************/
/**
 * \file
 * \brief SPI׼ӿ
 *
 * \internal
 * \par Modification history
 * - 1.01 15-08-17  tee, modified some interface. 
 * - 1.00 14-11-01  jon, first implementation.
 * \endinternal
 */

#ifndef __AM_SPI_H
#define __AM_SPI_H

#ifdef __cplusplus
extern "C" {
#endif

#include "am_common.h"
#include "am_list.h"

/**
 * \addtogroup am_if_spi
 * \copydoc am_spi.h
 * @{
 */

/**
 * \name SPIģʽ־am_spi_mkdev()mode
 * @{
 */

#define AM_SPI_CPHA        0x01  /**< \brief ʱλ             */
#define AM_SPI_CPOL        0x02  /**< \brief ʱӼԿ             */
#define AM_SPI_CS_HIGH     0x04  /**< \brief ƬѡЧ                 */
#define AM_SPI_LSB_FIRST   0x08  /**< \brief λȳķʽ֡ */
#define AM_SPI_3WIRE       0x10  /**< \brief 3ģʽSI/SO ź߹  */
#define AM_SPI_LOOP        0x20  /**< \brief ػģʽ                   */
#define AM_SPI_NO_CS       0x40  /**< \brief 豸, Ƭѡ         */
#define AM_SPI_READY       0x80  /**< \brief READYź,ӻʹźͣ */

#define AM_SPI_MODE_0      (0 | 0)                     /**< \brief SPIģʽ0 */
#define AM_SPI_MODE_1      (0 | AM_SPI_CPHA)           /**< \brief SPIģʽ1 */
#define AM_SPI_MODE_2      (AM_SPI_CPOL | 0)           /**< \brief SPIģʽ2 */
#define AM_SPI_MODE_3      (AM_SPI_CPOL | AM_SPI_CPHA) /**< \brief SPIģʽ3 */

/** @} */

/**
 * \name SPIñ־am_spi_mktrans()flags
 * @{
 */

/** \brief SPIڶȡMOSIߵƽĬΪ͵ƽ    */
#define AM_SPI_READ_MOSI_HIGH    0x01

/** @} */

/** 
 * \brief SPIϢṹ
 */
typedef struct am_spi_info {
    uint32_t  min_speed;     /**< \brief СƵ  */
    uint32_t  max_speed;     /**< \brief Ƶ  */ 
    uint32_t  features;      /**< \brief SPI,ֵ֧ļSPIñ־ */
} am_spi_info_t;

struct am_spi_device;        /**< \brief SPIӻ豸Ľṹ */
struct am_spi_transfer;      /**< \brief SPIĽṹ     */
struct am_spi_message;       /**< \brief SPIϢĽṹ     */

/**
 * \brief SPIṹ
 */
struct am_spi_drv_funcs {

    /** \brief ȡSPIϢ */
    int (*pfn_spi_info_get) (void *p_drv, am_spi_info_t *p_info);
    
    /** \brief SPIӻ豸   */
    int (*pfn_spi_setup) (void *p_drv, struct am_spi_device *p_dev);
    
    /** \brief SPIϢ䣬ɺûص  */
    int (*pfn_spi_msg_start)(void                   *p_drv,
                             struct am_spi_device   *p_dev,
                             struct am_spi_message  *p_msg);
};

/**
 * \brief SPI ׼ṹ
 */
typedef struct am_spi_serv {
    struct am_spi_drv_funcs   *p_funcs;    /**< \brief SPI  SPI     */
    void                      *p_drv;      /**< \brief SPI  SPI */
} am_spi_serv_t;

/** \brief SPI ׼ */
typedef am_spi_serv_t *am_spi_handle_t;

/**
 * \brief SPI 
 *
 * - SPI дĸǵڶĸЭӦṩ \a rx_buf /
 * \a tx_buf ĳЩ£ͬṩݴDMAַĻ
 * ʹDMAԽݴĴۡ
 *
 * - 仺 \a tx_buf ΪNULL \a rx_buf ʱ0ᱻƳϡ
 * ջ \a rx_buf ΪNULLݽᱻֻ \a len
 * ֽڻᱻƳ롣ƳǴĲ磬Ƴ3ֽڶִС
 *  16  20 bitǰÿʹ2ֽڣʹ4ֽڡ
 *
 * - ڴУǰCPUĵıֽ˳ţֽ˳ (ˣ
 * 豸ģʽ־б AM_SPI_LSB_FIRST)磬\a bits_per_word Ϊ
 * 16Ϊ 2N ֽڳ (\a len = 2N) CPUֽ˳㱣N16-bit֡
 *
 * -  SPI ִС2ݴα8-bitʱЩڴеֽbit
 * ڴУҶģˣδ (rx) δʹ (tx) λЧλ
 *
 * - SPI俪ʼʱƬѡЧͨһֱЧֱϢһ
 * ɡʹ \a cs_change ӰƬѡ:
 *  -# ô䲻Ϣһñ־ϢмƬѡΪЧ
 *     ַʽתƬѡҪֹһоƬõ am_spi_message_t
 *     ִоƬ顣
 *  -# ôϢһƬѡЧֱһ䡣ڶ豸SPI
 *     ϣûҪ͸豸Ϣܵһɣ
 *     ͸һ豸Ϣʼִк󣬱ʹǰ豸ƬѡЧǣĳ
 *     Щ£ñ־Ա֤ȷԡĳЩ豸ҪЭһϵе
 *     am_spi_message_t ɣһϢǰһϢĽ
 *     ƬѡΪЧ
 */
typedef struct am_spi_transfer {
    const void *p_txbuf;         /**< \brief ָݻָ   */
    void       *p_rxbuf;         /**< \brief ָݻָ   */
    size_t      nbytes;          /**< \brief ݳ(ֽ)         */

    /** \brief ִСΪ0ʹ am_spi_device_t õֵ */
    uint8_t     bits_per_word;   

    /** \brief ɺǷӰƬѡ: 1-Ӱ 0-Ӱ     */
    uint8_t     cs_change;      

    /** \brief ڱ֮ڸıƬѡ״̬֮ǰʱ΢    */
    uint16_t    delay_usecs;

    /**
     * \brief Ϊδѡһ am_spi_device_t еĬòͬٶȣ
     * 0 ʾʹĬֵ
     */
    uint32_t    speed_hz;

    /**
     * \brief δ̵һЩ
     *
     * ڶȡMOSIŵ״̬
     */
    uint32_t     flags;

    /** \brief 㣬ڽϢtransferһ */
    struct am_list_head trans_node;

} am_spi_transfer_t;

/**
 * \brief SPI Ϣ
 *
 * 1SPIϢִһԭӵݴУÿ am_spi_transfer ʾ
 * ԭӵģζκSPIϢʹSPIߣֱִɡ
 * ϵͳУϢǱŶӵģ͸ĳ am_spi_device_t ϢFIFO
 * ˳򱻴
 *
 * \attention
 * ײύ am_spi_message_t Ĵ븺Լڴ档е㲻Ҫ
 * ȷõƼʹ am_spi_msg_init() ʼ am_spi_message_t 
 * ύϢԼĴ֮󣬲ܲЩڴ,ֱϢĴɻص
 * ص
 * صú󣬱Ϣϣͨ p_msg->status жϢɵ״̬
 *  \a AM_OK    Ϣɹ
 *  \a -AM_EIO  ִ
 *
 * Ϣ״̬ΣԴ p_msg->actual_length 鿴ɹɵֽ
 */
typedef struct am_spi_message {
    struct am_list_head transfers;          /**< \brief ϢĴ */

    struct am_spi_device *p_spi_dev;        /**< \brief ϢSPI豸 */

    am_pfnvoid_t    pfn_complete;   /**< \brief ɻص */
    void           *p_arg;          /**< \brief ݸ pfn_complete Ĳ */
    size_t          actual_length;  /**< \brief гɹ͵ֽ */
    int             status;         /**< \brief Ϣ״̬ */

    void           *ctlrdata[2];    /**< \brief ʹãӦòӦ */
} am_spi_message_t;

/** \brief SPI豸ṹ */
typedef struct am_spi_device {

     am_spi_handle_t  handle;       /**< \brief 豸SPI׼handle */
    
    /**
     * \brief ݴ1֣ȽϳִСΪ8-bit12-bitڴ
     * УִСӦ2ݴη磬20-bitԴСӦʹ32-bitڴ档ÿ
     * 豸ı䣬Ϊ0ʾʹĬϵġִСΪ8-bit
     *䡱 am_spi_transfer_t.bits_per_word شá
     */
    uint8_t          bits_per_word;

    /** \brief 豸SPIģʽ־οSPIģʽ־*/
    uint16_t         mode;

    /**
     * \brief 豸ֵ֧ٶ
     *
     * 䡱 am_spi_transfer_t.speed_hz شá
     */
    uint32_t         max_speed_hz;

    /**
     * \brief Ƭѡű
     *
     *  am_spi_device.pfunc_cs ΪNULLʱʹôáĬϵ͵ƽЧ
     * am_spi_device.mode  AM_SPI_CS_HIGH ־ãߵƽЧ
     */
    int              cs_pin;

    /**
     * \brief ԶƬѡƺ
     *
     * ʹô˺豸Ƭѡ:  \a state Ϊ1ʱƬѡЧΪ0ʱƬѡЧ
     * ΪNULLʱʹ am_spi_device.cs_pin ΪƬѡţ
     * һʵƬѡƺ
     *
     * \param p_dev
     */
    void      (*pfunc_cs)(struct am_spi_device *p_dev, int state);

} am_spi_device_t;

/**
 * \brief SPI豸ṹ
 *
 * \param[in] p_dev         : SPIӻ豸ָ
 * \param[in] handle        : SPI׼
 * \param[in] bits_per_word : ִСΪ0ʱʹĬϵġ8-bitִС
 * \param[in] mode          : 豸ģʽ־SPIģʽ־
 * \param[in] max_speed_hz  : 豸ֵ֧ٶ
 * \param[in] cs_pin        : Ƭѡű(\a pfunc_cs ΪNULLʱЧ)
 * \param[in] pfunc_cs      : Ƭѡƺ(Ϊ NULL ʱcs_pin Ч)
 *
 * \return 
 *
 * \par 
 *  am_spi_setup()
 */
am_static_inline
void am_spi_mkdev (am_spi_device_t *p_dev,
                   am_spi_handle_t  handle,
                   uint8_t          bits_per_word,
                   uint16_t         mode,
                   uint32_t         max_speed_hz,
                   int              cs_pin,
                   void           (*pfunc_cs)(am_spi_device_t *, int ))
{
    p_dev->handle        = handle;
    p_dev->bits_per_word = bits_per_word;
    p_dev->mode          = mode;
    p_dev->max_speed_hz  = max_speed_hz;
    p_dev->cs_pin        = cs_pin;
    p_dev->pfunc_cs      = pfunc_cs;
}

/**
 * \brief SPIṹ
 *
 * \param[in] p_trans       : ָSPIṹָ
 * \param[in] p_txbuf       : ָݻָ,ʱӦΪNULL
 * \param[in] p_rxbuf       : ָݻָ,ʱӦΪNULL
 * \param[in] nbytes        : ǰݳȣֽ16λʱֽӦ
 *                             2 * Nݸ
 * \param[in] cs_change     : ǷӰƬѡ0-Ӱ죬1-Ӱ
 * \param[in] bits_per_word : λΪ0ʱʹ豸Ĭϵλ
 * \param[in] delay_usecs   : ʱλus
 * \param[in] speed_hz      : ʣΪ0ʱʹ豸Ĭϵ
 * \param[in] flags         : SPIƱ־λAM_SPI_TRANS_*ĺֵ
 *                            AM_SPI_TRANS_* ORֵ
 *
 * \return 
 *
 * \note ͺͽݻЧȷǵݳһµġ
 *       ڻͺݴֽ
 *        - 1  ~ 8 λʱuint8_t ͣnbytes = ݴĸ
 *        - 9  ~ 16λʱuint16_tͣnbytes = 2 * ݴĸ
 *        - 17 ~ 31λʱuint32_tͣnbytes = 4 * ݴĸ
 */
am_static_inline
void am_spi_mktrans (am_spi_transfer_t *p_trans,
                     const void        *p_txbuf,
                     void              *p_rxbuf,
                     uint32_t           nbytes,
                     uint8_t            cs_change,
                     uint8_t            bits_per_word,
                     uint16_t           delay_usecs,
                     uint32_t           speed_hz,
                     uint32_t           flags)
{
    p_trans->p_txbuf        = p_txbuf;
    p_trans->p_rxbuf        = p_rxbuf;
    p_trans->nbytes         = nbytes;
    p_trans->cs_change      = cs_change;
    p_trans->bits_per_word  = bits_per_word;
    p_trans->delay_usecs    = delay_usecs;
    p_trans->speed_hz       = speed_hz;
    p_trans->flags          = flags;
}

/**
 * \brief ʼϢ
 *
 * \param[in] p_msg        : SPIϢָ
 * \param[in] pfn_complete : ɻص
 * \param[in] p_arg        : ݸصĲ
 *
 * \return 
 */
am_static_inline
void am_spi_msg_init (am_spi_message_t  *p_msg,
                      am_pfnvoid_t       pfn_complete,
                      void              *p_arg)
{
    AM_INIT_LIST_HEAD(&p_msg->transfers);

    p_msg->p_spi_dev     = NULL;
    p_msg->actual_length = 0;
    p_msg->pfn_complete  = pfn_complete;
    p_msg->p_arg         = p_arg;
    p_msg->status        = -AM_ENOTCONN;
    p_msg->ctlrdata[0]   = NULL;
    p_msg->ctlrdata[1]   = NULL;
}

/**
 * \brief ӵϢĩβ
 *
 * \param[in] p_msg   : ָϢָ
 * \param[in] p_trans : ϢĴ
 *
 * \return 
 */
am_static_inline
void am_spi_trans_add_tail (am_spi_message_t  *p_msg,
                            am_spi_transfer_t *p_trans)
{
    am_list_add_tail(&p_trans->trans_node, &p_msg->transfers);
}

/**
 * \brief Ϣɾ
 * \param[in] p_trans : ɾĴ
 * \return 
 * \note ɾǰȷôȷʵѾϢ
 */
am_static_inline
void am_spi_trans_del (am_spi_transfer_t *p_trans)
{
    am_list_del(&p_trans->trans_node);
}

/** 
 * \brief ȡSPIϢ
 * 
 * \param[in]  handle : SPI׼
 * \param[out] p_info : ڻȡSPIϢָ
 *
 * \retval  AM_OK     : ȡϢɹ
 * \retval -AM_EINVAL : ȡϢʧ, 
 */
am_static_inline
int am_spi_info_get (am_spi_handle_t handle, am_spi_info_t *p_info)
{
    return handle->p_funcs->pfn_spi_info_get(handle->p_drv,p_info);
}

/**
 * \brief SPIӻ豸
 *
 * \attention SPIӻ豸ʹǰȵñã籾
 *            һʹôSPIӻ
 *
 * \param[in] p_dev : SPIӻ豸
 *
 * \retval   AM_OK           : óɹ
 * \retval  -AM_ENOTSUP      : ʧܣֵ֧λģʽ
 *
 * \par ʾ
 * \code
 * am_spi_device_t spi_dev;     // 豸ṹ
 *
 * // ʼ豸ṹ
 * am_spi_mkdev(&spi_dev,
 *              handle,         // SPI׼handle
 *              8             // ִСΪ8-bit
 *              AM_SPI_MODE_0,  // SPI ģʽ0
 *              500000,         // ֵ֧ٶ 500000 Hz
 *              PIO0_3,         // ƬѡΪ PIO0_3
 *              NULL);          // ԶƬѡƺ
 *
 * // 豸
 * am_spi_setup(&spi_dev);
 *
 * \endcode
 */
am_static_inline
int am_spi_setup (am_spi_device_t *p_dev)
{
    return p_dev->handle->p_funcs->pfn_spi_setup(p_dev->handle->p_drv, p_dev);
}

/**
 * \brief SPI亯
 *
 * \param[in] p_dev   : SPIӻ豸
 * \param[in] p_msg   : Ϣ
 *
 * \retval  AM_OK     : д
 * \retval -AM_EINVAL : 
 *  Լ״̬ͨصȡ״̬λ
 *    \li  AM_OK  : 
 *    \li -AM_EIO : 
 */
am_static_inline
int am_spi_msg_start (am_spi_device_t   *p_dev,
                      am_spi_message_t  *p_msg)
{
    return p_dev->handle->p_funcs->pfn_spi_msg_start(p_dev->handle->p_drv, 
                                                     p_dev, 
                                                     p_msg);
}

/**
 * \brief д
 *
 * ȽSPIдٽж
 * ʵӦУдΪַΪҪӸõַȡݡ
 *
 * \param[in]  p_dev   : SPIӻ豸
 * \param[in]  p_txbuf : ݷͻ
 * \param[in]  n_tx    : Ҫ͵ֽڸ
 * \param[out] p_rxbuf : ݽջ
 * \param[in]  n_rx    : Ҫյֽڸ
 *
 * \retval AM_OK      : Ϣɹ
 * \retval -AM_EINVAL : 
 * \retval -AM_EIO    : 
 */
int am_spi_write_then_read (am_spi_device_t *p_dev,
                            const uint8_t   *p_txbuf,
                            size_t           n_tx,
                            uint8_t         *p_rxbuf,
                            size_t           n_rx);

/**
 * \brief ִд
 *
 * ִдηݻ0ͻ1еݡ
 * ʵӦУ0Ϊַ1ΪҪдõַݡ
 *
 * \param[in] p_dev    : SPIӻ豸
 * \param[in] p_txbuf0 : ݷͻ0
 * \param[in] n_tx0    : 0ݸ
 * \param[in] p_txbuf1 : ݷͻ1
 * \param[in] n_tx1    : 1ݸ
 *
 * \retval AM_OK      : Ϣɹ
 * \retval -AM_EINVAL : 
 * \retval -AM_EIO    : 
 */
int am_spi_write_then_write (am_spi_device_t *p_dev,
                             const uint8_t   *p_txbuf0,
                             size_t           n_tx0,
                             const uint8_t   *p_txbuf1,
                             size_t           n_tx1);

/** 
 * @} 
 */

#ifdef __cplusplus
}
#endif

#endif /* __AM_SPI_H */

/*end of file */
