/*******************************************************************************
*                                 AMetal
*                       ----------------------------
*                       innovating embedded platform
*
* Copyright (c) 2001-2018 Guangzhou ZHIYUAN Electronics Co., Ltd.
* All rights reserved.
*
* Contact information:
* web site:    http://www.zlg.cn/
*******************************************************************************/
/**
 * \file
 * \brief ʱ׼
 *
 * ʹñҪͷļ am_time.h
 *
 * \par ʹʾ
 * \code
 * \endcode
 *
 * \internal
 * \par modification history
 * - 1.00 13-09-06  zen, first implementation
 * \endinternal
 */

#ifndef __AM_TIME_H
#define __AM_TIME_H

#ifdef __cplusplus
extern "C" {
#endif

#include "ametal.h"
#include "time.h"

/**
 * \addtogroup am_if_time
 * \copydoc am_time.h
 * @{
 */

/** \brief ʱ䣬1970  1  1  0 ʱ 0  0   */
typedef time_t am_time_t;

/** \brief ȷʱ䣬ʾʱ */
typedef struct am_timespec {
    am_time_t       tv_sec;   /**< \brief    */
    unsigned long   tv_nsec;  /**< \brief  */
} am_timespec_t;

/**
 * \brief ϸʱ, ṹ <time.h> е struct tm , ʹ <time.h> 
 * ĸ (ʹǿת (struct tm *)xx )
 */
typedef struct am_tm {
    int tm_sec;     /**< \brief seconds after the minute     - [0, 59]  */
    int tm_min;     /**< \brief minutes after the hour       - [0, 59]  */
    int tm_hour;    /**< \brief hours after midnight         - [0, 23]  */
    int tm_mday;    /**< \brief day of the month             - [1, 31]  */
    int tm_mon;     /**< \brief months since January         - [0, 11]  */
    int tm_year;    /**< \brief years since 1900                        */
    int tm_wday;    /**< \brief days since Sunday            - [0, 6]   */
    int tm_yday;    /**< \brief days since January 1         - [0, 365] */
    int tm_isdst;   /**< \brief Daylight Saving Time flag               */
} am_tm_t;

/**
 * \brief ȡǰʱ ( <time.h>  time() Ϊͬ)
 *
 * \note ʹ am_time_to_tm() ʱתΪϸʱ
 *
 * \param[out] p_time   ΪNULL, 򷵻ʱ
 *
 * \return ǰʱ
 *
 * \par ʾ
 * \code
 * #include "am_time.h"
 *
 * am_time_t time;      // ʱ
 * am_tm_t   tm;        // ϸʱ
 *
 * time = am_time(NULL);                            // õʱ
 * printf("%s\n", ctime(const time_t *)&time);      // ӡʱ
 *
 * am_time_to_tm(time, &tm);                        // ʱתΪϸʱ
 * printf("%s\n", asctime(const struct tm *)&tm);   // ӡϸʱ
 * \endcode
 */
am_time_t am_time(am_time_t *p_time);

/**
 * \brief ȡǰʱ (롢ʾ)
 *
 *  am_time(), am_tm_get(), ܶȡһֵ
 *
 * \note ʹ am_tm_get() ֱӻȡǰʱϸֱʾʽ
 *
 * \param[out] p_tv  ʱֵ
 *
 * \retval AM_OK    ɹ
 * \retval -AM_EINVAL  
 *
 * \par ʾ
 * \code
 * #include "am_time.h"
 *
 * am_timespec_t timespec;      // ʱ
 *
 * am_timespec_get(&timespec);                            // õʱ
 * printf("%s", ctime((const time_t *)&timespec.tv_sec)); // ӡʱ
 * printf(" %d ms \n", timespec.tv_nsec / 1000000);       // ӡ
 * \endcode
 * \sa am_time() am_timespec_set()
 */
int am_timespec_get(am_timespec_t *p_tv);

/**
 * \brief õǰʱ
 *
 * \note ʹ am_tm_set() ϸֱʾʽõǰʱ
 *
 * \param[in] p_tv Ҫõʱ
 *
 * \retval AM_OK    ɹ
 * \retval -AM_EINVAL  
 *
 * \par ʾ
 * \code
 * #include "am_time.h"
 *
 * am_tm_t       tm;            // ϸʱ
 * am_timespec_t timespec;      // ʱ
 *
 * tm.tm_sec    = 6;
 * tm.tm_min    = 6;
 * tm.tm_hour   = 6;
 * tm.tm_mday   = 6;
 * tm.tm_mon    = 6 - 1;        // ʵ·Ҫ1
 * tm.tm_year   = 2013 - 1900;  // 1900
 * tm.tm_isdst  = -1;           // ʱ
 *
 * am_tm_to_time(&tm, &timespec.tv_sec);    // תΪʱ
 * timespec.tv_nsec = 0;                    // 벿0
 *
 * am_timespec_set(&timespec);              // ʱ
 *
 * \endcode

 * \sa am_time() am_timespec_Get()
 */
int am_timespec_set(am_timespec_t *p_tv);

/**
 * \brief ȡǰʱ (ϸʽʾ)
 *
 * \note ʹ am_tm_get() ֱӻȡǰʱϸֱʾʽ
 *
 * \param[out] p_tm  ʱֵ
 *
 * \retval AM_OK    ɹ
 * \retval -AM_EINVAL  
 *
 * \par ʾ
 * \code
 * #include "am_time.h"
 * am_tm_t tm;          // ϸʱ
 *
 * am_tm_get(&tm);      // õǰʱ
 * printf("%s", asctime((const struct tm *)&tm)); // ӡʱ
 *
 * printf("%d-%d-%d %d:%d:%d wday:%d yday:%d\n",
 *        tm.tm_year + 1900, tm.tm_mon + 1, tm.tm_mday,
 *        tm.tm_hour, tm.tm_min, tm.tm_sec,
 *        tm.tm_wday, tm.tm_yday);              // ӡʱ
 *
 * \endcode
 * \sa am_time() am_tm_set()
 */
int am_tm_get (am_tm_t *p_tm);

/**
 * \brief õǰʱ(ϸʽʾ)
 *
 * \param[in] p_tm Ҫõʱ
 *
 * \retval AM_OK    ɹ
 * \retval -AM_EINVAL  
 *
 * \par ʾ
 * \code
 * #include "am_time.h"
 *
 * am_tm_t       tm;            // ϸʱ
 *
 * tm.tm_sec    = 6;
 * tm.tm_min    = 6;
 * tm.tm_hour   = 6;
 * tm.tm_mday   = 6;
 * tm.tm_mon    = 6 - 1;        // ʵ·Ҫ1
 * tm.tm_year   = 2013 - 1900;  // 1900
 * tm.tm_isdst  = -1;           // ʱ
 *
 * am_tm_set(&tm);              // ʱ
 *
 * \endcode
 *
 * \sa am_time() am_timespec_Get()
 */
int am_tm_set (am_tm_t *p_tm);

/**
 * \brief ϸʱʽתΪʱʽ
 *
 * \param[in,out] p_tm    ϸʱ, Ա tm_wday, tm_yday ڵú󱻸;
 *                        ʱЧ, tm_isdst Ϊ, ЧΪ0, Ϣ
 *                        Ϊ
 * \param[out]    p_time  ʱ
 *
 * \retval AM_OK       ɹ
 * \retval -AM_EINVAL  
 * \retval -EBADF      ϸʱ, ϸʱЧֵο am_tm_t
 */
int am_tm_to_time (am_tm_t *p_tm, am_time_t *p_time);

/**
 * \brief ʱʽתΪϸʱʽ
 *
 * \attention 
 *
 * \param[in]   p_time  ʱ
 * \param[out]  p_tm    ϸʱ
 *
 * \retval AM_OK       ɹ
 * \retval -AM_EINVAL  
 */
int am_time_to_tm (am_time_t *p_time, am_tm_t *p_tm);

/**
 * \brief ϵͳʱʼ
 *
 * \param[in] rtc_handle        : RTCʵʱʱӾ
 * \param[in] update_sysclk_ns  : ʹϵͳʱӸµʱλ룩
 * \param[in] update_rtc_s      : ʹRTCµʱλ룩
 *
 * \retval AM_OK       ɹ
 * \retval -AM_EINVAL  
 */
int am_time_init (void           *rtc_handle,
                  unsigned int    update_sysclk_ns,
                  unsigned int    update_rtc_s);

/**
 * \brief RTCϵͳʱ
 */
int am_time_update (void);

/** @} am_if_time */

#ifdef __cplusplus
}
#endif

#endif /* __AM_TIME_H */

/* end of file */

