/*******************************************************************************
*                                 AMetal
*                       ----------------------------
*                       innovating embedded platform
*
* Copyright (c) 2001-2018 Guangzhou ZHIYUAN Electronics Co., Ltd.
* All rights reserved.
*
* Contact information:
* web site:    http://www.zlg.cn/
*******************************************************************************/

/**
 * \file
 * \brief ӻԱ׼ӿ
 *
 * ʹñҪͷļ am_vdebug.h
 *
 * \par ʾ
 * \code
 *  #include "ametal.h"
 *  #include "am_vdebug.h"
 *
 *  int age = 18;
 *  am_kprintf("I'm %d years old!\n", age);
 * \endcode
 *
 * \internal
 * \par modification history:
 * - 1.00 15-01-16  tee, first implementation
 * \endinternal
 */

#ifndef __AM_VDEBUG_H
#define __AM_VDEBUG_H

#include <stdarg.h>
#include <stdlib.h>
#include "am_uart.h"

#ifdef __cplusplus
extern "C" {
#endif

/**
 * \addtogroup am_if_vdebug
 * \copydoc am_vdebug.h
 * @{
 */

/**
 * \brief AM_VDEBUG, AM_DBG_INFO() 꽫Ϣ
 *        AM_DBG_INFO()ʵΪ
 */
#ifdef AM_VDEBUG
#define AM_DBG_INFO(...)    (void)am_kprintf(__VA_ARGS__)
#else
#define AM_DBG_INFO(...)
#endif

/** \brief format log message and output */
#ifdef AM_VDEBUG
#define AM_LOGF(msg)    (void)am_kprintf msg
#else
#define AM_LOGF(msg)
#endif

/** \brief format information message */
#ifdef AM_VDEBUG_INFO
#define AM_INFOF(info)  AM_LOGF(info)
#else
#define AM_INFOF(info)
#endif

/** \brief format information message */
#ifdef AM_VDEBUG_DEBUG
#define AM_DBGF(info)   AM_LOGF(info)
#else
#define AM_DBGF(info)
#endif

/** \brief format warning message */
#ifdef AM_VDEBUG_WARN
#define AM_WARNF(info)  AM_LOGF(info)
#else
#define AM_WARNF(info)
#endif

/** \brief format error message */
#ifdef AM_VDEBUG_ERROR
#define AM_ERRF(info)   AM_LOGF(info)
#else
#define AM_ERRF(info)
#endif

/**
 * \brief Գʼ
 * \param[in] handle   : ׼ĴڷHandle, Ϣô
 * \param[in] baudrate : ʹõĲ
 * \return 
 * \note ʹõϢǰȷԴѵøúʼ 
 */
void am_debug_init (am_uart_handle_t handle, uint32_t baudrate);

/**
 * \brief ʽַ
 *
 * ú vXXXprintf() Ļ vXXXprintf() ڴ˺
 *
 * \param[in] fil     :  Ϊ f_putc  f_puts ĵڶ
 * \param[in] f_putc  :  ָ룬ָĺһַ
 * \param[in] f_puts  :  ָ룬ָĺһַ
 * \param[in] str     :  ʽַ
 * \param[in] arp     :  ʽַĲ
 *
 * \return ʵַ
 */
int am_vfprintf_do (void       *fil,
                    int       (*f_putc) (const char  c, void *fil),
                    int       (*f_puts) (const char *s, void *fil),
                    const char *str,
                    va_list     arp);

/**
 * \brief ʽַһָȵַ
 *
 * \param[in] buf  : ʽַĻ
 * \param[in] sz   : С
 * \param[in] fmt  : ʽַ
 * \param[in] args : ʽַĲ
 *
 * \return ʵ뻺ַĸ
 */
int am_vsnprintf (char *buf, size_t sz, const char *fmt, va_list args);

/**
 * \brief ʽַһָȵַ
 *
 * \param[in] buf  : ʽַĻ
 * \param[in] sz   : С
 * \param[in] fmt  : ʽַ
 * \param[in] ...  : ʽַĲб
 *
 * \return ʵ뻺ַĸ
 *
 * \par ʾ
 * \code
 *  #include "ametal.h"
 *  #include "am_vdebug.h"
 *  #include <string.h>
 *
 *  char buf[64];
 *  int age = 18;
 *  am_snprintf(&buf[0], sizeof(buf), "I'm %d years old!\n", age);
 * \endcode
 */
#ifndef AM_SNPRINTF_DEFINED
int am_snprintf (char *buf, size_t sz, const char *fmt, ...);
#endif  /* AM_SNPRINTF_DEFINED */

/**
 * \brief ں˸ʽӡ
 *
 * ʽַǰĿ̨
 *
 * \param[in] fmt : ʽַ
 * \param[in] ... : ʽַĲб
 *
 * \return ַ̨
 *
 * \par ʾ
 * \code
 *  #include "ametal.h"
 *  #include "am_vdebug.h"
 *
 *  int age = 18;
 *  am_kprintf("I'm %d years old!\n", age);
 * \endcode
 */
#ifndef AM_KPRINTF_DEFINED
int am_kprintf (const char *fmt, ...);
#endif  /* AM_KPRINTF_DEFINED */

/**
 * @} 
 */

#ifdef __cplusplus
}
#endif

#endif /* __AM_VDEBUG_H */

/* end of file */
