/*******************************************************************************
*                                 AMetal
*                       ----------------------------
*                       innovating embedded platform
*
* Copyright (c) 2001-2018 Guangzhou ZHIYUAN Electronics Co., Ltd.
* All rights reserved.
*
* Contact information:
* web site:    http://www.zlg.cn/
*******************************************************************************/

/**
 * \file
 * \brief CAN hardware operation interface
 *
 * -note:ڶͬһĴждʱӦĹܲͬ
 *
 * \internal
 * \par Modification history
 * - 1.00 17-09-05  zcb, first implementation
 * \endinternal
 */

#ifndef __AMHW_ZMF159_CAN_H
#define __AMHW_ZMF159_CAN_H

#ifdef __cplusplus
extern "C" {
#endif

#include "am_types.h"
#include "ametal.h"
#include "am_bitops.h"

/**
 * \addtogroup amhw_zmf159_if_can
 * \copydoc amhw_zmf159_can.h
 * @{
 */

/**
 * \brief ZMF159 CANĴṹ
 */
typedef struct amhw_zmf159_can {

    union {
        __IO  uint32_t cr;              /**< \brief 0x00 basic ƼĴ */
        __IO  uint32_t mod;             /**< \brief 0x00 peli ģʽĴ */
    }mode;

    __IO  uint32_t cmr;                 /**< \brief 0x04 Ĵ */
    __IO  uint32_t sr;                  /**< \brief 0x08 ״̬Ĵ */
    __I   uint32_t ir;                  /**< \brief 0x0c жϼĴ */

    union {

        struct {
            __IO  uint32_t acr;         /**< \brief 0x10 մĴ */
            __IO  uint32_t amr;         /**< \brief 0x14 μĴ */
        }basic;

        struct {
            __IO  uint32_t ier;         /**< \brief 0x10 жʹܼĴ */
                  uint32_t reserve0;
        }peli;

    }accept;

    __IO  uint32_t btr0;                /**< \brief 0x18 ߶ʱ0 */
    __IO  uint32_t btr1;                /**< \brief 0x1c ߶ʱ1 */

    union {

        struct {
                  uint32_t reserve1[2]; /**< \brief  */
            __IO  uint32_t txid0;       /**< \brief 0x28 id10 ~ 3 */
            __IO  uint32_t txid1;       /**< \brief 0x2c id2 ~ 0/rtr/dlc0-4 */
            __IO  uint32_t txdr[8];     /**< \brief 0x30 DATA1 - 8 */
            __IO  uint32_t rxid0;       /**< \brief 0x50 id10 ~ 3 */
            __IO  uint32_t rxid1;       /**< \brief 0x54 id2 ~ 0/rtr/dlc0-4 */
            __IO  uint32_t rxdr[8];     /**< \brief 0x58 DATA1 - 8 */
                  uint32_t reserve2;    /**< \brief 0x78  */
        }basic;

        struct {
                  uint32_t reserve1[3]; /**< \brief  */
            __IO  uint32_t alc;         /**< \brief 0x2c ܲöʧ׽Ĵ */
            __IO  uint32_t ecc;         /**< \brief 0x30 벶׽Ĵ */
            __IO  uint32_t ewlr;        /**< \brief 0x34 󱨾ƼĴ */
            __IO  uint32_t rxerr;       /**< \brief 0x38 RXĴ */
            __IO  uint32_t txerr;       /**< \brief 0x3c TXĴ */
            __IO  uint32_t sff;         /**< \brief 0x40 RX/TX֡ϢĴ */
            __IO  uint32_t id0;         /**< \brief 0x44 RX/TXʶ1 */
            __IO  uint32_t id1;         /**< \brief 0x48 RX/TXʶ2 */
            __IO  uint32_t data[10];    /**< \brief 0x4c RX/TX */
            __IO  uint32_t rmc;         /**< \brief 0x74 RXϢ */
            __IO  uint32_t rbsa;        /**< \brief 0x78 RXʼַ */
        }peli;

    }info;

    __IO  uint32_t cdr;                 /**< \brief 0x7c ʱӷƵ */

}amhw_zmf159_can_t;

/**
 * \brief CANģʽ
 */
typedef enum amhw_zmf159_can_type {
    AMHW_ZMF159_CAN_BASIC_CAN = 0,        /**< \brief basic can */
    AMHW_ZMF159_CAN_PELI_CAN,             /**< \brief peli can */
}amhw_zmf159_can_type_t;

/**
 * \brief CANܶ
 *
 * \note basic canֻ֪иλģʽģʽ˯ģʽ
 *
 */
typedef enum amhw_zmf159_can_mode_func {
    AMHW_ZMF159_CAN_MODE_RESET = 0,       /**< \brief λģʽ */
    AMHW_ZMF159_CAN_MODE_ONLY_LISTEN,     /**< \brief ֻģʽ */
    AMHW_ZMF159_CAN_MODE_SELF_CHECK,      /**< \brief Լģʽ */
    AMHW_ZMF159_CAN_MODE_SLEEP = 4,       /**< \brief ˯ģʽ */
    AMHW_ZMF159_CAN_MODE_RUN,             /**< \brief ģʽ */
}amhw_zmf159_can_mode_t;

/**
 * \brief CANݽշʽ
 */
typedef enum amhw_zmf159_can_recv_mode {
    AMHW_ZMF159_CAN_RECV_POLL = 0,        /**< \brief ѯģʽ */
    AMHW_ZMF159_CAN_RECV_INT,             /**< \brief жϽģʽ */
}amhw_zmf159_can_recv_mode_t;

/**
 * \brief peli CAN˲ʽ
 */
typedef enum amhw_zmf159_can_peli_filter_mode {
    AMHW_ZMF159_CAN_PELI_SINGLE_FILTER,   /**< \brief peli can ˲ģʽ */
    AMHW_ZMF159_CAN_PELI_DOUBLE_FLITER,   /**< \brief peli can ˫˲ģʽ */
}amhw_zmf159_can_peli_filter_mode_t;

/******************************************************************************/

/**
 * \brief CANж
 *
 * \note basic canֻ֧ǰ5жϣлжûʹܺͽλֻʹܻ߽ǰ4ж
 *
 * @{
 */

#define AMHW_ZMF159_CAN_INT_NUM              (8ul)

#define AMHW_ZMF159_CAN_INT_BUS_ERR          (0x1 << 7ul)  /**< \brief ߴж */
#define AMHW_ZMF159_CAN_INT_ARBITRATION_LOST (0x1 << 6ul)  /**< \brief ٲöʧж */
#define AMHW_ZMF159_CAN_INT_ERR_PASSIVE      (0x1 << 5ul)  /**< \brief ж */
#define AMHW_ZMF159_CAN_INT_WAKEUP           (0x1 << 4ul)  /**< \brief ж */
#define AMHW_ZMF159_CAN_INT_OVER             (0x1 << 3ul)  /**< \brief ж */
#define AMHW_ZMF159_CAN_INT_ERR              (0x1 << 2ul)  /**< \brief ж */
#define AMHW_ZMF159_CAN_INT_TRAN             (0x1 << 1ul)  /**< \brief ж */
#define AMHW_ZMF159_CAN_INT_RECV             (0x1 << 0ul)  /**< \brief ж */

/**
 * @}
 */

/**
 * \brief canжʹ
 *
 * \param[in] p_hw_can   : ָCANĴָ
 * \param[in] type       : canͣöamhw_zmf159_can_mode_t
 * \param[in] flag       : жϱ־ο궨AMHW_ZMF159_CAN_INT_
 *
 * \return 
 */
am_static_inline
void amhw_zmf159_can_int_enable (amhw_zmf159_can_t      *p_hw_can,
                                 amhw_zmf159_can_type_t  type,
                                 uint32_t                flag)
{
    if (type == AMHW_ZMF159_CAN_BASIC_CAN) {
        p_hw_can->mode.cr |= (flag << 1);
    } else if (type == AMHW_ZMF159_CAN_PELI_CAN) {
        p_hw_can->accept.peli.ier |= (flag);
    }
}

/**
 * \brief canжϽ
 *
 * \param[in] p_hw_can   : ָCANĴָ
 * \param[in] type       : canͣöamhw_zmf159_can_mode_t
 * \param[in] flag       : жϱ־ο궨AMHW_ZMF159_CAN_INT_
 *
 * \return 
 */
am_static_inline
void amhw_zmf159_can_int_disable (amhw_zmf159_can_t      *p_hw_can,
                                  amhw_zmf159_can_type_t  type,
                                  uint32_t                flag)
{
    if (type == AMHW_ZMF159_CAN_BASIC_CAN) {
        p_hw_can->mode.cr &= ~(flag << 1);
    } else if (type == AMHW_ZMF159_CAN_PELI_CAN) {
        p_hw_can->accept.peli.ier &= ~(flag);
    }
}

/**
 * \brief canжϱ־ȡжжϴԴ
 *
 * \param[in] p_hw_can   : ָCANĴָ
 *
 * \return жϵжԴ
 */
am_static_inline
uint8_t amhw_zmf159_can_int_status_get (amhw_zmf159_can_t *p_hw_can)
{
    return (uint8_t)(p_hw_can->ir & 0xff);
}

/******************************************************************************/

/**
 * \brief can ģʽ
 *
 * \param[in] p_hw_can ָCANĴָ
 * \param[in] type     canģʽöamhw_zmf159_can_type_t
 * \param[in] p_now    ǰģʽ
 * \param[in] update   ģʽ
 *
 * \return 
 *
 * \note basic canģʽָֻ֧λС˯ģʽ
 *
 */
void amhw_zmf159_can_mode_set (amhw_zmf159_can_t      *p_hw_can,
                               amhw_zmf159_can_type_t  type,
                               amhw_zmf159_can_mode_t *p_now,
                               amhw_zmf159_can_mode_t  update);

/******************************************************************************/

/**
 * \brief Ĵλ
 *
 * \note Ĵbasic canжȫΪ0xff,peli canжȫΪ0x00
 *
 * @{
 */

#define AMHW_ZMF159_CAN_CMR_MASK     (0x1F)

#define AMHW_ZMF159_CAN_CMR_GTS      (4ul)  /**< \brief ˯(basic) */
#define AMHW_ZMF159_CAN_CMR_SRR      (4ul)  /**< \brief Խ(peli) */
#define AMHW_ZMF159_CAN_CMR_CDO      (3ul)  /**< \brief  */
#define AMHW_ZMF159_CAN_CMR_RRB      (2ul)  /**< \brief ͷŽջ */
#define AMHW_ZMF159_CAN_CMR_AT       (1ul)  /**< \brief ֹ */
#define AMHW_ZMF159_CAN_CMR_TR       (0ul)  /**< \brief  */

/**
 * @}
 */

/**
 * \brief дcan
 *
 * \param[in] p_hw_can   : ָCANĴָ
 * \param[in] cmr        : غ궨壬οAMHW_ZMF159_CAN_CMR_
 *
 * \return 
 */
am_static_inline
void amhw_zmf159_can_cmr_set (amhw_zmf159_can_t *p_hw_can, uint32_t cmr)
{
    p_hw_can->cmr |= (0x1 << cmr) & AMHW_ZMF159_CAN_CMR_MASK;
}

/**
 * \brief basic CAN˯ģʽ
 *
 * \param[in] p_hw_can   : ָCANĴָ
 *
 * \return 
 */
am_static_inline
void amhw_zmf159_basic_can_go_to_sleep (amhw_zmf159_can_t *p_hw_can)
{
    p_hw_can->cmr |= (0x1 << AMHW_ZMF159_CAN_CMR_GTS);
}

/**
 * \brief basic CAN
 *
 * \param[in] p_hw_can   : ָCANĴָ
 *
 * \return 
 */
am_static_inline
void amhw_zmf159_basic_can_wake_up (amhw_zmf159_can_t *p_hw_can)
{
    p_hw_can->cmr &= ~(0x1 << AMHW_ZMF159_CAN_CMR_GTS);
}
/******************************************************************************/

/**
 * \brief ״̬Ĵλ
 * @{
 */

#define AMHW_ZMF159_CAN_SR_MASK         (0xFF)

#define AMHW_ZMF159_CAN_SR_BS           (0x1 << 7ul)  /**< \brief ״̬λ */
#define AMHW_ZMF159_CAN_BUS_CLOSE       (1ul)         /**< \brief ߹ر */
#define AMHW_ZMF159_CAN_BUS_OPEN        (0ul)         /**< \brief ߿ */

#define AMHW_ZMF159_CAN_SR_ES           (0x1 << 6ul)  /**< \brief ״̬λ */
#define AMHW_ZMF159_CAN_ERROR           (1ul)         /**< \brief  */
#define AMHW_ZMF159_CAN_OK              (0ul)         /**< \brief OK */

#define AMHW_ZMF159_CAN_SR_TS           (0x1 << 5ul)  /**< \brief ״̬λ */
#define AMHW_ZMF159_CAN_TRANS_RUN       (1ul)         /**< \brief  */
#define AMHW_ZMF159_CAN_TRANS_IDLE      (0ul)         /**< \brief  */

#define AMHW_ZMF159_CAN_SR_RS           (0x1 << 4ul)  /**< \brief ״̬λ */
#define AMHW_ZMF159_CAN_RECV_RUN        (1ul)         /**< \brief  */
#define AMHW_ZMF159_CAN_RECV_IDLE       (0ul)         /**< \brief  */

#define AMHW_ZMF159_CAN_SR_TCS          (0x1 << 3ul)  /**< \brief ״̬λ */
#define AMHW_ZMF159_CAN_TRANS_FINISH    (1ul)         /**< \brief  */
#define AMHW_ZMF159_CAN_TRANS_FINISH_NO (0ul)         /**< \brief δ */

#define AMHW_ZMF159_CAN_SR_TBS          (0x1 << 2ul)  /**< \brief ͻ״̬λ */
#define AMHW_ZMF159_CAN_TX_BUF_UNLOCK   (1ul)         /**< \brief ͻͷ */
#define AMHW_ZMF159_CAN_TX_BUF_LOCK     (0ul)         /**< \brief ͻ */

#define AMHW_ZMF159_CAN_SR_DOS          (0x1 << 1ul)  /**< \brief ״̬λ */
#define AMHW_ZMF159_CAN_DATA_OVER       (1ul)         /**< \brief  */
#define AMHW_ZMF159_CAN_DATA_OVER_NO    (0ul)         /**< \brief ȱ */

#define AMHW_ZMF159_CAN_SR_RBS          (0x1 << 0ul)  /**< \brief ջ״̬ */
#define AMHW_ZMF159_CAN_RX_BUF_FULL     (1ul)         /**< \brief  */
#define AMHW_ZMF159_CAN_RX_BUF_EMPTY    (0ul)         /**< \brief  */

/**
 * @}
 */

/**
 * \brief ȡCAN״̬
 *
 * \param[in] p_hw_can   : ָCANĴָ
 *
 * \return CAN״̬
 */
am_static_inline
uint32_t amhw_zmf159_can_status_get (amhw_zmf159_can_t *p_hw_can)
{
    return (p_hw_can->sr & AMHW_ZMF159_CAN_SR_MASK);
}

/******************************************************************************/

/**
 * \brief Ԥֵ
 *
 * \param[in] p_hw_can   : ָCANĴָ
 * \param[in] brp        : Ԥֵ6bit
 *
 * \return 
 */
am_static_inline
void amhw_zmf159_can_brp_set (amhw_zmf159_can_t *p_hw_can, uint8_t brp)
{
    p_hw_can->btr0 = (p_hw_can->btr0 & (~0x3f)) | (brp & 0x3f);
}

/**
 * \brief ȡԤֵ
 *
 * \param[in] p_hw_can   : ָCANĴָ
 *
 * \return Ԥֵ6bit
 */
am_static_inline
uint8_t amhw_zmf159_can_brp_get (amhw_zmf159_can_t *p_hw_can)
{
    return (uint8_t)(p_hw_can->btr0 & 0x3f);
}

/**
 * \brief ͬת
 *
 * \param[in] p_hw_can   : ָCANĴָ
 * \param[in] sjw        : ͬתֵ2bit
 *
 * \return 
 */
am_static_inline
void amhw_zmf159_can_sjw_set (amhw_zmf159_can_t *p_hw_can, uint8_t sjw)
{
    p_hw_can->btr0 = (p_hw_can->btr0 & (~(0x3 << 6))) | ((sjw & 0x3) << 6);
}

/**
 * \brief ȡͬת
 *
 * \param[in] p_hw_can   : ָCANĴָ
 *
 * \return ͬתȣ2bit
 */
am_static_inline
uint8_t amhw_zmf159_can_sjw_get (amhw_zmf159_can_t *p_hw_can)
{
    return (uint8_t)((p_hw_can->btr0 >> 6) & 0x3);
}

/**
 * \brief ģʽ
 *
 * \param[in] p_hw_can   : ָCANĴָ
 * \param[in] sam        : ģʽֵ1bitȣ1 ߲
 *                                         0 ߲һ
 *
 * \return 
 */
am_static_inline
void amhw_zmf159_can_sam_set (amhw_zmf159_can_t *p_hw_can, uint8_t sam)
{
    p_hw_can->btr1 = (p_hw_can->btr1 & (~(0x1 << 7))) | ((sam & 0x1) << 7);
}

/**
 * \brief ȡģʽ
 *
 * \param[in] p_hw_can   : ָCANĴָ
 *
 * \retval    0 ߲һ
 *            1 ߲
 */
am_static_inline
uint8_t amhw_zmf159_can_sam_get (amhw_zmf159_can_t *p_hw_can)
{
    return (uint8_t)((p_hw_can->btr1 >> 7) & 0x1);
}

/**
 * \brief ʱ1
 *
 * \param[in] p_hw_can   : ָCANĴָ
 * \param[in] tseg1        : ģʽֵ4bit
 *
 * \return 
 */
am_static_inline
void amhw_zmf159_can_tseg1_set (amhw_zmf159_can_t *p_hw_can, uint8_t tseg1)
{
    p_hw_can->btr1 = (p_hw_can->btr1 & (~0xf)) | (tseg1 & 0xf);
}

/**
 * \brief ȡʱ1
 *
 * \param[in] p_hw_can   : ָCANĴָ
 *
 * \return ʱ14bit
 */
am_static_inline
uint8_t amhw_zmf159_can_tseg1_get (amhw_zmf159_can_t *p_hw_can)
{
    return (uint8_t)(p_hw_can->btr1 & 0xf);
}

/**
 * \brief ʱ2
 *
 * \param[in] p_hw_can   : ָCANĴָ
 * \param[in] tseg2        : ģʽֵ3bit
 *
 * \return 
 */
am_static_inline
void amhw_zmf159_can_tseg2_set (amhw_zmf159_can_t *p_hw_can, uint8_t tseg2)
{
    p_hw_can->btr1 = (p_hw_can->btr1 & (~(0x7 << 4))) | ((tseg2 & 0x7) << 4);
}

/**
 * \brief ʱ2
 *
 * \param[in] p_hw_can ָCANĴָ
 *
 * \return 
 */
am_static_inline
uint8_t amhw_zmf159_can_tseg2_get (amhw_zmf159_can_t *p_hw_can)
{
    return (uint8_t)((p_hw_can->btr1 >> 4) & 0x7);
}

/******************************************************************************/

/**
 * \brief basicCANλ
 *
 * \param[in] p_hw_can   : ָCANĴָ
 *
 * \return 
 */
am_static_inline
void amhw_zmf159_basic_can_reset_set (amhw_zmf159_can_t *p_hw_can)
{
    p_hw_can->mode.cr |= 0x1;
}

/**
 * \brief basicCANλ
 *
 * \param[in] p_hw_can   : ָCANĴָ
 *
 * \return 
 */
am_static_inline
void amhw_zmf159_basic_can_reset_clr (amhw_zmf159_can_t *p_hw_can)
{
    p_hw_can->mode.cr &= ~0x1;
}

/**
 * \brief ȡbasicCANλ־
 *
 * \param[in] p_hw_can   : ָCANĴָ
 *
 * \return 
 */
am_static_inline
am_bool_t amhw_zmf159_basic_can_reset_get (amhw_zmf159_can_t *p_hw_can)
{
    return ((p_hw_can->mode.cr & 0x1) ? AM_TRUE : AM_FALSE);
}

/**
 * \brief basicCANմ
 *
 * \param[in] p_hw_can   : ָCANĴָ
 * \param[in] ac         : մ
 *
 * \return 
 */
am_static_inline
void amhw_zmf159_basic_can_ac_set (amhw_zmf159_can_t *p_hw_can, uint8_t ac)
{
    p_hw_can->accept.basic.acr = ac;
}

/**
 * \brief ȡbasicCANմ
 *
 * \param[in] p_hw_can   : ָCANĴָ
 *
 * \return մ
 */
am_static_inline
uint8_t amhw_zmf159_basic_can_ac_get (amhw_zmf159_can_t *p_hw_can)
{
    return (uint8_t)(p_hw_can->accept.basic.acr & 0xff);
}

/**
 * \brief basicCANֵ
 *
 * \param[in] p_hw_can   : ָCANĴָ
 * \param[in] am         : ֵ
 *
 * \return 
 */
am_static_inline
void amhw_zmf159_basic_can_am_set (amhw_zmf159_can_t *p_hw_can, uint8_t am)
{
    p_hw_can->accept.basic.amr = am;
}

/**
 * \brief ȡbasicCANֵ
 *
 * \param[in] p_hw_can   : ָCANĴָ
 *
 * \return ֵ
 */
am_static_inline
uint8_t amhw_zmf159_basic_can_am_get (amhw_zmf159_can_t *p_hw_can)
{
    return (uint8_t)(p_hw_can->accept.basic.amr & 0xff);
}

/**
 * \brief basicCANд11λͱʶ
 *
 * \param[in] p_hw_can   : ָCANĴָ
 * \param[in] id         : ʶ11bit
 *
 * \return 
 */
am_static_inline
void amhw_zmf159_basic_can_tx_id_write (amhw_zmf159_can_t *p_hw_can,
                                        uint16_t           id)
{
    p_hw_can->info.basic.txid0 = (id >> 3) & 0xff;
    p_hw_can->info.basic.txid1 = (p_hw_can->info.basic.txid1 &
                                              (~(0x7 << 5))) |
                                              ((id & 0x7) << 5);
}

/**
 * \brief basicCANԶ̱־λRTRд
 *
 * \param[in] p_hw_can   : ָCANĴָ
 * \param[in] rtr        : rtrֵ0/1
 *
 * \return 
 */
am_static_inline
void amhw_zmf159_basic_can_tx_rtr_write (amhw_zmf159_can_t *p_hw_can,
                                         uint8_t            rtr)
{
    p_hw_can->info.basic.txid1 = (p_hw_can->info.basic.txid1 &
                                              (~(0x1 << 4))) |
                                              ((rtr & 0x1) << 4);
}

/**
 * \brief basicCANдݳ
 *
 * \param[in] p_hw_can   : ָCANĴָ
 * \param[in] len        : ݳ0-8bytes
 *
 * \return 
 */
am_static_inline
void amhw_zmf159_basic_can_tx_data_len_set (amhw_zmf159_can_t *p_hw_can,
                                            uint8_t            len)
{
    p_hw_can->info.basic.txid1 = (p_hw_can->info.basic.txid1 & (~0xf)) |
                                                           (len & 0xf) ;
}

/**
 * \brief basicCANд
 *
 * \param[in] p_hw_can   : ָCANĴָ
 * \param[in] offset     : ƫ0-8
 * \param[in] val        : ֵ
 *
 * \return 
 */
am_static_inline
void amhw_zmf159_basic_can_tx_data_write (amhw_zmf159_can_t *p_hw_can,
                                          uint8_t            offset,
                                          uint8_t            val)
{
    p_hw_can->info.basic.txdr[offset] = val & 0xff;
}

/**
 * \brief basicCAN11λձʶ
 *
 * \param[in] p_hw_can   : ָCANĴָ
 *
 * \return ʶ
 */
am_static_inline
uint16_t amhw_zmf159_basic_can_rx_id_read (amhw_zmf159_can_t *p_hw_can)
{
    return (uint16_t)((p_hw_can->info.basic.rxid0 & 0xff) << 3) |
                     ((p_hw_can->info.basic.rxid1 & 0xff) >> 5) ;
}

/**
 * \brief basicCANԶ̱־λRTR
 *
 * \param[in] p_hw_can   : ָCANĴָ
 *
 * \return rtrֵ
 */
am_static_inline
uint8_t amhw_zmf159_basic_can_rx_rtr_read (amhw_zmf159_can_t *p_hw_can)
{
    return (uint8_t)((p_hw_can->info.basic.rxid1 >> 4) & 0x1);
}

/**
 * \brief basicCANݳ
 *
 * \param[in] p_hw_can   : ָCANĴָ
 *
 * \return ݳ0-8bytes
 */
am_static_inline
uint8_t amhw_zmf159_basic_can_rx_data_len_read (amhw_zmf159_can_t *p_hw_can)
{
    return (uint8_t)(p_hw_can->info.basic.rxid1 & 0xf);
}

/**
 * \brief basicCANյ
 *
 * \param[in] p_hw_can   : ָCANĴָ
 * \param[in] offset     : ƫ0-8
 *
 * \return ֵ
 */
am_static_inline
uint8_t amhw_zmf159_basic_can_rx_data_read (amhw_zmf159_can_t *p_hw_can,
                                            uint8_t            offset)
{
    return (uint8_t)(p_hw_can->info.basic.rxdr[offset] & 0xff);
}

/******************************************************************************/

/**
 * \brief ģʽĴλ
 * @{
 */

#define AMHW_ZMF159_CAN_PELI_MODE_SM       (4ul)  /**< \brief ˯ģʽ */
#define AMHW_ZMF159_CAN_PELI_MODE_AFM      (3ul)  /**< \brief ˲ģʽ */
#define AMHW_ZMF159_CAN_PELI_MODE_STM      (2ul)  /**< \brief Լģʽ */
#define AMHW_ZMF159_CAN_PELI_MODE_LOM      (1ul)  /**< \brief ֻģʽ */
#define AMHW_ZMF159_CAN_PELI_MODE_RM       (0ul)  /**< \brief λģʽ */

/**
 * @}
 */

/**
 * \brief peli CANģʽ
 *
 * \param[in] p_hw_can   : ָCANĴָ
 * \param[in] flag       : οAMHW_ZMF159_CAN_PELI_MODE_
 *
 * \return 
 */
am_static_inline
void amhw_zmf159_peli_can_mod_set (amhw_zmf159_can_t *p_hw_can,
                                   uint32_t           flag)
{
    p_hw_can->mode.mod |= (0x1 << flag);
}

/**
 * \brief peli CANģʽ
 *
 * \param[in] p_hw_can   : ָCANĴָ
 * \param[in] flag       : οAMHW_ZMF159_CAN_PELI_MODE_
 *
 * \return 
 */
am_static_inline
void amhw_zmf159_peli_can_mod_clr (amhw_zmf159_can_t *p_hw_can,
                                   uint32_t           flag)
{
    p_hw_can->mode.mod &= ~(0x1 << flag);
}

/**
 * \brief peli CANģʽȡ
 *
 * \param[in] p_hw_can   : ָCANĴָ
 *
 * \return ģʽ
 */
am_static_inline
uint8_t amhw_zmf159_peli_can_mod_get (amhw_zmf159_can_t *p_hw_can)
{
    return (uint8_t)((p_hw_can->mode.mod & 0x1f) &
                            ~(0x1 << AMHW_ZMF159_CAN_PELI_MODE_AFM));
}

/**
 * \brief peli CAN˲ģʽ
 *
 * \param[in] p_hw_can   : ָCANĴָ
 * \param[in] mode       : οöamhw_zmf159_can_peli_filter_mode_t
 *
 * \return 
 */
am_static_inline
void amhw_zmf159_peli_filter_mod_set (amhw_zmf159_can_t *p_hw_can,
                           amhw_zmf159_can_peli_filter_mode_t mode)
{
    if (AMHW_ZMF159_CAN_PELI_SINGLE_FILTER == mode) {
        p_hw_can->mode.mod |= 0x1 << AMHW_ZMF159_CAN_PELI_MODE_AFM;
    } else if (AMHW_ZMF159_CAN_PELI_DOUBLE_FLITER == mode) {
        p_hw_can->mode.mod &= ~(0x1 << AMHW_ZMF159_CAN_PELI_MODE_AFM);
    }
}

/**
 * \brief ȡpeli CAN˲ģʽ
 *
 * \param[in] p_hw_can   : ָCANĴָ
 *
 * \return οöamhw_zmf159_can_peli_filter_mode_t
 */
am_static_inline amhw_zmf159_can_peli_filter_mode_t
amhw_zmf159_peli_filter_mod_get (amhw_zmf159_can_t *p_hw_can)
{
    return (p_hw_can->mode.mod & (0x1 << AMHW_ZMF159_CAN_PELI_MODE_AFM) ?
                                     AMHW_ZMF159_CAN_PELI_SINGLE_FILTER :
                                     AMHW_ZMF159_CAN_PELI_DOUBLE_FLITER);
}

/**
 * \brief peli CAN ٲöʧ׽Ĵֵȡ
 *
 * \param[in] p_hw_can   : ָCANĴָ
 *
 * \return Ĵֵ,ֵӦ
 *
 * \note 1. 0 - 11ʾٲöʧʶbit1 - bit11
 *       2. 11ʾٲöʧSRTRλ
 *       3. 12ʾٲöʧIDEλ
 *       4. 13 - 30ʾٲöʧʶbit12 - bit29
 *       5. 31ʾٲöʧRTRλ
 */
am_static_inline
uint8_t amhw_zmf159_peli_can_alc_get (amhw_zmf159_can_t *p_hw_can)
{
    return (uint8_t)(p_hw_can->info.peli.alc & 0x1f);
}

/**
 * \brief peli CAN
 * @{
 */

#define AMHW_ZMF159_CAN_PELI_ECC_ERRC_MASK    (0x3)  /**< \brief  */
#define AMHW_ZMF159_CAN_PELI_ECC_ERRC_SHIFT   (6ul)  /**< \brief ʼλ */
#define AMHW_ZMF159_CAN_PELI_ECC_ERRC_BIT     (0ul)  /**< \brief λ */
#define AMHW_ZMF159_CAN_PELI_ECC_ERRC_MODE    (1ul)  /**< \brief ʽ */
#define AMHW_ZMF159_CAN_PELI_ECC_ERRC_FILL    (2ul)  /**< \brief  */
#define AMHW_ZMF159_CAN_PELI_ECC_ERRC_OTHER   (3ul)  /**< \brief  */

/** \brief 1:RX;0:TX */
#define AMHW_ZMF159_CAN_PELI_ECC_DIR_RX             (0x1 << 3ul)

#define AMHW_ZMF159_CAN_PELI_ECC_CODE_MASK          (0x1f)
#define AMHW_ZMF159_CAN_PELI_ECC_CODE_SHIFT         (0ul)

/** \brief ID.28-ID.21 */
#define AMHW_ZMF159_CAN_PELI_ECC_CODE_ID28_21       (2ul)
/** \brief ֡ʼ */
#define AMHW_ZMF159_CAN_PELI_ECC_CODE_FRAME_STRAT   (3ul)
/** \brief SRTRλ */
#define AMHW_ZMF159_CAN_PELI_ECC_CODE_SRTR          (4ul)
/** \brief IDEλ */
#define AMHW_ZMF159_CAN_PELI_ECC_CODE_IDE           (5ul)
/** \brief ID.20-ID.18 */
#define AMHW_ZMF159_CAN_PELI_ECC_CODE_ID20_18       (6ul)
/** \brief ID.17-ID.13 */
#define AMHW_ZMF159_CAN_PELI_ECC_CODE_ID17_13       (7ul)
/** \brief CRC */
#define AMHW_ZMF159_CAN_PELI_ECC_CODE_CRC_ARRAY     (8ul)
/** \brief λ0 */
#define AMHW_ZMF159_CAN_PELI_ECC_CODE_RESERVE0      (9ul)
/** \brief  */
#define AMHW_ZMF159_CAN_PELI_ECC_CODE_DATA          (10ul)
/** \brief ݳȴ */
#define AMHW_ZMF159_CAN_PELI_ECC_CODE_DATA_LEN      (11ul)
/** \brief RTRλ */
#define AMHW_ZMF159_CAN_PELI_ECC_CODE_RTR           (12ul)
/** \brief λ1 */
#define AMHW_ZMF159_CAN_PELI_ECC_CODE_RESERVR1      (13ul)
/** \brief ID.4-ID.0 */
#define AMHW_ZMF159_CAN_PELI_ECC_CODE_ID4_0         (14ul)
/** \brief ID.12-ID.5 */
#define AMHW_ZMF159_CAN_PELI_ECC_CODE_ID12_5        (15ul)
/** \brief ־ */
#define AMHW_ZMF159_CAN_PELI_ECC_CODE_ACTIVITY_ERR  (17ul)
/** \brief ֹ */
#define AMHW_ZMF159_CAN_PELI_ECC_CODE_BREAK_OFF     (18ul)
/** \brief ֧䣨ƣλ */
#define AMHW_ZMF159_CAN_PELI_ECC_CODE_CR_BIT        (19ul)
/** \brief ־ */
#define AMHW_ZMF159_CAN_PELI_ECC_CODE_PASSIVE       (22ul)
/** \brief  */
#define AMHW_ZMF159_CAN_PELI_ECC_CODE_ERR           (23ul)
/** \brief CRC */
#define AMHW_ZMF159_CAN_PELI_ECC_CODE_CRC           (24ul)
/** \brief Ӧͨ */
#define AMHW_ZMF159_CAN_PELI_ECC_CODE_ACK_CH        (25ul)
/** \brief ֡ */
#define AMHW_ZMF159_CAN_PELI_ECC_CODE_FRAME_END     (26ul)
/** \brief Ӧ */
#define AMHW_ZMF159_CAN_PELI_ECC_CODE_ACK           (27ul)
/** \brief ־ */
#define AMHW_ZMF159_CAN_PELI_ECC_CODE_OVERFLOW      (28ul)

/**
 * @}
 */

/**
 * \brief peli CAN 벶׽Ĵֵȡ
 *
 * \param[in] p_hw_can   : ָCANĴָ
 *
 * \return Ĵֵ
 */
am_static_inline
uint8_t amhw_zmf159_peli_can_ecc_get (amhw_zmf159_can_t *p_hw_can)
{
    return (uint8_t)(p_hw_can->info.peli.ecc & 0xff);
}

/**
 * \brief peli CAN 󱨾ƼĴд
 *
 * \param[in] p_hw_can   : ָCANĴָ
 * \param[in] val        : ֵ
 *
 * \return 
 */
am_static_inline
void amhw_zmf159_peli_can_ewlr_write (amhw_zmf159_can_t *p_hw_can,
                                      uint8_t            val)
{
    p_hw_can->info.peli.ewlr = val & 0xff;
}

/**
 * \brief peli CAN 󱨾ƼĴȡ
 *
 * \param[in] p_hw_can   : ָCANĴָ
 *
 * \return ֵ
 */
am_static_inline
uint8_t  amhw_zmf159_peli_can_ewlr_read (amhw_zmf159_can_t *p_hw_can)
{
    return (uint8_t)(p_hw_can->info.peli.ewlr & 0xff);
}

/**
 * \brief peli CAN RXĴд
 *
 * \param[in] p_hw_can   : ָCANĴָ
 * \param[in] init       : ʼֵ
 *
 * \return 
 */
am_static_inline
void amhw_zmf159_peli_can_rxerr_write (amhw_zmf159_can_t *p_hw_can,
                                       uint8_t            init)
{
    p_hw_can->info.peli.rxerr = init & 0xff;
}

/**
 * \brief peli CAN RXĴȡ
 *
 * \param[in] p_hw_can   : ָCANĴָ
 *
 * \return ֵ
 */
am_static_inline
uint8_t  amhw_zmf159_peli_can_rxerr_read (amhw_zmf159_can_t *p_hw_can)
{
    return (uint8_t)(p_hw_can->info.peli.rxerr & 0xff);
}

/**
 * \brief peli CAN TXĴд
 *
 * \param[in] p_hw_can   : ָCANĴָ
 * \param[in] init       : ʼֵ
 *
 * \return 
 */
am_static_inline
void amhw_zmf159_peli_can_txerr_write (amhw_zmf159_can_t *p_hw_can,
                                       uint8_t            init)
{
    p_hw_can->info.peli.txerr = init & 0xff;
}

/**
 * \brief peli CAN TXĴȡ
 *
 * \param[in] p_hw_can   : ָCANĴָ
 *
 * \return ֵ
 */
am_static_inline
uint8_t  amhw_zmf159_peli_can_txerr_read (amhw_zmf159_can_t *p_hw_can)
{
    return (uint8_t)(p_hw_can->info.peli.txerr & 0xff);
}

/**
 * \brief ffĴλ
 *
 * \note ffĴֻһȫд룬һλһλд
 *
 * @{
 */

#define AMHW_ZMF159_CAN_PELI_SFF_FF         (0x1 << 7)  /**< \brief Զ̱־λ */
#define AMHW_ZMF159_CAN_PELI_SFF_RTR        (0x1 << 6)  /**< \brief RTR־ */

#define AMHW_ZMF159_CAN_PELI_SFF_DLC_SHIFT  (0ul)       /**< \brief DLCƶλ */
#define AMHW_ZMF159_CAN_PELI_SFF_DLC_MASK   (0xf)       /**< \brief DLC */

/**
 * @}
 */

/**
 * \brief peli CAN SFF
 *
 * \param[in] p_hw_can   : ָCANĴָ
 * \param[in] val        : ֵ
 *
 * \return 
 */
am_static_inline
void amhw_zmf159_peli_can_sff_set (amhw_zmf159_can_t *p_hw_can,
                                   uint8_t            val)
{
    p_hw_can->info.peli.sff = val;
}

/**
 * \brief peli CAN SFFȡ
 *
 * \param[in] p_hw_can   : ָCANĴָ
 *
 * \return Ĵֵ
 */
am_static_inline
uint8_t amhw_zmf159_peli_can_sff_get (amhw_zmf159_can_t *p_hw_can)
{
    return (p_hw_can->info.peli.sff & 0xff);
}

/**
 * \brief peli CAN ID
 *
 * \param[in] p_hw_can   : ָCANĴָ
 * \param[in] is_extend  : AM_TRUE չ֡AM_FALSE ׼֡
 * \param[in] id         : id
 *
 * \return 
 */
am_static_inline
void  amhw_zmf159_peli_can_id_set (amhw_zmf159_can_t *p_hw_can,
                                   am_bool_t          is_extend,
                                   uint32_t           id)
{
    if (AM_TRUE == is_extend) {
        p_hw_can->info.peli.data[1] = (id & 0x1f) << 3;
        p_hw_can->info.peli.data[0] = (id >> 5)  & 0xff;
        p_hw_can->info.peli.id1     = (id >> 13) & 0xff;
        p_hw_can->info.peli.id0     = (id >> 21) & 0xff;
    } else {
        p_hw_can->info.peli.id0 = (id >> 3) & 0xff;
        p_hw_can->info.peli.id1 = (id & 0x7) << 5;
    }
}

/**
 * \brief peli CAN IDȡ
 *
 * \param[in] p_hw_can   : ָCANĴָ
 * \param[in] is_extend  : AM_TRUE չ֡AM_FALSE ׼֡
 *
 * \return id
 */
am_static_inline
uint32_t amhw_zmf159_peli_can_id_get (amhw_zmf159_can_t *p_hw_can,
                                      am_bool_t          is_extend)
{
    uint32_t temp = 0;

    if (AM_TRUE == is_extend) {
        temp |= (p_hw_can->info.peli.id0 & 0xff) << 21;
        temp |= (p_hw_can->info.peli.id1 & 0xff) << 13;
        temp |= (p_hw_can->info.peli.data[0] & 0xff) << 5;
        temp |= (p_hw_can->info.peli.data[1] >> 3) & 0x1f;
    } else {
        temp |= (p_hw_can->info.peli.id0 & 0xff) << 3;
        temp |= (p_hw_can->info.peli.id1 >> 5) & 0x7;
    }

    return temp;
}

/**
 * \brief peli CAN д
 *
 * \param[in] p_hw_can   : ָCANĴָ
 * \param[in] is_extend  : AM_TRUE չ֡AM_FALSE ׼֡
 * \param[in] offset     : ƫ
 * \param[in] val        : 
 *
 * \return 
 */
am_static_inline
void  amhw_zmf159_peli_can_data_write (amhw_zmf159_can_t *p_hw_can,
                                       am_bool_t          is_extend,
                                       uint8_t            offset,
                                       uint8_t            val)
{
    if (AM_TRUE == is_extend) {
        p_hw_can->info.peli.data[offset + 2] = val;
    } else {
        p_hw_can->info.peli.data[offset] = val;
    }
}

/**
 * \brief peli CAN ݶȡ
 *
 * \param[in] p_hw_can   : ָCANĴָ
 * \param[in] is_extend  : AM_TRUE չ֡AM_FALSE ׼֡
 * \param[in] offset     : ƫ
 *
 * \return 
 */
am_static_inline
uint8_t amhw_zmf159_peli_can_data_read (amhw_zmf159_can_t *p_hw_can,
                                        am_bool_t          is_extend,
                                        uint8_t            offset)
{
    if (AM_TRUE == is_extend) {
        return (uint8_t)(p_hw_can->info.peli.data[offset + 2] & 0xff);
    } else {
        return (uint8_t)(p_hw_can->info.peli.data[offset] & 0xff);
    }
}

/**
 * \brief peli CAN մ
 *
 * \param[in] p_hw_can   : ָCANĴָ
 * \param[in] ac         : ˲ֵ
 *
 * \return 
 */
am_static_inline
void amhw_zmf159_peli_can_ac_set (amhw_zmf159_can_t *p_hw_can,
                                  uint32_t           ac)
{
    p_hw_can->info.peli.data[0] = ac & 0xff;
    p_hw_can->info.peli.id1     = (ac >> 8) & 0xff;
    p_hw_can->info.peli.id0     = (ac >> 16) & 0xff;
    p_hw_can->info.peli.sff     = (ac >> 24) & 0xff;
}

/**
 * \brief peli CAN մȡ
 *
 * \param[in] p_hw_can   : ָCANĴָ
 *
 * \return ac
 */
am_static_inline
uint32_t amhw_zmf159_peli_can_ac_get (amhw_zmf159_can_t *p_hw_can)
{
    uint32_t temp = 0;

    temp |= (p_hw_can->info.peli.sff     & 0xff) << 24;
    temp |= (p_hw_can->info.peli.id0     & 0xff) << 16;
    temp |= (p_hw_can->info.peli.id1     & 0xff) << 8;
    temp |= (p_hw_can->info.peli.data[0] & 0xff);

    return temp;
}

/**
 * \brief peli CAN μĴ
 *
 * \param[in] p_hw_can   : ָCANĴָ
 * \param[in] am         : ֵ
 *
 * \return 
 */
am_static_inline
void amhw_zmf159_peli_can_am_set (amhw_zmf159_can_t *p_hw_can,
                                  uint32_t           am)
{
    p_hw_can->info.peli.data[4] = am & 0xff;
    p_hw_can->info.peli.data[3] = (am >> 8) & 0xff;
    p_hw_can->info.peli.data[2] = (am >> 16) & 0xff;
    p_hw_can->info.peli.data[1] = (am >> 24) & 0xff;
}

/**
 * \brief peli CAN μĴȡ
 *
 * \param[in] p_hw_can   : ָCANĴָ
 *
 * \return am
 */
am_static_inline
uint32_t amhw_zmf159_peli_can_am_get (amhw_zmf159_can_t *p_hw_can)
{
    uint32_t temp = 0;

    temp |= (p_hw_can->info.peli.data[1] & 0xff) << 24;
    temp |= (p_hw_can->info.peli.data[2] & 0xff) << 16;
    temp |= (p_hw_can->info.peli.data[3] & 0xff) << 8;
    temp |= (p_hw_can->info.peli.data[4] & 0xff);

    return temp;
}

/******************************************************************************/

/**
 * \brief CANģʽãڸλд
 *
 * \param[in] p_hw_can   : ָCANĴָ
 * \param[in] type       : öamhw_zmf159_can_mode_t
 *
 * \return wu
 */
am_static_inline
void amhw_zmf159_can_type_set (amhw_zmf159_can_t      *p_hw_can,
                               amhw_zmf159_can_type_t  type)
{
    if (type == AMHW_ZMF159_CAN_BASIC_CAN) {
        p_hw_can->cdr &= ~(0x1 << 7);
    } else if (type == AMHW_ZMF159_CAN_PELI_CAN) {
        p_hw_can->cdr |= (0x1 << 7);
    }
}

/**
 * @} amhw_zmf159_if_can
 */

#ifdef __cplusplus
}
#endif

#endif /* __AMHW_ZMF159_CAN_H */

/* end of file */
