
/*******************************************************************************
*                                 AMetal
*                       ----------------------------
*                       innovating embedded platform
*
* Copyright (c) 2001-2018 Guangzhou ZHIYUAN Electronics Co., Ltd.
* All rights reserved.
*
* Contact information:
* web site:    http://www.zlg.cn/
*******************************************************************************/

/**
 * \file
 * \brief CLKʱӱ׼ӿ
 *
 * \internal
 * \par Modification history
 * - 1.00 19-06-27  yrh, first implementation
 * \endinternal
 */

#ifndef __AM_ZLG237_CLK_H
#define __AM_ZLG237_CLK_H

#ifdef __cplusplus
extern "C" {
#endif

#include "ametal.h"
#include "am_clk.h"
#include "amhw_zlg237_rcc.h"

/**
 * \addtogroup am_zlg237_if_clk
 * \copydoc am_zlg237_clk.h
 * @{
 */
/**
 * \brief ϵͳʱԴ
 */
typedef enum {
    AM_ZLG237_SYSCLK_HSI, /**< \brief HSI Ϊϵͳʱ */
    AM_ZLG237_SYSCLK_HSE, /**< \brief HSE Ϊϵͳʱ */
    AM_ZLG237_SYSCLK_PLL, /**< \brief PLL Ϊϵͳʱ */
    AM_ZLG237_SYSCLK_LSI, /**< \brief LSI Ϊϵͳʱ */
    AM_ZLG237_SYSCLK_LSE  /**< \brief LSE Ϊϵͳʱ */
}am_zlg237_sys_clk_src;

/**
 * \brief CLK 豸Ϣṹ
 */
typedef struct am_zlg237_clk_devinfo {
    /**
     * \brief ϵͳʱԴѡ
     *
     * AM_ZLG237_SYSCLK_HSI : HSI Ϊϵͳʱ
     * AM_ZLG237_SYSCLK_HSE : HSE Ϊϵͳʱ
     * AM_ZLG237_SYSCLK_PLL : PLL Ϊϵͳʱ
     * AM_ZLG237_SYSCLK_LSI : LSI Ϊϵͳʱ
     * AM_ZLG237_SYSCLK_LSE : LSE Ϊϵͳʱ
     */

    uint32_t sysclk_src;
    /**
     * \brief HSEOSCⲿƵ
     *
     *  pllin_srcѡ AMHW_ZLG237_PLLCLK_HSE  PLLIN = hse_osc
     */
    uint32_t hse_osc;

    /**
     * \brief LSEOSCⲿƵ
     *
     *  rtc_srcѡ AMHW_ZLG237_RTCCLK_LSE  rtcclk = lse_osc
     */
    uint32_t lse_osc;

    /** \brief
     *    PLL ʱԴѡ
     *    -# AMHW_ZLG237_PLLCLK_HSI : HSI ΪPLLʱ
     *    -# AMHW_ZLG237_PLLCLK_HSE : HSE ΪPLLʱ
     */
    amhw_zlg237_pll_clk_src pllin_src;

    /**
     * \brief PLL Ƶϵѡ2-17
     *        PLLOUT = PLLIN * pll_mul
     */
    uint8_t pll_mul;

    /** \brief USBƵϵ
     *
     *  AMHW_ZLG237_PLL_USB_DIV1P5 : PLL 1.5 ƵΪusbʱ ,USBCLK = PLLOUT / 1.5;
     *  AMHW_ZLG237_PLL_USB_DIV1   : PLL 1 ƵΪusbʱ ,  USBCLK = PLLOUT / 1
     *
     *  ó48Mhz
     */
    amhw_zlg237_pll_usb_clk_div usb_div;

    /**
     * \brief AHBƵϵAHBCLK = PLLOUT / DIV,AHBƵΪ80Mhz
     *
     *    ahb_div |  DIV
     *   ---------------------
     *        0-7 |   1
     *        8   |   2
     *        9   |   4
     *        10  |   8
     *        11  |   16
     *        12  |   64
     *        13  |   128
     *        14  |   256
     *        15  |   512
     */
    uint8_t ahb_div;

    /**
     * \brief APB1ƵϵAPB1CLK = AHBCLK / (2 ^ apb1_div)
     *        APB1ƵΪ80Mhz
     */
    uint8_t apb1_div;

    /**
     * \brief APB2ƵϵAPB2CLK = AHBCLK / (2 ^ apb2_div)
     *        APB2ƵΪ36Mhz
     */
    uint8_t apb2_div;

    /**
     * \brief ADCƵϵADCCLK = APB2 / DIV
     *    adc_div |  DIV
     *   -----------------
     *          0 | 2 Ƶ
     *          1 | 4 Ƶ
     *          2 | 8 Ƶ
     *          3 | 16Ƶ
     *
     *        ADCƵΪ14Mhz
     */
    uint8_t adc_div;

    /** \brief ƽ̨ʼŵȹ */
    void  (*pfn_plfm_init)(void);

    /** \brief ƽ̨ʼ */
    void  (*pfn_plfm_deinit)(void);

} am_zlg237_clk_devinfo_t;

/**
 * \brief CLK 豸ṹ
 */
typedef struct am_zlg237_clk_dev {

    /** \brief ָCLK 豸Ϣָ */
    const am_zlg237_clk_devinfo_t *p_devinfo;

    /** \brief PLLƵ */
    uint32_t sys_clk;

    /** \brief PLLƵ */
    uint32_t pllin_clk;

    /** \brief PLLƵ */
    uint32_t pllout_clk;

    /** \brief AHBʱƵ */
    uint32_t ahb_clk;

    /** \brief APB1ʱƵ */
    uint32_t apb1_clk;

    /** \brief APB2ʱƵ */
    uint32_t apb2_clk;

    /** \brief USBʱƵ */
    uint32_t usb_clk;

    /** \brief RTCʱƵ */
    uint32_t rtc_clk;

} am_zlg237_clk_dev_t;

/**
 * \brief ȡʱƵ
 */
int am_clk_rate_get (am_clk_id_t clk_id);

/**
 * \brief CLK ʼ
 *
 * \param[in] p_dev     : ָCLK豸ָ
 * \param[in] p_devinfo : ָCLK豸Ϣָ
 *
 * \retval AM_OK : ɹ
 */
int am_zlg237_clk_init (am_zlg237_clk_dev_t           *p_dev,
                        const am_zlg237_clk_devinfo_t *p_devinfo);

/**
 * \brief CLK 踴λ
 *
 * \param[in] clk_id  ʱ ID (ƽ̨), μ \ref grp_clk_id
 *
 * \retval AM_OK : ɹ
 */
int am_zlg237_clk_reset (am_clk_id_t clk_id);

/**
 * \brief Ƶֵ
 *
 *  ƵʱʱҪô˺ڲ洢Ƶֵ
 *  CLK_AHBCLK_APB1CLK_APB2Ч
 *
 * \param[in] clk_id  ʱ ID (ƽ̨), μ \ref grp_clk_id
 * \param[in] clk     µƵֵ
 *
 * \retval AM_OK : ɹ
 *
 * \note ע˺
 */
int am_zlg237_clk_update (am_clk_id_t clk_id, int clk);

/**
 * \brief ȡPLLʱԴ
 *  CLK_AHBCLK_APB1CLK_APB2Ч
 *
 * \return clkʱϢ
 */
const am_zlg237_clk_dev_t *am_zlg237_clk_devinfo_get (void);

/**
 * \brief ߻CLKƵֵ
 *
 *
 * \param[in] clk_id  ʱ ID (ƽ̨), μ \ref grp_clk_id
 *
 * \return: ضӦ߻ķƵֵ
 *
 */
int am_zlg237_div_get (am_clk_id_t clk_id);

/**
 * @}
 */

#ifdef __cplusplus
}
#endif

#endif /* __AM_ZLG237_CLK_H */

/* end of file */
