/*******************************************************************************
*                                 AMetal
*                       ----------------------------
*                       innovating embedded platform
*
* Copyright (c) 2001-2018 Guangzhou ZHIYUAN Electronics Co., Ltd.
* All rights reserved.
*
* Contact information:
* web site:    http://www.zlg.cn/
*******************************************************************************/

/**
 * \file
 * \brief CLKʱӱ׼ӿ
 *
 * \internal
 * \par Modification history
 * - 1.00 20-03-18  yrz, first implementation
 * \endinternal
 */

#ifndef __AM_ZSN700_CLK_H
#define __AM_ZSN700_CLK_H

#ifdef __cplusplus
extern "C" {
#endif

#include "ametal.h"
#include "am_clk.h"
#include "hw/amhw_zsn700_rcc.h"

/**
 * \addtogroup am_zsn700_if_clk
 * \copydoc am_zsn700_clk.h
 * @{
 */

/**
 * \brief CLK 豸Ϣṹ
 */
typedef struct am_zsn700_clk_devinfo {

    /**
     * \brief XTHOSCⲿƵ
     */
    uint32_t xth_osc;

    /**
     * \brief XTLOSCⲿƵ
     */
    uint32_t xtl_osc;

    /**
     * \brief RCHڲʱӾƵʣHz
     *
     * \note Ƶѡ    24M 22.12M16M 8M4M
     *       ƵʣĬʹ4M
     */
    amhw_zsn700_rch_fre_t rch_osc;

    /**
     * \brief RCLڲʱӾƵʣHz
     *
     * \note Ƶѡ    38.4k        32.768K
     *       ƵʣĬʹ38.4k
     */
    amhw_zsn700_rcl_fre_t rcl_osc;

    /** \brief
     *    PLL ʱԴѡ
     *    -# AMHW_ZSN700_PLL_INPUT_FRE_SRC_XTH_XTAL : XTHɵʱ
     *    -# AMHW_ZSN700_PLL_INPUT_FRE_SRC_XTH_PF00 : XTHӹܽPF00ʱ
     *    -# AMHW_ZSN700_PLL_INPUT_FRE_SRC_XTH_RCH  : RCHʱ
     */
    amhw_zsn700_pll_input_fre_src pllin_src;

    /**
     * \brief PLL ƵϵΧ0X2~0XC
     *        PLLOUT = PLLIN * pll_mul
     */
    uint8_t pll_mul;

    /** \brief
     *    ϵͳʱԴѡ
     *    -# AMHW_ZSN700_SYSCLK_RCH : ڲʱΪϵͳʱ
     *    -# AMHW_ZSN700_SYSCLK_XTH : ⲿʱΪϵͳʱ
     *    -# AMHW_ZSN700_SYSCLK_RCL : ڲʱΪϵͳʱ
     *    -# AMHW_ZSN700_SYSCLK_XTL   ⲿʱΪϵͳʱ
     *    -# AMHW_ZSN700_SYSCLK_PLL  ڲPLLΪϵͳʱ
     */
    amhw_zsn700_sys_clk_src sysclk_src;

    /**
     * \brief HCLK = SYSCLK / (2 ^ hclk_div)
     */
    uint8_t hclk_div;

    /**
     * \brief PCLKƵϵPCLK = HCLK / (2 ^ pclk_div)
     */
    uint8_t pclk_div;

    /** \brief ƽ̨ʼŵȹ */
    void  (*pfn_plfm_init)(void);

    /** \brief ƽ̨ʼ */
    void  (*pfn_plfm_deinit)(void);

} am_zsn700_clk_devinfo_t;

/**
 * \brief CLK 豸ṹ
 */
typedef struct am_zsn700_clk_dev {

    /** \brief ָCLK 豸Ϣָ */
    const am_zsn700_clk_devinfo_t *p_devinfo;

    /** \brief PLLƵ */
    uint32_t pllin_clk;

    /** \brief PLLƵ */
    uint32_t pllout_clk;

    /** \brief sysƵ */
    uint32_t sys_clk;

    /** \brief sys */
    amhw_zsn700_sys_clk_src sys_type;

    /** \brief ϵͳʱƵ */
    uint32_t hclk;

    /** \brief ʱƵ */
    uint32_t pclk;
} am_zsn700_clk_dev_t;

/**
 * \brief CLK ʼ
 *
 * \param[in] p_dev     : ָCLK豸ָ
 * \param[in] p_devinfo : ָCLK豸Ϣָ
 *
 * \retval AM_OK : ɹ
 */
int am_zsn700_clk_init (am_zsn700_clk_dev_t           *p_dev,
                        const am_zsn700_clk_devinfo_t *p_devinfo);

/**
 * \brief CLK 踴λ
 *
 * \param[in] clk_id  ʱ ID (ƽ̨), μ \ref grp_clk_id
 *
 * \retval AM_OK : ɹ
 */
int am_zsn700_clk_reset (am_clk_id_t clk_id);

/**
 * \brief Ƶֵ
 *
 *  ƵʱʱҪô˺ڲ洢Ƶֵ
 *  CLK_AHBCLK_APB1CLK_APB2Ч
 *
 * \param[in] clk_id  ʱ ID (ƽ̨), μ \ref grp_clk_id
 * \param[in] clk     µƵֵ
 *
 * \retval AM_OK : ɹ
 *
 * \note ע˺
 */
int am_zsn700_clk_update (am_clk_id_t clk_id, int clk);

/**
 * \brief ȡPLLʱԴ
 *  CLK_AHBCLK_APB1CLK_APB2Ч
 *
 * \return clkʱϢ
 */
const am_zsn700_clk_dev_t *am_zsn700_clk_devinfo_get (void);

/**
 * \brief ߻CLKƵֵ
 *
 *
 * \param[in] clk_id  ʱ ID (ƽ̨), μ \ref grp_clk_id
 *
 * \return: ضӦ߻ķƵֵ
 *
 */
int am_zsn700_div_get (am_clk_id_t clk_id);

/**
 * @}
 */

#ifdef __cplusplus
}
#endif

#endif /* __AM_ZSN700_CLK_H */

/* end of file */
