/*******************************************************************************
*                                 AMetal
*                       ----------------------------
*                       innovating embedded platform
*
* Copyright (c) 2001-2018 Guangzhou ZHIYUAN Electronics Co., Ltd.
* All rights reserved.
*
* Contact information:
* web site:    http://www.zlg.cn/
*******************************************************************************/

/**
 * \file
 * \brief DMADMAӿ
 *
 * \internal
 * \par Modification History
 * - 1.00 19-09-17  zp, first implementation
 * \endinternal
 */

#ifndef __AM_ZSL42x_DMA_H
#define __AM_ZSL42x_DMA_H

#ifdef __cplusplus
extern "C" {
#endif

#include "am_types.h"
#include "hw/amhw_zsl42x_dma.h"

/**
 * \addtogroup am_zsl42x_if_dma
 * \copydoc am_zsl42x_dma.h
 * @{
 */

/**
 * \name ͨжϱ־
 * \anchor am_zsl42x_dma_chan_cfg_flags
 * @{
 */

#define AM_ZSL42x_DMA_INT_COMPLETE            0  /** \brief жϱʶ */
#define AM_ZSL42x_DMA_INT_ERR_ADDR_OVERFLOW   1  /** \brief ַ */
#define AM_ZSL42x_DMA_INT_ERR_STOPASK         2  /** \brief ֹֹͣ */
#define AM_ZSL42x_DMA_INT_ERR_SRCADDR         3  /** \brief Դַʴ */
#define AM_ZSL42x_DMA_INT_ERR_DSTADDR         4  /** \brief Ŀĵصַʴ */
#define AM_ZSL42x_DMA_INT_ERR_STOP            5  /** \brief ͣ */

/** @} */

/** \brief DMAжϻص */
typedef void (*am_zsl42x_pfn_dma_isr_t)(void *p_arg, uint32_t flag);

/** \brief DMAжϻصϢ */
struct am_zsl42x_dma_int_info {

    /** \brief DMAص */
    am_zsl42x_pfn_dma_isr_t  pfn_isr;

    /** \brief صĵһڲ */
    void                    *p_arg;
};

/** \brief DMA豸Ϣ */
typedef struct am_zsl42x_dma_devinfo {

    /** \brief ָDMAĴָ */
    uint32_t   dma_reg_base;

    /** \brief DMAжϺſʼ */
    uint8_t    inum;

    /** \brief ƽ̨ʼʱӣŵȹ */
    void     (*pfn_plfm_init)(void);

    /** \brief ƽ̨ʼ */
    void     (*pfn_plfm_deinit)(void);

} am_zsl42x_dma_devinfo_t;

/**
 * \brief DMA豸ʵ
 */
typedef struct am_zsl42x_dma_dev {

    /** \brief ָDMA豸Ϣָ */
    const am_zsl42x_dma_devinfo_t *p_devinfo;

}am_zsl42x_dma_dev_t;

/**
 * \brief ͨ0ȼ
 */
void am_zsl42x_dma_priority_ch0_highest (void);

/**
 * \brief ͨ0ͨ1ȼͬѯʽ
 */
void am_zsl42x_dma_priority_same (void);

/**
 * \brief ԴΪǣ
 */
void am_zsl42x_dma_chan_soft_ask_start (int chan);

/**
 * \brief ԴΪǣֹͣ
 */
void am_zsl42x_dma_chan_soft_ask_stop (int chan);

/**
 * \brief DMAͨ
 *
 * \param[in] chan : DMA ͨţֵΪDMA_CHAN_* (#DMA_CHAN_1)  (#DMA_CHAN_2)
 * \param[in] src  : 趨ͨĴԴ鿴zsl42x_dma_chan.h궨壩
 *
 */
void am_zsl42x_dma_chan_src_set (int chan, uint8_t src);

/**
 * \brief DMAͨ
 *
 * \param[in] chan : DMA ͨţֵΪDMA_CHAN_* (#DMA_CHAN_1)  (#DMA_CHAN_2)
 * \param[in] src  : 趨ͨĴԴ鿴zsl42x_dma_chan.h궨壩
 *
 * \retval AM_OK    : óɹ
 * \retval AM_ERROR : ô󣬸֮ͨǰù
 */
int am_zsl42x_dma_chan_start (int chan);

/**
 * \brief ֹͣͨ
 * \param[in] chan : DMA ͨţֵΪDMA_CHAN_* (#DMA_CHAN_1)  (#DMA_CHAN_2)
 *
 * \retval AM_OK   : ɹ
 */
int am_zsl42x_dma_chan_stop (int chan);

/**
 * \brief 
 *
 * \param[in] p_desc   : ָDMAָ
 * \param[in] src_addr : Դ׵ַ
 * \param[in] dst_addr : Ŀ׵ַ
 * \param[in] nbytes   : ֽ
 * \param[in] flags    : ò
 *
 * \retval  AM_OK      : ɹ
 * \retval -AM_EINVAL  : δ룬Ч
 */
int am_zsl42x_dma_xfer_desc_build (amhw_zsl42x_dma_xfer_desc_t *p_desc,
                                   uint32_t                  src_addr,
                                   uint32_t                  dst_addr,
                                   uint32_t                  nbytes,
                                   uint32_t                  flags);

/**
 * \brief ʼDMA
 *
 * \param[in] p_desc  : ָDMAָ
 * \param[in] type    : DMAģʽ жӦödma_transfer_type_t
 * \param[in] chan    : DMA ͨţֵΪDMA_CHAN_* (#DMA_CHAN_1)  (#DMA_CHAN_2)
 *
 * \retval  AM_OK     : ɹ
 * \retval -AM_EINVAL : Ч
 */
int am_zsl42x_dma_xfer_desc_chan_cfg (amhw_zsl42x_dma_xfer_desc_t    *p_desc,
                                      amhw_zsl42x_dma_transfer_type_t type,
                                      uint8_t                         chan);

/**
 * \brief DMAص
 *
 * \attention ûصĵڶɴãòȡֵΧ AM_ZSL42x_DMA_INT*
 *            ļϷ궨崦
 *
 * \param[in] chan    : DMA ͨţֵΪDMA_CHAN_* (#DMA_CHAN_1)  (#DMA_CHAN_2)
 * \param[in] pfn_isr : صָ
 * \param[in] p_arg   : صĵһڲ ӹУӦͨд϶Ӧ֣
 *                      DMA0ͨòΪ(void *)0
 *
 * \retval  AM_OK     : ӳɹ
 * \retval -AM_EPERM  : 
 */
int am_zsl42x_dma_isr_connect (int                       chan,
                               am_zsl42x_pfn_dma_isr_t      pfn_isr,
                               void                     *p_arg);

/**
 * \brief ɾDMAص
 *
 * \attention ûصĵڶɴãòȡֵΧ AM_ZSL42x_DMA_INT*
 *            ļϷ궨崦
 *            ûصĵǲжϵͨȡֵΧDMA_CHAN_* (#DMA_CHAN_1)
 *
 * \param[in] chan    : DMA ͨţֵΪDMA_CHAN_* (#DMA_CHAN_1)  (#DMA_CHAN_2)
 * \param[in] pfn_isr : صָ
 * \param[in] p_arg   : صĵһڲӹУӦͨд϶Ӧ֣
 *                      DMA0ͨòΪ(void *)0
 *
 * \retval  AM_OK     : ɾɹ
 * \retval -AM_EPERM  : 
 */
int am_zsl42x_dma_isr_disconnect (int                      chan,
                                  am_zsl42x_pfn_dma_isr_t  pfn_isr,
                                  void                    *p_arg);

/**
 * \breif ȡ  ĳͨǰʣֽ
 *
 * \param[in] chan DMA ͨţֵΪ DMA_CHAN_* (#DMA_CHAN_1)  (#DMA_CHAN_2)
 *
 * \retval  ֽ
 */
uint16_t am_zsl42x_dma_tran_data_get (int chan);

/**
 * \breif  ĳͨǰʣֽ
 *
 * \param[in] chan            DMA ͨţֵΪ DMA_CHAN_* (#DMA_CHAN_1)  (#DMA_CHAN_2)
 * \param[in] trans_data_byte Ҫֽ
 *
 * \retval  
 *
 * \note ôֽ֮ǰֹͣͨ
 */
void am_zsl42x_dma_tran_data_size (int chan, uint32_t trans_data_byte);

/**
 * \breif ÿδʱС
 *
 * \param[in] chan            DMA ͨţֵΪ DMA_CHAN_* (#DMA_CHAN_1)  (#DMA_CHAN_2)
 * \param[in] block_data_num  С
 *
 * \retval  
 *
 * \note ôֽ֮ǰֹͣͨ
 */
void am_zsl42x_dma_block_data_size (int chan, uint8_t block_data_num);

/**
 * \brief DMAʼ
 *
 * \param[in] p_dev         : ָDMA豸ָ
 * \param[in] p_dma_devinfo : ָDMA豸Ϣָ
 *
 * \retval  AM_OK       : ʼɹ
 */
int am_zsl42x_dma_init (am_zsl42x_dma_dev_t           *p_dev,
                        const am_zsl42x_dma_devinfo_t *p_dma_devinfo);

/**
 * \brief DMAȥʼ
 *
 * \return 
 */
void am_zsl42x_dma_deinit (void);

/**
 * @}
 */

#ifdef __cplusplus
}
#endif

#endif /* __AM_ZSL42x_DMA_H */

/* end of file */
