/*******************************************************************************
*                                 AMetal
*                       ----------------------------
*                       innovating embedded platform
*
* Copyright (c) 2001-2018 Guangzhou ZHIYUAN Electronics Co., Ltd.
* All rights reserved.
*
* Contact information:
* web site:    http://www.zlg118.cn/
*******************************************************************************/

/**
 * \file
 * \brief DMA hardware operation interface.
 *
 * 1. 2 independent programmable DMA channel.
 * 2. 8,16,32 bits data to transfer.
 * 3. support programmable source address,destination address and data size
 *
 * һ Ӳ AMHW_ZLG118_DMA_CHAN_CNT DMAͨĿʹñļʱ
 * Ӧ soc_cfg.h ļжúꡣ
 *
 * \internal
 * \par Modification history
 * - 1.00 17-04-10  ari, first implementation
 * \endinternal
 */

#ifndef __AMHW_ZLG118_DMA_H
#define __AMHW_ZLG118_DMA_H

#ifdef __cplusplus
extern "C" {
#endif

#include "am_types.h"
#include "am_common.h"
#include "soc_cfg.h"
#include "am_vdebug.h"
/**
 * \addtogroup amhw_zlg118_if_dma
 * \copydoc amhw_zlg118_dma.h
 * @{
 */

/** \brief DMA ͨĿ  */
#define AMHW_ZLG118_DMA_CHAN_CNT  SOC_CFG_ZLG_DMA_CHAN_CNT

/**
 * \brief structure of DMA controller register
 */
typedef struct amhw_zlg118_dma_chan {
    __IO uint32_t  dma_conf;           /**< \brief ͨüĴ */
         uint32_t  reserved[3];        /**< \brief Reserved */
    struct {                        
        __IO uint32_t  dma_confa;      /**< \brief ͨAĴ */
        __IO uint32_t  dma_confb;      /**< \brief ͨBĴ */
        __IO uint32_t  dma_srcadr;     /**< \brief ͨԴַĴ */
        __IO uint32_t  dma_dstadr;     /**< \brief ͨĿĵַĴ */
    } chcfg[AMHW_ZLG118_DMA_CHAN_CNT]; /**< \brief DMAͨṹ */

} amhw_zlg118_dma_t;

/**
 * \brief DMA transfer mode select
 */
typedef enum {
    AMHW_ZLG118_DMA_PER_TO_MER,     /**< \brief DMA peripheral to memory mode */
    AMHW_ZLG118_DMA_MER_TO_PER,     /**< \brief DMA memory to peripheral mode */
    AMHW_ZLG118_DMA_MER_TO_MER,     /**< \brief DMA memory to memory mode */
    AMHW_ZLG118_DMA_PER_TO_PER      /**< \brief DMA peripheral to peripheral mode */
} amhw_zlg118_dma_transfer_type_t;

/**
 * \brief DMA descriptor
 */
typedef struct amhw_zlg118_dma_xfer_desc {
    volatile uint32_t          xfercfg;        /**< \brief transfer configuration */
    volatile uint32_t          src_addr;       /**< \brief source address */
    volatile uint32_t          dst_addr;       /**< \brief destination address */
    volatile uint32_t          nbytes;         /**< \brief the bytes of data to transfer */
} amhw_zlg118_dma_xfer_desc_t;

/**
 * \brief DMA ͨʹܿλ     ʹ
 *
 * \param[in] p_hw_dma   : ָDMAĴṹָ
 *
 * \return none
 */
am_static_inline
void amhw_zlg118_dma_enable(amhw_zlg118_dma_t *p_hw_dma)
{
    p_hw_dma->dma_conf |= (0x1ul << 31);
}

/**
 * \brief DMA ͨʹܿλ     
 *
 * \param[in] p_hw_dma   : ָDMAĴṹָ
 *
 * \return none
 */
am_static_inline
void amhw_zlg118_dma_disable(amhw_zlg118_dma_t *p_hw_dma)
{
    p_hw_dma->dma_conf &= ~(0x1ul << 31);
}

/**
 * \brief DMA ͨĴ״̬   ȡ
 *
 * \param[in] p_hw_dma   : ָDMAĴṹָ
 *
 * \note  κڴз DMAC λá1
 *            1."0"д CONF:EN λ ͨ
 *            2."0000"ֵд CONF:HALT λ ͨͣ
 *
 *         CONF:EN=1 CONF:HALT=0000Чеͨʹܲ λ㡣
 *
 * \return none
 */
am_static_inline
am_bool_t amhw_zlg118_dma_transfer_check (amhw_zlg118_dma_t  *p_hw_dma)
{
    return ((p_hw_dma->dma_conf >> 30) & 0x1ul) ? AM_TRUE : AM_FALSE;
}

/**
 * \brief DMA ͨĴ״̬   
 *
 * \param[in] p_hw_dma : ָDMAĴṹָ
 * \param[in] set      : ֵ
 *
 * \note  ܱλ RW,CPUд"1"Ӱ DMAĲȻ
 *        CONFĴҪ¶ֲı䵱ǰλ ӦȶλȻдֵͬ
 *
 * \return none
 */
am_static_inline
void amhw_zlg118_dma_transfer_set (amhw_zlg118_dma_t *p_hw_dma, uint8_t set)
{
    p_hw_dma->dma_conf = (p_hw_dma->dma_conf & (~(0x1ul << 30))) |
                         ((set & 0x1ul) << 30);
}

/**< \brief DMA ͨȼ̶ CH0 ȼ > CH1 ȼ */
#define  AMHW_ZLG118_DMA_CHAN_PRIORITY_CH0_BEST   (0ul << 28)
/**< \brief DMA ͨȼѯʽ */
#define  AMHW_ZLG118_DMA_CHAN_PRIORITY_POLL       (1ul << 28)
/**
 * \brief DMA ͨȼ
 *
 * \param[in] p_hw_dma : ָDMAĴṹָ
 * \param[in] set      : ȼ궨
 *
 * \return none
 */
am_static_inline
void amhw_zlg118_dma_priority_set (amhw_zlg118_dma_t *p_hw_dma, uint32_t set)
{
    p_hw_dma->dma_conf = (p_hw_dma->dma_conf & (~(1ul << 28))) | set;
}

/**
 * \brief DMA  HALTλͨĴͣȡ
 *
 * \param[in] p_hw_dma : ָDMAĴṹָ
 * \param[in] set      : ֵ
 *
 * \note  ЩλΪ"0000"ֵʱ,ڴͨͣ״̬
 *        Ϊ"0000" д
 *
 *        ⲿĴЧͣ״̬ͨԴǿ鴫ͻ䣬
 *        ͣ صͨҲ䡣ΪɴͣĴ䣬Ҫͣȡ
 *        Ĵ
 *
 *        ЩλͣλͨüĴ
 *
 * \return none
 */
am_static_inline
void amhw_zlg118_dma_halt_set (amhw_zlg118_dma_t *p_hw_dma, uint8_t set)
{
    p_hw_dma->dma_conf = (p_hw_dma->dma_conf & (~(0xful << 24))) |
                         ((set & 0xful) << 24);
}

/**
 * \brief DMA ĳһͨʹܿλ    ʹ
 *
 * \param[in] p_hw_dma   : ָDMAĴṹָ
 * \param[in] channel    : ͨ
 *
 * \note λ"1", ͨʹܣȴʼ书(CONF:EN  "1")
 *        MSK λ (CONFB[0]) λ "0" DMAɴλ"0"
 *       λΪ"0", ֹͨݴ䡣ͬʱ ڴУôֹͣ䡣
 *       λǿֹͣڴͨͬʱλüĴ
 *
 * \return none
 */
am_static_inline
void amhw_zlg118_dma_chan_enable(amhw_zlg118_dma_t *p_hw_dma,
                                 uint8_t            channel)
{
    p_hw_dma->chcfg[channel].dma_confa |= (0x1ul << 31);
}

/**
 * \brief DMA ĳһͨʹܿλ    
 *
 * \param[in] p_hw_dma   : ָDMAĴṹָ
 * \param[in] channel    : ͨ
 *
 * \note λ"1", ͨʹܣȴʼ书(CONF:EN  "1")
 *        MSK λ (CONFB[0]) λ "0" DMAɴλ"0"
 *       λΪ"0", ֹͨݴ䡣ͬʱ ڴУôֹͣ䡣
 *       λǿֹͣڴͨͬʱλüĴ
 *
 * \return none
 */
am_static_inline
void amhw_zlg118_dma_chan_disable(amhw_zlg118_dma_t *p_hw_dma,
                                  uint8_t            channel)
{
    p_hw_dma->chcfg[channel].dma_confa &= ~(0x1ul << 31);
}

/**
 * \brief DMA PASλƸͨĴͣȡ        ͣ
 *
 * \param[in] p_hw_dma   : ָDMAĴṹָ
 * \param[in] channel    : ͨ
 *
 * \note λ "1" ڴͨͣ״̬λ 0¿ʼ䡣
 *       λΪ"0" ͨĴ󡣼ⲿ贫Чͣ״̬ͨ
 *       Դǿ鴫ͻ䣬ͨῪʼ䣬ʹͣλ
 *       ΪͣĴ䣬ͣĴ
 *
 *       λͣλͨüĴ
 *
 * \return none
 */
am_static_inline
void amhw_zlg118_dma_chan_stop(amhw_zlg118_dma_t *p_hw_dma,
                                 uint8_t          channel)
{
    p_hw_dma->chcfg[channel].dma_confa |= (0x1ul << 30);
}

/**
 * \brief DMA PASλƸͨĴͣȡ        ʼ
 *
 * \param[in] p_hw_dma   : ָDMAĴṹָ
 * \param[in] channel    : ͨ
 *
 * \note λ "1" ڴͨͣ״̬λ 0¿ʼ䡣
 *       λΪ"0" ͨĴ󡣼ⲿ贫Чͣ״̬ͨ
 *       Դǿ鴫ͻ䣬ͨῪʼ䣬ʹͣλ
 *       ΪͣĴ䣬ͣĴ
 *
 *       λͣλͨüĴ
 *
 * \return none
 */
am_static_inline
void amhw_zlg118_dma_chan_start(amhw_zlg118_dma_t *p_hw_dma,
                                  uint8_t          channel)
{
    p_hw_dma->chcfg[channel].dma_confa &= ~(0x1ul << 30);
}

/**
 * \brief DMA ST λڲͨ
 *
 * \param[in] p_hw_dma   : ָDMAĴṹָ
 * \param[in] channel    : ͨ
 * \param[in] stat       : ֵ
 *
 * \note λ"1", 䴥ͨʼ䡣ɺ DMAλ"0"
 *       λڴ"0" ֹͣ
 *
 * \return none
 */
am_static_inline
void amhw_zlg118_dma_chan_soft_transfer_ask(amhw_zlg118_dma_t *p_hw_dma,
                                            uint8_t            channel,
                                            uint8_t            stat)
{
    p_hw_dma->chcfg[channel].dma_confa =
        (p_hw_dma->chcfg[channel].dma_confa & (~(0x1ul << 29))) |
        ((stat & 0x1ul) << 29) ;
}

/**
 * \brief DMA TRI_SEL[6:0]λѡĴԴ
 *
 * \param[in] p_hw_dma   : ָDMAĴṹָ
 * \param[in] channel    : ͨ
 * \param[in] src        : 趨鿴zlg118_dma_chan.h궨壩
 *
 * \return none
 */
am_static_inline
void amhw_zlg118_dma_chan_trigger_src_set(amhw_zlg118_dma_t *p_hw_dma,
                                          uint8_t            channel,
                                          uint8_t            src)
{
    p_hw_dma->chcfg[channel].dma_confa =
        (p_hw_dma->chcfg[channel].dma_confa & (~(0x7ful << 22))) |
        ((src & 0x7ful) << 22) ;
}

/**
 * \brief DMA BC[3:0]λָ/ͻĸ
 *
 * \param[in] p_hw_dma   : ָDMAĴṹָ
 * \param[in] channel    : ͨ
 * \param[in] num        : ֵ
 *
 * \note Ϊʱ"0000"
 *       "BC[3:0]= - 1"ĿΪ 16
 *
 *       ⼸λڴĹпȥͨһԴַʣһĿĵַķʳɹɣ
 *       BC[3:0] ȥ 1
 *
 *        CONFB:RC=1 £俪ʼʱõֵڴɺء
 *        CONFB:RC=0 £ɹɴ󣬸λ㣬ͬʱڴͣĴǳɹʱֲֵ䡣
 *
 * \return none
 */
am_static_inline
void amhw_zlg118_dma_chan_bc_set(amhw_zlg118_dma_t *p_hw_dma,
                                 uint8_t            channel,
                                 uint8_t            num)
{
    p_hw_dma->chcfg[channel].dma_confa =
        (p_hw_dma->chcfg[channel].dma_confa & (~(0xful << 16))) |
        ((num & 0xful) << 16);
}

/**
 * \brief DMA TC[15:0] λָ/ͻ/Ĵ
 *
 * \param[in] p_hw_dma   : ָDMAĴṹָ
 * \param[in] channel    : ͨ
 * \param[in] num        : ֵ
 *
 * \note 趨ֵ"TC =  - 1". Ϊ 65536
 *
 *       ʱЩֵǿԶġһο鴫ɣ TC ȥ 1
 *
 *        CONFB:RC=1 £俪ʼʱõֵڴɺء
 *        CONFB:RC=0 £ɹɴ󣬸λ㣬ͬʱڴͣĴǳɹʱֲֵ䡣
 *
 * \return none
 */
am_static_inline
void amhw_zlg118_dma_chan_tran_data_num_set (amhw_zlg118_dma_t *p_hw_dma,
                                             uint8_t            channel,
                                             uint16_t           num)
{
    p_hw_dma->chcfg[channel].dma_confa =
        (p_hw_dma->chcfg[channel].dma_confa & (~(0xfffful << 0))) |
        ((num & 0xfffful) << 0);
}

/**
 * \brief DMA TC[15:0] λָ/ͻ/Ĵ     ȡ
 *
 * \param[in] p_hw_dma   : ָDMAĴṹָ
 * \param[in] channel    : ͨ
 *
 * \note 趨ֵ"TC =  - 1". Ϊ 65536
 *
 *       ʱЩֵǿԶġһο鴫ɣ TC ȥ 1
 *
 *        CONFB:RC=1 £俪ʼʱõֵڴɺء
 *        CONFB:RC=0 £ɹɴ󣬸λ㣬ͬʱڴͣĴǳɹʱֲֵ䡣
 *
 * \return ǰʣֽ
 */
am_static_inline
uint16_t amhw_zlg118_dma_chan_tran_data_num_get (amhw_zlg118_dma_t *p_hw_dma,
                                                 uint8_t            channel)
{
    return ( p_hw_dma->chcfg[channel].dma_confa & 0xfffful);
}


#define  AMHW_ZLG118_DMA_CHAN_TRANSFER_MODE_BLOCK    (0UL << 28) /**< \brief Blockģʽ */
#define  AMHW_ZLG118_DMA_CHAN_TRANSFER_MODE_BURST    (1UL << 28) /**< \brief Burstģʽ */

#define  AMHW_ZLG118_DMA_CHAN_SIZE_8BIT              (0UL << 26) /**< \brief ݿ 8bit */
#define  AMHW_ZLG118_DMA_CHAN_SIZE_16BIT             (1UL << 26) /**< \brief ݿ 16bit */
#define  AMHW_ZLG118_DMA_CHAN_SIZE_32BIT             (2UL << 26) /**< \brief ݿ 32bit */

#define  AMHW_ZLG118_DMA_CHAN_SRC_ADD_INC_DISABLE    (1UL << 25) /**< \brief Դַ */
#define  AMHW_ZLG118_DMA_CHAN_SRC_ADD_INC_ENABLE     (0UL << 25) /**< \brief Դַʹ */

#define  AMHW_ZLG118_DMA_CHAN_DST_ADD_INC_DISABLE    (1UL << 24) /**< \brief Ŀַ  */
#define  AMHW_ZLG118_DMA_CHAN_DST_ADD_INC_ENABLE     (0UL << 24) /**< \brief Ŀַʹ  */

#define  AMHW_ZLG118_DMA_CHAN_RELOAD_COUNTER_DISABLE (0UL << 23) /**< \brief ؽ  */
#define  AMHW_ZLG118_DMA_CHAN_RELOAD_COUNTER_ENABLE  (1UL << 23) /**< \brief ʹ  */

#define  AMHW_ZLG118_DMA_CHAN_RELOAD_SRC_ADD_DISABLE (0UL << 22) /**< \brief Դַؽ  */
#define  AMHW_ZLG118_DMA_CHAN_RELOAD_SRC_ADD_ENABLE  (1UL << 22) /**< \brief Դַʹ  */

#define  AMHW_ZLG118_DMA_CHAN_RELOAD_DST_ADD_DISABLE (0UL << 21) /**< \brief Ŀַؽ  */
#define  AMHW_ZLG118_DMA_CHAN_RELOAD_DST_ADD_ENABLE  (1UL << 21) /**< \brief Ŀַʹ  */

#define  AMHW_ZLG118_DMA_CHAN_INT_ERR_DISABLE        (0UL << 20) /**< \brief жϽ  */
#define  AMHW_ZLG118_DMA_CHAN_INT_ERR_ENABLE         (1UL << 20) /**< \brief жʹ  */

#define  AMHW_ZLG118_DMA_CHAN_INT_TX_CMP_DISABLE     (0UL << 19) /**< \brief жϽ */
#define  AMHW_ZLG118_DMA_CHAN_INT_TX_CMP_ENABLE      (1UL << 19) /**< \brief жʹ */

#define  AMHW_ZLG118_DMA_CHAN_CIRCULAR_MODE_DISABLE  (0UL << 0)  /**< \brief ѭģʽ */
#define  AMHW_ZLG118_DMA_CHAN_CIRCULAR_MODE_ENABLE   (1UL << 0)  /**< \brief ѭģʽʹ */

/**
 * \brief DMA ͨ
 *
 * \param[in] p_hw_dma   : ָDMAĴṹָ
 * \param[in] channel    : ͨ
 * \param[in] num        : ñ־
 *
 * \return none
 */
am_static_inline
void amhw_zlg118_dma_chan_config_set(amhw_zlg118_dma_t *p_hw_dma,
                                     uint8_t            channel,
                                     uint32_t           flag)

{
    p_hw_dma->chcfg[channel].dma_confb =
        (p_hw_dma->chcfg[channel].dma_confb & (~0x3ff8fffful)) |
        flag;
}


#define  AMHW_ZLG118_DMA_CHAN_INT_ERR      (1UL << 20) /**< \brief δж  */
#define  AMHW_ZLG118_DMA_CHAN_INT_TX_CMP   (1UL << 19) /**< \brief ж */
/**
 * \brief DMA ĳһͨжʹ
 *
 * \param[in] p_hw_dma   : ָDMAĴṹָ
 * \param[in] channel    : ͨ
 * \param[in] num        : ñ־
 *
 * \return none
 */
am_static_inline
void amhw_zlg118_dma_chan_int_enable(amhw_zlg118_dma_t *p_hw_dma,
                                     uint8_t            channel,
                                     uint32_t           flag)
{
    p_hw_dma->chcfg[channel].dma_confb |= flag;
}

/**
 * \brief DMA ĳһͨжϽ
 *
 * \param[in] p_hw_dma   : ָDMAĴṹָ
 * \param[in] channel    : ͨ
 * \param[in] num        : ñ־
 *
 * \return none
 */
am_static_inline
void amhw_zlg118_dma_chan_int_disable(amhw_zlg118_dma_t *p_hw_dma,
                                      uint8_t            channel,
                                      uint32_t           flag)
{
    p_hw_dma->chcfg[channel].dma_confb &= ~flag;
}

/**
 * \brief DMA ״̬־
 *@{
 */
typedef enum {
    AMHW_ZLG118_DMA_FLAG_DEFAULT = 0,               /* ʼĬֵ */
    AMHW_ZLG118_DMA_FLAG_ERR_ADDR_OVERFLOW,         /* ַ */
    AMHW_ZLG118_DMA_FLAG_STOP_ASK,                  /* ֹֹͣ */
    AMHW_ZLG118_DMA_FLAG_ERR_SRCADDR,               /* Դַʴ */
    AMHW_ZLG118_DMA_FLAG_ERR_DSTADDR,               /* Ŀĵصַʴ */
    AMHW_ZLG118_DMA_FLAG_TX_COMPLETE,               /* ɹ */
    AMHW_ZLG118_DMA_FLAG_RESERVE,                   /*  */
    AMHW_ZLG118_DMA_FLAG_TX_STOP,                   /* ͣ */
}amhw_zlg118_dma_flag_t;


/**
 * \brief ״̬־ȡ
 *
 * \param[in] p_hw_dma : ָDMAĴṹָ
 * \param[in] channel  : ͨ
 * \param[in] flag     : ״̬־
 *
 * \return AM_TRUE or AM_FALSE
 */
am_static_inline
am_bool_t amhw_zlg118_dma_chan_stat_check (amhw_zlg118_dma_t     *p_hw_dma,
                                           uint8_t                channel,
                                           amhw_zlg118_dma_flag_t flag)
{
    return ((((p_hw_dma->chcfg[channel].dma_confb >> 16) & 0x7ul) == flag) ? AM_TRUE : AM_FALSE);
}

/**
 * \brief ״̬־
 *
 * \param[in] p_hw_dma    : ָDMAĴṹָ
 * \param[in] channel     : ͨ
 *
 * \return none
 */
am_static_inline
void amhw_zlg118_dma_chan_flag_clear (amhw_zlg118_dma_t *p_hw_dma,
                                      uint8_t            channel)
{
    p_hw_dma->chcfg[channel].dma_confb &= ~(0x7ul << 16);
}

/**
 * \brief Դַ
 *
 * \param[in] p_hw_dma   : ָDMAĴṹָ
 * \param[in] channel    : ͨ
 * \param[in] address    : ַ
 *
 * \note   CONFB:FS=1Դַ趨Ϊ̶ֵҲı䡣
 *
 *         CONFB:FS=0  CONFB:RS=0ʱԴַ CONFB:WIDTH[1:0]λ
 *        ɹɣֵΪɵַһַδɹɣֵΪͣʱ趨ֵ
 *
 *         CONFB:FS=0 CONFB:RS=1ʱԴַ CONFB:WIDTH[1:0]λ
 *        ɺ󣬴俪ʼʱ趨Դַ¼صĴλС
 *
 * \return none
 */
am_static_inline
void amhw_zlg118_dma_chan_src_address_set(amhw_zlg118_dma_t *p_hw_dma,
                                          uint8_t            channel,
                                          uint32_t           address)
{
    p_hw_dma->chcfg[channel].dma_srcadr = address;
}

/**
 * \brief Ŀַ
 *
 * \param[in] p_hw_dma   : ָDMAĴṹָ
 * \param[in] channel    : ͨ
 * \param[in] address    : ַ
 *
 * \note   CONFB:FD=1 ĿĵַΪ̶ַҲı䡣
 *
 *         CONFB:FD=0  CONFB:RD=0, ʱĿĵַ CONFB:WIDTH[1:0]λ
 *        ɹɣֵΪɵַһַδɹɣֵΪͣʱ趨ֵ
 *
 *         CONFB:FD=0  CONFB:RD=1, ʱĿĵַ CONFB:WIDTH[1:0]λ
 *        ɺ󣬴俪ʼʱ趨Ŀĵַ¼صĴλС

 * \return none
 */
am_static_inline
void amhw_zlg118_dma_chan_dst_address_set(amhw_zlg118_dma_t *p_hw_dma,
                                          uint8_t            channel,
                                          uint32_t           address)
{
    p_hw_dma->chcfg[channel].dma_dstadr = address;
}

/**
 * @} amhw_if_zlg118_dma
 */

#ifdef __cplusplus
}
#endif

#endif  /* __AMHW_ZLG118_DMA_H */

/* end of file */
