/*******************************************************************************
*                                 AMetal
*                       ----------------------------
*                       innovating embedded platform
*
* Copyright (c) 2001-2018 Guangzhou ZHIYUAN Electronics Co., Ltd.
* All rights reserved.
*
* Contact information:
* web site:    http://www.zlg.cn/
*******************************************************************************/

/**
 * \file
 * \brief EXTI ӿ
 *
 * \internal
 * \par Modification history
 * - 1.00 19-06-27  yrh, first implementation
 * \endinternal
 */

#ifndef __AMHW_ZLG237_EXTI_H
#define __AMHW_ZLG237_EXTI_H

#ifdef __cplusplus
extern "C" {
#endif

#include "am_types.h"

/*
 * \brief ṹεĿʼ
 */
#if defined(__CC_ARM)
  #pragma push
  #pragma anon_unions
#elif defined(__ICCARM__)
  #pragma language=extended
#elif defined(__GNUC__)
#elif defined(__TMS470__)
#elif defined(__TASKING__)
  #pragma warning 586
#else
  #warning Not supported compiler t
#endif

/**
 * \addtogroup amhw_zlg237_if_exti
 * \copydoc amhw_zlg237_exti.h
 * @{
 */

/**
 * \brief EXTIĴṹ
 */
typedef struct amhw_zlg237_exti {
    __IO uint32_t imr;      /**< \brief жμĴ */
    __IO uint32_t emr;      /**< \brief ¼μĴ */
    __IO uint32_t rtsr;     /**< \brief شѡĴ */
    __IO uint32_t ftsr;     /**< \brief ½شѡĴ */
    __IO uint32_t swier;    /**< \brief ж¼Ĵ */
    __IO uint32_t pr;       /**< \brief ж/¼Ĵ */
} amhw_zlg237_exti_t;

/**
 * \brief ߱ö
 */
typedef enum {
    AMHW_ZLG237_LINE_NUM0  = (1ul << 0),    /**< \brief 0 */
    AMHW_ZLG237_LINE_NUM1  = (1ul << 1),    /**< \brief 1 */
    AMHW_ZLG237_LINE_NUM2  = (1ul << 2),    /**< \brief 2 */
    AMHW_ZLG237_LINE_NUM3  = (1ul << 3),    /**< \brief 3 */
    AMHW_ZLG237_LINE_NUM4  = (1ul << 4),    /**< \brief 4 */
    AMHW_ZLG237_LINE_NUM5  = (1ul << 5),    /**< \brief 5 */
    AMHW_ZLG237_LINE_NUM6  = (1ul << 6),    /**< \brief 6 */
    AMHW_ZLG237_LINE_NUM7  = (1ul << 7),    /**< \brief 7 */
    AMHW_ZLG237_LINE_NUM8  = (1ul << 8),    /**< \brief 8 */
    AMHW_ZLG237_LINE_NUM9  = (1ul << 9),    /**< \brief 9 */
    AMHW_ZLG237_LINE_NUM10 = (1ul << 10),   /**< \brief 10 */
    AMHW_ZLG237_LINE_NUM11 = (1ul << 11),   /**< \brief 11 */
    AMHW_ZLG237_LINE_NUM12 = (1ul << 12),   /**< \brief 12 */
    AMHW_ZLG237_LINE_NUM13 = (1ul << 13),   /**< \brief 13 */
    AMHW_ZLG237_LINE_NUM14 = (1ul << 14),   /**< \brief 14 */
    AMHW_ZLG237_LINE_NUM15 = (1ul << 15),   /**< \brief 15 */
    AMHW_ZLG237_LINE_NUM16 = (1ul << 16),   /**< \brief 16ӵ PVD  */
    AMHW_ZLG237_LINE_NUM17 = (1ul << 17),   /**< \brief 17ӵRTC¼ */
    AMHW_ZLG237_LINE_NUM18 = (1ul << 18),   /**< \brief 18ӵUSB¼ */
    AMHW_ZLG237_LINE_NUM19 = (1ul << 19),   /**< \brief 19ӵȽ1 */
    AMHW_ZLG237_LINE_NUM20 = (1ul << 20),   /**< \brief 20ӵȽ1 */
}amhw_zlg237_line_num_t;

/**
 * \brief ж/¼ö
 */
typedef enum
{
    AMHW_ZLG237_EXTI_TRIGGER_RISING         = 0x08,
    AMHW_ZLG237_EXTI_TRIGGER_FALLING        = 0x0C,
    AMHW_ZLG237_EXTI_TRIGGER_RISING_FALLING = 0x10
}amwh_zlg237_extitrigger_type_t;

/**
 * \brief  xϵж
 *
 * \param[in]   p_hw_exti : ָⲿж/¼ƿָ
 * \param[in]   num : ߱
 *              AMHW_ZLG237_LINE_NUM* ֵ
 *              AMHW_ZLG237_LINE_NUM*Ļֵ
 *              :
 *               - AMHW_ZLG237_LINE_NUM0
 *               - AMHW_ZLG237_LINE_NUM0 | AMHW_ZLG237_LINE_NUM1
 *
 * \return 
 */
am_static_inline
void amhw_zlg237_exti_imr_set(amhw_zlg237_exti_t     *p_hw_exti,
                              amhw_zlg237_line_num_t  num)
{
    p_hw_exti->imr |= (num & 0x7ffff);
}

/**
 * \brief  xϵж
 *
 * \param[in]   p_hw_exti : ָⲿж/¼ƿָ
 * \param[in]   num : ߱
 *              AMHW_ZLG237_LINE_NUM* ֵ
 *              AMHW_ZLG237_LINE_NUM*Ļֵ
 *              :
 *               - AMHW_ZLG237_LINE_NUM0
 *               - AMHW_ZLG237_LINE_NUM0 | AMHW_ZLG237_LINE_NUM1
 *
 * \return 
 */
am_static_inline
void amhw_zlg237_exti_imr_clear(amhw_zlg237_exti_t     *p_hw_exti,
                                amhw_zlg237_line_num_t  num)
{
    p_hw_exti->imr &= ~(num & 0x7ffff);
}

/**
 * \brief  xϵ¼
 *
 * \param[in]   p_hw_exti : ָⲿж/¼ƿָ
 * \param[in]    num : ߱
 *              AMHW_ZLG237_LINE_NUM* ֵ
 *              AMHW_ZLG237_LINE_NUM*Ļֵ
 *              :
 *               - AMHW_ZLG237_LINE_NUM0
 *               - AMHW_ZLG237_LINE_NUM0 | AMHW_ZLG237_LINE_NUM1
 *
 * \return 
 */
am_static_inline
void amhw_zlg237_exti_emr_set(amhw_zlg237_exti_t    *p_hw_exti,
                              amhw_zlg237_line_num_t num)
{
    p_hw_exti->emr |= (num & 0x7ffff);
}

/**
 * \brief  xϵ¼
 *
 * \param[in]   p_hw_exti : ָⲿж/¼ƿָ
 * \param[in]   num : ߱
 *              AMHW_ZLG237_LINE_NUM* ֵ
 *              AMHW_ZLG237_LINE_NUM*Ļֵ
 *              :
 *               - AMHW_ZLG237_LINE_NUM0
 *               - AMHW_ZLG237_LINE_NUM0 | AMHW_ZLG237_LINE_NUM1
 *
 * \return 
 */
am_static_inline
void amhw_zlg237_exti_emr_clear(amhw_zlg237_exti_t     *p_hw_exti,
                                amhw_zlg237_line_num_t  num)
{
    p_hw_exti->emr &= ~(num & 0x7ffff);
}

/**
 * \brief  xϵشжϺ¼
 *
 * \param[in]   p_hw_exti : ָⲿж/¼ƿָ
 * \param[in]   num : ߱
 *              AMHW_ZLG237_LINE_NUM* ֵ
 *              AMHW_ZLG237_LINE_NUM*Ļֵ
 *              :
 *               - AMHW_ZLG237_LINE_NUM0
 *               - AMHW_ZLG237_LINE_NUM0 | AMHW_ZLG237_LINE_NUM1
 *
 * \return 
 */
am_static_inline
void amhw_zlg237_exti_rtsr_set(amhw_zlg237_exti_t     *p_hw_exti,
                               amhw_zlg237_line_num_t  num)
{
    p_hw_exti->rtsr |= (num & 0x7ffff);
}

/**
 * \brief ֹ xϵشжϺ¼
 *
 * \param[in]   p_hw_exti : ָⲿж/¼ƿָ
 * \param[in]   num : ߱
 *              AMHW_ZLG237_LINE_NUM* ֵ
 *              AMHW_ZLG237_LINE_NUM*Ļֵ
 *              :
 *               - AMHW_ZLG237_LINE_NUM0
 *               - AMHW_ZLG237_LINE_NUM0 | AMHW_ZLG237_LINE_NUM1
 *
 * \return 
 */
am_static_inline
void amhw_zlg237_exti_rtsr_clear(amhw_zlg237_exti_t     *p_hw_exti,
                                 amhw_zlg237_line_num_t  num)
{
    p_hw_exti->rtsr &= ~(num & 0x7ffff);
}

/**
 * \brief  xϵ½شжϺ¼
 *
 * \param[in]   p_hw_exti : ָⲿж/¼ƿָ
 * \param[in]   num : ߱
 *              AMHW_ZLG237_LINE_NUM* ֵ
 *              AMHW_ZLG237_LINE_NUM*Ļֵ
 *              :
 *               - AMHW_ZLG237_LINE_NUM0
 *               - AMHW_ZLG237_LINE_NUM0 | AMHW_ZLG237_LINE_NUM1
 *
 * \return 
 */
am_static_inline
void amhw_zlg237_exti_ftsr_set(amhw_zlg237_exti_t     *p_hw_exti,
                               amhw_zlg237_line_num_t  num)
{
    p_hw_exti->ftsr |= (num & 0x7ffff);
}

/**
 * \brief ֹ xϵ½شжϺ¼
 *
 * \param[in]   p_hw_exti : ָⲿж/¼ƿָ
 * \param[in]   num : ߱
 *              AMHW_ZLG237_LINE_NUM* ֵ
 *              AMHW_ZLG237_LINE_NUM*Ļֵ
 *              :
 *               - AMHW_ZLG237_LINE_NUM0
 *               - AMHW_ZLG237_LINE_NUM0 | AMHW_ZLG237_LINE_NUM1
 *
 * \return 
 */
am_static_inline
void amhw_zlg237_exti_ftsr_clear(amhw_zlg237_exti_t     *p_hw_exti,
                                 amhw_zlg237_line_num_t  num)
{
    p_hw_exti->ftsr &= ~(num & 0x7ffff);
}

/**
 * \brief  xϵж
 *
 * \param[in]   p_hw_exti : ָⲿж/¼ƿָ
 * \param[in]   num : ߱
 *              AMHW_ZLG237_LINE_NUM* ֵ
 *              AMHW_ZLG237_LINE_NUM*Ļֵ
 *              :
 *               - AMHW_ZLG237_LINE_NUM0
 *               - AMHW_ZLG237_LINE_NUM0 | AMHW_ZLG237_LINE_NUM1
 *
 * \return 
 */
am_static_inline
void amhw_zlg237_exti_swier_set(amhw_zlg237_exti_t     *p_hw_exti,
                                amhw_zlg237_line_num_t  num)
{
    p_hw_exti->swier |= (num & 0x7ffff);
}

/**
 * \brief жϱ־
 *
 * \param[in]   p_hw_exti : ָⲿж/¼ƿָ
 * \param[in]   num : ߱
 *              AMHW_ZLG237_LINE_NUM* ֵ
 *              AMHW_ZLG237_LINE_NUM*Ļֵ
 *              :
 *               - AMHW_ZLG237_LINE_NUM0
 *               - AMHW_ZLG237_LINE_NUM0 | AMHW_ZLG237_LINE_NUM1
 *
 * \return 
 */
am_static_inline
void amhw_zlg237_exti_pending_clear(amhw_zlg237_exti_t     *p_hw_exti,
                                    amhw_zlg237_line_num_t  num)
{
    p_hw_exti->pr |= (num & 0x7ffff);
}

/**
 * \brief ȡⲿжϱ־
 *
 * \param[in]   p_hw_exti : ָⲿж/¼ƿָ
 * \param[in]   num : ߱
 *
 * \return жϱ־
 *              AMHW_ZLG237_LINE_NUM* ֵ
 *              AMHW_ZLG237_LINE_NUM*Ļֵ
 *              :
 *               - AMHW_ZLG237_LINE_NUM0
 *               - AMHW_ZLG237_LINE_NUM0 | AMHW_ZLG237_LINE_NUM1
 *
 */
am_static_inline
uint32_t amhw_zlg237_exti_pr_read(amhw_zlg237_exti_t     *p_hw_exti,
                                  amhw_zlg237_line_num_t  num)
{
    return (p_hw_exti->pr & num);
}
/**
 * @}
 */
/*
 * \brief ṹεĽ
 */

#if defined(__CC_ARM)
  #pragma pop
#elif defined(__ICCARM__)
#elif defined(__GNUC__)
#elif defined(__TMS470__)
#elif defined(__TASKING__)
  #pragma warning restore
#else
  #warning Not supported compiler t
#endif

#ifdef __cplusplus
}
#endif /* __AMHW_ZLG237_EXTI_H */

#endif

/* end of file */
