/*******************************************************************************
*                                 AMetal
*                       ----------------------------
*                       innovating embedded platform
*
* Copyright (c) 2001-2019 Guangzhou ZHIYUAN Electronics Co., Ltd.
* All rights reserved.
*
* Contact information:
* web site:    http://www.zlg.cn/
*******************************************************************************/

/**
 * \file
 * \brief Flashӿ
 *
 * \internal
 * \par Modification History
 * - 1.01 20-05-13  licl, add byte and half word program
 * - 1.00 19-09-04  zp, first implementation
 * \endinternal
 */

#ifndef __AM_ZSL42x_FLASH_H
#define __AM_ZSL42x_FLASH_H

#ifdef __cplusplus
extern "C" {
#endif

#include "am_types.h"
#include "am_bitops.h"
#include "zsl42x_soc_cfg.h"
#include "hw/amhw_zsl42x_flash.h"

/**
 * \addtogroup am_zsl42x_if_flash
 * \copydoc am_zsl42x_flash.h
 * @{
 */

#if defined ( __CC_ARM   )
#pragma anon_unions
#endif

/**
 * \brief öFLASH
 *
 * \param[in] p_hw_flash  ָFLASHĴָ
 * \param[in] waitcycle   
 *
 * \return AM_OK: ɹ
 */
int32_t am_zsl42x_flash_waitcycle(amhw_zsl42x_flash_t              *p_hw_flash,
                                amhw_zsl42x_flash_read_waittime   waitcycle);

/**
 * \brief ҳ
 *
 * \param[in] p_hw_flash  ָFLASHĴָ
 * \param[in] start_addr  ҳʼַ
 *
 * \return AM_OK: ɹ, -AM_EINVAL: ַ
 */
int32_t am_zsl42x_flash_sector_erase (amhw_zsl42x_flash_t  *p_hw_flash,
                                    uint32_t            addr);

/**
 * \brief ҳֽڱ
 *
 * \param[in] p_hw_flash ָFLASHĴָ
 * \param[in] dst_addr   д뵽flashʼַ
 * \param[in] p_src      Ҫд뵽flashеݵʼַ
 * \param[in] size       дֽ(8bit)ĸsizeҳС(SECTOR_SIZE)ʱӦֻȲһҳ
 *
 * \return >0: ʵʳɹд <0: ʧ
 */
int32_t am_zsl42x_flash_sector_byte_program (amhw_zsl42x_flash_t    *p_hw_flash,
                                           uint32_t              dst_addr,
                                           const uint8_t        *p_src,
                                           uint32_t              size);

/**
 * \brief ҳֱ
 *
 * \param[in] p_hw_flash ָFLASHĴָ
 * \param[in] dst_addr   д뵽flashʼַ
 * \param[in] p_src      Ҫд뵽flashеݵʼַ
 * \param[in] size       д(16bit)ĸsize * 2 ҳС(SECTOR_SIZE)ʱӦֻȲһҳ
 *
 * \return >0: ʵʳɹд <0: ʧ
 */
int32_t am_zsl42x_flash_sector_halfword_program (amhw_zsl42x_flash_t    *p_hw_flash,
                                               uint32_t              dst_addr,
                                               const uint16_t       *p_src,
                                               uint32_t              size);

/**
 * \brief ҳֱ
 *
 * \param[in] p_hw_flash ָFLASHĴָ
 * \param[in] dst_addr   д뵽flashʼַ
 * \param[in] p_src      Ҫд뵽flashеݵʼַ
 * \param[in] size       д(32bit)ĸsize * 4 ҳС(SECTOR_SIZE)ʱӦֻȲһҳ
 *
 * \return >0: ʵʳɹд <0: ʧ
 */
int32_t am_zsl42x_flash_sector_program (amhw_zsl42x_flash_t    *p_hw_flash,
                                      uint32_t              dst_addr,
                                      const uint32_t       *p_src,
                                      uint32_t              size);

/**
 * \brief ҳ(SRAMЧ)
 *
 * \param[in] p_hw_flash ָFLASHĴָ
 *
 * \return AM_OK: ɹ: ʧ
 */
int32_t am_zsl42x_flash_all_sector_erase (amhw_zsl42x_flash_t *p_hw_flash);

/**
 * \brief Flashģʼ
 *
 * \param[in] p_hw_flash ָFLASHĴָ
 * \param[in] freqcfg    flashʱ
 * \param[in] dpstb_able dpstbʹܿ
 *
 * \return AM_OK: ɹ: ʧ
 */
int32_t am_zsl42x_flash_init(amhw_zsl42x_flash_t   *p_hw_flash,
                           uint8_t              freqcfg,
                           am_bool_t            dpstb_able);

/**
 * @}
 */

#ifdef __cplusplus
}
#endif

#endif /** __AM_ZSL42x_FLASH_H */
