/*******************************************************************************
*                                 AMetal
*                       ----------------------------
*                       innovating embedded platform
*
* Copyright (c) 2001-2018 Guangzhou ZHIYUAN Electronics Co., Ltd.
* All rights reserved.
*
* Contact information:
* web site:    http://www.zlg.cn/
*******************************************************************************/

/**
 * \file
 * \brief GPIO hardware operation interface
 *
 * \internal
 * \par Modification history
 * - 1.00 20-03-18  yrz, first implementation
 * \endinternal
 */

#ifndef __AMHW_ZSN700_GPIO_H
#define __AMHW_ZSN700_GPIO_H

#ifdef __cplusplus
extern "C" {
#endif

#include "am_types.h"
#include "ametal.h"

/**
 * \addtogroup amhw_zsn700_if_gpio
 * \copydoc amhw_zsn700_gpio.h
 * @{
 */

#define AMHW_ZSN700_GPIO_PORT_MAX 6  /**< \brief GPIO GPIO˿ */

/**
 * \brief GPIO żĴṹ
 */
typedef struct {

    /**
     * \brief ˿ڹ(64ÿӦһ)
     *        PA0~PA15 
     *        PB0~PB15 
     *        PC0~PC15 
     *        PD0~PD15
     */
    __IO  uint32_t pin_x_sel_I[64];

} amhw_zsn700_gpio_pin_t;

/**
 * \brief GPIO ˿ڼĴṹ
 */
typedef struct {
    __IO  uint32_t dir;         /**< \brief üĴ */
    __IO  uint32_t in;          /**< \brief ֵĴ*/
    __IO  uint32_t out;         /**< \brief ֵüĴ*/
    __IO  uint32_t ads;         /**< \brief ģüĴ*/
    __IO  uint32_t bset;        /**< \brief λλĴ*/
    __IO  uint32_t bclr;        /**< \brief λĴ*/
    __IO  uint32_t bsetclr;     /**< \brief λλĴ*/
    __IO  uint32_t dr;          /**< \brief üĴ*/
    __IO  uint32_t pu;          /**< \brief ʹüĴ*/
    __IO  uint32_t pd;          /**< \brief ʹüĴ*/
    __IO  uint32_t reserve;     /**< \brief λ */
    __IO  uint32_t od;          /**< \brief ©üĴ*/
    __IO  uint32_t hie;         /**< \brief ߵƽжʹüĴ*/
    __IO  uint32_t lie;         /**< \brief ͵ƽжʹüĴ*/
    __IO  uint32_t rie;         /**< \brief жʹüĴ*/
    __IO  uint32_t fie;         /**< \brief ½жʹüĴ*/

}amhw_zsn700_gpio_port_t;

/**
 * \brief GPIO ˿жϼĴṹ
 */
typedef struct {
    __IO  uint32_t stat;          /**< \brief ж״̬Ĵ*/
          uint32_t reserve0[3];   /**< \brief λ */
    __IO  uint32_t iclr;          /**< \brief жĴ*/
          uint32_t reserve1[11];  /**< \brief λ */
}amhw_zsn700_gpio_port_int_t;

/**
 * \brief GPIO ˿ڸĴṹ
 */
typedef struct {
    __IO  uint32_t ctrl0;         /**< \brief ˿ڸüĴ0 */
    __IO  uint32_t ctrl1;         /**< \brief ˿ڸüĴ1 */
    __IO  uint32_t ctrl2;         /**< \brief ˿ڸüĴ2 */
    __IO  uint32_t timgs;         /**< \brief ˿ڸܶʱſѡ */
    __IO  uint32_t times;         /**< \brief ˿ڸܶʱETRѡ */
    __IO  uint32_t timcps;        /**< \brief ˿ڸܶʱѡ */
    __IO  uint32_t pcas;          /**< \brief ˿ڸPCAѡ */
    __IO  uint32_t pcnt;          /**< \brief ˿ڸPCNTѡ */
    __I   uint32_t reserve[824];  /**< \brief λ */
}amhw_zsn700_gpio_port_support_t;

/**
 * \brief һṹ壨ĸ˿Ϊһ飬PA-PD
 */
typedef struct {
    amhw_zsn700_gpio_pin_t          gpio_pin_x;
    amhw_zsn700_gpio_port_t         gpio_port[4];
    amhw_zsn700_gpio_port_int_t     gpio_int_reg[4];
    amhw_zsn700_gpio_port_support_t gpio_sup_reg;
} amhw_zsn700_gpio_group_t;

/**
 * \brief GPIOĴṹ
 */
typedef struct amhw_zsn700_gpio {
    amhw_zsn700_gpio_group_t    gpio[(AMHW_ZSN700_GPIO_PORT_MAX/4) + 1];
} amhw_zsn700_gpio_t;


/**
 * \brief GPIOŸ
 * \note ÿŲһΨһȷͨGPIOãӦֵΪ0
 */
typedef enum {
    AMHW_ZSN700_AFIO_NO = 0,
    AMHW_ZSN700_AFIO_1,
    AMHW_ZSN700_AFIO_2,
    AMHW_ZSN700_AFIO_3,
    AMHW_ZSN700_AFIO_4,
    AMHW_ZSN700_AFIO_5,
    AMHW_ZSN700_AFIO_6,
    AMHW_ZSN700_AFIO_7,
}amhw_zsn700_gpio_afio_t;

/**
 * \brief Ÿ
 *
 * \param[in] p_hw_gpio  : ָGPIOĴָ
 * \param[in] afio_choice: Ÿѡ
 * \param[in] pin        : űţֵΪ PIO* (#PIOA_0)
 *
 * \return 
 */
am_static_inline
void amhw_zsn700_gpio_afio_set (amhw_zsn700_gpio_t      *p_hw_gpio,
                                amhw_zsn700_gpio_afio_t  afio_choice,
                                int                      pin)
{
    p_hw_gpio->gpio[pin / 64].gpio_pin_x.pin_x_sel_I[pin % 64] = afio_choice;
}

/**
 * \brief Ÿȡ
 *
 * \param[in] p_hw_gpio  : ָGPIOĴָ
 * \param[in] pin        : űţֵΪ PIO* (#PIOA_0)
 *
 * \return ŵǰõľֵ
 */
am_static_inline
amhw_zsn700_gpio_afio_t amhw_zsn700_gpio_afio_get (amhw_zsn700_gpio_t *p_hw_gpio,
                                                   int                 pin)
{
    return (amhw_zsn700_gpio_afio_t)p_hw_gpio->gpio[pin / 64].gpio_pin_x.pin_x_sel_I[pin % 64];
}

/**
 * \brief GPIOŷΪ
 *
 * \param[in] p_hw_gpio  : ָGPIOĴָ
 * \param[in] pin        : űţֵΪ PIO* (#PIOA_0)
 *
 * \return 
 */
am_static_inline
void amhw_zsn700_gpio_pin_dir_output (amhw_zsn700_gpio_t *p_hw_gpio,
                                      int                 pin)
{
    p_hw_gpio->gpio[pin / 64].gpio_port[(pin % 64) / 16].dir &= ~(0x1ul << (pin % 16));
}

/**
 * \brief GPIOŷΪ
 *
 * \param[in] p_hw_gpio  : ָGPIOĴָ
 * \param[in] pin        : űţֵΪ PIO* (#PIOA_0)
 *
 * \return 
 */
am_static_inline
void amhw_zsn700_gpio_pin_dir_input (amhw_zsn700_gpio_t *p_hw_gpio, int pin)
{
    p_hw_gpio->gpio[pin / 64].gpio_port[(pin % 64) / 16].dir |= (0x1ul << (pin % 16));
}

/**
 * \brief ȡGPIO
 *
 * \param[in] p_hw_gpio  : ָGPIOĴָ
 * \param[in] pin        : űţֵΪ PIO* (#PIOA_0)
 *
 * \return 1  
 *         0  
 */
am_static_inline
int amhw_zsn700_gpio_pin_dir_get (amhw_zsn700_gpio_t *p_hw_gpio, int pin)
{
    return (p_hw_gpio->gpio[pin / 64].gpio_port[(pin % 64) / 16].dir & (0x1ul << (pin % 16))) ? 1 : 0;
}

/**
 * \brief GPIOƽ
 *
 * \param[in] p_hw_gpio : ָGPIOĴָ
 * \param[in] pin       : ű,ֵΪ PIO* (#PIOA_0)
 *
 * \retval 0 : ͵ƽ
 * \retval 1 : ߵƽ
 */
am_static_inline
int amhw_zsn700_gpio_pin_input_get(amhw_zsn700_gpio_t *p_hw_gpio, int pin)
{
    return (p_hw_gpio->gpio[pin / 64].gpio_port[(pin % 64) / 16].in & (0x1ul << (pin % 16))) ? 1 : 0;
}

/**
 * \brief GPIOŵƽΪ
 *
 * \param[in] p_hw_gpio : ָGPIOĴָ
 * \param[in] pin       : ű,ֵΪ PIO* (#PIOA_0)
 *
 * \return 
 *
 * \note óɿ©ⲿߡ
 */
am_static_inline
void amhw_zsn700_gpio_pin_out_high (amhw_zsn700_gpio_t *p_hw_gpio, int pin)
{
    p_hw_gpio->gpio[pin / 64].gpio_port[(pin % 64) / 16].out |= (0x1ul << (pin % 16));
}

/**
 * \brief GPIOŵƽΪ
 *
 * \param[in] p_hw_gpio : ָGPIOĴָ
 * \param[in] pin       : ű,ֵΪ PIO* (#PIOA_0)
 *
 * \return 
 *
 */
am_static_inline
void amhw_zsn700_gpio_pin_out_low (amhw_zsn700_gpio_t *p_hw_gpio, int pin)
{
    p_hw_gpio->gpio[pin / 64].gpio_port[(pin % 64) / 16].out &= ~(0x1ul << (pin % 16));
}

/**
 * \brief GPIOƽ
 *
 * \param[in] p_hw_gpio : ָGPIOĴָ
 * \param[in] pin       : ű,ֵΪ PIO* (#PIOA_0)
 *
 * \retval 0 : ͵ƽ
 * \retval 1 : ߵƽ
 */
am_static_inline
int amhw_zsn700_gpio_pin_output_get (amhw_zsn700_gpio_t *p_hw_gpio, int pin)
{
    return (p_hw_gpio->gpio[pin / 64].gpio_port[(pin % 64) / 16].out & (0x1ul << (pin % 16))) ? 1 : 0;
}

/**
 * \brief GPIOΪģ˿
 *
 * \param[in] p_hw_gpio : ָGPIOĴָ
 * \param[in] pin       : ű,ֵΪ PIO* (#PIOA_0)
 *
 * \return 
 */
am_static_inline
void amhw_zsn700_gpio_pin_analog_set (amhw_zsn700_gpio_t *p_hw_gpio, int pin)
{
    p_hw_gpio->gpio[pin / 64].gpio_port[(pin % 64) / 16].ads |= (0x1ul << (pin % 16));
}

/**
 * \brief GPIOΪֶ˿
 *
 * \param[in] p_hw_gpio : ָGPIOĴָ
 * \param[in] pin       : ű,ֵΪ PIO* (#PIOA_0)
 *
 * \return 
 */
am_static_inline
void amhw_zsn700_gpio_pin_digital_set (amhw_zsn700_gpio_t *p_hw_gpio, int pin)
{
    p_hw_gpio->gpio[pin / 64].gpio_port[(pin % 64) / 16].ads &= ~(0x1ul << (pin % 16));
}

/**
 * \brief ȡǰ˿ֶ˿/ģ˿
 *
 * \param[in] p_hw_gpio : ָGPIOĴָ
 * \param[in] pin       : ű,ֵΪ PIO* (#PIOA_0)
 *
 * \retval 0 : ֶ˿
 * \retval 1 : ģ˿
 */
am_static_inline
int amhw_zsn700_gpio_pin_analog_digital_get (amhw_zsn700_gpio_t *p_hw_gpio, int pin)
{
    return (p_hw_gpio->gpio[pin / 64].gpio_port[(pin % 64) / 16].ads & (0x1ul << (pin % 16))) ? 1 : 0;
}

/**
 * \brief λλĴ     λ
 *
 * \param[in] p_hw_gpio : ָGPIOĴָ
 * \param[in] pin       : ű,ֵΪ PIO* (#PIOA_0)
 *
 * \return 
 */
am_static_inline
void amhw_zsn700_gpio_pin_bit_set (amhw_zsn700_gpio_t *p_hw_gpio, int pin)
{
    p_hw_gpio->gpio[pin / 64].gpio_port[(pin % 64) / 16].bset |= (0x1ul << (pin % 16));
}

/**
 * \brief λĴ    
 *
 * \param[in] p_hw_gpio : ָGPIOĴָ
 * \param[in] pin       : ű,ֵΪ PIO* (#PIOA_0)
 *
 * \return 
 */
am_static_inline
void amhw_zsn700_gpio_pin_bit_clr (amhw_zsn700_gpio_t *p_hw_gpio, int pin)
{
    p_hw_gpio->gpio[pin / 64].gpio_port[(pin % 64) / 16].bclr |= (0x1ul << (pin % 16));
}

/**
 * \brief λĴ   
 *
 * \param[in] p_hw_gpio : ָGPIOĴָ
 * \param[in] pin       : ű,ֵΪ PIO* (#PIOA_0)
 *
 * \return 
 */
am_static_inline
void amhw_zsn700_gpio_pin_bit_hold (amhw_zsn700_gpio_t *p_hw_gpio, int pin)
{
    p_hw_gpio->gpio[pin / 64].gpio_port[(pin % 64) / 16].bset &= ~(0x1ul << (pin % 16));
    p_hw_gpio->gpio[pin / 64].gpio_port[(pin % 64) / 16].bclr &= ~(0x1ul << (pin % 16));
}


/**
 * \brief λλĴ   λ
 *
 * \param[in] p_hw_gpio : ָGPIOĴָ
 * \param[in] pin       : ű,ֵΪ PIO* (#PIOA_0)
 *
 *
 * \note üĴ    16bit -- λ  Ӧλд1
 *              16bit --   Ӧλд1
 *
 *       16bit͵16bit ͬλͬʱд1ʱ
 *       иȼö˿ڱ㡣
 *
 * \return 
 */
am_static_inline
void amhw_zsn700_gpio_pin_bitsetclr_set (amhw_zsn700_gpio_t *p_hw_gpio, int pin)
{
    p_hw_gpio->gpio[pin / 64].gpio_port[(pin % 64) / 16].bsetclr |= (0x1ul << ((pin % 16) + 16));
}

/**
 * \brief λλĴ    
 *
 * \param[in] p_hw_gpio : ָGPIOĴָ
 * \param[in] pin       : ű,ֵΪ PIO* (#PIOA_0)
 *
 * \note üĴ    16bit -- λ  Ӧλд1
 *              16bit --   Ӧλд1
 *
 *       16bit͵16bit ͬλͬʱд1ʱ
 *       иȼö˿ڱ㡣
 *
 * \return 
 */
am_static_inline
void amhw_zsn700_gpio_pin_bitsetclr_clr (amhw_zsn700_gpio_t *p_hw_gpio, int pin)
{
    p_hw_gpio->gpio[pin / 64].gpio_port[(pin % 64) / 16].bsetclr |= (0x1ul << (pin % 16));
}


/**
 * \brief λλĴ    
 *
 * \param[in] p_hw_gpio : ָGPIOĴָ
 * \param[in] pin       : ű,ֵΪ PIO* (#PIOA_0)
 *
 * \return 
 */
am_static_inline
void amhw_zsn700_gpio_pin_bitsetclr_hold (amhw_zsn700_gpio_t *p_hw_gpio, int pin)
{
    p_hw_gpio->gpio[pin / 64].gpio_port[(pin % 64) / 16].bsetclr &= ~(0x1ul << (pin % 16));
    p_hw_gpio->gpio[pin / 64].gpio_port[(pin % 64) / 16].bsetclr &= ~(0x1ul << ((pin % 16) + 16));
}

/**
 * \brief ʹüĴ    ʹ
 *
 * \param[in] p_hw_gpio : ָGPIOĴָ
 * \param[in] pin       : ű,ֵΪ PIO* (#PIOA_0)
 *
 * \return 
 */
am_static_inline
void amhw_zsn700_gpio_pin_pu_enable (amhw_zsn700_gpio_t *p_hw_gpio, int pin)
{
    p_hw_gpio->gpio[pin / 64].gpio_port[(pin % 64) / 16].pu |= (0x1ul << (pin % 16));
}

/**
 * \brief ʹüĴ   
 *
 * \param[in] p_hw_gpio : ָGPIOĴָ
 * \param[in] pin       : ű,ֵΪ PIO* (#PIOA_0)
 *
 * \return 
 */
am_static_inline
void amhw_zsn700_gpio_pin_pu_disable (amhw_zsn700_gpio_t *p_hw_gpio, int pin)
{
    p_hw_gpio->gpio[pin / 64].gpio_port[(pin % 64) / 16].pu &= ~(0x1ul << (pin % 16));
}

/**
 * \brief ʹüĴ    ʹ
 *
 * \param[in] p_hw_gpio : ָGPIOĴָ
 * \param[in] pin       : ű,ֵΪ PIO* (#PIOA_0)
 *
 * \return 
 */
am_static_inline
void amhw_zsn700_gpio_pin_pd_enable (amhw_zsn700_gpio_t *p_hw_gpio, int pin)
{
    p_hw_gpio->gpio[pin / 64].gpio_port[(pin % 64) / 16].pd |= (0x1ul << (pin % 16));
}

/**
 * \brief ʹüĴ   
 *
 * \param[in] p_hw_gpio : ָGPIOĴָ
 * \param[in] pin       : ű,ֵΪ PIO* (#PIOA_0)
 *
 * \return 
 */
am_static_inline
void amhw_zsn700_gpio_pin_pd_disable (amhw_zsn700_gpio_t *p_hw_gpio, int pin)
{
    p_hw_gpio->gpio[pin / 64].gpio_port[(pin % 64) / 16].pd &= ~(0x1ul << (pin % 16));
}

/**
 * \brief ©üĴ    ʹ
 *
 * \param[in] p_hw_gpio : ָGPIOĴָ
 * \param[in] pin       : ű,ֵΪ PIO* (#PIOA_0)
 *
 * \return 
 */
am_static_inline
void amhw_zsn700_gpio_pin_od_enable (amhw_zsn700_gpio_t *p_hw_gpio, int pin)
{
    p_hw_gpio->gpio[pin / 64].gpio_port[(pin % 64) / 16].od |= (0x1ul << (pin % 16));
}

/**
 * \brief ©üĴ   
 *
 * \param[in] p_hw_gpio : ָGPIOĴָ
 * \param[in] pin       : ű,ֵΪ PIO* (#PIOA_0)
 *
 * \return 
 */
am_static_inline
void amhw_zsn700_gpio_pin_od_disable (amhw_zsn700_gpio_t *p_hw_gpio, int pin)
{
    p_hw_gpio->gpio[pin / 64].gpio_port[(pin % 64) / 16].od &= ~(0x1ul << (pin % 16));
}

/**
 * \brief üĴ    
 *
 * \param[in] p_hw_gpio : ָGPIOĴָ
 * \param[in] pin       : ű,ֵΪ PIO* (#PIOA_0)
 *
 * \return 
 */
am_static_inline
void amhw_zsn700_gpio_pin_driver_low (amhw_zsn700_gpio_t *p_hw_gpio, int pin)
{
    p_hw_gpio->gpio[pin / 64].gpio_port[(pin % 64) / 16].dr |= (0x1ul << (pin % 16));
}

/**
 * \brief üĴ  
 *
 * \param[in] p_hw_gpio : ָGPIOĴָ
 * \param[in] pin       : ű,ֵΪ PIO* (#PIOA_0)
 *
 * \return 
 */
am_static_inline
void amhw_zsn700_gpio_pin_driver_high (amhw_zsn700_gpio_t *p_hw_gpio, int pin)
{
    p_hw_gpio->gpio[pin / 64].gpio_port[(pin % 64) / 16].dr &= ~(0x1ul << (pin % 16));
}

/**
 * \brief ߵƽжʹüĴ    ʹ
 *
 * \param[in] p_hw_gpio : ָGPIOĴָ
 * \param[in] pin       : ű,ֵΪ PIO* (#PIOA_0)
 *
 * \return 
 */
am_static_inline
void amhw_zsn700_gpio_pin_high_int_enable (amhw_zsn700_gpio_t *p_hw_gpio, int pin)
{
    p_hw_gpio->gpio[pin / 64].gpio_port[(pin % 64) / 16].hie |= (0x1ul << (pin % 16));
}

/**
 * \brief ߵƽжʹüĴ   
 *
 * \param[in] p_hw_gpio : ָGPIOĴָ
 * \param[in] pin       : ű,ֵΪ PIO* (#PIOA_0)
 *
 * \return 
 */
am_static_inline
void amhw_zsn700_gpio_pin_high_int_disable (amhw_zsn700_gpio_t *p_hw_gpio, int pin)
{
    p_hw_gpio->gpio[pin / 64].gpio_port[(pin % 64) / 16].hie &= ~(0x1ul << (pin % 16));
}

/**
 * \brief ͵ƽжʹüĴ    ʹ
 *
 * \param[in] p_hw_gpio : ָGPIOĴָ
 * \param[in] pin       : ű,ֵΪ PIO* (#PIOA_0)
 *
 * \return 
 */
am_static_inline
void amhw_zsn700_gpio_pin_low_int_enable (amhw_zsn700_gpio_t *p_hw_gpio, int pin)
{
    p_hw_gpio->gpio[pin / 64].gpio_port[(pin % 64) / 16].lie |= (0x1ul << (pin % 16));
}

/**
 * \brief ͵ƽжʹüĴ   
 *
 * \param[in] p_hw_gpio : ָGPIOĴָ
 * \param[in] pin       : ű,ֵΪ PIO* (#PIOA_0)
 *
 * \return 
 */
am_static_inline
void amhw_zsn700_gpio_pin_low_int_disable (amhw_zsn700_gpio_t *p_hw_gpio, int pin)
{
    p_hw_gpio->gpio[pin / 64].gpio_port[(pin % 64) / 16].lie &= ~(0x1ul << (pin % 16));
}

/**
 * \brief жʹüĴ    ʹ
 *
 * \param[in] p_hw_gpio : ָGPIOĴָ
 * \param[in] pin       : ű,ֵΪ PIO* (#PIOA_0)
 *
 * \return 
 */
am_static_inline
void amhw_zsn700_gpio_pin_raising_int_enable (amhw_zsn700_gpio_t *p_hw_gpio, int pin)
{
    p_hw_gpio->gpio[pin / 64].gpio_port[(pin % 64) / 16].rie |= (0x1ul << (pin % 16));
}

/**
 * \brief жʹüĴ   
 *
 * \param[in] p_hw_gpio : ָGPIOĴָ
 * \param[in] pin       : ű,ֵΪ PIO* (#PIOA_0)
 *
 * \return 
 */
am_static_inline
void amhw_zsn700_gpio_pin_raising_int_disable (amhw_zsn700_gpio_t *p_hw_gpio, int pin)
{
    p_hw_gpio->gpio[pin / 64].gpio_port[(pin % 64) / 16].rie &= ~(0x1ul << (pin % 16));
}

/**
 * \brief ½жʹüĴ    ʹ
 *
 * \param[in] p_hw_gpio : ָGPIOĴָ
 * \param[in] pin       : ű,ֵΪ PIO* (#PIOA_0)
 *
 * \return 
 */
am_static_inline
void amhw_zsn700_gpio_pin_falling_int_enable (amhw_zsn700_gpio_t *p_hw_gpio, int pin)
{
    p_hw_gpio->gpio[pin / 64].gpio_port[(pin % 64) / 16].fie |= (0x1ul << (pin % 16));
}

/**
 * \brief ½жʹüĴ   
 *
 * \param[in] p_hw_gpio : ָGPIOĴָ
 * \param[in] pin       : ű,ֵΪ PIO* (#PIOA_0)
 *
 * \return 
 */
am_static_inline
void amhw_zsn700_gpio_pin_falling_int_disable (amhw_zsn700_gpio_t *p_hw_gpio, int pin)
{
    p_hw_gpio->gpio[pin / 64].gpio_port[(pin % 64) / 16].fie &= ~(0x1ul << (pin % 16));
}

/**
 * \brief ж״̬Ĵ  ȡǷжϴ
 *
 * \param[in] p_hw_gpio : ָGPIOĴָ
 * \param[in] pin       : ű,ֵΪ PIO* (#PIOA_0)
 *
 * \retval 1 : жϴ
 * \retval 0 : жϴ
 */
am_static_inline
int amhw_zsn700_gpio_pin_int_flag_get (amhw_zsn700_gpio_t *p_hw_gpio, int pin)
{
    return (p_hw_gpio->gpio[pin / 64].gpio_int_reg[(pin % 64) / 16].stat & (0x1ul << (pin % 16))) ? 1 : 0;
}

/**
 * \brief жĴ  ־
 *
 * \param[in] p_hw_gpio : ָGPIOĴָ
 * \param[in] pin       : ű,ֵΪ PIO* (#PIOA_0)
 *
 * \retval none
 */
am_static_inline
void amhw_zsn700_gpio_pin_int_flag_clr (amhw_zsn700_gpio_t *p_hw_gpio, int pin)
{
    p_hw_gpio->gpio[pin / 64].gpio_int_reg[(pin % 64) / 16].iclr |= (0x1ul << (pin % 16));
    p_hw_gpio->gpio[pin / 64].gpio_int_reg[(pin % 64) / 16].iclr &= ~(0x1ul << (pin % 16));
    p_hw_gpio->gpio[pin / 64].gpio_int_reg[(pin % 64) / 16].iclr |= (0x1ul << (pin % 16));
}

/**
 * \brief жĴ  ı
 *
 * \param[in] p_hw_gpio : ָGPIOĴָ
 * \param[in] pin       : ű,ֵΪ PIO* (#PIOA_0)
 *
 * \retval none
 */
am_static_inline
void amhw_zsn700_gpio_pin_int_flag_hold (amhw_zsn700_gpio_t *p_hw_gpio, int pin)
{
    p_hw_gpio->gpio[pin / 64].gpio_int_reg[(pin % 64) / 16].iclr |= (0x1ul << (pin % 16));
}

/**
 * \brief ˿ڸüĴ0  DEEPSLEEPģʽ
 *
 * \param[in] p_hw_gpio : ָGPIOĴָ
 *
 * \note ϵͳ ACTIVE/SLEEPģʽʱϵͳʱӲᱻرգԽ IESELΪ 0
 *       ʱ˿жϵⲿźԴϵͳʱ֮ͬжźţ˳ⲿźԴ
 *       ë̡
 *       ϵͳ DEEPSLEEPģʽʱϵͳʱӽرգԽ IESELΪ 1
 *       ʱ˿жϵⲿźԴֱӲжźţ˳ⲿźԴë̡
 *
 * \return 
 */
am_static_inline
void amhw_zsn700_gpio_sup_int_mode_deepsleep (amhw_zsn700_gpio_t *p_hw_gpio)
{
    p_hw_gpio->gpio[0].gpio_sup_reg.ctrl0 |= (0x1ul);
}

/**
 * \brief ˿ڸüĴ0  ACTIVE/SLEEPģʽ
 *
 * \param[in] p_hw_gpio : ָGPIOĴָ
 *
 * \note ϵͳ ACTIVE/SLEEPģʽʱϵͳʱӲᱻرգԽ IESELΪ 0
 *       ʱ˿жϵⲿźԴϵͳʱ֮ͬжźţ˳ⲿźԴ
 *       ë̡
 *       ϵͳ DEEPSLEEPģʽʱϵͳʱӽرգԽ IESELΪ 1
 *       ʱ˿жϵⲿźԴֱӲжźţ˳ⲿźԴë̡
 *
 * \return 
 */
am_static_inline
void amhw_zsn700_gpio_sup_int_mode_active_or_sleep (amhw_zsn700_gpio_t *p_hw_gpio)
{
    p_hw_gpio->gpio[0].gpio_sup_reg.ctrl0 &= ~(0x1ul);
}

/**
 * \brief IRѡ  
 *
 * \param[in] p_hw_gpio : ָGPIOĴָ
 *
 * \return 
 */
am_static_inline
void amhw_zsn700_gpio_sup_irout_polarity_forward (amhw_zsn700_gpio_t *p_hw_gpio)
{
    p_hw_gpio->gpio[0].gpio_sup_reg.ctrl1 &= ~((0x1ul << 14));
}

/**
 * \brief IRѡ  
 *
 * \param[in] p_hw_gpio : ָGPIOĴָ
 *
 * \return 
 */
am_static_inline
void amhw_zsn700_gpio_sup_irout_polarity_reverse (amhw_zsn700_gpio_t *p_hw_gpio)
{
    p_hw_gpio->gpio[0].gpio_sup_reg.ctrl1 |= ((0x1ul << 14));
}

/**
 * \brief IRԻȡ
 *
 * \param[in] p_hw_gpio : ָGPIOĴָ
 *
 * \retval 0 : 
 * \retval 1 : 
 */
am_static_inline
int amhw_zsn700_gpio_sup_irout_polarity_get (amhw_zsn700_gpio_t *p_hw_gpio)
{
    return (p_hw_gpio->gpio[0].gpio_sup_reg.ctrl1 & (0x1ul << 14)) ? 1 : 0;
}

/**
 * \brief hclkſ   ſ
 *
 * \param[in] p_hw_gpio : ָGPIOĴָ
 *
 * \return 
 */
am_static_inline
void amhw_zsn700_gpio_sup_hclk_gate (amhw_zsn700_gpio_t *p_hw_gpio)
{
    p_hw_gpio->gpio[0].gpio_sup_reg.ctrl1 &= ~((0x1ul << 13));
}

/**
 * \brief hclkſ    
 *
 * \param[in] p_hw_gpio : ָGPIOĴָ
 *
 * \return 
 */
am_static_inline
void amhw_zsn700_gpio_sup_hclk_output (amhw_zsn700_gpio_t *p_hw_gpio)
{
    p_hw_gpio->gpio[0].gpio_sup_reg.ctrl1 |= ((0x1ul << 13));
}

/**
 * \brief pclkſ   ſ
 *
 * \param[in] p_hw_gpio : ָGPIOĴָ
 *
 * \return 
 */
am_static_inline
void amhw_zsn700_gpio_sup_pclk_gate (amhw_zsn700_gpio_t *p_hw_gpio)
{
    p_hw_gpio->gpio[0].gpio_sup_reg.ctrl1 &= ~((0x1ul << 12));
}

/**
 * \brief pclkſ    
 *
 * \param[in] p_hw_gpio : ָGPIOĴָ
 *
 * \return 
 */
am_static_inline
void amhw_zsn700_gpio_sup_pclk_output (amhw_zsn700_gpio_t *p_hw_gpio)
{
    p_hw_gpio->gpio[0].gpio_sup_reg.ctrl1 |= ((0x1ul << 12));
}

/**
 * \brief hclkƵѡ 
 *        pclkƵѡ 
 *        tclkƵѡ
 *        ö
 */
typedef enum {
    AMHW_ZSN700_GPIO_SUP_CLK_DIV_NO = 0ul,
    AMHW_ZSN700_GPIO_SUP_CLK_DIV_2,
    AMHW_ZSN700_GPIO_SUP_CLK_DIV_4,
    AMHW_ZSN700_GPIO_SUP_CLK_DIV_8,
}amhw_zsn700_gpio_sup_clk_div_t;

/**
 * \brief hclkƵѡ
 *
 * \param[in] p_hw_gpio : ָGPIOĴָ
 * \param[in] div_value : Ƶϵ    amhw_zsn700_gpio_sup_clk_div_tö
 *
 * \return 
 */
am_static_inline
void amhw_zsn700_gpio_sup_hclk_div_set (amhw_zsn700_gpio_t             *p_hw_gpio,
                                        amhw_zsn700_gpio_sup_clk_div_t  div_value)
{
    p_hw_gpio->gpio[0].gpio_sup_reg.ctrl1 = (p_hw_gpio->gpio[0].gpio_sup_reg.ctrl1 & (~(0x3ul << 10))) |
                                            ((div_value & 0x3ul) << 10);
}

/**
 * \brief hclkƵϵȡ
 *
 * \param[in] p_hw_gpio : ָGPIOĴָ
 *
 * \return ǰƵϵ amhw_zsn700_gpio_sup_clk_div_töеһ
 */
am_static_inline
amhw_zsn700_gpio_sup_clk_div_t amhw_zsn700_gpio_sup_hclk_div_get (amhw_zsn700_gpio_t *p_hw_gpio)
{
    return (amhw_zsn700_gpio_sup_clk_div_t)((p_hw_gpio->gpio[0].gpio_sup_reg.ctrl1 >> 10) & 0x3ul);
}

/**
 * \brief pclkƵѡ
 *
 * \param[in] p_hw_gpio : ָGPIOĴָ
 * \param[in] div_value : Ƶϵ    amhw_zsn700_gpio_sup_clk_div_tö
 *
 * \return 
 */
am_static_inline
void amhw_zsn700_gpio_sup_pclk_div_set (amhw_zsn700_gpio_t             *p_hw_gpio,
                                        amhw_zsn700_gpio_sup_clk_div_t  div_value)
{
    p_hw_gpio->gpio[0].gpio_sup_reg.ctrl1 = (p_hw_gpio->gpio[0].gpio_sup_reg.ctrl1 & (~(0x3ul << 8))) |
                                            ((div_value & 0x3ul) << 8);
}

/**
 * \brief pclkƵϵȡ
 *
 * \param[in] p_hw_gpio : ָGPIOĴָ
 *
 * \return ǰƵϵ amhw_zsn700_gpio_sup_clk_div_töеһ
 */
am_static_inline
amhw_zsn700_gpio_sup_clk_div_t amhw_zsn700_gpio_sup_pclk_div_get (amhw_zsn700_gpio_t *p_hw_gpio)
{
    return (amhw_zsn700_gpio_sup_clk_div_t)((p_hw_gpio->gpio[0].gpio_sup_reg.ctrl1 >> 8) & 0x3ul);
}

/**
 * \brief ⲿʱźԴѡ  
 *        SPI0SSNźԴѡ 
 *        SPI1SSNźԴѡ
 *        ö
 */
typedef enum {
    AMHW_ZSN700_GPIO_SUP_EXTCLK_SSN01_HIGH = 0,
    AMHW_ZSN700_GPIO_SUP_EXTCLK_SSN01_PA3,
    AMHW_ZSN700_GPIO_SUP_EXTCLK_SSN01_PA4,
    AMHW_ZSN700_GPIO_SUP_EXTCLK_SSN01_PA6,
    AMHW_ZSN700_GPIO_SUP_EXTCLK_SSN01_PA8,
    AMHW_ZSN700_GPIO_SUP_EXTCLK_SSN01_PA9,
    AMHW_ZSN700_GPIO_SUP_EXTCLK_SSN01_PA12,
    AMHW_ZSN700_GPIO_SUP_EXTCLK_SSN01_PA15,
    AMHW_ZSN700_GPIO_SUP_EXTCLK_SSN01_PB1,
    AMHW_ZSN700_GPIO_SUP_EXTCLK_SSN01_PB2,
    AMHW_ZSN700_GPIO_SUP_EXTCLK_SSN01_PB5,
    AMHW_ZSN700_GPIO_SUP_EXTCLK_SSN01_PB6,
    AMHW_ZSN700_GPIO_SUP_EXTCLK_SSN01_PB9,
    AMHW_ZSN700_GPIO_SUP_EXTCLK_SSN01_PB10,
    AMHW_ZSN700_GPIO_SUP_EXTCLK_SSN01_PB12,
    AMHW_ZSN700_GPIO_SUP_EXTCLK_SSN01_PB14,
}amhw_zsn700_gpio_sup_extclk_or_ssn01_src_t;

/**
 * \brief SPI0 SSNźԴѡ
 *
 * \param[in] p_hw_gpio : ָGPIOĴָ
 * \param[in] ssn0_src  : źԴ    amhw_zsn700_gpio_sup_extclk_or_ssn01_src_tö
 *
 * \return 
 */
am_static_inline
void amhw_zsn700_gpio_sup_spi0_ssn_set (amhw_zsn700_gpio_t                         *p_hw_gpio,
                                        amhw_zsn700_gpio_sup_extclk_or_ssn01_src_t  ssn0_src)
{
    p_hw_gpio->gpio[0].gpio_sup_reg.ctrl1 = (p_hw_gpio->gpio[0].gpio_sup_reg.ctrl1 & (~(0xful << 4))) |
                                            ((ssn0_src & 0xful) << 4);
}

/**
 * \brief SPI0 SSNźԴȡ
 *
 * \param[in] p_hw_gpio : ָGPIOĴָ
 *
 * \return ǰƵϵ amhw_zsn700_gpio_sup_extclk_or_ssn01_src_töеһ
 */
am_static_inline
amhw_zsn700_gpio_sup_extclk_or_ssn01_src_t amhw_zsn700_gpio_sup_spi0_ssn_get (amhw_zsn700_gpio_t *p_hw_gpio)
{
    return (amhw_zsn700_gpio_sup_extclk_or_ssn01_src_t)((p_hw_gpio->gpio[0].gpio_sup_reg.ctrl1 >> 4) & 0xful);
}

/**
 * \brief SPI1 SSNźԴѡ
 *
 * \param[in] p_hw_gpio : ָGPIOĴָ
 * \param[in] ssn1_src  : źԴ    amhw_zsn700_gpio_sup_extclk_or_ssn01_src_tö
 *
 * \return 
 */
am_static_inline
void amhw_zsn700_gpio_sup_spi1_ssn_set (amhw_zsn700_gpio_t                         *p_hw_gpio,
                                        amhw_zsn700_gpio_sup_extclk_or_ssn01_src_t  ssn1_src)
{
    p_hw_gpio->gpio[0].gpio_sup_reg.ctrl2 = (p_hw_gpio->gpio[0].gpio_sup_reg.ctrl2 & (~(0xful << 0))) |
                                            ((ssn1_src & 0xful) << 0);
}

/**
 * \brief SPI1 SSNźԴȡ
 *
 * \param[in] p_hw_gpio : ָGPIOĴָ
 *
 * \return ǰƵϵ amhw_zsn700_gpio_sup_extclk_or_ssn01_src_töеһ
 */
am_static_inline
amhw_zsn700_gpio_sup_extclk_or_ssn01_src_t amhw_zsn700_gpio_sup_spi1_ssn_get (amhw_zsn700_gpio_t *p_hw_gpio)
{
    return (amhw_zsn700_gpio_sup_extclk_or_ssn01_src_t)((p_hw_gpio->gpio[0].gpio_sup_reg.ctrl2 >> 0) & 0xful);
}

/**
 * \brief ⲿʱźԴѡ
 *
 * \param[in] p_hw_gpio : ָGPIOĴָ
 * \param[in] ssn1_src  : źԴ    amhw_zsn700_gpio_sup_extclk_or_ssn01_src_tö
 *
 * \return 
 */
am_static_inline
void amhw_zsn700_gpio_sup_ext_clk_set (amhw_zsn700_gpio_t                         *p_hw_gpio,
                                       amhw_zsn700_gpio_sup_extclk_or_ssn01_src_t  extclk_src)
{
    p_hw_gpio->gpio[0].gpio_sup_reg.ctrl1 = (p_hw_gpio->gpio[0].gpio_sup_reg.ctrl1 & (~(0xful << 0))) |
                                            ((extclk_src & 0xful) << 0);
}

/**
 * \brief ⲿʱźԴȡ
 *
 * \param[in] p_hw_gpio : ָGPIOĴָ
 *
 * \return ǰƵϵ amhw_zsn700_gpio_sup_extclk_or_ssn01_src_töеһ
 */
am_static_inline
amhw_zsn700_gpio_sup_extclk_or_ssn01_src_t amhw_zsn700_gpio_sup_ext_clk_get (amhw_zsn700_gpio_t *p_hw_gpio)
{
    return (amhw_zsn700_gpio_sup_extclk_or_ssn01_src_t)((p_hw_gpio->gpio[0].gpio_sup_reg.ctrl1 >> 0) & 0xful);
}

/**
 * \brief AHB߷ģʽ   ʹFASTIO߷PxIN,PxOUTĴ
 *
 * \param[in] p_hw_gpio : ָGPIOĴָ
 *
 * \return 
 */
am_static_inline
void amhw_zsn700_gpio_sup_inout_fastio_bus_mode (amhw_zsn700_gpio_t *p_hw_gpio)
{
    p_hw_gpio->gpio[0].gpio_sup_reg.ctrl2 &= ~(0x3ul << 15);
}

/**
 * \brief AHB߷ģʽ   ʹAHB߷PxIN,PxOUTĴ
 *
 * \param[in] p_hw_gpio : ָGPIOĴָ
 *
 * \return 
 */
am_static_inline
void amhw_zsn700_gpio_sup_inout_ahb_bus_mode (amhw_zsn700_gpio_t *p_hw_gpio)
{
    p_hw_gpio->gpio[0].gpio_sup_reg.ctrl2 |= (0x3ul << 15);
}

/**
 * \brief AHB߷ģʽȡ
 *
 * \param[in] p_hw_gpio : ָGPIOĴָ
 *
 * \return   0   ʹFASTIO߷PxIN,PxOUTĴ
 *           1   ʹAHB߷PxIN,PxOUTĴ
 */
am_static_inline
int amhw_zsn700_gpio_sup_inout_bus_get (amhw_zsn700_gpio_t *p_hw_gpio)
{
    return (p_hw_gpio->gpio[0].gpio_sup_reg.ctrl2 & (0x3ul << 15)) ? 1 : 0;
}

/**
 * \brief tclkźԴѡ
 *        ö
 */
typedef enum {
    AMHW_ZSN700_GPIO_SUP_TCLK_NO = 0,
    AMHW_ZSN700_GPIO_SUP_TCLK_RCH,
    AMHW_ZSN700_GPIO_SUP_TCLK_XTH,
    AMHW_ZSN700_GPIO_SUP_TCLK_PLL,
}amhw_zsn700_gpio_sup_tclk_src_t;

/**
 * \brief tclkƵѡ
 *
 * \param[in] p_hw_gpio : ָGPIOĴָ
 * \param[in] div_value : Ƶϵ    amhw_zsn700_gpio_sup_clk_div_tö
 *
 * \return 
 */
am_static_inline
void amhw_zsn700_gpio_sup_tclk_div_set (amhw_zsn700_gpio_t             *p_hw_gpio,
                                        amhw_zsn700_gpio_sup_clk_div_t  div_value)
{
    p_hw_gpio->gpio[0].gpio_sup_reg.ctrl2 = (p_hw_gpio->gpio[0].gpio_sup_reg.ctrl2 & (~(0x3ul << 6))) |
                                            ((div_value & 0x3ul) << 6);
}

/**
 * \brief tclkƵϵȡ
 *
 * \param[in] p_hw_gpio : ָGPIOĴָ
 *
 * \return ǰƵϵ amhw_zsn700_gpio_sup_clk_div_töеһ
 */
am_static_inline
amhw_zsn700_gpio_sup_clk_div_t amhw_zsn700_gpio_sup_tclk_div_get (amhw_zsn700_gpio_t *p_hw_gpio)
{
    return (amhw_zsn700_gpio_sup_clk_div_t)((p_hw_gpio->gpio[0].gpio_sup_reg.ctrl2 >> 6) & 0x3ul);
}

/**
 * \brief tclkźԴѡ
 *
 * \param[in] p_hw_gpio : ָGPIOĴָ
 * \param[in] tclk_src  : Ƶϵ    amhw_zsn700_gpio_sup_tclk_src_tö
 *
 * \return 
 */
am_static_inline
void amhw_zsn700_gpio_sup_tclk_src_set (amhw_zsn700_gpio_t             *p_hw_gpio,
                                        amhw_zsn700_gpio_sup_clk_div_t  tclk_src)
{
    p_hw_gpio->gpio[0].gpio_sup_reg.ctrl2 = (p_hw_gpio->gpio[0].gpio_sup_reg.ctrl2 & (~(0x3ul << 4))) |
                                            ((tclk_src & 0x3ul) << 4);
}

/**
 * \brief ttclkźԴȡ
 *
 * \param[in] p_hw_gpio : ָGPIOĴָ
 *
 * \return ǰƵϵ amhw_zsn700_gpio_sup_tclk_src_töеһ
 */
am_static_inline
amhw_zsn700_gpio_sup_tclk_src_t amhw_zsn700_gpio_sup_tclk_src_get (amhw_zsn700_gpio_t *p_hw_gpio)
{
    return (amhw_zsn700_gpio_sup_tclk_src_t)((p_hw_gpio->gpio[0].gpio_sup_reg.ctrl2 >> 4) & 0x3ul);
}

/**
 * \brief LPTimer1ʱGATEѡ
 *
 * \param[in] p_hw_gpio : ָGPIOĴָ
 * \param[in] src       : 趨ֵ
 *
 * \note ѡֵοûֲGPIO½ڵ
 *         --˿ڸ PCA ѡ(GPIO_PCAS)
 *
 * \return 
 */
am_static_inline
void amhw_zsn700_gpio_sup_lptim1_gate_src_set (amhw_zsn700_gpio_t *p_hw_gpio,
                                               uint8_t             src)
{
    p_hw_gpio->gpio[0].gpio_sup_reg.pcas = (p_hw_gpio->gpio[0].gpio_sup_reg.pcas & (~(0x7ul << 6))) |
                                           ((src & 0x7ul) << 6);
}

/**
 * \brief LPTimer1ʱGATEȡ
 *
 * \param[in] p_hw_gpio : ָGPIOĴָ
 *
 * \note ѡֵοûֲGPIO½ڵ
 *       --˿ڸ PCA ѡ(GPIO_PCAS)
 *
 * \return 
 */
am_static_inline
uint8_t amhw_zsn700_gpio_sup_lptim1_gate_src_get (amhw_zsn700_gpio_t *p_hw_gpio)
{
    return ((p_hw_gpio->gpio[0].gpio_sup_reg.pcas >> 6) & 0x7ul);
}

/**
 * \brief LPTimer0ʱGATEѡ
 *
 * \param[in] p_hw_gpio : ָGPIOĴָ
 * \param[in] src       : 趨ֵ
 *
 * \note ѡֵοûֲGPIO½ڵ
 *         --˿ڸܶʱſѡ(GPIO_TIMGS)
 *
 * \return 
 */
am_static_inline
void amhw_zsn700_gpio_sup_lptim0_gate_src_set (amhw_zsn700_gpio_t *p_hw_gpio,
                                               uint8_t             src)
{
    p_hw_gpio->gpio[0].gpio_sup_reg.timgs = (p_hw_gpio->gpio[0].gpio_sup_reg.timgs & (~(0x7ul << 12))) |
                                            ((src & 0x7ul) << 12);
}

/**
 * \brief LPTimer0ʱGATEȡ
 *
 * \param[in] p_hw_gpio : ָGPIOĴָ
 *
 * \note ѡֵοûֲGPIO½ڵ
 *       --˿ڸܶʱſѡ(GPIO_TIMGS)
 *
 * \return 
 */
am_static_inline
uint8_t amhw_zsn700_gpio_sup_lptim0_gate_src_get (amhw_zsn700_gpio_t *p_hw_gpio)
{
    return ((p_hw_gpio->gpio[0].gpio_sup_reg.timgs >> 12) & 0x7ul);
}

/**
 * \brief Timer3ʱGATEѡ
 *
 * \param[in] p_hw_gpio : ָGPIOĴָ
 * \param[in] src       : 趨ֵ
 *
 * \note ѡֵοûֲGPIO½ڵ
 *         --˿ڸܶʱſѡ(GPIO_TIMGS)
 *
 * \return 
 */
am_static_inline
void amhw_zsn700_gpio_sup_tim3_gate_src_set (amhw_zsn700_gpio_t *p_hw_gpio,
                                             uint8_t             src)
{
    p_hw_gpio->gpio[0].gpio_sup_reg.timgs = (p_hw_gpio->gpio[0].gpio_sup_reg.timgs & (~(0x7ul << 9))) |
                                            ((src & 0x7ul) << 9);
}

/**
 * \brief Timer3ʱGATEȡ
 *
 * \param[in] p_hw_gpio : ָGPIOĴָ
 *
 * \note ѡֵοûֲGPIO½ڵ
 *       --˿ڸܶʱſѡ(GPIO_TIMGS)
 *
 * \return 
 */
am_static_inline
uint8_t amhw_zsn700_gpio_sup_tim3_gate_src_get (amhw_zsn700_gpio_t *p_hw_gpio)
{
    return ((p_hw_gpio->gpio[0].gpio_sup_reg.timgs >> 9) & 0x7ul);
}

/**
 * \brief Timer2ʱGATEѡ
 *
 * \param[in] p_hw_gpio : ָGPIOĴָ
 * \param[in] src       : 趨ֵ
 *
 * \note ѡֵοûֲGPIO½ڵ
 *         --˿ڸܶʱſѡ(GPIO_TIMGS)
 *
 * \return 
 */
am_static_inline
void amhw_zsn700_gpio_sup_tim2_gate_src_set (amhw_zsn700_gpio_t *p_hw_gpio,
                                             uint8_t             src)
{
    p_hw_gpio->gpio[0].gpio_sup_reg.timgs = (p_hw_gpio->gpio[0].gpio_sup_reg.timgs & (~(0x7ul << 6))) |
                                            ((src & 0x7ul) << 6);
}

/**
 * \brief Timer3ʱGATEȡ
 *
 * \param[in] p_hw_gpio : ָGPIOĴָ
 *
 * \note ѡֵοûֲGPIO½ڵ
 *       --˿ڸܶʱſѡ(GPIO_TIMGS)
 *
 * \return 
 */
am_static_inline
uint8_t amhw_zsn700_gpio_sup_tim2_gate_src_get (amhw_zsn700_gpio_t *p_hw_gpio)
{
    return ((p_hw_gpio->gpio[0].gpio_sup_reg.timgs >> 6) & 0x7ul);
}

/**
 * \brief Timer1ʱGATEѡ
 *
 * \param[in] p_hw_gpio : ָGPIOĴָ
 * \param[in] src       : 趨ֵ
 *
 * \note ѡֵοûֲGPIO½ڵ
 *         --˿ڸܶʱſѡ(GPIO_TIMGS)
 *
 * \return 
 */
am_static_inline
void amhw_zsn700_gpio_sup_tim1_gate_src_set (amhw_zsn700_gpio_t *p_hw_gpio,
                                             uint8_t             src)
{
    p_hw_gpio->gpio[0].gpio_sup_reg.timgs = (p_hw_gpio->gpio[0].gpio_sup_reg.timgs & (~(0x7ul << 3))) |
                                            ((src & 0x7ul) << 3);
}

/**
 * \brief Timer1ʱGATEȡ
 *
 * \param[in] p_hw_gpio : ָGPIOĴָ
 *
 * \note ѡֵοûֲGPIO½ڵ
 *       --˿ڸܶʱſѡ(GPIO_TIMGS)
 *
 * \return 
 */
am_static_inline
uint8_t amhw_zsn700_gpio_sup_tim1_gate_src_get (amhw_zsn700_gpio_t *p_hw_gpio)
{
    return ((p_hw_gpio->gpio[0].gpio_sup_reg.timgs >> 3) & 0x7ul);
}

/**
 * \brief Timer0ʱGATEѡ
 *
 * \param[in] p_hw_gpio : ָGPIOĴָ
 * \param[in] src       : 趨ֵ
 *
 * \note ѡֵοûֲGPIO½ڵ
 *         --˿ڸܶʱſѡ(GPIO_TIMGS)
 *
 * \return 
 */
am_static_inline
void amhw_zsn700_gpio_sup_tim0_gate_src_set (amhw_zsn700_gpio_t *p_hw_gpio,
                                             uint8_t             src)
{
    p_hw_gpio->gpio[0].gpio_sup_reg.timgs = (p_hw_gpio->gpio[0].gpio_sup_reg.timgs & (~(0x7ul))) |
                                            (src & 0x7ul);
}

/**
 * \brief Timer0ʱGATEȡ
 *
 * \param[in] p_hw_gpio : ָGPIOĴָ
 *
 * \note ѡֵοûֲGPIO½ڵ
 *       --˿ڸܶʱſѡ(GPIO_TIMGS)
 *
 * \return 
 */
am_static_inline
uint8_t amhw_zsn700_gpio_sup_tim0_gate_src_get (amhw_zsn700_gpio_t *p_hw_gpio)
{
    return (p_hw_gpio->gpio[0].gpio_sup_reg.timgs & 0x7ul);
}

/**
 * \brief LPTimer1ʱETRѡ
 *
 * \param[in] p_hw_gpio : ָGPIOĴָ
 * \param[in] src       : 趨ֵ
 *
 * \note ѡֵοûֲGPIO½ڵ
 *         --˿ڸ PCA ѡ(GPIO_PCAS)
 *
 * \return 
 */
am_static_inline
void amhw_zsn700_gpio_sup_lptim1_etr_src_set (amhw_zsn700_gpio_t *p_hw_gpio,
                                              uint8_t             src)
{
    p_hw_gpio->gpio[0].gpio_sup_reg.pcas = (p_hw_gpio->gpio[0].gpio_sup_reg.pcas & (~(0x7ul << 9))) |
                                            ((src & 0x7ul) << 9);
}

/**
 * \brief LPTimer1ʱETRȡ
 *
 * \param[in] p_hw_gpio : ָGPIOĴָ
 *
 * \note ѡֵοûֲGPIO½ڵ
 *       --˿ڸ PCA ѡ(GPIO_PCAS)
 *
 * \return 
 */
am_static_inline
uint8_t amhw_zsn700_gpio_sup_lptim1_etr_src_get (amhw_zsn700_gpio_t *p_hw_gpio)
{
    return ((p_hw_gpio->gpio[0].gpio_sup_reg.pcas >> 9) & 0x7ul);
}

/**
 * \brief LPTimer0ʱETRѡ
 *
 * \param[in] p_hw_gpio : ָGPIOĴָ
 * \param[in] src       : 趨ֵ
 *
 * \note ѡֵοûֲGPIO½ڵ
 *         --˿ڸܶʱſѡ(GPIO_TIMGS)
 *
 * \return 
 */
am_static_inline
void amhw_zsn700_gpio_sup_lptim0_etr_src_set (amhw_zsn700_gpio_t *p_hw_gpio,
                                              uint8_t             src)
{
    p_hw_gpio->gpio[0].gpio_sup_reg.times = (p_hw_gpio->gpio[0].gpio_sup_reg.times & (~(0x7ul << 12))) |
                                            ((src & 0x7ul) << 12);
}

/**
 * \brief LPTimer0ʱETRȡ
 *
 * \param[in] p_hw_gpio : ָGPIOĴָ
 *
 * \note ѡֵοûֲGPIO½ڵ
 *       --˿ڸܶʱ ETRѡ(GPIO_TIMES)
 *
 * \return 
 */
am_static_inline
uint8_t amhw_zsn700_gpio_sup_lptim0_etr_src_get (amhw_zsn700_gpio_t *p_hw_gpio)
{
    return ((p_hw_gpio->gpio[0].gpio_sup_reg.times >> 12) & 0x7ul);
}

/**
 * \brief Timer3ʱETRѡ
 *
 * \param[in] p_hw_gpio : ָGPIOĴָ
 * \param[in] src       : 趨ֵ
 *
 * \note ѡֵοûֲGPIO½ڵ
 *         --˿ڸܶʱſѡ(GPIO_TIMGS)
 *
 * \return 
 */
am_static_inline
void amhw_zsn700_gpio_sup_tim3_etr_src_set (amhw_zsn700_gpio_t *p_hw_gpio,
                                            uint8_t             src)
{
    p_hw_gpio->gpio[0].gpio_sup_reg.times = (p_hw_gpio->gpio[0].gpio_sup_reg.times & (~(0x7ul << 9))) |
                                            ((src & 0x7ul) << 9);
}

/**
 * \brief Timer3ʱETRȡ
 *
 * \param[in] p_hw_gpio : ָGPIOĴָ
 *
 * \note ѡֵοûֲGPIO½ڵ
 *       --˿ڸܶʱ ETRѡ(GPIO_TIMES)
 *
 * \return 
 */
am_static_inline
uint8_t amhw_zsn700_gpio_sup_tim3_etr_src_get (amhw_zsn700_gpio_t *p_hw_gpio)
{
    return ((p_hw_gpio->gpio[0].gpio_sup_reg.times >> 9) & 0x7ul);
}

/**
 * \brief Timer2ʱETRѡ
 *
 * \param[in] p_hw_gpio : ָGPIOĴָ
 * \param[in] src       : 趨ֵ
 *
 * \note ѡֵοûֲGPIO½ڵ
 *         --˿ڸܶʱſѡ(GPIO_TIMGS)
 *
 * \return 
 */
am_static_inline
void amhw_zsn700_gpio_sup_tim2_etr_src_set (amhw_zsn700_gpio_t *p_hw_gpio,
                                            uint8_t             src)
{
    p_hw_gpio->gpio[0].gpio_sup_reg.times = (p_hw_gpio->gpio[0].gpio_sup_reg.times & (~(0x7ul << 6))) |
                                            ((src & 0x7ul) << 6);
}

/**
 * \brief Timer2ʱETRȡ
 *
 * \param[in] p_hw_gpio : ָGPIOĴָ
 *
 * \note ѡֵοûֲGPIO½ڵ
 *       --˿ڸܶʱ ETRѡ(GPIO_TIMES)
 *
 * \return 
 */
am_static_inline
uint8_t amhw_zsn700_gpio_sup_tim2_etr_src_get (amhw_zsn700_gpio_t *p_hw_gpio)
{
    return ((p_hw_gpio->gpio[0].gpio_sup_reg.times >> 6) & 0x7ul);
}

/**
 * \brief Timer1ʱETRѡ
 *
 * \param[in] p_hw_gpio : ָGPIOĴָ
 * \param[in] src       : 趨ֵ
 *
 * \note ѡֵοûֲGPIO½ڵ
 *         --˿ڸܶʱſѡ(GPIO_TIMGS)
 *
 * \return 
 */
am_static_inline
void amhw_zsn700_gpio_sup_tim1_etr_src_set (amhw_zsn700_gpio_t *p_hw_gpio,
                                            uint8_t             src)
{
    p_hw_gpio->gpio[0].gpio_sup_reg.times = (p_hw_gpio->gpio[0].gpio_sup_reg.times & (~(0x7ul << 3))) |
                                            ((src & 0x7ul) << 3);
}

/**
 * \brief Timer1ʱETRȡ
 *
 * \param[in] p_hw_gpio : ָGPIOĴָ
 *
 * \note ѡֵοûֲGPIO½ڵ
 *       --˿ڸܶʱ ETRѡ(GPIO_TIMES)
 *
 * \return 
 */
am_static_inline
uint8_t amhw_zsn700_gpio_sup_tim1_etr_src_get (amhw_zsn700_gpio_t *p_hw_gpio)
{
    return ((p_hw_gpio->gpio[0].gpio_sup_reg.times >> 3) & 0x7ul);
}

/**
 * \brief Timer0ʱETRѡ
 *
 * \param[in] p_hw_gpio : ָGPIOĴָ
 * \param[in] src       : 趨ֵ
 *
 * \note ѡֵοûֲGPIO½ڵ
 *         --˿ڸܶʱſѡ(GPIO_TIMGS)
 *
 * \return 
 */
am_static_inline
void amhw_zsn700_gpio_sup_tim0_etr_src_set (amhw_zsn700_gpio_t *p_hw_gpio,
                                            uint8_t             src)
{
    p_hw_gpio->gpio[0].gpio_sup_reg.times = (p_hw_gpio->gpio[0].gpio_sup_reg.times & (~(0x7ul << 0))) |
                                            ((src & 0x7ul) << 0);
}

/**
 * \brief Timer0ʱETRȡ
 *
 * \param[in] p_hw_gpio : ָGPIOĴָ
 *
 * \note ѡֵοûֲGPIO½ڵ
 *       --˿ڸܶʱ ETRѡ(GPIO_TIMES)
 *
 * \return 
 */
am_static_inline
uint8_t amhw_zsn700_gpio_sup_tim0_etr_src_get (amhw_zsn700_gpio_t *p_hw_gpio)
{
    return ((p_hw_gpio->gpio[0].gpio_sup_reg.times >> 0) & 0x7ul);
}

/**
 * \brief Timer3ʱCH0Bѡ
 *
 * \param[in] p_hw_gpio : ָGPIOĴָ
 * \param[in] src       : 趨ֵ
 *
 * \note ѡֵοûֲGPIO½ڵ
 *         --˿ڸܶʱѡ(GPIO_TIMCPS)
 *
 * \return 
 */
am_static_inline
void amhw_zsn700_gpio_sup_tim3_ch0b_src_set (amhw_zsn700_gpio_t *p_hw_gpio,
                                             uint8_t             src)
{
    p_hw_gpio->gpio[0].gpio_sup_reg.timcps = (p_hw_gpio->gpio[0].gpio_sup_reg.timcps & (~(0x7ul << 12))) |
                                             ((src & 0x7ul) << 12);
}

/**
 * \brief Timer3ʱCH0Bȡ
 *
 * \param[in] p_hw_gpio : ָGPIOĴָ
 *
 * \note ѡֵοûֲGPIO½ڵ
 *       --˿ڸܶʱѡ(GPIO_TIMCPS)
 *
 * \return 
 */
am_static_inline
uint8_t amhw_zsn700_gpio_sup_tim3_ch0b_src_get (amhw_zsn700_gpio_t *p_hw_gpio)
{
    return ((p_hw_gpio->gpio[0].gpio_sup_reg.timcps >> 12) & 0x7ul);
}

/**
 * \brief Timer3ʱCH0Aѡ
 *
 * \param[in] p_hw_gpio : ָGPIOĴָ
 * \param[in] src       : 趨ֵ
 *
 * \note ѡֵοûֲGPIO½ڵ
 *         --˿ڸܶʱѡ(GPIO_TIMCPS)
 *
 * \return 
 */
am_static_inline
void amhw_zsn700_gpio_sup_tim3_ch0a_src_set (amhw_zsn700_gpio_t *p_hw_gpio,
                                             uint8_t             src)
{
    p_hw_gpio->gpio[0].gpio_sup_reg.timcps = (p_hw_gpio->gpio[0].gpio_sup_reg.timcps & (~(0x7ul << 9))) |
                                             ((src & 0x7ul) << 9);
}

/**
 * \brief Timer3ʱCH0Aȡ
 *
 * \param[in] p_hw_gpio : ָGPIOĴָ
 *
 * \note ѡֵοûֲGPIO½ڵ
 *       --˿ڸܶʱѡ(GPIO_TIMCPS)
 *
 * \return 
 */
am_static_inline
uint8_t amhw_zsn700_gpio_sup_tim3_ch0a_src_get (amhw_zsn700_gpio_t *p_hw_gpio)
{
    return ((p_hw_gpio->gpio[0].gpio_sup_reg.timcps >> 9) & 0x7ul);
}

/**
 * \brief Timer2ʱCHAѡ
 *
 * \param[in] p_hw_gpio : ָGPIOĴָ
 * \param[in] src       : 趨ֵ
 *
 * \note ѡֵοûֲGPIO½ڵ
 *         --˿ڸܶʱѡ(GPIO_TIMCPS)
 *
 * \return 
 */
am_static_inline
void amhw_zsn700_gpio_sup_tim2_cha_src_set (amhw_zsn700_gpio_t *p_hw_gpio,
                                             uint8_t             src)
{
    p_hw_gpio->gpio[0].gpio_sup_reg.timcps = (p_hw_gpio->gpio[0].gpio_sup_reg.timcps & (~(0x7ul << 6))) |
                                             ((src & 0x7ul) << 6);
}

/**
 * \brief Timer2ʱCHAȡ
 *
 * \param[in] p_hw_gpio : ָGPIOĴָ
 *
 * \note ѡֵοûֲGPIO½ڵ
 *       --˿ڸܶʱѡ(GPIO_TIMCPS)
 *
 * \return 
 */
am_static_inline
uint8_t amhw_zsn700_gpio_sup_tim2_cha_src_get (amhw_zsn700_gpio_t *p_hw_gpio)
{
    return ((p_hw_gpio->gpio[0].gpio_sup_reg.timcps >> 6) & 0x7ul);
}

/**
 * \brief Timer1ʱCHAѡ
 *
 * \param[in] p_hw_gpio : ָGPIOĴָ
 * \param[in] src       : 趨ֵ
 *
 * \note ѡֵοûֲGPIO½ڵ
 *         --˿ڸܶʱѡ(GPIO_TIMCPS)
 *
 * \return 
 */
am_static_inline
void amhw_zsn700_gpio_sup_tim1_cha_src_set (amhw_zsn700_gpio_t *p_hw_gpio,
                                             uint8_t             src)
{
    p_hw_gpio->gpio[0].gpio_sup_reg.timcps = (p_hw_gpio->gpio[0].gpio_sup_reg.timcps & (~(0x7ul << 3))) |
                                             ((src & 0x7ul) << 3);
}

/**
 * \brief Timer1ʱCHAȡ
 *
 * \param[in] p_hw_gpio : ָGPIOĴָ
 *
 * \note ѡֵοûֲGPIO½ڵ
 *       --˿ڸܶʱѡ(GPIO_TIMCPS)
 *
 * \return 
 */
am_static_inline
uint8_t amhw_zsn700_gpio_sup_tim1_cha_src_get (amhw_zsn700_gpio_t *p_hw_gpio)
{
    return ((p_hw_gpio->gpio[0].gpio_sup_reg.timcps >> 3) & 0x7ul);
}

/**
 * \brief Timer0ʱCHAѡ
 *
 * \param[in] p_hw_gpio : ָGPIOĴָ
 * \param[in] src       : 趨ֵ
 *
 * \note ѡֵοûֲGPIO½ڵ
 *         --˿ڸܶʱѡ(GPIO_TIMCPS)
 *
 * \return 
 */
am_static_inline
void amhw_zsn700_gpio_sup_tim0_cha_src_set (amhw_zsn700_gpio_t *p_hw_gpio,
                                             uint8_t             src)
{
    p_hw_gpio->gpio[0].gpio_sup_reg.timcps = (p_hw_gpio->gpio[0].gpio_sup_reg.timcps & (~(0x7ul << 0))) |
                                             ((src & 0x7ul) << 0);
}

/**
 * \brief Timer0ʱCHAȡ
 *
 * \param[in] p_hw_gpio : ָGPIOĴָ
 *
 * \note ѡֵοûֲGPIO½ڵ
 *       --˿ڸܶʱѡ(GPIO_TIMCPS)
 *
 * \return 
 */
am_static_inline
uint8_t amhw_zsn700_gpio_sup_tim0_cha_src_get (amhw_zsn700_gpio_t *p_hw_gpio)
{
    return ((p_hw_gpio->gpio[0].gpio_sup_reg.timcps >> 0) & 0x7ul);
}

/**
 * \brief PCA ECIʱѡ
 *
 * \param[in] p_hw_gpio : ָGPIOĴָ
 * \param[in] src       : 趨ֵ
 *
 * \note ѡֵοûֲGPIO½ڵ
 *         --˿ڸ PCA ѡ(GPIO_PCAS)
 *
 * \return 
 */
am_static_inline
void amhw_zsn700_gpio_sup_pca_eci_src_set (amhw_zsn700_gpio_t *p_hw_gpio,
                                           uint8_t             src)
{
    p_hw_gpio->gpio[0].gpio_sup_reg.pcas = (p_hw_gpio->gpio[0].gpio_sup_reg.pcas & (~(0x7ul << 3))) |
                                           ((src & 0x7ul) << 3);
}

/**
 * \brief PCA ECIʱȡ
 *
 * \param[in] p_hw_gpio : ָGPIOĴָ
 *
 * \note ѡֵοûֲGPIO½ڵ
 *       --˿ڸ PCA ѡ(GPIO_PCAS)
 *
 * \return 
 */
am_static_inline
uint8_t amhw_zsn700_gpio_sup_pca_eci_src_get (amhw_zsn700_gpio_t *p_hw_gpio)
{
    return ((p_hw_gpio->gpio[0].gpio_sup_reg.pcas >> 3) & 0x7ul);
}

/**
 * \brief PCA CH0 ѡ
 *
 * \param[in] p_hw_gpio : ָGPIOĴָ
 * \param[in] src       : 趨ֵ
 *
 * \note ѡֵοûֲGPIO½ڵ
 *         --˿ڸ PCA ѡ(GPIO_PCAS)
 *
 * \return 
 */
am_static_inline
void amhw_zsn700_gpio_sup_pca_ch0_src_set (amhw_zsn700_gpio_t *p_hw_gpio,
                                           uint8_t             src)
{
    p_hw_gpio->gpio[0].gpio_sup_reg.pcas = (p_hw_gpio->gpio[0].gpio_sup_reg.pcas & (~(0x7ul << 0))) |
                                           ((src & 0x7ul) << 0);
}

/**
 * \brief PCA CH0 ȡ
 *
 * \param[in] p_hw_gpio : ָGPIOĴָ
 *
 * \note ѡֵοûֲGPIO½ڵ
 *       --˿ڸ PCA ѡ(GPIO_PCAS)
 *
 * \return 
 */
am_static_inline
uint8_t amhw_zsn700_gpio_sup_pca_ch0_src_get (amhw_zsn700_gpio_t *p_hw_gpio)
{
    return ((p_hw_gpio->gpio[0].gpio_sup_reg.pcas >> 0) & 0x7ul);
}

/**
 * \brief PCNTѡ
 *        ö
 */
typedef enum {
    AMHW_ZSN700_GPIO_SUP_PCNT_NO = 0,
    AMHW_ZSN700_GPIO_SUP_PCNT_VC0_OUT,
    AMHW_ZSN700_GPIO_SUP_PCNT_VC1_OUT,
    AMHW_ZSN700_GPIO_SUP_PCNT_VC2_OUT,
}amhw_zsn700_gpio_sup_pcnt_in_src_t;

/**
 * \brief PCNT_S1ѡ
 *
 * \param[in] p_hw_gpio : ָGPIOĴָ
 * \param[in] src       : 趨ֵ   amhw_zsn700_gpio_sup_pcnt_in_src_tö
 *
 * \return 
 */
am_static_inline
void amhw_zsn700_gpio_sup_pcnt_s1_src_set (amhw_zsn700_gpio_t                *p_hw_gpio,
                                           amhw_zsn700_gpio_sup_pcnt_in_src_t src)
{
    p_hw_gpio->gpio[0].gpio_sup_reg.pcnt = (p_hw_gpio->gpio[0].gpio_sup_reg.pcnt & (~(0x3ul << 2))) |
                                           ((src & 0x3ul) << 2);
}

/**
 * \brief PCNT_S1ȡ
 *
 * \param[in] p_hw_gpio : ָGPIOĴָ
 *
 * \return amhw_zsn700_gpio_sup_pcnt_in_src_tö
 */
am_static_inline
amhw_zsn700_gpio_sup_pcnt_in_src_t amhw_zsn700_gpio_sup_pcnt_s1_src_get (amhw_zsn700_gpio_t *p_hw_gpio)
{
    return (amhw_zsn700_gpio_sup_pcnt_in_src_t)((p_hw_gpio->gpio[0].gpio_sup_reg.pcnt >> 2) & 0x3ul);
}

/**
 * \brief PCNT_S0ѡ
 *
 * \param[in] p_hw_gpio : ָGPIOĴָ
 * \param[in] src       : 趨ֵ   amhw_zsn700_gpio_sup_pcnt_in_src_tö
 *
 * \return 
 */
am_static_inline
void amhw_zsn700_gpio_sup_pcnt_s0_src_set (amhw_zsn700_gpio_t                *p_hw_gpio,
                                           amhw_zsn700_gpio_sup_pcnt_in_src_t src)
{
    p_hw_gpio->gpio[0].gpio_sup_reg.pcnt = (p_hw_gpio->gpio[0].gpio_sup_reg.pcnt & (~(0x3ul << 0))) |
                                           ((src & 0x3ul) << 0);
}

/**
 * \brief PCNT_S0ȡ
 *
 * \param[in] p_hw_gpio : ָGPIOĴָ
 *
 * \return amhw_zsn700_gpio_sup_pcnt_in_src_tö
 */
am_static_inline
amhw_zsn700_gpio_sup_pcnt_in_src_t amhw_zsn700_gpio_sup_pcnt_s0_src_get (amhw_zsn700_gpio_t *p_hw_gpio)
{
    return (amhw_zsn700_gpio_sup_pcnt_in_src_t)((p_hw_gpio->gpio[0].gpio_sup_reg.pcnt >> 0) & 0x3ul);
}

typedef enum amhw_zsn700_gpiomode
{
    AMHW_ZSN700_GPIO_MODE_IPU = 0,         /**< \brief ģʽ  */
    AMHW_ZSN700_GPIO_MODE_IPD = 1,         /**< \brief ģʽ */
    AMHW_ZSN700_GPIO_MODE_AIN = 2,         /**< \brief ģģʽ */
    AMHW_ZSN700_GPIO_MODE_IN_FLOATING = 3, /**< \brief ģʽ */
    AMHW_ZSN700_GPIO_MODE_OUT_PP = 4,      /**< \brief ģʽ */
    AMHW_ZSN700_GPIO_MODE_OUT_OD = 5,      /**< \brief ©ģʽ */
    AMHW_ZSN700_GPIO_MODE_AOUT = 6         /**< \brief ģģʽ */
}amhw_zsn700_gpiomode_t;

/**
 * \brief GPIOŵģʽ
 *
 * \param[in] p_hw_gpio : ָGPIOĴָ
 * \param[in] gpio_mode : ŵģʽֵΪ amhw_zlg217_gpiomode_t ö
 * \param[in] pin       : űţֵΪ PIO* (#PIOA_0)
 *
 * \return 
 */
void amhw_zsn700_gpio_pin_mode_set (amhw_zsn700_gpio_t     *p_hw_gpio,
                                    amhw_zsn700_gpiomode_t  gpio_mode,
                                    int                     pin);


/**
 * @} amhw_zsn700_if_gpio
 */

#ifdef __cplusplus
}
#endif

#endif /* __AMHW_ZSN700_GPIO_H */

/* end of file */
