/*******************************************************************************
*                                 AMetal
*                       ----------------------------
*                       innovating embedded platform
*
* Copyright (c) 2001-2018 Guangzhou ZHIYUAN Electronics Co., Ltd.
* All rights reserved.
*
* Contact information:
* web site:    http://www.zlg.cn/
*******************************************************************************/

/**
 * \file
 * \brief I2CI2C׼ӿ
 *
 * \internal
 * \par Modification History
 * - 1.01 17-12-28  pea, add a timeout mechanism, enable TX_ABRT and STOP_DET
 *                  interrupt
 * - 1.00 17-04-24  sdy, first implementation
 * \endinternal
 */

#ifndef __AM_ZLG_I2C_H
#define __AM_ZLG_I2C_H

#ifdef __cplusplus
extern "C" {
#endif

#include "ametal.h"
#include "am_int.h"
#include "am_i2c.h"
#include "am_list.h"
#include "am_softimer.h"

#include "am_vdebug.h"

/**
 * \addtogroup am_zlg_if_i2c
 * \copydoc am_zsl42x_i2c.h
 * @{
 */

/**
 * \brief I2C豸Ϣṹ
 */
typedef struct am_zsl42x_i2c_devinfo {

    /** \brief I2CĴַ */
    uint32_t  i2c_regbase;

    /** \brief ʱID */
    int       clk_num;

    /** \brief I2CжϺ */
    uint16_t  inum;

    /** \brief I2C */
    uint32_t  speed;

    /**
     * \brief ʱʱֵ
     *
     * I2Cݴڼ䣬˶ʱʱκӦ,Ϊʱ
     * ǰ䲢λڲ߼
     */
    uint32_t  timeout_ms;

    /** \brief I2C߻ָ */
    void    (*pfn_bus_clean)(void);

    /** \brief ƽ̨ʼ */
    void    (*pfn_plfm_init)(void);

    /** \brief ƽ̨ʼ */
    void    (*pfn_plfm_deinit)(void);

} am_zsl42x_i2c_devinfo_t;

/**
 * \brief I2C豸ṹ
 */
typedef struct am_zsl42x_i2c_dev {

    /** \brief ׼I2C */
    am_i2c_serv_t                           i2c_serv;

    /** \brief I2CϢ */
    struct am_list_head                     msg_list;

    /** \brief ָI2Cṹָ,ͬһʱֻܴһ */
    am_i2c_transfer_t                      *p_cur_trans;

    /** \brief ǰڴϢ */
    am_i2c_message_t                       *p_cur_msg;

    /** \brief ǷɱϢ */
    volatile am_bool_t                      is_complete;

    /** \brief ݽ/ͼ */
    volatile uint32_t                       data_ptr;

    /** \brief ǰ״̬0˴δ˴ӻַ1˴ûдӻַ */
    volatile uint8_t                        trans_state;

    /** \brief æʶ */
    volatile am_bool_t                      busy;

    /** \brief ״̬ */
    volatile uint8_t                        state;

    /** \brief ӻַ */
    volatile uint16_t                       slave_addr;

    /** \brief ʱʱ */
    am_softimer_t                           softimer;

    /** \brief ʱʱ״̬ */
    volatile am_bool_t                      softimer_state;

    /** \brief Ƿж״̬ */
    volatile am_bool_t                      is_abort;

    /** \brief ָI2C豸Ϣָ */
    const am_zsl42x_i2c_devinfo_t             *p_devinfo;

} am_zsl42x_i2c_dev_t;

/**
 * \brief I2Cʼ
 *
 * \param[in] p_dev     : ָI2C豸ṹָ
 * \param[in] p_devinfo : ָI2C豸Ϣṹָ
 *
 * \return I2C׼
 */
am_i2c_handle_t am_zsl42x_i2c_init (am_zsl42x_i2c_dev_t           *p_dev,
                                 const am_zsl42x_i2c_devinfo_t *p_devinfo);

/**
 * \brief I2Cʼ
 *
 * \param[in] handle : 豸I2C׼
 *
 * \return 
 */
void am_zsl42x_i2c_deinit (am_i2c_handle_t handle);

/** @} */

#ifdef __cplusplus
}
#endif

#endif /* __AM_ZLG_I2C_H */

/* end of file */
