/*******************************************************************************
*                                 AMetal
*                       ----------------------------
*                       innovating embedded platform
*
* Copyright (c) 2001-2018 Guangzhou ZHIYUAN Electronics Co., Ltd.
* All rights reserved.
*
* Contact information:
* web site:    http://www.zsn700.cn/
*******************************************************************************/

/**
 * \file
 * \brief LVD ײӿ
 * \internal
 * \par Modification History
 * - 1.00 20-03-18
 * \endinternal
 */

#ifndef __AMHW_ZSN700_LVD_H
#define __AMHW_ZSN700_LVD_H

#ifdef __cplusplus
extern "C" {
#endif

#include "ametal.h"
#include "am_types.h"

/**
 * \addtogroup amhw_zsn700_if_lvd
 * \copydoc amhw_zsn700_lvd.h
 * @{
 */

/**
 * \brief LVD - Ĵ
 */
typedef struct amhw_zsn700_lvd {
         uint8_t   res[40];      /**< \brief   */
    __IO uint32_t  lvccr;        /**< \brief  LVD üĴ */
    __IO uint32_t  lvcifr;       /**< \brief  LVD жϼĴ */
} amhw_zsn700_lvd_t;

/**
 * \brief LVD жʹ
 *
 * \param[in] p_hw_lvd : ָLVDĴַָ
 *
 * \retval : 
 */
am_static_inline
void amhw_zsn700_lvd_int_enable (amhw_zsn700_lvd_t *p_hw_lvd)
{
    p_hw_lvd->lvccr |= (1 << 15);
}

/**
 * \brief LVD жϽ
 *
 * \param[in] p_hw_lvd : ָLVDĴַָ
 *
 * \retval : 
 */
am_static_inline
void amhw_zsn700_lvd_int_disable (amhw_zsn700_lvd_t *p_hw_lvd)
{
    p_hw_lvd->lvccr &= ~(1 << 15);
}

/*
 * \brief LVD ʹ
 */
#define AMHW_ZSN700_LVD_HTEN    (1U << 14)    /**< \brief ߵƽʹ */
#define AMHW_ZSN700_LVD_RTEN    (1U << 13)    /**< \brief شʹ */
#define AMHW_ZSN700_LVD_FTEN    (1U << 12)    /**< \brief ½شʹ */

/**
 * \brief LVD ʹ
 *
 * \param[in] p_hw_lvd : ָLVDĴַָ
 * \param[in] flag    : ο궨壺LVD ʹ
 *
 * \retval : 
 */
am_static_inline
void amhw_zsn700_lvd_tri_enable (amhw_zsn700_lvd_t *p_hw_lvd, uint32_t flag)
{
    switch (flag){

        case AMHW_ZSN700_LVD_FTEN : p_hw_lvd->lvccr |= flag;
                                    p_hw_lvd->lvccr &= ~AMHW_ZSN700_LVD_RTEN;
                                    p_hw_lvd->lvccr &= ~AMHW_ZSN700_LVD_HTEN;
            break;
        case AMHW_ZSN700_LVD_RTEN : p_hw_lvd->lvccr |= flag;
                                    p_hw_lvd->lvccr &= ~AMHW_ZSN700_LVD_FTEN;
                                    p_hw_lvd->lvccr &= ~AMHW_ZSN700_LVD_HTEN;
            break;
        case AMHW_ZSN700_LVD_HTEN : p_hw_lvd->lvccr |= flag;
                                    p_hw_lvd->lvccr &= ~AMHW_ZSN700_LVD_FTEN;
                                    p_hw_lvd->lvccr &= ~AMHW_ZSN700_LVD_RTEN;
            break;
        case AMHW_ZSN700_LVD_FTEN | AMHW_ZSN700_LVD_RTEN :
            p_hw_lvd->lvccr |= AMHW_ZSN700_LVD_FTEN;
            p_hw_lvd->lvccr |= AMHW_ZSN700_LVD_RTEN;
            p_hw_lvd->lvccr &= ~AMHW_ZSN700_LVD_HTEN;
            break;
        case AMHW_ZSN700_LVD_FTEN | AMHW_ZSN700_LVD_HTEN :
            p_hw_lvd->lvccr |= AMHW_ZSN700_LVD_FTEN;
            p_hw_lvd->lvccr |= AMHW_ZSN700_LVD_HTEN;
            p_hw_lvd->lvccr &= ~AMHW_ZSN700_LVD_RTEN;
            break;
        case AMHW_ZSN700_LVD_HTEN | AMHW_ZSN700_LVD_RTEN :
            p_hw_lvd->lvccr |= AMHW_ZSN700_LVD_HTEN;
            p_hw_lvd->lvccr |= AMHW_ZSN700_LVD_RTEN;
            p_hw_lvd->lvccr &= ~AMHW_ZSN700_LVD_FTEN;
            break;
        case AMHW_ZSN700_LVD_HTEN | AMHW_ZSN700_LVD_RTEN | AMHW_ZSN700_LVD_FTEN:
            p_hw_lvd->lvccr |= AMHW_ZSN700_LVD_HTEN;
            p_hw_lvd->lvccr |= AMHW_ZSN700_LVD_RTEN;
            p_hw_lvd->lvccr |= AMHW_ZSN700_LVD_FTEN;
            break;
        default:
            p_hw_lvd->lvccr &= ~AMHW_ZSN700_LVD_HTEN;
            p_hw_lvd->lvccr &= ~AMHW_ZSN700_LVD_RTEN;
            p_hw_lvd->lvccr &= ~AMHW_ZSN700_LVD_FTEN;
    }
}

/* * \brief LVD 
 */
#define AMHW_ZSN700_LVD_HTDIS    (1U << 14)    /**< \brief ߵƽ */
#define AMHW_ZSN700_LVD_RTDIS    (1U << 13)    /**< \brief ش */
#define AMHW_ZSN700_LVD_FTDIS    (1U << 12)    /**< \brief ½ش */

/**
 * \brief LVD 
 *
 * \param[in] p_hw_lvd : ָLVDĴַָ
 * \param[in] flag    : ο궨壺LVD 
 *
 * \retval : 
 */
am_static_inline
void amhw_zsn700_lvd_tri_disable (amhw_zsn700_lvd_t *p_hw_lvd, uint32_t flag)
{
    p_hw_lvd->lvccr &= ~flag;
}

/**
 * \brief LVD ˲ʱ
 */
typedef enum amhw_zsn700_lvd_deb_time {
    AMHW_ZSN700_LVD_DEB_TIME_7_US = 0,
    AMHW_ZSN700_LVD_DEB_TIME_14_US,
    AMHW_ZSN700_LVD_DEB_TIME_28_US,
    AMHW_ZSN700_LVD_DEB_TIME_112_US,
    AMHW_ZSN700_LVD_DEB_TIME_450_US,
    AMHW_ZSN700_LVD_DEB_TIME_1P8_MS,
    AMHW_ZSN700_LVD_DEB_TIME_7P2_MS,
    AMHW_ZSN700_LVD_DEB_TIME_28P8_MS,
}amhw_zsn700_lvd_deb_time_t;

/**
 * \brief LVD ˲ʱѡ
 *
 * \param[in] p_hw_lvd : ָLVDĴַָ
 * \param[in] flag    : οö٣LVD ˲ʱ
 *
 * \retval : 
 */
am_static_inline
void amhw_zsn700_lvd_deb_time_sel (amhw_zsn700_lvd_t *p_hw_lvd, uint32_t flag)
{
    p_hw_lvd->lvccr = (p_hw_lvd->lvccr & (~(0x7 << 9))) | (flag << 9);
}

/**
 * \brief LVD ˲ʹ
 *
 * \param[in] p_hw_lvd : ָLVDĴַָ
 *
 * \retval : 
 */
am_static_inline
void amhw_zsn700_lvd_deb_time_enable (amhw_zsn700_lvd_t *p_hw_lvd)
{
    p_hw_lvd->lvccr |= (1 << 8);
}

/**
 * \brief LVD ˲
 *
 * \param[in] p_hw_lvd : ָLVDĴַָ
 *
 * \retval : 
 */
am_static_inline
void amhw_zsn700_lvd_deb_time_disable (amhw_zsn700_lvd_t *p_hw_lvd)
{
    p_hw_lvd->lvccr &= ~(1 << 8);
}

/**
 * \brief LVD ֵѹ
 */
typedef enum amhw_zsn700_lvd_vids {
    AMHW_ZSN700_LVD_VIDS_1P8_V = 0,
    AMHW_ZSN700_LVD_VIDS_1P9_V,
    AMHW_ZSN700_LVD_VIDS_2P0_V,
    AMHW_ZSN700_LVD_VIDS_2P1_V,
    AMHW_ZSN700_LVD_VIDS_2P2_V,
    AMHW_ZSN700_LVD_VIDS_2P3_V,
    AMHW_ZSN700_LVD_VIDS_2P4_V,
    AMHW_ZSN700_LVD_VIDS_2P5_V,
    AMHW_ZSN700_LVD_VIDS_2P6_V,
    AMHW_ZSN700_LVD_VIDS_2P7_V,
    AMHW_ZSN700_LVD_VIDS_2P8_V,
    AMHW_ZSN700_LVD_VIDS_2P9_V,
    AMHW_ZSN700_LVD_VIDS_3P0_V,
    AMHW_ZSN700_LVD_VIDS_3P1_V,
    AMHW_ZSN700_LVD_VIDS_3P2_V,
    AMHW_ZSN700_LVD_VIDS_3P3_V,
}amhw_zsn700_lvd_vids_t;

/**
 * \brief LVD ֵѹѡ
 *
 * \param[in] p_hw_lvd : ָLVDĴַָ
 * \param[in] flag    : οö٣LVD ֵѹ
 *
 * \retval : 
 */
am_static_inline
void amhw_zsn700_lvd_vids_sel (amhw_zsn700_lvd_t *p_hw_lvd, uint32_t flag)
{
    p_hw_lvd->lvccr = (p_hw_lvd->lvccr & (~(0xf << 4))) | (flag << 4);
}

/*
 * \brief LVD Դѡ
 */
#define ANHW_ZSN700_LVD_SRC_PB07 (3U) /**< \brief PB07˿ѹ */
#define ANHW_ZSN700_LVD_SRC_PB08 (2U) /**< \brief PB08˿ѹ */
#define ANHW_ZSN700_LVD_SRC_PC13 (1U) /**< \brief PC13˿ѹ */
#define ANHW_ZSN700_LVD_SRC_AVCC (0U) /**< \brief AVCCԴѹ*/

/**
 * \brief LVD Դѡ
 *
 * \param[in] p_hw_lvd : ָLVDĴַָ
 * \param[in] flag    : ο궨壺LVD Դѡ
 *
 * \retval : 
 */
am_static_inline
void amhw_zsn700_lvd_src_sel (amhw_zsn700_lvd_t *p_hw_lvd, uint32_t flag)
{
    p_hw_lvd->lvccr = (p_hw_lvd->lvccr & (~(0x3 << 2))) | (flag << 2);
}

/*
 * \brief LVD ѡ
 */
#define ANHW_ZSN700_LVD_TRI_ACT_SYSTEM_RESET  (1U)   /**< \brief ϵͳλ */
#define ANHW_ZSN700_LVD_TRI_ACT_NVIC_INT      (0U)   /**< \brief NVICж */

/**
 * \brief LVD ѡ
 *
 * \param[in] p_hw_lvd : ָLVDĴַָ
 * \param[in] flag    : ο궨壺LVD ѡ
 *
 * \retval : 
 */
am_static_inline
void amhw_zsn700_lvd_tri_act_sel (amhw_zsn700_lvd_t *p_hw_lvd, uint32_t flag)
{
    p_hw_lvd->lvccr = (p_hw_lvd->lvccr & (~(0x1 << 1))) | (flag << 1);
}

/**
 * \brief LVD ʹ
 *
 * \param[in] p_hw_lvd : ָLVDĴַָ
 *
 * \retval : 
 */
am_static_inline
void amhw_zsn700_lvd_enable (amhw_zsn700_lvd_t *p_hw_lvd)
{
    p_hw_lvd->lvccr |= (1 << 0);
}

/**
 * \brief LVD 
 *
 * \param[in] p_hw_lvd : ָLVDĴַָ
 *
 * \retval : 
 */
am_static_inline
void amhw_zsn700_lvd_disable (amhw_zsn700_lvd_t *p_hw_lvd)
{
    p_hw_lvd->lvccr &= ~(1 << 0);
}

/**
 * \brief LVD crĴ
 *
 * \param[in] p_hw_lvd : ָLVDĴַָ
 *
 * \retval : 
 */
am_static_inline
void amhw_zsn700_lvd_cr_clr (amhw_zsn700_lvd_t *p_hw_lvd)
{
    p_hw_lvd->lvccr = 0;
}

/**
 * \brief LVD ˲״̬ȡ
 *
 * \param[in] p_hw_lvd : ָLVDĴַָ
 *
 * \retval : LVD ˲״̬
 */
am_static_inline
int amhw_zsn700_flt_status_check (amhw_zsn700_lvd_t *p_hw_lvd)
{
    return (p_hw_lvd->lvcifr & (1 << 1));
}

/**
 * \brief LVD ж״̬ȡ
 *
 * \param[in] p_hw_lvd : ָLVDĴַָ
 *
 * \retval : LVD ж״̬
 */
am_static_inline
int amhw_zsn700_int_status_check (amhw_zsn700_lvd_t *p_hw_lvd)
{
    return (p_hw_lvd->lvcifr & 0x1);
}

/**
 * \brief ж״̬־
 *
 * \param[in] p_hw_lvd : ָLVDĴַָ
 *
 * \retval : 
 */
am_static_inline
void amhw_zsn700_int_falg_clr (amhw_zsn700_lvd_t *p_hw_lvd)
{
    p_hw_lvd->lvcifr &= ~(1 << 0);
}

/**
 * @} amhw_if_zsn700_lvd
 */

#ifdef __cplusplus
}
#endif

#endif /* __AMHW_ZSN700_LVD_H */

/* end of file */
