/*******************************************************************************
*                                 AMetal
*                       ----------------------------
*                       innovating embedded platform
*
* Copyright (c) 2001-2018 Guangzhou ZHIYUAN Electronics Co., Ltd.
* All rights reserved.
*
* Contact information:
* web site:    http://www.zsn700.cn/
*******************************************************************************/

/**
 * \file
 * \brief OPA ײӿ
 * \internal
 * \par Modification History
 * - 1.00 20-03-14
 * \endinternal
 */

#ifndef __AMHW_ZSN700_OPA_H
#define __AMHW_ZSN700_OPA_H

#ifdef __cplusplus
extern "C" {
#endif

#include "ametal.h"
#include "am_types.h"

/**
 * \addtogroup amhw_zsn700_if_opa
 * \copydoc amhw_zsn700_opa.h
 * @{
 */

/**
 * \brief OPA - Ĵ
 */
typedef struct amhw_zsn700_opa {
         uint32_t  res1[12];     /**< \brief   */
    __IO uint32_t  opacr0;       /**< \brief  OPA üĴ0 */
         uint32_t  res2[2];      /**< \brief   */
    __IO uint32_t  opacr1;       /**< \brief  OPA üĴ1 */
} amhw_zsn700_opa_t;

/**
 * \brief OPA ʹ
 */
#define  AMHW_ZSN700_OPA_OEN1   (1U << 3U)
#define  AMHW_ZSN700_OPA_OEN2   (1U << 4U)
#define  AMHW_ZSN700_OPA_OEN3   (1U << 5U)
#define  AMHW_ZSN700_OPA_OEN4   (1U << 6U)

/**
 * \brief OPA ʹ
 *
 * \param[in] p_hw_opa : ָOPAĴַָ
 * \param[in] flag     : xʹ룬ο궨壺OPA ʹ
 *
 * \retval : 
 */
am_static_inline
void amhw_zsn700_opa_out_en (amhw_zsn700_opa_t *p_hw_opa, uint32_t flag)
{
    p_hw_opa->opacr0 |= flag;
}

/**
 * \brief OPA 
 */
#define  AMHW_ZSN700_OPA_ODIS1   ~(1U << 3U)
#define  AMHW_ZSN700_OPA_ODIS2   ~(1U << 4U)
#define  AMHW_ZSN700_OPA_ODIS3   ~(1U << 5U)
#define  AMHW_ZSN700_OPA_ODIS4   ~(1U << 6U)

/**
 * \brief OPA 
 *
 * \param[in] p_hw_opa : ָOPAĴַָ
 * \param[in] flag     : x룬ο궨壺OPA 
 *
 * \retval : 
 */
am_static_inline
void amhw_zsn700_opa_out_dis (amhw_zsn700_opa_t *p_hw_opa, uint32_t flag)
{
    p_hw_opa->opacr0 &= flag;
}

/**
 * \brief OPA DACʹOP3λӻʹ
 *
 * \param[in] p_hw_opa : ָOPAĴַָ
 *
 * \note : OPAʹܲͬʱʹ
 *
 * \retval : 
 */
am_static_inline
void amhw_zsn700_opabuf_en (amhw_zsn700_opa_t *p_hw_opa)
{
    p_hw_opa->opacr0 |= (1 << 2);
    p_hw_opa->opacr0 &= ~(1 << 0);
}

/**
 * \brief OPA DACʹOP3λӻ
 *
 * \param[in] p_hw_opa : ָOPAĴַָ
 *
 * \retval : 
 */
am_static_inline
void amhw_zsn700_opabuf_dis (amhw_zsn700_opa_t *p_hw_opa)
{
    p_hw_opa->opacr0 &= ~(1 << 2);
}

/**
 * \brief OPA ԶУʹ
 *
 * \param[in] p_hw_opa : ָOPAĴַָ
 *
 * \retval : 
 */
am_static_inline
void amhw_zsn700_opa_az_en (amhw_zsn700_opa_t *p_hw_opa)
{
    p_hw_opa->opacr0 |= (1 << 1);
}

/**
 * \brief OPA ԶУ
 *
 * \param[in] p_hw_opa : ָOPAĴַָ
 *
 * \retval : 
 */
am_static_inline
void amhw_zsn700_opa_az_dis (amhw_zsn700_opa_t *p_hw_opa)
{
    p_hw_opa->opacr0 &= ~(1 << 1);
}

/**
 * \brief OPA ʹ
 *
 * \param[in] p_hw_opa : ָOPAĴַָ
 *
 * \note : OPABUFʹܲͬʱʹ
 *
 * \retval : 
 */
am_static_inline
void amhw_zsn700_opa_en (amhw_zsn700_opa_t *p_hw_opa)
{
    p_hw_opa->opacr0 |= (1 << 0);
    p_hw_opa->opacr0 &= ~(1 << 2);
}

/**
 * \brief OPA 
 *
 * \param[in] p_hw_opa : ָOPAĴַָ
 *
 * \retval : 
 */
am_static_inline
void amhw_zsn700_opa_dis (amhw_zsn700_opa_t *p_hw_opa)
{
    p_hw_opa->opacr0 &= ~(1 << 0);
}

/**
 * \brief ԶУ׼
 */
typedef enum amhw_zsn700_opa_clk_sel {
    AMHW_ZSN700_OPA_CLK_1 = 0,
    AMHW_ZSN700_OPA_CLK_2,
    AMHW_ZSN700_OPA_CLK_4,
    AMHW_ZSN700_OPA_CLK_8,
    AMHW_ZSN700_OPA_CLK_16,
    AMHW_ZSN700_OPA_CLK_32,
    AMHW_ZSN700_OPA_CLK_64,
    AMHW_ZSN700_OPA_CLK_128,
    AMHW_ZSN700_OPA_CLK_256,
    AMHW_ZSN700_OPA_CLK_512,
    AMHW_ZSN700_OPA_CLK_1024,
    AMHW_ZSN700_OPA_CLK_2048,
    AMHW_ZSN700_OPA_CLK_4096,
}amhw_zsn700_opa_clk_sel_t;

/**
 * \brief OPA ԶУ׼
 *
 * \param[in] p_hw_opa : ָOPAĴַָ
 * \param[in]  flag    : У׼ʱڣοöٶ壺ԶУ׼
 *
 * \retval : 
 */
am_static_inline
void amhw_zsn700_opa_clk_sel (amhw_zsn700_opa_t *p_hw_opa, uint32_t flag)
{
    p_hw_opa->opacr1 = (p_hw_opa->opacr1 & (~(0xf << 4))) | (flag << 4);
}

/**
 * \brief OPA ԶУѡ
 */
#define  AMHW_ZSN700_OPA_AZ_SW      1U /**< \brief У׼ */
#define  AMHW_ZSN700_OPA_AZ_SW_TRI  2U /**< \brief У׼ */
#define  AMHW_ZSN700_OPA_AZ_ADC     3U /**< \brief ADCУ׼ */

/**
 * \brief OPA ԶУѡ
 *
 * \param[in] p_hw_opa : ָOPAĴַָ
 * \param[in]  flag    : ԶУ㷽ʽο궨壺OPA ԶУѡ
 *
 * \retval : 
 */
am_static_inline
void amhw_zsn700_opa_az_way_sel (amhw_zsn700_opa_t *p_hw_opa, uint32_t flag)
{
    if (flag == AMHW_ZSN700_OPA_AZ_ADC){
        p_hw_opa->opacr1 |= (1 << 0);
    }else if (flag == AMHW_ZSN700_OPA_AZ_SW){
        p_hw_opa->opacr1 |= (1 << 3);
    }else {
        p_hw_opa->opacr1 &= ~(1 << 3);
    }
}

/**
 * \brief OPA У׼
 */
#define AMHW_ZSN700_AZ_SW_START        (1U << 2U)  /**< \brief У׼ */
#define AMHW_ZSN700_AZ_SW_TRI_START    (1U << 1U)  /**< \brief У׼ */

/**
 * \brief OPA У׼
 *
 * \param[in] p_hw_opa : ָOPAĴַָ
 * \param[in]  flag    : ԶУ㷽ʽο궨壺OPA У׼
 *
 * \retval : 
 */
am_static_inline
void amhw_zsn700_az_start (amhw_zsn700_opa_t *p_hw_opa, uint32_t flag)
{
    p_hw_opa->opacr1 |= flag;
}

/**
 * \brief OPA У׼ֹͣ
 *
 * \param[in] p_hw_opa : ָOPAĴַָ
 *
 * \retval : 
 */
am_static_inline
void amhw_zsn700_az_stop (amhw_zsn700_opa_t *p_hw_opa)
{
    p_hw_opa->opacr1 &= ~(1 << 1);
}


/**
 * @} amhw_if_zsn700_opa
 */

#ifdef __cplusplus
}
#endif

#endif /* __AMHW_ZSN700_OPA_H */

/* end of file */
