/*******************************************************************************
*                                 AMetal
*                       ----------------------------
*                       innovating embedded platform
*
* Copyright (c) 2001-2018 Guangzhou ZHIYUAN Electronics Co., Ltd.
* All rights reserved.
*
* Contact information:
* web site:    http://www.zlg.cn/
*******************************************************************************/

/**
 * \file
 * \brief PCAӿ
 *
 * \internal
 * \par Modification History
 * - 1.00 19-10-09
 * \endinternal
 */

#ifndef __AM_ZLG118_PCA_H
#define __AM_ZLG118_PCA_H

#ifdef __cplusplus
extern "C" {
#endif

#include "ametal.h"
#include "hw/amhw_zlg118_pca.h"

/* жϱ־ö */
typedef enum am_zlg118_pca_int {
    ZLG118_PCA_CF   = (0x1 << 7),   /* ־ */
    ZLG118_PCA_CCF4 = (0x1 << 4),   /* ģ4Ƚ/־λ */
    ZLG118_PCA_CCF3 = (0x1 << 3),   /* ģ3Ƚ/־λ */
    ZLG118_PCA_CCF2 = (0x1 << 2),   /* ģ2Ƚ/־λ */
    ZLG118_PCA_CCF1 = (0x1 << 1),   /* ģ1Ƚ/־λ */
    ZLG118_PCA_CCF0 = (0x1 << 0)    /* ģ0Ƚ/־λ */
} am_zlg118_pca_int_t;

/* ʱԴö */
typedef enum am_zlg118_pca_clk {
    ZLG118_PCA_PCLK32  = (0x0),  /* PCLK/32 */
    ZLG118_PCA_PCLK16  = (0x1),  /* PCLK/16 */
    ZLG118_PCA_PCLK8   = (0x2),  /* PCLK/8 */
    ZLG118_PCA_PCLK4   = (0x3),  /* PCLK/4 */
    ZLG118_PCA_PCLK2   = (0x4),  /* PCLK/2 */
    ZLG118_PCA_TIM0_OF = (0x5),  /* timer0 overflow */
    ZLG118_PCA_TIM1_OF = (0x6),  /* timer1 overflow */
    ZLG118_PCA_EC1     = (0x7),  /* ECI ⲿʱӣʱPCLKķƵ */
} am_zlg118_pca_clk_t;

/* Ƚϲģʽö */
typedef enum am_zlg118_pca_ccapm {
    ZLG118_PCA_ECOM = (0x1 << 6),   /* Ƚܿλ */
    ZLG118_PCA_MAT  = (0x1 << 3),   /* ƥλ */
    ZLG118_PCA_PWM  = (0x1 << 1),   /* ƿλ */
    ZLG118_PCA_CCIE = (0x1 << 0)    /* PCAʹж */
} am_zlg118_pca_ccapm_t;

/* زö */
typedef enum am_zlg118_pca_capn {
    ZLG118_PCA_CAPP_EN  = (0x1 << 1),   /* ز */
    ZLG118_PCA_CAPP_DIS = (0x0 << 1),   /* ֹز */
    ZLG118_PCA_CAPN_EN  = (0x1),        /* ز */
    ZLG118_PCA_CAPN_DIS = (0x0),        /* ֹز */
} am_zlg118_pca_capn_t;

/* תö */
typedef enum am_zlg118_pca_tog {
    ZLG118_PCA_TOG_EN   = (0x1),   /* ʹܷת */
    ZLG118_PCA_TOG_DIS  = (0x0),   /* ܷת */
} am_zlg118_pca_tog_t;

/* PWMѡ */
typedef enum am_zlg118_pca_pwm {
    ZLG118_PCA_PWM_8  = 0,   /* 8λPWM */
    ZLG118_PCA_PWM_16 = 1    /* 16λPWM */
} am_zlg118_pca_pwm_t;

/**
 * \brief PCAӦͨGPIOϢ
 */
typedef struct {
    uint32_t gpio;          /**< \brief ӦGPIOܽź */
    uint32_t pwmunc;        /**< \brief PWMģʽӦGPIO */
    uint32_t capfunc;       /**< \brief CAPģʽӦGPIOĬ */
} am_zlg118_pca_ioinfo_t;

/**
 * \brief PCNT豸Ϣṹ
 */
typedef struct am_zlg118_pca_devinfo {
    uint32_t                pca_regbase;        /* Ĵַ */
    uint32_t                clk_num;            /* ʱID */
    uint16_t                inum;               /* жϺ */
    am_zlg118_pca_clk_t     clk_src;            /* ʱԴѡ */
    am_zlg118_pca_ioinfo_t  *ioinfo;            /* Ϣ */
    void                    (*pfn_plfm_init)(void);         /*ƽ̨ʼ */
    void                    (*pfn_plfm_deinit)(void);       /* ƽ̨ʼ */
} am_zlg118_pca_devinfo_t;

typedef struct am_zlg118_pca_dev {
    amhw_zlg118_pca_t             *p_hw_pca;
    const am_zlg118_pca_devinfo_t *p_devinfo;
} am_zlg118_pca_dev_t;

typedef am_zlg118_pca_dev_t* am_zlg118_pca_handle_t;

/**
 * \addtogroup am_zlg_if_pca
 * \copydoc am_zlg_pca.h
 * @{
 */

/**
 * \brief PCAʼ
 *
 * \return PCA
 */
am_zlg118_pca_handle_t am_zlg118_pca_init (
        am_zlg118_pca_dev_t             *p_dev,
        const am_zlg118_pca_devinfo_t   *p_devinfo);

/**
 * \brief PCAʼ
 *
 * \param[in] handle : 豸PCA׼
 *
 * \return 
 */
void am_zlg118_pca_deinit (am_zlg118_pca_handle_t handle);

/**
 * \brief PCAزģʽ
 *
 * \param[in] handle : 豸PCA׼
 * \param[in] num    : PCA ģ
 * \param[in] chan   : ͨ
 * \param[in] cap    : ѡ
 *
 * \return AM_OK:ɹ
 *         :ʧ
 */
int am_zlg118_pca_cap_set (am_zlg118_pca_handle_t handle,
                           uint8_t                num,
                           uint8_t                chan,
                           am_zlg118_pca_capn_t   cap);

/**
 * \brief PCAȽϼģʽ
 *
 * \param[in] handle : 豸PCA׼
 * \param[in] num    : PCA ģ
 * \param[in] carr   : ڼĴֵ
 * \param[in] ccap   : ȽϲĴֵ
 *
 * \return AM_OK:ɹ
 *         :ʧ
 */
int am_zlg118_pca_cmp_cnt_set (am_zlg118_pca_handle_t handle,
                               uint8_t                num,
                               uint16_t               carr,
                               uint16_t               ccap);

/**
 * \brief PCAģʽ
 *
 * \param[in] handle  : 豸PCA׼
 * \param[in] num     : PCA ģ
 * \param[in] chan    : ͨ
 * \param[in] pwm_sel : PWMѡ
 * \param[in] carr    : ڼĴֵ
 * \param[in] ccap    : ȽϲĴֵ
 *
 * 8bitռձ  = (255 - ccap) / 100 * 100%
 * 16bitռձ= (carr - ccap) / carr * 100%
 *
 * \return AM_OK:ɹ
 *         :ʧ
 */
int am_zlg118_pca_pwm_set (am_zlg118_pca_handle_t handle,
                           uint8_t                num,
                           uint8_t                chan,
                           am_zlg118_pca_pwm_t    pwm_sel,
                           uint16_t               carr,
                           uint16_t               ccap);

/**
 * \brief PCAʼ
 *
 * \param[in] handle : 豸PCA׼
 *
 * \return
 */
void am_zlg118_pca_start(am_zlg118_pca_handle_t handle);

/**
 * \brief PCAֹͣ
 *
 * \param[in] handle : 豸PCA׼
 *
 * \return
 */
void am_zlg118_pca_stop(am_zlg118_pca_handle_t handle);

/** @} */

#ifdef __cplusplus
}
#endif

#endif /* __AM_ZLG118_PCA_H */

/* end of file */
