/*******************************************************************************
*                                 AMetal
*                       ----------------------------
*                       innovating embedded platform
*
* Copyright (c) 2001-2018 Guangzhou ZHIYUAN Electronics Co., Ltd.
* All rights reserved.
*
* Contact information:
* web site:    http://www.zlg.cn/
*******************************************************************************/

/**
 * \file
 * \brief PCNTӿ
 *
 * \internal
 * \par Modification history
 * - 1.00 20-03-14
 * \endinternal
 */

#ifndef __AMHW_ZSN700_PCNT_H
#define __AMHW_ZSN700_PCNT_H

#ifdef __cplusplus
extern "C" {
#endif

#include "am_types.h"

/*
 * \brief ṹεĿʼ
 */
#if defined(__CC_ARM)
  #pragma push
  #pragma anon_unions
#elif defined(__GNUC__)
#elif defined(__TASKING__)
  #pragma warning 586
#else
  #warning Not supported compiler t
#endif

/**
 * \addtogroup amhw_zsn700_if_pcnt
 * \copydoc amhw_zsn700_pcnt.h
 * @{
 */

/**
 * \brief PCNTĴṹ
 */
typedef struct amhw_zsn700_pcnt {
    __IO uint32_t run;      /* Ĵ */
    __IO uint32_t ctrl;     /* ƼĴ */
    __IO uint32_t flt;      /* ˲ƼĴ */
    __IO uint32_t tocr;     /* ʱƼĴ */
    __O  uint32_t cmd;      /* Ĵ */
    __I  uint32_t sr1;      /* ״̬Ĵ1 */
    __I  uint32_t cnt;      /* Ĵ */
    __I  uint32_t top;      /* Ĵ */
    __IO uint32_t buf;      /* Ĵ */
    __I  uint32_t ifr;      /* жϱʶĴ */
    __O  uint32_t icr;      /* жĴ */
    __IO uint32_t ien;      /* жʹܼĴ */
    __I  uint32_t sr2;      /* ״̬Ĵ2 */
    __IO uint32_t dbg;      /* ѡĴ */
} amhw_zsn700_pcnt_t;

/**
 * \brief PCNT
 *
 * \papcnt[in]  p_hw_pcnt  : ָPCNTĴṹָ
 *
 * \return 
 */
am_static_inline
void amhw_zsn700_pcnt_start (amhw_zsn700_pcnt_t *p_hw_pcnt)
{
    p_hw_pcnt->run |= 0x1ul;
}

/**
 * \brief PCNTֹͣ
 *
 * \papcnt[in]  p_hw_pcnt  : ָPCNTĴṹָ
 *
 * \return 
 */
am_static_inline
void amhw_zsn700_pcnt_stop (amhw_zsn700_pcnt_t *p_hw_pcnt)
{
    p_hw_pcnt->run &= ~0x1ul;
}

/**
 * \brief PCNT/ֹͣ״̬ȡ
 *
 * \papcnt[in]  p_hw_pcnt  : ָPCNTĴṹָ
 *
 * \return 0:ֹͣ, 1:
 */
am_static_inline
uint8_t amhw_zsn700_pcnt_run_get (amhw_zsn700_pcnt_t *p_hw_pcnt)
{
    return p_hw_pcnt->run & 0x1ul;
}

/**
 * \brief S1ͨѡ
 *
 * \papcnt[in]  p_hw_pcnt  : ָPCNTĴṹָ
 * \papcnt[in]  mode       : ѡ 1ȡ 0ȡ
 *
 * \return 
 */
am_static_inline
void amhw_zsn700_pcnt_s1p_set (amhw_zsn700_pcnt_t *p_hw_pcnt, uint8_t mode)
{
        p_hw_pcnt->ctrl = (p_hw_pcnt->ctrl & ~(0x1ul << 6)) | \
                          ((mode & 0x1) << 6);
}

/**
 * \brief S0ͨѡ
 *
 * \papcnt[in]  p_hw_pcnt  : ָPCNTĴṹָ
 * \papcnt[in]  mode       : ѡ 1ȡ 0ȡ
 *
 * \return 
 */
am_static_inline
void amhw_zsn700_pcnt_s0p_set (amhw_zsn700_pcnt_t *p_hw_pcnt, uint8_t mode)
{
        p_hw_pcnt->ctrl = (p_hw_pcnt->ctrl & ~(0x1ul << 5)) | \
                          ((mode & 0x1) << 5);
}

/**
 * \brief ѡ(Եͨģʽ˫ͨǽģʽЧ)
 *
 * \papcnt[in]  p_hw_pcnt  : ָPCNTĴṹָ
 * \papcnt[in]  dir        :  1 0Ӽ
 *
 * \return 
 */
am_static_inline
void amhw_zsn700_pcnt_dir_set (amhw_zsn700_pcnt_t *p_hw_pcnt, uint8_t dir)
{
        p_hw_pcnt->ctrl = (p_hw_pcnt->ctrl & ~(0x1ul << 4)) | \
                          ((dir & 0x1) << 4);
}

/**
 * \brief ȡ(Եͨģʽ˫ͨǽģʽЧ)
 *
 * \papcnt[in]  p_hw_pcnt  : ָPCNTĴṹָ
 *
 * \return  1 0Ӽ
 */
am_static_inline
uint8_t amhw_zsn700_pcnt_dir_get (amhw_zsn700_pcnt_t *p_hw_pcnt)
{
    return (p_hw_pcnt->ctrl >> 4) & 0x1ul;
}

/**
 * \brief ʱѡ
 *
 * \papcnt[in]  p_hw_pcnt  : ָPCNTĴṹָ
 * \papcnt[in]  clksel     : ʱѡ 0:PCLK
 *                                     1:PCLK
 *                                     2:XTL
 *                                     3:RCL
 *
 * \return 
 */
am_static_inline
void amhw_zsn700_pcnt_clk_set (amhw_zsn700_pcnt_t *p_hw_pcnt, uint8_t clksel)
{
    p_hw_pcnt->ctrl = (p_hw_pcnt->ctrl & ~(0x3ul << 2)) | \
                      ((clksel & 0x3ul)<< 2);
}

/**
 * \brief ģʽѡ
 *
 * \papcnt[in]  p_hw_pcnt  : ָPCNTĴṹָ
 * \papcnt[in]  mode       : ʱѡ 0:ͨģʽ
 *                                     1:ͨģʽ
 *                                     2:˫ͨǽģʽ
 *                                     3:˫ͨģʽ
 *
 * \return 
 */
am_static_inline
void amhw_zsn700_pcnt_mode_set (amhw_zsn700_pcnt_t *p_hw_pcnt, uint8_t mode)
{
    p_hw_pcnt->ctrl = (p_hw_pcnt->ctrl & ~0x3ul) | (mode & 0x3ul);
}

/**
 * \brief ˲ʹ
 *
 * \papcnt[in]  p_hw_pcnt  : ָPCNTĴṹָ
 *
 * \return 
 */
am_static_inline
void amhw_zsn700_pcnt_flt_enable (amhw_zsn700_pcnt_t *p_hw_pcnt)
{
    p_hw_pcnt->flt |= (0x1ul << 16);
}

/**
 * \brief ˲
 *
 * \papcnt[in]  p_hw_pcnt  : ָPCNTĴṹָ
 *
 * \return 
 */
am_static_inline
void amhw_zsn700_pcnt_flt_disable (amhw_zsn700_pcnt_t *p_hw_pcnt)
{
    p_hw_pcnt->flt &= ~(0x1ul << 16);
}

/**
 * \brief ˲ֵ
 *
 * \papcnt[in]  p_hw_pcnt  : ָPCNTĴṹָ
 * \papcnt[in]  top        : ֵ,Ϊ0
 *
 * \return 
 */
am_static_inline
void amhw_zsn700_pcnt_debtop_set (amhw_zsn700_pcnt_t *p_hw_pcnt, uint8_t top)
{
    if(top > 0) {
        p_hw_pcnt->flt = (p_hw_pcnt->flt & ~(0x7ul << 13)) | \
                         ((top & 0x7ul) << 13);
    }
}

/**
 * \brief ˲ʱӷƵϵ
 *
 * \papcnt[in]  p_hw_pcnt  : ָPCNTĴṹָ
 * \papcnt[in]  div        : Ƶϵ
 *
 * \return 
 */
am_static_inline
void amhw_zsn700_pcnt_clkdiv_set (amhw_zsn700_pcnt_t *p_hw_pcnt, uint16_t div)
{
    p_hw_pcnt->flt = (p_hw_pcnt->flt & ~0x1ffful) | (div & 0x1ffful);
}

/**
 * \brief ʱʹ
 *
 * \papcnt[in]  p_hw_pcnt  : ָPCNTĴṹָ
 *
 * \return 
 */
am_static_inline
void amhw_zsn700_pcnt_timeover_enable (amhw_zsn700_pcnt_t *p_hw_pcnt)
{
    p_hw_pcnt->tocr |= (0x1ul << 16);
}

/**
 * \brief ʱܽ
 *
 * \papcnt[in]  p_hw_pcnt  : ָPCNTĴṹָ
 *
 * \return 
 */
am_static_inline
void amhw_zsn700_pcnt_timeover_disable (amhw_zsn700_pcnt_t *p_hw_pcnt)
{
    p_hw_pcnt->tocr &= ~(0x1ul << 16);
}

/**
 * \brief ʱֵ
 *
 * \papcnt[in]  p_hw_pcnt  : ָPCNTĴṹָ
 * \papcnt[in]  th         : ʱֵ
 *
 * \return 
 */
am_static_inline
void amhw_zsn700_pcnt_th_set (amhw_zsn700_pcnt_t *p_hw_pcnt, uint16_t th)
{
    p_hw_pcnt->tocr = (p_hw_pcnt->tocr & ~0xffful) | (th & 0xffful);
}

/**
 * \brief Ĵ
 *
 * \papcnt[in]  p_hw_pcnt  : ָPCNTĴṹָ
 * \papcnt[in]  cmd        : 
 *
 * \return 
 */
am_static_inline
void amhw_zsn700_pcnt_cmd_set (amhw_zsn700_pcnt_t *p_hw_pcnt, uint32_t cmd)
{
    p_hw_pcnt->cmd |= cmd;
}

/**
 * \brief ˫ͨȡ
 *
 * \papcnt[in]  p_hw_pcnt  : ָPCNTĴṹָ
 *
 * \return 0:Ӽ, 1:
 */
am_static_inline
uint8_t amhw_zsn700_pcnt_ddir_get (amhw_zsn700_pcnt_t *p_hw_pcnt)
{
    return p_hw_pcnt->sr1 & 0x1ul;
}

/**
 * \brief ֵȡ
 *
 * \papcnt[in]  p_hw_pcnt  : ָPCNTĴṹָ
 *
 * \return ֵ
 */
am_static_inline
uint16_t amhw_zsn700_pcnt_cnt_get (amhw_zsn700_pcnt_t *p_hw_pcnt)
{
    return p_hw_pcnt->cnt & 0xfffful;
}

/**
 * \brief ֵ/ֵȡ
 *
 * \papcnt[in]  p_hw_pcnt  : ָPCNTĴṹָ
 *
 * \return ֵ
 */
am_static_inline
uint16_t amhw_zsn700_pcnt_top_get (amhw_zsn700_pcnt_t *p_hw_pcnt)
{
    return p_hw_pcnt->top & 0xfffful;
}

/**
 * \brief ȡ
 *
 * \papcnt[in]  p_hw_pcnt  : ָPCNTĴṹָ
 *
 * \return ֵ
 */
am_static_inline
uint16_t amhw_zsn700_pcnt_buf_get (amhw_zsn700_pcnt_t *p_hw_pcnt)
{
    return p_hw_pcnt->buf & 0xfffful;
}

/**
 * \brief ֵ
 *
 * \papcnt[in]  p_hw_pcnt  : ָPCNTĴṹָ
 * \papcnt[in]  buf        : ֵ
 *
 * \return 
 */
am_static_inline
void amhw_zsn700_pcnt_buf_set (amhw_zsn700_pcnt_t *p_hw_pcnt, uint16_t buf)
{
    p_hw_pcnt->buf = buf;
}

/**
 * \brief жϱʶȡ
 *
 * \papcnt[in]  p_hw_pcnt  : ָPCNTĴṹָ
 * \papcnt[in]  ie         : жϱʶ
 *
 * \return 0:жϱʶ, 0:жϱʶ
 */
am_static_inline
uint16_t amhw_zsn700_pcnt_int_get (amhw_zsn700_pcnt_t *p_hw_pcnt, uint8_t ie)
{
    return p_hw_pcnt->buf & ie;
}

/**
 * \brief жϱʶ
 *
 * \papcnt[in]  p_hw_pcnt  : ָPCNTĴṹָ
 * \papcnt[in]  ie         : жϱʶ
 *
 * \return
 */
am_static_inline
void amhw_zsn700_pcnt_int_clear (amhw_zsn700_pcnt_t *p_hw_pcnt, uint8_t ie)
{
    p_hw_pcnt->buf &= ~ie;
}

/**
 * \brief жʹ
 *
 * \papcnt[in]  p_hw_pcnt  : ָPCNTĴṹָ
 * \papcnt[in]  ie         : жϱʶ
 *
 * \return
 */
am_static_inline
void amhw_zsn700_pcnt_int_enable (amhw_zsn700_pcnt_t *p_hw_pcnt, uint8_t ie)
{
    p_hw_pcnt->ien |= ie;
}

/**
 * \brief жϽ
 *
 * \papcnt[in]  p_hw_pcnt  : ָPCNTĴṹָ
 * \papcnt[in]  ie         : жϱʶ
 *
 * \return
 */
am_static_inline
void amhw_zsn700_pcnt_int_disable (amhw_zsn700_pcnt_t *p_hw_pcnt, uint8_t ie)
{
    p_hw_pcnt->ien &= ~ie;
}

/**
 * \brief ͬ״̬ȡ
 *
 * \papcnt[in]  p_hw_pcnt  : ָPCNTĴṹָ
 * \papcnt[in]  cmd         : ͬ
 *
 * \return 0:ͬ, 0:ͬ
 */
am_static_inline
uint16_t amhw_zsn700_pcnt_stat_get (amhw_zsn700_pcnt_t *p_hw_pcnt, uint8_t cmd)
{
    return p_hw_pcnt->sr2 & cmd;
}

/**
 * \brief ѡ
 *
 * \papcnt[in]  p_hw_pcnt  : ָPCNTĴṹָ
 * \papcnt[in]  sel        : ѡ
 *
 * \return 
 */
am_static_inline
void amhw_zsn700_pcnt_dbg_set (amhw_zsn700_pcnt_t *p_hw_pcnt, uint8_t sel)
{
    p_hw_pcnt->buf = sel;
}

/**
 * @}
 */
/*
 * \brief ṹεĽ
 */

#if defined(__CC_ARM)
  #pragma pop
#elif defined(__GNUC__)
#elif defined(__TASKING__)
  #pragma warning restore
#else
  #warning Not supported compiler t
#endif

#ifdef __cplusplus
}
#endif /* __AMHW_ZSN700_PCNT_H */

#endif

/* end of file */
