/*******************************************************************************
*                                 AMetal
*                       ----------------------------
*                       innovating embedded platform
*
* Copyright (c) 2001-2018 Guangzhou ZHIYUAN Electronics Co., Ltd.
* All rights reserved.
*
* Contact information:
* web site:    http://www.zlg.cn/
*******************************************************************************/

/**
 * \file
 * \brief PWR ӿ
 *
 *
 * \internal
 * \par Modification history
 * - 1.00 19-07-23  zp, first implementation
 * \endinternal
 */

#ifndef __AM_ZLG237_PWR_H
#define __AM_ZLG237_PWR_H

#ifdef __cplusplus
extern "C" {
#endif

#include "ametal.h"
#include "hw/amhw_zlg237_pwr.h"
#include "../bkp/hw/amhw_zlg237_bkp.h"

/**
 * \addtogroup am_zlg237_if_pwr
 * \copydoc am_zlg237_pwr.h
 * @{
 */

/**
 * \brief PWR ģʽ
 */
typedef enum am_zlg237_pwr_mode {
    AM_ZLG237_PWR_MODE_RUN = 0,      /**< \brief ģʽ */
    AM_ZLG237_PWR_MODE_SLEEP,        /**< \brief ˯ģʽ */
    AM_ZLG237_PWR_MODE_STOP,         /**< \brief ֹͣģʽ */
    AM_ZLG237_PWR_MODE_STANBY,       /**< \brief ģʽ */
} am_zlg237_pwr_mode_t;

/** \brief PWR PVD ѹϢ */
typedef struct am_zlg237_pwr_pvd_info {
    amhw_zlg237_pvd_lever_set_t pvd_v_level; /**< \brief PVD ѹѡ */

    /**
     *  \brief PVD ѹģʽ.
     *   0ʾѹӸ½趨ֵʱж
     *   1ʾѹӵ趨ֵʱжϣ
     *   2ʾѹ½Խ趨ֵʱж
     */
    uint8_t                     pvd_mode;

    am_bool_t                   enable;      /**< \brief Ƿʹ PVD ѹ */
} am_zlg237_pwr_pvd_info_t;

/**
 * \brief PWR ģʽ
 */
typedef struct am_zlg237_pwr_mode_init {
    am_zlg237_pwr_mode_t  pwr_mode; /**< \brief ָ PWR Ĵָ */
    int                   pin;      /**< \brief  */
} am_zlg237_pwr_mode_init_t;

/**
 * \brief PWR 豸Ϣṹ
 */
typedef struct am_zlg237_pwr_devinfo {

    uint32_t  pwr_regbase;                   /**< \brief PWR Ĵַ */

    uint32_t  bkp_regbase;                   /**< \brief BKP Ĵַ */

    int       ahb_clk_num;                   /**< \brief AHB ʱ ID */
    int       apb1_clk_num;                  /**< \brief APB1 ʱ ID */
    int       apb2_clk_num;                  /**< \brief APB2 ʱ ID */

    uint8_t   inum;                          /**< \brief PVD жϺ */

    am_zlg237_pwr_mode_init_t *p_pwr_mode;   /**< \brief ָ PWR ģʽָ */

    am_zlg237_pwr_pvd_info_t  *p_pvd_info;   /**< \brief ָ PVD ѹϢָ */

    void (*pfn_plfm_init)(void);             /**< \brief ƽ̨ʼ */

    void (*pfn_plfm_deinit)(void);           /**< \brief ƽ̨ȥʼ */

} am_zlg237_pwr_devinfo_t;

/**
 * \brief PWR 豸ṹ
 */
typedef struct am_zlg237_pwr_dev {

    /** \brief ָ PWR 豸Ϣָ */
    const am_zlg237_pwr_devinfo_t *p_pwrdevinfo;

    /** \brief PWR ģʽ */
    am_zlg237_pwr_mode_t           pwr_mode;

} am_zlg237_pwr_dev_t;

/** \brief PWR 豸Ͷ */
typedef am_zlg237_pwr_dev_t *am_zlg237_pwr_handle_t;

/**
 * \brief PWR ʼ
 *
 * \param[in] p_dev     ָ PWR 豸ṹָ
 * \param[in] p_devinfo ָ PWR 豸Ϣṹ峣ָ
 *
 * \return  am_zlg237_pwr_handle_t ͵ľ
 */
am_zlg237_pwr_handle_t
am_zlg237_pwr_init (am_zlg237_pwr_dev_t           *p_dev,
                    const am_zlg237_pwr_devinfo_t *p_devinfo);

/**
 * \brief PWR ȥʼ
 *
 * \param[in] 
 *
 * \return 
 */
void am_zlg237_pwr_deinit (void);

/**
 * \brief ȡϵͳģʽ
 *
 * \param[in] mode         PWR ģʽ壬ֵΪ am_zlg237_pwr_mode_t һö
 * \param[in] pfn_callback ѻص
 * \param[in] p_arg        ѻص 
 *
 * \return 
 *
 * \note ѵõ PWR ģʽ am_zlg237_pwr_mode_into ѡ PWR
 *       ģʽͬ AM_ZLG237_PWR_MODE_STANBY ģʽ£ѻḴλעжϻ
 *       ص
 */
void am_zlg237_wake_up_cfg (am_zlg237_pwr_mode_t mode,
                            am_pfnvoid_t         pfn_callback,
                            void                *p_arg);

/**
 * \brief  PWR ģʽ
 *
 * \param[in] mode PWR ģʽ
 *
 * \retval  AM_OK      ģʽлЧ
 * \retval  AM_ERROR   ģʽл
 * \retval -AM_EINVAL  ʽ
 * \retval -AM_ENOTSUP WAKE_UP Ϊߵƽܽģʽ
 *
 * \note 1. ֻ WKUP Ϊ͵ƽʱܽģʽ ҴģʽѺִиλ
 *       2.  PWR ģʽΪ AM_ZLG237_PWR_MODE_STANBY ʧܺWKUP ŻᱻΪ
 *          ģʽ
 */
int am_zlg237_pwr_mode_into (am_zlg237_pwr_mode_t mode);

/**
 * \brief ȡ PWR ģʽ
 *
 * \param[in] 
 *
 * \return am_zlg237_pwr_mode_t һö͵ֵ
 */
am_zlg237_pwr_mode_t am_zlg237_pwr_mode_get (void);

/**
 * \brief  PVD ѹϢ
 *
 * \param[in] pwr_handle   PWR Դ豸
 * \param[in] pfn_callback ѹص
 * \param[in] p_arg        ѹص
 *
 * \retval AM_OK óɹ
 */
int am_zlg237_pwr_pvd_cfg (am_zlg237_pwr_handle_t pwr_handle,
                           am_pfnvoid_t           pfn_callback,
                           void                  *p_arg);

/**
 * @}
 */

#ifdef __cplusplus
}
#endif

#endif /* __AM_ZLG237_PWR_H */

/* end of file */
