/*******************************************************************************
*                                 AMetal
*                       ----------------------------
*                       innovating embedded platform
*
* Copyright (c) 2001-2018 Guangzhou ZHIYUAN Electronics Co., Ltd.
* All rights reserved.
*
* Contact information:
* web site:    http://www.zlg.cn/
*******************************************************************************/

/**
 * \file
 * \brief Դӿ
 *
 * \internal
 * \par History
 * - 1.00 19-07-18  zp, first implementation
 * \endinternal
 */

#ifndef __AMHW_ZLG237_PWR_H
#define __AMHW_ZLG237_PWR_H

#ifdef __cplusplus
extern "C" {
#endif

#include "am_types.h"
#include "ametal.h"

/**
 * \addtogroup amhw_zlg237_if_pwr
 * \copydoc amhw_zlg237_pwr.h
 * @{
 */

/**
  * \brief ϵͳƼĴṹ
  */
typedef struct amhw_zlg237_pwr {
    __IO  uint32_t cr;          /**< \brief ԴƼĴ */
    __IO  uint32_t csr;         /**< \brief Դ/״̬Ĵ */
    __I   uint32_t reserve[2];  /**< \brief  */
    __IO  uint32_t ldo;         /**< \brief Ƭ LDO ģʽѹƼĴ */
    __IO  uint32_t ldo_stop;    /**< \brief Ƭ LDO STOPģʽѹƼĴ */
} amhw_zlg237_pwr_t;

/**
 * \name ˯ģʽ ͣģʽ£ѹѹЭ鿴ԭֲᣩ
 * @{
 */
#define AM_ZLG237_PDDS_STOP_MODE_LDO_ON      0 /* 00 ͣģʽ  ѹѹ(LDO)*/
#define AM_ZLG237_PDDS_STOP_MODE_LDO_OFF     1 /* 01 ͣģʽ  ѹѹ(LDO)͹ģʽ*/
#define AM_ZLG237_PDDS_STANDBY_MODE_LDO_ON   2 /* 10 ģʽ */
#define AM_ZLG237_PDDS_STANDBY_MODE_LDO_OFF  3 /* 11 ģʽ */
/** @} */

/**
 * \brief ȵģʽѡ
 *
 * \param[in] mode       ȵģʽ,ֵΪ  AM_ZLG237_PDDS_STOP_MODE_LDO_ON     
 *                                       AM_ZLG237_PDDS_STOP_MODE_LDO_OFF    
 *                                       AM_ZLG237_PDDS_STANDBY_MODE_LDO_ON  
 *                                       AM_ZLG237_PDDS_STANDBY_MODE_LDO_OFF
 * \param[in] p_hw_pwr   ָԴƼĴָ
 *
 * \return 
 */
am_static_inline
void amhw_zlg237_pwr_pdds_lpds_mode_set (amhw_zlg237_pwr_t *p_hw_pwr,
                                         uint8_t            mode)
{
    p_hw_pwr->cr = (p_hw_pwr->cr & (~(3ul << 0))) | (mode << 0) ;
}

/**
 * \name λ뻽λ
 * @{
 */
#define AM_ZLG237_STANDBY_FLAG_CLEAR  (1UL << 3)
#define AM_ZLG237_WAKEUP_FLAG_CLEAR   (1UL << 2)
/** @} */
/**
 * \brief ־λ뻽λ
 *
 * \param[in] p_hw_pwr    ָԴƼĴָ
 * \param[in] status_flg  ־λѡֵΪ  AM_ZLG237_*_FLAG_CLEAR һ
 *                           (# AM_ZLG237_STANDBY_FLAG_CLEAR)
 *
 * \return 
 */
am_static_inline
void amhw_zlg237_pwr_stauts_flag_clear (amhw_zlg237_pwr_t *p_hw_pwr,
                                        uint8_t            status_flg)
{
    p_hw_pwr->cr |= status_flg;
}


/**
 * \name ԴѹPVDʹ
 * @{
 */
#define AM_ZLG237_PVDE_ENABLE        1
#define AM_ZLG237_PVDE_DISABLE       0
/** @} */
/**
 * \brief PVDѹʹ
 *
 * \param[in] p_hw_pwr    ָԴƼĴָ
 * \param[in] pvd_enable  ѹʹܣֵΪ  AM_ZLG237_PVDE_* һ
 *                           (# AM_ZLG237_PVDE_ENABLE)
 *
 * \return 
 */
am_static_inline
void amhw_zlg237_pwr_pvd_detect_enable (amhw_zlg237_pwr_t *p_hw_pwr,
                                        uint8_t            pvd_enable)
{
    p_hw_pwr->cr = (p_hw_pwr->cr & (~(1ul << 4))) | (pvd_enable << 4) ;
}

/**
 * \brief PVDƽѡ
 */
typedef enum amhw_zlg237_pvd_lever_set {
    AMHW_ZLG237_PVD_LEVER_2_2V  = 0,  /**< \brief 2.2V PVDѹ */
    AMHW_ZLG237_PVD_LEVER_2_3V  = 1,  /**< \brief 2.3V PVDѹ */
    AMHW_ZLG237_PVD_LEVER_2_4V  = 2,  /**< \brief 2.4V PVDѹ */
    AMHW_ZLG237_PVD_LEVER_2_5V  = 3,  /**< \brief 2.5V PVDѹ */
    AMHW_ZLG237_PVD_LEVER_2_6V  = 4,  /**< \brief 2.6V PVDѹ */
    AMHW_ZLG237_PVD_LEVER_2_7V  = 5,  /**< \brief 2.7V PVDѹ */
    AMHW_ZLG237_PVD_LEVER_2_8V  = 6,  /**< \brief 2.8V PVDѹ */
    AMHW_ZLG237_PVD_LEVER_2_9V  = 7,  /**< \brief 2.9V PVDѹ */
} amhw_zlg237_pvd_lever_set_t;
/**
 * \brief PVѹѡ
 *
 * \param[in] p_hw_pwr ָԴƼĴָ
 * \param[in] pvd_leverѹⷶΧѡ, ֵΪ amhw_zlg237_pvd_lever_set_t һö͵ֵ
 *
 * \return 
 */
am_static_inline
void amhw_zlg237_pvd_lever_set (amhw_zlg237_pwr_t           *p_hw_pwr,
                                amhw_zlg237_pvd_lever_set_t  pvd_lever)
{
    p_hw_pwr->cr = (p_hw_pwr->cr & (~(7ul << 5))) | (pvd_lever << 5) ;
}

/**
 * \name BKPݼĴʹ
 * @{
 */
#define AM_ZLG237_BKP_ENABLE         1
#define AM_ZLG237_BKP_DISABLE        0
/** @} */
/**
 * \brief ʱݼĴʹ
 *
 * \param[in] p_hw_pwr    ָԴƼĴָ
 * \param[in] bkp_enable  ѹʹܣֵΪ  AM_ZLG237_BKP_* һ
 *                           (# AM_ZLG237_BKP_ENABLE)
 *
 * \return 
 */
am_static_inline
void amhw_zlg237_pwr_bkp_access_enable (amhw_zlg237_pwr_t *p_hw_pwr,
                                        uint8_t            bkp_enable)
{
    p_hw_pwr->cr = (p_hw_pwr->cr & (~(1ul << 8))) | (bkp_enable << 8) ;
}

/**
 * \name WAKE_UPʹ
 * @{
 */
#define AM_ZLG237_WAKEUP_ENABLE      1
#define AM_ZLG237_WAKEUP_DISABLE     0
/** @} */
/**
 * \brief ʹWAKE_UP
 *
 * \param[in] p_hw_pwr ָԴƼĴָ
 * \param[in] wakeup_enʹֵΪ  AM_ZLG237_WAKEUP_ENABLE  
 *                                      AM_ZLG237_WAKEUP_DISABLE
 *
 * \return 
 */
am_static_inline
void amhw_zlg237_wake_up_enable (amhw_zlg237_pwr_t *p_hw_pwr, uint8_t wakeup_en)
{
    p_hw_pwr->csr = (p_hw_pwr->csr & (~(1ul << 8))) | (wakeup_en << 8) ;
}


/**
 * \brief Դ״̬־
 */
typedef enum amhw_zlg237_pwr_status_flag {
    AMHW_ZLG237_PVD_OUTPUT_FLAG  = (1UL << 2),  /**< \brief PVD־λ */
    AMHW_ZLG237_PVD_SBF_FLAG     = (1UL << 1),  /**< \brief ־λ */
    AMHW_ZLG237_PVD_WUF_FLAG     = (1UL << 0),  /**< \brief ѱ־λ */
} amhw_zlg237_pwr_status_flag_t;
/**
 * \brief ȡԴ״̬λ
 *
 * \param[in] p_hw_pwr   ָԴƼĴָ
 * \param[in] status_flagԴ״̬λֵΪ amhw_zlg237_pwr_status_flag_t һö
 *
 * \return ֵΪ0ʱñ־λ1
 */
am_static_inline
uint32_t amhw_zlg237_pwr_csr_status_get (
    amhw_zlg237_pwr_t *p_hw_pwr, amhw_zlg237_pwr_status_flag_t status_flag)
{
    return p_hw_pwr->csr & status_flag;
}

/**
 * \brief MCURUNģʽ£ƬLDOѹ
 */
typedef enum amhw_zlg237_pwr_ldo_run_set {
    AMHW_ZLG237_PWR_LDO_RUN_1_20V  = 0,     /**< \brief 1.20V Ƭ LDOѹ */
    AMHW_ZLG237_PWR_LDO_RUN_1_24V  = 1,     /**< \brief 1.24V Ƭ LDOѹ */
    AMHW_ZLG237_PWR_LDO_RUN_1_28V  = 2,     /**< \brief 1.28V Ƭ LDOѹ */
    AMHW_ZLG237_PWR_LDO_RUN_1_32V  = 3,     /**< \brief 1.32V Ƭ LDOѹ */
    AMHW_ZLG237_PWR_LDO_RUN_1_36V  = 4,     /**< \brief 1.36V Ƭ LDOѹ */
    AMHW_ZLG237_PWR_LDO_RUN_1_40V  = 5,     /**< \brief 1.40V Ƭ LDOѹ */
    AMHW_ZLG237_PWR_LDO_RUN_1_44V  = 6,     /**< \brief 1.44V Ƭ LDOѹ */
    AMHW_ZLG237_PWR_LDO_RUN_1_52V  = 7,     /**< \brief 1.52V Ƭ LDOѹ */
    AMHW_ZLG237_PWR_LDO_RUN_1_56V  = 8,     /**< \brief 1.56V Ƭ LDOѹ */
    AMHW_ZLG237_PWR_LDO_RUN_1_60V  = 9,     /**< \brief 1.60V Ƭ LDOѹ */
    AMHW_ZLG237_PWR_LDO_RUN_1_64V  = 10,    /**< \brief 1.64V Ƭ LDOѹ */
    AMHW_ZLG237_PWR_LDO_RUN_1_68V  = 11,    /**< \brief 1.68V Ƭ LDOѹ */
} amhw_zlg237_pwr_ldo_run_t;
/**
 * \brief ƬLDO ģʽѹƼĴ
 *
 * \param[in] p_hw_pwr   ָԴƼĴָ
 * \param[in] status_flagԴ״̬λֵΪ amhw_zlg237_pwr_ldo_run_t һö
 *
 * \note ĴλоƬڲԼĴֹıʼֵоƬܿ쳣
 *
 * \return   AM_OK   :óɹ
 *           AM_ERRORʧܣ
 */
am_static_inline
int amhw_zlg237_pwr_ldo_run_set (
    amhw_zlg237_pwr_t *p_hw_pwr, amhw_zlg237_pwr_ldo_run_t ldo_run_vol)
{
    if((ldo_run_vol & 0xful) <= AMHW_ZLG237_PWR_LDO_RUN_1_68V) {

        /* ݲΪöͱܽжӦ */
        p_hw_pwr->ldo = (p_hw_pwr->ldo & (~(0xful << 0))) |
                        ((ldo_run_vol & 0xful) << 0) ;

        return AM_OK;
    } else {

        return AM_ERROR;
    }
}

/**
 * \brief MCUSTOPģʽ£ƬLDOѹ
 */
typedef enum amhw_zlg237_pwr_ldo_stop_set {
    AMHW_ZLG237_PWR_STOP_1_20V  = 0,     /**< \brief 1.20V Ƭ LDOѹ */
    AMHW_ZLG237_PWR_STOP_1_24V  = 1,     /**< \brief 1.24V Ƭ LDOѹ */
    AMHW_ZLG237_PWR_STOP_1_28V  = 2,     /**< \brief 1.28V Ƭ LDOѹ */
    AMHW_ZLG237_PWR_STOP_1_32V  = 3,     /**< \brief 1.32V Ƭ LDOѹ */
    AMHW_ZLG237_PWR_STOP_1_36V  = 4,     /**< \brief 1.36V Ƭ LDOѹ */
    AMHW_ZLG237_PWR_STOP_1_40V  = 5,     /**< \brief 1.40V Ƭ LDOѹ */
    AMHW_ZLG237_PWR_STOP_1_44V  = 6,     /**< \brief 1.44V Ƭ LDOѹ */
    AMHW_ZLG237_PWR_STOP_1_52V  = 7,     /**< \brief 1.52V Ƭ LDOѹ */
    AMHW_ZLG237_PWR_STOP_1_56V  = 8,     /**< \brief 1.56V Ƭ LDOѹ */
    AMHW_ZLG237_PWR_STOP_1_60V  = 9,     /**< \brief 1.60V Ƭ LDOѹ */
    AMHW_ZLG237_PWR_STOP_1_64V  = 10,    /**< \brief 1.64V Ƭ LDOѹ */
    AMHW_ZLG237_PWR_STOP_1_68V  = 11,    /**< \brief 1.68V Ƭ LDOѹ */
} amhw_zlg237_pwr_ldo_stop_set_t;
/**
 * \brief ƬLDO STOPģʽģʽѹƼĴ
 *
 * \param[in] p_hw_pwr   ָԴƼĴָ
 * \param[in] status_flagԴ״̬λ
 *                         ֵΪ amhw_zlg237_pwr_ldo_stop_set_t һö
 *
 * \note ĴλоƬڲԼĴֹıʼֵоƬܿ쳣
 *
 * \return   AM_OK   :óɹ
 *           AM_ERRORʧܣ
 */
am_static_inline
int amhw_zlg237_pwr_ldo_stop_set (
    amhw_zlg237_pwr_t *p_hw_pwr, amhw_zlg237_pwr_ldo_stop_set_t ldo_stop_vol)
{
    if((ldo_stop_vol & 0xful) <= AMHW_ZLG237_PWR_STOP_1_68V) {

        /* ݲΪöͱܽжӦ */
        p_hw_pwr->ldo_stop = (p_hw_pwr->ldo_stop & (~(0xful << 0))) |
                             ((ldo_stop_vol & 0xful) << 0) ;

        return AM_OK;
    }
    return AM_ERROR;
}

/**
 * @}
 */
#ifdef __cplusplus
}
#endif

#endif /* __AMHW_ZLG237_PWR_H */

/* end of file */
