/*******************************************************************************
*                                 AMetal
*                       ----------------------------
*                       innovating embedded platform
*
* Copyright (c) 2001-2018 Guangzhou ZHIYUAN Electronics Co., Ltd.
* All rights reserved.
*
* Contact information:
* web site:    http://www.zlg.cn/
*******************************************************************************/

/**
 * \file
 * \brief Դӿ
 *
 * \internal
 * \par History
 * - 1.00 17-04-17  nwt, first implementation
 * \endinternal
 */

#ifndef __AMHW_ZLG_PWR_H
#define __AMHW_ZLG_PWR_H

#ifdef __cplusplus
extern "C" {
#endif

#include "am_types.h"

/**
 * \addtogroup amhw_zlg_if_pwr
 * \copydoc amhw_zlg_pwr.h
 * @{
 */

/**
 * \name λ뻽λ
 * @{
 */

#define AM_ZLG_STANDBY_FLAG_CLEAR  (1UL << 3)
#define AM_ZLG_WAKEUP_FLAG_CLEAR   (1UL << 2)

/** @} */

/**
 * \name ˯ģʽ
 * @{
 */

#define AM_ZLG_PDDS_STANDBY_MODE  1
#define AM_ZLG_PDDS_STOP_MODE     0

/** @} */

/**
 * \name ԴѹPVDʹ
 * @{
 */
#define AM_ZLG_PVDE_ENABLE        1
#define AM_ZLG_PVDE_DISABLE       0

/** @} */

/**
 * \name BKPݼĴʹ
 * @{
 */
#define AM_ZLG_BKP_ENABLE         1
#define AM_ZLG_BKP_DISABLE        0

/** @} */

/**
 * \name WAKE_UPʹ
 * @{
 */
#define AM_ZLG_WAKEUP_ENABLE      1
#define AM_ZLG_WAKEUP_DISABLE     0

/** @} */

/**
  * \brief ϵͳƼĴṹ
  */
typedef struct amhw_zlg_pwr {
    __IO  uint32_t cr;          /**< \brief ԴƼĴ */
    __IO  uint32_t csr;         /**< \brief Դ/״̬Ĵ */
} amhw_zlg_pwr_t;

/**
 * \brief PVDƽѡ
 */
typedef enum amhw_zlg_pvd_lever_set {
    AMHW_ZLG_PVD_LEVER_1_8V  = 0,  /**< \brief 1.8V PVDѹ */
    AMHW_ZLG_PVD_LEVER_2_1V  = 1,  /**< \brief 2.1V PVDѹ */
    AMHW_ZLG_PVD_LEVER_2_4V  = 2,  /**< \brief 2.4V PVDѹ */
    AMHW_ZLG_PVD_LEVER_2_7V  = 3,  /**< \brief 2.7V PVDѹ */
    AMHW_ZLG_PVD_LEVER_3_0V  = 4,  /**< \brief 3.0V PVDѹ */
    AMHW_ZLG_PVD_LEVER_3_3V  = 5,  /**< \brief 3.3V PVDѹ */
    AMHW_ZLG_PVD_LEVER_3_6V  = 6,  /**< \brief 3.6V PVDѹ */
    AMHW_ZLG_PVD_LEVER_3_9V  = 7,  /**< \brief 3.9V PVDѹ */
    AMHW_ZLG_PVD_LEVER_4_2V  = 8,  /**< \brief 4.2V PVDѹ */
    AMHW_ZLG_PVD_LEVER_4_5V  = 9,  /**< \brief 4.5V PVDѹ */
    AMHW_ZLG_PVD_LEVER_4_8V  = 10, /**< \brief 4.8V PVDѹ */
} amhw_zlg_pvd_lever_set_t;

/**
 * \brief Դ״̬־
 */
typedef enum amhw_zlg_pwr_status_flag {
    AMHW_ZLG_PVD_OUTPUT_FLAG  = (1UL << 2),  /**< \brief PVD־λ */
    AMHW_ZLG_PVD_SBF_FLAG     = (1UL << 1),  /**< \brief ־λ */
    AMHW_ZLG_PVD_WUF_FLAG     = (1UL << 0),  /**< \brief ѱ־λ */
} amhw_zlg_pwr_status_flag_t;

/**
 * \brief ȵģʽѡ
 *
 * \param[in] pdds_mode  ȵģʽ,ֵΪ  AM_ZLG_PDDS_*_MODE һ
 *                         (# AM_ZLG_PDDS_STOP_MODE)
 * \param[in] p_hw_pwr   ָԴƼĴָ
 *
 * \return 
 */
am_static_inline
void amhw_zlg_pwr_pdds_mode_set (amhw_zlg_pwr_t *p_hw_pwr, uint8_t pdds_mode)
{
    AM_BITS_SET(p_hw_pwr->cr, 0, 2, (pdds_mode << 1));
}

/**
 * \brief PVDѹʹ
 *
 * \param[in] p_hw_pwr    ָԴƼĴָ
 * \param[in] pvd_enable  ѹʹܣֵΪ  AM_ZLG_PVDE_* һ
 *                           (# AM_ZLG_PVDE_ENABLE)
 *
 * \return 
 */
am_static_inline
void amhw_zlg_pwr_pvd_detect_enable (amhw_zlg_pwr_t *p_hw_pwr, uint8_t pvd_enable)
{
    AM_BIT_MODIFY(p_hw_pwr->cr, 4, pvd_enable);
}

/**
 * \brief ʱݼĴʹ
 *
 * \param[in] p_hw_pwr    ָԴƼĴָ
 * \param[in] bkp_enable  ѹʹܣֵΪ  AM_ZLG_BKP_* һ
 *                           (# AM_ZLG_BKP_ENABLE)
 *
 * \return 
 */
am_static_inline
void amhw_zlg_pwr_bkp_access_enable (amhw_zlg_pwr_t *p_hw_pwr, uint8_t bkp_enable)
{
    AM_BIT_MODIFY(p_hw_pwr->cr, 8, bkp_enable);
}

/**
 * \brief ־λ뻽λ
 *
 * \param[in] p_hw_pwr    ָԴƼĴָ
 * \param[in] status_flg  ־λѡֵΪ  AM_ZLG_*_FLAG_CLEAR һ
 *                           (# AM_ZLG_STANDBY_FLAG_CLEAR)
 *
 * \return 
 */
am_static_inline
void amhw_zlg_pwr_stauts_flag_clear (amhw_zlg_pwr_t *p_hw_pwr, uint8_t status_flg)
{
    AM_BIT_SET_MASK(p_hw_pwr->cr, status_flg);
}

/**
 * \brief PVѹѡ
 *
 * \param[in] p_hw_pwr ָԴƼĴָ
 * \param[in] pvd_leverѹⷶΧѡ, ֵΪ amhw_zlg_pvd_lever_set_t һö͵ֵ
 *
 * \return 
 */
am_static_inline
void amhw_zlg_pvd_lever_set (amhw_zlg_pwr_t *p_hw_pwr, uint8_t pvd_lever)
{
    AM_BITS_SET(p_hw_pwr->cr, 9, 4, pvd_lever);
}

/**
 * \brief ʹWAKE_UP
 *
 * \param[in] p_hw_pwr ָԴƼĴָ
 * \param[in] wakeup_enʹֵΪ AM_ZLG_WAKEUP_* һ꣬
 *                      (# AM_ZLG_WAKEUP_ENABLE)
 *
 * \return 
 */
am_static_inline
void amhw_zlg_wake_up_enable (amhw_zlg_pwr_t *p_hw_pwr, uint8_t wakeup_en)
{
    p_hw_pwr->csr |= (wakeup_en << 8);
}

/**
 * \brief ȡԴ״̬λ
 *
 * \param[in] p_hw_pwr   ָԴƼĴָ
 * \param[in] status_flagԴ״̬λֵΪ amhw_zlg_pwr_status_flag_t һö
 *
 * \return ֵΪ0ʱñ־λ1
 */
am_static_inline
uint32_t amhw_zlg_pwr_csr_status_get (amhw_zlg_pwr_t *p_hw_pwr, uint8_t status_flag)
{
    return p_hw_pwr->csr & status_flag;
}

/**
 * @}
 */

#ifdef __cplusplus
}
#endif

#endif /* __AMHW_ZLG_PWR_H */

/* end of file */
