/*******************************************************************************
*                                 AMetal
*                       ----------------------------
*                       innovating embedded platform
*
* Copyright (c) 2001-2018 Guangzhou ZHIYUAN Electronics Co., Ltd.
* All rights reserved.
*
* Contact information:
* web site:    http://www.zlg.cn/
*******************************************************************************/

/**
 * \file
 * \brief
 *
 * \internal
 * \par Modification history
 * - 1.00 19-06-27  yrh, first implementation
 * \endinternal
 */

#ifndef __AMHW_ZLG237_RCC_H
#define __AMHW_ZLG237_RCC_H

#ifdef __cplusplus
extern "C" {
#endif

#include "am_types.h"
#include "zlg237_periph_map.h"

/*
 * \brief ṹεĿʼ
 */
#if defined(__CC_ARM)
  #pragma push
  #pragma anon_unions
#elif defined(__ICCARM__)
  #pragma language=extended
#elif defined(__GNUC__)
#elif defined(__TMS470__)
#elif defined(__TASKING__)
  #pragma warning 586
#else
  #warning Not supported compiler t
#endif

/**
 * \addtogroup amhw_zlg237_if_rcc
 * \copydoc amhw_zlg237_rcc.h
 * @{
 */

/**
 * \brief CRCĴṹ
 */
typedef struct amhw_zlg237_rcc {
  __IO uint32_t cr;        /**< \brief ʱӿƼĴ */
  __IO uint32_t cfgr;      /**< \brief ʱüĴ */
  __IO uint32_t cir;       /**< \brief ʱжϼĴ */
  __IO uint32_t apb2rstr;  /**< \brief APB2踴λĴ */
  __IO uint32_t apb1rstr;  /**< \brief APB1踴λĴ */
  __IO uint32_t ahbenr;    /**< \brief AHBʱʹܼĴ */
  __IO uint32_t apb2enr;   /**< \brief APB2ʱʹܼĴ */
  __IO uint32_t apb1enr;   /**< \brief APB1ʱʹܼĴ */
  __IO uint32_t bdcr;      /**< \brief ݿƼĴ */
  __IO uint32_t csr;       /**< \brief ״̬Ĵ */
  __IO uint32_t wts;       /**< \brief hseʱԴƼĴ */
  __I  uint32_t reserve;   /**< \brief  */
  __IO uint32_t lpclk_ctl; /**< \brief ͹ʱüĴ */

} amhw_zlg237_rcc_t;

/**
 * \brief ϵͳʱԴ
 */
typedef enum {
    AMHW_ZLG237_SYSCLK_HSI,       /**< \brief HSI Ϊϵͳʱ */
    AMHW_ZLG237_SYSCLK_HSE,       /**< \brief HSE Ϊϵͳʱ */
    AMHW_ZLG237_SYSCLK_PLL,       /**< \brief PLL Ϊϵͳʱ */
    AMHW_ZLG237_SYSCLK_LSI_OR_LSE /**< \brief LSI   LSE Ϊϵͳʱ */
}amhw_zlg237_sys_clk_src;

/**
 * \brief ϵͳʱ
 */
typedef enum {
    AMHW_ZLG237_LCLK_LSI,  /**< \brief ڲʱ */
    AMHW_ZLG237_LCLK_LSE,  /**< \brief ⲿʱ */
}amhw_zlg237_lclk;


/**
 * \brief RTCʱԴ
 */
typedef enum {
    AMHW_ZLG237_RTCCLK_LSE = 1,       /**< \brief PLL Ϊϵͳʱ */
    AMHW_ZLG237_RTCCLK_LSI = 2,       /**< \brief HSI   6 ƵΪϵͳʱ */
    AMHW_ZLG237_RTCCLK_HSE_DIV128 =3, /**< \brief HSE Ϊϵͳʱ */
}amhw_zlg237_rtc_clk_src;

/**
 * \brief PLLʱԴ
 */
typedef enum {
    AMHW_ZLG237_PLLCLK_HSI = 0,  /**< \brief HSI ΪPLLʱ */
    AMHW_ZLG237_PLLCLK_HSE = 1,  /**< \brief HSE ΪPLLʱ */
}amhw_zlg237_pll_clk_src;

/**
 * \brief usb Ƶϵ
 */
typedef enum {
    AMHW_ZLG237_PLL_USB_DIV1P5 = 0, /**< \brief PLL 1.5 ƵΪusbʱ */
    AMHW_ZLG237_PLL_USB_DIV1   = 1, /**< \brief PLL 1 ƵΪusbʱ */
}amhw_zlg237_pll_usb_clk_div;

/**
 * \brief ʱжϱ־
 */
typedef enum {
    AMHW_ZLG237_CLKINT_INDEX_LSI = 0,  /**< \brief LSI־ */
    AMHW_ZLG237_CLKINT_INDEX_LSE = 1,  /**< \brief LSE־ */
    AMHW_ZLG237_CLKINT_INDEX_HSI,      /**< \brief HSI־ */
    AMHW_ZLG237_CLKINT_INDEX_HSE,      /**< \brief HSE־ */
    AMHW_ZLG237_CLKINT_INDEX_PLL,      /**< \brief PLL־ */
    AMHW_ZLG237_CLKINT_INDEX_CSSF = 7, /**< \brief CSSF(ʱӰȫϵͳж)־ */
}amhw_zlg237_clk_int_index;

/**
 * \brief APB2ö
 */
typedef enum {
    AMHW_ZLG237_RCC_APB2_AFIO   = 0,  /**< \brief AFIO */
    AMHW_ZLG237_RCC_APB2_IOPA   = 2,  /**< \brief IOPA */
    AMHW_ZLG237_RCC_APB2_IOPB   = 3,  /**< \brief IOPB */
    AMHW_ZLG237_RCC_APB2_IOPC   = 4,  /**< \brief IOPC */
    AMHW_ZLG237_RCC_APB2_IOPD   = 5,  /**< \brief IOPD */
    AMHW_ZLG237_RCC_APB2_ADC1   = 9,  /**< \brief ADC1 */
    AMHW_ZLG237_RCC_APB2_ADC2   = 10, /**< \brief ADC2 */
    AMHW_ZLG237_RCC_APB2_TIM1   = 11, /**< \brief TIM1 */
    AMHW_ZLG237_RCC_APB2_SPI1   = 12, /**< \brief SPI1 */
    AMHW_ZLG237_RCC_APB2_UART1  = 14, /**< \brief UART1 */
}amhw_zlg237_apb2_peripheral;

/**
 * \brief APB1ö
 */
typedef enum {
    AMHW_ZLG237_RCC_APB1_TIM2   = 0,    /**< \brief TIM2ʱ */
    AMHW_ZLG237_RCC_APB1_TIM3   = 1,    /**< \brief TIM3ʱ */
    AMHW_ZLG237_RCC_APB1_TIM4   = 2,    /**< \brief TIM4ʱ */
    AMHW_ZLG237_RCC_APB1_WWDG   = 11,   /**< \brief WWDGڿŹ */
    AMHW_ZLG237_RCC_APB1_SPI2   = 14,   /**< \brief SPI2 */
    AMHW_ZLG237_RCC_APB1_USART2  = 17,  /**< \brief USART2 */
    AMHW_ZLG237_RCC_APB1_USART3  = 18,  /**< \brief USART3 */
    AMHW_ZLG237_RCC_APB1_I2C1   = 21,   /**< \brief I2C1 */
    AMHW_ZLG237_RCC_APB1_I2C2   = 22,   /**< \brief I2C2 */
    AMHW_ZLG237_RCC_APB1_USB    = 23,   /**< \brief USB */
    AMHW_ZLG237_RCC_APB1_CAN    = 25,   /**< \brief CAN */
    AMHW_ZLG237_RCC_APB1_BKP    = 27,   /**< \brief BKP */
    AMHW_ZLG237_RCC_APB1_PWR    = 28,   /**< \brief Դӿ */
}amhw_zlg237_apb1_peripheral;

/**
 * \brief AHBö
 */
typedef enum {
    AMHW_ZLG237_RCC_AHB_DMA   = 0,   /**< \brief DMA */
    AMHW_ZLG237_RCC_AHB_SRAM  = 2,   /**< \brief SRAM */
    AMHW_ZLG237_RCC_AHB_FLITF = 4,   /**< \brief FLITF */
    AMHW_ZLG237_RCC_AHB_CRC   = 6,   /**< \brief CRC */
}amhw_zlg237_ahb_peripheral;

/**
 * \brief λ־ö
 */
typedef enum {
    AMHW_ZLG237_RCC_CSR_PINRSTF   = (1ul << 26),      /**< \brief NRSTܽŸλ־ */
    AMHW_ZLG237_RCC_CSR_PORRSTF   = (1ul << 27),      /**< \brief ϵ/縴λ־ */
    AMHW_ZLG237_RCC_CSR_SFTRSTF   = (1ul << 28),      /**< \brief λ־ */
    AMHW_ZLG237_RCC_CSR_IWDGRSTF  = (1ul << 29),      /**< \brief Źλ־ */
    AMHW_ZLG237_RCC_CSR_WWDGRSTF  = (1ul << 30),      /**< \brief ڿŹλ־ */
    AMHW_ZLG237_RCC_CSR_LPWRRSTF  = (int)(1ul << 31), /**< \brief ͹Ĺλ־ */
}amhw_zlg237_reset_flag_t;


/**
 * \brief ڲʱʹ
 *
 * \return 
 *
 */
am_static_inline
void amhw_zlg237_rcc_hsion_enable (void)
{
    ZLG237_RCC->cr |= 1ul;
}

/**
 * \brief ڲʱӾ־ȡ
 *
 * \retval TRUE  : ڲ 8MHzʱûо
 * \retval FALSE : ڲ 8MHzʱӾ
 */
am_static_inline
am_bool_t amhw_zlg237_rcc_hsirdy_read (void)
{
    return (am_bool_t)((ZLG237_RCC->cr & 0x2ul) ? AM_TRUE : AM_FALSE);
}

/**
 * \brief ⲿʱ(HSE )ʹ
 *
 * \return 
 */
am_static_inline
void amhw_zlg237_rcc_hseon_enable (void)
{
    ZLG237_RCC->cr |= (1ul << 16);
}

/**
 * \brief ⲿʱӾ־ȡ
 *
 * \retval TRUE  : ⲿʱûо
 * \retval FALSE : ⲿʱӾ
 */
am_static_inline
am_bool_t amhw_zlg237_rcc_hserdy_read (void)
{
    return (am_bool_t)((ZLG237_RCC->cr & (1ul << 17)) ? AM_TRUE : AM_FALSE);
}

/**
 * \brief ⲿʱ·ʹ
 *
 * \return 
 */
am_static_inline
void amhw_zlg237_rcc_hsebyp_enable (void)
{
    ZLG237_RCC->cr |= (1ul << 18);
}

/**
 * \brief ʱӰȫϵͳʹ
 *
 * \details ⲿ 8-24MHzʱӼ
 *
 * \return 
 */
am_static_inline
void amhw_zlg237_rcc_ccson_enable (void)
{
    ZLG237_RCC->cr |= (1ul << 19);
}


/**
 * \brief PLL ʹ
 *
 * \details ֹͣģʽʱλӲ㡣
 *           PLL ʱӱѡҪΪϵͳʱʱ,
 *          λܱ㡣
 *
 * \return 
 */
am_static_inline
void amhw_zlg237_rcc_pll_enable (void)
{
    ZLG237_RCC->cr |= (1ul << 24);
}

/**
 * \brief PLL 
 *
 * \details ֹͣģʽʱλӲ㡣
 *           PLL ʱӱѡҪΪϵͳʱʱ,
 *          λܱ㡣
 *
 * \return 
 */
am_static_inline
void amhw_zlg237_rcc_pll_disable (void)
{
    ZLG237_RCC->cr &= ~(1ul << 24);
}

/**
 * \brief PLL ʱӾ־ȡ
 *
 * \retval TRUE  : PLL δ
 * \retval FALSE : PLL 
 */
am_static_inline
am_bool_t amhw_zlg237_rcc_pllrdy_read (void)
{
    return (am_bool_t)((ZLG237_RCC->cr & (1ul << 25)) ? AM_TRUE : AM_FALSE);
}

/**
 * \brief ϵͳʱл
 *
 * \param[in]  src ϵͳʱԴ
 *
 * \return 
 */
am_static_inline
void amhw_zlg237_rcc_sys_clk_set (amhw_zlg237_sys_clk_src src)
{

    ZLG237_RCC->cfgr = ((src & 0x3ul) << 0) |
                              (ZLG237_RCC->cfgr & ~(0x3ul << 0));
}

/**
 * \brief ǰϵͳʱӻȡ
 *
 * \return ǰϵͳʱԴ
 */
am_static_inline
amhw_zlg237_sys_clk_src amhw_zlg237_rcc_sys_clk_get (void)
{

    return (amhw_zlg237_sys_clk_src)((ZLG237_RCC->cfgr >> 2ul) & 0x3ul);
}

/**
 * \brief AHB ԤƵ
 *
 * \param[in]  div ԤƵֵ
 *
 *       - 0-7  Ƶ
 *       - 8    2Ƶ
 *       - 9    4Ƶ
 *       - 10   8Ƶ
 *       - 11   16Ƶ
 *       - 12   64Ƶ
 *       - 13   128Ƶ
 *       - 14   256Ƶ
 *       - 15   512Ƶ
 *
 *
 * \return 
 */
am_static_inline
void amhw_zlg237_rcc_ahb_div_set (uint8_t div)
{

    ZLG237_RCC->cfgr = ((div & 0xful) << 4) |
                              (ZLG237_RCC->cfgr & ~(0xful << 4));
}

/**
 * \brief   APB ԤƵAPB1
 *
 * \param[in]  div ԤƵֵ
 *
 *       - 0-3  Ƶ
 *       - 4    2Ƶ
 *       - 5    4Ƶ
 *       - 6    8Ƶ
 *       - 7    16Ƶ
 *
 * \return 
 */
am_static_inline
void amhw_zlg237_rcc_apb1_div_set (uint8_t div)
{

    ZLG237_RCC->cfgr = ((div & 0x7ul) << 8) |
                              (ZLG237_RCC->cfgr & ~(0x7ul << 8));
}

/**
 * \brief ȡ  APB ԤƵAPB1
 *
 * \return ԤƵֵ
 *       - 0-3  Ƶ
 *       - 4    2Ƶ
 *       - 5    4Ƶ
 *       - 6    8Ƶ
 *       - 7    16Ƶ
 */
am_static_inline
uint8_t amhw_zlg237_rcc_apb1_div_get (void)
{

    return (uint8_t)((ZLG237_RCC->cfgr >> 8) & 0x7ul);
}

/**
 * \brief   APB ԤƵAPB2
 *
 * \param[in]  div ԤƵֵ
 *
 *       - 0-3  Ƶ
 *       - 4    2Ƶ
 *       - 5    4Ƶ
 *       - 6    8Ƶ
 *       - 7    16Ƶ
 *
 * \return 
 */
am_static_inline
void amhw_zlg237_rcc_apb2_div_set (uint8_t div)
{

    ZLG237_RCC->cfgr = ((div & 0x7ul) << 11) |
                              (ZLG237_RCC->cfgr & ~(0x7ul << 11));
}

/**
 * \brief ȡ  APB ԤƵAPB2
 *
 * \return ԤƵֵ
 *       - 0-3  Ƶ
 *       - 4    2Ƶ
 *       - 5    4Ƶ
 *       - 6    8Ƶ
 *       - 7    16Ƶ
 */
am_static_inline
uint8_t amhw_zlg237_rcc_apb2_div_get (void)
{

    return (uint8_t)((ZLG237_RCC->cfgr >> 11) & 0x7ul);
}


/**
 * \brief  ADC ԤƵ
 *
 * \param[in]  div ԤƵֵ
 *
 *       - 0    2 Ƶ
 *       - 1    4 Ƶ
 *       - 2    8 Ƶ
 *       - 3    16Ƶ
 *
 * \return 
 */
am_static_inline
void amhw_zlg237_rcc_adc_div_set (uint8_t div)
{

    ZLG237_RCC->cfgr = ((div & 0x3ul) << 14) |
                              (ZLG237_RCC->cfgr & ~(0x3ul << 14));
}

/**
 * \brief ȡ ADC ԤƵ
 *
 * \return ԤƵֵ
 *       - 0    2 Ƶ
 *       - 1    4 Ƶ
 *       - 2    8 Ƶ
 *       - 3    16Ƶ
 */
am_static_inline
uint8_t amhw_zlg237_rcc_adc_div_get (void)
{

    return (uint8_t)((ZLG237_RCC->cfgr >> 14) & 0x3ul);
}


/**
 * \brief PLLʱѡ
 *
 * \param[in]  src PLLʱԴ
 *
 * \return 
 */
am_static_inline
void amhw_zlg237_rcc_pll_clk_set (amhw_zlg237_pll_clk_src src)
{

    ZLG237_RCC->cfgr = ((src & 0x3ul) << 16) |
                              (ZLG237_RCC->cfgr & ~(0x3ul << 16));
}

/**
 * \brief ǰPLLʱӻȡ
 *
 * \return ǰPLLʱԴ
 */
am_static_inline
amhw_zlg237_pll_clk_src amhw_zlg237_rcc_pll_clk_get (void)
{

    return (amhw_zlg237_pll_clk_src)((ZLG237_RCC->cfgr >> 16ul) & 0x3ul);
}

/**
 * \brief ѡ HSE ƵΪ PLL 
 *
 * \param[in]  div Ƶֵ
 *
 *      - 0   Ƶ
 *      - 1   2Ƶ
 *
 * \return 
 */
am_static_inline
void amhw_zlg237_rcc_hseforpll_set (uint8_t div)
{

    ZLG237_RCC->cfgr = ((div & 0x1ul) << 17) |
                              (ZLG237_RCC->cfgr & ~(0x1ul << 17));
}


///**
// * \brief PLLƵϵ
// *
// * \param[in]  div Ƶֵ
// *
// * \return 
// */
//am_static_inline
//void amhw_zlg237_rcc_plldm_set (uint8_t div)
//{
//    ZLG237_RCC->cr = ((div & 0x7ul) << 20) |
//                            (ZLG237_RCC->cr & ~(0x7ul << 20));
//}

/**
 * \brief PLLƵϵ
 *
 * \param[in]  mul Ƶֵ
 *
 * \return 
 */
am_static_inline
void amhw_zlg237_rcc_plldn_set (uint8_t mul)
{

    ZLG237_RCC->cfgr = ((mul & 0xful) << 18) |
                            (ZLG237_RCC->cfgr & ~(0xful << 18));
}


/**
 * \brief ѡ USB ԤƵ
 *
 * \param[in]  div Ƶֵ
 *
 *      - AMHW_ZLG237_PLL_USB_DIV1P5 PLL ʱ1.5ƵΪ USB ʱ
 *      - AMHW_ZLG237_PLL_USB_DIV1    PLL ʱֱΪ USBʱ
 *
 * \return 
 */
am_static_inline
void amhw_zlg237_rcc_pllforusb_set (amhw_zlg237_pll_usb_clk_div index)
{

    ZLG237_RCC->cfgr = ((index & 0x1ul) << 22) |
                              (ZLG237_RCC->cfgr & ~(0x1ul << 22));
}

/**
 * \brief USB ԤƵȡ
 *
 * \retval  0   PLL ʱֱΪ USB ʱ
 * \retval  1   PLL ʱ2 ƵΪ USBʱ
 * \retval  2   PLL ʱ3 ƵΪ USBʱ
 * \retval  3   PLL ʱ4 ƵΪ USBʱ
 */
am_static_inline
uint8_t amhw_zlg237_rcc_pllforusb_get (void)
{

    return (uint8_t)((ZLG237_RCC->cfgr >> 22ul) & 0x1ul);
}

/**
 * \brief ΢ʱѡ
 *
 * \details 1.ʱл MCO ʱԴʱܻᱻض,
 *          2.ϵͳʱΪ MCO ܽʱ뱣֤ʱƵ
 *            ʲ 50MHzIOƵʣ
 *
 * \param[in]  src ʱԴ
 *
 *      - 0-3  ûʱ
 *      - 4    ϵͳʱӣSYSCLK
 *      - 5    HSI ʱ
 *      - 6    HSE ʱ
 *      - 7    PLL ʱ2 Ƶ
 *
 * \return 
 */
am_static_inline
void amhw_zlg237_rcc_mco_src_set (uint8_t src)
{

    ZLG237_RCC->cfgr = ((src & 0x7ul) << 24ul) |
                              (ZLG237_RCC->cfgr & ~(0x7ul << 24ul));
}

/**
 * \brief ѡ HSI ƵΪ PLL οʱ
 *
 * \param[in]  div Ƶֵ
 *
 *      - 0   2Ƶ
 *      - 1   Ƶ
 *
 * \return 
 */
am_static_inline
void amhw_zlg237_rcc_pllcrpre_set (uint8_t div)
{

    ZLG237_RCC->cfgr = ((div & 0x1ul) << 31ul) |
                              (ZLG237_RCC->cfgr & ~(0x1ul << 31ul));
}


/**
 * \brief жʹ
 *
 * \param[in]  index ־
 *
 * \return 
 */
am_static_inline
void amhw_zlg237_rcc_clkint_enable (amhw_zlg237_clk_int_index index)
{

    ZLG237_RCC->cir |= ((index & 0x1ful) << 8);
}

/**
 * \brief жϽ
 *
 * \param[in]  index ־
 *
 * \return 
 */
am_static_inline
void amhw_zlg237_rcc_clkint_disable (amhw_zlg237_clk_int_index index)
{

    ZLG237_RCC->cir &= ~((index & 0x1ful) << 8);
}

/**
 * \brief ȡжϱ־
 *
 * \return жϱ־
 */
am_static_inline
amhw_zlg237_clk_int_index amhw_zlg237_rcc_clkint_flag_read (void)
{

    return (amhw_zlg237_clk_int_index)(ZLG237_RCC->cir & 0x9ful);
}

/**
 * \brief жϱ־
 *
 * \return 
 */
am_static_inline
void amhw_zlg237_rcc_clkint_flag_clear (amhw_zlg237_clk_int_index index)
{

    ZLG237_RCC->cir |= ((index & 0x9ful) << 16);
}

/**
 * \brief λAPB2
 *
 * \param[in]  reset Ҫλ
 *
 * \return 
 */
am_static_inline
void amhw_zlg237_rcc_apb2_reset (amhw_zlg237_apb2_peripheral reset)
{

    ZLG237_RCC->apb2rstr |= (1ul << reset);
}

/**
 * \brief ֹͣλAPB2
 *
 * \param[in]  reset Ҫλ
 *
 * \return 
 */
am_static_inline
void amhw_zlg237_rcc_apb2_reset_stop (amhw_zlg237_apb2_peripheral reset)
{

    ZLG237_RCC->apb2rstr &= ~(1ul << reset);
}

/**
 * \brief λAPB1
 *
 * \param[in]  reset Ҫλ
 *
 * \return 
 */
am_static_inline
void amhw_zlg237_rcc_apb1_reset (amhw_zlg237_apb1_peripheral reset)
{

    ZLG237_RCC->apb1rstr |= (1ul << reset);
}

/**
 * \brief ֹͣλAPB1
 *
 * \param[in]  reset Ҫλ
 *
 * \return 
 */
am_static_inline
void amhw_zlg237_rcc_apb1_reset_stop (amhw_zlg237_apb1_peripheral reset)
{

    ZLG237_RCC->apb1rstr &= ~(1ul << reset);
}

/**
 * \brief ʹAPB1
 *
 * \param[in]  peri Ҫʹܵ
 *
 * \return 
 */
am_static_inline
void amhw_zlg237_rcc_apb1_enable (amhw_zlg237_apb1_peripheral peri)
{

    ZLG237_RCC->apb1enr |= (1ul << peri);
}

/**
 * \brief APB1
 *
 * \param[in]  peri Ҫܵ
 *
 * \return 
 */
am_static_inline
void amhw_zlg237_rcc_apb1_disable (amhw_zlg237_apb1_peripheral peri)
{

    ZLG237_RCC->apb1enr &= ~(1ul << peri);
}

/**
 * \brief ʹAPB2
 *
 * \param[in]  peri Ҫʹܵ
 *
 * \return 
 */
am_static_inline
void amhw_zlg237_rcc_apb2_enable (amhw_zlg237_apb2_peripheral peri)
{

    ZLG237_RCC->apb2enr |= (1ul << peri);
}

/**
 * \brief APB2
 *
 * \param[in]  peri Ҫܵ
 *
 * \return 
 */
am_static_inline
void amhw_zlg237_rcc_apb2_disable (amhw_zlg237_apb2_peripheral peri)
{

    ZLG237_RCC->apb2enr &= ~(1ul << peri);
}

/**
 * \brief ʹAHB
 *
 * \param[in]  peri Ҫʹܵ
 *
 * \return 
 */
void amhw_zlg237_rcc_ahb_enable (amhw_zlg237_ahb_peripheral peri);

/**
 * \brief AHB
 *
 * \param[in]  peri Ҫܵ
 *
 * \return 
 *
 */
void amhw_zlg237_rcc_ahb_disable (amhw_zlg237_ahb_peripheral peri);

/**
 * \brief ʹLSI
 *
 * \return 
 */
am_static_inline
void amhw_zlg237_rcc_lsi_enable (void)
{

    ZLG237_RCC->csr |= (1ul << 0);
}

/**
 * \brief LSI
 *
 * \return 
 */
am_static_inline
void amhw_zlg237_rcc_lsi_disable (void)
{

    ZLG237_RCC->csr &= ~(1ul << 0);
}

/**
 * \brief ڲʱӾ־ȡ
 *
 * \retval TRUE  : ڲ 40kHz ʱӾ
 * \retval FALSE : ڲ 40kHz ʱδ
 */
am_static_inline
am_bool_t amhw_zlg237_rcc_lsirdy_read (void)
{
    return (am_bool_t)((ZLG237_RCC->csr & (1ul << 1)) ? AM_TRUE : AM_FALSE);
}

/**
 * \brief ȡλ־
 *
 * \return λ־
 */
am_static_inline
amhw_zlg237_reset_flag_t amhw_zlg237_rcc_reset_flag (void)
{

    return (amhw_zlg237_reset_flag_t)(ZLG237_RCC->csr & 0xfc000000);
}

/**
 * \brief λ־
 *
 * \return 
 */
am_static_inline
void amhw_zlg237_rcc_reset_flag_clear (void)
{

    ZLG237_RCC->csr |= (1ul << 24);
}

/**
 * \brief ⲿʹ
 *
 * \return 
 */
am_static_inline
void amhw_zlg237_rcc_lse_enable (void)
{

    ZLG237_RCC->bdcr |= (1ul << 0);
}
/**
 * \brief ⲿ
 *
 * \return 
 */
am_static_inline
void amhw_zlg237_rcc_lse_disable (void)
{
    ZLG237_RCC->bdcr &= ~(1ul << 0);
}

/**
 * \brief ⲿʱӾ־ȡ
 *
 * \retval TRUE  : ⲿ 32kHz ʱӾ
 * \retval FALSE : ⲿ 32kHz ʱδ
 */
am_static_inline
am_bool_t amhw_zlg237_rcc_lserdy_read (void)
{
    return (am_bool_t)((ZLG237_RCC->bdcr & (1ul << 1)) ? AM_TRUE : AM_FALSE);
}

/**
 * \brief ⲿʱ·״̬
 *
 * \return 
 */
am_static_inline
void amhw_zlg237_rcc_bdcr_lsebyp_set (uint32_t lsebyp)
{
    ZLG237_RCC->bdcr |= (lsebyp << 2);
}
/**
 * \brief ⲿʱ·״̬
 *
 * \retval TRUE  : LSE ʱӱ·
 * \retval FALSE : LSE ʱδ·
 */
am_static_inline
am_bool_t amhw_zlg237_rcc_bdcr_lsebyp_get (void)
{
    return (am_bool_t)((ZLG237_RCC->bdcr & 0x4ul) ? AM_TRUE : AM_FALSE);
}
/**
 * \brief RTCʱѡ
 *
 * \param[in]  src RTCʱԴ
 *
 * \return 
 */
am_static_inline
void amhw_zlg237_rcc_bdcr_rtc_clk_set (amhw_zlg237_rtc_clk_src src)
{

    ZLG237_RCC->bdcr = ((src & 0x3ul) << 8) |
                              (ZLG237_RCC->bdcr & ~(0x3ul << 8));
}

/**
 * \brief RTCʱӻȡ
 *
 * \retval  rtcʱԴȡ
 */
am_static_inline
amhw_zlg237_rtc_clk_src amhw_zlg237_rcc_bdcr_rtc_clk_get (void)
{
    return (amhw_zlg237_rtc_clk_src)((ZLG237_RCC->bdcr >> 8) & 0x3ul);
}

/**
 * \brief RTCʱʹ
 *
 * \return 
 */
am_static_inline
void amhw_zlg237_rcc_bdcr_rtc_enable (void)
{

    ZLG237_RCC->bdcr |= (1ul << 15);
}
/**
 * \brief RTCʱӽ
 *
 * \return 
 */
am_static_inline
void amhw_zlg237_rcc_bdcr_rtc_disable (void)
{

    ZLG237_RCC->bdcr &= ~(1ul << 15);
}

/**
 * \brief λ
 * \return 
 */
am_static_inline
void amhw_zlg237_rcc_bdcr_bdrst_reset (void)
{

    ZLG237_RCC->bdcr |= (1ul << 16);
}
/**
 * \brief λ
 * \return 
 */
am_static_inline
void amhw_zlg237_rcc_bdcr_bdrst_reset_end (void)
{

    ZLG237_RCC->bdcr &= ~(1ul << 16);
}

/**
 * \brief HSE ȴʱ
 *
 * \param[in]  value ֵ
 *
 * ȴʱ = value * 8 * hseʱ
 */
am_static_inline
void amhw_zlg237_rcc_hse_wt_set(uint16_t value)
{
    ZLG237_RCC->wts = ((value & 0x1ffful) << 0) |
                                 (ZLG237_RCC->wts & ~(0x1ffful << 0));
}

/**
 * \brief XCH_IOp[2:0]  HSE
 */
typedef enum amhw_zlg237_rcc_hse_iop_set {
    AMHW_ZLG237_RCC_HSE_IOP_20 = 0,/**< \brief ԼΪ20uAƼʹ0.5~3MHz */
    AMHW_ZLG237_RCC_HSE_IOP_30,    /**< \brief ԼΪ30uAƼʹ0.5~4MHz */
    AMHW_ZLG237_RCC_HSE_IOP_40,    /**< \brief ԼΪ40uAƼʹ0.5~5MHz */
    AMHW_ZLG237_RCC_HSE_IOP_50,    /**< \brief ԼΪ50uAƼʹ1~6MHz */
    AMHW_ZLG237_RCC_HSE_IOP_60,    /**< \brief ԼΪ60uAƼʹ2~12MHz */
    AMHW_ZLG237_RCC_HSE_IOP_80,    /**< \brief ԼΪ80uAƼʹ2~15MHz */
    AMHW_ZLG237_RCC_HSE_IOP_100,   /**< \brief ԼΪ100uAƼʹ3~20MHz */
    AMHW_ZLG237_RCC_HSE_IOP_200,   /**< \brief ԼΪ200uAƼʹ5~30MHz */
} amhw_zlg237_rcc_hse_iop_set_t;
/**
 * \brief XCH_IOp[2:0]  HSE
 *
 * \param[in] status_flagHSEã
 *                        ֵΪ amhw_zlg237_rcc_hse_iop_set_t һö
 *
 */
am_static_inline
void amhw_zlg237_rcc_hse_iop_set (amhw_zlg237_rcc_hse_iop_set_t hse_iop_set)
{
    ZLG237_RCC->wts = (ZLG237_RCC->wts & ~(0x7ul << 16)) |
                      ((hse_iop_set & 0x7ul) << 16) ;

}

/**
 * \brief HSE Ƭģbypass
 *
 *
 *  0 HSE ʱźžƬģʹá
 *  1 bypass ƬHSE ģ
 */
am_static_inline
void amhw_zlg237_rcc_hsenf_byp_cfg(uint8_t index)
{
    ZLG237_RCC->wts = ((index & 0x1ul) << 24) |
                                       (ZLG237_RCC->wts & ~(0x1ul << 24));
}

/**
 * \brief ϵͳʱѡ
 *
 * \param[in]  index : ʱ
 *
 *      - AMHW_ZLG237_LCLK_LSI   ڲLSIʱ
 *      - AMHW_ZLG237_LCLK_LSE   ⲿLSEʱ
 */
am_static_inline
void amhw_zlg237_rcc_lclk_sel(amhw_zlg237_lclk index)
{
    ZLG237_RCC->lpclk_ctl = ((index & 0x1ul) << 0) |
                                       (ZLG237_RCC->lpclk_ctl & ~(0x1ul << 0));
}

/**
 * \brief USB1.1ʱʹ
 *
 */
am_static_inline
void amhw_zlg237_rcc_usbclk_enable(void)
{
    ZLG237_RCC->lpclk_ctl |= ( 0x1ul << 8);
}

/**
 * \brief USB1.1ʱӽ
 *
 */
am_static_inline
void amhw_zlg237_rcc_usbclk_disable(void)
{
    ZLG237_RCC->lpclk_ctl &= ~( 0x1ul << 8);
}

/**
 * @}
 */
/*
 * \brief ṹεĽ
 */

#if defined(__CC_ARM)
  #pragma pop
#elif defined(__ICCARM__)
#elif defined(__GNUC__)
#elif defined(__TMS470__)
#elif defined(__TASKING__)
  #pragma warning restore
#else
  #warning Not supported compiler t
#endif

#ifdef __cplusplus
}
#endif /* __ZLG237_RCC_H */

#endif

/* end of file */
