/*******************************************************************************
*                                 AMetal
*                       ----------------------------
*                       innovating embedded platform
*
* Copyright (c) 2001-2018 Guangzhou ZHIYUAN Electronics Co., Ltd.
* All rights reserved.
*
* Contact information:
* web site:    http://www.zlg.cn/
*******************************************************************************/

/**
 * \file
 * \brief RTC drivers for standard interface
 *
 * \internal
 * \par Modification history
 * - 1.00 19-10-16  zp, first implementation
 * \endinternal
 */

#ifndef __AM_ZLG118_RTC_H
#define __AM_ZLG118_RTC_H

#ifdef __cplusplus
extern "C" {
#endif

#include "am_rtc.h"
#include "hw/amhw_zlg118_rtc.h"
#include "hw/amhw_zlg118_rcc.h"

/**
 * \addtogroup am_zlg118_if_rtc
 * \copydoc am_zlg118_rtc.h
 * @{
 */

#define  AM_ZLG118_RTC_CALLBACK_PERIOD  (1 << 0) /**< \brief RTCжϻص */
#define  AM_ZLG118_RTC_CALLBACK_ALARM   (1 << 1) /**< \brief RTCжϻص */

/**
 * \brief ZLG118 1Hzƽṹ
 */
typedef struct {
    am_bool_t enable;                 /* Ƿʹ1Hz
                                       * AM_TRUE  ʹ Ķõעͣ
                                       * AM_FALSE ܣЧ
                                       */
    float     ppm;                    /* ֵ-274.6 ~ 212.6ppmΧر*/
    am_bool_t adjust_enable;          /* ppmڷΧʱ£
                                       * ѡǷʱΪʱ
                                       *
                                       * AM_TRUE  ʱΪʱ
                                       * AM_FALSE رոʱΪʱ
                                       */
    amhw_zlg118_rtcclk_adjust adjust_clk; /* ʱӲʱѡ PCLKƵͬ  */
} am_zlg118_rtc_1hz_out_t;


/**
 * \brief ZLG118 RTC 豸Ϣ
 */
typedef struct am_zlg118_rtc_devinfo {
    uint32_t                  rtc_regbase;           /**< \brief RTC Ĵַ */
    amhw_zlg118_rtc_clk_src_t rtc_clk_sour;          /**< \brief RTC ʱԴ */
    int                       rtc_inum;              /**< \brief RTC жϺ */
    am_zlg118_rtc_1hz_out_t   ctrl_1hz_out;
    void                    (*pfn_plfm_init)(void);  /**< \brief ƽ̨ʼ */
    void                    (*pfn_plfm_deinit)(void);/**< \brief ƽ̨ʼ */
} am_zlg118_rtc_devinfo_t;

/**
 * \brief ZLG118 RTC 豸
 */
typedef struct am_zlg118_rtc_dev {
    am_rtc_serv_t                  rtc_serv;        /**< \brief RTC ׼ */
    uint8_t                        int_state;       /**< \brief жʹܱ־ */
    am_bool_t                      rtc_continue;    /**< \brief RTCǷ */
    am_pfnvoid_t                   pfn_callback[2]; /**< \brief ص */
    void                          *p_arg[2];        /**< \brief ص */
    const am_zlg118_rtc_devinfo_t *p_devinfo;       /**< \brief RTC豸Ϣ */
}am_zlg118_rtc_dev_t;

/**
 * \brief ȡ RTC ֵ
 *
 * \param[in]  handle RTC ׼
 * \param[in]  handle RTC ׼
 * \param[in] *sec    루0-59
 * \param[in] *min    ֣0-59
 * \param[in] *hour   ʱ0-23
 * \param[in] *mday   ÿµĵڼ죨0-29,0-28,0-30,0-31
 * \param[in] *wday   һܵĵڼ죨0-6Ӧ-
 * \param[in] *mon    £1-12
 * \param[in] *year   ꣨0-99
 *
 * \retval  AM_OK     ɹ
 * \retval -AM_EINVAL 
 */
am_err_t am_zlg118_rtc_cnt_get (am_rtc_handle_t handle,
                                uint8_t        *sec,
                                uint8_t        *min,
                                uint8_t        *hour,
                                uint8_t        *mday,
                                uint8_t        *wday,
                                uint8_t        *mon,
                                uint8_t        *year);

/**
 * \brief  RTC ֵ
 *
 * \param[in] handle RTC ׼
 * \param[in] sec    루0-59
 * \param[in] min    ֣0-59
 * \param[in] hour   ʱ0-23
 * \param[in] mday   ÿµĵڼ죨0-29,0-28,0-30,0-31
 * \param[in] wday   һܵĵڼ죨0-6Ӧ-
 * \param[in] mon    £1-12
 * \param[in] year   ꣨0-99
 *
 * \retval  AM_OK     ɹ
 * \retval -AM_EINVAL 
 */
am_err_t am_zlg118_rtc_cnt_set (am_rtc_handle_t handle,
                                uint8_t         sec,
                                uint8_t         min,
                                uint8_t         hour,
                                uint8_t         mday,
                                uint8_t         wday,
                                uint8_t         mon,
                                uint8_t         year);

/**
 * \brief  RTC жϻص
 *
 * \param[in] handle       RTC ׼
 * \param[in] type         жͣѡ AM_ZLG118_RTC_CALLBACK_SECOND
 *                                   AM_ZLG118_RTC_CALLBACK_PERIOD
 * \param[in] pfn_callback ص
 * \param[in] p_arg        ص
 *
 * \retval  AM_OK     ɹ
 * \retval -AM_EINVAL 
 */
am_err_t am_zlg118_rtc_callback_set (am_rtc_handle_t handle,
                                     uint8_t         type,
                                     am_pfnvoid_t    pfn_callback,
                                     void           *p_arg);

/**
 * \brief RTCжʹ
 *
 * \param[in] handle RTC׼
 * \param[in] type   жͣöٶamhw_zlg118_rtc_period_int_type_t
 * \param[in] set_s  Ϊϵͳʱжϲʱamhw_zlg118_rtc_period_int_time_t
 * \param[in] set_c  ΪԶʱжϲʱ䣨0.5 ~ 32s
 *
 * \retval  AM_OK     ɹ
 * \retval -AM_EINVAL 
 */
am_err_t am_zlg118_rtc_period_int_enable(
              am_rtc_handle_t                   handle,
              amhw_zlg118_rtc_period_int_type_t type,
              amhw_zlg118_rtc_period_int_time_t set_s,
              float                             set_c);

/**
 * \brief RTCжʹ
 *
 * \param[in] handle RTC׼
 * \param[in] sec    ʱ-루0-59
 * \param[in] min    ʱ-֣0-59
 * \param[in] hour   ʱ-ʱ0-59
 * \param[in] wday   ʱ-ܼ0-6Ӧ-
 *
 * \retval  AM_OK     ɹ
 * \retval -AM_EINVAL 
 */
am_err_t am_zlg118_rtc_alarm_clock_int_enable(am_rtc_handle_t handle,
                                              uint8_t         sec,
                                              uint8_t         min,
                                              uint8_t         hour,
                                              uint8_t         wday);

/**
 * \brief RTC жʧ
 *
 * \param[in] handle RTC ׼
 * \param[in] type   жͣѡ AM_ZLG118_RTC_CALLBACK_PERIOD
 *                             AM_ZLG118_RTC_CALLBACK_ALARM
 *
 * \retval  AM_OK     ɹ
 * \retval -AM_EINVAL 
 */
am_err_t am_zlg118_rtc_int_disable (am_rtc_handle_t handle,
                                    uint8_t         type);

/**
 * \brief ʼ RTC
 *
 * \param[in] p_dev     ָ RTC 豸ָ
 * \param[in] p_devinfo ָ RTC 豸Ϣָ
 *
 * \return RTC ׼Ϊ NULLʼʧ
 */
am_rtc_handle_t am_zlg118_rtc_init (am_zlg118_rtc_dev_t           *p_dev,
                                    const am_zlg118_rtc_devinfo_t *p_devinfo);

/**
 * \brief ʼ RTC
 *
 * \param[in] handle ʹ am_zlg118_rtc_init() ȡ RTC ׼
 *
 * \return 
 */
void am_zlg118_rtc_deinit (am_rtc_handle_t handle);

/** @} */

/**
 * @} zlg_if_rtc
 */

#ifdef __cplusplus
}
#endif

#endif/* __ZLG118_RTC_H */

/* end of file */
