/*******************************************************************************
*                                 AMetal
*                       ----------------------------
*                       innovating embedded platform
*
* Copyright (c) 2001-2018 Guangzhou ZHIYUAN Electronics Co., Ltd.
* All rights reserved.
*
* Contact information:
* web site:    http://www.zlg.cn/
*******************************************************************************/

/**
 * \file
 * \brief RTC drivers for standard interface
 *
 * \internal
 * \par Modification history
 * - 1.11 20-04-23 zcb, Add RTC interrupt management interface
 * - 1.10 20-04-17 zcb, Adapt standard drive and delete interfaces that
 *                      are not related to standards
 * - 1.00 19-10-16  zp, first implementation
 * \endinternal
 */

#ifndef __AM_ZSL42x_RTC_H
#define __AM_ZSL42x_RTC_H

#ifdef __cplusplus
extern "C" {
#endif

#include "am_rtc.h"
#include "hw/amhw_zsl42x_rtc.h"
#include "hw/amhw_zsl42x_rcc.h"

/**
 * \addtogroup am_zsl42x_if_rtc
 * \copydoc am_zsl42x_rtc.h
 * @{
 */

/**
 * \name ZSL42x RTC ж
 * @{
 */

/** \brief RTC ж */
#define  AM_ZSL42x_RTC_INT_PERIOD  (0x0)

/** \brief RTC ж */
#define  AM_ZSL42x_RTC_INT_ALARM   (0x1)

/** \brief RTC ж */
#define  AM_ZSL42x_RTC_INT_CNT     (AM_ZSL42x_RTC_INT_ALARM + 1)

/** @} */

/**
 * \brief ZSL42x RTC 豸Ϣ
 */
typedef struct am_zsl42x_rtc_devinfo {

    /** \brief RTC Ĵַ */
    uint32_t                     regbase;

    /** \brief RTC жϺ */
    int                          inum;

    /** \brief RTC ʼΪ ZSL42x RTC ݱʾΧΪ 0-99˱ָʼʱ䣬1900 */
    uint32_t                     year_epoch;

    /** \brief ƽ̨ʼ */
    void                       (*pfn_plfm_init) (void);
    /** \brief ƽ̨ʼ */
    void                       (*pfn_plfm_deinit) (void);
} am_zsl42x_rtc_devinfo_t;

/**
 * \brief ZSL42x RTC 豸
 */
typedef struct am_zsl42x_rtc_dev {

    /** \brief RTC ׼ */
    am_rtc_serv_t                rtc_serv;

    /** \brief RTC豸Ϣ */
    const am_zsl42x_rtc_devinfo_t *p_devinfo;

    /** \brief ص */
    am_pfnvoid_t                 pfn_callback[AM_ZSL42x_RTC_INT_CNT];

    /** \brief ص */
    void                        *p_arg[AM_ZSL42x_RTC_INT_CNT];
}am_zsl42x_rtc_dev_t;

/**
 * \brief ʼ RTC
 *
 * \param[in] p_dev        : ָ RTC 豸ָ
 * \param[in] p_devinfo    : ָ RTC 豸Ϣָ
 *
 * \return RTC ׼Ϊ NULLʼʧ
 */
am_rtc_handle_t am_zsl42x_rtc_init (am_zsl42x_rtc_dev_t           *p_dev,
                                  const am_zsl42x_rtc_devinfo_t *p_devinfo);

/**
 * \brief ʼ RTC
 *
 * \param[in] handle       : ʹ am_zsl42x_rtc_init() ȡ RTC ׼
 *
 * \return 
 */
void am_zsl42x_rtc_deinit (am_rtc_handle_t            handle);

/******************************************************************************/

/**
 * \brief  RTC жûص
 *
 * \param[in] handle       : RTC ж϶Ӧ RTC  RTC 
 * \param[in] type         : жͣAM_ZSL42x_RTC_INT_xxxx
 * \param[in] pfn_callback : ص
 * \param[in] p_arg        : ص
 *
 * \note  Ϊ RTC ֻһжԴҪ׼жϿ׼ʱ
 *         RTC ڳ׼ RTC ӿӶ RTC жϵĹ
 *
 * \retval  AM_OK     ɹ
 * \retval -AM_EINVAL 
 */
am_err_t am_zsl42x_rtc_int_connect (am_rtc_handle_t   handle,
                                  uint8_t           type,
                                  am_pfnvoid_t      pfn_callback,
                                  void             *p_arg);

/**
 * \brief RTC жʹ
 *
 * \param[in] handle       : RTC ׼
 * \param[in] type         : жͣAM_ZSL42x_RTC_INT_xxxx
 *
 * \retval  AM_OK     ɹ
 * \retval -AM_EINVAL 
 */
am_err_t am_zsl42x_rtc_int_disable (am_rtc_handle_t   handle,
                                  uint8_t           type);

/**
 * \brief RTC жϽ
 *
 * \param[in] handle       : RTC ׼
 * \param[in] type         : жͣAM_ZSL42x_RTC_INT_xxxx
 *
 * \retval  AM_OK     ɹ
 * \retval -AM_EINVAL 
 */
am_err_t am_zsl42x_rtc_int_enable  (am_rtc_handle_t   handle,
                                  uint8_t           type);

/**
 * @}
 */

#ifdef __cplusplus
}
#endif

#endif/* __ZSL42x_RTC_H */

/* end of file */
