/*******************************************************************************
*                                 AMetal
*                       ----------------------------
*                       innovating embedded platform
*
* Copyright (c) 2001-2018 Guangzhou ZHIYUAN Electronics Co., Ltd.
* All rights reserved.
*
* Contact information:
* web site:    http://www.zlg.cn/
*******************************************************************************/

/**
 * \file
 * \brief RTC interfaces for operating hardware.
 *
 * \internal
 * \par Modification history
 * - 1.00 19-07-23  zp, first implementation
 * \endinternal
 */

#ifndef __AMHW_ZLG237_RTC_H
#define __AMHW_ZLG237_RTC_H

#ifdef __cplusplus
extern "C" {
#endif

#include "am_types.h"
#include "am_common.h"

/**
 * \addtogroup amhw_zlg237_if_rtc
 * \copydoc amhw_zlg237_rtc.h
 * @{
 */

/**
 * \brief RTC Ĵṹ嶨
 */
typedef struct amhw_zlg237_rtc {
    __IO uint16_t crh;         /**< \brief RTC ƼĴλ */
    __I  uint16_t reserve0;    /**< \brief λ */
    __IO uint16_t crl;         /**< \brief RTC ƼĴλ */
    __I  uint16_t reserve1;    /**< \brief λ */
    __IO uint16_t prlh;        /**< \brief RTC ԤƵװؼĴλ */
    __I  uint16_t reserve2;    /**< \brief λ */
    __IO uint16_t prll;        /**< \brief RTC ԤƵװؼĴλ */
    __I  uint16_t reserve3;    /**< \brief λ */
    __IO uint16_t divh;        /**< \brief RTC ԤƵƵӼĴλ */
    __I  uint16_t reserve4;    /**< \brief λ */
    __IO uint16_t divl;        /**< \brief RTC ԤƵƵӼĴλ */
    __I  uint16_t reserve5;    /**< \brief λ */
    __IO uint16_t cnth;        /**< \brief RTC Ĵλ */
    __I  uint16_t reserve6;    /**< \brief λ */
    __IO uint16_t cntl;        /**< \brief RTC Ĵλ */
    __I  uint16_t reserve7;    /**< \brief λ */
    __IO uint16_t alrh;        /**< \brief RTC ӼĴλ */
    __I  uint16_t reserve8;    /**< \brief λ */
    __IO uint16_t alrl;        /**< \brief RTC ӼĴλ */
    __I  uint16_t reserve9;    /**< \brief λ */
} amhw_zlg237_rtc_t;

/**
 * \brief RTC ƼĴλжö
 */
typedef enum {
    AMHW_ZLG237_RTC_SECIE = 0,
    AMHW_ZLG237_RTC_ALRIE = 1,
    AMHW_ZLG237_RTC_OWIE  = 2,
} amhw_zlg237_rtc_crh_int_permit;

/**
 * \brief RTC ƼĴλ״̬־ö
 */
typedef enum {
    AMHW_ZLG237_RTC_SECF  = 0,
    AMHW_ZLG237_RTC_ALRF  = 1,
    AMHW_ZLG237_RTC_OWF   = 2,
    AMHW_ZLG237_RTC_RSF   = 3,
    AMHW_ZLG237_RTC_RTOFF = 5,
} amhw_zlg237_rtc_crl_status;

/**
 * \brief ж
 *
 * \param[in] p_hw_rtc ָ amhw_zlg237_rtc_t ṹָ
 * \param[in] permit   RTC ƼĴλжö
 *
 * \return 
 */
am_static_inline
void amhw_zlg237_rtc_crh_allow_int (amhw_zlg237_rtc_t              *p_hw_rtc,
                                    amhw_zlg237_rtc_crh_int_permit  permit)
{
    p_hw_rtc->crh |= (1 << permit);
}

/**
 * \brief жϽֹ
 *
 * \param[in] p_hw_rtc ָ amhw_zlg237_rtc_t ṹָ
 * \param[in] permit   RTC ƼĴλжö
 *
 * \return 
 */
am_static_inline
void amhw_zlg237_rtc_crh_forbid_int (amhw_zlg237_rtc_t              *p_hw_rtc,
                                     amhw_zlg237_rtc_crh_int_permit  permit)
{
    p_hw_rtc->crh &= ~(1 << permit);
}

/**
 * \brief ƼĴλ״̬λ
 *
 * \param[in] p_hw_rtc ָ amhw_zlg237_rtc_t ṹָ
 * \param[in] status    RTC ƼĴλ״̬־ö
 *
 * \retval AM_TRUE  ־λλ
 * \retval AM_FALSE ־λδλ
 */
am_static_inline
am_bool_t amhw_zlg237_rtc_crl_read_statu (amhw_zlg237_rtc_t          *p_hw_rtc,
                                          amhw_zlg237_rtc_crl_status  status)
{
    return (am_bool_t)((p_hw_rtc->crl & (1ul << status)) ? AM_TRUE : AM_FALSE);
}

/**
 * \brief ƼĴλ״̬λ
 *
 * \param[in] p_hw_rtc ָ amhw_zlg237_rtc_t ṹָ
 * \param[in] status   RTC ƼĴλ״̬־ö
 *
 * \return 
 */
am_static_inline
void amhw_zlg237_rtc_clr_status_clear (amhw_zlg237_rtc_t          *p_hw_rtc,
                                       amhw_zlg237_rtc_crl_status  status)
{
    p_hw_rtc->crl &= ~(1 << status);
}

/**
 * \brief  RTC ģʽ
 *
 * \param[in] p_hw_rtc ָ amhw_zlg237_rtc_t ṹָ
 *
 * \return 
 */
am_static_inline
void amhw_zlg237_rtc_crl_cnf_enter (amhw_zlg237_rtc_t *p_hw_rtc)
{
    p_hw_rtc->crl |= (1 << 4);
}

/**
 * \brief ˳ RTC ģʽ
 *
 * \param[in] p_hw_rtc ָ amhw_zlg237_rtc_t ṹָ
 *
 * \return 
 */
am_static_inline
void amhw_zlg237_rtc_crl_cnf_out (amhw_zlg237_rtc_t *p_hw_rtc)
{
    p_hw_rtc->crl &= ~(1 << 4);
}

/**
 * \brief RTC ԤƵװؼĴλд
 *
 * \param[in] p_hw_rtc ָ amhw_zlg237_rtc_t ṹָ
 * \param[in] divh     ԤƵλ
 *
 * \return 
 */
am_static_inline
void amhw_zlg237_rtc_prlh_div_write (amhw_zlg237_rtc_t *p_hw_rtc, uint16_t divh)
{
    p_hw_rtc->prlh = (divh & 0x000F);
}

/**
 * \brief RTC ԤƵװؼĴλд
 *
 * \param[in] p_hw_rtc ָ amhw_zlg237_rtc_t ṹָ
 * \param[in] divl     ԤƵλ
 *
 * \return 
 */
am_static_inline
void amhw_zlg237_rtc_prll_div_write (amhw_zlg237_rtc_t *p_hw_rtc, uint16_t divl)
{
    p_hw_rtc->prll = divl;
}

/**
 * \brief RTC ԤƵװؼĴλ
 *
 * \param[in] p_hw_rtc ָ amhw_zlg237_rtc_t ṹָ
 *
 * \return RTC ԤƵĴλ״ֵ̬
 */
am_static_inline
uint16_t amhw_zlg237_rtc_divh_div_read (amhw_zlg237_rtc_t *p_hw_rtc)
{
    return p_hw_rtc->divh;
}

/**
 * \brief RTC ԤƵװؼĴλ
 *
 * \param[in] p_hw_rtc ָ amhw_zlg237_rtc_t ṹָ
 *
 * \return RTC ԤƵĴλ״ֵ̬
 */
am_static_inline
uint16_t amhw_zlg237_rtc_divl_div_read (amhw_zlg237_rtc_t *p_hw_rtc)
{
    return p_hw_rtc->divl;
}

/**
 * \brief RTC Ĵλú
 *
 * \param[in] p_hw_rtc ָ amhw_zlg237_rtc_t ṹָ
 * \param[in] cnth     λ
 *
 * \return 
 */
am_static_inline
void amhw_zlg237_rtc_cnth_set (amhw_zlg237_rtc_t *p_hw_rtc, uint16_t cnth)
{
    p_hw_rtc->cnth = cnth;
}

/**
 * \brief RTC Ĵλú
 *
 * \param[in] p_hw_rtc ָ amhw_zlg237_rtc_t ṹָ
 * \param[in] cntl     λ
 *
 * \return 
 */
am_static_inline
void amhw_zlg237_rtc_cntl_set (amhw_zlg237_rtc_t *p_hw_rtc, uint16_t cntl)
{
    p_hw_rtc->cntl = cntl;
}

/**
 * \brief ȡ RTC Ĵλֵ
 *
 * \param[in] p_hw_rtc ָ amhw_zlg237_rtc_t ṹָ
 *
 * \return λֵ
 */
am_static_inline
uint16_t amhw_zlg237_rtc_cnth_get (amhw_zlg237_rtc_t *p_hw_rtc)
{
    return p_hw_rtc->cnth;
}

/**
 * \brief ȡ RTC Ĵλֵ
 *
 * \param[in] p_hw_rtc ָ amhw_zlg237_rtc_t ṹָ
 *
 * \return Ĵλֵ
 */
am_static_inline
uint16_t amhw_zlg237_rtc_cntl_get (amhw_zlg237_rtc_t *p_hw_rtc)
{
    return p_hw_rtc->cntl;
}

/**
 * \brief RTC ӼĴλд
 *
 * \param[in] p_hw_rtc ָ amhw_zlg237_rtc_t ṹָ
 * \param[in] alarmh   ӼĴλ
 *
 * \return 
 */
am_static_inline
void amhw_zlg237_rtc_alrh_set (amhw_zlg237_rtc_t *p_hw_rtc, uint16_t alarmh)
{
    p_hw_rtc->alrh = alarmh;
}

/**
 * \brief RTC ӼĴλд
 *
 * \param[in] p_hw_rtc ָ amhw_zlg237_rtc_t ṹָ
 * \param[in] alarml   ӼĴλ
 *
 * \return 
 */
am_static_inline
void amhw_zlg237_rtc_alrl_set (amhw_zlg237_rtc_t *p_hw_rtc, uint16_t alarml)
{
    p_hw_rtc->alrl = alarml;
}

/**
 * @}
 */

#ifdef __cplusplus
}
#endif

#endif/* __ZLG237_RTC_H */

/* end of file */
