/*******************************************************************************
*                                 AMetal
*                       ----------------------------
*                       innovating embedded platform
*
* Copyright (c) 2001-2018 Guangzhou ZHIYUAN Electronics Co., Ltd.
* All rights reserved.
*
* Contact information:
* web site:    http://www.zlg118.cn/
*******************************************************************************/

/**
 * \file
 * \brief SPI Ӳӿ
 *
 * \internal
 * \par Modification history
 * - 1.00 19-0-17  zp, first implementation
 * \endinternal
 */

#ifndef __AMHW_ZLG118_SPI_H
#define __AMHW_ZLG118_SPI_H

#ifdef __cplusplus
extern "C" {
#endif

#include "am_types.h"
#include "am_bitops.h"

/**
 * \addtogroup amhw_zlg118_if_spi
 * \copydoc amhw_zlg118_spi.h
 * @{
 */

/**
 * \brief SPI structure of register
 */
typedef struct amhw_zlg118_spi {
    __IO uint32_t  cr;       /**< \brief SPI üĴ */
    __IO uint32_t  ssn;      /**< \brief SPI ƬѡüĴ */
    __I  uint32_t  stat;     /**< \brief SPI ״̬Ĵ */
    __IO uint32_t  data;     /**< \brief SPI ݼĴ */
    __IO uint32_t  cr2;      /**< \brief SPI üĴ 2 */
    __O  uint32_t  iclr;     /**< \brief SPI жĴ */
} amhw_zlg118_spi_t;

/**
 * \brief ò
 */
typedef enum {
    AMHW_ZLG118_SPI_BAUD_RATE_PCLK_DIV2 = 0,  /**< \brief Ϊʱӵ2Ƶ */
    AMHW_ZLG118_SPI_BAUD_RATE_PCLK_DIV4,      /**< \brief Ϊʱӵ4Ƶ */
    AMHW_ZLG118_SPI_BAUD_RATE_PCLK_DIV8,      /**< \brief Ϊʱӵ8Ƶ */
    AMHW_ZLG118_SPI_BAUD_RATE_PCLK_DIV16,     /**< \brief Ϊʱӵ16Ƶ */
    AMHW_ZLG118_SPI_BAUD_RATE_PCLK_DIV32,     /**< \brief Ϊʱӵ32Ƶ */
    AMHW_ZLG118_SPI_BAUD_RATE_PCLK_DIV64,     /**< \brief Ϊʱӵ64Ƶ */
    AMHW_ZLG118_SPI_BAUD_RATE_PCLK_DIV128,    /**< \brief Ϊʱӵ128Ƶ */
    AMHW_ZLG118_SPI_BAUD_RATE_PCLK_0,         /**< \brief Ϊ0 */
}amhw_zlg118_spi_baud_rate_t;

/**
 * \brief SPI     ȡ
 * \param[in] p_hw_spi  : ָSPIĴṹָ
 *
 * \return ò(amhw_zlg118_spi_baud_rate_tö)
 */
am_static_inline
amhw_zlg118_spi_baud_rate_t
amhw_zlg118_spi_baud_rate_get (amhw_zlg118_spi_t *p_hw_spi)
{
   return (amhw_zlg118_spi_baud_rate_t)((p_hw_spi->cr & 0x3ul) | 
                                        ((p_hw_spi->cr >> 5) & 0x4ul));
}

/**
 * \brief SPI     
 * \param[in] p_hw_spi  : ָSPIĴṹָ
 * \param[in] baud_rate : ò(amhw_zlg118_spi_baud_rate_tö)
 *
 * \return none
 */
am_static_inline
void amhw_zlg118_spi_baud_rate_set (amhw_zlg118_spi_t          *p_hw_spi,
                                    amhw_zlg118_spi_baud_rate_t baud_rate)
{
    /* bit7  bit1 bit0  */
    p_hw_spi->cr &= ~(0x83ul);

    /* bit7ֵ */
    p_hw_spi->cr |= ((baud_rate & 0x4ul) << 5);

    /* bit1bit0ֵ */
    p_hw_spi->cr |= ((baud_rate & 0x3ul) << 0);
}

/**
 * \brief SPI ģʹ
 * \param[in] p_hw_spi : ָSPIĴṹָ
 * \param[in] flag     : AM_TRUE or AM_FALSE
 * \return none
 */
am_static_inline
void amhw_zlg118_spi_enable (amhw_zlg118_spi_t *p_hw_spi, am_bool_t flag)
{
    p_hw_spi->cr = (p_hw_spi->cr & (~(1u << 6))) | (flag << 6);
}

/**
 * \brief /ӻģʽѡ
 */
typedef enum {
    AMHW_ZLG118_SPI_MODE_TYPE_MASTER = 1,  /**< \brief ģʽ */
    AMHW_ZLG118_SPI_MODE_TYPE_SLAVER = 0,  /**< \brief ӻģʽ */
}amhw_zlg118_spi_mode_type_t;

/**
 * \brief SPI /ӻģʽѡ
 * \param[in] p_hw_spi : ָSPIĴṹָ
 * \param[in] flag     : amhw_zlg118_spi_mode_type_tö
 *
 * \return none
 */
am_static_inline
void amhw_zlg118_spi_mode_sel (amhw_zlg118_spi_t          *p_hw_spi,
                               amhw_zlg118_spi_mode_type_t mode_type)
{
    p_hw_spi->cr = (p_hw_spi->cr & (~(1u << 4))) | (mode_type << 4);
}

/**
 * \brief ʱģʽѡ
 * \ @{
 */
#define AMHW_ZLG118_SPI_CLK_MODE_0    0 /**< \brief Clock mode 0 */
#define AMHW_ZLG118_SPI_CLK_MODE_1    1 /**< \brief Clock mode 1 */
#define AMHW_ZLG118_SPI_CLK_MODE_2    2 /**< \brief Clock mode 2 */
#define AMHW_ZLG118_SPI_CLK_MODE_3    3 /**< \brief Clock mode 3 */

/** @}*/

/**
 * \brief ʱģʽѡ
 * \param[in] p_hw_spi : ָSPIĴṹָ
 * \param[in] flag     : ģʽú궨
 *
 * \return none
 */
am_static_inline
void amhw_zlg118_spi_clk_mode_set (amhw_zlg118_spi_t *p_hw_spi, uint32_t flag)
{
    p_hw_spi->cr = (p_hw_spi->cr & (~(3u << 2))) | (flag << 2);
}

/**
 * \brief ģʽ, ssnֵ SPI_CS˿ڵƽߵ
 * \param[in] p_hw_spi  : ָSPIĴṹָ
 * \param[in] flag      : AM_TRUE : ƬѡЧ
 *                        AM_FALSE: ƬѡʧЧ
 *
 * \return none
 */
am_static_inline
void amhw_zlg118_spi_cs_sel (amhw_zlg118_spi_t *p_hw_spi, am_bool_t flag)
{
    p_hw_spi->ssn = (~flag) & 0x1;
}

/**
 * \brief SPI״̬־
 *  @{
 */
#define AMHW_ZLG118_SPI_FLAG_TX_FINISH        (1U << 7) /* ־ */
#define AMHW_ZLG118_SPI_FLAG_SLAVER_SSN_ERR   (1U << 5) /* ӻģʽSSN־*/
#define AMHW_ZLG118_SPI_FLAG_MASTER_ERR       (1U << 4) /* ģʽ־ */
#define AMHW_ZLG118_SPI_FLAG_BUSY             (1U << 3) /* SPIæ־ */
#define AMHW_ZLG118_SPI_FLAG_TX_EMPTY         (1U << 2) /* ͻձ־ */
#define AMHW_ZLG118_SPI_FLAG_RX_NOT_EMPTY     (1U << 1) /* ջǿձ־ */
/**
 * \brief ־Чж
 * \param[in] p_hw_spi : ָSPIĴṹָ
 * \param[in] flag     : ־궨
 *
 * \return AM_TRUE :־Ч
 *         AM_FALSE:־Ч
 */
am_static_inline
am_bool_t amhw_zlg118_spi_flag_check (amhw_zlg118_spi_t *p_hw_spi,
                                      uint8_t            flag)
{
    return ((p_hw_spi->stat & flag)) ? AM_TRUE : AM_FALSE;
}

/**
 * \brief  SPIж
 * \param[in] p_hw_spi : ָSPIĴṹָ
 *
 * \return none
 */
am_static_inline
void amhw_zlg118_spi_flag_clr (amhw_zlg118_spi_t *p_hw_spi)
{
    p_hw_spi->iclr &= ~(0x1ul << 0);
}

/**
 * \brief  SPIжϱ
 * \param[in] p_hw_spi : ָSPIĴṹָ
 *
 * \return none
 */
am_static_inline
void amhw_zlg118_spi_flag_hold (amhw_zlg118_spi_t *p_hw_spi)
{
    p_hw_spi->iclr |= (0x1ul << 0);
}

/**
 * \brief  8bitд
 * \param[in] p_hw_spi : ָSPIĴṹָ
 * \param[in] data     : 
 * \param[in] lsbfirst : λȷͱ־Ӳֻиλȷ͹ܡΪӦ׼㣬ʵֵλȷ͹
 *
 *
 * \return none
 */
am_static_inline
void amhw_zlg118_spi_tx_data8_put (amhw_zlg118_spi_t *p_hw_spi,
                                   uint8_t            data,
                                   am_bool_t          lsbfirst)
{
    uint8_t temp = 0,i = 0;

    /* λ */
    if(lsbfirst) {
        for(i = 0; i < 8; i++) {
            if(((data >> i) & 0x1ul) == 1) {
                temp |= (1 << (7 - i));
            }
        }
        data = temp;
    }

    p_hw_spi->data = data;
}

/**
 * \brief 8bitݶ
 * \param[in] p_hw_spi : ָSPIĴṹָ
 * \param[in] lsbfirst : λȷͱ־Ӳֻиλȷ͹ܡΪӦ׼㣬ʵֵλȷ͹
 *
 * \return 
 */
am_static_inline
uint8_t amhw_zlg118_spi_rx_data8_get (amhw_zlg118_spi_t *p_hw_spi,
                                      am_bool_t          lsbfirst)
{
    uint8_t temp = 0,i = 0;

    uint8_t data = p_hw_spi->data;


    /* λ */
    if(lsbfirst) {
        for(i = 0; i < 8; i++) {
            if(((data >> i) & 0x1ul) == 1) {
                temp |= (1 << (7 - i));
            }
        }
        data = temp;
    }

    return data;
}

/**
 * \brief ջǿжʹ
 * \param[in] p_hw_spi : ָSPIĴṹָ
 *
 * \return none
 */
am_static_inline
void amhw_zlg118_spi_int_rx_not_empty_enable (amhw_zlg118_spi_t *p_hw_spi)
{
    p_hw_spi->cr2 |= (0x1ul << 6);
}

/**
 * \brief ջǿжϽ
 * \param[in] p_hw_spi : ָSPIĴṹָ
 *
 * \return none
 */
am_static_inline
void amhw_zlg118_spi_int_rx_not_empty_disable (amhw_zlg118_spi_t *p_hw_spi)
{
    p_hw_spi->cr2 &= ~(0x1ul << 6);
}

/**
 * \brief ͻжʹ
 * \param[in] p_hw_spi : ָSPIĴṹָ
 *
 * \return none
 */
am_static_inline
void amhw_zlg118_spi_tx_empty_enable (amhw_zlg118_spi_t *p_hw_spi)
{
    p_hw_spi->cr2 |= (0x1ul << 5);
}

/**
 * \brief ͻжϽ
 * \param[in] p_hw_spi : ָSPIĴṹָ
 *
 * \return none
 */
am_static_inline
void amhw_zlg118_spi_tx_empty_disable (amhw_zlg118_spi_t *p_hw_spi)
{
    p_hw_spi->cr2 &= ~(0x1ul << 5);
}

/**
 * \brief DMAӲʷʹ
 * \param[in] p_hw_spi : ָSPIĴṹָ
 *
 * \return none
 */
am_static_inline
void amhw_zlg118_spi_dma_tx_enable (amhw_zlg118_spi_t *p_hw_spi)
{
    p_hw_spi->cr2 |= (0x1ul << 4);
}

/**
 * \brief DMAӲʷͽ
 * \param[in] p_hw_spi : ָSPIĴṹָ
 *
 * \return none
 */
am_static_inline
void amhw_zlg118_spi_dma_tx_disable (amhw_zlg118_spi_t *p_hw_spi)
{
    p_hw_spi->cr2 &= ~(0x1ul << 4);
}

/**
 * \brief DMAӲʽʹ
 * \param[in] p_hw_spi : ָSPIĴṹָ
 *
 * \return none
 */
am_static_inline
void amhw_zlg118_spi_dma_rx_enable (amhw_zlg118_spi_t *p_hw_spi)
{
    p_hw_spi->cr2 |= (0x1ul << 3);
}

/**
 * \brief DMAӲʽս
 * \param[in] p_hw_spi : ָSPIĴṹָ
 *
 * \return none
 */
am_static_inline
void amhw_zlg118_spi_dma_rx_disable (amhw_zlg118_spi_t *p_hw_spi)
{
    p_hw_spi->cr2 &= ~(0x1ul << 3);
}

/**
 * \brief SPIжʹ
 * \param[in] p_hw_spi : ָSPIĴṹָ
 *
 * \return none
 */
am_static_inline
void amhw_zlg118_spi_int_enable (amhw_zlg118_spi_t *p_hw_spi)
{
    p_hw_spi->cr2 |= (0x1ul << 2);
}

/**
 * \brief SPIжϽ
 * \param[in] p_hw_spi : ָSPIĴṹָ
 *
 * \return none
 */
am_static_inline
void amhw_zlg118_spi_int_disable (amhw_zlg118_spi_t *p_hw_spi)
{
    p_hw_spi->cr2 &= ~(0x1ul << 2);
}

/**
 * @} amhw_zlg118_if_spi
 */

#ifdef __cplusplus
}
#endif

#endif /* __AMHW_ZLG118_SPI_H */

/* end of file */
