/*******************************************************************************
*                                 AMetal
*                       ----------------------------
*                       innovating embedded platform
*
* Copyright (c) 2001-2018 Guangzhou ZHIYUAN Electronics Co., Ltd.
* All rights reserved.
*
* Contact information:
* web site:    http://www.zlg.cn/
*******************************************************************************/

/**
 * \file
 * \brief SPI Ӳӿ
 *
 * 1. ֧ʹӻ䣻
 * 2. ֵ֧ģʽ
 * 3. ȫ˫ͬ
 * 4. ֧DMA(ͺͽ)
 * 5. ʿã
 * 6. ʱλͼԿã
 * 7. LSBMSB䣻
 * 8. ƥݣƥɹʱжϣ
 *
 * \internal
 * \par Modification history
 * - 1.00 19-07-18  ari, first implementation
 * \endinternal
 */

#ifndef _AMHW_ZLG237_SPI_H_
#define _AMHW_ZLG237_SPI_H_

#ifdef __cplusplus
extern "C" {
#endif

#include "am_types.h"
#include "am_bitops.h"

/**
 * \addtogroup amhw_zlg237_if_spi
 * \copydoc amhw_zlg237_spi.h
 * @{
 */

/**
 * \brief SPI structure of register
 */
typedef struct amhw_zlg237_spi {
    __IO uint16_t  cr1;           /**< \brief SPI ƼĴ1 */
    __I  uint16_t  reserve1;      /**< \brief λ */
    __IO uint16_t  cr2;           /**< \brief SPI ƼĴ2 */
    __I  uint16_t  reserve2;      /**< \brief λ */
    __IO uint16_t  sr;            /**< \brief SPI ״̬Ĵ */
    __I  uint16_t  reserve3;      /**< \brief λ */
    __IO uint16_t  dr;            /**< \brief SPI ݼĴ */
    __I  uint16_t  reserve4;      /**< \brief λ */
    __IO uint16_t  crcpr;         /**< \brief SPI CRCʽĴ */
    __I  uint16_t  reserve5;      /**< \brief λ */
    __I  uint16_t  rxcrcr;        /**< \brief SPI RX_CRCĴ */
    __I  uint16_t  reserve6;      /**< \brief λ */
    __I  uint16_t  txcrcr;        /**< \brief SPI TX_CRCĴ */
    __I  uint16_t  reserve7;      /**< \brief λ */
    __IO uint16_t  cr3;           /**< \brief SPI ƼĴ3 */
    __I  uint16_t  reserve8;      /**< \brief λ */
} amhw_zlg237_spi_t;

/**
 * \brief BIDIMODE ģʽѡ񡪡˫˫ģʽ
 * \param[in] p_hw_spi : ָSPIṹָ
 *
 * \return none
 */
am_static_inline
void amhw_zlg237_spi_bidimode_double (amhw_zlg237_spi_t *p_hw_spi)
{
    p_hw_spi->cr1 &= ~(1u << 15);
}

/**
 * \brief BIDIMODE ģʽѡ񡪡˫ģʽ
 * \param[in] p_hw_spi : ָSPIṹָ
 *
 * \return none
 */
am_static_inline
void amhw_zlg237_spi_bidimode_single (amhw_zlg237_spi_t *p_hw_spi)
{
    p_hw_spi->cr1 |= (1u << 15);
}

/**
 * \brief BIDIOE ˫ģʽơ루ֻգ
 * \param[in] p_hw_spi : ָSPIṹָ
 *
 * \note  BIDIMODEλһڡ˫ģʽݵ
 *
 * \return none
 */
am_static_inline
void amhw_zlg237_spi_bidioe_input (amhw_zlg237_spi_t *p_hw_spi)
{
    p_hw_spi->cr1 &= ~(1u << 14);
}

/**
 * \brief BIDIOE ˫ģʽơֻ
 * \param[in] p_hw_spi : ָSPIṹָ
 *
 * \note  BIDIMODEλһڡ˫ģʽݵ
 *
 * \return none
 */
am_static_inline
void amhw_zlg237_spi_bidioe_output (amhw_zlg237_spi_t *p_hw_spi)
{
    p_hw_spi->cr1 |= (1u << 14);
}

/**
 * \brief CRCENӲ CRCУ
 * \param[in] p_hw_spi : ָSPIṹָ
 *
 * \note ֻڽֹ SPI ʱ(SPE=0)дλλֻȫ˫ģʽʹá
 *
 * \return none
 */
am_static_inline
void amhw_zlg237_spi_crc_disable (amhw_zlg237_spi_t *p_hw_spi)
{
    p_hw_spi->cr1 &= ~(1u << 13);
}

/**
 * \brief CRCENӲ CRCУʹ
 * \\param[in] p_hw_spi : ָSPIṹָ
 *
 * \note ֻڽֹ SPI ʱ(SPE=0)дλλֻȫ˫ģʽʹ
 *
 * \return none
 */
am_static_inline
void amhw_zlg237_spi_crc_enable (amhw_zlg237_spi_t *p_hw_spi)
{
    p_hw_spi->cr1 |= (1u << 13);
}

/**
 * \brief current status
 *  @{
 */
#define AMHW_ZLG237_SPI_CRC_NEXT_BUFF          0     /**< һ͵ֵԷͻ */
#define AMHW_ZLG237_SPI_CRC_NEXT_CRCREG        1     /**< һ͵ֵCRCĴ */

#define AMHW_ZLG237_SPI_DATA_8BIT              0     /**<  8λ֡ʽ */
#define AMHW_ZLG237_SPI_DATA_16BIT             1     /**< 16λ֡ʽ */
/** @}*/

/**
 * \brief CRCNEXT һ͵ֵԴ
 * \param[in] p_hw_spi : ָSPIṹָ
 * \param[in] flag : ӦΪ   AMHW_ZLG237_SPI_CRC_NEXT_BUFF  
 *                            AMHW_ZLG237_SPI_CRC_NEXT_CRCREG
 *
 * \note  SPI_DRĴдһݺӦøλ
 *
 * \return none
 */
am_static_inline
void amhw_zlg237_spi_crcnext_set (amhw_zlg237_spi_t *p_hw_spi, uint8_t flag)
{
    p_hw_spi->cr1 = (p_hw_spi->cr1 & (~(1u << 12))) | ((uint16_t)flag << 12);
}

/**
 * \brief DFF ֡ʽ
 * \param[in] p_hw_spi : ָSPIṹָ
 * \param[in] flag : ӦΪ   AMHW_ZLG237_SPI_DATA_8BIT  
 *                            AMHW_ZLG237_SPI_DATA_16BIT
 *
 * \note ֻе SPIֹ(SPE=0)ʱдλ
 *
 * \return none
 */
am_static_inline
void amhw_zlg237_spi_data_length_set (amhw_zlg237_spi_t *p_hw_spi, uint8_t flag)
{
    p_hw_spi->cr1 = (p_hw_spi->cr1 & (~(1u << 11))) | ((uint16_t)flag << 11);
}

/**
 * \brief RXONLY ֻս
 * \param[in] p_hw_spi : ָSPIṹָ
 *
 * \note λ BIDIMODE λһڡ˫˫ģʽµĴ䷽ڶ豸
 *       УδʵĴ豸ϸλ 1ʹֻбʵĴ豸Ӷ
 *       ݳͻ
 *
 * \return none
 */
am_static_inline
void amhw_zlg237_spi_rxonly_disable (amhw_zlg237_spi_t *p_hw_spi)
{
    p_hw_spi->cr1 &= ~(1u << 10);
}

/**
 * \brief RXONLY ֻʹ
 * \param[in] p_hw_spi : ָSPIṹָ
 *
 * \note λ BIDIMODE λһڡ˫˫ģʽµĴ䷽ڶ豸
 *       УδʵĴ豸ϸλ 1ʹֻбʵĴ豸Ӷ
 *       ݳͻ
 *
 * \return none
 */
am_static_inline
void amhw_zlg237_spi_rxonly_enable (amhw_zlg237_spi_t *p_hw_spi)
{
    p_hw_spi->cr1 |= (1u << 10);
}

/**
 * \brief SSM 豸ܣNSSܣ
 * \param[in] p_hw_spi : ָSPIṹָ
 *
 * \note  SSM λʱNSS ϵĵƽ SSI λֵ
 *
 * \return none
 */
am_static_inline
void amhw_zlg237_spi_ssm_disable (amhw_zlg237_spi_t *p_hw_spi)
{
    p_hw_spi->cr1 &= ~(1u << 9);
}

/**
 * \brief SSM 豸ʹ NSSʹܣ
 * \param[in] p_hw_spi : ָSPIṹָ
 *
 * \note  SSM λʱNSS ϵĵƽ SSI λֵ
 *
 * \return none
 */
am_static_inline
void amhw_zlg237_spi_ssm_enable (amhw_zlg237_spi_t *p_hw_spi)
{
    p_hw_spi->cr1 |= (1u << 9);
}

/**
 * \brief current status
 *  @{
 */
#define AMHW_ZLG237_SPI_SSI_TO_NSS_IGNORED      0     /**< NSSź  */
#define AMHW_ZLG237_SPI_SSI_TO_NSS_ENABLE       1     /**< NSSʹ  */

#define AMHW_ZLG237_SPI_LSB_FIRST_SEND_MSB      0     /**< ֡ʽ   ȷ MSB */
#define AMHW_ZLG237_SPI_LSB_FIRST_SEND_LSB      1     /**< ֡ʽ   ȷLSB */
/** @}*/

/**
 * \brief SSI ڲ豸ѡ
 * \param[in] p_hw_spi : ָSPIṹָ
 * \param[in] flag : ӦΪ   AMHW_ZLG237_SPI_SSI_TO_NSS_IGNORED  
 *                            AMHW_ZLG237_SPI_SSI_TO_NSS_ENABLE
 *
 * \note λֻ SSMλΪ1ʱ塣 NSSϵĵƽ NSSϵ I/OЧ޹
 *
 * \return none
 */
am_static_inline
void amhw_zlg237_spi_ssi_set (amhw_zlg237_spi_t *p_hw_spi, uint8_t flag)
{
    p_hw_spi->cr1 = (p_hw_spi->cr1 & (~(1u << 8))) | ((uint16_t)flag << 8);
}

/**
 * \brief LSBFIRST ֡ʽ
 * \param[in] p_hw_spi : ָSPIṹָ
 * \param[in] flag : ӦΪ   AMHW_ZLG237_SPI_LSB_FIRST_MSB  
 *                            AMHW_ZLG237_SPI_LSB_FIRST_LSB
 *
 * \note ͨڽʱܸıλֵ
 *
 * \return none
 */
am_static_inline

void amhw_zlg237_spi_lsbfirst_set (amhw_zlg237_spi_t *p_hw_spi, uint8_t flag)
{
    p_hw_spi->cr1 = (p_hw_spi->cr1 & (~(1u << 7))) | ((uint16_t)flag << 7);
}

/**
 * \brief SPE SPI
 * \param[in] p_hw_spi : ָSPIṹָ
 *
 * \return none
 */
am_static_inline
void amhw_zlg237_spi_disable (amhw_zlg237_spi_t *p_hw_spi)
{
    p_hw_spi->cr1 &= ~(1u << 6);
}

/**
 * \brief SPE SPIʹ
 * \param[in] p_hw_spi : ָSPIṹָ
 *
 * \return none
 */
am_static_inline
void amhw_zlg237_spi_enable (amhw_zlg237_spi_t *p_hw_spi)
{
    p_hw_spi->cr1 |= (1u << 6);
}

/**
 * \brief current status
 *  @{
 */
#define AMHW_ZLG237_SPI_BAUDRATE_PRESCALER_2       0     /**< fPCLK/2 */
#define AMHW_ZLG237_SPI_BAUDRATE_PRESCALER_4       1     /**< fPCLK/4 */
#define AMHW_ZLG237_SPI_BAUDRATE_PRESCALER_8       2     /**< fPCLK/8 */
#define AMHW_ZLG237_SPI_BAUDRATE_PRESCALER_16      3     /**< fPCLK/16 */
#define AMHW_ZLG237_SPI_BAUDRATE_PRESCALER_32      4     /**< fPCLK/32 */
#define AMHW_ZLG237_SPI_BAUDRATE_PRESCALER_64      5     /**< fPCLK/64 */
#define AMHW_ZLG237_SPI_BAUDRATE_PRESCALER_128     6     /**< fPCLK/128 */
#define AMHW_ZLG237_SPI_BAUDRATE_PRESCALER_256     7     /**< fPCLK/256 */
/** @}*/

/**
 * \brief BR[2:0] ʷƵ
 * \param[in] p_hw_spi : ָSPIṹָ
 * \param[in] flag : ӦΪ   AMHW_ZLG237_SPI_BAUDRATE_PRESCALER_2
 *                            .
 *                            .
 *                            .
 *                            AMHW_ZLG237_SPI_BAUDRATE_PRESCALER_256
 *
 * \note ͨڽʱܸıλֵ
 *
 * \return none
 */
am_static_inline
void amhw_zlg237_spi_baudratefre_set (amhw_zlg237_spi_t *p_hw_spi, uint8_t flag)
{
    p_hw_spi->cr1 = (p_hw_spi->cr1 & (~(0x7u << 3))) | ((uint16_t)( 0x7u & flag ) << 3);
}

/**
 * \brief current status
 *  @{
 */
#define AMHW_ZLG237_SPI_MASTER                   1     /**< Ϊ豸 */
#define AMHW_ZLG237_SPI_SLAVER                   0     /**< Ϊ豸 */
/** @}*/

/**
 * \brief MSTR 豸ѡ
 * \param[in] p_hw_spi : ָSPIṹָ
 * \param[in] flag : ӦΪ   AMHW_ZLG237_SPI_MASTER  
 *                            AMHW_ZLG237_SPI_MASTER
 *
 * \note ͨڽʱܸıλֵ
 *
 * \return none
 */
am_static_inline
void amhw_zlg237_spi_master_salver_set (amhw_zlg237_spi_t *p_hw_spi, uint16_t flag)
{
    p_hw_spi->cr1 = (p_hw_spi->cr1 & (~(1u << 2))) | ((uint16_t)flag << 2);
}

/**
 * \brief SPI Clock mode set
 * \ @{
 */
#define AMHW_ZLG237_SPI_CLK_MODE_0    (0U << 0) /* ģʽ0е͵ƽһʱأ*/
#define AMHW_ZLG237_SPI_CLK_MODE_1    (1U << 0) /* ģʽ1е͵ƽڶʱأ*/
#define AMHW_ZLG237_SPI_CLK_MODE_2    (2U << 0) /* ģʽ2иߵƽһʱأ*/
#define AMHW_ZLG237_SPI_CLK_MODE_3    (3U << 0) /* ģʽ3иߵƽڶʱأ*/
/** @}*/

/**
 * \brief SPI ʱģʽѡ
 * \param[in] p_hw_spi : ָSPIṹָ
 * \param[in] flag : ӦΪ   AMHW_ZLG_SPI_CLK_MODE_0  
 *                            AMHW_ZLG_SPI_CLK_MODE_1  
 *                            AMHW_ZLG_SPI_CLK_MODE_2  
 *                            AMHW_ZLG_SPI_CLK_MODE_3  
 *
 * \note ͨڽʱܸıλֵ
 *
 * \return none
 */
am_static_inline
void amhw_zlg237_spi_clk_mode_set (amhw_zlg237_spi_t *p_hw_spi, uint8_t flag)
{
    p_hw_spi->cr1 = (p_hw_spi->cr1 & (~(3u << 0))) | ((uint16_t)flag << 0);
}

/**
 * \brief current status
 *  @{
 */
#define AMHW_ZLG237_SPI_INT_TX_EMPTY          (1u << 7)     /**< ͻжʹܿλ */
#define AMHW_ZLG237_SPI_INT_RX_NOT_EMPTY      (1u << 6)     /**< ջǿжʹܿλ */
#define AMHW_ZLG237_SPI_INT_ERROR             (1u << 5)     /**< жʹܿλ */
#define AMHW_ZLG237_SPI_INT_ALL             (0x7u << 5)     /**< жʹܿλ */

/** @}*/

/**
 * \brief cr2жϿƽ
 * \param[in] p_hw_spi : ָSPIṹָ
 * \param[in] flag : ӦΪ   AMHW_ZLG237_SPI_INT_TX_EMPTY      
 *                            AMHW_ZLG237_SPI_INT_RX_NOT_EMPTY  
 *                            AMHW_ZLG237_SPI_INT_ERROR
 *
 * \return none
 */
am_static_inline
void amhw_zlg237_spi_int_disable (amhw_zlg237_spi_t *p_hw_spi, uint8_t flag)
{
    p_hw_spi->cr2 &= ~flag;
}

/**
 * \brief cr2жϿʹ
 * \param[in] p_hw_spi : ָSPIṹָ
 * \param[in] flag : ӦΪ   AMHW_ZLG237_SPI_INT_TX_EMPTY      
 *                            AMHW_ZLG237_SPI_INT_RX_NOT_EMPTY  
 *                            AMHW_ZLG237_SPI_INT_ERROR
 *
 * \return none
 */
am_static_inline
void amhw_zlg237_spi_int_enable (amhw_zlg237_spi_t *p_hw_spi, uint8_t flag)
{
    p_hw_spi->cr2 |= flag;
}

/**
 * \brief SSOE ӲSPINSS
 * \param[in] p_hw_spi : ָSPIṹָ
 *
 * \return none
 */
am_static_inline
void amhw_zlg237_spi_ssout_disable (amhw_zlg237_spi_t *p_hw_spi)
{
    p_hw_spi->cr2 &= ~(1u << 2);
}

/**
 * \brief SSOE ӲSPINSSʹ
 * \param[in] p_hw_spi : ָSPIṹָ
 *
 * \return none
 */
am_static_inline
void amhw_zlg237_spi_ssout_enable (amhw_zlg237_spi_t *p_hw_spi)
{
    p_hw_spi->cr2 |= (1u << 2);
}

/**
 * \brief TXDMAEN ͻ DMA
 * \param[in] p_hw_spi : ָSPIṹָ
 *
 * \return none
 */
am_static_inline
void amhw_zlg237_spi_dma_tx_disable (amhw_zlg237_spi_t *p_hw_spi)
{
    p_hw_spi->cr2 &= ~(1u << 1);
}

/**
 * \brief TXDMAEN ͻ DMAʹ
 * \param[in] p_hw_spi : ָSPIṹָ
 *
 * \return none
 */
am_static_inline
void amhw_zlg237_spi_dma_tx_enable (amhw_zlg237_spi_t *p_hw_spi)
{
    p_hw_spi->cr2 |= (1u << 1);
}

/**
 * \brief RXDMAEN ջDMA
 * \param[in] p_hw_spi : ָSPIṹָ
 *
 * \return none
 */
am_static_inline
void amhw_zlg237_spi_dma_rx_disable (amhw_zlg237_spi_t *p_hw_spi)
{
    p_hw_spi->cr2 &= ~(1u << 0);
}

/**
 * \brief RXDMAEN ջDMAʹ
 * \param[in] p_hw_spi : ָSPIṹָ
 *
 * \return none
 */
am_static_inline
void amhw_zlg237_spi_dma_rx_enable (amhw_zlg237_spi_t *p_hw_spi)
{
    p_hw_spi->cr2 |= (1u << 0);
}

/**
 * \brief current status
 *  @{
 */
/** æ־
 *
 *  note: λӲλ߸λ
 */
#define AMHW_ZLG237_SPI_BUSY_FLAG            (1u << 7)

/** ־
 *
 *  note: λӲλиλ
 */
#define AMHW_ZLG237_SPI_OVERRUN_FLAG         (1u << 6)

/** ģʽ־
 *
 *  note: λӲλиλ
 */
#define AMHW_ZLG237_SPI_MDOE_ERROR_FLAG      (1u << 5)

/** CRC־
 *
 *  note λӲλд0λ
 */
#define AMHW_ZLG237_SPI_CRC_ERROR_FLAG       (1u << 4)

/** ־
 *
 *  note: ñ־λӲá1һ塮0
 *        SPI ģʽ²ʹá
 */
#define AMHW_ZLG237_SPI_UNDERRUN_FLAG        (1u << 3)

/** ־
 *  ־ԣúķֱֵ
 *              AM_TRUE  : Ҫ߽
 *              AM_FALSE : Ҫ߽
 *
 *  note  SPI ģʽ²ʹá PCM ģʽ塣
 */
#define AMHW_ZLG237_SPI_CHANNEL_SIDE_FLAG    (1u << 2)

/** ͻΪձ־
 *
 *  note ֻ
 */
#define AMHW_ZLG237_SPI_TX_EMPTY_FLAG        (1u << 1)

/** ջǿձ־
 *
 *  note ֻ
 */
#define AMHW_ZLG237_SPI_RX_NOT_EMPTY_FLAG    (1u << 0)

/** @}*/

/**
 * \brief ״̬Ĵ־λȡ
 * \param[in] p_hw_spi : ָSPIṹָ
 * \param[in] flag : ӦΪ   AMHW_ZLG237_SPI_BUSY_FLAG          
 *                            AMHW_ZLG237_SPI_OVERRUN_FLAG       
 *                            AMHW_ZLG237_SPI_MDOE_ERROR_FLAG    
 *                            AMHW_ZLG237_SPI_CRC_ERROR_FLAG     
 *                            AMHW_ZLG237_SPI_UNDERRUN_FLAG      
 *                            AMHW_ZLG237_SPI_CHANNEL_SIDE_FLAG  
 *                            AMHW_ZLG237_SPI_TX_EMPTY_FLAG      
 *                            AMHW_ZLG237_SPI_RX_NOT_EMPTY_FLAG
 *
 * \return none
 */
am_static_inline
am_bool_t amhw_zlg237_spi_status_flag_check (amhw_zlg237_spi_t *p_hw_spi, uint8_t flag)
{
    return (p_hw_spi->sr & flag) ? AM_TRUE : AM_FALSE;
}

/**
 * \brief ״̬Ĵ־λ
 * \param[in] p_hw_spi : ָSPIṹָ
 * \param[in] flag     : ӦֻΪ   AMHW_ZLG237_SPI_CRC_ERROR_FLAG
 *                       궨ֻиλ
 *
 * \return none
 */
am_static_inline
void amhw_zlg237_spi_status_flag_clr (amhw_zlg237_spi_t *p_hw_spi, uint8_t flag)
{
    p_hw_spi->sr &= ~flag;
}

/**
 * \brief дݺ
 * \param[in] p_hw_spi : ָSPIṹָ
 * \param[in] data     : д
 *
 * \note :  SPI_CR1 DFFλ֡ʽѡݵķͺͽտ 8λ 16λġ
 *         Ϊ֤ȷĲҪ SPI ֮ǰȷ֡ʽ.
 *
 *          8 λݣ 8λģͺͽʱֻõSPI_DR[7:0]ڽʱ
 *         SPI_DR[15:8]ǿΪ 0
 *
 *          16 λݣ 16 λģͺͽʱõݼĴ SPI_DR[15:0]
 *
 * \return none
 */
am_static_inline
void amhw_zlg237_spi_tx_put (amhw_zlg237_spi_t *p_hw_spi, uint16_t data)
{
    p_hw_spi->dr = (uint16_t)data;
}

/**
 * \brief ݺ
 * \param[in] p_hw_spi : ָSPIṹָ
 *
 * \note :  SPI_CR1 DFFλ֡ʽѡݵķͺͽտ 8λ 16λġ
 *         Ϊ֤ȷĲҪ SPI ֮ǰȷ֡ʽ
 *
 *          8 λݣ 8λģͺͽʱֻõSPI_DR[7:0]ڽʱ
 *         SPI_DR[15:8]ǿΪ 0
 *
 *          16 λݣ 16 λģͺͽʱõݼĴ SPI_DR[15:0]
 *
 * \return 
 */
am_static_inline
uint16_t amhw_zlg237_spi_rx_get (amhw_zlg237_spi_t *p_hw_spi)
{
    return p_hw_spi->dr;
}

/**
 * \brief CRCʽĴ
 * \param[in] p_hw_spi : ָSPIṹָ
 * \param[in] flag     : дCRCʱõĶʽ
 *
 * \note   üĴ CRCʱõĶʽ
 *         临λֵΪ 0x0007Ӧÿֵ
 *
 * \return none
 */
am_static_inline
void amhw_zlg237_spi_crcpr_write (amhw_zlg237_spi_t *p_hw_spi, uint16_t crc_data)
{
    p_hw_spi->crcpr = crc_data;
}

/**
 * \brief CRCʽĴ  
 * \param[in] p_hw_spi : ָSPIṹָ
 *
 * \note   üĴ CRCʱõĶʽ
 *         临λֵΪ 0x0007Ӧÿֵ
 *
 * \return ǰĴ洢CRCʱõĶʽ
 */
am_static_inline
uint16_t amhw_zlg237_spi_crcpr_read (amhw_zlg237_spi_t *p_hw_spi)
{
    return p_hw_spi->crcpr;
}

/**
 * \brief RxCRCĴֻ
 * \param[in] p_hw_spi : ָSPIṹָ
 *
 * \note   SPI_CR1 CRCEN λд롮1ʱüĴλ
 *         CRCʹSPI_CRCPR еĶʽ
 *
 *         ֡ʽΪ8λʱ8λ㣬ҰCRC8ķУ֡
 *         ʽΪ 16λʱĴе16λ㣬ҰCRC16ı׼
 *
 *         BSY ־Ϊ1ʱüĴܶȷֵ
 *
 * \return յֽڼCRCֵ
 */
am_static_inline
uint16_t amhw_zlg237_spi_rxcrc_read (amhw_zlg237_spi_t *p_hw_spi)
{
    return p_hw_spi->rxcrcr;
}

/**
 * \brief TxCRCĴֻ
 * \param[in] p_hw_spi : ָSPIṹָ
 *
 * \note   SPI_CR1 CRCEN λд롮1ʱüĴλ
 *         CRCʹSPI_CRCPR еĶʽ
 *
 *         ֡ʽΪ8λʱ8λ㣬ҰCRC8ķУ֡
 *         ʽΪ 16λʱĴе16λ㣬ҰCRC16ı׼
 *
 *         BSY ־Ϊ1ʱüĴܶȷֵ
 *
 * \return Ҫ͵ֽڼCRCֵ
 */
am_static_inline
uint16_t amhw_zlg237_spi_txcrc_read (amhw_zlg237_spi_t *p_hw_spi)
{
    return p_hw_spi->txcrcr;
}

/**
 * \brief FlexLenEn ɱ
 * \param[in] p_hw_spi : ָSPIṹָ
 *
 * \note ʹ2~32ݰȿ书ܡ
 *
 * \return none
 */
am_static_inline
void amhw_zlg237_spi_flexlength_disable (amhw_zlg237_spi_t *p_hw_spi)
{
    p_hw_spi->cr3 &= ~(1u << 7);
}

/**
 * \brief FlexLenEn ɱʹ
 * \param[in] p_hw_spi : ָSPIṹָ
 *
 * \note ʹ2~32ݰȿ书ܣλLen[4:0]
 *
 * \return none
 */
am_static_inline
void amhw_zlg237_spi_flexlength_enable (amhw_zlg237_spi_t *p_hw_spi)
{
    p_hw_spi->cr3 |= (1u << 7);
}

/**
 * \brief Len[4:0] ɱãλ
 * \param[in] p_hw_spi : ָSPIṹָ
 * \param[in] flag     : ݰ趨ֵ-1
 *
 * \note   ݰΪLen[4:0]+1أLen[4:0] =5b00101, ݰΪ6ء
 *         趨ֵ[4:0]λЧ
 *
 * \return none
 */
am_static_inline
void amhw_zlg237_spi_flex_length_set (amhw_zlg237_spi_t *p_hw_spi, uint8_t length)
{
    p_hw_spi->crcpr = ( p_hw_spi->crcpr & (~(0x1fu << 0))) | ((0x1fu & length) << 0);
}

/**
 * \brief Len[4:0] ɱݶλ
 * \param[in] p_hw_spi : ָSPIṹָ
 *
 * \note   ݰΪLen[4:0]+1أLen[4:0] =5b00101, ݰΪ6ء
 *
 * \return ǰݰ趨ֵʵʰ-1
 */
am_static_inline
uint8_t amhw_zlg237_flex_length_get (amhw_zlg237_spi_t *p_hw_spi)
{
    return (p_hw_spi->crcpr) & 0x1fu;
}

/**
 * @} amhw_zlg_if_spi
 */

#ifdef __cplusplus
}
#endif

#endif /* SOC_ZLG_DRIVERS_INCLUDE_SPI_HW_AMHW_ZLG237_SPI_H_ */
