/*******************************************************************************
*                                 AMetal
*                       ----------------------------
*                       innovating embedded platform
*
* Copyright (c) 2001-2018 Guangzhou ZHIYUAN Electronics Co., Ltd.
* All rights reserved.
*
* Contact information:
* web site:    http://www.zlg.cn/
*******************************************************************************/

/**
 * \file
 * \brief ʱTIMӿ
 *
 * \internal
 * \par History
 * - 1.01 20-04-23 zcb, Modify the CCR register declaration and
 *                      read and write according to the channel number
 * - 1.00 19-09-10  zp, first implementation
 * \endinternal
 */

#ifndef __AMHW_ZSL42x_TIM_H
#define __AMHW_ZSL42x_TIM_H

#ifdef __cplusplus
extern "C" {
#endif

#include "am_types.h"
#include "am_bitops.h"

/**
 * \addtogroup amhw_zsl42x_if_tim
 * \copydoc amhw_zsl42x_tim.h
 * @{
 */

/**
  * \brief ʱĴṹ
  */
typedef struct amhw_zsl42x_tim {
    __IO uint32_t arr;            /**< \brief TimerؼĴ/ */
    __IO uint32_t cnt;            /**< \brief Timer16λģʽĴ */
    __IO uint32_t cnt32;          /**< \brief Timer32λģʽĴ */

         union mxcr {
             __IO uint32_t m0cr;  /**< \brief Timerģʽ0ƼĴ */
             __IO uint32_t m1cr;  /**< \brief Timerģʽ1ƼĴ */
             __IO uint32_t m23cr; /**< \brief Timerģʽ23ƼĴ */
         }mxcr_t;                 /**< \brief Timerģʽ0ƼĴͬģʽ*/

    __IO uint32_t ifr;            /**< \brief Timerжϱ־ */
    __IO uint32_t iclr;           /**< \brief TimerжĴ */
    __IO uint32_t mscr;           /**< \brief ģʽ */
    __IO uint32_t fltr;           /**< \brief ˲ */
    __IO uint32_t adtr;           /**< \brief ADC */
    __IO uint32_t crchx[3];       /**< \brief ȽϵԪ012ƼĴ */
    __IO uint32_t dtr;            /**< \brief Ĵ */
    __IO uint32_t rcr;            /**< \brief ظĴ */
    __IO uint32_t arrdm;          /**< \brief TimerؼĴ/ӳַ */

    __IO uint32_t ccr[6];         /**< \brief ȽϲĴ */
} amhw_zsl42x_tim_t;

/**
 * \brief ʱ
 */
typedef enum amhw_zsl42x_tim_type {
    AMHW_ZSL42x_TIM_TYPE_TIM0 = 0,  /**< \brief TIM0 */
    AMHW_ZSL42x_TIM_TYPE_TIM1,      /**< \brief TIM1 */
    AMHW_ZSL42x_TIM_TYPE_TIM2,      /**< \brief TIM2 */
    AMHW_ZSL42x_TIM_TYPE_TIM3,      /**< \brief TIM3 */
} amhw_zsl42x_tim_type_t;

/**
 * \brief ſؿ
 */
typedef enum amhw_zsl42x_tim_gate {
    AMHW_ZSL42x_TIM_GATE_DISABLE = 0,
    AMHW_ZSL42x_TIM_GATE_ENABLE,
} amhw_zsl42x_tim_gate_t;

/*
 * ʱͨŶ
 *
 * ʱ012ֻ2ֱͨCH0ACH0B(طΪͨ01)
 * ʱ36ֱͨCH0ACH0BCH1ACH1BCH2ACH2B
 *
 * CHxACHxBĳЩΪһͨ
 */
#define ZSL42x_TIM_CH0A    0
#define ZSL42x_TIM_CH0B    1
#define ZSL42x_TIM_CH1A    2
#define ZSL42x_TIM_CH1B    3
#define ZSL42x_TIM_CH2A    4
#define ZSL42x_TIM_CH2B    5

/**
 * \brief ARR-16λģʽؼĴ         õǰֵ
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 *
 * \return ARRؼĴֵ
 */
am_static_inline
uint16_t amhw_zsl42x_tim_arr_count_get (amhw_zsl42x_tim_t *p_hw_tim)
{
    return p_hw_tim->arr;
}

/**
 * \brief ARR-16λģʽؼĴ       õǰֵ
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 * \param[in] value    : õļֵ
 *
 *\ return 
 */
am_static_inline
void amhw_zsl42x_tim_arr_count_set (amhw_zsl42x_tim_t *p_hw_tim, uint16_t value)
{
    p_hw_tim->arr = value;
}

/**
 * \brief CNT-16λģʽĴ         õǰֵ
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 *
 * \return CNTĴֵ
 */
am_static_inline
uint16_t amhw_zsl42x_tim_cnt16_count_get (amhw_zsl42x_tim_t *p_hw_tim)
{
    return p_hw_tim->cnt;
}

/**
 * \brief CNT-16λģʽĴ       õǰֵ
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 * \param[in] value    : õļֵ
 *
 *\ return 
 */
am_static_inline
void amhw_zsl42x_tim_cnt16_count_set (amhw_zsl42x_tim_t *p_hw_tim, uint16_t value)
{
    p_hw_tim->cnt = value;
}

/**
 * \brief CNT32-32λģʽĴ         õǰֵ
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 *
 * \note ģʽ0 32λʱɼģʽЧģʽֹдüĴ
 *
 * \return CNT32Ĵֵ
 */
am_static_inline
uint32_t amhw_zsl42x_tim_cnt32_count_get (amhw_zsl42x_tim_t *p_hw_tim)
{
    return p_hw_tim->cnt32;
}

/**
 * \brief CNT32-32λģʽĴ       õǰֵ
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 * \param[in] value    : õļֵ
 *
 * \note ģʽ0 32λʱɼģʽЧģʽֹдüĴ
 *
 * \return 
 */
am_static_inline
void amhw_zsl42x_tim_cnt32_count_set (amhw_zsl42x_tim_t *p_hw_tim, uint32_t value)
{
    p_hw_tim->cnt32 = value;
}

/**
 * \brief ʱģʽö
 */
typedef enum {
    AMHW_ZSL42x_TIM_MODE_TIMING = 0,        /* ʱģʽ */
    AMHW_ZSL42x_TIM_MODE_PWC,               /* PWCģʽ */
    AMHW_ZSL42x_TIM_MODE_ZIGZAG,            /* ݲģʽ */
    AMHW_ZSL42x_TIM_MODE_TRIANGULAR,        /* ǲģʽ */
}amhw_zsl42x_tim_mode_t;

/**
 * \brief MODE ģʽ  ȡ
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 *
 * \note ʱͬģʽѡλ̶ٸṹԣֻĳһĴɡ
 *
 * \return õĹģʽamhw_zsl42x_tim_mode_tö
 */
am_static_inline
amhw_zsl42x_tim_mode_t amhw_zsl42x_tim_mode_get (amhw_zsl42x_tim_t *p_hw_tim)
{
    return (amhw_zsl42x_tim_mode_t)((p_hw_tim->mxcr_t.m0cr >> 12) & 0x3ul);
}

/**
 * \brief CMODE ģʽ  
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 * \param[in] mode     : õĹģʽamhw_zsl42x_tim_mode_tö
 *
 * \note ʱͬģʽѡλ̶ٸṹԣֻĳһĴɡ
 *
 * \return 
 */
am_static_inline
void amhw_zsl42x_tim_mode_set (amhw_zsl42x_tim_t     *p_hw_tim,
                               amhw_zsl42x_tim_mode_t mode)
{
    p_hw_tim->mxcr_t.m0cr = (p_hw_tim->mxcr_t.m0cr & (~(0x3ul << 12))) |
                            ((mode & 0x3ul) << 12);
}

/**
 * \brief CTEN ʱʹ
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 *
 * \note ʱͬģʽѡλ̶ٸṹԣֻĳһĴɡ
 *
 * \return 
 */
am_static_inline
void amhw_zsl42x_tim_disable (amhw_zsl42x_tim_t *p_hw_tim)
{
    p_hw_tim->mxcr_t.m0cr &= ~(0x1ul << 0);
}

/**
 * \brief CTEN ʱʹ
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 *
 * \note ʱͬģʽѡλ̶ٸṹԣֻĳһĴɡ
 *
 * \return 
 */
am_static_inline
void amhw_zsl42x_tim_enable (amhw_zsl42x_tim_t *p_hw_tim)
{
    p_hw_tim->mxcr_t.m0cr |= (0x1ul << 0);
}

/**
 * \brief ڲʱӷƵѡ
 */
typedef enum {
    AMHW_ZSL42x_TIM_CLK_DIV1 = 0,    /* 1Ƶ */
    AMHW_ZSL42x_TIM_CLK_DIV2,        /* 2Ƶ */
    AMHW_ZSL42x_TIM_CLK_DIV4,        /* 4Ƶ */
    AMHW_ZSL42x_TIM_CLK_DIV8,        /* 8Ƶ */
    AMHW_ZSL42x_TIM_CLK_DIV16,       /* 16Ƶ */
    AMHW_ZSL42x_TIM_CLK_DIV32,       /* 32Ƶ */
    AMHW_ZSL42x_TIM_CLK_DIV64,       /* 64Ƶ */
    AMHW_ZSL42x_TIM_CLK_DIV256,      /* 256Ƶ */
}amhw_zsl42x_tim_clkdiv_t;

/**
 * \brief PRS ڲʱӷƵѡ   ȡ
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 *
 * \return õķƵϵamhw_zsl42x_tim_clkdiv_tö
 */
am_static_inline
amhw_zsl42x_tim_clkdiv_t
    amhw_zsl42x_tim_mode_clkdiv_get (amhw_zsl42x_tim_t *p_hw_tim)
{
    return (amhw_zsl42x_tim_clkdiv_t)((p_hw_tim->mxcr_t.m0cr >> 4) & 0x7ul);
}

/**
 * \brief PRS ڲʱӷƵѡ   
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 * \param[in] clkdiv   : õķƵϵamhw_zsl42x_tim_clkdiv_tö
 *
 * \return 
 */
am_static_inline
void amhw_zsl42x_tim_mode_clkdiv_set (amhw_zsl42x_tim_t       *p_hw_tim,
                                      amhw_zsl42x_tim_clkdiv_t clkdiv)
{
    p_hw_tim->mxcr_t.m0cr = (p_hw_tim->mxcr_t.m0cr & (~(0x7ul << 4))) |
                            ((clkdiv & 0x7ul) << 4);
}

/**
 * \brief ʱѡ
 */
typedef enum {
    AMHW_ZSL42x_TIM_CLK_SRC_TCLK = 0,   /* ڲʱTCLK */
    AMHW_ZSL42x_TIM_CLK_SRC_ETR,        /* ⲿʱETR */
}amhw_zsl42x_tim_clk_src_t;
/**
 * \brief CT ʱѡ   ȡ
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 *
 * \return õʱԴamhw_zsl42x_tim_clk_src_tö
 */
am_static_inline
amhw_zsl42x_tim_clk_src_t
    amhw_zsl42x_tim_mode_clk_src_get (amhw_zsl42x_tim_t *p_hw_tim)
{
    return (amhw_zsl42x_tim_clk_src_t)((p_hw_tim->mxcr_t.m0cr >> 2) & 0x1ul);
}

/**
 * \brief CT ʱѡ   
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 * \param[in] src      : õʱԴamhw_zsl42x_tim_clk_src_tö
 *
 * \return 
 */
am_static_inline
void amhw_zsl42x_tim_mode_clk_src_set(amhw_zsl42x_tim_t        *p_hw_tim,
                                      amhw_zsl42x_tim_clk_src_t src)
{
    p_hw_tim->mxcr_t.m0cr = (p_hw_tim->mxcr_t.m0cr & (~(0x1ul << 2))) |
                            ((src & 0x1ul) << 2);
}

/******************************************************************************/
/***************************Timerģʽ0ƼĴ     ؿƺ**************************/
/******************************************************************************/

/**
 * \brief UIE жϽ
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 *
 * \return 
 */
am_static_inline
void amhw_zsl42x_tim_mode0_int_update_disable (amhw_zsl42x_tim_t *p_hw_tim)
{
    p_hw_tim->mxcr_t.m0cr &= ~(0x1ul << 10);
}

/**
 * \brief UIE жʹ
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 *
 * \return 
 */
am_static_inline
void amhw_zsl42x_tim_mode0_int_update_enable (amhw_zsl42x_tim_t *p_hw_tim)
{
    p_hw_tim->mxcr_t.m0cr |= (0x1ul << 10);
}

/**
 * \brief GATEP ˿GATEԿ   ߵƽ
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 *
 * \return 
 */
am_static_inline
void amhw_zsl42x_tim_mode0_gate_high (amhw_zsl42x_tim_t *p_hw_tim)
{
    p_hw_tim->mxcr_t.m0cr &= ~(0x1ul << 9);
}

/**
 * \brief GATEP ˿GATEԿ   ͵ƽ
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 *
 * \return 
 */
am_static_inline
void amhw_zsl42x_tim_mode0_gate_low (amhw_zsl42x_tim_t *p_hw_tim)
{
    p_hw_tim->mxcr_t.m0cr |= (0x1ul << 9);
}

/**
 * \brief GATE ʱſؽ
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 *
 * \return 
 */
am_static_inline
void amhw_zsl42x_tim_mode0_gate_disable (amhw_zsl42x_tim_t *p_hw_tim)
{
    p_hw_tim->mxcr_t.m0cr &= ~(0x1ul << 8);
}

/**
 * \brief GATE ʱſʹ
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 *
 * \return 
 */
am_static_inline
void amhw_zsl42x_tim_mode0_gate_enable (amhw_zsl42x_tim_t *p_hw_tim)
{
    p_hw_tim->mxcr_t.m0cr |= (0x1ul << 8);
}

/**
 * \brief TOGEN ģʽ0·ת
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 *
 * \note 1תʹ
 *       0תرCHA,CHBΪ͵ƽ
 *
 * \return 
 */
am_static_inline
void amhw_zsl42x_tim_mode0_toggle_disable (amhw_zsl42x_tim_t *p_hw_tim)
{
    p_hw_tim->mxcr_t.m0cr &= ~(0x1ul << 3);
}

/**
 * \brief TOGEN ģʽ0·תʹ
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 *
 * \note 1תʹ
 *       0תرCHA,CHBΪ͵ƽ
 *
 * \return 
 */
am_static_inline
void amhw_zsl42x_tim_mode0_toggle_enable (amhw_zsl42x_tim_t *p_hw_tim)
{
    p_hw_tim->mxcr_t.m0cr |= (0x1ul << 3);
}

/**
 * \brief ģʽѡ32ʱ/16ʱģʽѡ
 *
 * \note 32λɼ   0 - 0xffffffffΧϼģʽ0У趨ֵ
 *       16λؼ   趨ֵ - 0xffff Χ趨ֵ
 *
 *       ע⣺ʹ32λɼڹ̶Ϊ 0 - 0xffffffff  ɸġ
 *
 */
typedef enum {
    AMHW_ZSL42x_TIM_MODE0_COUNTER_32 = 0,   /* 32λɼ  */
    AMHW_ZSL42x_TIM_MODE0_COUNTER_16,       /* 16λؼ */
}amhw_zsl42x_tim_mode0_counter_t;
/**
 * \brief MD ģʽѡ32ʱ/16ʱģʽѡ   ȡ
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 *
 * \return õ32ʱ/16ʱģʽamhw_zsl42x_tim_mode0_counter_tö
 */
am_static_inline
amhw_zsl42x_tim_mode0_counter_t amhw_zsl42x_tim_mode0_counter_get (amhw_zsl42x_tim_t  *p_hw_tim)
{
    return (amhw_zsl42x_tim_mode0_counter_t)((p_hw_tim->mxcr_t.m0cr >> 1) & 0x1ul);
}

/**
 * \brief MD ģʽѡ32ʱ/16ʱģʽѡ   
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 * \param[in] src      : õ32ʱ/16ʱģʽamhw_zsl42x_tim_mode0_counter_tö
 *
 * \return 
 */
am_static_inline
void amhw_zsl42x_tim_mode0_counter_set(amhw_zsl42x_tim_t              *p_hw_tim,
                                       amhw_zsl42x_tim_mode0_counter_t src)
{
    p_hw_tim->mxcr_t.m0cr = (p_hw_tim->mxcr_t.m0cr & (~(0x1ul << 1))) |
                            ((src & 0x1ul) << 1);
}

/**
 * \brief UIF ж ־ȡ
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 *
 * \return AM_TRUE   ־Ч
 *         AM_FALSE  ־Ч
 */
am_static_inline
am_bool_t amhw_zsl42x_tim_mode0_int_update_flag_check(amhw_zsl42x_tim_t *p_hw_tim)
{
    return (p_hw_tim->ifr & 0x1ul) ? AM_TRUE : AM_FALSE;
}

/**
 * \brief UIF ж ־
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 *
 * \return 
 */
am_static_inline
void amhw_zsl42x_tim_mode0_int_update_flag_clr(amhw_zsl42x_tim_t *p_hw_tim)
{
    p_hw_tim->iclr &= ~(0x1ul << 0);
}

/**
 * \brief MOE ת
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 *
 * \note 0ת˿Ϊ״̬
 *       1ת˿Ϊ״̬
 *
 * \return 
 */
am_static_inline
void amhw_zsl42x_tim_mode0_toggle_output (amhw_zsl42x_tim_t *p_hw_tim)
{
    p_hw_tim->dtr |= (0x1ul << 12);
}

/**
 * \brief MOE ת
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 *
 * \note 0ת˿Ϊ״̬
 *       1ת˿Ϊ״̬
 *
 * \return 
 */
am_static_inline
void amhw_zsl42x_tim_mode0_toggle_input (amhw_zsl42x_tim_t *p_hw_tim)
{
    p_hw_tim->dtr |= (0x1ul << 12);
}


/******************************************************************************/
/***************************Timerģʽ1ƼĴ     ؿƺ**************************/
/******************************************************************************/
/**
 * \brief Oneshot δģʽѡ   δ
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 *
 * \note 1һԶ ٴβҪʹCTEN
 *       0ѭ
 *
 * \return 
 */
am_static_inline
void amhw_zsl42x_tim_mode1_trigger_single (amhw_zsl42x_tim_t *p_hw_tim)
{
    p_hw_tim->mxcr_t.m1cr |= (0x1ul << 14);
}

/**
 * \brief Oneshot δģʽѡ  ѭ
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 *
 * \note 1һԶ ٴβҪʹCTEN
 *       0ѭ
 *
 * \return 
 */
am_static_inline
void amhw_zsl42x_tim_mode1_trigger_cyclic (amhw_zsl42x_tim_t *p_hw_tim)
{
    p_hw_tim->mxcr_t.m1cr &= ~(0x1ul << 14);
}

/**
 * \brief UIE жϽ
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 *
 * \return 
 */
am_static_inline
void amhw_zsl42x_tim_mode1_int_update_disable (amhw_zsl42x_tim_t *p_hw_tim)
{
    p_hw_tim->mxcr_t.m1cr &= ~(0x1ul << 10);
}

/**
 * \brief UIE жʹ
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 *
 * \return 
 */
am_static_inline
void amhw_zsl42x_tim_mode1_int_update_enable (amhw_zsl42x_tim_t *p_hw_tim)
{
    p_hw_tim->mxcr_t.m1cr |= (0x1ul << 10);
}

/**
 * \brief CIEA ȲжϽ
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 *
 * \return 
 */
am_static_inline
void amhw_zsl42x_tim_mode1_int_edge_finish_disable (amhw_zsl42x_tim_t *p_hw_tim)
{
    p_hw_tim->crchx[0] &= ~(0x1ul << 10);
}

/**
 * \brief CIEA Ȳжʹ
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 *
 * \return 
 */
am_static_inline
void amhw_zsl42x_tim_mode1_int_edge_finish_enable (amhw_zsl42x_tim_t *p_hw_tim)
{
    p_hw_tim->crchx[0] |= (0x1ul << 10);
}

/**
 * \brief Ȳѡ
 */
typedef enum {
    AMHW_ZSL42x_TIM_PULSE_CYCLE_RISING,    /* - */
    AMHW_ZSL42x_TIM_PULSE_WIDTH_LOW,       /* ͵ƽ */
    AMHW_ZSL42x_TIM_PULSE_WIDTH_HIGH,      /* ߵƽ */
    AMHW_ZSL42x_TIM_PULSE_CYCLE_FALLING,   /* - */
}amhw_zsl42x_tim_measure_edge_t;
/**
 * \brief Edg2nd Edg1st Ȳѡ  ȡ
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 *
 * \return Ȳѡ(amhw_zsl42x_tim_measure_edge_tö)
 */
am_static_inline
amhw_zsl42x_tim_measure_edge_t
    amhw_zsl42x_tim_mode1_measure_edge_get (amhw_zsl42x_tim_t *p_hw_tim)
{
    return (amhw_zsl42x_tim_measure_edge_t)((p_hw_tim->mxcr_t.m1cr >> 8) & 0x3ul);
}

/**
 * \brief Edg2nd Edg1st Ȳѡ  
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 * \param[in] choice   : Ȳѡ(amhw_zsl42x_tim_measure_edge_tö)
 *
 * \return 
 */
am_static_inline
void amhw_zsl42x_tim_mode1_measure_edge_set(
    amhw_zsl42x_tim_t             *p_hw_tim,
    amhw_zsl42x_tim_measure_edge_t choice)
{
    p_hw_tim->mxcr_t.m1cr =(p_hw_tim->mxcr_t.m1cr & (~(0x3ul << 8))) |
                           ((choice & 0x3ul) << 8);
}

/**
 * \brief UIF ж ־ȡ
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 *
 * \return AM_TRUE   ־Ч
 *         AM_FALSE  ־Ч
 */
am_static_inline
am_bool_t amhw_zsl42x_tim_mode1_int_update_flag_check(
    amhw_zsl42x_tim_t *p_hw_tim)
{
    return (p_hw_tim->ifr & 0x1ul) ? AM_TRUE : AM_FALSE;
}

/**
 * \brief UIF ж ־
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 *
 * \return 
 */
am_static_inline
void amhw_zsl42x_tim_mode1_int_update_flag_clr(amhw_zsl42x_tim_t *p_hw_tim)
{
    p_hw_tim->iclr &= ~(0x1ul << 0);
}

/**
 * \brief UIF Ȳж   ־ȡ
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 *
 * \return AM_TRUE   ־Ч
 *         AM_FALSE  ־Ч
 */
am_static_inline
am_bool_t amhw_zsl42x_tim_mode1_int_edge_flag_check(amhw_zsl42x_tim_t *p_hw_tim)
{
    return ((p_hw_tim->ifr >> 2) & 0x1ul) ? AM_TRUE : AM_FALSE;
}

/**
 * \brief UIF Ȳж   ־
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 *
 * \return 
 */
am_static_inline
void amhw_zsl42x_tim_mode1_int_edge_flag_clr (amhw_zsl42x_tim_t *p_hw_tim)
{
    p_hw_tim->iclr &= ~(0x1ul << 2);
}

/**
 * \brief CCR0A Ȳ
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 *
 * \return  Ȳ
 */
am_static_inline
uint16_t amhw_zsl42x_tim_mode1_measure_data_get (amhw_zsl42x_tim_t *p_hw_tim)
{
    return p_hw_tim->ccr[0] & 0xfffful;
}

/**
 * \brief IB0S CH0Bѡ      CH0B
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 *
 * \note 0CH0B
 *       1ڲTSѡź
 *
 * \note PWMʱԶѡGATE˿ΪCH0B
 *
 * \return 
 */
am_static_inline
void amhw_zsl42x_tim_mode1_ch0b_in_ch0b (amhw_zsl42x_tim_t *p_hw_tim)
{
    p_hw_tim->mscr &= ~(0x1ul << 12);
}

/**
 * \brief IB0S CH0Bѡ     ڲTSѡź
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 *
 * \note 0CH0B
 *       1ڲTSѡź
 *
 * \note PWMʱԶѡGATE˿ΪCH0B
 *
 * \return 
 */
am_static_inline
void amhw_zsl42x_tim_mode1_ch0b_in_ts (amhw_zsl42x_tim_t *p_hw_tim)
{
    p_hw_tim->mscr |= (0x1ul << 12);
}

/**
 * \brief IA0S  CH0Aѡ      CH0A
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 *
 * \note 0CH0A
 *       1CH0A CH1A CH2A XOR(TIM3)              CH0A ETR GATE XOR(TIM0)
 *
 * \note Ϊ1˿ڵһ˿ڱ仯仯
 *
 * \return 
 */
am_static_inline
void amhw_zsl42x_tim_mode1_ch0a_in_ch0a (amhw_zsl42x_tim_t *p_hw_tim)
{
    p_hw_tim->mscr &= ~(0x1ul << 11);
}

/**
 * \brief IA0S  CH0Aѡ     ڲTSѡź
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 *
 * \note 0CH0A
 *       1CH0A CH1A CH2A XOR(TIM3)              CH0A ETR GATE XOR(TIM0)
 *
 * \note Ϊ1˿ڵһ˿ڱ仯仯
 *
 * \return 
 */
am_static_inline
void amhw_zsl42x_tim_mode1_ch0a_in_other (amhw_zsl42x_tim_t *p_hw_tim)
{
    p_hw_tim->mscr |= (0x1ul << 11);
}

/**
 * \brief ѡ
 */
typedef enum {
    AMHW_ZSL42x_TIM_TS_SRC_ETR_ETFP = 0,    /* ˿ETR˲λѡźETFP */
    AMHW_ZSL42x_TIM_TS_SRC_ITR0,            /* ڲź ITR0 */
    AMHW_ZSL42x_TIM_TS_SRC_ITR1,            /* ڲź ITR1 */
    AMHW_ZSL42x_TIM_TS_SRC_ITR2,            /* ڲź ITR2 */
    AMHW_ZSL42x_TIM_TS_SRC_ITR3,            /* ڲź ITR3 */
    AMHW_ZSL42x_TIM_TS_SRC_CH0A_EDGE,       /* ˿CH0Aıź */
    AMHW_ZSL42x_TIM_TS_SRC_CH0A_IAFP,       /* ˿CH0A˲λѡźIAFP */
    AMHW_ZSL42x_TIM_TS_SRC_CH0B_IBFP,       /* ˿CH0B˲λѡźIBFP */
}amhw_zsl42x_tim_ts_src_t;
/**
 * \brief TS ѡ  ȡ
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 *
 * \return ѡ(amhw_zsl42x_tim_ts_src_tö)
 */
am_static_inline
amhw_zsl42x_tim_ts_src_t
    amhw_zsl42x_tim_mode1_ts_src_get (amhw_zsl42x_tim_t *p_hw_tim)
{
    return (amhw_zsl42x_tim_ts_src_t)((p_hw_tim->mscr >> 5) & 0x7ul);
}

/**
 * \brief TS ѡ  ȡ
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 * \param[in] choice   : ѡ(amhw_zsl42x_tim_ts_src_tö)
 *
 * \return 
 */
am_static_inline
void amhw_zsl42x_tim_mode1_ts_src_set(amhw_zsl42x_tim_t       *p_hw_tim,
                                      amhw_zsl42x_tim_ts_src_t choice)
{
    p_hw_tim->mscr =(p_hw_tim->mscr & (~(0x7ul << 5))) |
                    ((choice & 0x7ul) << 5);
}

/**
 * \brief ETP  ETRλѡ      ͬλ
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 *
 * \return 
 */
am_static_inline
void amhw_zsl42x_tim_mode1_etr_in_phase_same (amhw_zsl42x_tim_t *p_hw_tim)
{
    p_hw_tim->fltr &= ~(0x1ul << 31);
}

/**
 * \brief ETP  ETRλѡ         
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 *
 * \return 
 */
am_static_inline
void amhw_zsl42x_tim_mode1_etr_in_phase_reversed (amhw_zsl42x_tim_t *p_hw_tim)
{
    p_hw_tim->fltr |= (0x1ul << 31);
}

/**
 * \brief ˲ƣ
 */
typedef enum {
    AMHW_ZSL42x_TIM_FILTER_NO = 0,          /* ˲Ч */
    AMHW_ZSL42x_TIM_FILTER_PCLKDIV1_3 = 4,  /* pclk 3Ч */
    AMHW_ZSL42x_TIM_FILTER_PCLKDIV4_3,      /* pclk/4 3Ч */
    AMHW_ZSL42x_TIM_FILTER_PCLKDIV16_3,     /* pclk/16 3Ч */
    AMHW_ZSL42x_TIM_FILTER_PCLKDIV64_3,     /* pclk/64 3Ч */
}amhw_zsl42x_tim_filter_t;

/**
 * \brief FLTET  ETR˲    ȡ
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 *
 * \return ˲(amhw_zsl42x_tim_filter_tö)
 */
am_static_inline
amhw_zsl42x_tim_filter_t
    amhw_zsl42x_tim_mode1_etr_fliter_get (amhw_zsl42x_tim_t *p_hw_tim)
{
    return (amhw_zsl42x_tim_filter_t)(amhw_zsl42x_tim_filter_t)((p_hw_tim->fltr >> 28) & 0x7ul);
}

/**
 * \brief FLTET  ETR˲     
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 * \param[in] choice   : ˲(amhw_zsl42x_tim_filter_tö)
 *
 * \return 
 */
am_static_inline
void amhw_zsl42x_tim_mode1_etr_fliter_set(amhw_zsl42x_tim_t       *p_hw_tim,
                                          amhw_zsl42x_tim_filter_t choice)
{
    p_hw_tim->fltr =(p_hw_tim->fltr & (~(0x7ul << 28))) |
                    ((choice & 0x7ul) << 28);
}

/**
 * \brief FLTB0  CHB˲    ȡ
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 *
 * \return ˲(amhw_zsl42x_tim_filter_tö)
 */
am_static_inline
amhw_zsl42x_tim_filter_t
    amhw_zsl42x_tim_mode1_chb_fliter_get (amhw_zsl42x_tim_t *p_hw_tim)
{
    return (amhw_zsl42x_tim_filter_t)((p_hw_tim->fltr >> 4) & 0x7ul);
}

/**
 * \brief FLTB0  CHB˲     
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 * \param[in] choice   : ˲(amhw_zsl42x_tim_filter_tö)
 *
 * \return 
 */
am_static_inline
void amhw_zsl42x_tim_mode1_chb_fliter_set(amhw_zsl42x_tim_t       *p_hw_tim,
                                          amhw_zsl42x_tim_filter_t choice)
{
    p_hw_tim->fltr =(p_hw_tim->fltr & (~(0x7ul << 4))) |
                    ((choice & 0x7ul) << 4);
}

/**
 * \brief FLTA0  CHA˲    ȡ
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 *
 * \return ˲(amhw_zsl42x_tim_filter_tö)
 */
am_static_inline
amhw_zsl42x_tim_filter_t
    amhw_zsl42x_tim_mode1_cha_fliter_get (amhw_zsl42x_tim_t *p_hw_tim)
{
    return (amhw_zsl42x_tim_filter_t)((p_hw_tim->fltr >> 0) & 0x7ul);
}

/**
 * \brief FLTA0  CHA˲    ȡ
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 * \param[in] choice   : ˲(amhw_zsl42x_tim_filter_tö)
 *
 * \return 
 */
am_static_inline
void amhw_zsl42x_tim_mode1_cha_fliter_set(amhw_zsl42x_tim_t       *p_hw_tim,
                                          amhw_zsl42x_tim_filter_t choice)
{
    p_hw_tim->fltr =(p_hw_tim->fltr & (~(0x7ul << 0))) |
                    ((choice & 0x7ul) << 0);
}




/******************************************************************************/
/***************************Timerģʽ23ƼĴ     ؿƺ*************************/
/******************************************************************************/
/**
 * \brief DIR  ϼ
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 *
 * \note ֻھݲģʽ¿дģʽֻдЧ.
 *       ģʽлĶģʽDIRԶ0¼ºʹģʽⲿλģʽDIRԶ㡣
 *
 * \return 
 */
am_static_inline
void amhw_zsl42x_tim_mode23_counter_up (amhw_zsl42x_tim_t  *p_hw_tim)
{
    p_hw_tim->mxcr_t.m23cr &= ~(0x1ul << 27);
}

/**
 * \brief DIR  ¼
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 *
 * \note ֻھݲģʽ¿дģʽֻдЧ.
 *       ģʽлĶģʽDIRԶ0¼ºʹģʽⲿλģʽDIRԶ㡣
 *
 * \return 
 */
am_static_inline
void amhw_zsl42x_tim_mode23_counter_down (amhw_zsl42x_tim_t  *p_hw_tim)
{
    p_hw_tim->mxcr_t.m23cr |= (0x1ul << 27);
}

/**
 * \brief BG  ɲ
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 *
 * \note Զ㣬д0Ч
 *
 * \return 
 */
am_static_inline
void amhw_zsl42x_tim_mode23_soft_brake (amhw_zsl42x_tim_t  *p_hw_tim)
{
    p_hw_tim->mxcr_t.m23cr |= (0x1ul << 26);
}

/**
 * \brief UG  
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 *
 * \note Զ㣬д0Ч
 *       ʼ»ĴӦĴʹܣԤƵҲᱻ
 *
 * \return 
 */
am_static_inline
void amhw_zsl42x_tim_mode23_soft_update (amhw_zsl42x_tim_t  *p_hw_tim)
{
    p_hw_tim->mxcr_t.m23cr |= (0x1ul << 25);
}

/**
 * \brief TG  
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 *
 * \note Զ㣬д0Ч
 *       ҪڴģʽSMS=2mode=2/3¶Դ
 *
 * \return 
 */
am_static_inline
void amhw_zsl42x_tim_mode23_soft_trigger (amhw_zsl42x_tim_t  *p_hw_tim)
{
    p_hw_tim->mxcr_t.m23cr |= (0x1ul << 24);
}

/**
 * \brief ĶAȽжģʽ
 */
typedef enum {
    AMHW_ZSL42x_TIM_CISA_INT_MODE_NO = 0,  /* ж */
    AMHW_ZSL42x_TIM_CISA_INT_MODE_RISING,  /* ж */
    AMHW_ZSL42x_TIM_CISA_INT_MODE_FALLING, /* ж */
    AMHW_ZSL42x_TIM_CISA_INT_MODE_BOTH,    /* ضж */
}amhw_zsl42x_tim_int_mode_t;

/**
 * \brief CIS ĶAȽжģʽ
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 * \param[in] mode     : amhw_zsl42x_tim_int_mode_tö
 *
 * \note ĶAȽжģʽ(BȽжϵCRxĴCISB)
 *
 * \return 
 */
am_static_inline
void amhw_zsl42x_tim_mode23_int_mode_set(amhw_zsl42x_tim_t         *p_hw_tim,
                                         amhw_zsl42x_tim_int_mode_t mode )
{
    p_hw_tim->mxcr_t.m23cr = (p_hw_tim->mxcr_t.m23cr & (~(0x3ul << 21))) |
                             ((mode & 0x3ul) << 21);
}

#define AMHW_ZSL42x_TIM_INT_BIE     (0x1UL << 20)  /* ɲжʹ */
#define AMHW_ZSL42x_TIM_INT_TIE     (0x1UL << 19)  /* жʹ */
#define AMHW_ZSL42x_TIM_INT_UIE     (0x1UL << 10)  /* жʹ */
#define AMHW_ZSL42x_TIM_INT_ALL     0xFF           /* ж */

/**
 * \brief 궨ʹ
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 * \param[in] type     : AMHW_ZSL42x_TIM_INT_BIE   
 *                       AMHW_ZSL42x_TIM_INT_TIE   
 *                       AMHW_ZSL42x_TIM_INT_UIE   
 *                       AMHW_ZSL42x_TIM_INT_ALL
 *
 * \return 
 */
am_static_inline
void amhw_zsl42x_tim_mode23_int_enable (amhw_zsl42x_tim_t *p_hw_tim,
                                        uint32_t           type)
{
    if(0xFF == type) {
        p_hw_tim->mxcr_t.m23cr |= AMHW_ZSL42x_TIM_INT_BIE;
        p_hw_tim->mxcr_t.m23cr |= AMHW_ZSL42x_TIM_INT_TIE;
        p_hw_tim->mxcr_t.m23cr |= AMHW_ZSL42x_TIM_INT_UIE;
    }
    else {
        p_hw_tim->mxcr_t.m23cr |= type;
    }
}

/**
 * \brief 궨
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 * \param[in] type     : AMHW_ZSL42x_TIM_INT_BIE   
 *                       AMHW_ZSL42x_TIM_INT_TIE   
 *                       AMHW_ZSL42x_TIM_INT_UIE   
 *                       AMHW_ZSL42x_TIM_INT_ALL
 *
 * \return 
 */
am_static_inline
void amhw_zsl42x_tim_mode23_int_disable (amhw_zsl42x_tim_t  *p_hw_tim,
                                     uint32_t           type)
{
    if(0xFF == type) {
        p_hw_tim->mxcr_t.m23cr &= ~AMHW_ZSL42x_TIM_INT_BIE;
        p_hw_tim->mxcr_t.m23cr &= ~AMHW_ZSL42x_TIM_INT_TIE;
        p_hw_tim->mxcr_t.m23cr &= ~AMHW_ZSL42x_TIM_INT_UIE;
    }
    else {
        p_hw_tim->mxcr_t.m23cr &= ~type;
    }
}

#define AMHW_ZSL42x_TIM_OCCE    (0x1UL << 23)  /* OCREFʹʹ */
#define AMHW_ZSL42x_TIM_TDE     (0x1UL << 18)  /* DMAʹ */
#define AMHW_ZSL42x_TIM_UDE     (0x1UL << 11)  /* DMAʹ */
#define AMHW_ZSL42x_TIM_BUFPEN  (0x1UL << 7)   /* ػʹ */
#define AMHW_ZSL42x_TIM_PWM2S   (0x1UL << 3)   /* OCREFAȽʹ */
#define AMHW_ZSL42x_TIM_COMP    (0x1UL << 1)   /* PWMʹ */
#define AMHW_ZSL42x_TIM_ALL     0XFF           /* ʹܣʱʹܳ⣩*/

/**
 * \brief 궨ʹ
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 * \param[in] type     : AMHW_ZSL42x_TIM_OCCE  
 *                       AMHW_ZSL42x_TIM_TDE   
 *                       AMHW_ZSL42x_TIM_UDE   
 *                       AMHW_ZSL42x_TIM_BUFPEN   
 *                       AMHW_ZSL42x_TIM_PWM2S   
 *                       AMHW_ZSL42x_TIM_COMP   
 *                       AMHW_ZSL42x_TIM_ALL
 *
 *
 * \note 1 OCREF_CLRźſOCREF
 *       0 OCREFOCREF_CLRӰ
 *
 * \return 
 */
am_static_inline
void amhw_zsl42x_tim_mode23_enable (amhw_zsl42x_tim_t *p_hw_tim,
                                    uint32_t           type)
{
    if(0xFF == type) {
        p_hw_tim->mxcr_t.m23cr |= AMHW_ZSL42x_TIM_OCCE;
        p_hw_tim->mxcr_t.m23cr |= AMHW_ZSL42x_TIM_TDE;
        p_hw_tim->mxcr_t.m23cr |= AMHW_ZSL42x_TIM_UDE;
    }
    else {
        p_hw_tim->mxcr_t.m23cr |= type;
    }
}

/**
 * \brief 궨
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 * \param[in] type     : AMHW_ZSL42x_TIM_OCCE  
 *                       AMHW_ZSL42x_TIM_TDE   
 *                       AMHW_ZSL42x_TIM_UDE   
 *                       AMHW_ZSL42x_TIM_BUFPEN   
 *                       AMHW_ZSL42x_TIM_PWM2S   
 *                       AMHW_ZSL42x_TIM_COMP   
 *                       AMHW_ZSL42x_TIM_ALL
 *
 * \note 1 OCREF_CLRźſOCREF
 *       0 OCREFOCREF_CLRӰ
 *
 * \return 
 */
am_static_inline
void amhw_zsl42x_tim_mode23_disable (amhw_zsl42x_tim_t  *p_hw_tim,
                                     uint32_t           type)
{
    if(0xFF == type) {
        p_hw_tim->mxcr_t.m23cr &= ~AMHW_ZSL42x_TIM_OCCE;
        p_hw_tim->mxcr_t.m23cr &= ~AMHW_ZSL42x_TIM_TDE;
        p_hw_tim->mxcr_t.m23cr &= ~AMHW_ZSL42x_TIM_UDE;
        p_hw_tim->mxcr_t.m23cr &= ~AMHW_ZSL42x_TIM_BUFPEN;
        p_hw_tim->mxcr_t.m23cr &= ~AMHW_ZSL42x_TIM_PWM2S;
        p_hw_tim->mxcr_t.m23cr &= ~AMHW_ZSL42x_TIM_COMP;
    }
    else {
        p_hw_tim->mxcr_t.m23cr &= ~type;
    }
}

/**
 * \brief URS Դ    0
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 *
 * \note 0//UG/ģʽλ
 *       1/
 *
 * \return 
 */
am_static_inline
void amhw_zsl42x_tim_mode23_uodate_src_0 (amhw_zsl42x_tim_t  *p_hw_tim)
{
    p_hw_tim->mxcr_t.m23cr &= ~(0x1UL << 17);
}

/**
 * \brief URS Դ    1
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 *
 * \note 0//UG/ģʽλ
 *       1/
 *
 * \return 
 */
am_static_inline
void amhw_zsl42x_tim_mode23_uodate_src_1 (amhw_zsl42x_tim_t  *p_hw_tim)
{
    p_hw_tim->mxcr_t.m23cr |= (0x1UL << 17);
}

/**
 * \brief OCCS  OCREFԴѡ    0
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 *
 * \note 0ѹȽVC VCѡVCx_OUTCFGĴ
 *       1ETR˿˲λѡź
 *
 *       OCCEЧʱ OC_clrOCREFıȽźΪ㣬OCMx>1ʱЧ 
 *       һuev¼Ƚ
 *
 * \return 
 */
am_static_inline
void amhw_zsl42x_tim_mode23_occref_src_0 (amhw_zsl42x_tim_t  *p_hw_tim)
{
    p_hw_tim->mxcr_t.m23cr &= ~(0x1UL << 16);
}

/**
 * \brief OCCS  OCREFԴѡ    1
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 *
 * \note 0//UG/ģʽλ
 *       1/
 *
 *       OCCEЧʱ OC_clrOCREFıȽźΪ㣬OCMx>1ʱЧ 
 *       һuev¼Ƚ
 *
 * \return 
 */
am_static_inline
void amhw_zsl42x_tim_mode23_occref_src_1 (amhw_zsl42x_tim_t  *p_hw_tim)
{
    p_hw_tim->mxcr_t.m23cr |= (0x1UL << 16);
}

/**
 * \brief CSG  GATE PWMģʽ²/Ƚѡ     
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 *
 * \note ֻPWMʱЧ,ʹCCR0BΪGATEıȽϻ򲶻ͨ
 *
 * \return 
 */
am_static_inline
void amhw_zsl42x_tim_mode23_gate_set_cap (amhw_zsl42x_tim_t  *p_hw_tim)
{
    p_hw_tim->mxcr_t.m23cr |= (0x1UL << 15);
}

/**
 * \brief CSG  GATE PWMģʽ²/Ƚѡ     Ƚ
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 *
 * \note ֻPWMʱЧ,ʹCCR0BΪGATEıȽϻ򲶻ͨ
 *
 * \return 
 */
am_static_inline
void amhw_zsl42x_tim_mode23_gate_set_cmp (amhw_zsl42x_tim_t  *p_hw_tim)
{
    p_hw_tim->mxcr_t.m23cr &= ~(0x1UL << 15);
}

/**
 * \brief CFG  GATEΪʱ زЧ     ȡ
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 *
 * \note ֻPWMʱЧ
 *
 * \return AM_TRUE :زЧ
 *         AM_FALSE:زЧ
 */
am_static_inline
am_bool_t amhw_zsl42x_tim_mode23_cfg_get (amhw_zsl42x_tim_t  *p_hw_tim)
{
    return ((p_hw_tim->mxcr_t.m23cr >> 9) & 0x1ul) ? AM_TRUE : AM_FALSE;
}

/**
 * \brief CFG  GATEΪʱ زЧ     Ч
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 *
 * \note ֻPWMʱЧ
 *
 * \return 
 */
am_static_inline
void amhw_zsl42x_tim_mode23_cfg_enable (amhw_zsl42x_tim_t  *p_hw_tim)
{
    p_hw_tim->mxcr_t.m23cr |= (0x1UL << 9);
}

/**
 * \brief CFG  GATEΪʱ زЧ    Ч
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 *
 * \note ֻPWMʱЧ
 *
 * \return 
 */
am_static_inline
void amhw_zsl42x_tim_mode23_cfg_disable (amhw_zsl42x_tim_t  *p_hw_tim)
{
    p_hw_tim->mxcr_t.m23cr &= ~(0x1UL << 9);
}

/**
 * \brief CFG  GATEΪʱ زЧ     ȡ
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 *
 * \note ֻPWMʱЧ
 *
 * \return AM_TRUE :زЧ
 *         AM_FALSE:زЧ
 */
am_static_inline
am_bool_t amhw_zsl42x_tim_mode23_crg_get (amhw_zsl42x_tim_t  *p_hw_tim)
{
    return ((p_hw_tim->mxcr_t.m23cr >> 8) & 0x1ul) ? AM_TRUE : AM_FALSE;
}

/**
 * \brief CFG  GATEΪʱ زЧ     Ч
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 *
 * \note ֻPWMʱЧ
 *
 * \return 
 */
am_static_inline
void amhw_zsl42x_tim_mode23_crg_enable (amhw_zsl42x_tim_t  *p_hw_tim)
{
    p_hw_tim->mxcr_t.m23cr |= (0x1UL << 8);
}

/**
 * \brief CFG  GATEΪʱ زЧ    Ч
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 *
 * \note ֻPWMʱЧ
 *
 * \return 
 */
am_static_inline
void amhw_zsl42x_tim_mode23_crg_disable (amhw_zsl42x_tim_t  *p_hw_tim)
{
    p_hw_tim->mxcr_t.m23cr &= ~(0x1UL << 8);
}

/**
 * \brief Oneshot δģʽѡ   δ
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 *
 * \note 1¼ºʱֹͣ
 *       0ѭ
 *
 * \return 
 */
am_static_inline
void amhw_zsl42x_tim_mode23_trigger_single (amhw_zsl42x_tim_t *p_hw_tim)
{
    p_hw_tim->mxcr_t.m23cr |= (0x1ul << 14);
}

/**
 * \brief Oneshot δģʽѡ  ѭ
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 *
 * \note 1¼ºʱֹͣ
 *       0ѭ
 *
 * \return 
 */
am_static_inline
void amhw_zsl42x_tim_mode23_trigger_cyclic (amhw_zsl42x_tim_t *p_hw_tim)
{
    p_hw_tim->mxcr_t.m23cr &= ~(0x1ul << 14);
}


#define AMHW_ZSL42x_TIM_INT_FLAG_TRIGGER   15    /* жϱ־ */
#define AMHW_ZSL42x_TIM_INT_FLAG_BRAKE     14    /* ɲжϱ־ */
#define AMHW_ZSL42x_TIM_INT_FLAG_CB2E      13    /* ͨCB2Eݶʧ־ */
#define AMHW_ZSL42x_TIM_INT_FLAG_CB1E      12    /* ͨCB1Eݶʧ־ */
#define AMHW_ZSL42x_TIM_INT_FLAG_CB0E      11    /* ͨCB0Eݶʧ־ */
#define AMHW_ZSL42x_TIM_INT_FLAG_CA2E      10    /* ͨCA2Eݶʧ־ */
#define AMHW_ZSL42x_TIM_INT_FLAG_CA1E      9     /* ͨCA1Eݶʧ־ */
#define AMHW_ZSL42x_TIM_INT_FLAG_CA0E      8     /* ͨCA0Eݶʧ־ */
#define AMHW_ZSL42x_TIM_INT_FLAG_CB2F      7     /* ͨCH2B/Ƚƥ־ */
#define AMHW_ZSL42x_TIM_INT_FLAG_CB1F      6     /* ͨCH1B/Ƚƥ־ */
#define AMHW_ZSL42x_TIM_INT_FLAG_CB0F      5     /* ͨCH0B/Ƚƥ־ */
#define AMHW_ZSL42x_TIM_INT_FLAG_CA2F      4     /* ͨCH2A/Ƚƥ־ */
#define AMHW_ZSL42x_TIM_INT_FLAG_CA1F      3     /* ͨCH1A/Ƚƥ־ */
#define AMHW_ZSL42x_TIM_INT_FLAG_CA0F      2     /* ͨCH0A/Ƚƥ־ */
#define AMHW_ZSL42x_TIM_INT_FLAG_UPDATE    0     /* ¼жϱ־ */
#define AMHW_ZSL42x_TIM_INT_FLAG_ALL     0xffff
/**
 * \brief ģʽ23  ־ȡ
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 * \param[in] flag     : Ӧ궨
 *                       AMHW_ZSL42x_TIM_INT_FLAG_TRIGGER
 *                       .
 *                       .
 *                       .
 *                       AMHW_ZSL42x_TIM_INT_FLAG_UPDATE
 *
 * \return AM_TRUE   ־Ч
 *         AM_FALSE  ־Ч
 */
am_static_inline
am_bool_t amhw_zsl42x_tim_mode23_int_flag_check(amhw_zsl42x_tim_t *p_hw_tim,
                                                uint32_t           flag)
{
    return (p_hw_tim->ifr  & (0x1ul << flag)) ? AM_TRUE : AM_FALSE;
}

/**
 * \brief ģʽ23   ־
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 * \param[in] flag     : Ӧ궨
 *                       AMHW_ZSL42x_TIM_INT_FLAG_TRIGGER
 *                       .
 *                       .
 *                       .
 *                       AMHW_ZSL42x_TIM_INT_FLAG_UPDATE
 *
 * \return 
 */
am_static_inline
void amhw_zsl42x_tim_mode23_int_flag_clr (amhw_zsl42x_tim_t *p_hw_tim,
                                          uint32_t           flag)
{
    if(flag == 0xffff) {
        p_hw_tim->iclr = 0;
    } else {
        p_hw_tim->iclr &= ~(0x1ul << flag);
    }
}

/**
 * \brief IB0S CH0Bѡ      CH0B
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 *
 * \note 0CH0B
 *       1ڲTSѡź
 *
 * \note PWMʱԶѡGATE˿ΪCH0B
 *
 * \return 
 */
am_static_inline
void amhw_zsl42x_tim_mode23_ch0b_in_ch0b (amhw_zsl42x_tim_t *p_hw_tim)
{
    p_hw_tim->mscr &= ~(0x1ul << 12);
}

/**
 * \brief IB0S CH0Bѡ     ڲTSѡź
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 *
 * \note 0CH0B
 *       1ڲTSѡź
 *
 * \note PWMʱԶѡGATE˿ΪCH0B
 *
 * \return 
 */
am_static_inline
void amhw_zsl42x_tim_mode23_ch0b_in_ts (amhw_zsl42x_tim_t *p_hw_tim)
{
    p_hw_tim->mscr |= (0x1ul << 12);
}

/**
 * \brief IA0S  CH0Aѡ      CH0A
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 *
 * \note 0CH0A
 *       1CH0A CH1A CH2A XOR(TIM3)              CH0A ETR GATE XOR(TIM0)
 *
 * \note Ϊ1˿ڵһ˿ڱ仯仯
 *
 * \return 
 */
am_static_inline
void amhw_zsl42x_tim_mode23_ch0a_in_ch0a (amhw_zsl42x_tim_t *p_hw_tim)
{
    p_hw_tim->mscr &= ~(0x1ul << 11);
}

/**
 * \brief IA0S  CH0Aѡ     ڲTSѡź
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 *
 * \note 0CH0A
 *       1CH0A CH1A CH2A XOR(TIM3)              CH0A ETR GATE XOR(TIM0)
 *
 * \note Ϊ1˿ڵһ˿ڱ仯仯
 *
 * \return 
 */
am_static_inline
void amhw_zsl42x_tim_mode23_ch0a_in_other (amhw_zsl42x_tim_t *p_hw_tim)
{
    p_hw_tim->mscr |= (0x1ul << 11);
}

/**
 * \brief ģʽѡ
 */
typedef enum {
    AMHW_ZSL42x_TIM_SLAVER_MODE_INCLK = 0,  /* ʹڲʱ */
    AMHW_ZSL42x_TIM_SLAVER_MODE_RESET,      /* λ */
    AMHW_ZSL42x_TIM_SLAVER_MODE_TRIGGER,    /* ģʽ */
    AMHW_ZSL42x_TIM_SLAVER_MODE_OUTCLK,     /* ⲿʱģʽ */
    AMHW_ZSL42x_TIM_SLAVER_MODE_COUNTER1,   /* ģʽ1 */
    AMHW_ZSL42x_TIM_SLAVER_MODE_COUNTER2,   /* ģʽ2 */
    AMHW_ZSL42x_TIM_SLAVER_MODE_COUNTER3,   /* ģʽ3 */
    AMHW_ZSL42x_TIM_SLAVER_MODE_GATE,       /* ſع */
}amhw_zsl42x_tim_slaver_mode_t;

/**
 * \brief SMS ģʽѡ  ȡ
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 *
 * \return ģʽѡ(amhw_zsl42x_tim_slaver_mode_tö)
 */
am_static_inline
amhw_zsl42x_tim_slaver_mode_t
    amhw_zsl42x_tim_mode23_slaver_mode_get (amhw_zsl42x_tim_t *p_hw_tim)
{
    return (amhw_zsl42x_tim_slaver_mode_t)((p_hw_tim->mscr >> 8) & 0x7ul);
}

/**
 * \brief SMS ģʽѡ  
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 * \param[in] choice   : ģʽѡ(amhw_zsl42x_tim_slaver_mode_tö)
 *
 * \return 
 */
am_static_inline
void amhw_zsl42x_tim_mode23_slaver_mode_set(amhw_zsl42x_tim_t         *p_hw_tim,
                                            amhw_zsl42x_tim_slaver_mode_t choice)
{
    p_hw_tim->mscr =(p_hw_tim->mscr & (~(0x7ul << 8))) |
                    ((choice & 0x7ul) << 8);
}

/**
 * \brief TS ѡ  ȡ
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 *
 * \return ѡ(amhw_zsl42x_tim_ts_src_tö)
 */
am_static_inline
amhw_zsl42x_tim_ts_src_t
    amhw_zsl42x_tim_mode23_ts_src_get (amhw_zsl42x_tim_t *p_hw_tim)
{
    return (amhw_zsl42x_tim_ts_src_t)((p_hw_tim->mscr >> 5) & 0x7ul);
}

/**
 * \brief TS ѡ 
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 * \param[in] choice   : ѡ(amhw_zsl42x_tim_ts_src_tö)
 *
 * \return 
 */
am_static_inline
void amhw_zsl42x_tim_mode23_ts_src_set(amhw_zsl42x_tim_t       *p_hw_tim,
                                      amhw_zsl42x_tim_ts_src_t choice)
{
    p_hw_tim->mscr =(p_hw_tim->mscr & (~(0x7ul << 5))) |
                    ((choice & 0x7ul) << 5);
}

/**
 * \brief MSM ѡ  
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 *
 * \note  0:ʱ
 *        1:ʱʹܣʹͼͬʱ
 *
 *        ʹôģʽʱģʽΪ0ģʽΪ1ʹӼͬʱ
 *
 * \return 
 */
am_static_inline
void amhw_zsl42x_tim_mode23_slaver_enable(amhw_zsl42x_tim_t *p_hw_tim)
{
    p_hw_tim->mscr &= ~(0x1ul << 4);
}

/**
 * \brief MSM ѡ  
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 *
 * \note  0:ʱ
 *        1:ʱʹܣʹͼͬʱ
 *
 *        ʹôģʽʱģʽΪ0ģʽΪ1ʹӼͬʱ
 *
 * \return 
 */
am_static_inline
void amhw_zsl42x_tim_mode23_master_enable(amhw_zsl42x_tim_t *p_hw_tim)
{
    p_hw_tim->mscr |= (0x1ul << 4);
}

/**
 * \brief CCDS ȽģʽDMAȽϴѡ    Ƚƥ䴥DMA
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 *
 * \note  0  Ƚƥ䴥DMA
 *        1  Ƚƥ䲻DMA¼´Ƚƥ䴥DMA
 *
 * \return 
 */
am_static_inline
void amhw_zsl42x_tim_mode23_cmp_dma(amhw_zsl42x_tim_t *p_hw_tim)
{
    p_hw_tim->mscr &= ~(0x1ul << 3);
}

/**
 * \brief CCDS ȽģʽDMAȽϴѡ   Ƚƥ䲻DMA¼´Ƚƥ䴥DMA
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 *
 * \note  0  Ƚƥ䴥DMA
 *        1  Ƚƥ䲻DMA¼´Ƚƥ䴥DMA
 *
 * \return 
 */
am_static_inline
void amhw_zsl42x_tim_mode23_cmp_update(amhw_zsl42x_tim_t *p_hw_tim)
{
    p_hw_tim->mscr |= (0x1ul << 3);
}

/**
 * \brief ģʽѡڲӵʱITRx
 */
typedef enum {
    AMHW_ZSL42x_TIM_MASTER_OUT_UG = 0,    /*  */
    AMHW_ZSL42x_TIM_MASTER_OUT_CTEN,      /* ʱʹ */
    AMHW_ZSL42x_TIM_MASTER_OUT_UEV,       /* ʱ¼ */
    AMHW_ZSL42x_TIM_MASTER_OUT_CMPSO,     /* Ƚƥѡ */
    AMHW_ZSL42x_TIM_MASTER_OUT_OCREF0A,   /* Ƚϲ */
    AMHW_ZSL42x_TIM_MASTER_OUT_OCREF0B_1A,/* Ƚϲ(ʱ012Ϊ0Bʱ3Ϊ1A) */
    AMHW_ZSL42x_TIM_MASTER_OUT_OCREF0B_2A,/* Ƚϲ(ʱ012Ϊ0Bʱ3Ϊ2A) */
    AMHW_ZSL42x_TIM_MASTER_OUT_OCREF0B,   /* Ƚϲ */
}amhw_zsl42x_tim_master_out_t;
/**
 * \brief MMS ģʽѡ ȡ
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 *
 * \return ģʽѡ(amhw_zsl42x_tim_master_out_tö)
 */
am_static_inline
amhw_zsl42x_tim_master_out_t
    amhw_zsl42x_tim_mode23_master_out_get (amhw_zsl42x_tim_t *p_hw_tim)
{
    return (amhw_zsl42x_tim_master_out_t)((p_hw_tim->mscr >> 0) & 0x7ul);
}

/**
 * \brief MMS ģʽѡ  
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 * \param[in] choice   : ģʽѡ(amhw_zsl42x_tim_master_out_tö)
 *
 * \return 
 */
am_static_inline
void amhw_zsl42x_tim_mode23_master_out_set(amhw_zsl42x_tim_t          *p_hw_tim,
                                           amhw_zsl42x_tim_master_out_t choice)
{
    p_hw_tim->mscr =(p_hw_tim->mscr & (~(0x7ul << 0))) |
                    ((choice & 0x7ul) << 0);
}

/**
 * \brief λö٣ӦڼĴеʼλ
 */
typedef enum {
    AMHW_ZSL42x_TIM_PHASE_ETP   = 31,   /* ETRλѡ */
    AMHW_ZSL42x_TIM_PHASE_BKP   = 27,   /* ɲBKλѡ */
    AMHW_ZSL42x_TIM_PHASE_CCPB2 = 23,   /* ȽϹܣCH2BͨȽλ */
    AMHW_ZSL42x_TIM_PHASE_CCPA2 = 19,   /* ȽϹܣCH2AͨȽλ */
    AMHW_ZSL42x_TIM_PHASE_CCPB1 = 15,   /* ȽϹܣCH1BͨȽλ */
    AMHW_ZSL42x_TIM_PHASE_CCPA1 = 11,   /* ȽϹܣCH1AͨȽλ */
    AMHW_ZSL42x_TIM_PHASE_CCPB0 = 7,    /* ÿλڲͬģʽ¾вͬ
                                         * ȽϹܣȽģʽ
                                         *        CCPBxȽCHBx˿ڼԿ
                                         *
                                         * ģʽſعܣλ
                                         *        CCPB0ģʽſأλⲿ
                                         *        ⲿʱʹCH0B˿뼫Կ
                                         */
    AMHW_ZSL42x_TIM_PHASE_CCPA0 = 3,    /* ÿλڲͬģʽ¾вͬ
                                         * ȽϹܣȽģʽ
                                         *        CCPAxȽCHAx˿ڼԿ
                                         *
                                         * ģʽſعܣλ
                                         *        CCPA0ģʽſأλⲿ
                                         *        ⲿʱʹCH0A˿뼫Կ
                                         */
}amhw_zsl42x_tim_phase_type_t;

/**
 * \brief ͬ
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 * \param[in] type     : ӦĿ(amhw_zsl42x_tim_phase_type_tö)
 *
 * \return 
 */
am_static_inline
void amhw_zsl42x_tim_mode23_phase_same (amhw_zsl42x_tim_t           *p_hw_tim,
                                        amhw_zsl42x_tim_phase_type_t type)
{
    p_hw_tim->fltr &= ~(0x1ul << type);
}

/**
 * \brief 
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 * \param[in] type     : ӦĿ(amhw_zsl42x_tim_phase_type_tö)
 *
 * \return 
 */
am_static_inline
void amhw_zsl42x_tim_mode23_phase_reversed (amhw_zsl42x_tim_t         *p_hw_tim,
                                            amhw_zsl42x_tim_phase_type_t type)
{
    p_hw_tim->fltr |= (0x1ul << type);
}

/**
 * \brief ˲ö٣ӦڼĴеʼλ
 */
typedef enum {
    AMHW_ZSL42x_TIM_FLITER_TYPE_FLTET = 28,   /* ETR˲ */
    AMHW_ZSL42x_TIM_FLITER_TYPE_FLTBK = 24,   /* ɲ˲ */
    AMHW_ZSL42x_TIM_FLITER_TYPE_FLTB2 = 20,   /* CH2Bͨ˲ */
    AMHW_ZSL42x_TIM_FLITER_TYPE_FLTA2 = 16,   /* CH2Aͨ˲ */
    AMHW_ZSL42x_TIM_FLITER_TYPE_FLTB1 = 12,   /* CH1Bͨ˲ */
    AMHW_ZSL42x_TIM_FLITER_TYPE_FLTA1 = 8,    /* CH1Aͨ˲ */
    AMHW_ZSL42x_TIM_FLITER_TYPE_FLTB0 = 4,    /* CH0Bͨ˲ */
    AMHW_ZSL42x_TIM_FLITER_TYPE_FLTA0 = 0,    /* CH0Aͨ˲ */
}amhw_zsl42x_tim_filter_type_t;

/**
 * \brief ˲    ȡ
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 * \param[in] type     : ӦĿ(amhw_zsl42x_tim_filter_type_tö)
 *
 * \return ˲(amhw_zsl42x_tim_filter_tö)
 */
am_static_inline
amhw_zsl42x_tim_filter_t
    amhw_zsl42x_tim_mode23_fliter_get (amhw_zsl42x_tim_t            *p_hw_tim,
                                       amhw_zsl42x_tim_filter_type_t type)
{
    return (amhw_zsl42x_tim_filter_t)((p_hw_tim->fltr >> type) & 0x7ul);
}

/**
 * \brief ˲     
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 * \param[in] type     : ӦĿ(amhw_zsl42x_tim_filter_type_tö)
 * \param[in] choice   : ˲(amhw_zsl42x_tim_filter_tö)
 *
 * \return 
 */
am_static_inline
void amhw_zsl42x_tim_mode23_fliter_set(amhw_zsl42x_tim_t            *p_hw_tim,
                                       amhw_zsl42x_tim_filter_type_t type,
                                       amhw_zsl42x_tim_filter_t      choice)
{
    p_hw_tim->fltr =(p_hw_tim->fltr & (~(0x7ul << type))) |
                    ((choice & 0x7ul) << type);
}

/**
 * \brief ȽϿ(Ƚ)
 *
 * \note PWMģʽ1
 *            Ƚϣ
 *                   ϼʱCNT<CCRxyߣ¼ʱCNT>CCRxyΪ͵ƽ
 *            ˫Ƚϣ
 *                   1 ݲϼ CCRxA<CNTCCRxBΪ͵ƽ
 *                   2 ݲ¼ CCRxA<CNTCCRxBΪߵƽ
 *                   3 ǲϼCNT<CCRxAߣ¼CNT>CCRxBΪ͵ƽ
 *
 *        PWMģʽ2
 *            Ƚϣ
 *                   ϼʱCNT<CCRxyͣ¼ʱCNT>CCRxyΪߵƽ
 *            ˫Ƚϣ
 *                   1 ݲϼ CCRxA<CNTCCRxBΪߵƽ
 *                   2 ݲ¼ CCRxA<CNTCCRxBΪ͵ƽ
 *                   3 ǲϼCNT<CCRxAͣ¼CNT>CCRxBΪߵƽ
 */
typedef enum {
    AMHW_ZSL42x_TIM_COMPARE_0 = 0,       /* ǿΪ0 */
    AMHW_ZSL42x_TIM_COMPARE_1,           /* ǿΪ1 */
    AMHW_ZSL42x_TIM_COMPARE_FIT_0,       /* ȽƥʱǿΪ0 */
    AMHW_ZSL42x_TIM_COMPARE_FIT_1,       /* ȽƥʱǿΪ1 */
    AMHW_ZSL42x_TIM_COMPARE_FIT_TOGGLE,  /* Ƚƥʱת */
    AMHW_ZSL42x_TIM_COMPARE_FIT_HIGH,    /* Ƚƥʱһڵĸߵƽ */
    AMHW_ZSL42x_TIM_COMPARE_PWM1,        /* PWMģʽ1 */
    AMHW_ZSL42x_TIM_COMPARE_PWM2,        /* PWMģʽ2 */
}amhw_zsl42x_tim_compare_t;

/**
 * \brief ȽϿö٣ӦڼĴеʼλ
 */
typedef enum {
    AMHW_ZSL42x_TIM_COMPARE_TYPE_FLTB2 = 20,   /* CH2BͨȽϿ */
    AMHW_ZSL42x_TIM_COMPARE_TYPE_FLTA2 = 16,   /* CH2AͨȽϿ */
    AMHW_ZSL42x_TIM_COMPARE_TYPE_FLTB1 = 12,   /* CH1BͨȽϿ */
    AMHW_ZSL42x_TIM_COMPARE_TYPE_FLTA1 = 8,    /* CH1AͨȽϿ */
    AMHW_ZSL42x_TIM_COMPARE_TYPE_FLTB0 = 4,    /* CH0BͨȽϿ */
    AMHW_ZSL42x_TIM_COMPARE_TYPE_FLTA0 = 0,    /* CH0AͨȽϿ */
}amhw_zsl42x_tim_compare_type_t;

/**
 * \brief ȽϿ    ȡ
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 * \param[in] type     : ӦĿ(amhw_zsl42x_tim_compare_type_tö)
 *
 * \return ȽϿ(amhw_zsl42x_tim_compare_tö)
 */
am_static_inline
amhw_zsl42x_tim_compare_t
    amhw_zsl42x_tim_mode23_compare_get (amhw_zsl42x_tim_t             *p_hw_tim,
                                        amhw_zsl42x_tim_compare_type_t type)
{
    return (amhw_zsl42x_tim_compare_t)((p_hw_tim->fltr >> type) & 0x7ul);
}

/**
 * \brief ȽϿ     
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 * \param[in] type     : ӦĿ(amhw_zsl42x_tim_compare_type_tö)
 * \param[in] choice   : ˲(amhw_zsl42x_tim_compare_tö)
 *
 * \return 
 */
am_static_inline
void amhw_zsl42x_tim_mode23_compare_set(amhw_zsl42x_tim_t             *p_hw_tim,
                                        amhw_zsl42x_tim_compare_type_t type,
                                        amhw_zsl42x_tim_compare_t      choice)
{
    p_hw_tim->fltr =(p_hw_tim->fltr & (~(0x7ul << type))) |
                    ((choice & 0x7ul) << type);
}

/* ADCƼĴTIMx_ADTRؿλ궨 */
#define AMHW_ZSL42x_TIM_ADC_TRIGGER_ADTE   (0x1UL << 7) /* ʹADCȫֿ */
#define AMHW_ZSL42x_TIM_ADC_TRIGGER_CMB2E  (0x1UL << 6) /* ͨ2BȽƥ䴥ADCʹ */
#define AMHW_ZSL42x_TIM_ADC_TRIGGER_CMB1E  (0x1UL << 5) /* ͨ1BȽƥ䴥ADCʹ */
#define AMHW_ZSL42x_TIM_ADC_TRIGGER_CMB0E  (0x1UL << 4) /* ͨ0BȽƥ䴥ADCʹ */
#define AMHW_ZSL42x_TIM_ADC_TRIGGER_CMA2E  (0x1UL << 3) /* ͨ2AȽƥ䴥ADCʹ */
#define AMHW_ZSL42x_TIM_ADC_TRIGGER_CMA1E  (0x1UL << 2) /* ͨ1AȽƥ䴥ADCʹ */
#define AMHW_ZSL42x_TIM_ADC_TRIGGER_CMA0E  (0x1UL << 1) /* ͨ0AȽƥ䴥ADCʹ */
#define AMHW_ZSL42x_TIM_ADC_TRIGGER_UEVE   (0x1UL << 0) /* ¼´ADCʹ */
#define AMHW_ZSL42x_TIM_ADC_TRIGGER_ALL    0xFF         /*  */

/**
 * \brief 궨ʹ
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 * \param[in] type     : AMHW_ZSL42x_TIM_ADC_TRIGGER_ADTE
 *                       .
 *                       .
 *                       .
 *                       AMHW_ZSL42x_TIM_ADC_TRIGGER_ALL
 *
 * \return 
 */
am_static_inline
void amhw_zsl42x_tim_mode23_adc_trigger_enable (amhw_zsl42x_tim_t *p_hw_tim,
                                                uint32_t           type)
{
    if(0xFF == type) {
        p_hw_tim->adtr |= AMHW_ZSL42x_TIM_ADC_TRIGGER_ADTE;
        p_hw_tim->adtr |= AMHW_ZSL42x_TIM_ADC_TRIGGER_CMB2E;
        p_hw_tim->adtr |= AMHW_ZSL42x_TIM_ADC_TRIGGER_CMB1E;
        p_hw_tim->adtr |= AMHW_ZSL42x_TIM_ADC_TRIGGER_CMB0E;
        p_hw_tim->adtr |= AMHW_ZSL42x_TIM_ADC_TRIGGER_CMA2E;
        p_hw_tim->adtr |= AMHW_ZSL42x_TIM_ADC_TRIGGER_CMA1E;
        p_hw_tim->adtr |= AMHW_ZSL42x_TIM_ADC_TRIGGER_CMA0E;
        p_hw_tim->adtr |= AMHW_ZSL42x_TIM_ADC_TRIGGER_UEVE;
    }
    else {
        p_hw_tim->adtr |= type;
    }
}

/**
 * \brief 궨
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 * \param[in] type     : AMHW_ZSL42x_TIM_ADC_TRIGGER_ADTE
 *                       .
 *                       .
 *                       .
 *                       AMHW_ZSL42x_TIM_ADC_TRIGGER_ALL
 *
 * \return 
 */
am_static_inline
void amhw_zsl42x_tim_mode23_adc_trigger_disable (amhw_zsl42x_tim_t  *p_hw_tim,
                                                 uint32_t           type)
{
    if(0xFF == type) {
        p_hw_tim->adtr &= ~AMHW_ZSL42x_TIM_ADC_TRIGGER_ADTE;
        p_hw_tim->adtr &= ~AMHW_ZSL42x_TIM_ADC_TRIGGER_CMB2E;
        p_hw_tim->adtr &= ~AMHW_ZSL42x_TIM_ADC_TRIGGER_CMB1E;
        p_hw_tim->adtr &= ~AMHW_ZSL42x_TIM_ADC_TRIGGER_CMB0E;
        p_hw_tim->adtr &= ~AMHW_ZSL42x_TIM_ADC_TRIGGER_CMA2E;
        p_hw_tim->adtr &= ~AMHW_ZSL42x_TIM_ADC_TRIGGER_CMA1E;
        p_hw_tim->adtr &= ~AMHW_ZSL42x_TIM_ADC_TRIGGER_CMA0E;
        p_hw_tim->adtr &= ~AMHW_ZSL42x_TIM_ADC_TRIGGER_UEVE;
    }
    else {
        p_hw_tim->adtr &= ~type;
    }
}

/**
 * \brief CCGB   ȽB
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 * \param[in] chan     : ͨ  ZSL42x_TIM_CH0A               
 *                            ZSL42x_TIM_CH0B               
 *                            ZSL42x_TIM_CH1A(TIM3ͨ)    
 *                            ZSL42x_TIM_CH1B(TIM3ͨ)    
 *                            ZSL42x_TIM_CH2A(TIM3ͨ)    
 *                            ZSL42x_TIM_CH2B(TIM3ͨ)    
 * \note ӲԶ
 *       ڱȽģʽֻж
 *       ڲģʽ²жϲҲֵĴ
 *
 * \return 
 */
am_static_inline
void amhw_zsl42x_tim_mode23_crchx_ccgb_enable (amhw_zsl42x_tim_t *p_hw_tim,
                                               uint8_t            chan)
{
    switch(chan / 2) {
        case 0:
            p_hw_tim->crchx[0] |= (0x1ul << 15);
            break;

        case 1:
            p_hw_tim->crchx[1] |= (0x1ul << 15);
            break;

        case 2:
            p_hw_tim->crchx[2] |= (0x1ul << 15);
            break;

        default:
            break;
    }
}

/**
 * \brief CCGA   ȽA
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 * \param[in] chan     : ͨ  ZSL42x_TIM_CH0A               
 *                            ZSL42x_TIM_CH0B               
 *                            ZSL42x_TIM_CH1A(TIM3ͨ)    
 *                            ZSL42x_TIM_CH1B(TIM3ͨ)    
 *                            ZSL42x_TIM_CH2A(TIM3ͨ)    
 *                            ZSL42x_TIM_CH2B(TIM3ͨ)    
 *
 * \note ӲԶ
 *       ڱȽģʽֻж
 *       ڲģʽ²жϲҲֵĴ
 *
 * \return 
 */
am_static_inline
void amhw_zsl42x_tim_mode23_crchx_ccga_enable (amhw_zsl42x_tim_t *p_hw_tim,
                                               uint8_t            chan)
{
    switch(chan / 2) {
        case 0:
            p_hw_tim->crchx[0] |= (0x1ul << 14);
            break;

        case 1:
            p_hw_tim->crchx[1] |= (0x1ul << 14);
            break;

        case 2:
            p_hw_tim->crchx[2] |= (0x1ul << 14);
            break;

        default:
            break;
    }
}

/**
 * \brief BͨȽƥ
 */
typedef enum {
    AMHW_ZSL42x_TIM_CISB_TYPE_NO = 0,   /* ƥ */
    AMHW_ZSL42x_TIM_CISB_TYPE_RISING,   /* ƥ */
    AMHW_ZSL42x_TIM_CISB_TYPE_FALLING,  /* ½ƥ */
    AMHW_ZSL42x_TIM_CISB_TYPE_BOTH,     /* ˫ƥ */
}amhw_zsl42x_tim_cisb_type_t;

/**
 * \brief CISB   BͨȽƥ    ȡ
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 * \param[in] chan     : ͨ  ZSL42x_TIM_CH0A               
 *                            ZSL42x_TIM_CH0B               
 *                            ZSL42x_TIM_CH1A(TIM3ͨ)    
 *                            ZSL42x_TIM_CH1B(TIM3ͨ)    
 *                            ZSL42x_TIM_CH2A(TIM3ͨ)    
 *                            ZSL42x_TIM_CH2B(TIM3ͨ)    
 *
 * \return amhw_zsl42x_tim_cisb_type_tö
 */
am_static_inline
amhw_zsl42x_tim_cisb_type_t
    amhw_zsl42x_tim_mode23_cisb_get (amhw_zsl42x_tim_t *p_hw_tim,
                                     uint8_t            chan)
{
    uint8_t get_value;
    switch(chan / 2) {
        case 0:
            get_value = (p_hw_tim->crchx[0] >> 12) & 0x3ul;
            break;

        case 1:
            get_value = (p_hw_tim->crchx[1] >> 12) & 0x3ul;
            break;

        case 2:
            get_value = (p_hw_tim->crchx[2] >> 12) & 0x3ul;
            break;

        default:
            break;
    }
    return (amhw_zsl42x_tim_cisb_type_t)get_value;
}

/**
 * \brief CISB   BͨȽƥ     
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 * \param[in] chan     : ͨ  ZSL42x_TIM_CH0A               
 *                            ZSL42x_TIM_CH0B               
 *                            ZSL42x_TIM_CH1A(TIM3ͨ)    
 *                            ZSL42x_TIM_CH1B(TIM3ͨ)    
 *                            ZSL42x_TIM_CH2A(TIM3ͨ)    
 *                            ZSL42x_TIM_CH2B(TIM3ͨ)    
 * \param[in] choice   : amhw_zsl42x_tim_cisb_type_tö
 *
 * \return 
 */
am_static_inline
void amhw_zsl42x_tim_mode23_cisb_set(amhw_zsl42x_tim_t           *p_hw_tim,
                                     uint8_t                      chan,
                                     amhw_zsl42x_tim_cisb_type_t  choice)
{
    switch(chan / 2) {
        case 0:
            p_hw_tim->crchx[0] = (p_hw_tim->crchx[0] & (~(0x3ul << 12))) |
                                 ((choice & 0x3ul) << 12);
            break;

        case 1:
            p_hw_tim->crchx[1] = (p_hw_tim->crchx[1] & (~(0x3ul << 12))) |
                                 ((choice & 0x3ul) << 12);
            break;

        case 2:
            p_hw_tim->crchx[2] = (p_hw_tim->crchx[2] & (~(0x3ul << 12))) |
                                 ((choice & 0x3ul) << 12);
            break;

        default:
            break;
    }
}

/**
 * \brief CDEB   BȽϴDMA
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 * \param[in] chan     : ͨ  ZSL42x_TIM_CH0A               
 *                            ZSL42x_TIM_CH0B               
 *                            ZSL42x_TIM_CH1A(TIM3ͨ)    
 *                            ZSL42x_TIM_CH1B(TIM3ͨ)    
 *                            ZSL42x_TIM_CH2A(TIM3ͨ)    
 *                            ZSL42x_TIM_CH2B(TIM3ͨ)    
 *
 * \return 
 */
am_static_inline
void amhw_zsl42x_tim_mode23_cdeb_dma_disable (amhw_zsl42x_tim_t *p_hw_tim,
                                             uint8_t            chan)
{
    switch(chan / 2) {
        case 0:
            p_hw_tim->crchx[0] &= ~(0x1ul << 11);
            break;

        case 1:
            p_hw_tim->crchx[1] &= ~(0x1ul << 11);
            break;

        case 2:
            p_hw_tim->crchx[2] &= ~(0x1ul << 11);
            break;

        default:
            break;
    }
}

/**
 * \brief CDEB   BȽϴDMAʹ
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 * \param[in] chan     : ͨ  ZSL42x_TIM_CH0A               
 *                            ZSL42x_TIM_CH0B               
 *                            ZSL42x_TIM_CH1A(TIM3ͨ)    
 *                            ZSL42x_TIM_CH1B(TIM3ͨ)    
 *                            ZSL42x_TIM_CH2A(TIM3ͨ)    
 *                            ZSL42x_TIM_CH2B(TIM3ͨ)    
 *
 * \return 
 */
am_static_inline
void amhw_zsl42x_tim_mode23_cdeb_dma_enable (amhw_zsl42x_tim_t *p_hw_tim,
                                             uint8_t            chan)
{
    switch(chan / 2) {
        case 0:
            p_hw_tim->crchx[0] |= (0x1ul << 11);
            break;

        case 1:
            p_hw_tim->crchx[1] |= (0x1ul << 11);
            break;

        case 2:
            p_hw_tim->crchx[2] |= (0x1ul << 11);
            break;

        default:
            break;
    }
}

/**
 * \brief CDEA   AȽϴDMA
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 * \param[in] chan     : ͨ  ZSL42x_TIM_CH0A               
 *                            ZSL42x_TIM_CH0B               
 *                            ZSL42x_TIM_CH1A(TIM3ͨ)    
 *                            ZSL42x_TIM_CH1B(TIM3ͨ)    
 *                            ZSL42x_TIM_CH2A(TIM3ͨ)    
 *                            ZSL42x_TIM_CH2B(TIM3ͨ)    
 *
 * \return 
 */
am_static_inline
void amhw_zsl42x_tim_mode23_cdea_dma_disable (amhw_zsl42x_tim_t *p_hw_tim,
                                             uint8_t            chan)
{
    switch(chan / 2) {
        case 0:
            p_hw_tim->crchx[0] &= ~(0x1ul << 10);
            break;

        case 1:
            p_hw_tim->crchx[1] &= ~(0x1ul << 10);
            break;

        case 2:
            p_hw_tim->crchx[2] &= ~(0x1ul << 10);
            break;

        default:
            break;
    }
}

/**
 * \brief CDEA   AȽϴDMAʹ
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 * \param[in] chan     : ͨ  ZSL42x_TIM_CH0A               
 *                            ZSL42x_TIM_CH0B               
 *                            ZSL42x_TIM_CH1A(TIM3ͨ)    
 *                            ZSL42x_TIM_CH1B(TIM3ͨ)    
 *                            ZSL42x_TIM_CH2A(TIM3ͨ)    
 *                            ZSL42x_TIM_CH2B(TIM3ͨ)    
 *
 * \return 
 */
am_static_inline
void amhw_zsl42x_tim_mode23_cdea_dma_enable (amhw_zsl42x_tim_t *p_hw_tim,
                                             uint8_t            chan)
{
    switch(chan / 2) {
        case 0:
            p_hw_tim->crchx[0] |= (0x1ul << 10);
            break;

        case 1:
            p_hw_tim->crchx[1] |= (0x1ul << 10);
            break;

        case 2:
            p_hw_tim->crchx[2] |= (0x1ul << 10);
            break;

        default:
            break;
    }
}

/**
 * \brief CIEB   BȽϴжϽ
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 * \param[in] chan     : ͨ  ZSL42x_TIM_CH0A               
 *                            ZSL42x_TIM_CH0B               
 *                            ZSL42x_TIM_CH1A(TIM3ͨ)    
 *                            ZSL42x_TIM_CH1B(TIM3ͨ)    
 *                            ZSL42x_TIM_CH2A(TIM3ͨ)    
 *                            ZSL42x_TIM_CH2B(TIM3ͨ)    
 *
 * \return 
 */
am_static_inline
void amhw_zsl42x_tim_mode23_cieb_int_disable (amhw_zsl42x_tim_t *p_hw_tim,
                                              uint8_t            chan)
{
    switch(chan / 2) {
        case 0:
            p_hw_tim->crchx[0] &= ~(0x1ul << 9);
            break;

        case 1:
            p_hw_tim->crchx[1] &= ~(0x1ul << 9);
            break;

        case 2:
            p_hw_tim->crchx[2] &= ~(0x1ul << 9);
            break;

        default:
            break;
    }
}

/**
 * \brief CIEB   BȽϴжʹ
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 * \param[in] chan     : ͨ  ZSL42x_TIM_CH0A               
 *                            ZSL42x_TIM_CH0B               
 *                            ZSL42x_TIM_CH1A(TIM3ͨ)    
 *                            ZSL42x_TIM_CH1B(TIM3ͨ)    
 *                            ZSL42x_TIM_CH2A(TIM3ͨ)    
 *                            ZSL42x_TIM_CH2B(TIM3ͨ)    
 *
 * \return 
 */
am_static_inline
void amhw_zsl42x_tim_mode23_cieb_int_enable (amhw_zsl42x_tim_t *p_hw_tim,
                                             uint8_t            chan)
{
    switch(chan / 2) {
        case 0:
            p_hw_tim->crchx[0] |= (0x1ul << 9);
            break;

        case 1:
            p_hw_tim->crchx[1] |= (0x1ul << 9);
            break;

        case 2:
            p_hw_tim->crchx[2] |= (0x1ul << 9);
            break;

        default:
            break;
    }
}

/**
 * \brief BȽϴжʹ״̬ȡ
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 * \param[in] chan     : ZSL42x_TIM_CH0B
 *                       ZSL42x_TIM_CH1B
 *                       ZSL42x_TIM_CH2B
 *
 * \return  AM_TRUE  ʹЧ
 *          AM_FALSE ʹЧ
 *
 */
am_static_inline
am_bool_t amhw_zsl42x_tim_mode23_cieb_int_get (amhw_zsl42x_tim_t *p_hw_tim,
                                               uint8_t            chan)
{
	am_bool_t result = AM_FALSE;

    switch(chan / 2) {
        case 0:
        	result = (p_hw_tim->crchx[0] & (0x1ul << 9)) ? AM_TRUE : AM_FALSE;
            break;

        case 1:
        	result = (p_hw_tim->crchx[1] & (0x1ul << 9)) ? AM_TRUE : AM_FALSE;
            break;

        case 2:
        	result = (p_hw_tim->crchx[2] & (0x1ul << 9)) ? AM_TRUE : AM_FALSE;
            break;

        default:
            break;
    }

    return result;
}

/**
 * \brief CIEA   AȽϴжϽ
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 * \param[in] chan     : ͨ  ZSL42x_TIM_CH0A               
 *                            ZSL42x_TIM_CH0B               
 *                            ZSL42x_TIM_CH1A(TIM3ͨ)    
 *                            ZSL42x_TIM_CH1B(TIM3ͨ)    
 *                            ZSL42x_TIM_CH2A(TIM3ͨ)    
 *                            ZSL42x_TIM_CH2B(TIM3ͨ)    
 *
 * \return 
 */
am_static_inline
void amhw_zsl42x_tim_mode23_ciea_int_disable (amhw_zsl42x_tim_t *p_hw_tim,
                                              uint8_t            chan)
{
    switch(chan / 2) {
        case 0:
            p_hw_tim->crchx[0] &= ~(0x1ul << 8);
            break;

        case 1:
            p_hw_tim->crchx[1] &= ~(0x1ul << 8);
            break;

        case 2:
            p_hw_tim->crchx[2] &= ~(0x1ul << 8);
            break;

        default:
            break;
    }
}

/**
 * \brief CIEA   AȽϴжʹ
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 * \param[in] chan     : ͨ  ZSL42x_TIM_CH0A               
 *                            ZSL42x_TIM_CH0B               
 *                            ZSL42x_TIM_CH1A(TIM3ͨ)    
 *                            ZSL42x_TIM_CH1B(TIM3ͨ)    
 *                            ZSL42x_TIM_CH2A(TIM3ͨ)    
 *                            ZSL42x_TIM_CH2B(TIM3ͨ)    
 *
 * \return 
 */
am_static_inline
void amhw_zsl42x_tim_mode23_ciea_int_enable (amhw_zsl42x_tim_t *p_hw_tim,
                                             uint8_t            chan)
{
    switch(chan / 2) {
        case 0:
            p_hw_tim->crchx[0] |= (0x1ul << 8);
            break;

        case 1:
            p_hw_tim->crchx[1] |= (0x1ul << 8);
            break;

        case 2:
            p_hw_tim->crchx[2] |= (0x1ul << 8);
            break;

        default:
            break;
    }
}

/**
 * \brief AȽϴжʹ״̬ȡ
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 * \param[in] chan     : ZSL42x_TIM_CH0A 
 *                       ZSL42x_TIM_CH1A 
 *                       ZSL42x_TIM_CH2A 
 *
 * \return  AM_TRUE  ʹЧ
 *          AM_FALSE ʹЧ
 *
 */
am_static_inline
am_bool_t amhw_zsl42x_tim_mode23_ciea_int_get (amhw_zsl42x_tim_t *p_hw_tim,
                                               uint8_t            chan)
{
	am_bool_t result = AM_FALSE;

    switch(chan / 2) {
        case 0:
        	result = (p_hw_tim->crchx[0] & (0x1ul << 8)) ? AM_TRUE : AM_FALSE;
            break;

        case 1:
        	result = (p_hw_tim->crchx[1] & (0x1ul << 8)) ? AM_TRUE : AM_FALSE;
            break;

        case 2:
        	result = (p_hw_tim->crchx[2] & (0x1ul << 8)) ? AM_TRUE : AM_FALSE;
            break;

        default:
            break;
    }

    return result;
}

/**
 * \brief BUFEB   ȽϹܣ BȽϻʹܿ     
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 * \param[in] chan     : ͨ  ZSL42x_TIM_CH0A               
 *                            ZSL42x_TIM_CH0B               
 *                            ZSL42x_TIM_CH1A(TIM3ͨ)    
 *                            ZSL42x_TIM_CH1B(TIM3ͨ)    
 *                            ZSL42x_TIM_CH2A(TIM3ͨ)    
 *                            ZSL42x_TIM_CH2B(TIM3ͨ)    
 *
 * \return 
 */
am_static_inline
void amhw_zsl42x_tim_mode23_buffb_disable (amhw_zsl42x_tim_t *p_hw_tim,
                                           uint8_t            chan)
{
    switch(chan / 2) {
        case 0:
            p_hw_tim->crchx[0] &= ~(0x1ul << 7);
            break;

        case 1:
            p_hw_tim->crchx[1] &= ~(0x1ul << 7);
            break;

        case 2:
            p_hw_tim->crchx[2] &= ~(0x1ul << 7);
            break;

        default:
            break;
    }
}

/**
 * \brief BUFEB   ȽϹܣ BȽϻʹܿ     ʹ
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 * \param[in] chan     : ͨ  ZSL42x_TIM_CH0A               
 *                            ZSL42x_TIM_CH0B               
 *                            ZSL42x_TIM_CH1A(TIM3ͨ)    
 *                            ZSL42x_TIM_CH1B(TIM3ͨ)    
 *                            ZSL42x_TIM_CH2A(TIM3ͨ)    
 *                            ZSL42x_TIM_CH2B(TIM3ͨ)    
 *
 * \return 
 */
am_static_inline
void amhw_zsl42x_tim_mode23_buffb_enable (amhw_zsl42x_tim_t *p_hw_tim,
                                          uint8_t            chan)
{
    switch(chan / 2) {
        case 0:
            p_hw_tim->crchx[0] |= (0x1ul << 7);
            break;

        case 1:
            p_hw_tim->crchx[1] |= (0x1ul << 7);
            break;

        case 2:
            p_hw_tim->crchx[2] |= (0x1ul << 7);
            break;

        default:
            break;
    }
}

/**
 * \brief BUFEA   ȽϹܣAȽϻʹܿ     
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 * \param[in] chan     : ͨ  ZSL42x_TIM_CH0A               
 *                            ZSL42x_TIM_CH0B               
 *                            ZSL42x_TIM_CH1A(TIM3ͨ)    
 *                            ZSL42x_TIM_CH1B(TIM3ͨ)    
 *                            ZSL42x_TIM_CH2A(TIM3ͨ)    
 *                            ZSL42x_TIM_CH2B(TIM3ͨ)    
 *
 * \return 
 */
am_static_inline
void amhw_zsl42x_tim_mode23_buffa_disable (amhw_zsl42x_tim_t *p_hw_tim,
                                           uint8_t            chan)
{
    switch(chan / 2) {
        case 0:
            p_hw_tim->crchx[0] &= ~(0x1ul << 6);
            break;

        case 1:
            p_hw_tim->crchx[1] &= ~(0x1ul << 6);
            break;

        case 2:
            p_hw_tim->crchx[2] &= ~(0x1ul << 6);
            break;

        default:
            break;
    }
}

/**
 * \brief BUFEA   ȽϹܣ AȽϻʹܿ     ʹ
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 * \param[in] chan     : ͨ  ZSL42x_TIM_CH0A               
 *                            ZSL42x_TIM_CH0B               
 *                            ZSL42x_TIM_CH1A(TIM3ͨ)    
 *                            ZSL42x_TIM_CH1B(TIM3ͨ)    
 *                            ZSL42x_TIM_CH2A(TIM3ͨ)    
 *                            ZSL42x_TIM_CH2B(TIM3ͨ)    
 *
 * \return 
 */
am_static_inline
void amhw_zsl42x_tim_mode23_buffa_enable (amhw_zsl42x_tim_t *p_hw_tim,
                                          uint8_t            chan)
{
    switch(chan / 2) {
        case 0:
            p_hw_tim->crchx[0] |= (0x1ul << 6);
            break;

        case 1:
            p_hw_tim->crchx[1] |= (0x1ul << 6);
            break;

        case 2:
            p_hw_tim->crchx[2] |= (0x1ul << 6);
            break;

        default:
            break;
    }
}

/**
 * \brief CSB   Bͨ/ȽϹѡ      Ƚģʽ
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 * \param[in] chan     : ͨ  ZSL42x_TIM_CH0A               
 *                            ZSL42x_TIM_CH0B               
 *                            ZSL42x_TIM_CH1A(TIM3ͨ)    
 *                            ZSL42x_TIM_CH1B(TIM3ͨ)    
 *                            ZSL42x_TIM_CH2A(TIM3ͨ)    
 *                            ZSL42x_TIM_CH2B(TIM3ͨ)    
 *
 * \return 
 */
am_static_inline
void amhw_zsl42x_tim_mode23_csb_compare (amhw_zsl42x_tim_t *p_hw_tim,
                                         uint8_t            chan)
{
    switch(chan / 2) {
        case 0:
            p_hw_tim->crchx[0] &= ~(0x1ul << 5);
            break;

        case 1:
            p_hw_tim->crchx[1] &= ~(0x1ul << 5);
            break;

        case 2:
            p_hw_tim->crchx[2] &= ~(0x1ul << 5);
            break;

        default:
            break;
    }
}

/**
 * \brief CSB   Bͨ/ȽϹѡ      ģʽ
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 * \param[in] chan     : ͨ  ZSL42x_TIM_CH0A               
 *                            ZSL42x_TIM_CH0B               
 *                            ZSL42x_TIM_CH1A(TIM3ͨ)    
 *                            ZSL42x_TIM_CH1B(TIM3ͨ)    
 *                            ZSL42x_TIM_CH2A(TIM3ͨ)    
 *                            ZSL42x_TIM_CH2B(TIM3ͨ)    
 *
 * \return 
 */
am_static_inline
void amhw_zsl42x_tim_mode23_csb_cap (amhw_zsl42x_tim_t *p_hw_tim,
                                     uint8_t            chan)
{
    switch(chan / 2) {
        case 0:
            p_hw_tim->crchx[0] |= (0x1ul << 5);
            break;

        case 1:
            p_hw_tim->crchx[1] |= (0x1ul << 5);
            break;

        case 2:
            p_hw_tim->crchx[2] |= (0x1ul << 5);
            break;

        default:
            break;
    }
}

/**
 * \brief CSA   Aͨ/ȽϹѡ      Ƚģʽ
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 * \param[in] chan     : ͨ  ZSL42x_TIM_CH0A               
 *                            ZSL42x_TIM_CH0B               
 *                            ZSL42x_TIM_CH1A(TIM3ͨ)    
 *                            ZSL42x_TIM_CH1B(TIM3ͨ)    
 *                            ZSL42x_TIM_CH2A(TIM3ͨ)    
 *                            ZSL42x_TIM_CH2B(TIM3ͨ)    
 *
 * \return 
 */
am_static_inline
void amhw_zsl42x_tim_mode23_csa_compare (amhw_zsl42x_tim_t *p_hw_tim,
                                         uint8_t            chan)
{
    switch(chan / 2) {
        case 0:
            p_hw_tim->crchx[0] &= ~(0x1ul << 4);
            break;

        case 1:
            p_hw_tim->crchx[1] &= ~(0x1ul << 4);
            break;

        case 2:
            p_hw_tim->crchx[2] &= ~(0x1ul << 4);
            break;

        default:
            break;
    }
}

/**
 * \brief CSA   Aͨ/ȽϹѡ      ģʽ
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 * \param[in] chan     : ͨ  ZSL42x_TIM_CH0A               
 *                            ZSL42x_TIM_CH0B               
 *                            ZSL42x_TIM_CH1A(TIM3ͨ)    
 *                            ZSL42x_TIM_CH1B(TIM3ͨ)    
 *                            ZSL42x_TIM_CH2A(TIM3ͨ)    
 *                            ZSL42x_TIM_CH2B(TIM3ͨ)    
 *
 * \return 
 */
am_static_inline
void amhw_zsl42x_tim_mode23_csa_cap (amhw_zsl42x_tim_t *p_hw_tim,
                                     uint8_t            chan)
{
    switch(chan / 2) {
        case 0:
            p_hw_tim->crchx[0] |= (0x1ul << 4);
            break;

        case 1:
            p_hw_tim->crchx[1] |= (0x1ul << 4);
            break;

        case 2:
            p_hw_tim->crchx[2] |= (0x1ul << 4);
            break;

        default:
            break;
    }
}

/**
 * \brief CFB   Bͨ½ز
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 * \param[in] chan     : ͨ  ZSL42x_TIM_CH0A               
 *                            ZSL42x_TIM_CH0B               
 *                            ZSL42x_TIM_CH1A(TIM3ͨ)    
 *                            ZSL42x_TIM_CH1B(TIM3ͨ)    
 *                            ZSL42x_TIM_CH2A(TIM3ͨ)    
 *                            ZSL42x_TIM_CH2B(TIM3ͨ)    
 *
 * \return 
 */
am_static_inline
void amhw_zsl42x_tim_mode23_cfb_disable (amhw_zsl42x_tim_t *p_hw_tim,
                                         uint8_t            chan)
{
    switch(chan / 2) {
        case 0:
            p_hw_tim->crchx[0] &= ~(0x1ul << 3);
            break;

        case 1:
            p_hw_tim->crchx[1] &= ~(0x1ul << 3);
            break;

        case 2:
            p_hw_tim->crchx[2] &= ~(0x1ul << 3);
            break;

        default:
            break;
    }
}

/**
 * \brief CFB   Bͨ½زʹ
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 * \param[in] chan     : ͨ  ZSL42x_TIM_CH0A               
 *                            ZSL42x_TIM_CH0B               
 *                            ZSL42x_TIM_CH1A(TIM3ͨ)    
 *                            ZSL42x_TIM_CH1B(TIM3ͨ)    
 *                            ZSL42x_TIM_CH2A(TIM3ͨ)    
 *                            ZSL42x_TIM_CH2B(TIM3ͨ)    
 *
 * \return 
 */
am_static_inline
void amhw_zsl42x_tim_mode23_cfb_enable (amhw_zsl42x_tim_t *p_hw_tim,
                                        uint8_t            chan)
{
    switch(chan / 2) {
        case 0:
            p_hw_tim->crchx[0] |= (0x1ul << 3);
            break;

        case 1:
            p_hw_tim->crchx[1] |= (0x1ul << 3);
            break;

        case 2:
            p_hw_tim->crchx[2] |= (0x1ul << 3);
            break;

        default:
            break;
    }
}

/**
 * \brief CRB   Bͨز
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 * \param[in] chan     : ͨ  ZSL42x_TIM_CH0A               
 *                            ZSL42x_TIM_CH0B               
 *                            ZSL42x_TIM_CH1A(TIM3ͨ)    
 *                            ZSL42x_TIM_CH1B(TIM3ͨ)    
 *                            ZSL42x_TIM_CH2A(TIM3ͨ)    
 *                            ZSL42x_TIM_CH2B(TIM3ͨ)    
 *
 * \return 
 */
am_static_inline
void amhw_zsl42x_tim_mode23_crb_disable (amhw_zsl42x_tim_t *p_hw_tim,
                                         uint8_t            chan)
{
    switch(chan / 2) {
        case 0:
            p_hw_tim->crchx[0] &= ~(0x1ul << 2);
            break;

        case 1:
            p_hw_tim->crchx[1] &= ~(0x1ul << 2);
            break;

        case 2:
            p_hw_tim->crchx[2] &= ~(0x1ul << 2);
            break;

        default:
            break;
    }
}

/**
 * \brief CRB   Bͨزʹ
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 * \param[in] chan     : ͨ  ZSL42x_TIM_CH0A               
 *                            ZSL42x_TIM_CH0B               
 *                            ZSL42x_TIM_CH1A(TIM3ͨ)    
 *                            ZSL42x_TIM_CH1B(TIM3ͨ)    
 *                            ZSL42x_TIM_CH2A(TIM3ͨ)    
 *                            ZSL42x_TIM_CH2B(TIM3ͨ)    
 *
 * \return 
 */
am_static_inline
void amhw_zsl42x_tim_mode23_crb_enable (amhw_zsl42x_tim_t *p_hw_tim,
                                        uint8_t            chan)
{
    switch(chan / 2) {
        case 0:
            p_hw_tim->crchx[0] |= (0x1ul << 2);
            break;

        case 1:
            p_hw_tim->crchx[1] |= (0x1ul << 2);
            break;

        case 2:
            p_hw_tim->crchx[2] |= (0x1ul << 2);
            break;

        default:
            break;
    }
}

/**
 * \brief CFA   Aͨ½ز
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 * \param[in] chan     : ͨ  ZSL42x_TIM_CH0A               
 *                            ZSL42x_TIM_CH0B               
 *                            ZSL42x_TIM_CH1A(TIM3ͨ)    
 *                            ZSL42x_TIM_CH1B(TIM3ͨ)    
 *                            ZSL42x_TIM_CH2A(TIM3ͨ)    
 *                            ZSL42x_TIM_CH2B(TIM3ͨ)    
 *
 * \return 
 */
am_static_inline
void amhw_zsl42x_tim_mode23_cfa_disable (amhw_zsl42x_tim_t *p_hw_tim,
                                         uint8_t            chan)
{
    switch(chan / 2) {
        case 0:
            p_hw_tim->crchx[0] &= ~(0x1ul << 1);
            break;

        case 1:
            p_hw_tim->crchx[1] &= ~(0x1ul << 1);
            break;

        case 2:
            p_hw_tim->crchx[2] &= ~(0x1ul << 1);
            break;

        default:
            break;
    }
}

/**
 * \brief CFA   Aͨ½زʹ
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 * \param[in] chan     : ͨ  ZSL42x_TIM_CH0A               
 *                            ZSL42x_TIM_CH0B               
 *                            ZSL42x_TIM_CH1A(TIM3ͨ)    
 *                            ZSL42x_TIM_CH1B(TIM3ͨ)    
 *                            ZSL42x_TIM_CH2A(TIM3ͨ)    
 *                            ZSL42x_TIM_CH2B(TIM3ͨ)    
 *
 * \return 
 */
am_static_inline
void amhw_zsl42x_tim_mode23_cfa_enable (amhw_zsl42x_tim_t *p_hw_tim,
                                        uint8_t            chan)
{
    switch(chan / 2) {
        case 0:
            p_hw_tim->crchx[0] |= (0x1ul << 1);
            break;

        case 1:
            p_hw_tim->crchx[1] |= (0x1ul << 1);
            break;

        case 2:
            p_hw_tim->crchx[2] |= (0x1ul << 1);
            break;

        default:
            break;
    }
}

/**
 * \brief CRA   Aͨز
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 * \param[in] chan     : ͨ  ZSL42x_TIM_CH0A               
 *                            ZSL42x_TIM_CH0B               
 *                            ZSL42x_TIM_CH1A(TIM3ͨ)    
 *                            ZSL42x_TIM_CH1B(TIM3ͨ)    
 *                            ZSL42x_TIM_CH2A(TIM3ͨ)    
 *                            ZSL42x_TIM_CH2B(TIM3ͨ)    
 *
 * \return 
 */
am_static_inline
void amhw_zsl42x_tim_mode23_cra_disable (amhw_zsl42x_tim_t *p_hw_tim,
                                         uint8_t            chan)
{
    switch(chan / 2) {
        case 0:
            p_hw_tim->crchx[0] &= ~(0x1ul << 0);
            break;

        case 1:
            p_hw_tim->crchx[1] &= ~(0x1ul << 0);
            break;

        case 2:
            p_hw_tim->crchx[2] &= ~(0x1ul << 0);
            break;

        default:
            break;
    }
}

/**
 * \brief CRA   Aͨزʹ
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 * \param[in] chan     : ͨ  ZSL42x_TIM_CH0A               
 *                            ZSL42x_TIM_CH0B               
 *                            ZSL42x_TIM_CH1A(TIM3ͨ)    
 *                            ZSL42x_TIM_CH1B(TIM3ͨ)    
 *                            ZSL42x_TIM_CH2A(TIM3ͨ)    
 *                            ZSL42x_TIM_CH2B(TIM3ͨ)    
 *
 * \return 
 */
am_static_inline
void amhw_zsl42x_tim_mode23_cra_enable (amhw_zsl42x_tim_t *p_hw_tim,
                                        uint8_t            chan)
{
    switch(chan / 2) {
        case 0:
            p_hw_tim->crchx[0] |= (0x1ul << 0);
            break;

        case 1:
            p_hw_tim->crchx[1] |= (0x1ul << 0);
            break;

        case 2:
            p_hw_tim->crchx[2] |= (0x1ul << 0);
            break;

        default:
            break;
    }
}

/**
 * \brief ȽϹɲƽ
 */
typedef enum {
    AMHW_ZSL42x_TIM_CMP_BRAKEOUT_Z = 0,      /*  */
    AMHW_ZSL42x_TIM_CMP_BRAKEOUT_NO,         /* Ӱ */
    AMHW_ZSL42x_TIM_CMP_BRAKEOUT_LOW,        /* ǿ͵ƽ */
    AMHW_ZSL42x_TIM_CMP_BRAKEOUT_HIGH,       /* ǿߵƽ */
}amhw_zsl42x_tim_cmp_brakeout_t;

/**
 * \brief BKSB  BͨȽϹɲƽ     ȡ
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 * \param[in] chan     : ͨ  ZSL42x_TIM_CH0A               
 *                            ZSL42x_TIM_CH0B               
 *                            ZSL42x_TIM_CH1A(TIM3ͨ)    
 *                            ZSL42x_TIM_CH1B(TIM3ͨ)    
 *                            ZSL42x_TIM_CH2A(TIM3ͨ)    
 *                            ZSL42x_TIM_CH2B(TIM3ͨ)    
 *
 * \return amhw_zsl42x_tim_cmp_brakeout_tö
 */
am_static_inline
amhw_zsl42x_tim_cmp_brakeout_t
    amhw_zsl42x_tim_mode23_bksb_get (amhw_zsl42x_tim_t *p_hw_tim,
                                     uint8_t            chan)
{
    uint8_t get_value;
    switch(chan / 2) {
        case 0:
            get_value = (p_hw_tim->crchx[0] >> 2) & 0x3ul;
            break;

        case 1:
            get_value = (p_hw_tim->crchx[1] >> 2) & 0x3ul;
            break;

        case 2:
            get_value = (p_hw_tim->crchx[2] >> 2) & 0x3ul;
            break;

        default:
            break;
    }
    return (amhw_zsl42x_tim_cmp_brakeout_t)get_value;
}

/**
 * \brief BKSB  BͨȽϹɲƽ     
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 * \param[in] chan     : ͨ  ZSL42x_TIM_CH0A               
 *                            ZSL42x_TIM_CH0B               
 *                            ZSL42x_TIM_CH1A(TIM3ͨ)    
 *                            ZSL42x_TIM_CH1B(TIM3ͨ)    
 *                            ZSL42x_TIM_CH2A(TIM3ͨ)    
 *                            ZSL42x_TIM_CH2B(TIM3ͨ)    
 * \param[in] choice   : amhw_zsl42x_tim_cmp_brakeout_tö
 *
 * \return 
 */
am_static_inline
void amhw_zsl42x_tim_mode23_bksb_set(amhw_zsl42x_tim_t             *p_hw_tim,
                                     uint8_t                        chan,
                                     amhw_zsl42x_tim_cmp_brakeout_t choice)
{
    switch(chan / 2) {
        case 0:
            p_hw_tim->crchx[0] = (p_hw_tim->crchx[0] & (~(0x3ul << 2))) |
                                 ((choice & 0x3ul) << 2);
            break;

        case 1:
            p_hw_tim->crchx[1] = (p_hw_tim->crchx[1] & (~(0x3ul << 2))) |
                                 ((choice & 0x3ul) << 2);
            break;

        case 2:
            p_hw_tim->crchx[2] = (p_hw_tim->crchx[2] & (~(0x3ul << 2))) |
                                 ((choice & 0x3ul) << 2);
            break;

        default:
            break;
    }
}

/**
 * \brief BKSA  AͨȽϹɲƽ     ȡ
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 * \param[in] chan     : ͨ  ZSL42x_TIM_CH0A               
 *                            ZSL42x_TIM_CH0B               
 *                            ZSL42x_TIM_CH1A(TIM3ͨ)    
 *                            ZSL42x_TIM_CH1B(TIM3ͨ)    
 *                            ZSL42x_TIM_CH2A(TIM3ͨ)    
 *                            ZSL42x_TIM_CH2B(TIM3ͨ)    
 *
 * \return amhw_zsl42x_tim_cmp_brakeout_tö
 */
am_static_inline
amhw_zsl42x_tim_cmp_brakeout_t
    amhw_zsl42x_tim_mode23_bksa_get (amhw_zsl42x_tim_t *p_hw_tim,
                                     uint8_t            chan)
{
    uint8_t get_value;
    switch(chan / 2) {
        case 0:
            get_value = (p_hw_tim->crchx[0] >> 0) & 0x3ul;
            break;

        case 1:
            get_value = (p_hw_tim->crchx[1] >> 0) & 0x3ul;
            break;

        case 2:
            get_value = (p_hw_tim->crchx[2] >> 0) & 0x3ul;
            break;

        default:
            break;
    }
    return (amhw_zsl42x_tim_cmp_brakeout_t)get_value;
}

/**
 * \brief BKSA  AͨȽϹɲƽ     
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 * \param[in] chan     : ͨ  ZSL42x_TIM_CH0A               
 *                            ZSL42x_TIM_CH0B               
 *                            ZSL42x_TIM_CH1A(TIM3ͨ)    
 *                            ZSL42x_TIM_CH1B(TIM3ͨ)    
 *                            ZSL42x_TIM_CH2A(TIM3ͨ)    
 *                            ZSL42x_TIM_CH2B(TIM3ͨ)    
 * \param[in] choice   : amhw_zsl42x_tim_cmp_brakeout_tö
 *
 * \return 
 */
am_static_inline
void amhw_zsl42x_tim_mode23_bksa_set(amhw_zsl42x_tim_t             *p_hw_tim,
                                     uint8_t                        chan,
                                     amhw_zsl42x_tim_cmp_brakeout_t choice)
{
    switch(chan / 2) {
        case 0:
            p_hw_tim->crchx[0] = (p_hw_tim->crchx[0] & (~(0x3ul << 0))) |
                                 ((choice & 0x3ul) << 0);
            break;

        case 1:
            p_hw_tim->crchx[1] = (p_hw_tim->crchx[1] & (~(0x3ul << 0))) |
                                 ((choice & 0x3ul) << 0);
            break;

        case 2:
            p_hw_tim->crchx[2] = (p_hw_tim->crchx[2] & (~(0x3ul << 0))) |
                                 ((choice & 0x3ul) << 0);
            break;

        default:
            break;
    }
}

#define AMHW_ZSL42x_TIM_DTR_VCE     (0x1UL << 14)  /* VCɲʹ */
#define AMHW_ZSL42x_TIM_DTR_SAFEEN  (0x1UL << 13)  /* Safety ɲʹ */
#define AMHW_ZSL42x_TIM_DTR_MOE     (0x1UL << 12)  /* PWMʹ */
#define AMHW_ZSL42x_TIM_DTR_AOE     (0x1UL << 11)  /* PWMԶʹ */
#define AMHW_ZSL42x_TIM_DTR_BKE     (0x1UL << 10)  /* ɲʹ */
#define AMHW_ZSL42x_TIM_DTR_DTEN    (0x1UL << 9)   /* ʹ */

/**
 * \brief ʱĴTIMx_DTR        
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 * \param[in] type     : AMHW_ZSL42x_TIM_DTR_VCE
 *                       .
 *                       .
 *                       .
 *                       AMHW_ZSL42x_TIM_DTR_DTEN
 *
 * \return 
 */
am_static_inline
void amhw_zsl42x_tim_mode23_dtr_disable (amhw_zsl42x_tim_t *p_hw_tim,
                                         uint16_t           type)
{
    p_hw_tim->dtr &= ~type;
}

/**
 * \brief ʱĴTIMx_DTR        ʹ
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 * \param[in] type     : AMHW_ZSL42x_TIM_DTR_VCE
 *                       .
 *                       .
 *                       .
 *                       AMHW_ZSL42x_TIM_DTR_DTEN
 *
 * \return 
 */
am_static_inline
void amhw_zsl42x_tim_mode23_dtr_enable (amhw_zsl42x_tim_t *p_hw_tim,
                                        uint16_t           type)
{
    p_hw_tim->dtr |= type;
}

/**
 * \brief bksel    ɲѡ       TIM1/2ʹñĵɲ
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 *
 * \note TIM0/TIM3ѡЧ
 *
 * \return 
 */
am_static_inline
void amhw_zsl42x_tim_mode23_bksel_mine (amhw_zsl42x_tim_t *p_hw_tim)
{
    p_hw_tim->dtr &= ~(0x1ul << 8);
}

/**
 * \brief bksel    ɲѡ      TIM1/2ʹ TIM0ĵɲ
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 *
 * \note TIM0/TIM3ѡЧ
 *
 * \return 
 */
am_static_inline
void amhw_zsl42x_tim_mode23_bksel_tim0 (amhw_zsl42x_tim_t *p_hw_tim)
{
    p_hw_tim->dtr |= (0x1ul << 8);
}

/**
 * \brief DTR  ʱ   ȡ
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 *
 * \note  DTR[7]   = 0     T = DTR[6:0]+2             2-129      step=1
 *
 *        DTR[7:6] = 10    T = {DTR[5:0]+64}*2 +2     130-256    step=2
 *
 *        DTR[7:5] = 110   T = {DTR[4:0]+32}*8 +2     258-506    step=8
 *
 *        DTR[7:5] = 111   T = {DTR[4:0]+32}*16 +2    514-1010   step=16
 *
 * \return 趨ֵ
 */
am_static_inline
uint8_t amhw_zsl42x_tim_mode23_dtr_get (amhw_zsl42x_tim_t *p_hw_tim)
{
    return ((p_hw_tim->dtr >> 0) & 0xfful);
}

/**
 * \brief DTR  ʱ   
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 * \param[in] data     : 趨ֵ(2-256258-506514-1010)
 *
 * \note  DTR[7]   = 0     T = DTR[6:0]+2             2-129      step=1
 *
 *        DTR[7:6] = 10    T = {DTR[5:0]+64}*2 +2     130-256    step=2
 *
 *        DTR[7:5] = 110   T = {DTR[4:0]+32}*8 +2     258-506    step=8
 *
 *        DTR[7:5] = 111   T = {DTR[4:0]+32}*16 +2    514-1010   step=16
 *
 * \return 
 */
am_static_inline
void amhw_zsl42x_tim_mode23_dtr_set(amhw_zsl42x_tim_t *p_hw_tim,
                                    uint16_t           data)
{
	if(data <= 2) {
		data = 0;
    }else if(data <= 129) {
        data = data - 2;
    } else if(data <= 257) {
        if(data == 257) {
            data = 256;
        }
        data = (((data - 2) >> 1) - 64) | 0x80;
    } else if(data <= 513) {
        if((data <= 513) || (data > 506)) {
            data = 506;
        }
        data = (((data - 2) >> 3) - 32) | 0xC0;
    } else if(data <= 1010) {
        data = (((data - 2) >> 4) - 32) | 0xE0;
    } else {
        data = 0;
    }

    p_hw_tim->dtr = (p_hw_tim->dtr & (~(0xfful << 0))) |
                    ((data & 0xfful) << 0);
}

/**
 * \brief RCR  ظڼֵ   ȡ
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 *
 * \note  RCR+1ڸ¼£ʱڲRCR_CNT1
 *        RCR_CNTRCRֵҲ¼UEVź
 *
 * \return 趨ֵ
 */
am_static_inline
uint8_t amhw_zsl42x_tim_mode23_rcr_get (amhw_zsl42x_tim_t *p_hw_tim)
{
    return ((p_hw_tim->rcr >> 0) & 0xfful);
}

/**
 * \brief RCR  ظڼֵ   
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 * \param[in] data     : 趨ֵ
 *
 * \note  RCR+1ڸ¼£ʱڲRCR_CNT1
 *        RCR_CNTRCRֵҲ¼UEVź
 *
 * \return 
 */
am_static_inline
void amhw_zsl42x_tim_mode23_rcr_set(amhw_zsl42x_tim_t *p_hw_tim,
                                    uint8_t            data)
{
    p_hw_tim->rcr = (p_hw_tim->rcr & (~(0xfful << 0))) |
                    ((data & 0xfful) << 0);
}

/**
 * \brief CCRȽϲĴȡ
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 * \param[in] chan     : ͨ  ZSL42x_TIM_CHxx
 *
 * \return 趨ֵ
 */
am_static_inline
uint16_t amhw_zsl42x_tim_mode23_ccr_get (amhw_zsl42x_tim_t   *p_hw_tim,
                                       uint8_t            chan)
{
    return p_hw_tim->ccr[chan] & 0xfffful;
}

/**
 * \brief CCRȽϲĴ
 *
 * \param[in] p_hw_tim : ָTIMʱĴָ
 * \param[in] chan     : ͨ  ZSL42x_TIM_CHxx
 * \param[in] data     : 趨ֵ
 *
 * \return 
 */
am_static_inline
void amhw_zsl42x_tim_mode23_ccr_set(amhw_zsl42x_tim_t   *p_hw_tim,
                                  uint8_t            chan,
                                  uint16_t           data)
{
    p_hw_tim->ccr[chan] = data;
}

/**
 * @}
 */

#ifdef __cplusplus
}
#endif

#endif /* __AMHW_ZSL42x_TIM_H */

/* end of file */
