/*******************************************************************************
*                                 AMetal
*                       ----------------------------
*                       innovating embedded platform
*
* Copyright (c) 2001-2018 Guangzhou ZHIYUAN Electronics Co., Ltd.
* All rights reserved.
*
* Contact information:
* web site:    http://www.zlg.cn/
*******************************************************************************/

/**
 * \file
 * \brief ʱADTIMPWMʵ
 *
 * \internal
 * \par Modification history
 * - 1.00 19-10-09  zp, first implementation
 * \endinternal
 */
#include "zsl42x_clk.h"
#include "am_zsl42x_adtim_pwm.h"
#include "am_int.h"
#include "am_clk.h"
#include "am_gpio.h"

/*******************************************************************************
* 
*******************************************************************************/

/** \brief PWM */
static int __adtim_pwm_config (void          *p_drv,
                               int            chan,
                               unsigned long  duty_ns,
                               unsigned long  period_ns);

/** \brief ʹPWM */
static int __adtim_pwm_enable (void *p_drv, int chan);

/** \brief PWM */
static int __adtim_pwm_disable (void *p_drv, int chan);

/** \brief PWM */
static const struct am_pwm_drv_funcs __g_adtim_pwm_drv_funcs = {
    __adtim_pwm_config,
    __adtim_pwm_enable,
    __adtim_pwm_disable,
};

/******************************************************************************/

/** \brief PWM */
static int __adtim_pwm_config (void          *p_drv,
                               int            chan,
                               unsigned long  duty_ns,
                               unsigned long  period_ns)
{
    am_zsl42x_adtim_pwm_dev_t *p_dev      = (am_zsl42x_adtim_pwm_dev_t *)p_drv;
    amhw_zsl42x_adtim_t       *p_hw_adtim = (amhw_zsl42x_adtim_t *)
                                            p_dev->p_devinfo->adtim_regbase;

    amhw_zsl42x_adtim_basecnt_cfg_t basecnt;

    amhw_zsl42x_adtim_timx_chx_port_cfg_t  portcfg;
    uint32_t clkfreq;
    uint16_t fre_div = 1;
    uint32_t period_c, duty_c, temp;

    /* Ϸ */
    if ((period_ns == 0) || (duty_ns > 4294967295UL) ||
        (period_ns > 4294967295UL) || (duty_ns > period_ns)) {
        return -AM_EINVAL;
    }

    clkfreq  = am_clk_rate_get(CLK_PCLK);

    /* õǼֵCNT, ʽns * 10e-9= cnt * (1/clkfrq) */
    period_c = (uint64_t)(period_ns) * (clkfreq) / (uint64_t)1000000000;
    duty_c   = (uint64_t)(duty_ns)   * (clkfreq) / (uint64_t)1000000000;

    /* С65536ʱƵ(ֵΪ1,1Ϊ1Ƶ) */
    temp = period_c / 65536 + 1;

    /* 16λʱҪȡúʵķƵֵ */
    for (fre_div = 1; fre_div < temp; ) {
        fre_div++;          /* Ƶϵ */
    }

    /* Ƶϵ */
    if(fre_div <= 1) {
        basecnt.cntclkdiv  = AMHW_ZSL42x_ADTIM_PCLK_DIV1;
        fre_div = 1;
    } else if(fre_div <= 2) {
        basecnt.cntclkdiv  = AMHW_ZSL42x_ADTIM_PCLK_DIV2;
        fre_div = 2;
    } else if(fre_div <= 4) {
        basecnt.cntclkdiv  = AMHW_ZSL42x_ADTIM_PCLK_DIV4;
        fre_div = 4;
    } else if(fre_div <= 8) {
        basecnt.cntclkdiv = AMHW_ZSL42x_ADTIM_PCLK_DIV8;
        fre_div = 8;
    } else if(fre_div <= 16) {
        basecnt.cntclkdiv  = AMHW_ZSL42x_ADTIM_PCLK_DIV16;
        fre_div = 16;
    } else if(fre_div <= 64) {
        basecnt.cntclkdiv  = AMHW_ZSL42x_ADTIM_PCLK_DIV64;
        fre_div = 64;
    } else if(fre_div <= 256) {
        basecnt.cntclkdiv  = AMHW_ZSL42x_ADTIM_PCLK_DIV256;
        fre_div = 256;
    } else {
        basecnt.cntclkdiv  = AMHW_ZSL42x_ADTIM_PCLK_DIV1024;
        fre_div = 1024;
    }

    basecnt.cntmode   = AMHW_ZSL42x_ADTIM_CNT_MODE_SAWTOOTH;
    basecnt.cntdir    = AMHW_ZSL42x_ADTIM_CNT_DIR_UP;

    /* öʱģʽ򡢼ʱӷƵϵ */
    amhw_zsl42x_adtim_init(p_hw_adtim, &basecnt);

    /* ¼PWMڼƵ */
    period_c = period_c / fre_div;
    duty_c   = duty_c   / fre_div;

    /* Ϸ */
    if ((period_c == 0) || (period_c > 65535UL)) {
        return -AM_EINVAL;
    }

    /* ü */
    amhw_zsl42x_adtim_setperiod(p_hw_adtim, period_c);

    /*  */
    amhw_zsl42x_adtim_clearcount(p_hw_adtim);

    duty_c = period_c - duty_c;

    portcfg.portmode  = AMHW_ZSL42x_ADTIM_PORT_MODE_CMP_OUTPUT;
    portcfg.outenable = AM_TRUE;
    portcfg.perc      = AMHW_ZSL42x_ADTIM_PERIOD_INV;
    portcfg.cmpc      = AMHW_ZSL42x_ADTIM_COMPARE_INV;
    portcfg.stastps   = AMHW_ZSL42x_ADTIM_STATE_SEL_SS;
    portcfg.disval    = AMHW_ZSL42x_ADTIM_DISVAL_NORM;
    portcfg.dissel    = AMHW_ZSL42x_ADTIM_DISSEL0;
    portcfg.fltenable = AM_FALSE;
    portcfg.fltclk    = AMHW_ZSL42x_ADTIM_FLTCLK_PCLK_DIV1;

    /* PWM */
    if(p_dev->p_devinfo->comp == 1) {

        amhw_zsl42x_adtim_setcomparevalue(p_hw_adtim, 
                                          AMHW_ZSL42x_ADTIM_COMPARE_A, 
                                          duty_c);
        amhw_zsl42x_adtim_setcomparevalue(p_hw_adtim, 
                                          AMHW_ZSL42x_ADTIM_COMPARE_B, 
                                          duty_c);

        portcfg.staout = (amhw_zsl42x_adtim_port_out_t)p_dev->p_devinfo->ocpolarity;
        portcfg.stpout = (amhw_zsl42x_adtim_port_out_t)p_dev->p_devinfo->ocpolarity;
        amhw_zsl42x_adtim_timxchxportcfg(p_hw_adtim,
                                         AMHW_ZSL42x_ADTIM_CHX_A,
                                         &portcfg);

        portcfg.staout = (amhw_zsl42x_adtim_port_out_t)(!p_dev->p_devinfo->ocpolarity);
        portcfg.stpout = (amhw_zsl42x_adtim_port_out_t)(!p_dev->p_devinfo->ocpolarity);
        amhw_zsl42x_adtim_timxchxportcfg(p_hw_adtim,
                                         AMHW_ZSL42x_ADTIM_CHX_B,
                                         &portcfg);

    /* PWM*/
    } else {

        amhw_zsl42x_adtim_setcomparevalue(p_hw_adtim, 
                                          (amhw_zsl42x_adtim_compare_x_t)chan, 
                                          duty_c);

        portcfg.staout = (amhw_zsl42x_adtim_port_out_t)p_dev->p_devinfo->ocpolarity;
        portcfg.stpout = (amhw_zsl42x_adtim_port_out_t)p_dev->p_devinfo->ocpolarity;
        amhw_zsl42x_adtim_timxchxportcfg(p_hw_adtim,
                                         (amhw_zsl42x_adtim_chx_t)chan,
                                         &portcfg);
    }

    return AM_OK;
}

/**
 * \brief ʹPWMͨ
 */
static int __adtim_pwm_enable (void *p_drv, int chan)
{
    int i = 0, t = 0, enable_flag = 0;
    am_zsl42x_adtim_pwm_dev_t      *p_dev      = (am_zsl42x_adtim_pwm_dev_t *)
                                                 p_drv;
    amhw_zsl42x_adtim_t            *p_hw_adtim = (amhw_zsl42x_adtim_t *)
                                                 p_dev->p_devinfo->adtim_regbase;
    am_zsl42x_adtim_pwm_chaninfo_t *p_chaninfo = p_dev->p_devinfo->p_chaninfo;

    uint8_t comp_pin_flag = 0;

    for(i = 0; i <= p_dev->p_devinfo->channels_num; i++) {

        if((p_chaninfo[i].channel & 0x7f) ==  chan) {

            /* PWM */
            if(p_dev->p_devinfo->comp == 1){

                /* Bͨʼ */
                for(t = 0; t <= p_dev->p_devinfo->channels_num; t++) {
                    if( p_chaninfo[t].channel == 0) {
                        am_gpio_pin_cfg(p_chaninfo[t].gpio, p_chaninfo[t].func);
                        comp_pin_flag |= (1 << 0);
                    }
                    if( p_chaninfo[t].channel == 1) {
                        am_gpio_pin_cfg(p_chaninfo[t].gpio, p_chaninfo[t].func);
                        comp_pin_flag |= (1 << 1);
                    }
                    if(comp_pin_flag == 0x3) {
                        break;
                    }
                }

                enable_flag = 1;
                break;

            /* PWM */
            } else {
                am_gpio_pin_cfg(p_chaninfo[chan].gpio, p_chaninfo[chan].func);

                enable_flag = 1;
                break;
            }
        }
    }

    /* ͨЧ */
    if(enable_flag == 0){
        return -AM_EINVAL;
    }

    /* жϱ־ */
    amhw_zsl42x_adtim_clearallirqflag(p_hw_adtim);

    /*  */
    amhw_zsl42x_adtim_clearcount(p_hw_adtim);

    /* ʱ */
    amhw_zsl42x_adtim_startcount(p_hw_adtim);

    return AM_OK;
}

/**
 * \brief PWMͨ
 */
static int __adtim_pwm_disable (void *p_drv, int chan)
{
    int i = 0, t = 0, disable_flag = 0;
    am_zsl42x_adtim_pwm_dev_t      *p_dev    = (am_zsl42x_adtim_pwm_dev_t *)p_drv;
    amhw_zsl42x_adtim_t            *p_hw_adtim = (amhw_zsl42x_adtim_t *)p_dev->p_devinfo->adtim_regbase;
    am_zsl42x_adtim_pwm_chaninfo_t *p_chaninfo = p_dev->p_devinfo->p_chaninfo;

    uint8_t comp_pin_flag = 0;

    for(i = 0; i <= p_dev->p_devinfo->channels_num; i++) {

        if((p_chaninfo[i].channel & 0x7f) ==  chan) {

            /* PWM */
            if(p_dev->p_devinfo->comp == 1){

                /* Bͨʼ */
                for(t = 0; t <= p_dev->p_devinfo->channels_num; t++) {
                    if( p_chaninfo[t].channel == 0) {
                        am_gpio_pin_cfg(p_chaninfo[t].gpio, p_chaninfo[t].dfunc);
                        comp_pin_flag |= (1 << 0);
                    }
                    if( p_chaninfo[t].channel == 1) {
                        am_gpio_pin_cfg(p_chaninfo[t].gpio, p_chaninfo[t].dfunc);
                        comp_pin_flag |= (1 << 1);
                    }
                    if(comp_pin_flag == 0x3) {
                        break;
                    }
                }

                disable_flag = 1;
                break;

            /* PWM */
            }else{
                am_gpio_pin_cfg(p_chaninfo[chan].gpio, p_chaninfo[chan].dfunc);

                disable_flag = 1;
                break;
            }
        }
    }

    /* ͨЧ */
    if(disable_flag == 0){
        return -AM_EINVAL;
    }

    /*  */
    amhw_zsl42x_adtim_clearcount(p_hw_adtim);

    /* ʱֹͣ */
    amhw_zsl42x_adtim_stopcount(p_hw_adtim);

    /* жϱ־ */
    amhw_zsl42x_adtim_clearallirqflag(p_hw_adtim);

    return AM_OK;
}

/**
  * \brief adtim pwmʼ
  */
am_pwm_handle_t am_zsl42x_adtim_pwm_init (am_zsl42x_adtim_pwm_dev_t      *p_dev,
                                     const am_zsl42x_adtim_pwm_devinfo_t *p_devinfo)
{
    amhw_zsl42x_adtim_basecnt_cfg_t basecnt;
    amhw_zsl42x_adtim_t  *p_hw_adtim = NULL;

    if (p_dev == NULL || p_devinfo == NULL) {
        return NULL;
    }

    p_dev->chan_max = 2;

    if(p_dev->chan_max < p_devinfo->channels_num) {
        return NULL;
    }

    if (p_devinfo->pfn_plfm_init) {
        p_devinfo->pfn_plfm_init();
    }

    p_dev->p_devinfo        = p_devinfo;
    p_hw_adtim              = (amhw_zsl42x_adtim_t *)p_dev->p_devinfo->adtim_regbase;
    p_dev->pwm_serv.p_funcs = (struct am_pwm_drv_funcs *)&__g_adtim_pwm_drv_funcs;
    p_dev->pwm_serv.p_drv   = p_dev;

    basecnt.cntmode   = AMHW_ZSL42x_ADTIM_CNT_MODE_SAWTOOTH;
    basecnt.cntdir    = AMHW_ZSL42x_ADTIM_CNT_DIR_UP;
    basecnt.cntclkdiv = AMHW_ZSL42x_ADTIM_PCLK_DIV1;

    /* öʱģʽ򡢼ʱӷƵϵ */
    amhw_zsl42x_adtim_init(p_hw_adtim, &basecnt);

    /*  */
    amhw_zsl42x_adtim_clearcount(p_hw_adtim);

    return &(p_dev->pwm_serv);
}

void am_zsl42x_adtim_pwm_deinit (am_pwm_handle_t handle)
{

    am_zsl42x_adtim_pwm_dev_t *p_dev    = (am_zsl42x_adtim_pwm_dev_t *)handle;
    amhw_zsl42x_adtim_t       *p_hw_adtim = NULL;

    if (p_dev == NULL ) {
        return;
    }

    p_hw_adtim   = (amhw_zsl42x_adtim_t *)p_dev->p_devinfo->adtim_regbase;

    /*  */
    amhw_zsl42x_adtim_clearcount(p_hw_adtim);

    /* رTIMģ */
    amhw_zsl42x_adtim_stopcount(p_hw_adtim);

    /* ֶλĴ */
    amhw_zsl42x_adtim_deinit(p_hw_adtim);

    p_dev->pwm_serv.p_drv = NULL;

    if (p_dev->p_devinfo->pfn_plfm_deinit) {
        p_dev->p_devinfo->pfn_plfm_deinit();
    }
}

/* end of file */
