/*******************************************************************************
*                                 AMetal
*                       ----------------------------
*                       innovating embedded platform
*
* Copyright (c) 2001-2018 Guangzhou ZHIYUAN Electronics Co., Ltd.
* All rights reserved.
*
* Contact information:
* web site:    http://www.zlg.cn/
*******************************************************************************/

/**
 * \file
 * \brief I2CӻI2Cӻ׼ӿ
 *
 * \internal
 * \par Modification History
 * - 1.00 20-03-18
 * \endinternal
 */

#include "ametal.h"
#include "am_zlg_i2c_slv.h"
#include "hw/amhw_zsn700_i2c.h"

/*******************************************************************************
  
*******************************************************************************/

/** \brief I2CӲʼ */
static int __i2c_slv_hard_init(am_zlg_i2c_slv_dev_t *p_dev);
/** \brief I2Cжϴ */
static void __i2c_slv_irq_handler (void *p_arg);
/** \brief I2Cӻ豸 */
static int __i2c_slv_setup (void *p_drv, am_i2c_slv_device_t *p_i2c_slv_dev);
/** \brief ر I2Cӻ豸 */
static int __i2c_slv_shutdown (void *p_drv, am_i2c_slv_device_t *p_i2c_slv_dev);
/** \brief ȡõĴӻ豸 */
static int __i2c_slv_num_get(void *p_drv);

/**
 * \brief I2C豸 
 */
static am_const struct am_i2c_slv_drv_funcs __g_i2c_slv_drv_funcs = {
    __i2c_slv_setup,
    __i2c_slv_shutdown,
    __i2c_slv_num_get,
};

/**
 *  \brief I2CӲʼ
 *  \note  1: ô豸ĵַ ַλ
 *         2ж
 */
static int __i2c_slv_hard_init (am_zlg_i2c_slv_dev_t *p_dev)
{
    amhw_zsn700_i2c_t *p_hw_i2c  = NULL;

    if (p_dev == NULL) {
        return -AM_EINVAL;
    }

    p_hw_i2c = (amhw_zsn700_i2c_t *) (p_dev->p_devinfo->i2c_regbase);

    /* I2CƼĴ */
    p_hw_i2c->i2c_cr = 0;

    /* ӻģʽӦ */
    amhw_zsn700_i2c_cr_set(p_hw_i2c, AMHW_ZSN700_I2C_REPLY_ENABLE);

    /* òʼ */
    amhw_zsn700_i2c_tm_disable(p_hw_i2c);

    return AM_OK;
}

/**
 * \brief ʼӻ豸
 */
static int __i2c_slv_setup (void *p_drv, am_i2c_slv_device_t *p_i2c_slv_dev)
{
    int                     i;
    uint8_t                 zlg_i2c_slv_dev_num ;
    am_zlg_i2c_slv_dev_t   *p_dev      = NULL;
    amhw_zsn700_i2c_t      *p_hw_i2c   = NULL;

    if ((p_drv == NULL) || (p_i2c_slv_dev == NULL)) {
        return -AM_EINVAL;
    }

    p_dev    = (am_zlg_i2c_slv_dev_t *)p_drv;
    p_hw_i2c = (amhw_zsn700_i2c_t *)(p_dev->p_devinfo->i2c_regbase);

    /* ȡܵĴ豸 */
    zlg_i2c_slv_dev_num = p_dev->zlg_i2c_slv_dev_num;

    for ( i = 0; i < zlg_i2c_slv_dev_num; i++) {
        if( NULL == p_dev->p_i2c_slv_dev[i]) {
            p_dev->p_i2c_slv_dev[i] = p_i2c_slv_dev;
            break ;
        }
    }

    /* ӻ豸ùˣ޷ôӻ豸 */
    if ( i == zlg_i2c_slv_dev_num) {
        return -AM_EAGAIN;
    }

    /* رI2Cò */
    amhw_zsn700_i2c_disable (p_hw_i2c);

    /* жǷ ҪӦ 㲥ַ */
    if ( p_i2c_slv_dev->dev_flags & AM_I2C_SLV_GEN_CALL_ACK) {
        amhw_zsn700_i2c_gen_call_ack(p_hw_i2c);
    } else {
        amhw_zsn700_i2c_gen_call_nack(p_hw_i2c);
    }

    /**
     * \brief   i Ŵӻ豸ַ
     *         i = 0 ,ŵ1ӵַ
     *         i = 1 ,ŵ2ӵַ Դ
     *
     * \note ZLGֻһӻַ ,ֱӴż 
     */
    amhw_zsn700_i2c_slave_addr_set (p_hw_i2c, p_i2c_slv_dev->dev_addr);

    amhw_zsn700_i2c_enable(p_hw_i2c);
    am_int_enable(p_dev->p_devinfo->inum);

    return AM_OK;
}

/**
 * \brief ر ӻ豸
 */
static int __i2c_slv_shutdown (void *p_drv, am_i2c_slv_device_t *p_i2c_slv_dev)
{
    am_zlg_i2c_slv_dev_t *p_dev    = (am_zlg_i2c_slv_dev_t *)p_drv;
    amhw_zsn700_i2c_t   *p_hw_i2c_slv = NULL;

    if ( (p_dev              == NULL) ||
         (p_i2c_slv_dev      == NULL)) {
        return -AM_EINVAL;
    }
    p_hw_i2c_slv = (amhw_zsn700_i2c_t *) p_dev->p_devinfo->i2c_regbase;

    /**
     * \brief ʧܶӦӵַ ,ݴӻַȷùرһӻ豸
     *
     * \note ZLGֻһ豸 ֱӹرʧI2C
     */
    p_dev->p_i2c_slv_dev[0] = NULL;
    am_int_disable(p_dev->p_devinfo->inum);
    amhw_zsn700_i2c_disable (p_hw_i2c_slv);

    return AM_OK;
}

/**
 * \brief ȡôӻ豸
 */
static int __i2c_slv_num_get (void *p_drv)
{
    int     i;
    int     count = 0;
    uint8_t zlg_i2c_slv_dev_num ;
    am_zlg_i2c_slv_dev_t *p_dev = (am_zlg_i2c_slv_dev_t *)p_drv;

    if ((p_dev == NULL)) {
        return -AM_EINVAL;
    }
    /* ȡܵĴ豸 */
    zlg_i2c_slv_dev_num = p_dev->zlg_i2c_slv_dev_num;

    for (i = 0; i < zlg_i2c_slv_dev_num; i++) {
        if (NULL == p_dev->p_i2c_slv_dev[i]) {
            count++;
        }
    }
    return count;
}

/**
 * \brief ʱص
 *
 * \note ڿʼź֮ 1뻹δֹͣź 򽫴ӻиλ
 */
static void __i2c_slv_timing_callback (void *p_arg)
{
    am_zlg_i2c_slv_dev_t  *p_dev     = ( am_zlg_i2c_slv_dev_t *) p_arg;
    amhw_zsn700_i2c_t     *p_hw_i2c  = NULL;
    p_hw_i2c  = (amhw_zsn700_i2c_t *) (p_dev->p_devinfo->i2c_regbase);

    /* ƽ̨ʼ֮ Ĵֵλ */
    if (p_dev->p_devinfo->pfn_plfm_init) {
        p_dev->p_devinfo->pfn_plfm_init();
    }

    /* رI2Cò */
    amhw_zsn700_i2c_disable (p_hw_i2c);

    /** жǷ ҪӦ 㲥ַ */
    if ( p_dev->p_i2c_slv_dev[0]->dev_flags & AM_I2C_SLV_GEN_CALL_ACK) {
        amhw_zsn700_i2c_gen_call_ack(p_hw_i2c);
    } else {
        amhw_zsn700_i2c_gen_call_nack(p_hw_i2c);
    }

    amhw_zsn700_i2c_slave_addr_set(p_hw_i2c, p_dev->p_i2c_slv_dev[0]->dev_addr);

    amhw_zsn700_i2c_enable(p_hw_i2c);

    __i2c_slv_hard_init(p_dev);

    /** ֹͣʱ */
    am_softimer_stop(&p_dev->softimer);
}

/**
 * \brief I2C жϴ
 *
 * \param[in] p_arg : ָI2C豸ṹָ
 *
 * \return 
 */
static void __i2c_slv_irq_handler (void *p_arg)
{
    static uint8_t           rx_len = 0;
    uint8_t                  state;
    uint8_t                  rx_data;
    uint8_t                  tx_data;
    amhw_zsn700_i2c_t       *p_hw_i2c_slv;
    am_zlg_i2c_slv_dev_t    *p_dev          = (am_zlg_i2c_slv_dev_t *)p_arg;
    am_i2c_slv_device_t     *p_i2c_slv_dev  = p_dev->p_i2c_slv_dev[0];

    p_hw_i2c_slv = (amhw_zsn700_i2c_t *)p_dev->p_devinfo->i2c_regbase;
    state = amhw_zsn700_i2c_status_get(p_hw_i2c_slv);

    switch(state) {
    case 0xa0:  /* յֹͣظʼ */
        if (NULL != p_i2c_slv_dev->p_cb_funs->pfn_tran_stop) {
            p_i2c_slv_dev->p_cb_funs->pfn_tran_stop(p_i2c_slv_dev->p_arg); /* readʼ */
        }
        rx_len = 0;
        amhw_zsn700_i2c_cr_set(p_hw_i2c_slv, AMHW_ZSN700_I2C_REPLY_ENABLE);
        break;

    case 0x60:  /* ѽյ(ƥ)SLA+WѽACK */
    case 0x70:  /* ѽͨõõַ0x00ѽACK */
        rx_len = 0;
        amhw_zsn700_i2c_cr_set(p_hw_i2c_slv, AMHW_ZSN700_I2C_REPLY_ENABLE);
        break;

    case 0x68:  /* ʱSLA+дʧٲãѽSLA+WѷACK */
    case 0x78:  /* ʱSLA+джʧٲãѽͨõõַѷACK */
    case 0x88:  /* ǰһѰַʹӵַѽֽڣѷطACK */
        amhw_zsn700_i2c_cr_set(p_hw_i2c_slv, AMHW_ZSN700_I2C_REPLY_ENABLE);
        break;

    case 0x80:  /* ǰһѰַʹӵַѽֽڣѷACK */
    case 0x98:  /* ǰһѰַʹͨõõַѽݣѷطACK */
    case 0x90:  /* ǰһѰַʹͨõõַѽݣѷACK */
        amhw_zsn700_i2c_cr_set(p_hw_i2c_slv, AMHW_ZSN700_I2C_REPLY_ENABLE);
        rx_data = amhw_zsn700_i2c_dat_read(p_hw_i2c_slv);//
        if(rx_len) {    /* Խյĵһ(ӻ豸ӵַ) */
            /* ܻص */
            if (p_dev->is_gen_call == AM_TRUE) {
                /** ǹ㲥 */
                if(p_i2c_slv_dev->p_cb_funs->pfn_gen_call) {
                    p_i2c_slv_dev->p_cb_funs->pfn_gen_call(p_i2c_slv_dev->p_arg, rx_data);
                }
            } else {
                if(p_i2c_slv_dev->p_cb_funs->pfn_rxbyte_put) {
                    p_i2c_slv_dev->p_cb_funs->pfn_rxbyte_put(p_i2c_slv_dev->p_arg, rx_data);
                }
            }
        }
        rx_len++;
        break;

    case 0xa8:  /* ѽSLA+RѷACK,ҪݲյACKλ */
    case 0xb0:  /* ʱSLA+джʧٲãѽSLA+RѷACK */
    case 0xc8:  /* װֽѱͣѽACK */
    case 0xb8:  /* ѷݣѽACK */
        if (NULL != p_i2c_slv_dev->p_cb_funs->pfn_txbyte_get) {
            p_i2c_slv_dev->p_cb_funs->pfn_txbyte_get(p_i2c_slv_dev->p_arg, &tx_data);
        }
        amhw_zsn700_i2c_dat_write(p_hw_i2c_slv,tx_data);
        break;

    case 0xc0:  /* ݣշACKn */
        if (NULL != p_i2c_slv_dev->p_cb_funs->pfn_tran_stop) {
            p_i2c_slv_dev->p_cb_funs->pfn_tran_stop(p_i2c_slv_dev->p_arg); /* read */
        }
        break;
    }

    /* жϱ־ */
    amhw_zsn700_i2c_cr_clear(p_hw_i2c_slv, AMHW_ZSN700_I2C_INT_FLAG);
}

/**
 * \brief I2Cʼ
 *
 * \note Ӳʼ ͨûÿʼ豸ʼ
 */
am_i2c_slv_handle_t am_zlg_i2c_slv_init (
        am_zlg_i2c_slv_dev_t           *p_dev,
        const am_zlg_i2c_slv_devinfo_t *p_devinfo)
{
    if (p_dev == NULL || p_devinfo == NULL ) {
        return NULL;
    }

    p_dev->i2c_slv_serv.p_funcs = \
            (struct am_i2c_slv_drv_funcs *)&__g_i2c_slv_drv_funcs;
    p_dev->i2c_slv_serv.p_drv   = p_dev;
    p_dev->p_devinfo            = p_devinfo;
    p_dev->zlg_i2c_slv_dev_num  = 1;  /* ֻһӻ豸 */
    p_dev->is_gen_call          = AM_FALSE;
    p_dev->p_i2c_slv_dev[0]     = NULL;

    if (p_devinfo->pfn_plfm_init) {
        p_devinfo->pfn_plfm_init();
    }

    /* ʼʱ */
    am_softimer_init(&p_dev->softimer, __i2c_slv_timing_callback, p_dev);

    __i2c_slv_hard_init(p_dev);

    /* ж */
    am_int_connect(p_dev->p_devinfo->inum,
                   __i2c_slv_irq_handler,
                   (void *)p_dev);

    return &(p_dev->i2c_slv_serv);
}

/**
 * \brief I2Cʼ
 */
void am_zlg_i2c_slv_deinit (am_i2c_slv_handle_t handle)
{
    amhw_zsn700_i2c_t    *p_hw_i2c_slv = NULL;
    am_zlg_i2c_slv_dev_t *p_dev        = NULL;

    if (NULL == handle) {
        return ;
    }

    p_dev        = (am_zlg_i2c_slv_dev_t *)handle;
    p_hw_i2c_slv = (amhw_zsn700_i2c_t *)p_dev->p_devinfo->i2c_regbase;

    amhw_zsn700_i2c_disable (p_hw_i2c_slv);

    /* ȥж */
    am_int_disable(p_dev->p_devinfo->inum);

    am_int_disconnect(p_dev->p_devinfo->inum,
                      __i2c_slv_irq_handler,
                      (void *)p_dev);

    if (p_dev->p_devinfo->pfn_plfm_deinit) {
        p_dev->p_devinfo->pfn_plfm_deinit();
    }
}

/* end of file */
