/*******************************************************************************
*                                 AMetal
*                       ----------------------------
*                       innovating embedded platform
*
* Copyright (c) 2001-2018 Guangzhou ZHIYUAN Electronics Co., Ltd.
* All rights reserved.
*
* Contact information:
* web site:    http://www.zlg.cn/
*******************************************************************************/

/**
 * \file
 * \brief RAMģʵ
 *
 * \internal
 * \par Modification History
 * - 1.00 19-10-08
 * \endinternal
 */

#include "zsl42x_inum.h"
#include "am_zsl42x_ram.h"
#include "am_types.h"
#include "am_bitops.h"
#include "am_int.h"
#include "am_vdebug.h"
#include "ametal.h"

/******************************************************************************
*   
******************************************************************************/

/** \brief RAM жϴ */
static void __ram_irq_handler (void *p_arg)
{
    amhw_zsl42x_ram_err_clear((amhw_zsl42x_ram_t *)p_arg);
    am_kprintf("RAM IRQ...\r\n");
}

/**
 * \brief RAMģʼ
 *
 * \param[in] p_hw_ram ָRAMĴָ
 *
 * \return 
 */
void am_zsl42x_ram_init (amhw_zsl42x_ram_t *p_hw_ram)
{
    /* ־ */
    amhw_zsl42x_ram_err_clear(p_hw_ram);
    /* ʹܳж */
    amhw_zsl42x_ram_ie_enable(p_hw_ram);

    /* ж */
    am_int_connect(INUM_FLASH_RAM, __ram_irq_handler, p_hw_ram);
    am_int_enable(INUM_FLASH_RAM);
}

/**
 * \brief RAM
 *
 * \param[in] p_hw_ram    ָRAMĴָ
 * \param[in] start_addr  ʼַ
 * \param[in] size        С
 *
 * \return >0: ִн, -AM_EINVAL: ַ
 */
int32_t am_zsl42x_ram_erase (amhw_zsl42x_ram_t *p_hw_ram,
                             uint32_t           start_addr,
                             uint16_t           size)
{
    uint16_t i;

    if (start_addr < RAM_ADDRESS_BASE) {
        start_addr += RAM_ADDRESS_BASE;
    }
    if (RAM_ADDRESS_SIZE < start_addr) {
        return -AM_EINVAL;
    }

    for (i = 0; i < size; i++) {
        if(start_addr + i > RAM_ADDRESS_SIZE) {
            break;
        }
        *(uint8_t *)(start_addr + i) = 0;
    }

    return AM_OK;
}

/**
 * \brief ̻򲿷
 *
 * \param[in] p_hw_ram  ָRAMĴָ
 * \param[in] dst_addr  д뵽ramʼַ
 * \param[in] p_src     Ҫд뵽ramеݵʼַ
 * \param[in] size      д(32bit)ĸ
 *
 * \retval i ʵʳɹд
 */
int32_t am_zsl42x_ram_write (amhw_zsl42x_ram_t *p_hw_ram,
                             uint32_t           dst_addr,
                             uint8_t           *p_src,
                             uint16_t           size)
{
    uint16_t i;

    if (dst_addr < RAM_ADDRESS_BASE) {
        dst_addr += RAM_ADDRESS_BASE;
    }

    /** sizeܴĴС */
    if (size > RAM_SIZE || dst_addr >= RAM_ADDRESS_SIZE) {
        return -AM_EINVAL;
    }

    for (i = 0; i < size; i++) {
        if(dst_addr + i > RAM_ADDRESS_SIZE) {
            break;
        }
        *(uint8_t *)(dst_addr + i) = *(p_src + i);
    }

    return i;
}

/**
 * \brief RAMȡ
 *
 * \param[in] p_hw_ram   ָRAMĴָ
 * \param[in] dst_addr   ȡramʼַ
 * \param[in] p_src      Ҫȡramеݵʼַ
 * \param[in] size       ȡֽڵĸ
 *
 * \retval i ʵʳɹȡ
 */
int32_t am_zsl42x_ram_read (amhw_zsl42x_ram_t *p_hw_ram,
                            uint32_t           dst_addr,
                            uint8_t           *p_src,
                            uint16_t           size)
{
    uint16_t i;

    if (dst_addr < RAM_ADDRESS_BASE) {
        dst_addr += RAM_ADDRESS_BASE;
    }

    /** sizeܴĴС */
    if (size > RAM_SIZE || dst_addr >= RAM_ADDRESS_SIZE) {
        return -AM_EINVAL;
    }

    for (i = 0; i < size; i++) {
        if(dst_addr + i > RAM_ADDRESS_SIZE) {
            break;
        }
        *(p_src + i) = *(uint8_t *)(dst_addr + i);
    }

    return i;
}

/* end of file */
