/*******************************************************************************
*                                 AMetal
*                       ----------------------------
*                       innovating embedded platform
*
* Copyright (c) 2001-2018 Guangzhou ZHIYUAN Electronics Co., Ltd.
* All rights reserved.
*
* Contact information:
* web site:    http://www.zlg.cn/
*******************************************************************************/

/**
 * \file
 * \brief TIMʱʵ
 *
 * \note һ¼ʱеļĴ£Ӳͬʱ URS λ
 * ø±־λTIMx_SRĴе UIF λظ¼ΪTIMx_RCR Ĵ
 * ֵ,ԶװӰӼĴԤװؼĴֵ  ԤƵĻԤ
 * װؼĴֵTIMx_PSC Ĵݣ
 *
 * \internal
 * \par Modification history
 * - 1.00 17-04-21  nwt, first implementation
 * \endinternal
 */
#include "am_int.h"
#include "am_clk.h"
#include "am_zlg_tim_timing.h"
#include "am_clk.h"

/*******************************************************************************
  ʱ
*******************************************************************************/

static const am_timer_info_t * __tim_timing_info_get (void *p_drv);

static int __tim_timing_clkin_freq_get (void *, uint32_t *);
static int __tim_timing_prescale_set (void *, uint8_t, uint32_t );
static int __tim_timing_prescale_get (void *, uint8_t, uint32_t *);
static int __tim_timing_count_get (void *, uint8_t, void *);
static int __tim_timing_rollover_get (void *, uint8_t, void *);
static int __tim_timing_disable (void *, uint8_t);
static int __tim_timing_enable (void *, uint8_t, void *);
static int __tim_timing_callback_set (void *, uint8_t, void (*)(void *), void *);

static void __tim_irq_handler (void *p_arg);

/*******************************************************************************
  ȫֱ
*******************************************************************************/

/* ʱϢ */
static const am_timer_info_t __g_tim_timing_info = {
    16,                                      /* 16λʱ               */
    1,                                       /* ͨ                   */
    AM_TIMER_CAN_INTERRUPT      |            /* Բж             */
    AM_TIMER_AUTO_RELOAD        |            /* ֧Զ             */
    AM_TIMER_INTERMEDIATE_COUNT |            /* ǰֵɶ         */
    AM_TIMER_SUPPORT_ALL_PRESCALE_1_TO_MAX,  /* ԤƵ1 ~ 65536 ֵ */
    65536                                    /* ԤƵֵ65536      */
};

/* TIMʱ */
static const struct am_timer_drv_funcs __g_tim_timing_drv_funcs = {
    __tim_timing_info_get,
    __tim_timing_clkin_freq_get,
    __tim_timing_prescale_set,
    __tim_timing_prescale_get,
    __tim_timing_count_get,
    __tim_timing_rollover_get,
    __tim_timing_enable,
    __tim_timing_disable,
    __tim_timing_callback_set,
};

/*******************************************************************************
  жϴ
*******************************************************************************/

void __tim_irq_handler (void *p_arg)
{
    am_zlg_tim_timing_dev_t *p_dev    = (am_zlg_tim_timing_dev_t *)p_arg;
    amhw_zlg_tim_t          *p_hw_tim = (amhw_zlg_tim_t *)p_dev->p_devinfo->tim_regbase;

    /*
     * жǷǶʱж
     */
    if (amhw_zlg_tim_status_flg_get(p_hw_tim, AMHW_ZLG_TIM_UIF) != 0 ) {

        if (p_dev->pfn_callback != NULL) {
            p_dev->pfn_callback(p_dev->p_arg);
        }

        /* ־ */
        amhw_zlg_tim_status_flg_clr(p_hw_tim, AMHW_ZLG_TIM_UIF);
    }
}

/*******************************************************************************
  ׼ʱʵ
*******************************************************************************/

static const am_timer_info_t * __tim_timing_info_get (void *p_drv)
{
    return &__g_tim_timing_info;
}

/******************************************************************************/
static int __tim_timing_clkin_freq_get (void *p_drv, uint32_t *p_freq)
{
    am_zlg_tim_timing_dev_t *p_dev = (am_zlg_tim_timing_dev_t *)p_drv;

    if ((p_dev == NULL) || (p_freq == NULL)) {
        return -AM_EINVAL;
    }

    /* ȡǰϵͳʱƵʣsystick ʹõϵͳʱ */
    *p_freq =  am_clk_rate_get(p_dev->p_devinfo->clk_num);

    return AM_OK;
}

/******************************************************************************/
static int __tim_timing_prescale_set (void    *p_drv, 
                                      uint8_t  chan, 
                                      uint32_t prescale)
{
    am_zlg_tim_timing_dev_t *p_dev = (am_zlg_tim_timing_dev_t *)p_drv;
    amhw_zlg_tim_t          *p_hw_tim;

    if ((p_drv == NULL) || (chan != 0)) {
        return -AM_EINVAL;
    }

    p_hw_tim = (amhw_zlg_tim_t *)p_dev->p_devinfo->tim_regbase;

    if ((prescale != 0) && (prescale <= 65536)) {
 
        amhw_zlg_tim_prescale_set(p_hw_tim, prescale - 1);  /* ÷Ƶֵ */

        return AM_OK;
    }

    return -AM_ENOTSUP;
}

/******************************************************************************/
static int __tim_timing_prescale_get (void     *p_drv, 
                                      uint8_t   chan, 
                                      uint32_t *p_prescale)
{
    am_zlg_tim_timing_dev_t *p_dev = (am_zlg_tim_timing_dev_t *)p_drv;
    amhw_zlg_tim_t          *p_hw_tim;

    if ((p_dev == NULL) || (p_prescale == NULL) || (chan != 0)) {
        return -AM_EINVAL;
    }
 
    p_hw_tim = (amhw_zlg_tim_t *)p_dev->p_devinfo->tim_regbase;

    /* ȡƵֵ */
    *p_prescale = amhw_zlg_tim_prescale_get(p_hw_tim);

    return AM_OK;
}

/******************************************************************************/
static int __tim_timing_count_get (void *p_drv, uint8_t chan, void *p_count)
{
    am_zlg_tim_timing_dev_t *p_dev = (am_zlg_tim_timing_dev_t *)p_drv;
    amhw_zlg_tim_t          *p_hw_tim;

    if ((p_dev == NULL) || (p_count == NULL) || (chan != 0)) {
        return -AM_EINVAL;
    }
 
    p_hw_tim = (amhw_zlg_tim_t *)p_dev->p_devinfo->tim_regbase;

    *(uint32_t *)p_count = amhw_zlg_tim_count_get(p_hw_tim);

    return AM_OK;
}

/******************************************************************************/
static int __tim_timing_rollover_get (void *p_drv,uint8_t chan,void *p_rollover)
{
    am_zlg_tim_timing_dev_t *p_dev = (am_zlg_tim_timing_dev_t *)p_drv;
    amhw_zlg_tim_t          *p_hw_tim;

    if ((p_dev == NULL) || (p_rollover == NULL) || (chan != 0)) {
        return -AM_EINVAL;
    }
 
    p_hw_tim = (amhw_zlg_tim_t *)p_dev->p_devinfo->tim_regbase;

    *(uint32_t *)p_rollover = amhw_zlg_tim_arr_get(p_hw_tim);

    return AM_OK;
}

/******************************************************************************/
static int __tim_timing_disable (void *p_drv, uint8_t chan)
{
    am_zlg_tim_timing_dev_t *p_dev = (am_zlg_tim_timing_dev_t *)p_drv;
    amhw_zlg_tim_t          *p_hw_tim;
    
    if ((p_dev == NULL) || (chan != 0)) {
        return -AM_EINVAL;
    }
    
    p_hw_tim = (amhw_zlg_tim_t *)p_dev->p_devinfo->tim_regbase;

    /* ܸж */
    amhw_zlg_tim_int_disable(p_hw_tim, AMHW_ZLG_TIM_UIE);

    amhw_zlg_tim_disable(p_hw_tim);

    return AM_OK;
}

/******************************************************************************/
static int __tim_timing_enable (void *p_drv, uint8_t chan, void *p_count)
{
    am_zlg_tim_timing_dev_t *p_dev = (am_zlg_tim_timing_dev_t *)p_drv;
    amhw_zlg_tim_t          *p_hw_tim;

    if ((p_dev == NULL) || (chan != 0) || (p_count == NULL)) {
        return -AM_EINVAL;
    }

    p_hw_tim = (amhw_zlg_tim_t *)p_dev->p_devinfo->tim_regbase;

    /* ֹʱж */
    amhw_zlg_tim_int_disable(p_hw_tim, AMHW_ZLG_TIM_UIE);

    /* ԶװĴֵ */
    amhw_zlg_tim_arr_set(p_hw_tim, *((uint32_t *)p_count));

    /* 0 */
    amhw_zlg_tim_count_set(p_hw_tim, 0);

    /* ¼³ʼPrescaler Repetition */
    amhw_zlg_tim_egr_set(p_hw_tim, AMHW_ZLG_TIM_UG);

    /* ¶ʱʱ¼,־λ */
    amhw_zlg_tim_status_flg_clr(p_hw_tim, AMHW_ZLG_TIM_UG);

    if (p_dev->pfn_callback != NULL) {

        /* ж */
        amhw_zlg_tim_int_enable(p_hw_tim, AMHW_ZLG_TIM_UIE);
    }

    /* ʹܶʱTIM */
    amhw_zlg_tim_enable(p_hw_tim);

    return AM_OK;
}

/******************************************************************************/
static int __tim_timing_callback_set (void     *p_drv,
                                      uint8_t   chan,
                                      void    (*pfn_callback)(void *),
                                      void     *p_arg)
{
    am_zlg_tim_timing_dev_t *p_dev = (am_zlg_tim_timing_dev_t *)p_drv;
    amhw_zlg_tim_t          *p_hw_tim;

    if ((p_dev == NULL) || (chan != 0)) {
        return -AM_EINVAL;
    }
 
    p_hw_tim = (amhw_zlg_tim_t *)p_dev->p_devinfo->tim_regbase;

    /* Ҫж */
    if (pfn_callback == NULL) {
        amhw_zlg_tim_int_disable(p_hw_tim, AMHW_ZLG_TIM_UIE);
    } else {
        p_dev->pfn_callback = pfn_callback;
        p_dev->p_arg   = p_arg;
    }

    return AM_OK;
}

/*******************************************************************************
  Public functions
*******************************************************************************/

am_timer_handle_t am_zlg_tim_timing_init (
    am_zlg_tim_timing_dev_t             *p_dev,
    const am_zlg_tim_timing_devinfo_t  *p_devinfo)
{
    amhw_zlg_tim_t *p_hw_tim = NULL;

    if (NULL == p_dev || NULL == p_devinfo ) {
        return NULL;
    }

    if (p_devinfo->pfn_plfm_init) {
        p_devinfo->pfn_plfm_init();
    }

    p_dev->p_devinfo          = p_devinfo;
    p_hw_tim                  = (amhw_zlg_tim_t *)p_dev->p_devinfo->tim_regbase;
    p_dev->timer_serv.p_funcs = (struct am_timer_drv_funcs *)&__g_tim_timing_drv_funcs;
    p_dev->timer_serv.p_drv   = p_dev;

    p_dev->pfn_callback = NULL;
    p_dev->p_arg        = NULL;

    if ((p_devinfo->tim_type == AMHW_ZLG_TIM_TYPE0) ||
        (p_devinfo->tim_type == AMHW_ZLG_TIM_TYPE1))  {

        /* ضģʽ */
        amhw_zlg_tim_cms_set(p_hw_tim, 0);

        /* ϼ */
        amhw_zlg_tim_dir_set(p_hw_tim, 0);
    }

    /* ʱӷָ:TDTS = Tck_tin */
    amhw_zlg_tim_ckd_set(p_hw_tim, 0);

    /* ¼ */
    amhw_zlg_tim_udis_enable(p_hw_tim);

    am_int_connect(p_dev->p_devinfo->inum, __tim_irq_handler, (void *)p_dev);

    am_int_enable(p_dev->p_devinfo->inum);

    return &(p_dev->timer_serv);
}

/******************************************************************************/
void am_zlg_tim_timing_deinit (am_timer_handle_t handle)
{

    am_zlg_tim_timing_dev_t *p_dev    = (am_zlg_tim_timing_dev_t *)handle;
    amhw_zlg_tim_t          *p_hw_tim = NULL;

    if (p_dev == NULL || p_dev->p_devinfo == NULL ) {
        return ;
    }

    p_hw_tim = (amhw_zlg_tim_t *)p_dev->p_devinfo->tim_regbase;

    /* رոж */
    amhw_zlg_tim_int_disable(p_hw_tim, AMHW_ZLG_TIM_UIE);

    /* رʱӣֹͣ */
    amhw_zlg_tim_disable(p_hw_tim);

    am_int_disconnect(p_dev->p_devinfo->inum, __tim_irq_handler, (void *)p_dev);
    am_int_disable(p_dev->p_devinfo->inum);

    p_dev->timer_serv.p_drv = NULL;

    if (p_dev->p_devinfo->pfn_plfm_deinit) {
        p_dev->p_devinfo->pfn_plfm_deinit();
    }
}

/* end of file */
