/*******************************************************************************
*                                 AMetal
*                       ----------------------------
*                       innovating embedded platform
*
* Copyright (c) 2001-2018 Guangzhou ZHIYUAN Electronics Co., Ltd.
* All rights reserved.
*
* Contact information:
* web site:    http://www.zlg118.cn/
*******************************************************************************/

/**
 * \file
 * \brief UART ʵ
 *
 * \internal
 * \par Modification history
 * - 1.00 19-09-19  zp, first implementation
 * \endinternal
 */
#include "am_zlg118_uart.h"
#include "am_clk.h"
#include "am_int.h"
#include "am_gpio.h"
#include "zlg118_pin.h"
#include "zlg118_clk.h"

/*******************************************************************************
* Functions declaration
*******************************************************************************/

/**
 * \brief ģʽѯжϣ
 */
int __uart_mode_set (am_zlg118_uart_dev_t *p_dev, uint32_t new_mode);

/**
 * \brief Ӳ
 */
int __uart_opt_set (am_zlg118_uart_dev_t *p_dev, uint32_t opts);

/**
 * \brief ؽ״̬
 */
static int __uart_flow_rxstat_set (am_zlg118_uart_dev_t *p_dev, uint32_t ctrl);

/**
 * \brief ģʽ
 */
static int __uart_flow_mode_set (am_zlg118_uart_dev_t *p_dev, uint32_t mode);

/**
 * \brief ط״̬ȡ
 */
static int __uart_flow_txstat_get (am_zlg118_uart_dev_t *p_dev);

/* ZLG118  */
static int __uart_ioctl (void *p_drv, int, void *);

static int __uart_tx_startup (void *p_drv);

static int __uart_callback_set (void *p_drv,
                                int   callback_type,
                                void *pfn_callback,
                                void *p_arg);

static int __uart_poll_getchar (void *p_drv, char *p_char);

static int __uart_poll_putchar (void *p_drv, char outchar);

#if 0
static int __uart_connect (void *p_drv);
#endif

static void __uart0_2_irq_handler (void *p_arg);
static void __uart1_3_irq_handler (void *p_arg);

/** \brief ׼ӿںʵ */
static const struct am_uart_drv_funcs __g_uart_drv_funcs = {
    __uart_ioctl,
    __uart_tx_startup,
    __uart_callback_set,
    __uart_poll_getchar,
    __uart_poll_putchar,
};

/******************************************************************************/

/**
 * \brief 豸ƺ
 *
 * аûȡʣģʽãж/ѯȡֵ֧ģʽӲѡõȹܡ
 */
static int __uart_ioctl (void *p_drv, int request, void *p_arg)
{
    am_zlg118_uart_dev_t *p_dev     = (am_zlg118_uart_dev_t *)p_drv;
    amhw_zlg118_uart_t   *p_hw_uart = (amhw_zlg118_uart_t *)
                                      p_dev->p_devinfo->uart_reg_base;

    int status = AM_OK;

    switch (request) {

    /*  */
    case AM_UART_BAUD_SET:

        /* ֻڷͿյĻϲ޸Ĳ */
        while (amhw_zlg118_uart_flag_check(
                   p_hw_uart, AMHW_ZLG118_UART_FLAG_TX_EMPTY) == AM_FALSE);

        status = amhw_zlg118_uart_baudrate_set(
                     p_hw_uart, p_dev->clk_rate, (uint32_t)p_arg);

        if (status > 0) {
            p_dev->baud_rate = status;
            status = AM_OK;
        } else {
            status = -AM_EIO;
        }
        break;

    /* ʻȡ */
    case AM_UART_BAUD_GET:
        *(int *)p_arg = p_dev->baud_rate;
        break;

    /* ģʽ */
    case AM_UART_MODE_SET:
        status = (__uart_mode_set(p_dev, (int)p_arg) == AM_OK)
                 ? AM_OK : -AM_EIO;
        break;

    /* ģʽȡ */
    case AM_UART_MODE_GET:
        *(int *)p_arg = p_dev->channel_mode;
        break;

    /* ȡڿõģʽ */
    case AM_UART_AVAIL_MODES_GET:
        *(int *)p_arg = AM_UART_MODE_INT | AM_UART_MODE_POLL;
        break;

    /* ѡ */
    case AM_UART_OPTS_SET:
        status = (__uart_opt_set(p_dev, (int)p_arg) == AM_OK) ? AM_OK : -AM_EIO;
        break;

    /* ѡȡ */
    case AM_UART_OPTS_GET:
        *(int *)p_arg = p_dev->options;
        break;

    /* 485ģʽ */
    case AM_UART_RS485_SET:
        if (p_dev->rs485_en != (am_bool_t)(int)p_arg) {
            p_dev->rs485_en = (am_bool_t)(int)p_arg;
        }
        break;

    /* 485ģʽ״̬ȡ */
    case AM_UART_RS485_GET:
        *(int *)p_arg = p_dev->rs485_en;
        break;

    /* ģʽ */
    case AM_UART_FLOWMODE_SET:
        __uart_flow_mode_set(p_dev, (int)p_arg);
    break;

    /* ý״̬ */
    case AM_UART_FLOWSTAT_RX_SET:
        __uart_flow_rxstat_set(p_dev, (int)p_arg);
    break;

    /* ȡ״̬ */
    case AM_UART_FLOWSTAT_TX_GET:
        *(int *)p_arg = __uart_flow_txstat_get(p_dev);
    break;

    default:
        status = -AM_EIO;
        break;
    }

    return (status);
}

/**
 * \brief ڷ(жģʽ)
 */
int __uart_tx_startup (void *p_drv)
{
    char data  = 0;

    am_zlg118_uart_dev_t *p_dev     = (am_zlg118_uart_dev_t *)p_drv;
    amhw_zlg118_uart_t   *p_hw_uart = (amhw_zlg118_uart_t *)p_dev->p_devinfo->uart_reg_base;

    /* ʹ 485 Ϳ */
    if (p_dev->rs485_en && p_dev->p_devinfo->pfn_rs485_dir) {
        p_dev->p_devinfo->pfn_rs485_dir(AM_TRUE);
    }

    /* ȴһδ */
    while (amhw_zlg118_uart_flag_check(
               p_hw_uart, AMHW_ZLG118_UART_FLAG_TX_EMPTY) == AM_FALSE);

    /* ȡݲ */
    if ((p_dev->pfn_txchar_get(p_dev->txget_arg, &data)) == AM_OK) {
        amhw_zlg118_uart_data_write(p_hw_uart, data);
    }

    /* ʹܷж */
    amhw_zlg118_uart_int_enable(p_hw_uart, AMHW_ZLG118_UART_INT_TX_COMPLETE);

    return AM_OK;
}

/**
 * \brief жϷص
 */
static int __uart_callback_set (void  *p_drv,
                                int    callback_type,
                                void  *pfn_callback,
                                void  *p_arg)
{
    am_zlg118_uart_dev_t *p_dev = (am_zlg118_uart_dev_t *)p_drv;

    switch (callback_type) {

    /* ÷ͻصеĻȡַص */
    case AM_UART_CALLBACK_TXCHAR_GET:
        p_dev->pfn_txchar_get = (am_uart_txchar_get_t)pfn_callback;
        p_dev->txget_arg      = p_arg;
        return (AM_OK);

    /* ýջصеĴŽַص */
    case AM_UART_CALLBACK_RXCHAR_PUT:
        p_dev->pfn_rxchar_put = (am_uart_rxchar_put_t)pfn_callback;
        p_dev->rxput_arg      = p_arg;
        return (AM_OK);

    /* ô쳣ص */
    case AM_UART_CALLBACK_ERROR:
        p_dev->pfn_err = (am_uart_err_t)pfn_callback;
        p_dev->err_arg = p_arg;
        return (AM_OK);

    default:
        return (-AM_ENOTSUP);
    }
}

/**
 * \brief ѯģʽ·һַ
 */
static int __uart_poll_putchar (void *p_drv, char outchar)
{
    am_zlg118_uart_dev_t *p_dev     = (am_zlg118_uart_dev_t *)p_drv;
    amhw_zlg118_uart_t   *p_hw_uart = (amhw_zlg118_uart_t *)p_dev->p_devinfo->uart_reg_base;

    /* ģǷ, AM_FALSE:æ; TURE:  */
    if(amhw_zlg118_uart_flag_check(
           p_hw_uart, AMHW_ZLG118_UART_FLAG_TX_EMPTY) == AM_FALSE) {
        return (-AM_EAGAIN);
    } else {

        if ((p_dev->rs485_en) && (p_dev->p_devinfo->pfn_rs485_dir != NULL)) {

            /*  485 Ϊģʽ */
            p_dev->p_devinfo->pfn_rs485_dir(AM_TRUE);
        }

        /* һַ */
        amhw_zlg118_uart_data_write(p_hw_uart, outchar);

        /* ȴ */
        while (amhw_zlg118_uart_flag_check(
                   p_hw_uart, AMHW_ZLG118_UART_FLAG_TX_EMPTY) == AM_FALSE);

        if (p_dev->rs485_en && p_dev->p_devinfo->pfn_rs485_dir) {
            p_dev->p_devinfo->pfn_rs485_dir(AM_FALSE);
        }
    }

    return (AM_OK);
}

/**
 * \brief ѯģʽ½ַ
 */
static int __uart_poll_getchar (void *p_drv, char *p_char)
{
    am_zlg118_uart_dev_t *p_dev     = (am_zlg118_uart_dev_t *)p_drv;
    amhw_zlg118_uart_t   *p_hw_uart = (amhw_zlg118_uart_t *)p_dev->p_devinfo->uart_reg_base;
    uint8_t           *p_inchar  = (uint8_t *)p_char;

    /* ģǷУAM_FALSE:æ,ڽ; TURE: Ѿյһַ */
    if(amhw_zlg118_uart_flag_check(
           p_hw_uart, AMHW_ZLG118_UART_FLAG_RX_COMPLETE) == AM_FALSE) {
        return (-AM_EAGAIN);
    } else {

        /* һַ */
        *p_inchar = amhw_zlg118_uart_data_read(p_hw_uart);

        amhw_zlg118_uart_flag_clr(p_hw_uart,
                                  AMHW_ZLG118_UART_FLAG_RX_COMPLETE);
    }

    return (AM_OK);
}

/**
 * \brief ôģʽ
 */
int __uart_mode_set (am_zlg118_uart_dev_t *p_dev, uint32_t new_mode)
{
    amhw_zlg118_uart_t *p_hw_uart = (amhw_zlg118_uart_t *)p_dev->p_devinfo->uart_reg_base;

    /* ֧ģʽ */
    if ((new_mode != AM_UART_MODE_POLL) && (new_mode != AM_UART_MODE_INT)) {
        return (AM_ERROR);
    }

    if (new_mode == AM_UART_MODE_INT) {

        if (p_dev->p_devinfo->inum == 6){
            am_int_connect(p_dev->p_devinfo->inum,
                           __uart0_2_irq_handler,
                           (void *)p_dev);
        }else{
            am_int_connect(p_dev->p_devinfo->inum,
                           __uart1_3_irq_handler,
                           (void *)p_dev);
        }

        am_int_enable(p_dev->p_devinfo->inum);

        /* ʹܽж */
        amhw_zlg118_uart_int_enable(p_hw_uart, AMHW_ZLG118_UART_INT_RX_COMPLETE);
    } else {

        /* رдж */
        amhw_zlg118_uart_int_disable(p_hw_uart, AMHW_ZLG118_UART_INT_ALL);
    }

    p_dev->channel_mode = new_mode;

    return (AM_OK);
}

/**
 * \brief ѡ
 */
int __uart_opt_set (am_zlg118_uart_dev_t *p_dev, uint32_t options)
{
    amhw_zlg118_uart_t *p_hw_uart = (amhw_zlg118_uart_t *)p_dev->p_devinfo->uart_reg_base;
    uint32_t            cfg_flags = 0;

    if (p_dev == NULL) {
        return -AM_EINVAL;
    }

    /* ֹͣλ */
    if (options & AM_UART_STOPB) {
        cfg_flags &= ~(0x3ul << 14);
        cfg_flags |= AMHW_ZLG118_UART_STOP_2_0_BIT;
    } else {
        cfg_flags &= ~(0x3ul << 14);
        cfg_flags |= AMHW_ZLG118_UART_STOP_1_0_BIT;
    }

    /* ü鷽ʽ */
    if (options & AM_UART_PARENB) {
        cfg_flags &= ~(0x3ul << 2);

        if (options & AM_UART_PARODD) {
            cfg_flags |= AMHW_ZLG118_UART_HW_PARITY_ODD;
        } else {
            cfg_flags |= AMHW_ZLG118_UART_HW_PARITY_EVEN;
        }
    } else {
        cfg_flags &= ~(0x3ul << 2);
        cfg_flags |= AMHW_ZLG118_UART_PARITY_NO;
    }

    /* Ч */
    amhw_zlg118_uart_stop_bit_sel(p_hw_uart  , (cfg_flags & (0x3ul << 14)));
    amhw_zlg118_uart_parity_bit_sel(p_hw_uart, (cfg_flags & (0x3ul << 2)));

    p_dev->options = options;

    return (AM_OK);
}
/******************************************************************************/
/**
 * \brief ״̬ãأ
 */
static int __uart_flow_rxstat_set (am_zlg118_uart_dev_t *p_dev, uint32_t ctrl)
{
    amhw_zlg118_uart_t *p_hw_uart = NULL;

    if (NULL == p_dev) {
        return -AM_EINVAL;
    }

    p_hw_uart = (amhw_zlg118_uart_t *)p_dev->p_devinfo->uart_reg_base;

    if (AM_UART_FLOWCTL_NO == p_dev->flowctl_mode) {
        return -AM_ENOTSUP;

    /* ӲأRTSŵλ */
    } else if (AM_UART_FLOWCTL_HW == p_dev->flowctl_mode) {
        if (AM_UART_FLOWSTAT_ON == ctrl) {
            /* RTSܽ,     */
            am_gpio_set(p_dev->p_devinfo->hwflowctl_cfg.rts_pin, 0);

        } else {
            /* RTSܽ,     */
            am_gpio_set(p_dev->p_devinfo->hwflowctl_cfg.rts_pin, 1);
        }
    } else {
        /* أXON/XOFFַ */
        if (AM_UART_FLOWSTAT_ON == ctrl) {
            amhw_zlg118_uart_data_write(p_hw_uart, AM_ZLG118_UART_XON);
        } else {
            amhw_zlg118_uart_data_write(p_hw_uart, AM_ZLG118_UART_XOFF);
        }
    }

    return AM_OK;
}

/******************************************************************************/
/**
 * \brief ģʽãأأӲأ
 */
static int __uart_flow_mode_set(am_zlg118_uart_dev_t *p_dev, uint32_t mode)
{
    amhw_zlg118_uart_t *p_hw_uart = NULL;

    if (NULL == p_dev) {
        return -AM_EINVAL;
    }

    p_hw_uart = (amhw_zlg118_uart_t *)p_dev->p_devinfo->uart_reg_base;
    p_dev->flowctl_mode = mode;

    /*  */
    if(AM_UART_FLOWCTL_NO == p_dev->flowctl_mode) {
        p_dev->flowctl_tx_stat = AM_UART_FLOWSTAT_ON;

        /* Ӳؽ     */
        amhw_zlg118_uart_disable(p_hw_uart, AMHW_ZLG118_UART_CTS);
        amhw_zlg118_uart_disable(p_hw_uart, AMHW_ZLG118_UART_RTS);

        /* ӲŽʼ */
        if(p_dev->p_devinfo->hwflowctl_cfg.enable == AM_TRUE) {

            /* ʼ */
            am_gpio_pin_cfg(p_dev->p_devinfo->hwflowctl_cfg.rts_pin,
                            PIO_INPUT_PU);
            am_gpio_pin_cfg(p_dev->p_devinfo->hwflowctl_cfg.cts_pin,
                            PIO_INPUT_PU);
        }

    /* Ӳ */
    } else if (AM_UART_FLOWCTL_HW == p_dev->flowctl_mode) {

        /* Ӳųʼ */
        if(p_dev->p_devinfo->hwflowctl_cfg.enable == AM_TRUE) {

            /* ʼ */
            am_gpio_pin_cfg(p_dev->p_devinfo->hwflowctl_cfg.rts_pin,
                            PIO_OUT_PP);
            am_gpio_pin_cfg(p_dev->p_devinfo->hwflowctl_cfg.cts_pin,
                            PIO_INPUT_FLOAT);
        }

        /* Ӳʹ     */
        amhw_zlg118_uart_enable(p_hw_uart, AMHW_ZLG118_UART_CTS);
        amhw_zlg118_uart_enable(p_hw_uart, AMHW_ZLG118_UART_RTS);
    } else {

        /* Ӳؽ     */
        amhw_zlg118_uart_disable(p_hw_uart, AMHW_ZLG118_UART_CTS);
        amhw_zlg118_uart_disable(p_hw_uart, AMHW_ZLG118_UART_RTS);

        /* ӲŽʼ */
        if(p_dev->p_devinfo->hwflowctl_cfg.enable == AM_TRUE) {

            /* ʼ */
            am_gpio_pin_cfg(p_dev->p_devinfo->hwflowctl_cfg.rts_pin,
                            PIO_INPUT_PU);
            am_gpio_pin_cfg(p_dev->p_devinfo->hwflowctl_cfg.cts_pin,
                            PIO_INPUT_PU);
        }

    }

    return AM_OK;
}

/******************************************************************************/
/**
 * \brief ״̬ȡ
 */
static int __uart_flow_txstat_get (am_zlg118_uart_dev_t *p_dev)
{
    amhw_zlg118_uart_t *p_hw_uart = NULL;

    if (NULL == p_dev) {
        return -AM_EINVAL;
    }

    p_hw_uart = (amhw_zlg118_uart_t *)p_dev->p_devinfo->uart_reg_base;

    if(AM_UART_FLOWCTL_HW == p_dev->flowctl_mode) {

        /* cts״̬仯ʱ״̬λЧ */
        if(amhw_zlg118_uart_flag_check(p_hw_uart,
                                       AMHW_ZLG118_UART_FLAG_CTS_TRIGGER) ==
                                       AM_TRUE) {

            /* cts״̬λ */
            amhw_zlg118_uart_flag_clr(p_hw_uart,
                                      AMHW_ZLG118_UART_FLAG_CTS_TRIGGER);

            /* жctsǰߵ͵ƽ */
            if (am_gpio_get(p_dev->p_devinfo->hwflowctl_cfg.cts_pin) == 0) {
                return (int)AM_UART_FLOWSTAT_ON;
            } else {
                return (int)AM_UART_FLOWSTAT_OFF;
            }
        } else {
            return (int)p_dev->flowctl_tx_stat;
        }
    } else {
        return (int)p_dev->flowctl_tx_stat;
    }
}

/*******************************************************************************
  UART interrupt request handler
*******************************************************************************/

/**
 * \brief ڽжϷ
 */
void __uart_irq_rx_handler (am_zlg118_uart_dev_t *p_dev)
{
    amhw_zlg118_uart_t *p_hw_uart = (amhw_zlg118_uart_t *)p_dev->p_devinfo->uart_reg_base;
    char data;

    /* ǷΪRxж */
    if (amhw_zlg118_uart_flag_check(
            p_hw_uart, AMHW_ZLG118_UART_FLAG_RX_COMPLETE) == AM_TRUE) {

        amhw_zlg118_uart_flag_clr(p_hw_uart, AMHW_ZLG118_UART_FLAG_RX_COMPLETE);

        /* ȡ½ */
        data = amhw_zlg118_uart_data_read(p_hw_uart);

        /* ½ */
        p_dev->pfn_rxchar_put(p_dev->rxput_arg, data);
    }
}

/**
 * \brief ڷжϷ
 */
void __uart_irq_tx_handler (am_zlg118_uart_dev_t *p_dev)
{
    amhw_zlg118_uart_t *p_hw_uart = (amhw_zlg118_uart_t *)p_dev->p_devinfo->uart_reg_base;

    char data;

    if (amhw_zlg118_uart_flag_check(
            p_hw_uart, AMHW_ZLG118_UART_FLAG_TX_COMPLETE) == AM_TRUE) {

        amhw_zlg118_uart_flag_clr(p_hw_uart, AMHW_ZLG118_UART_FLAG_TX_COMPLETE);

        /* ȡݲ */
        if ((p_dev->pfn_txchar_get(p_dev->txget_arg, &data)) == AM_OK) {
            amhw_zlg118_uart_data_write(p_hw_uart, data);
        } else {

            /* ûݴ;͹رշж */
            amhw_zlg118_uart_int_disable(p_hw_uart,
                                         AMHW_ZLG118_UART_FLAG_TX_COMPLETE);

            /* 485Ϳ */
            if ((p_dev->rs485_en) && (p_dev->p_devinfo->pfn_rs485_dir)) {

                /*  485 Ϊģʽ */
                p_dev->p_devinfo->pfn_rs485_dir(AM_FALSE);
            }
        }
    }
}

/**
 * \brief 0-2жϷ
 */

/**< \brief жԴҪȫֵһ豸Ϣ */
extern am_zlg118_uart_dev_t  *__gp_uart0;
extern am_zlg118_uart_dev_t  *__gp_uart2;
void __uart0_2_irq_handler (void *p_arg)
{
    /* ͬһжԴжǷͬʱ־ */
    uint8_t  flag_event    = 0;
    uint32_t uart_int_stat = 0;

    amhw_zlg118_uart_t    *p_hw_uart  = NULL;
    am_zlg118_uart_dev_t  *p_dev      = NULL;

    amhw_zlg118_uart_t    *p_hw_uart0 = NULL;
    amhw_zlg118_uart_t    *p_hw_uart2 = NULL;

    am_zlg118_uart_dev_t  *p_dev0     = __gp_uart0;
    if (p_dev0 != NULL){
        p_hw_uart0 = (amhw_zlg118_uart_t *) p_dev0->p_devinfo->uart_reg_base;
    }

    am_zlg118_uart_dev_t  *p_dev2     = __gp_uart2;
    if (p_dev2 != NULL){
        p_hw_uart2 = (amhw_zlg118_uart_t *) p_dev2->p_devinfo->uart_reg_base;
    }

    /* жĸʹжϣͬʱʹã */
    if ( amhw_zlg118_uart_int_check (p_hw_uart0) &&
        !amhw_zlg118_uart_int_check (p_hw_uart2)){
        p_hw_uart = p_hw_uart0;
        p_dev = p_dev0;
        flag_event = 0;
    }else if (!amhw_zlg118_uart_int_check (p_hw_uart0) &&
               amhw_zlg118_uart_int_check (p_hw_uart2)){
        p_hw_uart = p_hw_uart2;
        p_dev = p_dev2;
        flag_event = 0;
    }else if (amhw_zlg118_uart_int_check (p_hw_uart0) &&
              amhw_zlg118_uart_int_check (p_hw_uart2)){
        flag_event = 1;
    }

    /* жж */
    if (!flag_event){
        uart_int_stat = amhw_zlg118_uart_flag_get(p_hw_uart);

        if (amhw_zlg118_uart_flag_check(
            p_hw_uart, AMHW_ZLG118_UART_FLAG_RX_COMPLETE) == AM_TRUE) {

                 __uart_irq_rx_handler(p_dev);

        } else if (amhw_zlg118_uart_flag_check(
            p_hw_uart, AMHW_ZLG118_UART_FLAG_TX_COMPLETE) == AM_TRUE) {

                __uart_irq_tx_handler(p_dev);

        } else {

        }
    }else{
        if (amhw_zlg118_uart_flag_check(
                  p_hw_uart0, AMHW_ZLG118_UART_FLAG_RX_COMPLETE) == AM_TRUE){
            uart_int_stat = amhw_zlg118_uart_flag_get(p_hw_uart0);
            __uart_irq_rx_handler(p_dev0);
        }else if (amhw_zlg118_uart_flag_check(
                p_hw_uart0, AMHW_ZLG118_UART_FLAG_TX_COMPLETE) == AM_TRUE){
            uart_int_stat = amhw_zlg118_uart_flag_get(p_hw_uart0);
            __uart_irq_tx_handler(p_dev0);
        }else if (amhw_zlg118_uart_flag_check(
                p_hw_uart2, AMHW_ZLG118_UART_FLAG_RX_COMPLETE) == AM_TRUE){
            uart_int_stat = amhw_zlg118_uart_flag_get(p_hw_uart2);
            __uart_irq_rx_handler(p_dev2);
        }else if (amhw_zlg118_uart_flag_check(
                  p_hw_uart2, AMHW_ZLG118_UART_FLAG_TX_COMPLETE) == AM_TRUE){
            uart_int_stat = amhw_zlg118_uart_flag_get(p_hw_uart2);
            __uart_irq_tx_handler(p_dev2);
        }else{
            ;
        }
    }

    /* ж */
    if ((p_dev->other_int_enable & uart_int_stat) != 0) {

        uart_int_stat &= p_dev->other_int_enable;

        if (p_dev->pfn_err != NULL) {
            p_dev->pfn_err(p_dev->err_arg,
                           AM_ZLG118_UART_ERRCODE_UART_OTHER_INT,
                           (void *)p_hw_uart,
                           1);
        }
    }

}

/**
 * \brief 1-3жϷ
 */
/**< \brief жԴҪȫֵһ豸Ϣ */
extern am_zlg118_uart_dev_t  *__gp_uart1;
extern am_zlg118_uart_dev_t  *__gp_uart3;
void __uart1_3_irq_handler (void *p_arg)
{
    /* ͬһжԴжǷͬʱ־ */
    uint8_t  flag_event    = 0;
    uint32_t uart_int_stat = 0;

    amhw_zlg118_uart_t    *p_hw_uart  = NULL;
    am_zlg118_uart_dev_t  *p_dev      = NULL;

    amhw_zlg118_uart_t    *p_hw_uart1 = NULL;
    amhw_zlg118_uart_t    *p_hw_uart3 = NULL;

    am_zlg118_uart_dev_t  *p_dev1     = __gp_uart1;
    if (p_dev1 != NULL){
        p_hw_uart1 = (amhw_zlg118_uart_t *) p_dev1->p_devinfo->uart_reg_base;
    }

    am_zlg118_uart_dev_t  *p_dev3     = __gp_uart3;
    if (p_dev3 != NULL){
        p_hw_uart3 = (amhw_zlg118_uart_t *) p_dev3->p_devinfo->uart_reg_base;
    }

    /* жĸʹжϣͬʱʹã */
    if ( amhw_zlg118_uart_int_check (p_hw_uart1) &&
        !amhw_zlg118_uart_int_check (p_hw_uart3)){
        p_hw_uart = p_hw_uart1;
        p_dev     = p_dev1;
        flag_event = 0;
    }else if (!amhw_zlg118_uart_int_check (p_hw_uart1) &&
              amhw_zlg118_uart_int_check (p_hw_uart3)){
        p_hw_uart = p_hw_uart3;
        p_dev     = p_dev3;
        flag_event = 0;
    }else if (amhw_zlg118_uart_int_check (p_hw_uart1) &&
              amhw_zlg118_uart_int_check (p_hw_uart3)){
        flag_event = 1;
    }

    /* жж */
    if (!flag_event){
        uart_int_stat = amhw_zlg118_uart_flag_get(p_hw_uart);

        if (amhw_zlg118_uart_flag_check(
            p_hw_uart, AMHW_ZLG118_UART_FLAG_RX_COMPLETE) == AM_TRUE) {

                 __uart_irq_rx_handler(p_dev);

        } else if (amhw_zlg118_uart_flag_check(
            p_hw_uart, AMHW_ZLG118_UART_FLAG_TX_COMPLETE) == AM_TRUE) {

                __uart_irq_tx_handler(p_dev);

        } else {

        }
    }else{
        if (amhw_zlg118_uart_flag_check(
                  p_hw_uart1, AMHW_ZLG118_UART_FLAG_RX_COMPLETE) == AM_TRUE){
            uart_int_stat = amhw_zlg118_uart_flag_get(p_hw_uart1);
            __uart_irq_rx_handler(p_dev1);
        }else if (amhw_zlg118_uart_flag_check(
                  p_hw_uart3, AMHW_ZLG118_UART_FLAG_RX_COMPLETE) == AM_TRUE){
            uart_int_stat = amhw_zlg118_uart_flag_get(p_hw_uart3);
            __uart_irq_rx_handler(p_dev3);
        }else if (amhw_zlg118_uart_flag_check(
                  p_hw_uart1, AMHW_ZLG118_UART_FLAG_TX_COMPLETE) == AM_TRUE){
            uart_int_stat = amhw_zlg118_uart_flag_get(p_hw_uart1);
            __uart_irq_tx_handler(p_dev1);
        }else if (amhw_zlg118_uart_flag_check(
                p_hw_uart3, AMHW_ZLG118_UART_FLAG_TX_COMPLETE) == AM_TRUE){
            uart_int_stat = amhw_zlg118_uart_flag_get(p_hw_uart3);
            __uart_irq_tx_handler(p_dev3);
        }else{
            ;
        }
    }

    /* ж */
    if ((p_dev->other_int_enable & uart_int_stat) != 0) {

        uart_int_stat &= p_dev->other_int_enable;

        if (p_dev->pfn_err != NULL) {
            p_dev->pfn_err(p_dev->err_arg,
                           AM_ZLG118_UART_ERRCODE_UART_OTHER_INT,
                           (void *)p_hw_uart,
                           1);
        }
    }

}

#if 0
/**
 * \brief UARTжϺӣʹжģʽʱҪô˺
 */
int __uart_connect (void *p_drv)
{
    am_zlg118_uart_dev_t *p_dev = (am_zlg118_uart_dev_t *)p_drv;

    /* жţж */
    am_int_connect(p_dev->p_devinfo->inum, __uart_irq_handler, (void *)p_dev);
    am_int_enable(p_dev->p_devinfo->inum);
    amhw_zlg118_uart_int_enable(p_dev->p_devinfo->uart_reg_base,
                                p_dev->other_int_enable);

    return AM_OK;
}
#endif /* 0 */

/**
 * \brief Ĭϻص
 *
 * \returns AW_ERROR
 */
static int __uart_dummy_callback (void *p_arg, char *p_outchar)
{
    return (AM_ERROR);
}

/**
 * \brief ڳʼ
 */
am_uart_handle_t am_zlg118_uart_init (am_zlg118_uart_dev_t           *p_dev,
                                      const am_zlg118_uart_devinfo_t *p_devinfo)
{
    amhw_zlg118_uart_t  *p_hw_uart;

    if (p_devinfo == NULL) {
        return NULL;
    }

    /* ȡò */
    p_hw_uart                = (amhw_zlg118_uart_t  *)p_devinfo->uart_reg_base;
    p_dev->p_devinfo         = p_devinfo;
    p_dev->uart_serv.p_funcs = (struct am_uart_drv_funcs *)&__g_uart_drv_funcs;
    p_dev->uart_serv.p_drv   = p_dev;
    p_dev->baud_rate         = p_devinfo->baud_rate;
    p_dev->options           = 0;

    /* ʼĬϻص */
    p_dev->pfn_txchar_get    = (int (*) (void *, char*))__uart_dummy_callback;
    p_dev->txget_arg         = NULL;
    p_dev->pfn_rxchar_put    = (int (*) (void *, char ))__uart_dummy_callback;
    p_dev->rxput_arg         = NULL;
    p_dev->pfn_err           =
                     (int (*) (void *, int, void*, int))__uart_dummy_callback;

    p_dev->err_arg           = NULL;

    p_dev->flowctl_mode      = AM_UART_FLOWCTL_NO;
    p_dev->flowctl_tx_stat   = AM_UART_FLOWSTAT_ON;

    p_dev->other_int_enable  = p_devinfo->other_int_enable  &
                               ~(AMHW_ZLG118_UART_INT_TX_COMPLETE |
                                 AMHW_ZLG118_UART_INT_RX_COMPLETE);

    p_dev->rs485_en          = AM_FALSE;

    if (p_dev->p_devinfo->pfn_plfm_init) {
        p_dev->p_devinfo->pfn_plfm_init();
    }

    /* ģʽ2/3ҿ˶ַԶʶ */
    if((p_devinfo->work_mode == 2) || (p_devinfo->work_mode == 3)) {

        if((p_devinfo->mut_addr.enable == AM_TRUE)) {

            /* ôӻַԱ豸͵ݶԣ*/
            amhw_zlg118_uart_slaver_addr_set(p_hw_uart,
                                             p_devinfo->mut_addr.addr);

            /* õַ */
            amhw_zlg118_uart_slaver_addr_set(p_hw_uart,
                                             p_devinfo->mut_addr.addr_mask);

            /* ַԶʶʹ */
            amhw_zlg118_uart_enable(p_hw_uart, AMHW_ZLG118_UART_MUT_ADR_AUTO);
        }
    }

    /* ģʽ */
    if (p_devinfo->work_mode == AMHW_ZLG118_UART_WORK_MODE_4) {
        amhw_zlg118_uart_single_line_half_enable(p_hw_uart);
        if ((p_devinfo->cfg_flags & (12u)) == AMHW_ZLG118_UART_PARITY_NO){
            amhw_zlg118_uart_mode_sel(p_hw_uart, AMHW_ZLG118_UART_WORK_MODE_1);
        }else {
            amhw_zlg118_uart_mode_sel(p_hw_uart, AMHW_ZLG118_UART_WORK_MODE_3);
        }
    } else {
        amhw_zlg118_uart_mode_sel(p_hw_uart, p_devinfo->work_mode);
    }
    p_dev->work_mode = p_devinfo->work_mode;

    if(p_devinfo->work_mode == AMHW_ZLG118_UART_WORK_MODE_0) {

        /* ģʽ0ͨʱӷƵϵЧ */
        amhw_zlg118_uart_clk_div_sel(p_hw_uart,
                                     AMHW_ZLG118_UART_CLK_DIV_MODE0_NO);
    } else {

        /* ͨʱӷƵϵΪС */
        amhw_zlg118_uart_clk_div_sel(p_hw_uart,
                                     AMHW_ZLG118_UART_CLK_DIV_MODE2_16);
    }

    /* ȡڼ鷽ʽѡ */
    if ((p_devinfo->cfg_flags & (12u)) == (AMHW_ZLG118_UART_HW_PARITY_ODD)) {
        p_dev->options |= AM_UART_PARENB | AM_UART_PARODD;
    } else if((p_devinfo->cfg_flags & (12u)) == (AMHW_ZLG118_UART_HW_PARITY_EVEN)){
        p_dev->options |= AM_UART_PARENB;
    } else {

    }

    /* ȡֹͣλѡ */
    if (p_devinfo->cfg_flags & (AMHW_ZLG118_UART_STOP_2_0_BIT)) {
        p_dev->options |= AM_UART_STOPB;
    } else {

    }

    /* ȴͿ */
    while (amhw_zlg118_uart_flag_check(p_hw_uart,
                                       AMHW_ZLG118_UART_FLAG_TX_EMPTY) ==
                                       AM_FALSE);

    __uart_opt_set (p_dev, p_dev->options);

    /* ģʽãأأӲأ*/
    __uart_flow_mode_set(p_dev, AM_UART_FLOWCTL_NO);

    p_dev->clk_rate = am_clk_rate_get(CLK_PCLK);

    /* ò */
    p_dev->baud_rate = amhw_zlg118_uart_baudrate_set(
                                     p_hw_uart,
                                     p_dev->clk_rate,
                                     p_devinfo->baud_rate);
    /* Ĭѯģʽ */
    __uart_mode_set(p_dev, AM_UART_MODE_POLL);

    /* ʹ
     * Mode0:     0; 1
     * Mode1/2/3: 0; 1 /
     */
    amhw_zlg118_uart_enable(p_hw_uart,AMHW_ZLG118_UART_RX);

    if (p_dev->p_devinfo->pfn_rs485_dir) {

        /* ʼ 485 Ϊģʽ */
        p_dev->p_devinfo->pfn_rs485_dir(AM_FALSE);
    }

    return &(p_dev->uart_serv);
}

/**
 * \brief ȥʼ
 */
void am_zlg118_uart_deinit (am_zlg118_uart_dev_t *p_dev)
{
    amhw_zlg118_uart_t *p_hw_uart = (amhw_zlg118_uart_t *)p_dev->p_devinfo->uart_reg_base;
    p_dev->uart_serv.p_funcs   = NULL;
    p_dev->uart_serv.p_drv     = NULL;

    if (p_dev->channel_mode == AM_UART_MODE_INT) {

        /* ĬΪѯģʽ */
        __uart_mode_set(p_dev, AM_UART_MODE_POLL);
    }

    amhw_zlg118_uart_disable(p_hw_uart,AMHW_ZLG118_UART_RX);
    amhw_zlg118_uart_disable(p_hw_uart,AMHW_ZLG118_UART_CTS);
    amhw_zlg118_uart_disable(p_hw_uart,AMHW_ZLG118_UART_RTS);
    amhw_zlg118_uart_disable(p_hw_uart,AMHW_ZLG118_UART_DMA_TX);
    amhw_zlg118_uart_disable(p_hw_uart,AMHW_ZLG118_UART_DMA_RX);
    amhw_zlg118_uart_disable(p_hw_uart,AMHW_ZLG118_UART_MUT_ADR_AUTO);

    am_int_disable(p_dev->p_devinfo->inum);

    if (p_dev->p_devinfo->pfn_plfm_deinit) {
        p_dev->p_devinfo->pfn_plfm_deinit();
    }
}

/* end of file */
